/*
 * $Id: //depot/prod/newport_news/main/modules/crypto-util/src/main/java/com/netapp/crypto/Aes128CbcPkcs5PaddingCipherDelegate.java#1 $
 *
 * Copyright (c) 2013 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.crypto;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.SecureRandom;
import java.util.concurrent.Callable;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;


/**
 * A {@link SymmetricCipherDelegate} that uses the
 * {@link Aes128CbcPkcs5PaddingCipherDelegate#TRANSFORMATION} transformation
 * with {@link Aes128CbcPkcs5PaddingCipherDelegate#KEY_BIT_LENGTH} bit keys.
 */
class Aes128CbcPkcs5PaddingCipherDelegate extends SymmetricCipherDelegate {

    private static final int ID = 0;
    private static final String TRANSFORMATION = "AES/CBC/PKCS5Padding";
    private static final int IV_BIT_LENGTH = 128;
    private static final String KEY_GENERATION_ALGORITHM = "AES";
    private static final int KEY_BIT_LENGTH = 128;

    public Aes128CbcPkcs5PaddingCipherDelegate(Callable<SecureRandom> secureRandomFactory) {
        super(
            ID,
            TRANSFORMATION,
            IV_BIT_LENGTH,
            KEY_GENERATION_ALGORITHM,
            KEY_BIT_LENGTH,
            secureRandomFactory);
    }

    @Override
    public byte[] encrypt(SecretKey key, byte[] plainText) throws InvalidKeyException, InvalidAlgorithmParameterException, IllegalBlockSizeException, BadPaddingException, IOException {

        byte[] iv = new byte[getCipherIvBitLength() / Byte.SIZE];
        getSecureRandom().nextBytes(iv);

        Cipher cipher = getCipher();
        cipher.init(Cipher.ENCRYPT_MODE, key, new IvParameterSpec(iv));

        byte[] cipherTextBytes = cipher.doFinal(plainText);

        ByteArrayOutputStream baos = new ByteArrayOutputStream(iv.length + cipherTextBytes.length);
        baos.write(iv);
        baos.write(cipherTextBytes);
        baos.close();
        return baos.toByteArray();
    }

    @Override
    public byte[] decrypt(SecretKey key, byte[] cipherText) throws IllegalBlockSizeException, BadPaddingException, IOException, InvalidKeyException, InvalidAlgorithmParameterException {

        int ivByteLength = getCipherIvBitLength() / Byte.SIZE;

        Cipher cipher = getCipher();
        cipher.init(Cipher.DECRYPT_MODE, key, new IvParameterSpec(cipherText, 0, ivByteLength));

        byte[] plainText = cipher.doFinal(cipherText, ivByteLength, cipherText.length - ivByteLength);
        return plainText;
    }

}
