package com.netapp.oci.netapp.performance.hostadapter;

import java.sql.ResultSet;
import java.sql.SQLException;

import com.netapp.oci.netapp.performance.hostadapter.interfaces.data.HostadapterMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

public class HostadapterMetaData {

    public static final Column[] SAMPLE_COLUMNS = new Column[]{
        new Column("bytesRead", "float"),
        new Column("bytesWritten", "float"),
        new Column("maxLinkDataRate", "bigint unsigned")
    };
    
    public static final Column[] SUMMARY_COLUMNS = new Column[]{
        new Column("bytesRead", "float"),
        new Column("bytesWritten", "float"),
        new Column("maxLinkDataRate", "bigint unsigned")
    };
    
   
    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.MAX} //Since maxLinkDataRate remains constant, Using MAX (as RAW option is unavailable).
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.MAX
    };

    private static ObjectParser<HostadapterMeasurements> sampleParser;

    static {
        
        sampleParser = new ObjectParser<HostadapterMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, HostadapterMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getBytesRead());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getBytesWritten());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMaxLinkDataRate());
            }

            public HostadapterMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                HostadapterMeasurements measurements = new HostadapterMeasurements();
                measurements.setBytesRead(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                measurements.setBytesWritten(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                measurements.setMaxLinkDataRate(PerformanceTableUtils.getLongNullable(resultSet, ++i));
                
               return measurements;
                
            }
        };
    }
    
    public static ObjectParser<HostadapterMeasurements> getSampleParser() {
        return sampleParser;
    }
    
    public static ObjectParser<HostadapterMeasurements> getSummaryParser() {
        return sampleParser;
    }
}
