package com.netapp.oci.netapp.performance.lun;

import com.netapp.oci.netapp.performance.lun.interfaces.data.LunMeasurements;
import com.netapp.oci.netapp.performance.lun.interfaces.data.LunRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * @author Anton Sergeev
 */
public class LunMetaData {
    public static final Column[] RAW_COLUMNS = new Column[]{
        new Column("avg_latency", "bigint unsigned"),
        new Column("avg_read_latency", "bigint unsigned"),
        new Column("avg_write_latency", "bigint unsigned"),
        new Column("other_ops", "bigint unsigned"),
        new Column("read_data", "bigint unsigned"),
        new Column("read_ops", "bigint unsigned"),
        new Column("total_ops", "bigint unsigned"),
        new Column("write_data", "bigint unsigned"),
        new Column("write_ops", "bigint unsigned"),
        new Column("read_partial_blocks", "bigint unsigned"),
        new Column("write_partial_blocks", "bigint unsigned"),
        new Column("opm_lun_throughput", "bigint unsigned")
    };
	
    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("avgLatency", "float"),
            new Column("avgReadLatency", "float"),
            new Column("avgWriteLatency", "float"),
            new Column("otherOps", "float"),
            new Column("readData", "float"),
            new Column("readOps", "float"),
            new Column("readPartialBlocks", "float"),
            new Column("totalOps", "float"),
            new Column("writeData", "float"),
            new Column("writeOps", "float"),
            new Column("writePartialBlocks", "float"),
            new Column("opmLunThroughput", "float")
    };

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("avgLatency", "float"),
            new Column("avgReadLatency", "float"),
            new Column("avgWriteLatency", "float"),
            new Column("otherOps", "float"),
            new Column("readData", "float"),
            new Column("readOps", "float"),
            new Column("readPartialBlocks", "float"),
            new Column("totalOps", "float"),
            new Column("writeData", "float"),
            new Column("writeOps", "float"),
            new Column("writePartialBlocks", "float"),
            new Column("opmLunThroughput", "float")
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {new RatioMethod("totalOps")},
            {new RatioMethod("readOps")},
            {new RatioMethod("writeOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("readOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("writeOps")},
            {SqlMethod.AVG}
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            new RatioMethod("totalOps"),
            new RatioMethod("readOps"),
            new RatioMethod("writeOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("readOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("writeOps"),
            SqlMethod.AVG
    };

    private static ObjectParser<LunRawMeasurements> rawParser;
    private static ObjectParser<LunMeasurements> sampleParser;

    static {
        sampleParser = new ObjectParser<LunMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, LunMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadPartialBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWritePartialBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmLunThroughput());
            }

            public LunMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new LunMeasurements(
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };

        rawParser = new ObjectParser<LunRawMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, LunRawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadPartialBlocks());                		
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWritePartialBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmLunThroughput());   
            }

            public LunRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new LunRawMeasurements(
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i), 
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<LunMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<LunMeasurements> getSampleParser() {
        return sampleParser;
    }

    public static ObjectParser<LunRawMeasurements> getRawParser() {
        return rawParser;
    }
    
}
