/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.shared;

import com.netapp.oci.netapp.performance.PerformanceConstants;
import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.framework.performance.PerformancePackageFactory;
import com.netapp.oci.platform.framework.performance.PersistenceFacade;
import com.netapp.oci.platform.framework.performance.tables.*;
import com.netapp.oci.platform.framework.performance.util.SampleTablePreparator;
import com.netapp.oci.platform.framework.performance.util.preparators.CopyPreparator;
import com.onaro.commons.types.Period;
import org.apache.log4j.Logger;

import java.sql.Connection;

/**
 * @author Anton Sergeev
 */
public abstract class AbstractPerformanceComponentsFactory<C, S> {
    protected String packageName;
    
    protected Column[] sampleColumns;
    
    protected Column[] summaryColumns;
    
    protected ObjectParser<C> sampleParser;
    
    protected ObjectParser<S> summaryParser;

    protected SqlMethod[] summaryCommonMethods;

    protected SqlMethod[][] sampleToSummaryMethods;

	public AbstractPerformanceComponentsFactory(String packageName, Column[] sampleColumns,
                                              Column[] summaryColumns, ObjectParser<C> sampleParser,
                                              ObjectParser<S> summaryParser, SqlMethod[] summaryCommonMethods,
                                              SqlMethod[][] sampleToSummaryMethods) {
        this.packageName = packageName;
        this.sampleColumns = sampleColumns;
        this.summaryColumns = summaryColumns;
        this.sampleParser = sampleParser;
        this.summaryParser = summaryParser;
        this.summaryCommonMethods = summaryCommonMethods;
        this.sampleToSummaryMethods = sampleToSummaryMethods;
    }

    public abstract String getPerformancePackageId(Long containerId);

    public abstract String getPackagePrefix();

    public String getPackageName() {
        return packageName;
    }

    public SqlMethod[] getSummaryCommonMethods() {
        return summaryCommonMethods;
    }

    public SqlMethod[][] getSampleToSummaryMethods() {
        return sampleToSummaryMethods;
    }

    public SampleTablePreparator getSampleTablePreparator(Logger logger) {
        return new CopyPreparator(0, logger);
    }

    public abstract SampleTable<C> getSampleTable(Connection connection, Long containerId);

    public CurrentSampleTable<C> getCurrentSampleTable(Connection connection, String containerKey, Long containerId) {
        CurrentSampleTable<C> table = new PartitionedCurrentSampleTable<C>(connection, sampleParser,
                containerKey, containerId, packageName, sampleColumns);
        table.setDatabase(PerformanceConstants.NETAPP_PERFORMANCE_DATABASE);
        return table;
    }

    public abstract SummaryTable<S> getSummaryTable(Connection connection, Long containerId);

    public PersistenceFacade<C, S> createPersistenceFacade(Connection connection, Long containerId)
            throws PerformanceException {
        try {
            return new PersistenceFacade<C, S>(connection, createPerformancePackageFactory(containerId));
        } catch (Exception e) {
            throw new PerformanceException("Failed to create persistence facade", e);
        }        
    }

    public PersistenceFacade<C, S> createPersistenceFacade(Connection connection, String containerKey, Long containerId)
            throws PerformanceException {
        try {
            return new PersistenceFacade<C, S>(connection, createPerformancePackageFactory(containerKey, containerId));
        } catch (Exception e) {
            throw new PerformanceException("Failed to create persistence facade", e);
        }
    }

    public PerformancePackageFactory<C, S> createPerformancePackageFactory(String containerKey, Long containerId) {
        try {
        	if(summaryColumns != null){
        		return new SimplePerformancePackageFactory<C, S>(containerKey, containerId, this);
        	}
        	else{
        		return new SamplePerformancePackageFactory<C, S>(containerKey, containerId, this);
        	}
        } catch (Exception e) {
            throw new RuntimeException("Failed to create persistence facade", e);
        }
    }

    public PerformancePackageFactory<C, S> createPerformancePackageFactory(Long containerId) {
        try {
        	if(summaryColumns != null){
                return new SimplePerformancePackageFactory<C, S>(containerId, this);
        	}
        	else{
                return new SamplePerformancePackageFactory<C, S>(containerId, this);
        	}
        } catch (Exception e) {
            throw new RuntimeException("Failed to create persistence facade", e);
        }
    }
}
