/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.shared.interfaces.impl;

import com.netapp.oci.netapp.Param;
import com.netapp.oci.netapp.db.NetAppPerformance;
import com.netapp.oci.netapp.performance.PerformanceConstants;
import com.netapp.oci.platform.common.InternalServerException;
import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.onaro.commons.types.Period;
import org.jboss.ejb3.annotation.ResourceAdapter;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.StatefulJob;
import javax.annotation.Resource;
import javax.annotation.security.RunAs;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.sql.DataSource;
import java.sql.Connection;

/**
 * @author Anton Sergeev
 */
@MessageDriven(activationConfig = {
        @ActivationConfigProperty(propertyName = "cronTrigger", propertyValue = "0 15 2 * * ?"),
        @ActivationConfigProperty(propertyName = "jobName", propertyValue = "NetappPerformanceDailyJob")})
@ResourceAdapter("quartz-ra.rar")
@RunAs("admin")
public class NetappPerformanceDailyJob implements StatefulJob {
    @Resource(name = "jdbc/OnaroDataSource", mappedName = "java:/onaro_pool", type = javax.sql.DataSource.class, authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        Connection connection = null;

        try {
            connection = ConnectionHelper.newConnection(dataSource);
            NetAppPerformance.INSTANCE.ensureUpdateHistoryLifeSpan(
                    connection,
                    System.currentTimeMillis(),
                    PerformanceConstants.getTablesLifeSpan(Param.Key.SAMPLE_TABLES_LIFESPAN_DAYS)
                            * Period.DAY.milliseconds());
        }
        catch (InternalServerException e) {
            throw new JobExecutionException(e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }

}
