package com.netapp.oci.netapp.performance.shared.interfaces.local;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;

import javax.ejb.Local;
import java.util.Map;

/**
 * EJB local interface to register listeners that interested in the sample data. The listener
 * is per sample type and identified by the lister name.
 * <ol start=0>
 * <li>	sample type - What is the sample type that listener wants to receive (eg. VolumeMeasurements)
 * <li>	listener name - what is the name of the listener (eg. StaticThreshold)
 * </ol>
 *  
 * @author Cheng Sun
 *
 */
@Local
public interface NetappPerformanceSampleListenerRegistry {
    String JNDI_NAME = "NetappPerformanceSampleListenerRegistry";

    /**
     * The sample types that a listener can subscribe to.
     */
    public static enum SampleType {
    	AGGREGATE,
    	CIFS_VSERVER,
    	DISK,
    	FCP_LIF,
    	FCP_LIF_VSERVER,
    	FCP_PORT,
    	ISCSI_LIF,
    	ISCSI_LIF_VSERVER,
    	LUN,
    	NETWORK_LIF,
    	NETWORK_LIF_VSERVER,
    	NETWORK_PORT,
    	NFS,
    	NFSV3,
    	NFSV4,
    	NFSV41,
    	NIC,
    	NODE,
    	PROCESSOR,
    	QOS_VOLUME_WORKLOAD,
    	QOS_WORKLOAD_QUEUE_DBLADE,
    	QOS_WORKLOAD_QUEUE_NBLADE,
    	QOS_WORKLOAD_DETAIL,
    	QOS_SERVICE_CENTER,
    	VOLUME,
    	VSERVER,
		VOLUMEVSERVER,
    	WAFL,
    	HOSTADAPTER,
    	CLUSTER
    }
    
    /**
     * Register a listener identified by the listener name to the sample type.
     * 
     * @param sampleType
     * @param listenerName
     * @param listener
     */
    void register(SampleType sampleType, String listenerName, PerformanceSampleListener listener) throws NetappInterfaceException;
    
    /**
     * Unregister the listener identified by the listenerName to the input sample type.
     * 
     * @param sampleType
     * @param listenerName
     */
    void unregister(SampleType sampleType, String listenerName) throws NetappInterfaceException;
    
    /**
     * Get all the listeners that subscribe the input sample type.
     * 
     * @param sampleType
     * @return
     */
    Map<String, PerformanceSampleListener> getSampleListeners(SampleType sampleType);

}
