/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.volume.interfaces.data;

import java.io.Serializable;

/**
 * @author Al Velella
 *         Date: Oct 4, 2011
 *         Time: 2:20:28 PM
 */
public class VolumeMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * Average latency in microseconds for the WFL Filesystem to process all the operations on the volume; not including
     * request processing or network communication
     */
    private float avgLatency;

    /**
     * Average time for the WAFL filesystem to process other CIFS operations to the volume; not including CIFS protocol
     * request processing or network communication
     */
    private float cifsOtherLatency;

    /**
     * Number of other cifs operations per second to the volume
     */
    private float cifsOtherOps;

    /**
     * Bytes read per second via cifs from the volume
     */
    private float cifsReadData;

    /**
     * Average time for the WAFL filesystem to process CIFS read requests to the volume; not including CIFS protocol request
     * processing or network commu
     */
    private float cifsReadLatency;

    /**
     * Number of cifs reads per second to the volume
     */
    private float cifsReadOps;

    /**
     * Bytes written per second via cifs to the volume
     */
    private float cifsWriteData;

    /**
     * Average time for the WAFL filesystem to process CIFS write requests to the volume; not including CIFS protocol request
     * processing or network communication
     */
    private float cifsWriteLatency;

    /**
     * Number of cifs writes per second to the volume
     */
    private float cifsWriteOps;

    /**
     * Average time for the WAFL filesystem to process other FCP protocol operations to the volume; not including FCP protocol
     * request processing or network communication
     */
    private float fcpOtherLatency;

    /**
     * Number of other block protocol operations per second to the volume
     */
    private float fcpOtherOps;

    /**
     * Bytes read per second via block protocol from the volume
     */
    private float fcpReadData;

    /**
     * Average time for the WAFL filesystem to process FCP protocol read operations to the volume; not including FCP protocol
     * request processing or network communication
     */
    private float fcpReadLatency;

    /**
     * Number of block protocol reads per second to the volume
     */
    private float fcpReadOps;

    /**
     * Bytes written per second via block protocol to the volume
     */
    private float fcpWriteData;

    /**
     * Average time for the WAFL filesystem to process FCP protocol write operations to the volume; not including FCP protocol
     * request processing or network communication
     */
    private float fcpWriteLatency;

    /**
     * Number of block protocol writes per second to the volume
     */
    private float fcpWriteOps;

    /**
     * Average time for the WAFL filesystem to process other iSCSI protocol operations to the volume; not including iSCSI
     * protocol request processing or network communication
     */
    private float iscsiOtherLatency;

    /**
     * Number of other block protocol operations per second to the volume
     */
    private float iscsiOtherOps;

    /**
     * Bytes read per second via block protocol from the volume
     */
    private float iscsiReadData;

    /**
     * Average time for the WAFL filesystem to process iSCSI protocol read operations to the volume; not including iSCSI
     * protocol request processing or network communication
     */
    private float iscsiReadLatency;

    /**
     * Number of block protocol reads per second to the volume
     */
    private float iscsiReadOps;

    /**
     * Bytes written per second via block protocol to the volume
     */
    private float iscsiWriteData;

    /**
     * Average time for the WAFL filesystem to process iSCSI protocol write operations to the volume; not including iSCSI
     * protocol request processing or network communication
     */
    private float iscsiWriteLatency;

    /**
     * Number of block protocol writes per second to the volume
     */
    private float iscsiWriteOps;

    /**
     * Average time for the WAFL filesystem to process other NFS operations to the volume; not including NFS protocol request
     * processing or network communication
     */
    private float nfsOtherLatency;

    /**
     * Number of other NFS operations per second to the volume
     */
    private float nfsOtherOps;

    /**
     * Bytes read per second via NFS from the volume
     */
    private float nfsReadData;

    /**
     * Average time for the WAFL filesystem to process NFS protocol read requests to the volume; not including NFS protocol
     * request processing or network communication
     */
    private float nfsReadLatency;

    /**
     * Number of NFS reads per second to the volume
     */
    private float nfsReadOps;

    /**
     * Bytes written per second via NFS to the volume
     */
    private float nfsWriteData;

    /**
     * Average time for the WAFL filesystem to process NFS protocol write requests to the volume; not including NFS protocol
     * request processing or network communication
     */
    private float nfsWriteLatency;

    /**
     * Number of NFS writes per second to the volume
     */
    private float nfsWriteOps;

    /**
     * Average latency in microseconds for the WAFL filesystem to process other operations to the volume; not including
     * request processing or network communication
     */
    private float otherLatency;

    /**
     * Number of other operations per second to the volume
     */
    private float otherOps;

    /**
     * Bytes read per second from the volume
     */
    private float readData;

    /**
     * Average latency in microseconds for the WAFL filesystem to process read request to the volume; not including request
     * processing or network communication
     */
    private float readLatency;

    /**
     * Number of reads per second to the volume
     */
    private float readOps;

    /**
     * Average time for the WAFL filesystem to process other block protocol requests to the volume; not including block
     * protocol request processing or network communication
     */
    private float sanOtherLatency;

    /**
     * Number of other block protocol operations per second to the volume
     */
    private float sanOtherOps;

    /**
     * Bytes read per second via block protocol from the volume
     */
    private float sanReadData;

    /**
     * Average time for the WAFL filesystem to process block protocol read requests to the volume; not including block
     * protocol request processing or network communication
     */
    private float sanReadLatency;

    /**
     * Number of block protocol reads per second to the volume
     */
    private float sanReadOps;

    /**
     * Bytes written per second via block protocol to the volume
     */
    private float sanWriteData;

    /**
     * Average time for the WAFL filesystem to process block protocol write requests to the volume; including block protocol
     * request processing or network communication
     */
    private float sanWriteLatency;

    /**
     * Number of block protocol writes per second to the volume
     */
    private float sanWriteOps;

    /**
     * Number of operations per second serviced by the volume
     */
    private float totalOps;

    /**
     * Bytes written per second to the volume
     */
    private float writeData;

    /**
     * Average latency in microseconds for the WAFL filesystem to process write request to the volume; not including request
     * processing or network communication
     */
    private float writeLatency;

    /**
     * Number of writes per second to the volume
     */
    private float writeOps;
    
    private float opmVolNfsOps;
    private float opmVolCifsOps;
    private float opmVolIscsiOps;
    private float opmVolFcpOps;
    

    public VolumeMeasurements() {
    }

    public VolumeMeasurements(float avgLatency, float cifsOtherLatency, float cifsOtherOps, float cifsReadData, float cifsReadLatency, float cifsReadOps, float cifsWriteData, float cifsWriteLatency, float cifsWriteOps, float fcpOtherLatency, float fcpOtherOps, float fcpReadData, float fcpReadLatency, float fcpReadOps, float fcpWriteData, float fcpWriteLatency, float fcpWriteOps, float iscsiOtherLatency, float iscsiOtherOps, float iscsiReadData, float iscsiReadLatency, float iscsiReadOps, float iscsiWriteData, float iscsiWriteLatency, float iscsiWriteOps, float nfsOtherLatency, float nfsOtherOps, float nfsReadData, float nfsReadLatency, float nfsReadOps, float nfsWriteData, float nfsWriteLatency, float nfsWriteOps, float otherLatency, float otherOps, float readData, float readLatency, float readOps, float sanOtherLatency, float sanOtherOps, float sanReadData, float sanReadLatency, float sanReadOps, float sanWriteData, float sanWriteLatency, float sanWriteOps, float totalOps, float writeData, float writeLatency, float writeOps, float opmVolNfsOps, float opmVolCifsOps, float opmVolIscsiOps, float opmVolFcpOps) {
        this.avgLatency = avgLatency;
        this.cifsOtherLatency = cifsOtherLatency;
        this.cifsOtherOps = cifsOtherOps;
        this.cifsReadData = cifsReadData;
        this.cifsReadLatency = cifsReadLatency;
        this.cifsReadOps = cifsReadOps;
        this.cifsWriteData = cifsWriteData;
        this.cifsWriteLatency = cifsWriteLatency;
        this.cifsWriteOps = cifsWriteOps;
        this.fcpOtherLatency = fcpOtherLatency;
        this.fcpOtherOps = fcpOtherOps;
        this.fcpReadData = fcpReadData;
        this.fcpReadLatency = fcpReadLatency;
        this.fcpReadOps = fcpReadOps;
        this.fcpWriteData = fcpWriteData;
        this.fcpWriteLatency = fcpWriteLatency;
        this.fcpWriteOps = fcpWriteOps;
        this.iscsiOtherLatency = iscsiOtherLatency;
        this.iscsiOtherOps = iscsiOtherOps;
        this.iscsiReadData = iscsiReadData;
        this.iscsiReadLatency = iscsiReadLatency;
        this.iscsiReadOps = iscsiReadOps;
        this.iscsiWriteData = iscsiWriteData;
        this.iscsiWriteLatency = iscsiWriteLatency;
        this.iscsiWriteOps = iscsiWriteOps;
        this.nfsOtherLatency = nfsOtherLatency;
        this.nfsOtherOps = nfsOtherOps;
        this.nfsReadData = nfsReadData;
        this.nfsReadLatency = nfsReadLatency;
        this.nfsReadOps = nfsReadOps;
        this.nfsWriteData = nfsWriteData;
        this.nfsWriteLatency = nfsWriteLatency;
        this.nfsWriteOps = nfsWriteOps;
        this.otherLatency = otherLatency;
        this.otherOps = otherOps;
        this.readData = readData;
        this.readLatency = readLatency;
        this.readOps = readOps;
        this.sanOtherLatency = sanOtherLatency;
        this.sanOtherOps = sanOtherOps;
        this.sanReadData = sanReadData;
        this.sanReadLatency = sanReadLatency;
        this.sanReadOps = sanReadOps;
        this.sanWriteData = sanWriteData;
        this.sanWriteLatency = sanWriteLatency;
        this.sanWriteOps = sanWriteOps;
        this.totalOps = totalOps;
        this.writeData = writeData;
        this.writeLatency = writeLatency;
        this.writeOps = writeOps;
        this.opmVolCifsOps = opmVolCifsOps; 
        this.opmVolNfsOps = opmVolNfsOps; 
        this.opmVolFcpOps = opmVolFcpOps; 
        this.opmVolIscsiOps = opmVolIscsiOps; 
    }

    public float getAvgLatency() {
        return avgLatency;
    }

    public float getCifsOtherLatency() {
        return cifsOtherLatency;
    }

    public float getCifsOtherOps() {
        return cifsOtherOps;
    }

    public float getCifsReadData() {
        return cifsReadData;
    }

    public float getCifsReadLatency() {
        return cifsReadLatency;
    }

    public float getCifsReadOps() {
        return cifsReadOps;
    }

    public float getCifsWriteData() {
        return cifsWriteData;
    }

    public float getCifsWriteLatency() {
        return cifsWriteLatency;
    }

    public float getCifsWriteOps() {
        return cifsWriteOps;
    }

    public float getFcpOtherLatency() {
        return fcpOtherLatency;
    }

    public float getFcpOtherOps() {
        return fcpOtherOps;
    }

    public float getFcpReadData() {
        return fcpReadData;
    }

    public float getFcpReadLatency() {
        return fcpReadLatency;
    }

    public float getFcpReadOps() {
        return fcpReadOps;
    }

    public float getFcpWriteData() {
        return fcpWriteData;
    }

    public float getFcpWriteLatency() {
        return fcpWriteLatency;
    }

    public float getFcpWriteOps() {
        return fcpWriteOps;
    }

    public float getIscsiOtherLatency() {
        return iscsiOtherLatency;
    }

    public float getIscsiOtherOps() {
        return iscsiOtherOps;
    }

    public float getIscsiReadData() {
        return iscsiReadData;
    }

    public float getIscsiReadLatency() {
        return iscsiReadLatency;
    }

    public float getIscsiReadOps() {
        return iscsiReadOps;
    }

    public float getIscsiWriteData() {
        return iscsiWriteData;
    }

    public float getIscsiWriteLatency() {
        return iscsiWriteLatency;
    }

    public float getIscsiWriteOps() {
        return iscsiWriteOps;
    }

    public float getNfsOtherLatency() {
        return nfsOtherLatency;
    }

    public float getNfsOtherOps() {
        return nfsOtherOps;
    }

    public float getNfsReadData() {
        return nfsReadData;
    }

    public float getNfsReadLatency() {
        return nfsReadLatency;
    }

    public float getNfsReadOps() {
        return nfsReadOps;
    }

    public float getNfsWriteData() {
        return nfsWriteData;
    }

    public float getNfsWriteLatency() {
        return nfsWriteLatency;
    }

    public float getNfsWriteOps() {
        return nfsWriteOps;
    }

    public float getOtherLatency() {
        return otherLatency;
    }

    public float getOtherOps() {
        return otherOps;
    }

    public float getReadData() {
        return readData;
    }

    public float getReadLatency() {
        return readLatency;
    }

    public float getReadOps() {
        return readOps;
    }

    public float getSanOtherLatency() {
        return sanOtherLatency;
    }

    public float getSanOtherOps() {
        return sanOtherOps;
    }

    public float getSanReadData() {
        return sanReadData;
    }

    public float getSanReadLatency() {
        return sanReadLatency;
    }

    public float getSanReadOps() {
        return sanReadOps;
    }

    public float getSanWriteData() {
        return sanWriteData;
    }

    public float getSanWriteLatency() {
        return sanWriteLatency;
    }

    public float getSanWriteOps() {
        return sanWriteOps;
    }

    public float getTotalOps() {
        return totalOps;
    }

    public float getWriteData() {
        return writeData;
    }

    public float getWriteLatency() {
        return writeLatency;
    }

    public float getWriteOps() {
        return writeOps;
   }
    
   public void setAvgLatency(float avgLatency) {
		this.avgLatency = avgLatency;
	}

	public void setCifsOtherLatency(float cifsOtherLatency) {
		this.cifsOtherLatency = cifsOtherLatency;
	}

	public void setCifsOtherOps(float cifsOtherOps) {
		this.cifsOtherOps = cifsOtherOps;
	}

	public void setCifsReadData(float cifsReadData) {
		this.cifsReadData = cifsReadData;
	}

	public void setCifsReadLatency(float cifsReadLatency) {
		this.cifsReadLatency = cifsReadLatency;
	}

	public void setCifsReadOps(float cifsReadOps) {
		this.cifsReadOps = cifsReadOps;
	}

	public void setCifsWriteData(float cifsWriteData) {
		this.cifsWriteData = cifsWriteData;
	}

	public void setCifsWriteLatency(float cifsWriteLatency) {
		this.cifsWriteLatency = cifsWriteLatency;
	}

	public void setCifsWriteOps(float cifsWriteOps) {
		this.cifsWriteOps = cifsWriteOps;
	}

	public void setFcpOtherLatency(float fcpOtherLatency) {
		this.fcpOtherLatency = fcpOtherLatency;
	}

	public void setFcpOtherOps(float fcpOtherOps) {
		this.fcpOtherOps = fcpOtherOps;
	}

	public void setFcpReadData(float fcpReadData) {
		this.fcpReadData = fcpReadData;
	}

	public void setFcpReadLatency(float fcpReadLatency) {
		this.fcpReadLatency = fcpReadLatency;
	}

	public void setFcpReadOps(float fcpReadOps) {
		this.fcpReadOps = fcpReadOps;
	}

	public void setFcpWriteData(float fcpWriteData) {
		this.fcpWriteData = fcpWriteData;
	}

	public void setFcpWriteLatency(float fcpWriteLatency) {
		this.fcpWriteLatency = fcpWriteLatency;
	}

	public void setFcpWriteOps(float fcpWriteOps) {
		this.fcpWriteOps = fcpWriteOps;
	}

	public void setIscsiOtherLatency(float iscsiOtherLatency) {
		this.iscsiOtherLatency = iscsiOtherLatency;
	}

	public void setIscsiOtherOps(float iscsiOtherOps) {
		this.iscsiOtherOps = iscsiOtherOps;
	}

	public void setIscsiReadData(float iscsiReadData) {
		this.iscsiReadData = iscsiReadData;
	}

	public void setIscsiReadLatency(float iscsiReadLatency) {
		this.iscsiReadLatency = iscsiReadLatency;
	}

	public void setIscsiReadOps(float iscsiReadOps) {
		this.iscsiReadOps = iscsiReadOps;
	}

	public void setIscsiWriteData(float iscsiWriteData) {
		this.iscsiWriteData = iscsiWriteData;
	}

	public void setIscsiWriteLatency(float iscsiWriteLatency) {
		this.iscsiWriteLatency = iscsiWriteLatency;
	}

	public void setIscsiWriteOps(float iscsiWriteOps) {
		this.iscsiWriteOps = iscsiWriteOps;
	}

	public void setNfsOtherLatency(float nfsOtherLatency) {
		this.nfsOtherLatency = nfsOtherLatency;
	}

	public void setNfsOtherOps(float nfsOtherOps) {
		this.nfsOtherOps = nfsOtherOps;
	}

	public void setNfsReadData(float nfsReadData) {
		this.nfsReadData = nfsReadData;
	}

	public void setNfsReadLatency(float nfsReadLatency) {
		this.nfsReadLatency = nfsReadLatency;
	}

	public void setNfsReadOps(float nfsReadOps) {
		this.nfsReadOps = nfsReadOps;
	}

	public void setNfsWriteData(float nfsWriteData) {
		this.nfsWriteData = nfsWriteData;
	}

	public void setNfsWriteLatency(float nfsWriteLatency) {
		this.nfsWriteLatency = nfsWriteLatency;
	}

	public void setNfsWriteOps(float nfsWriteOps) {
		this.nfsWriteOps = nfsWriteOps;
	}

	public void setOtherLatency(float otherLatency) {
		this.otherLatency = otherLatency;
	}

	public void setOtherOps(float otherOps) {
		this.otherOps = otherOps;
	}

	public void setReadData(float readData) {
		this.readData = readData;
	}

	public void setReadLatency(float readLatency) {
		this.readLatency = readLatency;
	}

	public void setReadOps(float readOps) {
		this.readOps = readOps;
	}

	public void setSanOtherLatency(float sanOtherLatency) {
		this.sanOtherLatency = sanOtherLatency;
	}

	public void setSanOtherOps(float sanOtherOps) {
		this.sanOtherOps = sanOtherOps;
	}

	public void setSanReadData(float sanReadData) {
		this.sanReadData = sanReadData;
	}

	public void setSanReadLatency(float sanReadLatency) {
		this.sanReadLatency = sanReadLatency;
	}

	public void setSanReadOps(float sanReadOps) {
		this.sanReadOps = sanReadOps;
	}

	public void setSanWriteData(float sanWriteData) {
		this.sanWriteData = sanWriteData;
	}

	public void setSanWriteLatency(float sanWriteLatency) {
		this.sanWriteLatency = sanWriteLatency;
	}

	public void setSanWriteOps(float sanWriteOps) {
		this.sanWriteOps = sanWriteOps;
	}

	public void setTotalOps(float totalOps) {
		this.totalOps = totalOps;
	}

	public void setWriteData(float writeData) {
		this.writeData = writeData;
	}

	public void setWriteLatency(float writeLatency) {
		this.writeLatency = writeLatency;
	}

	public void setWriteOps(float writeOps) {
		this.writeOps = writeOps;
	}

public float getOpmVolNfsOps(){
	   return opmVolNfsOps;
   }
   
   public void setOpmVolNfsOps(float opmVolNfsOps){
	   this.opmVolNfsOps = opmVolNfsOps;
   }

   public float getOpmVolCifsOps() {
	   return opmVolCifsOps;
   }

   public void setOpmVolCifsOps(float opmVolCifsOps) {
	   this.opmVolCifsOps = opmVolCifsOps;
   }

   public float getOpmVolIscsiOps() {
	   return opmVolIscsiOps;
   }

   public void setOpmVolIscsiOps(float opmVolIscsiOps) {
	   this.opmVolIscsiOps = opmVolIscsiOps;
   }

   public float getOpmVolFcpOps() {
	   return opmVolFcpOps;
   }

   public void setOpmVolFcpOps(float opmVolFcpOps) {
	   this.opmVolFcpOps = opmVolFcpOps;
   }

	@Override
    public int hashCode() {
        int result = (avgLatency != +0.0f ? Float.floatToIntBits(avgLatency) : 0);
        result = 31 * result + (cifsOtherLatency != +0.0f ? Float.floatToIntBits(cifsOtherLatency) : 0);
        result = 31 * result + (cifsOtherOps != +0.0f ? Float.floatToIntBits(cifsOtherOps) : 0);
        result = 31 * result + (cifsReadData != +0.0f ? Float.floatToIntBits(cifsReadData) : 0);
        result = 31 * result + (cifsReadLatency != +0.0f ? Float.floatToIntBits(cifsReadLatency) : 0);
        result = 31 * result + (cifsReadOps != +0.0f ? Float.floatToIntBits(cifsReadOps) : 0);
        result = 31 * result + (cifsWriteData != +0.0f ? Float.floatToIntBits(cifsWriteData) : 0);
        result = 31 * result + (cifsWriteLatency != +0.0f ? Float.floatToIntBits(cifsWriteLatency) : 0);
        result = 31 * result + (cifsWriteOps != +0.0f ? Float.floatToIntBits(cifsWriteOps) : 0);
        result = 31 * result + (fcpOtherLatency != +0.0f ? Float.floatToIntBits(fcpOtherLatency) : 0);
        result = 31 * result + (fcpOtherOps != +0.0f ? Float.floatToIntBits(fcpOtherOps) : 0);
        result = 31 * result + (fcpReadData != +0.0f ? Float.floatToIntBits(fcpReadData) : 0);
        result = 31 * result + (fcpReadLatency != +0.0f ? Float.floatToIntBits(fcpReadLatency) : 0);
        result = 31 * result + (fcpReadOps != +0.0f ? Float.floatToIntBits(fcpReadOps) : 0);
        result = 31 * result + (fcpWriteData != +0.0f ? Float.floatToIntBits(fcpWriteData) : 0);
        result = 31 * result + (fcpWriteLatency != +0.0f ? Float.floatToIntBits(fcpWriteLatency) : 0);
        result = 31 * result + (fcpWriteOps != +0.0f ? Float.floatToIntBits(fcpWriteOps) : 0);
        result = 31 * result + (iscsiOtherLatency != +0.0f ? Float.floatToIntBits(iscsiOtherLatency) : 0);
        result = 31 * result + (iscsiOtherOps != +0.0f ? Float.floatToIntBits(iscsiOtherOps) : 0);
        result = 31 * result + (iscsiReadData != +0.0f ? Float.floatToIntBits(iscsiReadData) : 0);
        result = 31 * result + (iscsiReadLatency != +0.0f ? Float.floatToIntBits(iscsiReadLatency) : 0);
        result = 31 * result + (iscsiReadOps != +0.0f ? Float.floatToIntBits(iscsiReadOps) : 0);
        result = 31 * result + (iscsiWriteData != +0.0f ? Float.floatToIntBits(iscsiWriteData) : 0);
        result = 31 * result + (iscsiWriteLatency != +0.0f ? Float.floatToIntBits(iscsiWriteLatency) : 0);
        result = 31 * result + (iscsiWriteOps != +0.0f ? Float.floatToIntBits(iscsiWriteOps) : 0);
        result = 31 * result + (nfsOtherLatency != +0.0f ? Float.floatToIntBits(nfsOtherLatency) : 0);
        result = 31 * result + (nfsOtherOps != +0.0f ? Float.floatToIntBits(nfsOtherOps) : 0);
        result = 31 * result + (nfsReadData != +0.0f ? Float.floatToIntBits(nfsReadData) : 0);
        result = 31 * result + (nfsReadLatency != +0.0f ? Float.floatToIntBits(nfsReadLatency) : 0);
        result = 31 * result + (nfsReadOps != +0.0f ? Float.floatToIntBits(nfsReadOps) : 0);
        result = 31 * result + (nfsWriteData != +0.0f ? Float.floatToIntBits(nfsWriteData) : 0);
        result = 31 * result + (nfsWriteLatency != +0.0f ? Float.floatToIntBits(nfsWriteLatency) : 0);
        result = 31 * result + (nfsWriteOps != +0.0f ? Float.floatToIntBits(nfsWriteOps) : 0);
        result = 31 * result + (otherLatency != +0.0f ? Float.floatToIntBits(otherLatency) : 0);
        result = 31 * result + (otherOps != +0.0f ? Float.floatToIntBits(otherOps) : 0);
        result = 31 * result + (readData != +0.0f ? Float.floatToIntBits(readData) : 0);
        result = 31 * result + (readLatency != +0.0f ? Float.floatToIntBits(readLatency) : 0);
        result = 31 * result + (readOps != +0.0f ? Float.floatToIntBits(readOps) : 0);
        result = 31 * result + (sanOtherLatency != +0.0f ? Float.floatToIntBits(sanOtherLatency) : 0);
        result = 31 * result + (sanOtherOps != +0.0f ? Float.floatToIntBits(sanOtherOps) : 0);
        result = 31 * result + (sanReadData != +0.0f ? Float.floatToIntBits(sanReadData) : 0);
        result = 31 * result + (sanReadLatency != +0.0f ? Float.floatToIntBits(sanReadLatency) : 0);
        result = 31 * result + (sanReadOps != +0.0f ? Float.floatToIntBits(sanReadOps) : 0);
        result = 31 * result + (sanWriteData != +0.0f ? Float.floatToIntBits(sanWriteData) : 0);
        result = 31 * result + (sanWriteLatency != +0.0f ? Float.floatToIntBits(sanWriteLatency) : 0);
        result = 31 * result + (sanWriteOps != +0.0f ? Float.floatToIntBits(sanWriteOps) : 0);
        result = 31 * result + (totalOps != +0.0f ? Float.floatToIntBits(totalOps) : 0);
        result = 31 * result + (writeData != +0.0f ? Float.floatToIntBits(writeData) : 0);
        result = 31 * result + (writeLatency != +0.0f ? Float.floatToIntBits(writeLatency) : 0);
        result = 31 * result + (writeOps != +0.0f ? Float.floatToIntBits(writeOps) : 0);
        result = 31 * result + (opmVolCifsOps != +0.0f ? Float.floatToIntBits(opmVolCifsOps) : 0);
        result = 31 * result + (opmVolFcpOps != +0.0f ? Float.floatToIntBits(opmVolFcpOps) : 0);
        result = 31 * result + (opmVolIscsiOps != +0.0f ? Float.floatToIntBits(opmVolIscsiOps) : 0);
        result = 31 * result + (opmVolNfsOps != +0.0f ? Float.floatToIntBits(opmVolNfsOps) : 0);
        return result;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof VolumeMeasurements)) return false;

        VolumeMeasurements that = (VolumeMeasurements) o;

        if (Float.compare(that.avgLatency, avgLatency) != 0) return false;
        if (Float.compare(that.cifsOtherLatency, cifsOtherLatency) != 0) return false;
        if (Float.compare(that.cifsOtherOps, cifsOtherOps) != 0) return false;
        if (Float.compare(that.cifsReadData, cifsReadData) != 0) return false;
        if (Float.compare(that.cifsReadLatency, cifsReadLatency) != 0) return false;
        if (Float.compare(that.cifsReadOps, cifsReadOps) != 0) return false;
        if (Float.compare(that.cifsWriteData, cifsWriteData) != 0) return false;
        if (Float.compare(that.cifsWriteLatency, cifsWriteLatency) != 0) return false;
        if (Float.compare(that.cifsWriteOps, cifsWriteOps) != 0) return false;
        if (Float.compare(that.fcpOtherLatency, fcpOtherLatency) != 0) return false;
        if (Float.compare(that.fcpOtherOps, fcpOtherOps) != 0) return false;
        if (Float.compare(that.fcpReadData, fcpReadData) != 0) return false;
        if (Float.compare(that.fcpReadLatency, fcpReadLatency) != 0) return false;
        if (Float.compare(that.fcpReadOps, fcpReadOps) != 0) return false;
        if (Float.compare(that.fcpWriteData, fcpWriteData) != 0) return false;
        if (Float.compare(that.fcpWriteLatency, fcpWriteLatency) != 0) return false;
        if (Float.compare(that.fcpWriteOps, fcpWriteOps) != 0) return false;
        if (Float.compare(that.iscsiOtherLatency, iscsiOtherLatency) != 0) return false;
        if (Float.compare(that.iscsiOtherOps, iscsiOtherOps) != 0) return false;
        if (Float.compare(that.iscsiReadData, iscsiReadData) != 0) return false;
        if (Float.compare(that.iscsiReadLatency, iscsiReadLatency) != 0) return false;
        if (Float.compare(that.iscsiReadOps, iscsiReadOps) != 0) return false;
        if (Float.compare(that.iscsiWriteData, iscsiWriteData) != 0) return false;
        if (Float.compare(that.iscsiWriteLatency, iscsiWriteLatency) != 0) return false;
        if (Float.compare(that.iscsiWriteOps, iscsiWriteOps) != 0) return false;
        if (Float.compare(that.nfsOtherLatency, nfsOtherLatency) != 0) return false;
        if (Float.compare(that.nfsOtherOps, nfsOtherOps) != 0) return false;
        if (Float.compare(that.nfsReadData, nfsReadData) != 0) return false;
        if (Float.compare(that.nfsReadLatency, nfsReadLatency) != 0) return false;
        if (Float.compare(that.nfsReadOps, nfsReadOps) != 0) return false;
        if (Float.compare(that.nfsWriteData, nfsWriteData) != 0) return false;
        if (Float.compare(that.nfsWriteLatency, nfsWriteLatency) != 0) return false;
        if (Float.compare(that.nfsWriteOps, nfsWriteOps) != 0) return false;
        if (Float.compare(that.otherLatency, otherLatency) != 0) return false;
        if (Float.compare(that.otherOps, otherOps) != 0) return false;
        if (Float.compare(that.readData, readData) != 0) return false;
        if (Float.compare(that.readLatency, readLatency) != 0) return false;
        if (Float.compare(that.readOps, readOps) != 0) return false;
        if (Float.compare(that.sanOtherLatency, sanOtherLatency) != 0) return false;
        if (Float.compare(that.sanOtherOps, sanOtherOps) != 0) return false;
        if (Float.compare(that.sanReadData, sanReadData) != 0) return false;
        if (Float.compare(that.sanReadLatency, sanReadLatency) != 0) return false;
        if (Float.compare(that.sanReadOps, sanReadOps) != 0) return false;
        if (Float.compare(that.sanWriteData, sanWriteData) != 0) return false;
        if (Float.compare(that.sanWriteLatency, sanWriteLatency) != 0) return false;
        if (Float.compare(that.sanWriteOps, sanWriteOps) != 0) return false;
        if (Float.compare(that.totalOps, totalOps) != 0) return false;
        if (Float.compare(that.writeData, writeData) != 0) return false;
        if (Float.compare(that.writeLatency, writeLatency) != 0) return false;
        if (Float.compare(that.writeOps, writeOps) != 0) return false;
        if (Float.compare(that.opmVolCifsOps, opmVolCifsOps) != 0) return false;
        if (Float.compare(that.opmVolFcpOps, opmVolFcpOps) != 0) return false;
        if (Float.compare(that.opmVolIscsiOps, opmVolIscsiOps) != 0) return false;
        if (Float.compare(that.opmVolNfsOps, opmVolNfsOps) != 0) return false;

        return true;
    }

    @Override
    public String toString() {
        return "VolumeMeasurements{" +
                "avgLatency=" + avgLatency +
                ", cifsOtherLatency=" + cifsOtherLatency +
                ", cifsOtherOps=" + cifsOtherOps +
                ", cifsReadData=" + cifsReadData +
                ", cifsReadLatency=" + cifsReadLatency +
                ", cifsReadOps=" + cifsReadOps +
                ", cifsWriteData=" + cifsWriteData +
                ", cifsWriteLatency=" + cifsWriteLatency +
                ", cifsWriteOps=" + cifsWriteOps +
                ", fcpOtherLatency=" + fcpOtherLatency +
                ", fcpOtherOps=" + fcpOtherOps +
                ", fcpReadData=" + fcpReadData +
                ", fcpReadLatency=" + fcpReadLatency +
                ", fcpReadOps=" + fcpReadOps +
                ", fcpWriteData=" + fcpWriteData +
                ", fcpWriteLatency=" + fcpWriteLatency +
                ", fcpWriteOps=" + fcpWriteOps +
                ", iscsiOtherLatency=" + iscsiOtherLatency +
                ", iscsiOtherOps=" + iscsiOtherOps +
                ", iscsiReadData=" + iscsiReadData +
                ", iscsiReadLatency=" + iscsiReadLatency +
                ", iscsiReadOps=" + iscsiReadOps +
                ", iscsiWriteData=" + iscsiWriteData +
                ", iscsiWriteLatency=" + iscsiWriteLatency +
                ", iscsiWriteOps=" + iscsiWriteOps +
                ", nfsOtherLatency=" + nfsOtherLatency +
                ", nfsOtherOps=" + nfsOtherOps +
                ", nfsReadData=" + nfsReadData +
                ", nfsReadLatency=" + nfsReadLatency +
                ", nfsReadOps=" + nfsReadOps +
                ", nfsWriteData=" + nfsWriteData +
                ", nfsWriteLatency=" + nfsWriteLatency +
                ", nfsWriteOps=" + nfsWriteOps +
                ", otherLatency=" + otherLatency +
                ", otherOps=" + otherOps +
                ", readData=" + readData +
                ", readLatency=" + readLatency +
                ", readOps=" + readOps +
                ", sanOtherLatency=" + sanOtherLatency +
                ", sanOtherOps=" + sanOtherOps +
                ", sanReadData=" + sanReadData +
                ", sanReadLatency=" + sanReadLatency +
                ", sanReadOps=" + sanReadOps +
                ", sanWriteData=" + sanWriteData +
                ", sanWriteLatency=" + sanWriteLatency +
                ", sanWriteOps=" + sanWriteOps +
                ", totalOps=" + totalOps +
                ", writeData=" + writeData +
                ", writeLatency=" + writeLatency +
                ", writeOps=" + writeOps +
                ", opmVolCifsOps=" + opmVolCifsOps +
                ", opmVolFcpOps=" + opmVolFcpOps +
                ", opmVolIscsiOps=" + opmVolIscsiOps +
                ", opmVolNfsOps=" + opmVolNfsOps +
                '}';
    }
}
