package com.netapp.oci.netapp.performance.volume.interfaces.data;

import java.io.Serializable;
import java.math.BigInteger;

public class VolumeRawMeasurements implements Serializable{
	private static final long serialVersionUID = 542381750472106020L;

	/**
     * Average latency in microseconds for the WFL Filesystem to process all the operations on the volume; not including
     * request processing or network communication
     */
    private BigInteger avgLatency;

    /**
     * Average time for the WAFL filesystem to process other CIFS operations to the volume; not including CIFS protocol
     * request processing or network communication
     */
    private BigInteger cifsOtherLatency;

    /**
     * Number of other cifs operations per second to the volume
     */
    private BigInteger cifsOtherOps;

    /**
     * Average time for the WAFL filesystem to process CIFS read requests to the volume; not including CIFS protocol request
     * processing or network commu
     */
    private BigInteger cifsReadLatency;

    /**
     * Number of cifs reads per second to the volume
     */
    private BigInteger cifsReadOps;

    /**
     * Average time for the WAFL filesystem to process CIFS write requests to the volume; not including CIFS protocol request
     * processing or network communication
     */
    private BigInteger cifsWriteLatency;

    /**
     * Number of cifs writes per second to the volume
     */
    private BigInteger cifsWriteOps;

    /**
     * Average time for the WAFL filesystem to process other FCP protocol operations to the volume; not including FCP protocol
     * request processing or network communication
     */
    private BigInteger fcpOtherLatency;

    /**
     * Number of other block protocol operations per second to the volume
     */
    private BigInteger fcpOtherOps;

    /**
     * Average time for the WAFL filesystem to process FCP protocol read operations to the volume; not including FCP protocol
     * request processing or network communication
     */
    private BigInteger fcpReadLatency;

    /**
     * Number of block protocol reads per second to the volume
     */
    private BigInteger fcpReadOps;

    /**
     * Average time for the WAFL filesystem to process FCP protocol write operations to the volume; not including FCP protocol
     * request processing or network communication
     */
    private BigInteger fcpWriteLatency;

    /**
     * Number of block protocol writes per second to the volume
     */
    private BigInteger fcpWriteOps;

    /**
     * Average time for the WAFL filesystem to process other iSCSI protocol operations to the volume; not including iSCSI
     * protocol request processing or network communication
     */
    private BigInteger iscsiOtherLatency;

    /**
     * Number of other block protocol operations per second to the volume
     */
    private BigInteger iscsiOtherOps;

    /**
     * Average time for the WAFL filesystem to process iSCSI protocol read operations to the volume; not including iSCSI
     * protocol request processing or network communication
     */
    private BigInteger iscsiReadLatency;

    /**
     * Number of block protocol reads per second to the volume
     */
    private BigInteger iscsiReadOps;

    /**
     * Average time for the WAFL filesystem to process iSCSI protocol write operations to the volume; not including iSCSI
     * protocol request processing or network communication
     */
    private BigInteger iscsiWriteLatency;

    /**
     * Number of block protocol writes per second to the volume
     */
    private BigInteger iscsiWriteOps;

    /**
     * Average time for the WAFL filesystem to process other NFS operations to the volume; not including NFS protocol request
     * processing or network communication
     */
    private BigInteger nfsOtherLatency;

    /**
     * Number of other NFS operations per second to the volume
     */
    private BigInteger nfsOtherOps;

    /**
     * Average time for the WAFL filesystem to process NFS protocol read requests to the volume; not including NFS protocol
     * request processing or network communication
     */
    private BigInteger nfsReadLatency;

    /**
     * Number of NFS reads per second to the volume
     */
    private BigInteger nfsReadOps;

    /**
     * Average time for the WAFL filesystem to process NFS protocol write requests to the volume; not including NFS protocol
     * request processing or network communication
     */
    private BigInteger nfsWriteLatency;

    /**
     * Number of NFS writes per second to the volume
     */
    private BigInteger nfsWriteOps;

    /**
     * Average latency in microseconds for the WAFL filesystem to process other operations to the volume; not including
     * request processing or network communication
     */
    private BigInteger otherLatency;

    /**
     * Number of other operations per second to the volume
     */
    private BigInteger otherOps;

    /**
     * Average latency in microseconds for the WAFL filesystem to process read request to the volume; not including request
     * processing or network communication
     */
    private BigInteger readLatency;

    /**
     * Number of reads per second to the volume
     */
    private BigInteger readOps;

    /**
     * Number of operations per second serviced by the volume
     */
    private BigInteger totalOps;

    /**
     * Average latency in microseconds for the WAFL filesystem to process protocol other request to the volume; not including request
     * processing or network communication
     */
    private String totalProtocolOtherLatency;

    /**
     * Average latency in microseconds for the WAFL filesystem to process protocol read request to the volume; not including request
     * processing or network communication
     */
    private String totalProtocolReadLatency;

    /**
     * Average latency in microseconds for the WAFL filesystem to process protocol write request to the volume; not including request
     * processing or network communication
     */
    private String totalProtocolWriteLatency;

    /**
     * Average latency in microseconds for the WAFL filesystem to process write request to the volume; not including request
     * processing or network communication
     */
    private BigInteger writeLatency;
    
    /**
     * Number of writes per second to the volume
     */
    private BigInteger writeOps;
    
    private BigInteger opmVolNfsOps;
    
    private BigInteger opmVolCifsOps;
    
    private BigInteger opmVolIscsiOps;
    
    private BigInteger opmVolFcpOps;
    
    public VolumeRawMeasurements() {
    }

	public VolumeRawMeasurements(BigInteger avgLatency,
			BigInteger cifsOtherLatency, BigInteger cifsOtherOps,
			BigInteger cifsReadLatency, BigInteger cifsReadOps,
			BigInteger cifsWriteLatency, BigInteger cifsWriteOps,
			BigInteger fcpOtherLatency, BigInteger fcpOtherOps,
			BigInteger fcpReadLatency, BigInteger fcpReadOps,
			BigInteger fcpWriteLatency, BigInteger fcpWriteOps,
			BigInteger iscsiOtherLatency, BigInteger iscsiOtherOps,
			BigInteger iscsiReadLatency, BigInteger iscsiReadOps,
			BigInteger iscsiWriteLatency, BigInteger iscsiWriteOps,
			BigInteger nfsOtherLatency, BigInteger nfsOtherOps,
			BigInteger nfsReadLatency, BigInteger nfsReadOps,
			BigInteger nfsWriteLatency, BigInteger nfsWriteOps,
			BigInteger otherLatency, BigInteger otherOps,
			BigInteger readLatency, BigInteger readOps, BigInteger totalOps,
			String totalProtocolOtherLatency,
			String totalProtocolReadLatency,
			String totalProtocolWriteLatency, BigInteger writeLatency,
			BigInteger writeOps, BigInteger opmVolNfsOps, BigInteger opmVolCifsOps, BigInteger opmVolIscsiOps, BigInteger opmVolFcpOps) {
		super();
		this.avgLatency = avgLatency;
		this.cifsOtherLatency = cifsOtherLatency;
		this.cifsOtherOps = cifsOtherOps;
		this.cifsReadLatency = cifsReadLatency;
		this.cifsReadOps = cifsReadOps;
		this.cifsWriteLatency = cifsWriteLatency;
		this.cifsWriteOps = cifsWriteOps;
		this.fcpOtherLatency = fcpOtherLatency;
		this.fcpOtherOps = fcpOtherOps;
		this.fcpReadLatency = fcpReadLatency;
		this.fcpReadOps = fcpReadOps;
		this.fcpWriteLatency = fcpWriteLatency;
		this.fcpWriteOps = fcpWriteOps;
		this.iscsiOtherLatency = iscsiOtherLatency;
		this.iscsiOtherOps = iscsiOtherOps;
		this.iscsiReadLatency = iscsiReadLatency;
		this.iscsiReadOps = iscsiReadOps;
		this.iscsiWriteLatency = iscsiWriteLatency;
		this.iscsiWriteOps = iscsiWriteOps;
		this.nfsOtherLatency = nfsOtherLatency;
		this.nfsOtherOps = nfsOtherOps;
		this.nfsReadLatency = nfsReadLatency;
		this.nfsReadOps = nfsReadOps;
		this.nfsWriteLatency = nfsWriteLatency;
		this.nfsWriteOps = nfsWriteOps;
		this.otherLatency = otherLatency;
		this.otherOps = otherOps;
		this.readLatency = readLatency;
		this.readOps = readOps;
		this.totalOps = totalOps;
		this.totalProtocolOtherLatency = totalProtocolOtherLatency;
		this.totalProtocolReadLatency = totalProtocolReadLatency;
		this.totalProtocolWriteLatency = totalProtocolWriteLatency;
		this.writeLatency = writeLatency;
		this.writeOps = writeOps;
		this.opmVolCifsOps = opmVolCifsOps;
		this.opmVolFcpOps = opmVolFcpOps;
		this.opmVolIscsiOps = opmVolIscsiOps;
		this.opmVolNfsOps = opmVolNfsOps;
	}

	public BigInteger getAvgLatency() {
		return avgLatency;
	}

	public void setAvgLatency(BigInteger avgLatency) {
		this.avgLatency = avgLatency;
	}

	public BigInteger getCifsOtherLatency() {
		return cifsOtherLatency;
	}

	public void setCifsOtherLatency(BigInteger cifsOtherLatency) {
		this.cifsOtherLatency = cifsOtherLatency;
	}

	public BigInteger getCifsOtherOps() {
		return cifsOtherOps;
	}

	public void setCifsOtherOps(BigInteger cifsOtherOps) {
		this.cifsOtherOps = cifsOtherOps;
	}

	public BigInteger getCifsReadLatency() {
		return cifsReadLatency;
	}

	public void setCifsReadLatency(BigInteger cifsReadLatency) {
		this.cifsReadLatency = cifsReadLatency;
	}

	public BigInteger getCifsReadOps() {
		return cifsReadOps;
	}

	public void setCifsReadOps(BigInteger cifsReadOps) {
		this.cifsReadOps = cifsReadOps;
	}

	public BigInteger getCifsWriteLatency() {
		return cifsWriteLatency;
	}

	public void setCifsWriteLatency(BigInteger cifsWriteLatency) {
		this.cifsWriteLatency = cifsWriteLatency;
	}

	public BigInteger getCifsWriteOps() {
		return cifsWriteOps;
	}

	public void setCifsWriteOps(BigInteger cifsWriteOps) {
		this.cifsWriteOps = cifsWriteOps;
	}

	public BigInteger getFcpOtherLatency() {
		return fcpOtherLatency;
	}

	public void setFcpOtherLatency(BigInteger fcpOtherLatency) {
		this.fcpOtherLatency = fcpOtherLatency;
	}

	public BigInteger getFcpOtherOps() {
		return fcpOtherOps;
	}

	public void setFcpOtherOps(BigInteger fcpOtherOps) {
		this.fcpOtherOps = fcpOtherOps;
	}

	public BigInteger getFcpReadLatency() {
		return fcpReadLatency;
	}

	public void setFcpReadLatency(BigInteger fcpReadLatency) {
		this.fcpReadLatency = fcpReadLatency;
	}

	public BigInteger getFcpReadOps() {
		return fcpReadOps;
	}

	public void setFcpReadOps(BigInteger fcpReadOps) {
		this.fcpReadOps = fcpReadOps;
	}

	public BigInteger getFcpWriteLatency() {
		return fcpWriteLatency;
	}

	public void setFcpWriteLatency(BigInteger fcpWriteLatency) {
		this.fcpWriteLatency = fcpWriteLatency;
	}

	public BigInteger getFcpWriteOps() {
		return fcpWriteOps;
	}

	public void setFcpWriteOps(BigInteger fcpWriteOps) {
		this.fcpWriteOps = fcpWriteOps;
	}

	public BigInteger getIscsiOtherLatency() {
		return iscsiOtherLatency;
	}

	public void setIscsiOtherLatency(BigInteger iscsiOtherLatency) {
		this.iscsiOtherLatency = iscsiOtherLatency;
	}

	public BigInteger getIscsiOtherOps() {
		return iscsiOtherOps;
	}

	public void setIscsiOtherOps(BigInteger iscsiOtherOps) {
		this.iscsiOtherOps = iscsiOtherOps;
	}

	public BigInteger getIscsiReadLatency() {
		return iscsiReadLatency;
	}

	public void setIscsiReadLatency(BigInteger iscsiReadLatency) {
		this.iscsiReadLatency = iscsiReadLatency;
	}

	public BigInteger getIscsiReadOps() {
		return iscsiReadOps;
	}

	public void setIscsiReadOps(BigInteger iscsiReadOps) {
		this.iscsiReadOps = iscsiReadOps;
	}

	public BigInteger getIscsiWriteLatency() {
		return iscsiWriteLatency;
	}

	public void setIscsiWriteLatency(BigInteger iscsiWriteLatency) {
		this.iscsiWriteLatency = iscsiWriteLatency;
	}

	public BigInteger getIscsiWriteOps() {
		return iscsiWriteOps;
	}

	public void setIscsiWriteOps(BigInteger iscsiWriteOps) {
		this.iscsiWriteOps = iscsiWriteOps;
	}

	public BigInteger getNfsOtherLatency() {
		return nfsOtherLatency;
	}

	public void setNfsOtherLatency(BigInteger nfsOtherLatency) {
		this.nfsOtherLatency = nfsOtherLatency;
	}

	public BigInteger getNfsOtherOps() {
		return nfsOtherOps;
	}

	public void setNfsOtherOps(BigInteger nfsOtherOps) {
		this.nfsOtherOps = nfsOtherOps;
	}

	public BigInteger getNfsReadLatency() {
		return nfsReadLatency;
	}

	public void setNfsReadLatency(BigInteger nfsReadLatency) {
		this.nfsReadLatency = nfsReadLatency;
	}

	public BigInteger getNfsReadOps() {
		return nfsReadOps;
	}

	public void setNfsReadOps(BigInteger nfsReadOps) {
		this.nfsReadOps = nfsReadOps;
	}

	public BigInteger getNfsWriteLatency() {
		return nfsWriteLatency;
	}

	public void setNfsWriteLatency(BigInteger nfsWriteLatency) {
		this.nfsWriteLatency = nfsWriteLatency;
	}

	public BigInteger getNfsWriteOps() {
		return nfsWriteOps;
	}

	public void setNfsWriteOps(BigInteger nfsWriteOps) {
		this.nfsWriteOps = nfsWriteOps;
	}

	public BigInteger getOtherLatency() {
		return otherLatency;
	}

	public void setOtherLatency(BigInteger otherLatency) {
		this.otherLatency = otherLatency;
	}

	public BigInteger getOtherOps() {
		return otherOps;
	}

	public void setOtherOps(BigInteger otherOps) {
		this.otherOps = otherOps;
	}

	public BigInteger getReadLatency() {
		return readLatency;
	}

	public void setReadLatency(BigInteger readLatency) {
		this.readLatency = readLatency;
	}

	public BigInteger getReadOps() {
		return readOps;
	}

	public void setReadOps(BigInteger readOps) {
		this.readOps = readOps;
	}

	public BigInteger getTotalOps() {
		return totalOps;
	}

	public void setTotalOps(BigInteger totalOps) {
		this.totalOps = totalOps;
	}

	public String getTotalProtocolOtherLatency() {
		return totalProtocolOtherLatency;
	}

	public void setTotalProtocolOtherLatency(String totalProtocolOtherLatency) {
		this.totalProtocolOtherLatency = totalProtocolOtherLatency;
	}

	public String getTotalProtocolReadLatency() {
		return totalProtocolReadLatency;
	}

	public void setTotalProtocolReadLatency(String totalProtocolReadLatency) {
		this.totalProtocolReadLatency = totalProtocolReadLatency;
	}

	public String getTotalProtocolWriteLatency() {
		return totalProtocolWriteLatency;
	}

	public void setTotalProtocolWriteLatency(String totalProtocolWriteLatency) {
		this.totalProtocolWriteLatency = totalProtocolWriteLatency;
	}

	public BigInteger getWriteLatency() {
		return writeLatency;
	}

	public void setWriteLatency(BigInteger writeLatency) {
		this.writeLatency = writeLatency;
	}

	public BigInteger getWriteOps() {
		return writeOps;
	}

	public BigInteger getOpmVolNfsOps() {
		return opmVolNfsOps;
	}

	public void setOpmVolNfsOps(BigInteger opmVolNfsOps) {
		this.opmVolNfsOps = opmVolNfsOps;
	}

	public BigInteger getOpmVolCifsOps() {
		return opmVolCifsOps;
	}

	public void setOpmVolCifsOps(BigInteger opmVolCifsOps) {
		this.opmVolCifsOps = opmVolCifsOps;
	}

	public BigInteger getOpmVolIscsiOps() {
		return opmVolIscsiOps;
	}

	public void setOpmVolIscsiOps(BigInteger opmVolIscsiOps) {
		this.opmVolIscsiOps = opmVolIscsiOps;
	}

	public BigInteger getOpmVolFcpOps() {
		return opmVolFcpOps;
	}

	public void setOpmVolFcpOps(BigInteger opmVolFcpOps) {
		this.opmVolFcpOps = opmVolFcpOps;
	}

	public static long getSerialversionuid() {
		return serialVersionUID;
	}

	public void setWriteOps(BigInteger writeOps) {
		this.writeOps = writeOps;
	}

}
