package com.netapp.oci.netapp.performance.volume.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Anton Sergeev
 */
@Remote
public interface VolumePerformanceQueryRemote {
    String JNDI_NAME = "VolumePerformanceQueryRemote";

    /**
     * Gets volume samples for period of time
     *
     * @param volumeIds set of volume ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of volume id to list ordered by time of volume samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<VolumeMeasurements>>> getSamples(Set<Long> volumeIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets volume samples for period of time
     *
     * @param containerId container id
     * @param volumeIds set of volume ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of volume id to list ordered by time of volume samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<VolumeMeasurements>>> getSamples(Long containerId, Set<Long> volumeIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given volume id's (volumeIds). The map returned will contain only volume ids that has a sample.
     * volume ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the volume ids passed.
     *
     * @param vServerId vServer id
     * @param volumeIds set of volume ids
     * @return map of volume id to latest sample SampleInfo<VolumeMeasurements>. volume ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<VolumeMeasurements>> getLatestSample(Long vServerId, Set<Long> volumeIds) throws NetappInterfaceException;

    /**
     * Gets volume samples for period of time
     *
     * @param containerId container id
     * @param volumeId volume Id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of volume samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<VolumeMeasurements>> getSamples(Long containerId, Long volumeId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets volume summaries for volume ids
     *
     * @param volumeIds set of volume ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of volume id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<VolumeMeasurements>>> getSummaries(Set<Long> volumeIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets volume summaries for volume ids
     *
     * @param vServerId vServer Id
     * @param volumeIds set of volume ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of volume id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<VolumeMeasurements>>> getSummaries(Long vServerId, Set<Long> volumeIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}
