package com.netapp.oci.netapp.performance.vserver.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.vserver.interfaces.data.VolumeVserverMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Nancy Walia
 *         Date: Jan 17, 2012
 */
@Remote
public interface VolumeVserverPerformanceQueryRemote {
    String JNDI_NAME = "VolumeVserverPerformanceQueryRemote";

    /**
     * Gets vserver samples for period of time
     *
     * @param vserverIds set of vserver ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of vserver id to list ordered by time of vserver samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<VolumeVserverMeasurements>>> getSamples(Set<Long> vserverIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets vserver samples for period of time
     *
     * @param containerId container id
     * @param vserverIds set of vserver ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of vserver id to list ordered by time of vserver samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<VolumeVserverMeasurements>>> getSamples(Long containerId, Set<Long> vserverIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets vserver samples for period of time
     *
     * @param containerId container id
     * @param vserverId vserver id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of vserver samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<VolumeVserverMeasurements>> getSamples(Long containerId, Long vserverId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given vServer id's (vServerIds). The map returned will contain only vServer ids that has a sample.
     * vServer ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the vServer ids passed.
     *
     * @param clusterId cluster id
     * @param vServerIds set of vServer ids
     * @return map of vServer id to latest sample SampleInfo<VserverMeasurements> vServer ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<VolumeVserverMeasurements>> getLatestSample(Long clusterId, Set<Long> vServerIds) throws NetappInterfaceException;

    /**
     * Gets vserver summaries for vserver ids
     *
     * @param vserverIds set of vserver ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of vserver id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<VolumeVserverMeasurements>>> getSummaries(Set<Long> vserverIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets vserver summaries for vserver ids
     *
     * @param clusterId cluster id
     * @param vserverIds set of vserver ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of vserver id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<VolumeVserverMeasurements>>> getSummaries(Long clusterId, Set<Long> vserverIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * returns true if samples exist over the time period
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}

