DROP DATABASE IF EXISTS netapp_performance;
CREATE DATABASE netapp_performance;

DROP DATABASE IF EXISTS netapp_performance_view;
CREATE DATABASE netapp_performance_view;

DROP DATABASE IF EXISTS netapp_model;
CREATE DATABASE netapp_model;

DROP DATABASE IF EXISTS netapp_model_view;
CREATE DATABASE netapp_model_view;

GRANT SELECT, SHOW VIEW ON netapp_model_view.* TO 'essentials_views'@'%';
GRANT SELECT, SHOW VIEW ON netapp_performance_view.* TO 'essentials_views'@'%';

CREATE TABLE netapp_model.update_history (
    starttime BIGINT(20) NOT NULL DEFAULT '0',
	duration BIGINT(20) NOT NULL DEFAULT '0',
	originid BIGINT(20) NOT NULL,
	physicalchange INT(11) NOT NULL DEFAULT '0',
	runnerchange INT(11) NOT NULL DEFAULT '0',
	PRIMARY KEY (starttime)
) ENGINE=INNODB;

CREATE TABLE netapp_model.upgrade_history (
    id BIGINT(20) NOT NULL AUTO_INCREMENT,
    starttime BIGINT(20) NOT NULL,
    duration BIGINT(20) NOT NULL,
    fromMajor INT NOT NULL,
    fromMinor INT NOT NULL,
    fromMaintenance INT NOT NULL,
    fromLevel INT NOT NULL,
    toMajor INT NOT NULL,
    toMinor INT NOT NULL,
    toMaintenance INT NOT NULL,
    toLevel INT NOT NULL,
    PRIMARY KEY (id)
) ENGINE= INNODB;

CREATE TABLE netapp_model.version (
    id BIGINT(20) NOT NULL AUTO_INCREMENT,
    major INT NOT NULL,
    minor INT NOT NULL,
    maintenance INT NOT NULL,
    patch INT NOT NULL,
    PRIMARY KEY (id)
    ) ENGINE=INNODB;

INSERT INTO netapp_model.version VALUES (NULL, 0, 0, 0, 0);

CREATE TABLE netapp_performance.update_history (
    time BIGINT(20) NOT NULL,
    package VARCHAR(255) NOT NULL,
    datasourceId BIGINT(20) NOT NULL,
    duration BIGINT(20) NOT NULL,
    containerKey VARCHAR(255) NOT NULL,
    samplesReported INT NOT NULL,
    samplesInserted INT NOT NULL,
    leaseTime BIGINT(20) NOT NULL,
    PRIMARY KEY (time, package, datasourceId, containerKey)
    ) ENGINE=InnoDB;

CREATE TABLE netapp_model.cluster (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs: cluster-identity-get.cluster-identity-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the cluster',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    serialNumber VARCHAR(255) DEFAULT NULL COMMENT 'Serial number',
    managementIp VARCHAR(255) DEFAULT NULL COMMENT 'Address used to acquire information about this cluster',
    version VARCHAR(255) DEFAULT NULL COMMENT 'Version of the cluster',
    versionGeneration INT DEFAULT NULL COMMENT 'First integer of the Data ONTAP version tuple corresponding to the lowest version across the cluster',
    versionMajor INT DEFAULT NULL COMMENT 'Second integer of the Data ONTAP version tuple corresponding to the lowest version across the cluster',
    versionMinor INT DEFAULT NULL COMMENT 'Third integer of the Data ONTAP version tuple corresponding to the lowest version across the cluster',
    rawDiskBytesTotal BIGINT(20) DEFAULT NULL COMMENT 'Aggregated from disks  regardless of state or role. ZAPIs: storage-disk-get-iter physical-blocks',
    rawDiskBytesUsed BIGINT(20) DEFAULT NULL COMMENT 'Aggregated from disks regardless of state or role. ZAPIs: storage-disk-get-iter used-blocks',
    contact varchar(255) DEFAULT NULL COMMENT 'The contact information for the cluster, ZAPIs: cluster-identity-get cluster-contact',
    location varchar(255) DEFAULT NULL COMMENT 'The location of the cluster, ZAPIs: cluster-identity-get cluster-location',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.node (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs: system-node-get-iter.node-details-info, system-get-node-info-iter.system-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the node',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    clusterId BIGINT(20) NOT NULL,
    partnerNodeId BIGINT(20) DEFAULT NULL,
    serialNumber VARCHAR(255) DEFAULT NULL COMMENT 'Serial number',
    version VARCHAR(255) DEFAULT NULL COMMENT 'Version of the cluster',
    model VARCHAR(255) DEFAULT NULL COMMENT 'Hardware model of this node.',
    uptime BIGINT(20) DEFAULT NULL COMMENT 'The uptime of this node, in seconds.',
    vendor VARCHAR(255) DEFAULT NULL COMMENT 'Hardware vendor of the node.',
    location VARCHAR(255) DEFAULT NULL COMMENT 'Location of the node.',
    failoverState VARCHAR(255) DEFAULT NULL COMMENT 'Failover state.   See ZAPI documentation on cf-status for more information.',
    cpuProcessorType VARCHAR(255) DEFAULT NULL COMMENT 'Processors type (defined by CPU manufacturer).',
    cpuProcessorId VARCHAR(255) DEFAULT NULL COMMENT 'Processors ID (defined by CPU manufacturer).',
    numberOfProcessors INT DEFAULT NULL COMMENT 'Number of processors on Node.',
    memorySizeMB BIGINT(20) DEFAULT NULL COMMENT 'Size of system memory in MB.',
    productType VARCHAR(255) DEFAULT NULL COMMENT 'Will be set to "V-Series" to indicate a v-series system for NetApp filers. For IBM filers, "gateway" will used.',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.vserver (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  vserver-get-iter.vserver-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the vserver',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    clusterId BIGINT(20) NOT NULL,
    rootVolumeId BIGINT(20),
    state ENUM('NOT_MAPPED','NONE','RUNNING','STOPPED','STARTING','STOPPING') DEFAULT NULL COMMENT 'Data serving state of Vserver',
    type ENUM('NOT_MAPPED','CLUSTER','DATA','ADMIN','NODE') DEFAULT NULL COMMENT 'Type of Vserver',
    allowedProtocols VARCHAR(255) DEFAULT NULL COMMENT 'Ordered list of allowed protocols. List of: cifs, nfs, iscsi, and fcp',
    disallowedProtocols VARCHAR(255) DEFAULT NULL COMMENT 'Ordered list of disallowed protocols. List of: cifs, nfs, iscsi, and fcp',
    nfsEnabled BOOL DEFAULT NULL COMMENT 'NFS access is supported and enabled',
    nfsV2Supported BOOL DEFAULT NULL COMMENT 'NFS V2 is supported',
    nfsV3Supported BOOL DEFAULT NULL COMMENT 'NFS V3 is supported',
    nfsV4Supported BOOL DEFAULT NULL COMMENT 'NFS V4 is supported',
    nfsV41Supported BOOL DEFAULT NULL COMMENT 'NFS V4.1 is supported',
    cifsEnabled BOOL DEFAULT NULL COMMENT 'CIFS access is supported and enabled',
    cifsServer VARCHAR(255) DEFAULT NULL COMMENT 'NetBIOS name of the CIFS server',
    cifsAuthenticationStyle ENUM('NOT_MAPPED','DOMAIN','WORKGROUP') DEFAULT NULL COMMENT 'Authentication style for CIFS clients',
    cifsDomain VARCHAR(255) DEFAULT NULL COMMENT 'Fully qualified domain name of the Windows Active Directory this CIFS server belongs to',
    cifsSmb2Supported BOOL DEFAULT NULL COMMENT 'CIFS SMB2 is supported',
    iscsiEnabled BOOL DEFAULT NULL COMMENT 'iSCSI access is supported and enabled',
    iscsiNodeName VARCHAR(255) DEFAULT NULL COMMENT 'iSCSI target name for the vserver',
    iscsiAliasName VARCHAR(255) DEFAULT NULL COMMENT 'iSCSI target alias',
    fcpEnabled BOOL DEFAULT NULL COMMENT 'FCP access is supported and enabled',
    fcpNodeName VARCHAR(255) DEFAULT NULL COMMENT 'WWN of the vserver FCP target',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    KEY rootVolumeId_index (rootVolumeId),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.aggregate (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs: aggr-get-iter.aggr-attributes',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the aggregate',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    clusterId BIGINT(20) NOT NULL,
    nodeId BIGINT(20) NOT NULL,
    type VARCHAR(255) DEFAULT NULL COMMENT 'The type of aggregate. Possible values: aggr, trad "aggr" (for aggregates that can contain flexible volumes) "trad" (for aggregates embedded in traditional volumes)',
    isHybrid BOOL DEFAULT NULL COMMENT 'If true, aggregate currently contains both SSD and non-SSD RAID groups',
    isHybridEnabled BOOL DEFAULT NULL COMMENT 'If true, aggregate is eligible to contain both SSD and non-SSD RAID groups',
    state ENUM('NOT_MAPPED','CREATING','DESTROYING','FAILED','FROZEN','INCONSISTENT','IRON_RESTRICTED','MOUNTING','ONLINE','OFFLINE','PARTIAL','QUIESCED','QUIESCING','RESTRICTED','REVERTED','UNKNOWN','UNMOUNTED','UNMOUNTING') DEFAULT NULL COMMENT 'State of aggregatha_paire',
    snapshotSizeTotal BIGINT(20) DEFAULT NULL COMMENT 'Total size (in bytes) of the referenced file system. If the referenced file system is restricted or offline, a value 0 is returned',
    snapshotSizeUsed BIGINT(20) DEFAULT NULL COMMENT 'Number of bytes used in the referenced file system. If the referenced file system is restricted or offline, a value of 0 is returned',
    hybridCacheSizeTotal BIGINT(20) DEFAULT NULL COMMENT 'Total cache size (in bytes) in a hybrid aggregate. If the referenced aggregate is restricted or offline, or if it is not a hybrid aggregate, a value of 0 is returned',
    sizeTotal BIGINT(20) DEFAULT NULL COMMENT 'Total size (in bytes) of the referenced file system . If the referenced file system is restricted or offline, a value of 0 is returned',
    sizeUsed BIGINT(20) DEFAULT NULL COMMENT 'Number of bytes used in the referenced file system. If the referenced file system is restricted or offline, a value of 0 is returned',
    totalReservedSpace BIGINT(20) DEFAULT NULL COMMENT 'The total disk space in bytes that is reserved on the referenced file system. The reserved space is already counted in the used space, so this element can be used to see what portion of the used space represents space reserved for future use',
    raidStatus VARCHAR(255) DEFAULT NULL COMMENT 'RAID status as string',
    raidType ENUM('NOT_MAPPED','RAID_DP','RAID_4','RAID_0','MIXED') DEFAULT NULL COMMENT 'DERVIED:Raid type of raid groups in this aggregate as derived from raidStatus field',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.plex (
    objid BIGINT(20) NOT NULL COMMENT 'Object ID within the platform server.  ZAPIs:  aggr-get-iter.aggr-attributes.plex-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the plex. Not available in ZAPI. Uses {AggregateUUID}.{PlexName}',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    aggregateId BIGINT(20) NOT NULL,
    clusterId BIGINT(20) NOT NULL,
    nodeId BIGINT(20) NOT NULL,
    isOnline BOOL NULL COMMENT 'if plex is online',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    PRIMARY KEY (objid)   ,
    FOREIGN KEY FK_aggregateId (aggregateId) REFERENCES netapp_model.aggregate (objId),
    FOREIGN KEY FK_clusterid (clusterid) REFERENCES netapp_model.cluster (objid),
    FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid)
    ) ENGINE=InnoDB;

CREATE TABLE netapp_model.raid_group (
    objid BIGINT(20) NOT NULL COMMENT 'Object ID within the platform server.  ZAPIs:   aggr-get-iter.aggr-attributes.raid-group-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the raid group. Not available in ZAPI. Uses {AggregateUUID}.{PlexName}',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    plexId BIGINT(20) NOT NULL,
    aggregateId BIGINT(20) NOT NULL,
    clusterId BIGINT(20) NOT NULL,
    nodeId BIGINT(20) NOT NULL,
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    PRIMARY KEY (objid)   ,
    FOREIGN KEY FK_plexId (plexId) REFERENCES netapp_model.plex (objId),
    FOREIGN KEY FK_aggregateId (aggregateId) REFERENCES netapp_model.aggregate (objId),
    FOREIGN KEY FK_clusterId (clusterid) REFERENCES netapp_model.cluster (objid),
    FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid)
    ) ENGINE=InnoDB;

CREATE TABLE netapp_model.disk (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  storage-disk-get-iter.storage-disk-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the disk',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    clusterId BIGINT(20) NOT NULL,
    nodeId BIGINT(20) NOT NULL,
    aggregateId BIGINT(20) DEFAULT NULL,
    raidGroupId BIGINT(20) NULL,
    interfaceType ENUM('NOT_MAPPED','ATA','BSAS','EATA','FCAL','LUN','MSATA','SAS','SATA','SCSI','SSD','XATA','XSAS','FSAS','UNKNOWN') DEFAULT NULL COMMENT 'Type of disk',
    model VARCHAR(255) DEFAULT NULL COMMENT 'Disk model string',
    firmwareRevision VARCHAR(255) DEFAULT NULL COMMENT 'Firmware revision of disk. The format of the firmware revision will vary depending on the type of disk and its vendor',
    rpm INT DEFAULT NULL COMMENT 'Disk RPM',
    serialNumber VARCHAR(255) DEFAULT NULL COMMENT 'Disk serial number',
    shelf VARCHAR(255) DEFAULT NULL COMMENT 'Disk shelf, if it can be determined',
    shelfBay VARCHAR(255) DEFAULT NULL COMMENT 'Disk shelf bay, if it can be determined',
    vendor VARCHAR(255) DEFAULT NULL COMMENT 'Disk vendor',
    containerType ENUM('NOT_MAPPED','AGGREGATE','BROKEN','LABELMAINT','MAINTENANCE','SPARE','UNASSIGNED','VOLUME','UNKNOWN') DEFAULT NULL COMMENT 'Type of overlying disk container',
    isFailed BOOL DEFAULT NULL COMMENT 'Disk failed',
    physicalBlocks BIGINT(20) DEFAULT NULL COMMENT 'RAID recorded disk capacity expressed in units of 4096-byte blocks',
    usedBlocks BIGINT(20) DEFAULT NULL COMMENT 'RAID recorded size of file system region on this disk, given in units of 4096-byte blocks.',
    bytesPerSector BIGINT(20) DEFAULT NULL COMMENT 'Number of bytes per disk sector. A sector count element, such as `capacity-sectors` and `right-size-sectors`, may be multiplied by this value to convert to a byte count.',
    raidPosition ENUM('NOT_MAPPED','COPY','DATA','DPARITY','ORPHAN','PARITY','PENDING','PRESENT') DEFAULT NULL COMMENT 'Position of disk relative to its container-type',
    isVirtual BOOL DEFAULT NULL COMMENT 'To determine if disk is virtual',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid),
    CONSTRAINT FOREIGN KEY FK_aggregateId (aggregateId) REFERENCES netapp_model.aggregate (objid),
    CONSTRAINT FOREIGN KEY FK_raidGroupid (raidGroupId) REFERENCES netapp_model.raid_group (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.storage_adapter (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  storage-adapter-get-adapter-list.adapter-list.adapter-name-elem, storage-adapter-get-adapter-info.adapter-details.adapter-detail-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the adapter. Not from ZAPI. Format is {NodeUUID.{adapterName}',
    clusterId BIGINT(20) NOT NULL,
    nodeId BIGINT(20) NOT NULL,
    name VARCHAR(255) DEFAULT NULL COMMENT 'Name of the adapter',
    fcNodeName VARCHAR(255) DEFAULT NULL COMMENT 'FCP World Wide Node Name (WWNN) of the adapter. Only for FC type adapters',
    isInUse bool DEFAULT NULL COMMENT 'Is the adapter inuse? Only for FC type adapters',
    model VARCHAR(255) DEFAULT NULL comment 'Model of the adapter. Only for FC type adapters',
    status ENUM ('ONLINE', 'OFFLINE', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Status of the adapter',
    adapterType ENUM ('ADT_IF_ATA', 'ADT_IF_PARALLEL_SCSI', 'ADT_IF_SAS', 'ADT_IF_FC', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Type of adapter present in the system',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.disk_path (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  storage-disk-get-iter.storage-disk-info.disk-paths.disk-path-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'UUID of the Disk Path equivalent to "{diskUUID}.{initiatorName}.{targetWWN}". This is not returned from ZAPI',
    clusterId BIGINT(20) NOT NULL,
    nodeId BIGINT(20) NOT NULL,
    diskId BIGINT(20) NOT NULL,
    initiatorPortId BIGINT(20) NOT NULL COMMENT 'Reference to initiator storage adapter in the cluster',
    arrayName VARCHAR(255) DEFAULT NULL COMMENT 'Name of the array providing the LUN',
    diskPort VARCHAR(255) DEFAULT NULL COMMENT 'Disk port associated with this path. Possible values are "A" or "B". Omitted for non-disk target',
    diskPortName VARCHAR(255) DEFAULT NULL COMMENT 'Disk port name associated with this path. This has the form {attachment-style}:{disk-port}, where <attachment-style> is either "FC" for FibreChannel, or "SA" for SAS, and <disk-port> is either "A" or "B". Omitted for non-disk target',
    initiatorSideSwitchPort VARCHAR(255) DEFAULT NULL COMMENT 'The name of the switch connected to the controller initiator port, or N/A when using direct attach',
    lun INT DEFAULT NULL COMMENT 'Logical Unit Number. Range: [0..65535]',
    lunPathUseState ENUM ('INU', 'RDY', 'ERR', 'QNG', 'NOT_MAPPED') DEFAULT NULL COMMENT 'ONTAP use of this path: In Use, Ready, High Error, Quiescing',
    targetWWPN VARCHAR(255) NOT NULL COMMENT 'World Wide Port Name of target port providing the disk',
    targetSideSwitchPort VARCHAR(255) NOT NULL COMMENT 'Name of the switch port connected to the target array, or UNKNOWN if direct attached',
    tpgn BIGINT(20) DEFAULT NULL COMMENT 'The Target Port Group Number of the array target port. Range: [0..2^64-1]',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid),
    CONSTRAINT FOREIGN KEY FK_diskId (diskId) REFERENCES netapp_model.disk (objid),
    CONSTRAINT FOREIGN KEY FK_initiatorPortId (initiatorPortId) REFERENCES netapp_model.storage_adapter (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.export_policy (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  export-policy-get-iter.export-policy-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the policy. Equal to the policy ID',
    name VARCHAR(255) DEFAULT NULL COMMENT 'Name of the policy',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    policyId BIGINT(20) DEFAULT NULL COMMENT 'Export rule index',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.export_rule (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  export-rule-get-iter.export-rule-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the export rule. Equal to {VserverUUID}.{policyID}.{ruleIndex}. Not returned from ZAPI',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    exportPolicyId BIGINT(20) NOT NULL,
    ruleIndex INT DEFAULT NULL COMMENT 'Export rule index',
    clientMatch VARCHAR(255) NOT NULL COMMENT 'Client match specification for Export rule. The clients specified are enforced with this Export rule. The rule with the higher rule index value takes precedence',
    accessProtocol VARCHAR(255) NOT NULL COMMENT 'Client access protocol. Default value is \'any\'. May be comma separated',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_exportPolicyId (exportPolicyId) REFERENCES netapp_model.export_policy (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.volume (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs: volume-get-iter.volume-attributes',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the volume',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    aggregateId BIGINT(20) NOT NULL,
    exportPolicyId BIGINT(20) DEFAULT NULL,
    cloneParentId BIGINT(20) DEFAULT NULL COMMENT 'Clone parent if applicable.',
    instanceUUID VARCHAR(255) DEFAULT NULL COMMENT 'Globally unique ID of the Volume that remains unchanged after a volume move',
    size BIGINT(20) DEFAULT NULL COMMENT 'Size of volume in bytes. Includes WAFL reserve and snapshot reserve',
    sizeTotal BIGINT(20) DEFAULT NULL COMMENT 'Usable size of volume in bytes for data (not including WAFL reserve or snapshot reserve)',
    sizeUsed BIGINT(20) DEFAULT NULL COMMENT 'Used size of the volume in bytes',
    isVserverRoot BOOL DEFAULT NULL COMMENT 'Is the volume the root of a vserver',
    state ENUM('NOT_MAPPED','ONLINE','RESTRICTED','OFFLINE','MIXED') DEFAULT NULL COMMENT 'Volume state',
    junctionPath VARCHAR(255) DEFAULT NULL COMMENT 'The junction path at which this volume is mounted',
    isJunctionActive BOOL DEFAULT NULL COMMENT 'If mounted volume is accessible',
    style ENUM('NOT_MAPPED','FLEXVOL','STRIPED','INFINIVOL') DEFAULT NULL COMMENT 'Volume style',
    isSisVolume BOOL DEFAULT NULL COMMENT 'True if volume contains shared data (deduplication, file clones) or compressed data',
    isDataProtectionMirror BOOL DEFAULT NULL COMMENT 'True if volume is a data protection mirror',
    isLoadSharingMirror BOOL DEFAULT NULL COMMENT 'True if volume is a load sharing mirror',
    isMoveMirror BOOL DEFAULT NULL COMMENT 'True if this is a move volume',
    isReplicaVolume BOOL DEFAULT NULL COMMENT 'True if this is a read only replica volume',
    isSpaceGuaranteeEnabled BOOL DEFAULT NULL COMMENT 'Whether or not the storage guarantee associated with the flexible volume is currently in effect',
    vmAlignSector INT DEFAULT NULL COMMENT 'The virtual machine alignment 512 bytes sector number',
    vmAlignSuffix VARCHAR(255) DEFAULT NULL COMMENT 'The virtual machine alignment suffix. The suffix such as ".xyz". is used to identify the files which needs to be aligned',
    overwriteReserve BIGINT(20) DEFAULT NULL COMMENT 'The size (in bytes) that is reserved for overwriting snapshoted data in an otherwise full volume',
    overwriteReserveRequired BIGINT(20) DEFAULT NULL COMMENT 'The reserved size (in bytes) that is required to ensure that the reserved space is sufficient to allow all space-reserved files and LUNs to be overwritten when the volume is full',
    overwriteReserveUsed BIGINT(20) DEFAULT NULL COMMENT 'The reserved size (in bytes) that is not available for new overwrites',
    overwriteReserveActualUsed BIGINT(20) DEFAULT NULL COMMENT 'The reserved size (in bytes) that has been used',
    snapshotReserveSize BIGINT(20) DEFAULT NULL COMMENT 'The size (in bytes) in the volume that has been set aside as reserve for snapshot usage',
    percentageSnapshotReserveUsed INT DEFAULT NULL COMMENT 'Percentage of the volume reserved for snapshots that has been used',
    sizeAvailableForSnapshot BIGINT(20) DEFAULT NULL COMMENT 'Total free space (in bytes) available in the volume and snapshot reserve',
    percentageFractionalReserve INT DEFAULT NULL COMMENT 'This field can be used to change the amount of space reserved for overwrites of reserved objects (e.g., files, LUNs) in a volume',
    deduplicationSpaceSaved BIGINT(20) DEFAULT NULL COMMENT 'The total disk space (in bytes) that is saved by deduplication',
    compressionSpaceSaved BIGINT(20) DEFAULT NULL COMMENT 'The total disk space (in bytes) that is saved by compressing blocks on the referenced file system',
    securityStyle ENUM('NOT_MAPPED','MIXED','NTFS','UNIX') DEFAULT NULL COMMENT 'The security style associated with this volume',
    hybridCacheEligibility ENUM ('READ', 'READ_WRITE', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Hybrid cacPlhe eligibility of a volume',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    KEY cloneParentId_index (cloneParentId),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_aggregateId (aggregateId) REFERENCES netapp_model.aggregate (objid),
    CONSTRAINT FOREIGN KEY FK_exportPolicyId (exportPolicyId) REFERENCES netapp_model.export_policy (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.qtree (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  qtree-list-iter.qtree-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the qtree. Not available in ZAPI. Uses {QtreeId}.{VolumeUUID}',
    name VARCHAR(255) NOT NULL COMMENT 'Name of the Qtree, blank if the root Qtree',
    qtreeIdentifier VARCHAR(255) DEFAULT NULL COMMENT 'ID of the qtree within its volume. This is not globally unique',
    clusterId BIGINT(20) NOT NULL,
    volumeId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    status ENUM('NOT_MAPPED','SNAPVAULTED','SNAPMIRRORED','NORMAL','READONLY') DEFAULT NULL COMMENT 'State of the Qtree',
    securityStyle ENUM('NOT_MAPPED','UNIX','NTFS','MIXED') DEFAULT NULL COMMENT 'Security style of the Qtree',
    oplocks VARCHAR(255) DEFAULT NULL COMMENT 'Is opportunistic locking enabled on this Qtree',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_aggregateId (volumeId) REFERENCES netapp_model.volume (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.cifs_share (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  cifs-share-get-iter.cifs-share',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the CIFS share. Equal to {ClusterUUID}.{VserverUUID}.{shareName}. Not returned from ZAPI',
    name VARCHAR(255) NOT NULL COMMENT 'Name of the CIFS share',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    volumeId BIGINT(20) DEFAULT NULL,
    qtreeId BIGINT(20) DEFAULT NULL,
    path VARCHAR(255) DEFAULT NULL COMMENT 'The file system path that is shared through this CIFS share',
    comment VARCHAR(255) DEFAULT NULL COMMENT 'Description of the CIFS share. CIFS clients see this description when browsing the Vserver\'s CIFS shares',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_volumeId (volumeId) REFERENCES netapp_model.volume (objid),
    CONSTRAINT FOREIGN KEY FK_qtreeId (qtreeId) REFERENCES netapp_model.qtree (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.fcp_port (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs: fcp-adapter-get-iter.fcp-config-adapter-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the fcp port. Not available in ZAPI. Uses fcp-config-adapter-info.port-name',
    clusterId BIGINT(20) NOT NULL,
    nodeId BIGINT(20) NOT NULL,
    nodeName VARCHAR(255) DEFAULT NULL COMMENT 'FCP World Wide Node Name (WWNN) of the adapter.',
    portName VARCHAR(255) DEFAULT NULL COMMENT 'FCP World Wide Port Name (WWPN) of adapter.',
    adapter VARCHAR(255) DEFAULT NULL COMMENT 'The slot name of the FC adapter.',
    maxSpeed INT DEFAULT NULL comment 'The maximum speed of this adapter in Gbit/sec.',
    speed  ENUM ('AUTO', 'GIGABIT_1', 'GIGABIT_2', 'GIGABIT_4', 'GIGABIT_8', 'GIGABIT_10','GIGABIT_16','NOT_MAPPED') DEFAULT NULL COMMENT 'Speed configured for this adapter',
    physicalProtocol ENUM ('FIBRE_CHANNEL', 'ETHERNET', 'NOT_MAPPED') DEFAULT NULL COMMENT 'The physical protocol of the adapter.',
    state ENUM ('STARTUP', 'UNINITIALIZED', 'INITIALIZING_FIRMWARE', 'LINK_NOT_CONNECTED', 'WAITING_FOR_LINK_UP', 'ONLINE', 'LINK_DISCONNECTED', 'RESETTING', 'OFFLINE', 'OFFLINED_BY_USER_OR_SYSTEM', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Status of the adapter.',
    status ENUM ('ONLINE', 'OFFLINE', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Status of the port.',
    switchPort VARCHAR(255) DEFAULT NULL COMMENT 'Switch and port this adapter is connected to. The string will be of the form X:Y where X is the name of the switch and Y is the port value.',
    dataLinkRate INT DEFAULT NULL COMMENT 'Data link rate in Gbits',
    mediaType ENUM ('UNDEF', 'AUTO', 'PTP', 'LOOP', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Media configured for this adapter',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.fcp_lif (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  net-interface-get-iter.net-interface-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of this LIF. Not available in ZAPI. Uses {VserverUUID}.{LIFName}',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    currentNodeId BIGINT(20) DEFAULT NULL,
    currentPortId BIGINT(20) DEFAULT NULL,
    homeNodeId BIGINT(20) DEFAULT NULL,
    homePortId BIGINT(20) DEFAULT NULL,
    isAutoRevert BOOL DEFAULT NULL COMMENT 'If true, data LIF will revert to its home node under certain circumstances such as startup, and load balancing migration is disabled automatically',
    isHome BOOL DEFAULT NULL COMMENT 'True if LIF is currently on home node and home port',
    nodeName VARCHAR(255) DEFAULT NULL COMMENT 'World Wide Node Name (WWNN) of the LIF. All LIFs in a Vserver have the same WWNN as the Vserver FCP service',
    portName VARCHAR(255) DEFAULT NULL COMMENT 'World Wide Port Name (WWPN) of the LIF',
    operationalStatus ENUM ('UP', 'DOWN', 'UNKNOWN', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Specifies the operational status of the LIF',
    administrativeStatus ENUM ('UP', 'DOWN', 'UNKNOWN', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Specifies the administrative status of the LIF',
    role ENUM ('UNDEF', 'CLUSTER', 'DATA', 'NODE_MGMT', 'INTERCLUSTER', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Specifies the role associated with the LIF',
    dataProtocols VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the list of data protocols configured on the LIF. This can be comma separated. Possible values: nfs, cifs, iscsi, fcp, fcache, none',
    failoverGroup VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the failover group name',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_currentNodeId (currentNodeId) REFERENCES netapp_model.node (objid),
    CONSTRAINT FOREIGN KEY FK_currentPortId (currentPortId) REFERENCES netapp_model.fcp_port (objid),
    CONSTRAINT FOREIGN KEY FK_homeNodeId (homeNodeId) REFERENCES netapp_model.node (objid),
    CONSTRAINT FOREIGN KEY FK_homePortId (homePortId) REFERENCES netapp_model.fcp_port (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.network_port (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs: net-port-get-iter.net-port-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the network port. Not available in ZAPI. Uses {PortName}.{MACAddress}',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    clusterId BIGINT(20) NOT NULL,
    nodeId BIGINT(20) NOT NULL,
    macAddress VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the MAC address of the port',
    administrativeSpeed ENUM ('UNDEF', 'AUTO', 'MEGABIT_10', 'MEGABIT_100', 'GIGABIT_1', 'GIGABIT_10', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Speed setting preferred by the port',
    operationalSpeed ENUM ('UNDEF', 'AUTO', 'MEGABIT_10', 'MEGABIT_100', 'GIGABIT_1', 'GIGABIT_10', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Speed setting preferred by the port',
    linkStatus ENUM ('UNKNOWN', 'UP', 'DOWN', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Specifies the link status of the port',
    role ENUM ('UNDEF', 'CLUSTER', 'DATA', 'NODE_MGMT', 'INTERCLUSTER', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Specifies the role associated with the port',
    portType ENUM ('PHYSICAL', 'IF_GROUP', 'VLAN', 'UNDEF', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Specifies the role associated with the port',
    vlanId BIGINT(20) DEFAULT NULL COMMENT 'For ports of type VLAN, the VLAN ID',
    vlanPort VARCHAR(255) DEFAULT NULL COMMENT 'For port type of VLAN, the physcial port on which the VLAN is created',
    ifgrpPort VARCHAR(255) DEFAULT NULL COMMENT 'For port of type IFGRP, specifies the physical port on which the IFGRP is created',
    mtu BIGINT(20) DEFAULT NULL COMMENT 'Specifies the maximum transmission unit of the port',
    operationalDuplex ENUM ('UNDEF', 'AUTO', 'HALF', 'FULL', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Duplex setting reported by the port after Ethernet auto-negotiation',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.network_lif (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  net-interface-get-iter.net-interface-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of this LIF. Not available in ZAPI. Uses {VserverUUID}.{LIFName}',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    currentNodeId BIGINT(20) DEFAULT NULL,
    currentPortId BIGINT(20) DEFAULT NULL,
    homeNodeId BIGINT(20) DEFAULT NULL,
    homePortId BIGINT(20) DEFAULT NULL,
    isAutoRevert BOOL DEFAULT NULL COMMENT 'If true, data LIF will revert to its home node under certain circumstances such as startup, and load balancing migration is disabled automatically',
    isHome BOOL DEFAULT NULL COMMENT 'True if LIF is currently on home node and home port',
    address VARCHAR(255) DEFAULT NULL COMMENT 'LIFs IP address. THis element is valid for all data prtocols except FCP',
    netmask VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the LIFs netmask. This element is valid for all data protocols except FCP',
    netmaskLength INT DEFAULT NULL COMMENT 'Specifies the number of bits in the netmask',
    dnsDomainName VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the unique, fully qualified domain name of the DNS zone of this LIF',
    routingGroupName VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the routing group, which enabled multiple LIFs to share a set of routing table entries. For example: d192.168.1.0/24 ("d" , "c" stands for cluster, n" stands for node management)',
    operationalStatus ENUM ('UP', 'DOWN', 'UNKNOWN', 'NOT_MAPPED') DEFAULT NULL COMMENT 'State of aggregate',
    administrativeStatus ENUM ('UP', 'DOWN', 'UNKNOWN', 'NOT_MAPPED') DEFAULT NULL COMMENT 'State of aggregate',
    role ENUM ('UNDEF', 'CLUSTER', 'DATA', 'NODE_MGMT', 'INTERCLUSTER', 'NOT_MAPPED', 'CLUSTER_MGMT') DEFAULT NULL COMMENT 'Specifies the role associated with the LIF',
    dataProtocols VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the list of data protocols configured on the LIF. This can be comma separated. Possible values: nfs, cifs, iscsi, fcp, fcache, none',
    failoverGroup VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the failover group name',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_currentNodeId (currentNodeId) REFERENCES netapp_model.node (objid),
    CONSTRAINT FOREIGN KEY FK_currentPortId (currentPortId) REFERENCES netapp_model.network_port (objid),
    CONSTRAINT FOREIGN KEY FK_homeNodeId (homeNodeId) REFERENCES netapp_model.node (objid),
    CONSTRAINT FOREIGN KEY FK_homePortId (homePortId) REFERENCES netapp_model.network_port (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.ha_pair (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  cf-get-iter.storage-failover-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the HA Pair',
    node1Id BIGINT(20) NOT NULL,
    node2Id BIGINT(20) NOT NULL,
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_node1Id (node1Id) REFERENCES netapp_model.node (objid),
    CONSTRAINT FOREIGN KEY FK_node2Id (node2Id) REFERENCES netapp_model.node (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.igroup (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  igroup-get-iter.initiator-group-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the igroup',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    osType VARCHAR(255) DEFAULT NULL COMMENT 'OS type of the initiator group',
    type ENUM ('NOT_MAPPED', 'ISCSI', 'FCP', 'MIXED') DEFAULT NULL COMMENT 'Type of the initiators in this group',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.igroup_initiator (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  igroup-get-iter.initiator-group-info.initiators.initiator-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the igroup initiator',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    clusterId BIGINT(20) NOT NULL,
    igroupId BIGINT(20) NOT NULL,
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_igroupId (igroupId) REFERENCES netapp_model.igroup (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.iscsi_portal_group (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs: net-port-get-iter.net-port-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the iSCSI Portal Group equivalent to "{clusterUUID}.{VserverUUID}.{portalGroupName}". This is not returned from ZAPI',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    tag INT DEFAULT NULL,
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.iscsi_portal_group_lif (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  iscsi-tpgroup-get-iter.iscsi-tpgroup-list-entry-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the iSCSI Portal Group LIF. This is not returned from ZAPI',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    targetPortalGroupId BIGINT(20) NOT NULL,
    lifId BIGINT(20) NOT NULL,
    interfaceName VARCHAR(255) DEFAULT NULL COMMENT 'Name of the interface',
    ipAddress VARCHAR(255) DEFAULT NULL COMMENT 'iSCSI target portal IP address',
    ipPort VARCHAR(255) DEFAULT NULL COMMENT 'iSCSI target portal TCP port',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_iscsiPortalGroupId (targetPortalGroupId) REFERENCES netapp_model.iscsi_portal_group (objid),
    CONSTRAINT FOREIGN KEY FK_lifId (lifId) REFERENCES netapp_model.network_lif (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.iscsi_security_entry (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  iscsi-initiator-auth-get-iter.iscsi-security-entry-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the security entry. Equal to "{clusterUUID}.{VserverUUID}.{initiatorNodeName}.{security}". Not returned from ZAPI',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    authenticationType ENUM ('CHAP', 'NONE', 'DENY', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Authentication type',
    initiator VARCHAR(255) DEFAULT NULL COMMENT 'Name of initiator',
    outboundUserName VARCHAR(255) DEFAULT NULL COMMENT 'Outbound CHAP user name, returned only if auth-type is CHAP, and outbound authentication is set for initiator.',
    username VARCHAR(255) DEFAULT NULL COMMENT 'Inbound CHAP user name, returned only if auth-type is CHAP.',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.iscsi_session (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  iscsi-session-get-iter.iscsi-session-list-entry-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the iSCSI Session. Equal to "{clusterUUID}.{VserverUUID}.{portalGroupName}.{targetSessionId}"',
    clusterId BIGINT(20) NOT NULL,
    targetPortalGroupId BIGINT(20) NOT NULL,
    initiatorSessionId VARCHAR(255) DEFAULT NULL COMMENT 'ISID for this session selected by initiator represented as 6 hexadecimal octets separated by colons, for example: "40:01:37:00:00:00".',
    targetSessionId INT DEFAULT NULL COMMENT 'The iSCSI session identifier assigned by the storage system',
    initiatorNodeName VARCHAR(255) DEFAULT NULL COMMENT 'Name of initiator. The initiator name must conform to RFC 3720, for example: "iqn.1987-06.com.initvendor1:appsrv.sn.2346"',
    initiatorAliasName VARCHAR(255) DEFAULT NULL COMMENT 'The user-friendly name assigned to initiator',
    maxConnections INT DEFAULT NULL COMMENT 'Maximum number of connections for this session',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_iscsiPortalGroupId (targetPortalGroupId) REFERENCES netapp_model.iscsi_portal_group (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.iscsi_session_connection (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  iscsi-session-get-iter.iscsi-session-list-entry-info.iscsi-session-connection-list-entries.iscsi-session-connection-list-entry-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the iSCSI Session connection. Equal to "{clusterUUID}.{VserverUUID}.{portalGroupName}.{targetSessionId}.{connectionId}". Not returned from ZAPI',
    clusterId BIGINT(20) NOT NULL,
    iscsiSessionId BIGINT(20) NOT NULL,
    lifId BIGINT(20) NOT NULL,
    connectionId INT DEFAULT NULL COMMENT 'Connection ID within the session',
    localIpAddress VARCHAR(255) DEFAULT NULL COMMENT 'Local storage system iSCSI target interface address',
    localIpPort VARCHAR(255) DEFAULT NULL COMMENT 'Local storage system iSCSI target TCP port',
    remoteIpAddress VARCHAR(255) DEFAULT NULL COMMENT 'Remote initiator TCP port',
    authenticationStyle ENUM ('CHAP', 'NONE', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Authentication method of this connection',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_iscsiSessionId (iscsiSessionId) REFERENCES netapp_model.iscsi_session (objid),
    CONSTRAINT FOREIGN KEY FK_lifId (lifId) REFERENCES netapp_model.network_lif (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.license (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  interim-license-list-get.licenses.interim-license-cluster-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the license.  DERIVED from {clusterUUID}.{package name}.{serial number}',
    package VARCHAR(255) NOT NULL COMMENT 'Name of licensed feature',
    clusterId BIGINT(20) NOT NULL,
    serialNumber VARCHAR(255) DEFAULT NULL COMMENT 'Serial number of owning entity (cluster in RR.x)',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.lun (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  lun-get-iter.lun-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the LUN',
    qtreeId BIGINT(20) NOT NULL,
    volumeId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    clusterId BIGINT(20) NOT NULL,
    path VARCHAR(255) DEFAULT NULL COMMENT 'Path of the LUN',
    isOnline BOOL DEFAULT NULL COMMENT 'Is LUN online?',
    size BIGINT(20) DEFAULT NULL COMMENT 'Size of the LUN in bytes',
    sizeUsed BIGINT(20) DEFAULT NULL COMMENT 'Used bytes in the LUN',
    isSpaceReservationEnabled BOOL DEFAULT NULL COMMENT 'Whether or not the LUN has space reservation enabled',
    isSpaceAllocEnabled BOOL DEFAULT NULL COMMENT 'Whether or not the LUN has space allocation enabled',
    isMapped BOOL DEFAULT NULL COMMENT 'Is the LUN mapped to any initiator?',
    serialNumber VARCHAR(255) DEFAULT NULL COMMENT 'Serial number of the LUN',
    multiProtocolType VARCHAR(255) DEFAULT NULL COMMENT 'The OS type of the LUN',
    alignment ENUM('NOT_MAPPED','ALIGNED','MISALIGNED','POSSIBLY_MISALIGNED','INDETERMINATE') DEFAULT NULL COMMENT 'Alignment state of the LUN',
    prefixSize BIGINT(20) DEFAULT NULL COMMENT 'Size of the prefix stream for this LUN in bytes',
    suffixSize BIGINT(20) DEFAULT NULL COMMENT 'Size of the suffix stream for this LUN in bytes',
    comment VARCHAR(255) DEFAULT NULL COMMENT 'User defined comments on LUN',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_qtreeId (qtreeId) REFERENCES netapp_model.qtree (objid),
    CONSTRAINT FOREIGN KEY FK_volumeId (volumeId) REFERENCES netapp_model.volume (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.lun_map (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  lun-map-get-iter.lun-map-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the LUN map. DERIVED from {igroupUUID}.{lunUUID}',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    igroupId BIGINT(20) NOT NULL,
    lunId BIGINT(20) NOT NULL,
    lun INTEGER DEFAULT NULL COMMENT 'Logical Unit Number which the lun is mapped to for the requested initiator. [04095]',
    path VARCHAR(255) DEFAULT NULL COMMENT 'Path of the LUN which is mapped to the requested initiator',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_igroupId (igroupId) REFERENCES netapp_model.igroup (objid),
    CONSTRAINT FOREIGN KEY FK_lunId (lunId) REFERENCES netapp_model.lun (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.portset (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  portset-get-iter.portset-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the portset.  DERIVED as the {vserverUUID}.{portset name}',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    type ENUM ('NOT_MAPPED', 'ISCSI', 'FCP', 'MIXED') DEFAULT NULL COMMENT 'Type of the ports in this group',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.portset_port (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  portset-get-iter.portset-info.portset-port-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the port. DERIVED from {clusterID},{porsetId},{name}',
    name VARCHAR(255) NOT NULL COMMENT 'String representing a member of a portset. In Data ONTAP Cluster-Mode, the port name is the name of an FCP data lif or iSCSI target portal group',
    clusterId BIGINT(20) NOT NULL,
    portsetId BIGINT(20) NOT NULL,
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_portsetId (portsetId) REFERENCES netapp_model.portset (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.portset_to_igroup (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  portset-get-iter.portset-info.initiator-group-info.initiator-group-name',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the portset igroup ref. DERIVED from {clusterId},{porsetId},{igroupId}',
    clusterId BIGINT(20) NOT NULL,
    portsetId BIGINT(20) NOT NULL,
    igroupId BIGINT(20) NOT NULL,
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_portsetId (portsetId) REFERENCES netapp_model.portset (objid),
    CONSTRAINT FOREIGN KEY FK_igroupId (igroupId) REFERENCES netapp_model.igroup (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.processor (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  system-get-node-info-iter.system-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'The UUID of the object. Equal to the {NodeUUID}.{processorNumber } Not returned from ZAPI.',
    processorNumber VARCHAR(255) NOT NULL COMMENT 'Processor Number',
    clusterId BIGINT(20) NOT NULL,
    nodeId BIGINT(20) NOT NULL COMMENT 'Object ID of the Node on which this processor resides',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;


CREATE TABLE netapp_model.quota (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  quota-report-iter.quota',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'UUID of the quota. Equal to the {VolumeUUID}.{QtreeUUID}.{QuotaType}.{QuotaTarget}. Not returned from ZAPI',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    volumeId BIGINT(20) NOT NULL COMMENT 'Reference to the volume to which this quota belongs to',
    qtreeId BIGINT(20) DEFAULT NULL COMMENT 'Reference to the qtree to which this quota applies',
    quotaTarget VARCHAR(255) DEFAULT NULL COMMENT 'For an explicit quota, this value is specified in the /etc/quotas file and the domain in the QUOTA_TARGET_DOMAIN directive is in effect. Multiple targets are comma separated. For a derived quota, the field is blank.',
    quotaType ENUM ('USER', 'GROUP', 'TREE', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Type of quota',
    diskLimit BIGINT(20) DEFAULT NULL COMMENT 'Maximum amount of disk space, in kilobytes, allowed for the quota target (hard disk space limit)',
    diskLimitUnlimited BOOL NOT NULL DEFAULT TRUE COMMENT 'True if there is no disk limit',
    diskUsed BIGINT(20) DEFAULT NULL COMMENT 'Current amount of disk space, in kilobytes, used by the quota target',
    diskUsedUnlimited BOOL NOT NULL DEFAULT TRUE COMMENT 'True if there is no limit fot the disk used',
    fileLimit BIGINT(20) DEFAULT NULL COMMENT 'Maximum number of files allowed for the quota target',
    fileLimitUnlimited BOOL NOT NULL DEFAULT TRUE COMMENT 'True if there is no file limit',
    fileUsed BIGINT(20) DEFAULT NULL COMMENT 'Current number of files used by the quota target',
    fileUsedUnlimited BOOL NOT NULL DEFAULT TRUE COMMENT 'True if there is no limit for file used',
    softDiskLimit BIGINT(20) DEFAULT NULL COMMENT 'Soft disk space limit, in kilobytes, for the quota target',
    softDiskLimitUnlimited BOOL NOT NULL DEFAULT TRUE COMMENT 'True if there is no soft disk limit',
    softFileLimit BIGINT(20) DEFAULT NULL COMMENT 'Soft file limit, in number of files, for the quota target',
    softFileLimitUnlimited BOOL NOT NULL DEFAULT TRUE COMMENT 'True if there is no soft file limit',
    threshold BIGINT(20) DEFAULT NULL COMMENT 'Disk space threshold, in kilobytes, for the quota target',
    thresholdUnlimited BOOL NOT NULL DEFAULT TRUE COMMENT 'True if there is no threshold limit',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_volumeId (volumeId) REFERENCES netapp_model.volume (objid),
    CONSTRAINT FOREIGN KEY FK_qtreeId (qtreeId) REFERENCES netapp_model.qtree (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;

CREATE TABLE netapp_model.quota_user (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  quota-report-iter.quota.quota-users.quota-user',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'UUID of the quota. Equal to the {QuotaUUID}.{quotaUserName} Not returned from ZAPI',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    quotaId BIGINT(20) NOT NULL COMMENT 'Reference to the quota to which this user belongs to',
    quotaUserId VARCHAR(255) DEFAULT NULL COMMENT 'The id of the user. The quota-user-type determines the format. For uid and gid, the format is an integer. For sid, the format is the usual S-* style',
    quotaUserName VARCHAR(255) DEFAULT NULL COMMENT 'Name of the user, group, or sid',
    quotaUserType ENUM ('SID', 'UID', 'GID', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Type of quota user',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_quotaId (quotaId) REFERENCES netapp_model.quota (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;


CREATE TABLE netapp_model.snap_mirror (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  snapmirror-get-iter.snapmirror-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'The UUID of the object. Equal to the {SourceLocation}.{DestinationLocation} Not returned from ZAPI',
    clusterId BIGINT(20) NOT NULL,
    destinationCluster VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the destination cluster of the SnapMirror relationship. The destination Vserver and destination volume must also be specified if using this parameter',
    destinationVolume VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the destination volume of the SnapMirror relationship. The destination cluster and destination Vserver must also be specified if using this parameter',
    destinationVserver VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the destination Vserver of the SnapMirror relationship. The destination cluster and destination volume must also be specified if using this parameter',
    destinationLocation VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the destination endpoint of the SnapMirror relationship in the format: [cluster:]//vserver/volume',
    sourceCluster VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the source cluster of the SnapMirror relationship. The source Vserver and source volume must also be specified if using this parameter',
    sourceVolume VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the source volume of the SnapMirror relationship. The source cluster and source Vserver must also be specified if using this parameter',
    sourceVserver VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the source Vserver of the SnapMirror relationship. The source cluster and source volume must also be specified if using this parameter',
    sourceLocation VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the source endpoint of the SnapMirror relationship in the format: [cluster:]//vserver/volume',
    mirrorState ENUM ('UNINITIALIZED', 'SNAPMIRRORED', 'BROKEN_OFF', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Specifies the mirror state of the SnapMirror relationship',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;



CREATE TABLE netapp_model.volume_clone (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  volume-get-iter.volume-attributes.volume-clone-attributes',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the Volume Clone Pair',
    cloneId BIGINT(20) NOT NULL,
    parentId BIGINT(20) NOT NULL,
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_cloneId (cloneId) REFERENCES netapp_model.volume (objid),
    CONSTRAINT FOREIGN KEY FK_parentId (parentId) REFERENCES netapp_model.volume (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;


CREATE TABLE netapp_performance.sample_current_aggregate (
    objid BIGINT(20) COMMENT 'Reference ID of aggregate',
    time BIGINT(20),
    empty BOOL,
    containerkey VARCHAR(255),
    containerid BIGINT(20) COMMENT 'Node container reference ID',
    cpReadBlocks FLOAT COMMENT 'Number of blocks read per second during a CP on the aggregate (blocks/s)',
    cpReads FLOAT COMMENT 'Number of blocks read per second during a CP on the aggregate (ops/s)',
    totalTransfers FLOAT COMMENT 'Total number of transfers per second serviced by the aggregate (ops/s)',
    userReadBlocks FLOAT COMMENT 'Number of blocks read per second on the aggregate (blocks/s)',
    userReads FLOAT COMMENT 'Number of user reads per second to the aggregate (ops/s)',
    userWriteBlocks FLOAT COMMENT 'Number of blocks written per second to the aggregate (blocks/s)',
    userWrites FLOAT COMMENT 'Number of user writes per second to the aggregate (ops/s)',
    KEY containerkey_index (containerkey),
    PRIMARY KEY (objid, containerid)
    ) ENGINE=INNODB
    PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));

CREATE TABLE netapp_performance.sample_current_disk (
    objid bigint(20),
    time bigint(20),
    empty bool,
    containerkey varchar(255),
    containerid bigint(20) COMMENT 'Cluster container reference ID',
    cpReadBlocks float COMMENT 'Number of blocks transferred for consistency point read operations per second (blocks/s)',
    cpReadChain float COMMENT 'Average number of blocks transferred in each consistency point read operation during a CP (blocks/s)',
    cpReadLatency float COMMENT 'Average latency per block in microseconds for consistency point read operations (microseconds)',
    cpReads float COMMENT 'Number of disk read operations initiated each second for consistency point processing (ops/s)',
    diskBusy float COMMENT 'Percentage of time there was at least one outstanding request to the disk (percentage, 0-100)',
    guaranteedReadBlocks float COMMENT 'Number of blocks transferred for guaranteed read operations per second (blocks/s)',
    guaranteedReadLatency float COMMENT 'Average latency per block in microseconds for guaranteed read operations (microseconds)',
    guaranteedReads float COMMENT 'Number of disk read operations initiated each second for raid reconstruct or scrubbing activities (ops/s)',
    guaranteedWriteBlocks float COMMENT 'Number of blocks transferred for guaranteed write operations per second (blocks/s)',
    guaranteedWriteLatency float COMMENT 'Average latency per block in microseconds for guaranteedwrite operations (microseconds)',
    guaranteedWrites float COMMENT 'Number of write read operations initiated each second for raid reconstruct or scrubbing activities (ops/s)',
    totalTransfers float COMMENT 'Total number of disk operations involving data transfer initiated per second (ops/s)',
    userReadBlocks float COMMENT 'Number of blocks transferred for user read operations per second (blocks/s)',
    userReadChain float COMMENT 'Average number of blocks transferred in each user read operation (blocks)',
    userReadLatency float COMMENT 'Average latency per block in microseconds for user read operations (microseconds)',
    userReads float COMMENT 'Number of disk read operations initiated each second for retrieving data or metadata associated with user requests (ops/s)',
    userWriteBlocks float COMMENT 'Number of blocks transferred for user write operations per second (blocks/s)',
    userWriteChain float COMMENT 'Average number of blocks transferred in each user write operation (blocks)',
    userWriteLatency float COMMENT 'Average latency per block in microseconds for user write operations (microseconds)',
    userWrites float COMMENT 'Number of disk write operations initiated each second for storing data or metadata associated with user requests (ops/s)',
    KEY containerkey_index (containerkey),
    PRIMARY KEY (objid, containerid)
    ) ENGINE=InnoDB
    PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));

CREATE TABLE netapp_performance.sample_current_fcplif (
    objid bigint(20),
    time bigint(20),
    empty bool,
    containerkey varchar(255),
    containerid bigint(20) COMMENT 'Vserver container reference ID',
    otherOps float COMMENT 'Number of operations that are not read or write (ops/s)',
    readData float COMMENT 'Amount of data read from the storage system (bytes/s)',
    readOps float COMMENT 'Number of read operations (op/s)',
    writeData float COMMENT 'Amount of data written to the storage system (bytes/s)',
    writeOps float COMMENT 'Number of write operations (ops/s)',
    KEY containerkey_index (containerkey),
    PRIMARY KEY (objid, containerid)
    ) ENGINE=InnoDB
    PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));

CREATE TABLE netapp_performance.sample_current_fcpport (
    objid bigint(20),
    time bigint(20),
    empty bool,
    containerkey varchar(255),
    containerid bigint(20) COMMENT 'Vserver container reference ID',
    linkFailure float COMMENT 'Number of link failures',
    otherOps float COMMENT 'Number of operations that are not read or write (ops/s)',
    readData float COMMENT 'Amount of data read from the storage system (bytes/s)',
    readOps float COMMENT 'Number of read operations (op/s)',
    recvErr float COMMENT ' Number of errors encountered during receive (RECV) phase',
    writeData float COMMENT 'Amount of data written to the storage system (bytes/s)',
    writeOps float COMMENT 'Number of write operations (ops/s)',
    KEY containerkey_index (containerkey),
    PRIMARY KEY (objid, containerid)
    ) ENGINE=InnoDB
    PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));

CREATE TABLE netapp_performance.sample_current_lun (
    objid bigint(20),
    time bigint(20),
    empty bool,
    containerkey varchar(255),
    containerid bigint(20) COMMENT 'Vserver container reference ID',
    avgLatency FLOAT COMMENT 'Average latency in milliseconds for all operations on the LUN (milliseconds)',
    avgReadLatency FLOAT COMMENT 'Average read latency in milliseconds for all operations on the LUN (milliseconds)',
    avgWriteLatency FLOAT COMMENT 'Average write latency in milliseconds for all operations on the LUN (milliseconds)',
    otherOps FLOAT COMMENT 'Number of other operations (ops/s)',
    readData FLOAT COMMENT 'Read bytes (bytes/s)',
    readOps FLOAT COMMENT 'Number of read operations (ops/s)',
    readPartialBlocks FLOAT COMMENT 'Percent of reads whose size is not a multiple of WAFL block size (percent, 0-100)',
    totalOps FLOAT COMMENT 'Total number of operations on the LUN (ops/s)',
    writeData FLOAT COMMENT 'Write bytes (bytes/s)',
    writeOps FLOAT COMMENT 'Number of write operations (ops/s)',
    writePartialBlocks FLOAT COMMENT 'Percent of writes whose size is not a multiple of WAFL block size (percent, 0-100)',
    KEY containerkey_index (containerkey),
    PRIMARY KEY (objid, containerid)
    ) ENGINE=InnoDB
    PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));

CREATE TABLE netapp_performance.sample_current_networklif (
    objid bigint(20),
    time bigint(20),
    empty bool,
    containerkey varchar(255),
    containerid bigint(20) COMMENT 'Vserver container reference ID',
    recvData float COMMENT 'Number of bytes received per second',
    recvErrors float COMMENT 'Number of received Errors per second',
    recvPacket float COMMENT 'Number of packets received per second',
    sentData float COMMENT 'Number of bytes sent per second',
    sentErrors float COMMENT 'Number of sent errors per second',
    sentPacket float COMMENT 'Number of packets sent per second',
    KEY containerkey_index (containerkey),
    PRIMARY KEY (objid, containerid)
    ) ENGINE=InnoDB
    PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));

CREATE TABLE netapp_performance.sample_current_networkport (
    objid bigint(20),
    time bigint(20),
    empty bool,
    containerkey varchar(255),
    containerid bigint(20) COMMENT 'Cluster container reference ID',
    collisions float COMMENT 'Collisions per second on CSMA interfaces',
    recvData float COMMENT 'Total bytes received per second',
    recvDropPackets float COMMENT 'Receive packets dropped per second',
    recvErrors float COMMENT 'Errors per second while receiving packets',
    recvMcasts float COMMENT 'Packets received per second via multicast',
    recvPackets float COMMENT 'Packets received per second',
    sendData float COMMENT 'Total bytes sent per second',
    sendErrors float COMMENT 'Errors per second while sending packets',
    sendMcasts float COMMENT 'Packets sent per second via multicast',
    sendPackets float COMMENT 'Packets sent per second',
    KEY containerkey_index (containerkey),
    PRIMARY KEY (objid, containerid)
    ) ENGINE=InnoDB
    PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));

CREATE TABLE netapp_performance.sample_current_node (
    objid bigint(20),
    time bigint(20),
    empty bool,
    containerkey varchar(255),
    containerid bigint(20) COMMENT 'Cluster container reference ID',
    avgProcessorBusy float COMMENT 'Average processor utilization across all processors in the system (percent, 0-100)',
    cifsOps float COMMENT 'CIFS operations per second (ops/s)',
    cpuBusy float COMMENT 'System CPU resource utilization (percent, 0-100)',
    diskDataRead float COMMENT 'Disk KB read per second (KB/s)',
    diskDataWritten float COMMENT 'Disk KB written per second (KB/s)',
    fcpOps float COMMENT 'FCP operations per second (ops/s)',
    iscsiOps float COMMENT 'iSCSI operations per second (ops/s)',
    netDataRecv float COMMENT 'Network KB received per second (KB/s)',
    netDataSent float COMMENT 'Network KB sent per second (KB/s)',
    nfsOps float COMMENT 'NFS operations per second (ops/s)',
    cifsOtherLatency float COMMENT '(Volume Aggregation) Average time for the WAFL filesystem to process other CIFS operations to the volume- not including CIFS protocol request processing or network communication (microseconds)',
    cifsOtherOps float COMMENT '(Volume Aggregation) Number of other cifs operations per second to the volume (ops/s)',
    cifsReadData float COMMENT '(Volume Aggregation)  Bytes read per second via cifs from the volume (bytes/s)',
    cifsReadLatency float COMMENT '(Volume Aggregation) Average time for the WAFL filesystem to process CIFS read requests to the volume- not including CIFS protocol request processing or network communication (microseconds)',
    cifsReadOps float COMMENT '(Volume Aggregation) Number of cifs reads per second to the volume (ops/s)',
    cifsWriteData float COMMENT '(Volume Aggregation) Bytes written per second via cifs to the volume (bytes/s)',
    cifsWriteLatency float COMMENT '(Volume Aggregation) Average time for the WAFL filesystem to process CIFS write requests to the volume- not including CIFS protocol request processing or network communication (microseconds)',
    cifsWriteOps float COMMENT '(Volume Aggregation) Number of cifs writes per second to the volume (ops/s)',
    fcpOtherLatency float COMMENT '(Volume Aggregation) Average time for the WAFL filesystem to process other FCP protocol operations to the volume- not including FCP protocol request processing or network communication (microseconds)',
    fcpOtherOps float COMMENT '(Volume Aggregation) Number of other block protocol operations per second to the volume (ops/s)',
    fcpReadData float COMMENT '(Volume Aggregation) Bytes read per second via block protocol from the volume (bytes/s)',
    fcpReadLatency float COMMENT '(Volume Aggregation) Average time for the WAFL filesystem to process FCP protocol read operations to the volume- not including FCP protocol request processing or network communication (microseconds)',
    fcpReadOps float COMMENT '(Volume Aggregation) Number of block protocol reads per second to the volume (ops/s)',
    fcpWriteData float COMMENT '(Volume Aggregation) Bytes written per second via block protocol to the volume (bytes/s)',
    fcpWriteLatency float COMMENT '(Volume Aggregation) Average time for the WAFL filesystem to process FCP protocol write operations to the volume- not including FCP protocol request processing or network communication (microseconds)',
    fcpWriteOps float COMMENT '(Volume Aggregation) Number of block protocol writes per second to the volume (ops/s)',
    iscsiOtherLatency float COMMENT '(Volume Aggregation) Average time for the WAFL filesystem to process other iSCSI protocol operations to the volume- not including iSCSI protocol request processing or network communication (microseconds)',
    iscsiOtherOps float COMMENT '(Volume Aggregation) Number of other block protocol operations per second to the volume (ops/s)',
    iscsiReadData float COMMENT '(Volume Aggregation) Bytes read per second via block protocol from the volume (bytes/s)',
    iscsiReadLatency float COMMENT '(Volume Aggregation) Average time for the WAFL filesystem to process iSCSI protocol read operations to the volume- not including iSCSI protocol request processing or network communication (microseconds) ',
    iscsiReadOps float COMMENT '(Volume Aggregation) Number of block protocol reads per second to the volume (ops/s)',
    iscsiWriteData float COMMENT '(Volume Aggregation) Bytes written per second via block protocol to the volume (bytes/s)',
    iscsiWriteLatency float COMMENT '(Volume Aggregation) Average time for the WAFL filesystem to process iSCSI protocol write operations to the volume- not including iSCSI protocol request processing or network communication (microseconds)',
    iscsiWriteOps float COMMENT '(Volume Aggregation) Number of block protocol writes per second to the volume (ops/s)',
    nfsOtherLatency float COMMENT '(Volume Aggregation) Average time for the WAFL filesystem to process other NFS operations to the volume- not including NFS protocol request processing or network communication (microseconds)',
    nfsOtherOps float COMMENT '(Volume Aggregation) Number of other NFS operations per second to the volume (ops/s)',
    nfsReadData float COMMENT '(Volume Aggregation) Bytes read per second via NFS from the volume (bytes/s)',
    nfsReadLatency float COMMENT '(Volume Aggregation) Average time for the WAFL filesystem to process NFS protocol read requests to the volume- not including NFS protocol request processing or network communication (microseconds)',
    nfsReadOps float COMMENT '(Volume Aggregation) Number of NFS reads per second to the volume (ops/s)',
    nfsWriteData float COMMENT '(Volume Aggregation) Bytes written per second via NFS to the volume (bytes/s)',
    nfsWriteLatency float COMMENT '(Volume Aggregation) Average time for the WAFL filesystem to process NFS protocol write requests to the volume- not including NFS protocol request processing or network communcation (microseconds)',
    nfsWriteOps float COMMENT '(Volume Aggregation) Number of NFS writes per second to the volume (ops/s)',
    KEY containerkey_index (containerkey),
    PRIMARY KEY (objid, containerid)
    ) ENGINE=InnoDB
    PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));

CREATE TABLE netapp_performance.sample_current_processor (
    objid bigint(20),
    time bigint(20),
    empty bool,
    containerkey varchar(255),
    containerid bigint(20) COMMENT 'Cluster container reference ID',
    processorBusy float COMMENT 'Percentage of elapsed time that the processor is executing non-idle processes',
    dBladeBusy float COMMENT 'Sum of elements from domain_busy representing d_blade usage:exempt,kahuna,raid,sm_exempt,storage,wafl_exempt,wafl_xcleaner',
    nBladeBusy float COMMENT 'Sum of elements from domain_busy representing n_blade usage:cifs,cluster,dnscache,nwk_exclusive,nwk_exempt,nwk_legacy,protocol,target',
    KEY containerkey_index (containerkey),
    PRIMARY KEY (objid, containerid)
    ) ENGINE=InnoDB
    PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));

CREATE TABLE netapp_performance.sample_current_status (
    package varchar(30),
    containerkey varchar(255),
    containerid bigint(20),
    lastsampletime bigint(20),
    PRIMARY KEY (package, containerkey)
    ) ENGINE=InnoDB;

CREATE TABLE netapp_performance.sample_current_volume (
    objid BIGINT(20) COMMENT 'Reference ID of volume',
    time BIGINT(20),
    empty BOOL,
    containerkey VARCHAR(255),
    containerid BIGINT(20) COMMENT 'Vserver container reference ID',
    avgLatency FLOAT COMMENT 'Average latency in microseconds for the WAFL filesystem to process all the operations on the volume- not including request processing or network communication (microseconds)',
    cifsOtherLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process other CIFS operations to the volume- not including CIFS protocol request processing or network communication (microseconds)',
    cifsOtherOps FLOAT COMMENT 'Number of other cifs operations per second to the volume (ops/s)',
    cifsReadData FLOAT COMMENT 'Bytes read per second via cifs from the volume (bytes/s)',
    cifsReadLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process CIFS read requests to the volume- not including CIFS protocol request processing or network communication (microseconds)',
    cifsReadOps FLOAT COMMENT 'Number of cifs reads per second to the volume (ops/s)',
    cifsWriteData FLOAT COMMENT 'Bytes written per second via cifs to the volume (bytes/s)',
    cifsWriteLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process CIFS write requests to the volume- not including CIFS protocol request processing or network communication (microseconds)',
    cifsWriteOps FLOAT COMMENT 'Number of cifs writes per second to the volume (ops/s)',
    fcpOtherLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process other FCP protocol operations to the volume- not including FCP protocol request processing or network communcation (microseconds)',
    fcpOtherOps FLOAT COMMENT 'Number of other block protocol operations per second to the volume (ops/s)',
    fcpReadData FLOAT COMMENT 'Bytes read per second via block protocol from the volume (bytes/s)',
    fcpReadLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process FCP protocol read operations to the volume- not including FCP protocol request processing or network communication (microseconds)',
    fcpReadOps FLOAT COMMENT 'Number of block protocol reads per second to the volume (ops/s)',
    fcpWriteData FLOAT COMMENT 'Bytes written per second via block protocol to the volume (bytes/s)',
    fcpWriteLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process FCP protocol write operations to the volume- not including FCP protocol request processing or network communication (microseconds)',
    fcpWriteOps FLOAT COMMENT 'Number of block protocol writes per second to the volume (ops/s)',
    iscsiOtherLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process other iSCSI protocol operations to the volume- not including iSCSI protocol request processing or network communication (microseconds)',
    iscsiOtherOps FLOAT COMMENT 'Number of other block protocol operations per second to the volume (ops/s)',
    iscsiReadData FLOAT COMMENT 'Bytes read per second via block protocol from the volume (byes/s)',
    iscsiReadLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process iSCSI protocol read operations to the volume- not including iSCSI protocol request processing or network communication (microseconds)',
    iscsiReadOps FLOAT COMMENT 'Number of block protocol reads per second to the volume (ops/s)',
    iscsiWriteData FLOAT COMMENT 'Bytes written per second via block protocol to the volume (bytes/s)',
    iscsiWriteLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process iSCSI protocol write operations to the volume- not including iSCSI protocol request processing or network communication (microseconds)',
    iscsiWriteOps FLOAT COMMENT 'Number of block protocol writes per second to the volume (ops/s)',
    nfsOtherLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process other NFS operations to the volume- not including NFS protocol request processing or network communication (microseconds)',
    nfsOtherOps FLOAT COMMENT 'Number of other NFS operations per second to the volume (ops/s)',
    nfsReadData FLOAT COMMENT 'Bytes read per second via NFS from the volume (bytes/s)',
    nfsReadLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process NFS protocol read requests to the volume- not including NFS protocol request processing or network communication (microseconds)',
    nfsReadOps FLOAT COMMENT 'Number of NFS reads per second to the volume (ops/s)',
    nfsWriteData FLOAT COMMENT 'Bytes written per second via NFS to the volume (bytes/s)',
    nfsWriteLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process NFS protocol write requests to the volume- not including NFS protocol request processing or network communication (microseconds)',
    nfsWriteOps FLOAT COMMENT 'Number of NFS writes per second to the volume (ops/s)',
    otherLatency FLOAT COMMENT 'Average latency in microseconds for the WAFL filesystem to process other operations to the volume- not including request processing or network communication (microseconds)',
    otherOps FLOAT COMMENT 'Number of other operations per second to the volume (ops/s)',
    readData FLOAT COMMENT 'Bytes read per second from the volume (bytes/s)',
    readLatency FLOAT COMMENT 'Average latency in microseconds for the WAFL filesystem to process read request to the volume- not including request processing or network communication (microseconds)',
    readOps FLOAT COMMENT 'Number of reads per second to the volume (ops/s)',
    sanOtherLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process other block protocol requests to the volume- not including block protocol request processing or network communication (microseconds)',
    sanOtherOps FLOAT COMMENT 'Number of other block protocol operations per second to the volume (ops/s)',
    sanReadData FLOAT COMMENT 'Bytes read per second via block protocol from the volume (bytes/s)',
    sanReadLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process block protocol read requests to the volume- not including block protocol request processing or network communication (microseconds)',
    sanReadOps FLOAT COMMENT 'Number of block protocol reads per second to the volume (ops/s)',
    sanWriteData FLOAT COMMENT 'Bytes written per second via block protocol to the volume (bytes/s)',
    sanWriteLatency FLOAT COMMENT 'Average time for the WAFL filesystem to process block protocol write requests to the volume- including block protocol request processing or network communication (microseconds)',
    sanWriteOps FLOAT COMMENT 'Number of block protocol writes per second to the volume (ops/s)',
    totalOps FLOAT COMMENT 'Number of operations per second serviced by the volume (ops/s)',
    writeData FLOAT COMMENT 'Bytes written per second to the volume (bytes/s)',
    writeLatency FLOAT COMMENT 'Average latency in microseconds for the WAFL filesystem to process write request to the volume- not including request processing or network communication (microseconds)',
    writeOps FLOAT COMMENT 'Number of writes per second to the volume (ops/s)',
    KEY containerkey_index (containerkey),
    PRIMARY KEY (objid, containerid)
    ) ENGINE=INNODB
    PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));

CREATE TABLE netapp_performance.sample_current_vserver (
    objid BIGINT(20) COMMENT 'Reference ID of vserver',
    time BIGINT(20),
    empty BOOL,
    containerkey VARCHAR(255),
    containerid BIGINT(20) COMMENT 'Vserver container reference ID',
    avgLatency FLOAT COMMENT 'volume:vserver-Average latency in microseconds for the WFL Filesystem to process all the operations on the volume of vserver-not including request processing or network communication(microseconds)',
    cifsOtherLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process other CIFS operations to the volume of vserver- not including CIFS protocol request processing or network communication (microseconds)',
    cifsOtherOps FLOAT COMMENT 'volume:vserver-Number of other cifs operations per second to the volume of vserver (ops/s)',
    cifsReadData FLOAT COMMENT 'volume:vserver-Bytes read per second via cifs from the volume of vserver (bytes/s)',
    cifsReadLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process CIFS read requests to the volume of vserver- not including CIFS protocol request processing or network communication (microseconds)',
    cifsReadOps FLOAT COMMENT 'volume:vserver-Number of cifs reads per second to the volume of vserver (ops/s)',
    cifsWriteData FLOAT COMMENT 'volume:vserver-Bytes written per second via cifs to the volume of vserver (bytes/s)',
    cifsWriteLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process CIFS write requests to the volume of vserver- not including CIFS protocol request processing or network communication (microseconds)',
    cifsWriteOps FLOAT COMMENT 'volume:vserver-Number of cifs writes per second to the volume of vserver (ops/s)',
    fcpOtherLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process other FCP protocol operations to the volume of vserver- not including FCP protocol request processing or network communcation (microseconds)',
    fcpOtherOps FLOAT COMMENT 'volume:vserver-Number of other block protocol operations per second to the volume of vserver (ops/s)',
    fcpReadData FLOAT COMMENT 'volume:vserver-Bytes read per second via block protocol from the volume of vserver (bytes/s)',
    fcpReadLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process FCP protocol read operations to the volume of vserver- not including FCP protocol request processing or network communication (microseconds)',
    fcpReadOps FLOAT COMMENT 'volume:vserver-Number of block protocol reads per second to the volume of vserver (ops/s)',
    fcpWriteData FLOAT COMMENT 'volume:vserver-Bytes written per second via block protocol to the volume of vserver (bytes/s)',
    fcpWriteLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process FCP protocol write operations to the volume of vserver- not including FCP protocol request processing or network communication (microseconds)',
    fcpWriteOps FLOAT COMMENT 'volume:vserver-Number of block protocol writes per second to the volume of vserver (ops/s)',
    iscsiOtherLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process other iSCSI protocol operations to the volume of vserver- not including iSCSI protocol request processing or network communication (microseconds)',
    iscsiOtherOps FLOAT COMMENT 'volume:vserver-Number of other block protocol operations per second to the volume of vserver (ops/s)',
    iscsiReadData FLOAT COMMENT 'volume:vserver-Bytes read per second via block protocol from the volume of vserver (bytes/s)',
    iscsiReadLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process iSCSI protocol read operations to the volume of vserver- not including iSCSI protocol request processing or network communication (microseconds)',
    iscsiReadOps FLOAT COMMENT 'volume:vserver-Number of block protocol reads per second to the volume of vserver (ops/s)',
    iscsiWriteData FLOAT COMMENT 'volume:vserver-Bytes written per second via block protocol to the volume of vserver (bytes/s)',
    iscsiWriteLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process iSCSI protocol write operations to the volume of vserver- not including iSCSI protocol request processing or network communication (microseconds)',
    iscsiWriteOps FLOAT COMMENT 'volume:vserver-Number of block protocol writes per second to the volume of vserver (ops/s)',
    nfsOtherLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process other NFS operations to the volume of vserver- not including NFS protocol request processing or network communication (microseconds)',
    nfsOtherOps FLOAT COMMENT 'volume:vserver-Number of other NFS operations per second to the volume of vserver (ops/s)',
    nfsReadData FLOAT COMMENT 'volume:vserver-Bytes read per second via NFS from the volume of vserver (bytes/s)',
    nfsReadLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process NFS protocol read requests to the volume of vserver- not including NFS protocol request processing or network communication (microseconds)',
    nfsReadOps FLOAT COMMENT 'volume:vserver-Number of NFS reads per second to the volume of vserver (ops/s)',
    nfsWriteData FLOAT COMMENT 'volume:vserver-Bytes written per second via NFS to the volume of vserver (bytes/s)',
    nfsWriteLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process NFS protocol write requests to the volume of vserver- not including NFS protocol request processing or network communication (microseconds)',
    nfsWriteOps FLOAT COMMENT 'volume:vserver-Number of NFS writes per second to the volume of vserver (ops/s)',
    otherLatency FLOAT COMMENT 'volume:vserver-Average latency in microseconds for the WAFL filesystem to process other operations to the volume of vserver- not including request processing or network communication (microseconds)',
    otherOps FLOAT COMMENT 'volume:vserver-Number of other operations per second to the volume of vserver (ops/s)',
    readData FLOAT COMMENT 'volume:vserver-Bytes read per second from the volume of vserver (bytes/s)',
    readLatency FLOAT COMMENT 'volume:vserver-Average latency in microseconds for the WAFL filesystem to process read request to the volume of vserver- not including request processing or network communication (microseconds)',
    readOps FLOAT COMMENT 'volume:vserver-Number of reads per second to the volume of vserver (ops/s)',
    sanOtherLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process other block protocol requests to the volume of vserver- not including block protocol request processing or network communication (microseconds)',
    sanOtherOps FLOAT COMMENT 'volume:vserver-Number of other block protocol operations per second to the volume of vserver (ops/s)',
    sanReadData FLOAT COMMENT 'volume:vserver-Bytes read per second via block protocol from the volume of vserver (bytes/s)',
    sanReadLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process block protocol read requests to the volume of vserver- not including block protocol request processing or network communication (microseconds)',
    sanReadOps FLOAT COMMENT 'volume:vserver-Number of block protocol reads per second to the volume of vserver (ops/s)',
    sanWriteData FLOAT COMMENT 'volume:vserver-Bytes written per second via block protocol to the volume of vserver (bytes/s)',
    sanWriteLatency FLOAT COMMENT 'volume:vserver-Average time for the WAFL filesystem to process block protocol write requests to the volume of vserver- including block protocol request processing or network communication (microseconds)',
    sanWriteOps FLOAT COMMENT 'volume:vserver-Number of block protocol writes per second to the volume of vserver (ops/s)',
    totalOps FLOAT COMMENT 'volume:vserver-Number of operations per second serviced by the volume of vserver (ops/s)',
    writeData FLOAT COMMENT 'volume:vserver-Bytes written per second to the volume of vserver (bytes/s)',
    writeLatency FLOAT COMMENT 'volume:vserver-Average latency in microseconds for the WAFL filesystem to process write request to the volume of vserver- not including request processing or network communication (microseconds)',
    writeOps FLOAT COMMENT 'volume:vserver-Number of writes per second to the volume of vserver (ops/s)',
    fcpLifAvgOtherLatency FLOAT COMMENT 'fcp_lif:vserver-Average latency for operations other than read and write (microseconds)',
    fcpLifAvgReadLatency FLOAT COMMENT 'fcp_lif:vserver-Average latency for read operations (microseconds)',
    fcpLifAvgWriteLatency FLOAT COMMENT 'fcp_lif:vserver-Average latency for write operations (microseconds)',
    fcpLifOtherOps FLOAT COMMENT 'fcp_lif:vserver-Number of operations that are not read or write (ops/s)',
    fcpLifReadData FLOAT COMMENT 'fcp_lif:vserver-Amount of data read from the storage system (bytes/s)',
    fcpLifReadOps FLOAT COMMENT 'fcp_lif:vserver-Number of read operations (ops/s)',
    fcpLifTotalOps FLOAT COMMENT 'fcp_lif:vserver-Total number of operations (ops/s)',
    fcpLifWriteData FLOAT COMMENT 'fcp_lif:vserver-Amount of data written to the storage system (bytes/s)',
    fcpLifWriteOps FLOAT COMMENT 'fcp_lif:vserver-Number of write operations (ops/s)',
    iscsiLifAvgOtherLatency FLOAT COMMENT 'iscsi_lif:vserver-Average latency for operations other than read and write (for example, Inquiry, Report LUNs, SCSI Task Management Functions) (microseconds)',
    iscsiLifAvgReadLatency FLOAT COMMENT 'iscsi_lif:vserver-Average latency for read operations (microseconds)',
    iscsiLifAvgWriteLatency FLOAT COMMENT 'iscsi_lif:vserver-Average latency for write operations (microseconds)',
    iscsiLifDataInBlocks FLOAT COMMENT 'iscsi_lif:vserver-number of blocks sent from iSCSI sessions on this LIF',
    iscsiLifDataOutBlocks FLOAT COMMENT 'iscsi_lif:vserver-number of blocks received from iSCSI sessions on this LIF',
    iscsiLifOtherOps FLOAT COMMENT 'iscsi_lif:vserver-iSCSI other operations per second on this LIF (ops/s)',
    iscsiLifReadOps FLOAT COMMENT 'iscsi_lif:vserver-iSCSI read operations per second on this LIF (ops/s)',
    iscsiLifWriteOps FLOAT COMMENT 'iscsi_lif:vserver-iSCSI write operations per second on this LIF (ops/s)',
    KEY containerkey_index (containerkey),
    PRIMARY KEY (objid, containerid)
    ) ENGINE=INNODB
    PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));
