CREATE TABLE netapp_model.volume_flexcache (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  volume-get-iter volume-attributes volume-flexcache-attributes',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the Volume Flexcache Pair',
    flexCacheId BIGINT(20) NOT NULL,
    originVolumeId BIGINT(20) NOT NULL,
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_flexCacheId (flexCacheId) REFERENCES netapp_model.volume (objid),
    CONSTRAINT FOREIGN KEY FK_originVolumeId (originVolumeId) REFERENCES netapp_model.volume (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB;


ALTER TABLE netapp_model.vserver ADD COLUMN resourceKey VARCHAR(255) NOT NULL COMMENT 'DERIVED.  A structured key that may be used to uniquely identify this resource globally.' AFTER uuid;
ALTER TABLE netapp_model.vserver ADD COLUMN volumeBytesAvail BIGINT(20) DEFAULT NULL COMMENT 'DERIVED.  Difference between volumeBytesTotal and volumeBytesUsed.' AFTER volumeBytesUsed;

ALTER TABLE netapp_model.node ADD COLUMN resourceKey VARCHAR(255) NOT NULL COMMENT 'DERIVED.  A structured key that may be used to uniquely identify this resource globally.' AFTER uuid;
ALTER TABLE netapp_model.node ADD COLUMN aggregateBytesTotal BIGINT(20) DEFAULT NULL COMMENT 'DERIVED.  Sum of bytes total (including snapshot) on all aggregates in the Node.' AFTER rawDiskBytesUsed;
ALTER TABLE netapp_model.node ADD COLUMN aggregateBytesUsed BIGINT(20) DEFAULT NULL COMMENT 'DERIVED.  Sum of bytes used (including snapshot) on all aggregates in the Node.' AFTER aggregateBytesTotal;

ALTER TABLE netapp_model.aggregate ADD COLUMN resourceKey VARCHAR(255) NOT NULL COMMENT 'DERIVED.  A structured key that may be used to uniquely identify this resource globally.' AFTER uuid;
ALTER TABLE netapp_model.aggregate ADD COLUMN sizeUsedPercent INT DEFAULT NULL COMMENT 'Size used on aggregate as a percentage of total.  ZAPIs: aggr-get-iter aggr-space-attributes percent-used-capacity' AFTER sizeUsed;
ALTER TABLE netapp_model.aggregate ADD COLUMN sizeAvailPercent INT DEFAULT NULL COMMENT 'DERIVED. Size available on aggregate as a percentage of total.' AFTER sizeAvail;

ALTER TABLE netapp_model.volume ADD COLUMN resourceKey VARCHAR(255) NOT NULL COMMENT 'DERIVED.  A structured key that may be used to uniquely identify this resource globally.' AFTER uuid;
ALTER TABLE netapp_model.volume ADD COLUMN flexCacheOriginId BIGINT(20) DEFAULT NULL COMMENT 'The name of the flex cache origin volume that contains the authoritative data.  Only applicable on FlexCache volumes.  ZAPIs: volume-get-iter volume-flexcache-attributes origin' AFTER cloneParentId;
ALTER TABLE netapp_model.volume ADD COLUMN sizeAvail BIGINT(20) DEFAULT NULL COMMENT 'DERIVED.  Difference between sizeTotal and sizeUsed.' AFTER sizeUsed;
ALTER TABLE netapp_model.volume ADD COLUMN percentageSnapshotReserve INT DEFAULT NULL COMMENT 'Percentage of the volume reserved for snapshots.  Integer 0-100.  ZAPIs: volume-get-iter volume-space-attributes percentage-snapshot-reserve' AFTER snapshotReserveSize;
ALTER TABLE netapp_model.volume ADD COLUMN sizeUsedBySnapshots BIGINT(20) DEFAULT NULL COMMENT 'The size (in bytes) that is used by snapshots in the volume.  ZAPIs: volume-get-iter size-volume-space-attributes size-used-by-snapshots' AFTER percentageSnapshotReserve;
ALTER TABLE netapp_model.volume ADD COLUMN inodeFilesUsed BIGINT(20) DEFAULT NULL COMMENT 'Number of user-visible files (inodes) used. This field is valid only when the volume is online.  ZAPIs: volume-get-iter volume-inode-attributes files-used' AFTER hybridCacheEligibility;
ALTER TABLE netapp_model.volume ADD COLUMN inodeFilesTotal BIGINT(20) DEFAULT NULL COMMENT 'Total user-visible file (inode) count, i.e., current maximum number of user-visible files (inodes) that this volume can currently hold.  ZAPIs: volume-get-iter volume-inode-attributes files-total' AFTER inodeFilesUsed;
ALTER TABLE netapp_model.volume ADD COLUMN inodeBlockType ENUM('NOT_MAPPED','32_BIT','64_BIT') DEFAULT NULL COMMENT 'The indirect block format of the volume.  ZAPIs: volume-get-iter volume-inode-attributes block-type' AFTER inodeFilesTotal;
ALTER TABLE netapp_model.volume ADD COLUMN quotaCommitted BIGINT(20) DEFAULT NULL COMMENT 'DERIVED.  Sum of quota limits on all Qtrees on this Volume.' AFTER inodeBlockType;
ALTER TABLE netapp_model.volume ADD COLUMN flexCacheMinReserve BIGINT(20) DEFAULT NULL COMMENT 'The amount of space requested to be preallocated in the aggregate hosting the FlexCache volume. ZAPIs: volume-get-iter volume-flexcache-attributes min-reserve' AFTER quotaCommitted;
ALTER TABLE netapp_model.volume ADD COLUMN spaceGuarantee ENUM('NOT_MAPPED','NONE','VOLUME','FILE') DEFAULT NULL COMMENT 'The storage guarantee option that is associated with the flexible volume. ZAPIs: volume-get-iter volume-space-attributes space-guarantee' AFTER isSpaceGuaranteeEnabled;
ALTER TABLE netapp_model.volume ADD COLUMN isSnapshotCloneDependencyEnabled BOOL DEFAULT NULL COMMENT 'Controls the locking of LUN clones. ZAPIs: volume-get-iter volume-snapshot-attributes snapshot-clone-dependency-enabled' AFTER flexCacheMinReserve;
ALTER TABLE netapp_model.volume ADD COLUMN isI2pEnabled BOOL DEFAULT NULL COMMENT 'If true, native inode-to-parent information is kept for inodes. ZAPIs: volume-get-iter volume-directory-attributes i2p-enabled' AFTER isSnapshotCloneDependencyEnabled;
ALTER TABLE netapp_model.volume ADD COLUMN isAutoSnapshotsEnabled BOOL DEFAULT NULL COMMENT 'If true, enable automatic snapshots on the volume. ZAPIs: volume-get-iter volume-snapshot-attributes auto-snapshots-enabled' AFTER isI2pEnabled;
ALTER TABLE netapp_model.volume ADD COLUMN isSnapDirAccessEnabled BOOL DEFAULT NULL COMMENT 'If true, enable the visible .snapshot directory that is normally present at system internal mount points. ZAPIs: volume-get-iter volume-snapshot-attributes snapdir-access-enabled' AFTER isAutoSnapshotsEnabled;
ALTER TABLE netapp_model.volume ADD COLUMN languageCode VARCHAR(255) DEFAULT NULL COMMENT 'The volume language code (e.g. en_US). ZAPIs: volume-get-iter volume-language-attributes language-code' AFTER isSnapDirAccessEnabled;
ALTER TABLE netapp_model.volume ADD COLUMN volType ENUM('NOT_MAPPED','RW','LS','DP','DC','TMP') DEFAULT NULL COMMENT 'The type of the volume. ZAPIs: volume-get-iter volume-id-attributes type' AFTER languageCode;
ALTER TABLE netapp_model.volume ADD COLUMN spaceMgmtOptionTryFirst ENUM('NOT_MAPPED','VOLUME_GROW','SNAP_DELETE') DEFAULT NULL COMMENT 'This field controls the strategy to try first when this volume is space-challenged. ZAPIs: volume-get-iter volume-space-attributes space-mgmt-option-try-first' AFTER volType;
ALTER TABLE netapp_model.volume ADD COLUMN overwriteReserveAvail BIGINT(20) DEFAULT NULL COMMENT 'DERIVED.  OverwriteReserve - OverwriteReserveUsed.' AFTER overwriteReserveUsed;
ALTER TABLE netapp_model.volume ADD COLUMN autoSizeMode ENUM('NOT_MAPPED','OFF','GROW','GROW_SHRINK') DEFAULT NULL COMMENT 'The current mode of operation of volume autosize.  ZAPIs: volume-get-iter volume-autosize-attributes mode' AFTER spaceMgmtOptionTryFirst;
ALTER TABLE netapp_model.volume ADD COLUMN autoSizeMaximumSize BIGINT(20) DEFAULT NULL COMMENT 'The maximum size (in bytes) to which the volume would be grown automatically.  ZAPIs: volume-get-iter volume-autosize-attributes maximum-size' AFTER autoSizeMode;
ALTER TABLE netapp_model.volume ADD COLUMN autoSizeIncrementSize BIGINT(20) DEFAULT NULL COMMENT 'The increment size (in bytes) by which the volume would be grown.  ZAPIs: volume-get-iter volume-autosize-attributes increment-size' AFTER autoSizeMaximumSize;
ALTER TABLE netapp_model.volume ADD COLUMN isAtimeUpdateEnabled BOOL DEFAULT NULL COMMENT 'If false, prevent the update of inode access times when a file is read.  ZAPIs: volume-get-iter volume-performance-attributes is-atime-update-enabled' AFTER autoSizeIncrementSize;
ALTER TABLE netapp_model.volume ADD COLUMN isCreateUcodeEnabled BOOL DEFAULT NULL COMMENT 'This field controls whether all directories in this volume are forced to use the UNICODE format when they are created, both from NFS and CIFS.  ZAPIs: volume-get-iter volume-language-attributes is-create-ucode-enabled' AFTER isAtimeUpdateEnabled;
ALTER TABLE netapp_model.volume ADD COLUMN isConvertUcodeEnabled BOOL DEFAULT NULL COMMENT 'This field controls whether all directories in this volume are forcibly converted to UNICODE format when accessed from both NFS and CIFS.  ZAPIs: volume-get-iter volume-language-attributes is-convert-ucode-enabled' AFTER isCreateUcodeEnabled;
ALTER TABLE netapp_model.volume ADD COLUMN cloneSplitEstimate BIGINT(20) DEFAULT NULL COMMENT 'If volume is a clone, the split Estimate.  ZAPIs: volume-clone-get-iter volume-clone-info split-estimate' AFTER isConvertUcodeEnabled;
ALTER TABLE netapp_model.volume ADD COLUMN cloneChildCount INT DEFAULT NULL COMMENT 'Number of clones for which this volume is the parent.  ZAPIs: volume-get-iter volume-clone-attributes clone-child-count' AFTER cloneSplitEstimate;
ALTER TABLE netapp_model.volume ADD COLUMN snapshotCount INT DEFAULT NULL COMMENT 'Number of Snapshot copies in the volume.  ZAPIs: volume-get-iter volume-snapshot-attributes snapshot-count' AFTER cloneChildCount;
ALTER TABLE netapp_model.volume ADD COLUMN isSnapshotAutoDeleteEnabled BOOL DEFAULT NULL COMMENT 'This option determines if the snapshot autodelete is currently enabled for the volume.  ZAPIs: volume-get-iter volume-snapshot-autodelete-attributes is-autodelete-enabled' AFTER snapshotCount;
ALTER TABLE netapp_model.volume ADD COLUMN snapshotAutoDeleteCommitment ENUM('NOT_MAPPED','TRY','DISRUPT','DESTROY') DEFAULT NULL COMMENT 'This option determines the snapshots that "snapshot autodelete" is allowed to delete to get back space.  ZAPIs: volume-get-iter volume-snapshot-autodelete-attributes commitment' AFTER isSnapshotAutoDeleteEnabled;
ALTER TABLE netapp_model.volume ADD COLUMN snapshotAutoDeleteDeleteOrder ENUM('NOT_MAPPED','OLDEST_FIRST','NEWEST_FIRST') DEFAULT NULL COMMENT 'This option determines if the oldest or newest snapshot is deleted first.  ZAPIs: volume-get-iter volume-snapshot-autodelete-attributes delete-order' AFTER snapshotAutoDeleteCommitment;
ALTER TABLE netapp_model.volume ADD COLUMN snapshotAutoDeleteDeferDelete ENUM('NOT_MAPPED','SCHEDULED','USER_CREATED','PREFIX','NONE') DEFAULT NULL COMMENT 'This option determines which kind of snapshots to delete.  ZAPIs: volume-get-iter volume-snapshot-autodelete-attributes defer-delete' AFTER snapshotAutoDeleteDeleteOrder;
ALTER TABLE netapp_model.volume ADD COLUMN snapshotAutoDeleteTargetFreeSpace INT DEFAULT NULL COMMENT 'This option determines when snapshot autodelete should stop deleting snapshot. ZAPIs: volume-get-iter volume-snapshot-autodelete-attributes target-free-space' AFTER snapshotAutoDeleteDeferDelete;
ALTER TABLE netapp_model.volume ADD COLUMN snapshotAutoDeleteTrigger ENUM('NOT_MAPPED','VOLUME','SNAP_RESERVE','SPACE_RESERVE') DEFAULT NULL COMMENT 'This option determines the condition in which snapshots should be automatically deleted.  ZAPIs: volume-get-iter volume-snapshot-autodelete-attributes trigger' AFTER snapshotAutoDeleteTargetFreeSpace;
ALTER TABLE netapp_model.volume ADD COLUMN snapshotAutoDeletePrefix VARCHAR(255) DEFAULT NULL COMMENT 'This option provides the prefix string for the "prefix" field value of the "defer-delete" option.   ZAPIs: volume-get-iter volume-snapshot-autodelete-attributes prefix' AFTER snapshotAutoDeleteTrigger;
ALTER TABLE netapp_model.volume ADD COLUMN snapshotAutoDeleteDestroyList VARCHAR(255) DEFAULT NULL COMMENT 'This options specifies the list of service that can be destroyed if the snapshot is backing that service.   ZAPIs: volume-get-iter volume-snapshot-autodelete-attributes destroy-list' AFTER snapshotAutoDeletePrefix;
ALTER TABLE netapp_model.volume ADD COLUMN sisStatus ENUM('NOT_MAPPED','IDLE','INITIALIZING','ACTIVE','UNDOING','PENDING','DOWNGRADING','DISABLED') DEFAULT NULL COMMENT 'Status of any sis operation running on the volume. This parameter is not supported on Infinite Volumes. ZAPIs: sis-get-iter status' AFTER snapshotAutoDeleteDestroyList;
ALTER TABLE netapp_model.volume ADD COLUMN sisState ENUM('NOT_MAPPED','DISABLED','ENABLED') DEFAULT NULL COMMENT 'State of sis configured on the volume.  ZAPIs: sis-get-iter state' AFTER sisStatus;
ALTER TABLE netapp_model.volume ADD COLUMN sisProgress VARCHAR(255) DEFAULT NULL COMMENT 'The progress of the current sis operation with information as to which stage of sis process is currently in progress and how much data is processed for that stage.  ZAPIs: sis-get-iter progress' AFTER sisState;
ALTER TABLE netapp_model.volume ADD COLUMN sisType ENUM('NOT_MAPPED','REGULAR','SNAPVAULT') DEFAULT NULL COMMENT 'Any sis volume with Snapvault qtree in it would be marked as Snapvault and all others would be returned as type Regular.  ZAPIs: sis-get-iter sis-status-info.type' AFTER sisProgress;
ALTER TABLE netapp_model.volume ADD COLUMN sisSchedule VARCHAR(255) DEFAULT NULL COMMENT 'The schedule for sis operation on the volume.  ZAPIs: sis-get-iter schedule' AFTER sisType;
ALTER TABLE netapp_model.volume ADD COLUMN sisLastOpBeginTimestamp BIGINT(20) DEFAULT NULL COMMENT 'Start timestamp of the last sis operation. The value is in seconds since January 1, 1970.  ZAPIs: sis-get-iter last-op-begin-timestamp' AFTER sisSchedule;
ALTER TABLE netapp_model.volume ADD COLUMN sisLastOpEndTimestamp BIGINT(20) DEFAULT NULL COMMENT 'End timestamp of the last sis operation. The value is in seconds since January 1, 1970.  ZAPIs: sis-get-iter last-op-end-timestamp' AFTER sisLastOpBeginTimestamp;
ALTER TABLE netapp_model.volume ADD COLUMN sisLastOpError VARCHAR(255) DEFAULT NULL COMMENT 'A human readable error message of the last sis operation. Present when there was an error. ZAPIs: sis-get-iter last-op-error' AFTER sisLastOpEndTimestamp;
ALTER TABLE netapp_model.volume ADD COLUMN sisLastOpState ENUM('NOT_MAPPED','SUCCESS','FAILURE') DEFAULT NULL COMMENT 'Completion status for the last operation. ZAPIs: sis-get-iter last-op-state' AFTER sisLastOpError;
ALTER TABLE netapp_model.volume ADD COLUMN sisLastOpSize BIGINT(20) DEFAULT NULL COMMENT 'The amount of data processed in bytes for the last sis operation.  ZAPIs: sis-get-iter last-op-size' AFTER sisLastOpState;
ALTER TABLE netapp_model.volume ADD COLUMN isSisCompressionEnabled BOOL DEFAULT NULL COMMENT 'Compression state of the volume. ZAPIs: sis-get-iter is-compression-enabled' AFTER sisLastOpSize;
ALTER TABLE netapp_model.volume ADD COLUMN isSisInlineCompressionEnabled BOOL DEFAULT NULL COMMENT 'Inline compression state of the volume.  ZAPIs: sis-get-iter is-inline-compression-enabled' AFTER isSisCompressionEnabled;
ALTER TABLE netapp_model.volume ADD COLUMN percentageDeduplicationSpaceSaved INT DEFAULT NULL COMMENT 'Percentage of the total disk space that is saved by deduplication and file cloning.  ZAPIs: volume-get-iter volume-sis-attributes percentage-deduplication-space-saved' AFTER isSisInlineCompressionEnabled;
ALTER TABLE netapp_model.volume ADD COLUMN percentageCompressionSpaceSaved INT DEFAULT NULL COMMENT 'Percentage of the total disk space that is saved by compressing blocks on the referenced file system.  ZAPIs: volume-get-iter volume-sis-attributes percentage-compression-space-saved' AFTER percentageDeduplicationSpaceSaved;
ALTER TABLE netapp_model.volume ADD COLUMN sizeUsedPercent INT DEFAULT NULL COMMENT 'DERIVED.  0-100.  Percentage bytes used out of total.' AFTER sizeUsed;
ALTER TABLE netapp_model.volume ADD COLUMN sizeAvailPercent INT DEFAULT NULL COMMENT 'DERIVED.  0-100.  Percentage bytes available out of total.' AFTER sizeAvail;
ALTER TABLE netapp_model.volume ADD COLUMN quotaOverCommitted BIGINT(20) DEFAULT NULL COMMENT 'DERIVED.  QuotaCommitted bytes adjusted by the volume size and snapshot size.' AFTER quotaCommitted;
















































