CREATE TABLE netapp_model.snapshot (
	objid BIGINT(20) NOT NULL,
	originid BIGINT(20) NOT NULL,
	uuid VARCHAR(255) NOT NULL COMMENT 'Globally UNIQUE ID of the snapshot',
	name VARCHAR(255) NOT NULL COMMENT 'Name of the snapshot.  ZAPI:  snapshot-get-ter snapshot-info name',
	resourceKey VARCHAR(255) NOT NULL COMMENT 'DERIVED.  A structured KEY that may be used TO uniquely identify this resource globally.',
	volumeId BIGINT(20) NOT NULL ,
	vserverId BIGINT(20) NOT NULL,
	clusterId BIGINT(20) NOT NULL,
	instanceUUID VARCHAR(255) DEFAULT NULL COMMENT 'The 128 bit unique snapshot identifier expressed in the form of UUID. This field uniquely identifies the snapshot\'s physical data layout.  ZAPI:  snapshot-get-ter snapshot-info snapshot-instance-uuid',
	versionUUID VARCHAR(255) DEFAULT NULL COMMENT 'The 128 bit unique snapshot identifier expressed in the form of UUID. This field uniquely identifies the snapshot\'s logical data layout.  ZAPI:  snapshot-get-ter snapshot-info snapshot-version-uuid',
	isBusy BOOL DEFAULT NULL COMMENT 'True if the snapshot is being used by an application.  ZAPI: snapshot-get-ter snapshot-info busy',
	accessTime BIGINT(20) DEFAULT NULL COMMENT 'The volume access time when the snapshot was created in seconds since Jan 1, 1970. This value will not change even if the snapshot is accessed. ZAPI: snapshot-get-ter snapshot-info access-time',
	dependency VARCHAR(255) DEFAULT NULL COMMENT 'Application(s) dependent on this snapshot.  ZAPI:  snapshot-get-ter snapshot-info dependency',
	percentageOfTotalBlocks INT DEFAULT NULL COMMENT 'Percentage of blocks owned by this snapshot, relative to the total number of blocks in the volume.  ZAPI: snapshot-get-ter snapshot-info percentage-of-total-blocks',
	cumulativePercentageOfTotalBlocks INT DEFAULT NULL COMMENT 'Percentage of blocks owned by this snapshot and all more recent snapshots, relative to the total number of blocks in the volume.  snapshot-get-ter snapshot-info cumulative-percentage-of-total-blocks',
	checksum INT NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	KEY updatetime_index (updatetime),
    KEY originId_index (originid),
	CONSTRAINT FOREIGN KEY FK_volumeId (volumeId) REFERENCES netapp_model.volume (objid),
	CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

ALTER TABLE netapp_model.volume ADD COLUMN oldestSnapshotTimestamp BIGINT(20) DEFAULT NULL COMMENT 'Creation time of the oldest snapshot of this volume' AFTER snapshotAutoDeleteDestroyList;