CREATE TABLE netapp_model.volume_move_job (
	objid BIGINT(20) NOT NULL,
	originid BIGINT(20) NOT NULL,
	uuid VARCHAR(255) NOT NULL COMMENT 'The job uuid of the move job.  ZAPI:  volume-move-get-iter volume-move-info job-uuid',
	resourceKey VARCHAR(255) NOT NULL COMMENT 'Globally UNIQUE ID of the volume move job',
	volumeId BIGINT(20) NOT NULL ,
	sourceAggregateId BIGINT(20) NOT NULL,
	destinationAggregateId BIGINT(20) NOT NULL,
	clusterId BIGINT(20) NOT NULL,
	jobIdentifier BIGINT(20) DEFAULT NULL COMMENT 'The ui job id of the move job.  ZAPI:  volume-move-get-iter volume-move-info job-id',
	startTime BIGINT(20) DEFAULT NULL COMMENT 'The date and time in cluster timezone when the volume move started.  ZAPI: volume-move-get-iter volume-move-info start-timestamp',
	completionTime BIGINT(20) DEFAULT NULL COMMENT 'The date and time in cluster timezone when the volume move completed (successfully or otherwise).   ZAPI: volume-move-get-iter volume-move-info actual-completion-timestamp',
	phase ENUM('NOT_MAPPED', 'QUEUED', 'INITIALIZING', 'REPLICATING', 'CUTOVER', 'CUTOVER_HARD_DEFERRED', 'CUTOVER_SOFT_DEFERRED', 'COMPLETED', 'CLEANING_UP', 'FAILED', 'RESTARTING') NOT NULL COMMENT 'The phase of the move operation.   ZAPI:  volume-move-get-iter volume-move-info phase',
	phaseRaw VARCHAR(255) DEFAULT NULL ,
	state ENUM('NOT_MAPPED', 'HEALTHY', 'DONE', 'FAILED', 'WARNING', 'ALERT') NOT NULL COMMENT 'The state of the volume move at the time of issuing the command and the system gathering up the information about the move.   ZAPI: volume-move-get-iter volume-move-info state',
	stateRaw VARCHAR(255) DEFAULT NULL ,
	details VARCHAR(255) DEFAULT NULL COMMENT 'This field contains details about any warnings, errors and state of the move operation.   ZAPI: volume-move-get-iter volume-move-info details',
	percentComplete INT DEFAULT NULL COMMENT 'The amount of work to move the volume completed thus far in terms of percentage.  ZAPI: volume-move-get-iter volume-move-info percent-complete',
	estimatedCompletionTime BIGINT(20) DEFAULT NULL COMMENT 'The approximate date and time in the cluster time zone when the entire volume move operation is expected to complete.  ZAPI: volume-move-get-iter volume-move-info estimated-completion-time',
	checksum INT NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	KEY updatetime_index (updatetime),
    KEY originId_index (originid),
	CONSTRAINT FOREIGN KEY FK_volumeId (volumeId) REFERENCES netapp_model.volume (objid),
	CONSTRAINT FOREIGN KEY FK_sourceAggregateId (sourceAggregateId) REFERENCES netapp_model.aggregate (objid),
	CONSTRAINT FOREIGN KEY FK_destinationAggregateId (destinationAggregateId) REFERENCES netapp_model.aggregate (objid),
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;