-- drop unneeded columns
ALTER TABLE netapp_model.snap_mirror DROP COLUMN destinationCluster;
ALTER TABLE netapp_model.snap_mirror DROP COLUMN destinationVserver;
ALTER TABLE netapp_model.snap_mirror DROP COLUMN destinationVolume;

-- add new fields
ALTER TABLE netapp_model.snap_mirror ADD COLUMN resourceKey VARCHAR(255) NOT NULL COMMENT 'A structured key that may be used to uniquely identify this resource globally' AFTER uuid;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN mirrorStateRaw VARCHAR(255) COMMENT 'Raw mirror state enum value' AFTER mirrorState;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN transferSnapshot VARCHAR(255) COMMENT 'The name of the current snapshot copy being transferred. This is set only when the relationship-status indicates that an activity is in progress' AFTER mirrorStateRaw;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN isConstituent BOOL COMMENT 'Constituent relationship' AFTER transferSnapshot;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN isHealthy BOOL COMMENT 'False if the last manual or scheduled update failed or was aborted, or if the last scheduled update was delayed. Otherwise true' AFTER isConstituent;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN maxTransferRate BIGINT(20) COMMENT 'Specifies the upper bound, in kilobytes per second, at which data is transferred between clusters' AFTER isHealthy;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN relationshipProgress BIGINT(20) COMMENT 'The total number of bytes that have been processed so far for the current activity of the relationship as returned in the relationship-status' AFTER maxTransferRate;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN relationshipStatus ENUM('IDLE', 'TRANSFERRING', 'CHECKING', 'QUIESCING', 'QUIESCED', 'QUEUED', 'PREPARING', 'FINALIZING', 'ABORTING', 'NOT_MAPPED') COMMENT 'Specifies the status of the SnapMirror relationship' AFTER relationshipProgress;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN relationshipStatusRaw VARCHAR(255) COMMENT 'Raw relationship status enum value' AFTER relationshipProgress;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN relationshipType ENUM('DATA_PROTECTION', 'LOAD_SHARING', 'VAULT', 'RESTORE', 'TRANSITION_DATA_PROTECTION', 'NOT_MAPPED') COMMENT 'Specifies the status of the SnapMirror relationship' AFTER relationshipStatusRaw;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN relationshipTypeRaw VARCHAR(255) COMMENT 'Raw relationship status enum value' AFTER relationshipType;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN tries INT COMMENT 'Specifies the maximum number of times to attempt each manual or scheduled transfer for a SnapMirror relationship. The default is eight times' AFTER relationshipTypeRaw;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN snapshotCheckpoint BIGINT(20) COMMENT 'The number of bytes transferred as recorded for the checkpoint of the current or most recent transfer snapshot' AFTER tries;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN snapshotProgress BIGINT(20) COMMENT 'The number of bytes transferred for the transfer snapshot. This is set only when the relationship-status indicates that an activity is in progress' AFTER snapshotCheckpoint;

-- recreate foreign key while we rename clusterId to destinationClusterId
ALTER TABLE netapp_model.snap_mirror DROP FOREIGN KEY snap_mirror_ibfk_1;
ALTER TABLE netapp_model.snap_mirror DROP KEY FK_clusterId;
ALTER TABLE netapp_model.snap_mirror CHANGE COLUMN clusterId destinationClusterId BIGINT(20) NOT NULL;
ALTER TABLE netapp_model.snap_mirror ADD CONSTRAINT FOREIGN KEY FK_destinationClusterId (destinationClusterId) REFERENCES netapp_model.cluster (objid);

-- required foreign keys
ALTER TABLE netapp_model.snap_mirror ADD COLUMN destinationVolumeId BIGINT(20) NULL AFTER destinationClusterId;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN destinationVserverId BIGINT(20) NULL AFTER destinationVolumeId;
ALTER TABLE netapp_model.snap_mirror ADD CONSTRAINT FOREIGN KEY FK_destinationVolumeId (destinationVolumeId) REFERENCES netapp_model.volume (objid);
ALTER TABLE netapp_model.snap_mirror ADD CONSTRAINT FOREIGN KEY FK_destinationVserverId (destinationVserverId) REFERENCES netapp_model.vserver (objid);

-- optional foreign keys (set if supplied by acquisition)
ALTER TABLE netapp_model.snap_mirror ADD COLUMN exportedSnapshotId BIGINT(20) COMMENT 'The exported snapshot copy on the destination volume' AFTER mirrorStateRaw;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN newestSnapshotId BIGINT(20) COMMENT 'The newest snapshot copy on the destination volume' AFTER exportedSnapshotId;
ALTER TABLE netapp_model.snap_mirror ADD CONSTRAINT FOREIGN KEY FK_exportedSnapshotId (exportedSnapshotId) REFERENCES netapp_model.snapshot (objid);
ALTER TABLE netapp_model.snap_mirror ADD CONSTRAINT FOREIGN KEY FK_newestSnapshotId (newestSnapshotId) REFERENCES netapp_model.snapshot (objid);

-- optional foreign keys (will be set by runner)
ALTER TABLE netapp_model.snap_mirror ADD COLUMN sourceClusterId BIGINT(20) NULL AFTER destinationLocation;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN sourceVolumeId BIGINT(20) NULL AFTER sourceClusterId;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN sourceVserverId BIGINT(20) NULL AFTER sourceVolumeId;
ALTER TABLE netapp_model.snap_mirror ADD CONSTRAINT FOREIGN KEY FK_sourceClusterId (sourceClusterId) REFERENCES netapp_model.cluster (objid);
ALTER TABLE netapp_model.snap_mirror ADD CONSTRAINT FOREIGN KEY FK_sourceVolumeId (sourceVolumeId) REFERENCES netapp_model.volume (objid);
ALTER TABLE netapp_model.snap_mirror ADD CONSTRAINT FOREIGN KEY FK_sourceVserverId (sourceVserverId) REFERENCES netapp_model.vserver (objid);
