-- This is a relationship table for the 'disk' table providing 'ownerNodeId' column for the 'disk' view.
CREATE TABLE netapp_model.disk_owner_node_relationship (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the Disk.',
    nodeId BIGINT(20) NULL COMMENT 'ID of the owner node of the disk.',
    checksum INT NOT NULL,
    changeDetailChecksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    PRIMARY KEY (objid),
    CONSTRAINT FOREIGN KEY FK_objid (objid) REFERENCES netapp_model.disk (objid),
    CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid)
    ) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

-- rename 'nodeId' column to 'homeNodeId'
ALTER TABLE netapp_model.disk DROP FOREIGN KEY `disk_ibfk_2`;
ALTER TABLE netapp_model.disk DROP KEY FK_nodeId;
ALTER TABLE netapp_model.disk CHANGE COLUMN nodeId homeNodeId BIGINT(20) NOT NULL;
ALTER TABLE netapp_model.disk ADD CONSTRAINT FOREIGN KEY FK_homeNodeId (homeNodeId) REFERENCES netapp_model.node (objid);

-- add additional columns to 'disk' tables.
ALTER TABLE netapp_model.disk ADD COLUMN resourceKey VARCHAR(255) NOT NULL COMMENT 'DERIVED. A structured key that may be used to uniquely identify this resource globally.' AFTER uuid;
ALTER TABLE netapp_model.disk ADD COLUMN uid VARCHAR(255) DEFAULT NULL COMMENT 'Disk unique identifier. Maximum length of 90 characters. ZAPIs:  storage-disk-get-iter storage-disk-info disk-uid' AFTER name;
ALTER TABLE netapp_model.disk ADD COLUMN isZeroed BOOL DEFAULT NULL COMMENT 'True if disk is in pre-zeroed state. ZAPIs: storage-disk-get-iter disk-aggregate-info is-zeroed' AFTER isVirtual;
ALTER TABLE netapp_model.disk ADD COLUMN isZeroing BOOL DEFAULT NULL COMMENT 'True only if disk position is ''pending'' and disk is in process of being zeroed.  ZAPIs:  storage-disk-get-iter disk-aggregate-info is-zeroing' AFTER isZeroed;
ALTER TABLE netapp_model.disk ADD COLUMN pool INT DEFAULT NULL COMMENT 'Pool to which disk is assigned.  ZAPIs: storage-disk-get-iter disk-ownership-info pool' AFTER serialNumber;
ALTER TABLE netapp_model.disk ADD COLUMN zeroingPercent INT DEFAULT NULL COMMENT 'Percent completion of disk zeroing.  ZAPIs: storage-disk-get-iter disk-aggregate-info zeroing-percent-complete' AFTER isZeroing;
ALTER TABLE netapp_model.disk ADD COLUMN checksumCompatibility VARCHAR(255) DEFAULT NULL COMMENT 'An indication of the checksum types that this disk is capable of supporting.  ZAPIs: storage-disk-get-iter disk-inventory-info checksum-compatability.' AFTER pool;
ALTER TABLE netapp_model.disk ADD COLUMN usedBytes BIGINT(20) DEFAULT NULL COMMENT 'DERIVED from usedBlocks * 4096 bytes/block.' AFTER usedBlocks;
ALTER TABLE netapp_model.disk ADD COLUMN totalBytes BIGINT(20) DEFAULT NULL COMMENT 'DERIVED from totalBlocks * 4096 bytes/block.' AFTER usedBytes;
ALTER TABLE netapp_model.disk ADD COLUMN containerTypeRaw VARCHAR(255) DEFAULT NULL AFTER containerType;
ALTER TABLE netapp_model.disk ADD COLUMN raidPositionRaw VARCHAR(255) DEFAULT NULL AFTER raidPosition;
ALTER TABLE netapp_model.disk ADD COLUMN interfaceTypeRaw VARCHAR(255) DEFAULT NULL AFTER interfaceType;
ALTER TABLE netapp_model.disk ADD COLUMN isMediaScrubbing BOOL DEFAULT NULL COMMENT 'True if media scrub is currently active for this disk.  ZAPIs: storage-disk-get-iter disk-aggregate-info is-media-scrubbing' AFTER zeroingPercent;
ALTER TABLE netapp_model.disk ADD COLUMN isOffline BOOL DEFAULT NULL COMMENT 'True if disk is offline.  ZAPIs: storage-disk-get-iter disk-aggregate-info is-offline' AFTER isMediaScrubbing;
ALTER TABLE netapp_model.disk ADD COLUMN isPrefailed BOOL DEFAULT NULL COMMENT 'True if the admin issued a ''disk fail'' or if the the system marked this disk for Rapid RAID Recovery.   ZAPIs: storage-disk-get-iter disk-aggregate-info is-prefailed' AFTER isOffline;
ALTER TABLE netapp_model.disk ADD COLUMN isReconstructing BOOL DEFAULT NULL COMMENT 'True if disk is in process of being reconstructed.   ZAPIs: storage-disk-get-iter disk-aggregate-info is-reconstructing' AFTER isPrefailed;
ALTER TABLE netapp_model.disk ADD COLUMN isReplacing BOOL DEFAULT NULL COMMENT 'True if the admin issued ''disk replace'' to replace this disk with a specified replacement disk.   ZAPIs: storage-disk-get-iter disk-aggregate-info is-replacing' AFTER isReconstructing;

-- add 'resourceKey' column to 'plex' and 'raid_group' tables
ALTER TABLE netapp_model.raid_group ADD COLUMN resourceKey VARCHAR(255) NOT NULL COMMENT 'DERIVED. A structured key that may be used to uniquely identify this resource globally.' AFTER uuid;
ALTER TABLE netapp_model.plex ADD COLUMN resourceKey VARCHAR(255) NOT NULL COMMENT 'DERIVED. A structured key that may be used to uniquely identify this resource globally.' AFTER uuid;

