CREATE TABLE netapp_model.job_schedule (
	objid BIGINT(20) NOT NULL,
	originid BIGINT(20) NOT NULL,
	uuid VARCHAR(255) NOT NULL COMMENT 'The uuid of the job schedule.',
	resourceKey VARCHAR(255) NOT NULL COMMENT 'DERIVED.  A structured key that may be used to uniquely identify this resource globally.',
    name VARCHAR(255) NOT NULL COMMENT 'The name of the job schedule.   ZAPIs:  job-schedule-iter job-schedule-info job-schedule-name',
    clusterId BIGINT(20) NOT NULL,
    description VARCHAR(255) DEFAULT NULL COMMENT 'The description of the job schedule.  ZAPIs:  job-schedule-iter job-schedule-info job-schedule-description',
    scheduleType ENUM('CRON', 'INTERVAL', 'BUILTIN', 'UNKNOWN', 'NOT_MAPPED') DEFAULT NULL COMMENT ' The type of job schedule.  ZAPIs: job-schedule-iter job-schedule-info job-schedule-type.',
    scheduleTypeRaw VARCHAR(255) DEFAULT NULL,
    cronDayOfMonth VARCHAR(255) DEFAULT NULL COMMENT 'The day(s) of the month when the job should be run.  ZAPIs: job-schedule-cron-get-iter job-schedule-cron-info job-schedule-cron-day.',
    cronDayOfWeek VARCHAR(255) DEFAULT NULL COMMENT 'The day(s) in the week when the job should be run.   ZAPIs:  job-schedule-cron-get-iter job-schedule-cron-info job-schedule-cron-day-of-week',
    cronHour VARCHAR(255) DEFAULT NULL COMMENT 'The hour(s) of the day when the job should be run.  ZAPIs: job-schedule-cron-get-iter job-schedule-cron-info job-schedule-cron-hour',
    cronMinute VARCHAR(255) DEFAULT NULL COMMENT 'The minute(s) of each hour when the job should be run.  ZAPIs:  job-schedule-cron-get-iter job-schedule-cron-info job-schedule-cron-minute',
    cronMonth VARCHAR(255) DEFAULT NULL COMMENT 'The month(s) when the job should be run.   ZAPIs: job-schedule-cron-get-iter job-schedule-cron-info job-schedule-cron-month.',
    intervalDays INT DEFAULT NULL COMMENT 'The number of days between jobs.  ZAPIs:  job-schedule-internval-get-iter job-schedule-interval-info job-schedule-interval-days',
    intervalHours INT DEFAULT NULL COMMENT 'The number of hours between jobs.   ZAPIs: job-schedule-internval-get-iter job-schedule-interval-info job-schedule-interval-hours',
    intervalMinutes INT DEFAULT NULL COMMENT 'The number of minutes between jobs   ZAPIs:  job-schedule-internval-get-iter job-schedule-interval-info job-schedule-interval-minutes',
    intervalSeconds INT DEFAULT NULL COMMENT 'The number of seconds between jobs.   ZAPIs: job-schedule-internval-get-iter job-schedule-interval-info job-schedule-interval-seconds',
	checksum INT NOT NULL,
	changeDetailChecksum INT NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	KEY updatetime_index (updatetime),
    KEY originId_index (originid),	
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

CREATE TABLE netapp_model.sis_policy (
	objid BIGINT(20) NOT NULL,
	originid BIGINT(20) NOT NULL,
	uuid VARCHAR(255) NOT NULL COMMENT 'The uuid of the sis policy.  DERIVED from {VserverUUID}.{policy name}',
	resourceKey VARCHAR(255) NOT NULL COMMENT 'DERIVED.  A structured key that may be used to uniquely identify this resource globally.',
    name VARCHAR(255) NOT NULL COMMENT 'The name of the sis policy.',
    vserverId BIGINT(20) NOT NULL,
    clusterId BIGINT(20) NOT NULL,
    jobScheduleId BIGINT(20) DEFAULT NULL,
    isEnabled BOOL DEFAULT NULL COMMENT 'If the value is true, the sis policy is active and sis operations will be triggered on the volume on the specified schedule',
    comment VARCHAR(255) DEFAULT NULL COMMENT 'A brief description of the policy.',
    duration VARCHAR(255) DEFAULT NULL COMMENT 'The duration in hours for which the scheduled sis operation should run',
    qosPolicy ENUM('BACKGROUND', 'BEST_EFFORT', 'NOT_MAPPED') DEFAULT NULL COMMENT 'QoS policy for the sis operation.',
    qosPolicyRaw VARCHAR(255) DEFAULT NULL,
    checksum INT NOT NULL,
	changeDetailChecksum INT NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	KEY updatetime_index (updatetime),
    KEY originId_index (originid),
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
	CONSTRAINT FOREIGN KEY FK_jobScheduleId (jobScheduleId) REFERENCES netapp_model.job_schedule (objid),
	PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

CREATE TABLE netapp_model.snapshot_policy (
	objid BIGINT(20) NOT NULL,
	originid BIGINT(20) NOT NULL,
	uuid VARCHAR(255) NOT NULL COMMENT 'The uuid of the snapshot policy. DERIVED from {VserverUUID}.{policy name}.   If cluster scoped, this will just be the policy name.',
	resourceKey VARCHAR(255) NOT NULL COMMENT 'DERIVED.  A structured key that may be used to uniquely identify this resource globally.',
    name VARCHAR(255) NOT NULL COMMENT 'A human readable string describing the name of the snapshot scheduling policy.',
    clusterId BIGINT(20) NOT NULL,
    isEnabled BOOL DEFAULT NULL COMMENT 'The state of the snapshot policy. If true, the snapshot policy is enabled and scheduled snapshots will be created on the volume associated with this policy.',
    comment VARCHAR(255) DEFAULT NULL COMMENT 'A human readable description associated with the snasphot policy.',    
    checksum INT NOT NULL,
	changeDetailChecksum INT NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	KEY updatetime_index (updatetime),
    KEY originId_index (originid),
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

CREATE TABLE netapp_model.snapshot_policy_schedule (
	objid BIGINT(20) NOT NULL,
	originid BIGINT(20) NOT NULL,
	uuid VARCHAR(255) NOT NULL COMMENT 'The uuid of the snapshot policy schedule.  DERIVED from {VserverUUID}.{snapshotPolicy.name}.{schedule.prefix}',
    resourceKey VARCHAR(255) NOT NULL COMMENT 'DERIVED.  A structured key that may be used to uniquely identify this resource globally.',
    snapshotPolicyId BIGINT(20) NOT NULL,
    clusterId BIGINT(20) NOT NULL,
    jobScheduleId BIGINT(20) DEFAULT NULL,
    prefix VARCHAR(255) DEFAULT NULL COMMENT 'Snapshot name prefix for the schedule present in the snapshot policy.',
    count BIGINT(20) DEFAULT NULL COMMENT 'Maximum number of snapshots to retain for the schedule present in the snapshot policy.',
    checksum INT NOT NULL,
	changeDetailChecksum INT NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	KEY updatetime_index (updatetime),
    KEY originId_index (originid),
	CONSTRAINT FOREIGN KEY FK_snapshotPolicyId (snapshotPolicyId) REFERENCES netapp_model.snapshot_policy (objid),
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	CONSTRAINT FOREIGN KEY FK_jobScheduleId (jobScheduleId) REFERENCES netapp_model.job_schedule (objid),
	PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

ALTER TABLE netapp_model.volume ADD COLUMN snapshotPolicyId BIGINT(20) DEFAULT NULL AFTER flexCacheOriginId;
ALTER TABLE netapp_model.volume ADD CONSTRAINT FOREIGN KEY FK_snapshotPolicyId (snapshotPolicyId) REFERENCES netapp_model.snapshot_policy (objid);

ALTER TABLE netapp_model.volume ADD COLUMN sisPolicyId BIGINT(20) DEFAULT NULL AFTER snapshotPolicyId;
ALTER TABLE netapp_model.volume ADD CONSTRAINT FOREIGN KEY FK_sisPolicyId (sisPolicyId) REFERENCES netapp_model.sis_policy (objid);

ALTER TABLE netapp_model.vserver ADD COLUMN snapshotPolicyId BIGINT(20) DEFAULT NULL AFTER rootVolumeId;
ALTER TABLE netapp_model.vserver ADD CONSTRAINT FOREIGN KEY FK_snapshotPolicyId (snapshotPolicyId) REFERENCES netapp_model.snapshot_policy (objid);

ALTER TABLE netapp_model.snap_mirror DROP COLUMN managingVserver;