-- Copyright (c) 2012 NetApp. All rights reserved

CREATE TABLE netapp_model.qos_volume_workload (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs: qos-workload-get-iter.qos-workload-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the QoS Workload',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name', 
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    volumeId BIGINT(20) NOT NULL COMMENT 'ID of the Volume the workload is associated with',
    workloadClass ENUM('NOT_MAPPED', 'PRESET', 'USER_DEFINED', 'SYSTEM_DEFINED') NOT NULL COMMENT 'Identifies how the workload was created', 
    checksum INT NOT NULL,
    changeDetailChecksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_volumeId (volumeId) REFERENCES netapp_model.volume (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;
    
CREATE TABLE netapp_model.qos_service_center (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier. ZAPIs: perf-object-instance-list-info-iter (with object-name "resource")',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the QoS Service Center',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name', 
    clusterId BIGINT(20) NOT NULL,
    nodeId BIGINT(20) NOT NULL,
    aggregateId BIGINT(20) NULL COMMENT 'Optional ID of the Aggregate the service center is associated with',
    parentResourceId BIGINT(20) NULL COMMENT 'ID of the parent service center. If the value is null then there is no parent',
    type ENUM('NOT_MAPPED', 'RESOURCE', 'DELAY_CENTER') NOT NULL COMMENT 'Identifies whether the service center is a resource or a delay center',
    checksum INT NOT NULL,
    changeDetailChecksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid),
    CONSTRAINT FOREIGN KEY FK_aggregateId (aggregateId) REFERENCES netapp_model.aggregate (objid),
    CONSTRAINT FOREIGN KEY FK_parentResourceId (parentResourceId) REFERENCES netapp_model.qos_service_center (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;
  
CREATE TABLE netapp_model.qos_workload_detail (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier. ZAPIs: perf-object-instance-list-info-iter (with object-name "workload_detail")',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the QoS Workload Detail',
    name VARCHAR(255) NOT NULL COMMENT 'Textual name', 
    clusterId BIGINT(20) NOT NULL,
    workloadId BIGINT(20) NOT NULL COMMENT 'ID of the QoS Workload item associated with this QoS Workload Detail',
    serviceCenterId BIGINT(20) NOT NULL COMMENT 'ID of the QoS Service Center item associated with this QoS Workload Detail',
    checksum INT NOT NULL,
    changeDetailChecksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_workloadId (workloadId) REFERENCES netapp_model.qos_volume_workload (objid),
    CONSTRAINT FOREIGN KEY FK_serviceCenterId (serviceCenterId) REFERENCES netapp_model.qos_service_center (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;
    
CREATE TABLE netapp_performance.sample_current_qos_volume_workload (
    objid BIGINT(20) COMMENT 'Reference ID of qos volume workload',
    time BIGINT(20),
    empty BOOL,
    containerkey VARCHAR(255),
    containerid BIGINT(20) COMMENT 'Vserver container reference ID',
    latency FLOAT COMMENT 'Average latency in microseconds for the WAFL filesystem to process all the operations on the volume- not including request processing or network communication (microseconds)',
    ops FLOAT COMMENT 'Number of operations per second serviced by the volume (ops/s)',
    totalData FLOAT COMMENT 'total of bytes read/written per second to the volume (bytes/s)',
    readData FLOAT COMMENT 'Bytes read per second from the volume (bytes/s)',
    readLatency FLOAT COMMENT 'Average latency in microseconds for the WAFL filesystem to process read request to the volume- not including request processing or network communication (microseconds)',
    readOps FLOAT COMMENT 'Number of reads per second to the volume (ops/s)',
    writeData FLOAT COMMENT 'Bytes written per second to the volume (bytes/s)',
    writeLatency FLOAT COMMENT 'Average latency in microseconds for the WAFL filesystem to process write request to the volume- not including request processing or network communication (microseconds)',
    writeOps FLOAT COMMENT 'Number of writes per second to the volume (ops/s)',
    KEY containerkey_index (containerkey),
    PRIMARY KEY (objid, containerid)
    ) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
    PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));
    
CREATE TABLE netapp_performance.sample_current_qos_workload_detail (
    objid BIGINT(20) COMMENT 'Reference ID of qos volume workload detail',
    time BIGINT(20),
    empty BOOL,
    containerkey VARCHAR(255),
    containerid BIGINT(20) COMMENT 'Vserver container reference ID',
    serviceTime FLOAT COMMENT 'for each I/O of the workload class what is the average number of visits to the queueing center to satisfy the I/O',
    waitTime FLOAT COMMENT 'the average time between a message/disk IO being sent to the queueing center and the start if its being serviced',
    visits FLOAT COMMENT 'average number of visits to the queueing center to satisfy the I/O',
    inLatencyPath BOOL COMMENT 'Represents if latency is in latency path of user defined workload.',
    KEY containerkey_index (containerkey),
    PRIMARY KEY (objid, containerid)
    ) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
    PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));
