-- Copyright (c) 2012 NetApp. All rights reserved

CREATE TABLE netapp_model.snap_mirror_policy (
	objid BIGINT(20) NOT NULL,
	originid BIGINT(20) NOT NULL,
	uuid VARCHAR(255) NOT NULL COMMENT 'The uuid of the snapmirror policy. DERIVED from {VserverUUID}.{policy name}.   If cluster scoped, this will just be the policy name.',
	resourceKey VARCHAR(1024) NOT NULL COMMENT 'DERIVED.  A structured key that may be used to uniquely identify this resource globally.',
    name VARCHAR(255) DEFAULT NULL COMMENT ' The name of the policy.',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) DEFAULT NULL,
    comment VARCHAR(255) DEFAULT NULL COMMENT 'A human readable description associated with the snapmirror policy.',
    totalKeep BIGINT(20) DEFAULT NULL COMMENT 'Total retention count for all rules in the policy.',
    transferPriority ENUM('LOW', 'NORMAL', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Specifies the priority at which a SnapMirror transfer runs.',
    transferPriorityRaw VARCHAR(255) DEFAULT NULL,
    checksum INT NOT NULL,
	changeDetailChecksum INT NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	KEY updatetime_index (updatetime),
    KEY originId_index (originid),
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
	PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

CREATE TABLE netapp_model.snap_mirror_policy_rule (
	objid BIGINT(20) NOT NULL,
	originid BIGINT(20) NOT NULL,
	uuid VARCHAR(255) NOT NULL COMMENT 'The uuid of the snapmirror policy rule. DERIVED from {SnapmirrorPolicyUUID}.{ordinalIndex}.',
    clusterId BIGINT(20) NOT NULL,
    snapMirrorPolicyId BIGINT(20) NOT NULL,
    keep VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the snapmirror copy retention count.',
    preserve BOOL DEFAULT NULL COMMENT 'Specifies whether snapmirror copy preserve is enabled',
    snapMirrorLabel VARCHAR(255) DEFAULT NULL COMMENT 'Specifies the snapmirror copy label.',
    warn BIGINT(20) DEFAULT NULL COMMENT 'Specifies the warning threshold count.',
    checksum INT NOT NULL,
	changeDetailChecksum INT NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	KEY updatetime_index (updatetime),
    KEY originId_index (originid),
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	CONSTRAINT FOREIGN KEY FK_snapMirrorPolicyId (snapMirrorPolicyId) REFERENCES netapp_model.snap_mirror_policy (objid),
	PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;