
CREATE TABLE netapp_model.storage_service (
	objid BIGINT(20) NOT NULL,
	originid BIGINT(20) NOT NULL,
	UUID VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the Storage Service. DERIVED from  {volumeUUID}.{storageServiceIdentifier}.',
	resourceKey VARCHAR(512) NOT NULL COMMENT 'DERIVED.  A structured key that may be used to uniquely identify this resource globally.',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    storageServiceIdentifier BIGINT(20) NOT NULL COMMENT 'Storage Service ID. ZAPIs: volume-storage-service-get-iter.storage-service-info.storage-service-id',
    storageServiceName VARCHAR(255) DEFAULT NULL COMMENT 'Storage Service Name. ZAPIs: volume-storage-service-get-iter.storage-service-info.storage-service',
    sizeUsed BIGINT(20) DEFAULT NULL COMMENT 'Derived.  Sum of sizeUsed over all constituent volumes assigned to this service. In bytes.',
    sizeTotal BIGINT(20) DEFAULT NULL COMMENT 'Derived.  Sum of sizeTotal over all constituent volumes assigned to this service. In bytes.',
    sizeUsedBySnapshots BIGINT(20) DEFAULT NULL COMMENT 'Derived.  Sum of sizeUsedBySnapshots over all constituent volumes assigned to this service. In bytes.',
    snapshotReserveSize BIGINT(20) DEFAULT NULL COMMENT 'Derived.  Sum of snapshotReserveSize over all constituent volumes assigned to this service. In bytes.',
    deduplicationSpaceSaved BIGINT(20) DEFAULT NULL COMMENT 'Derived.  Sum of deduplicationSpaceSaved over all constituent volumes assigned to this service. In bytes.',
    compressionSpaceSaved BIGINT(20) DEFAULT NULL COMMENT 'Derived.  Sum of compressionSpaceSaved over all constituent volumes assigned to this service. In bytes.',
    checksum INT NOT NULL,
	changeDetailChecksum INT NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	KEY updatetime_index (updatetime),
    KEY originId_index (originid),
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
	PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

ALTER TABLE netapp_model.volume ADD COLUMN storageServiceId BIGINT(20) DEFAULT NULL AFTER sisPolicyId;
ALTER TABLE netapp_model.volume ADD CONSTRAINT FOREIGN KEY FK_storageServiceId (storageServiceId) REFERENCES netapp_model.storage_service (objid);

CREATE TABLE netapp_model.storage_service_volume_relationship (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the relationship. Not from ZAPI',
    storageServiceId BIGINT(20) NULL COMMENT 'ID of the storage service',
    volumeId BIGINT(20) NULL COMMENT 'ID of the volume the storage service references',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    changeDetailChecksum INT NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    PRIMARY KEY (objid),
    CONSTRAINT FOREIGN KEY FK_objid (objid) REFERENCES netapp_model.storage_service (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;