CREATE TABLE netapp_model.flash_device (
    objid BIGINT(20) NOT NULL,
    originid BIGINT(20) NOT NULL,
    UUID VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of FlashDevice',
    resourceKey VARCHAR(512) NOT NULL COMMENT 'DERIVED.  A structured key that may be used to uniquely identify this resource globally.',
    clusterId BIGINT(20) NOT NULL,
    nodeId BIGINT(20) NOT NULL,
    slotNumber INT DEFAULT NULL COMMENT 'The name of routing group.',
    serialNumber VARCHAR(255) DEFAULT NULL COMMENT '10-digit serial number of the flash device ',
    modelName VARCHAR(255) DEFAULT NULL COMMENT 'Model name of the flash device in XXXXXX-XX format.',
    firmwareRevision VARCHAR(255) DEFAULT NULL COMMENT 'Firmware revision of FPGA on the flash device. ',
    capacityGB INT DEFAULT NULL COMMENT 'Advertised capacity of the device, in gigabytes.',
    percentOnline INT DEFAULT NULL COMMENT 'Percentage of device capacity that is currently online.',
    status ENUM('NOT_MAPPED', 'ONLINE', 'OFFLINE_FAILED', 'OFFLINE_THRESHOLD') DEFAULT NULL COMMENT 'The current status of the device',
    statusRaw VARCHAR(255) DEFAULT NULL COMMENT 'The current status of the device - ZAPI raw value',
    serviceTime INT DEFAULT NULL COMMENT 'Total number of hours the device was operational.',
    thresholdProfile VARCHAR(255) DEFAULT NULL COMMENT 'The name of the threshold profile being used for this flash device.',
    checksum INT NOT NULL,
    changeDetailChecksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid),
    PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;


ALTER TABLE netapp_model.lun ADD COLUMN lunClass ENUM('NOT_MAPPED', 'REGULAR', 'PROTCOL_ENDPOINT', 'VVOL') DEFAULT NULL COMMENT 'The class of the LUN' AFTER suffixSize;
ALTER TABLE netapp_model.lun ADD COLUMN lunClassRaw VARCHAR(255) DEFAULT NULL COMMENT 'The class of the LUN - ZAPI raw value' AFTER lunClass;

ALTER TABLE netapp_model.snapshot_policy_schedule ADD COLUMN snapMirrorLabel VARCHAR(255) DEFAULT NULL COMMENT 'Snapshot SnapMirror Label for the schedule present in the snapshot policy.' AFTER prefix;