CREATE TABLE netapp_model.cifs_share_acl (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the CIFS Share Access Control List. Equal to {VserverUUID}.{shareName}.{userOrGroup}  Not returned from ZAPI',
    resourceKey VARCHAR(512) NOT NULL COMMENT 'DERIVED. A structured key that may be used to uniquely identify this resource globally',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    cifsShareId BIGINT(20) NOT NULL,
    userOrGroup VARCHAR(255) DEFAULT NULL COMMENT 'The user or group name for which the permissions are listed',
    permission ENUM('NOT_MAPPED','NO_ACCESS','READ','CHANGE','FULL_CONTROL') DEFAULT NULL COMMENT 'The access rights that the user or group has on the defined CIFS share',
    permissionRaw VARCHAR(255) DEFAULT NULL COMMENT 'The access rights that the user or group has on the defined CIFS share - ZAPI raw value',
    checksum INT NOT NULL,
    changeDetailChecksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_cifsShareId (cifsShareId) REFERENCES netapp_model.cifs_share (objid),
    PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

