-- Copyright (c) 2013 NetApp. All rights reserved

CREATE TABLE netapp_model.storage_shelf_cable (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of ApplicationRecord. Not from ZAPI',
    resourceKey VARCHAR(512) NOT NULL COMMENT 'DERIVED. A structured key that may be used to uniquely identify this resource globally',
    clusterId BIGINT(20) NOT NULL,
    nodeId BIGINT(20) NOT NULL,
    shelfId BIGINT(20) NOT NULL,
    channelName VARCHAR(255) NOT NULL COMMENT 'Storage controller channel the shelf is connected to.',
    isShelfChannelFailure  BOOL DEFAULT NULL COMMENT 'Indicates whether any shelves on this channel have experienced a failure.',
    endIdentifier VARCHAR(255) DEFAULT NULL COMMENT 'Each cable has two ends. This field shows which end of the cable is connected to the shelf.',
    length VARCHAR(255) DEFAULT NULL COMMENT 'Cable length',
    manufacturer VARCHAR(255) DEFAULT NULL COMMENT 'Manufacturer of the cable.',
    partNumber VARCHAR(255) DEFAULT NULL COMMENT 'Part number of the cable as assigned by the cable manufacturer.',
    serialNumber VARCHAR(255) DEFAULT NULL COMMENT 'Cable identifier. For backwards compatibility this is reported as cable-serial-no.',
    technology VARCHAR(255) DEFAULT NULL COMMENT 'Cable Technology.',
    type VARCHAR(255) DEFAULT NULL COMMENT 'Type of the cable being used',
    connectorDesignator VARCHAR(255) DEFAULT NULL COMMENT 'Connector Designator',
    connectorNumber INT DEFAULT NULL COMMENT 'Connector Number',
    isCableConnected BOOL DEFAULT NULL COMMENT 'Indicates whether a cable is connected at this connector location. No further information will be provided if a cable is not connected at this connector location. Note that a cable connection does not necessarily mean that the shelf is connected to a storage controller. This will also depend on the other end of the cable.',
    isConnectorError BOOL DEFAULT NULL COMMENT 'Indicates whether the connector element has encountered an error.',
    swapCount INT DEFAULT NULL COMMENT 'Number of times, since last boot, a cable has been inserted into this connector',
    shelfModule VARCHAR(255) DEFAULT NULL COMMENT 'Shelf module for this connector.',
    checksum INT NOT NULL,
    changeDetailChecksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    objState ENUM ('LIVE', 'DEAD') DEFAULT 'LIVE' NOT NULL COMMENT 'Internal object state',
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    KEY objState_index (objState,objid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid),
    CONSTRAINT FOREIGN KEY FK_shelfId (shelfId) REFERENCES netapp_model.storage_shelf (objid),
    PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;


ALTER TABLE netapp_model.storage_adapter ADD COLUMN state ENUM('NOT_MAPPED', 'UP', 'DOWN', 'OFFLINE_SOFT', 'OFFLINE_HARD', 'OFFLINE_LOOPBACK')DEFAULT NULL COMMENT 'Current state of the adapter' after fcPortName;
ALTER TABLE netapp_model.storage_adapter ADD COLUMN stateRaw  VARCHAR(255) DEFAULT NULL COMMENT 'Raw value of Current state of the adapter' after state;

ALTER TABLE netapp_model.storage_adapter ADD COLUMN cableEndIdentifier  VARCHAR(255) DEFAULT NULL COMMENT 'Each cable has two ends. This field shows which end of the cable is connected to the shelf.' after stateRaw;
ALTER TABLE netapp_model.storage_adapter ADD COLUMN cableLength  VARCHAR(255) DEFAULT NULL COMMENT 'Cable Length.' after cableEndIdentifier;
ALTER TABLE netapp_model.storage_adapter ADD COLUMN cableManufacturer  VARCHAR(255) DEFAULT NULL COMMENT 'Manufacturer of the cable.' after cableLength;
ALTER TABLE netapp_model.storage_adapter ADD COLUMN cablePartNumber  VARCHAR(255) DEFAULT NULL COMMENT 'Part number of the cable as assigned by the cable manufacturer.' after cableManufacturer;
ALTER TABLE netapp_model.storage_adapter ADD COLUMN cableSerialNumber  VARCHAR(255) DEFAULT NULL COMMENT 'Cable identifier. For backwards compatibility this is reported as cable-serial-number.' after cablePartNumber;
ALTER TABLE netapp_model.storage_adapter ADD COLUMN cableTechnology  VARCHAR(255) DEFAULT NULL COMMENT 'Cable Technology.' after cableSerialNumber;

