-- Copyright (c) 2013 NetApp. All rights reserved

CREATE TABLE netapp_model.lif (
  objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPIs:  net-interface-get-iter.net-interface-info',
  originid BIGINT(20) NOT NULL,
  uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of this LIF. Not available in ZAPI. Uses {VserverUUID}.{LIFName}',
  name VARCHAR(255) NOT NULL COMMENT 'Textual name',
  clusterId BIGINT(20) NOT NULL,
  vserverId BIGINT(20) NOT NULL,
  currentNodeId BIGINT(20) DEFAULT NULL,
  homeNodeId BIGINT(20) DEFAULT NULL,
  lifType ENUM('NETWORK_LIF', 'FCP_LIF') NOT NULL COMMENT 'Lif type',
  isAutoRevert TINYINT(1) DEFAULT NULL COMMENT 'If true, data LIF will revert to its home node under certain circumstances such as startup, and load balancing migration is disabled automatically',
  isHome TINYINT(1) DEFAULT NULL COMMENT 'True if LIF is currently on home node and home port',
  operationalStatus ENUM('UP','DOWN','UNKNOWN','NOT_MAPPED') COLLATE utf8_bin DEFAULT NULL COMMENT 'State of aggregate',
  operationalStatusRaw VARCHAR(255) COLLATE utf8_bin DEFAULT NULL,
  administrativeStatus ENUM('UP','DOWN','UNKNOWN','NOT_MAPPED') COLLATE utf8_bin DEFAULT NULL COMMENT 'State of aggregate',
  administrativeStatusRaw VARCHAR(255) COLLATE utf8_bin DEFAULT NULL,
  role ENUM('UNDEF','CLUSTER','DATA','NODE_MGMT','INTERCLUSTER','NOT_MAPPED','CLUSTER_MGMT') COLLATE utf8_bin DEFAULT NULL COMMENT 'Specifies the role associated with the LIF',
  roleRaw VARCHAR(255) COLLATE utf8_bin DEFAULT NULL,
  dataProtocols VARCHAR(255) COLLATE utf8_bin DEFAULT NULL COMMENT 'Specifies the list of data protocols configured on the LIF. This can be comma separated. Possible values: nfs, cifs, iscsi, fcp, fcache, none',
  failoverGroup VARCHAR(255) COLLATE utf8_bin DEFAULT NULL COMMENT 'Specifies the failover group name',
  objState ENUM ('LIVE', 'DEAD') DEFAULT 'LIVE' NOT NULL COMMENT 'Internal object state',
  updatetime BIGINT(20) NOT NULL,
  PRIMARY KEY (objid),
  KEY updatetime_index (updatetime),
  KEY originId_index (originid),
  KEY objState_index (objState,objid),
  CONSTRAINT FK_clusterId FOREIGN KEY (clusterId) REFERENCES netapp_model.cluster (objid),
  CONSTRAINT FK_vserverId FOREIGN KEY (vserverId) REFERENCES netapp_model.vserver (objid),
  CONSTRAINT FK_currentNodeId FOREIGN KEY (currentNodeId) REFERENCES netapp_model.node (objid),
  CONSTRAINT FK_homeNodeId FOREIGN KEY (homeNodeId) REFERENCES netapp_model.node (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;


INSERT INTO netapp_model.lif (
objId, originid, UUID, name, clusterId, vserverId, currentNodeId, homeNodeId, lifType, isAutoRevert, isHome, operationalStatus, operationalStatusRaw,
administrativeStatus, administrativeStatusRaw, role, roleRaw, dataProtocols, failoverGroup, objState, updateTime)
SELECT objId, originid, UUID, name, clusterId, vserverId, currentNodeId, homeNodeId, 'NETWORK_LIF', isAutoRevert, isHome, operationalStatus, operationalStatusRaw,
administrativeStatus, administrativeStatusRaw, role, roleRaw, dataProtocols, failoverGroup, objState, updateTime FROM netapp_model.network_lif;


INSERT INTO netapp_model.lif (
objId, originid, UUID, NAME, clusterId, vserverId, currentNodeId, homeNodeId, lifType, isAutoRevert, isHome, operationalStatus, operationalStatusRaw,
administrativeStatus, administrativeStatusRaw, role, roleRaw, dataProtocols, failoverGroup, objState, updateTime)
SELECT objId, originid, UUID, NAME, clusterId, vserverId, currentNodeId, homeNodeId, 'FCP_LIF', isAutoRevert, isHome, operationalStatus, operationalStatusRaw,
administrativeStatus, administrativeStatusRaw, role, roleRaw, dataProtocols, failoverGroup, objState, updateTime FROM netapp_model.fcp_lif;

ALTER TABLE netapp_model.network_lif DROP COLUMN  NAME;
ALTER TABLE netapp_model.network_lif DROP FOREIGN KEY network_lif_ibfk_5;
ALTER TABLE netapp_model.network_lif DROP FOREIGN KEY network_lif_ibfk_3;
ALTER TABLE netapp_model.network_lif DROP COLUMN  currentNodeId;
ALTER TABLE netapp_model.network_lif DROP COLUMN  homeNodeId;
ALTER TABLE netapp_model.network_lif DROP COLUMN  isAutoRevert;
ALTER TABLE netapp_model.network_lif DROP COLUMN  isHome;
ALTER TABLE netapp_model.network_lif DROP COLUMN  operationalStatus;
ALTER TABLE netapp_model.network_lif DROP COLUMN  operationalStatusRaw;
ALTER TABLE netapp_model.network_lif DROP COLUMN  administrativeStatus;
ALTER TABLE netapp_model.network_lif DROP COLUMN  administrativeStatusRaw;
ALTER TABLE netapp_model.network_lif DROP COLUMN  role;
ALTER TABLE netapp_model.network_lif DROP COLUMN  roleRaw;
ALTER TABLE netapp_model.network_lif DROP COLUMN  dataProtocols;
ALTER TABLE netapp_model.network_lif DROP COLUMN  failoverGroup;

ALTER TABLE netapp_model.fcp_lif DROP COLUMN  NAME;
ALTER TABLE netapp_model.fcp_lif DROP FOREIGN KEY  fcp_lif_ibfk_5;
ALTER TABLE netapp_model.fcp_lif DROP FOREIGN KEY  fcp_lif_ibfk_3;
ALTER TABLE netapp_model.fcp_lif DROP COLUMN  currentNodeId;
ALTER TABLE netapp_model.fcp_lif DROP COLUMN  homeNodeId;
ALTER TABLE netapp_model.fcp_lif DROP COLUMN  isAutoRevert;
ALTER TABLE netapp_model.fcp_lif DROP COLUMN  isHome;
ALTER TABLE netapp_model.fcp_lif DROP COLUMN  operationalStatus;
ALTER TABLE netapp_model.fcp_lif DROP COLUMN  operationalStatusRaw;
ALTER TABLE netapp_model.fcp_lif DROP COLUMN  administrativeStatus;
ALTER TABLE netapp_model.fcp_lif DROP COLUMN  administrativeStatusRaw;
ALTER TABLE netapp_model.fcp_lif DROP COLUMN  role;
ALTER TABLE netapp_model.fcp_lif DROP COLUMN  roleRaw;
ALTER TABLE netapp_model.fcp_lif DROP COLUMN  dataProtocols;
ALTER TABLE netapp_model.fcp_lif DROP COLUMN  failoverGroup;
