-- Copyright (c) 2013 NetApp. All rights reserved

CREATE TABLE netapp_model.qos_workload_constituent (
  objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.',
  originid BIGINT(20) NOT NULL,
  uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the QoS Workload Constituent Derived: node;parent workload uuid',
  resourceKey VARCHAR(512)  NOT NULL,
  clusterId BIGINT(20) NOT NULL,
  nodeId BIGINT(20) NOT NULL,
  workloadId BIGINT(20) NOT NULL COMMENT 'ID of the QoS Workload associated with this QoS Workload Constituent',
  checksum INT(11) NOT NULL,
  changeDetailChecksum INT(11) NOT NULL,
  createtime BIGINT(20) NOT NULL,
  updatetime BIGINT(20) NOT NULL,
  objState ENUM('LIVE','DEAD'),
  KEY FK_clusterId (clusterId),
  KEY FK_workloadId (workloadId),
  KEY FK_nodeId (nodeId),
  KEY updatetime_index (updatetime),
  KEY originId_index (originid),
  KEY objState_index (objState, objid),
  CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
  CONSTRAINT FOREIGN KEY FK_workloadId (workloadId) REFERENCES netapp_model.qos_volume_workload (objid),
  CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node(objid),
  PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

CREATE TABLE netapp_performance.sample_current_qos_workload_constituent (
  objid bigint(20) NOT NULL DEFAULT '0' COMMENT 'Qos workload constituent Id',
  time bigint(20) DEFAULT NULL,
  empty tinyint(1) DEFAULT NULL,
  containerkey varchar(255) COLLATE utf8_bin DEFAULT NULL,
  containerid bigint(20) NOT NULL DEFAULT '0' COMMENT 'Cluster ID',
  dirtyBuffers float DEFAULT NULL COMMENT 'The number of dirty buffers that were accumulated per second by the workload constituent',
  freedBlocks float DEFAULT NULL COMMENT 'The number of free blocks that were accumulated per second by the workload constituent',
  readDone float DEFAULT NULL COMMENT 'The number of RAID IO read done messages that were accumulated per second by the workload constituent',
  readAheadXStripe float DEFAULT NULL COMMENT 'The number of cross-stripe read ahead messages that were issued per second on behalf of the workload constituent',
  PRIMARY KEY (objid,containerid),
  KEY containerkey_index (containerkey)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

