-- Copyright (c) 2013 NetApp. All rights reserved
CREATE TABLE netapp_model.qos_workload_node_relationship (
  objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.',
  originid BIGINT(20) NOT NULL,
  uuid varchar(255) COLLATE utf8_bin NOT NULL COMMENT 'Globally unique ID of the relationship',
  resourceKey VARCHAR(512)  NOT NULL,
  clusterId BIGINT(20) NOT NULL,
  nodeId BIGINT(20) NOT NULL COMMENT 'ID of the Node',
  workloadId BIGINT(20) NOT NULL COMMENT 'ID of the QoS Workload',
  checksum INT(11) NOT NULL,
  changeDetailChecksum INT(11) NOT NULL,
  createtime BIGINT(20) NOT NULL,
  updatetime BIGINT(20) NOT NULL,
  objState ENUM('LIVE','DEAD'),
  KEY FK_clusterId (clusterId),
  KEY FK_workloadId (workloadId),
  KEY FK_nodeId (nodeId),
  KEY updatetime_index (updatetime),
  KEY originId_index (originid),
  KEY objState_index (objState, objid),
  CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
  CONSTRAINT FOREIGN KEY FK_workloadId (workloadId) REFERENCES netapp_model.qos_volume_workload (objid),
  CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node(objid),
  PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;


CREATE TABLE netapp_performance.sample_current_qos_workload_queue_dblade (
  objid bigint(20) NOT NULL DEFAULT '0' COMMENT 'Qos workload node relationship Id',
  time bigint(20) DEFAULT NULL,
  empty tinyint(1) DEFAULT NULL,
  containerkey varchar(255) COLLATE utf8_bin DEFAULT NULL,
  containerid bigint(20) NOT NULL DEFAULT '0' COMMENT 'Cluster ID',
  cpuDBladeResidenceTime  bigint(20) DEFAULT NULL COMMENT 'CPU Dblade residence time, measured in microseconds, not microseconds/visit.',
  cpuDBladeUtilServiceTime bigint(20) DEFAULT NULL COMMENT 'CPU Dblade util time, measured in microseconds, not microseconds/visit.',
  cpuDBladeBackgroundServiceTime  bigint(20) DEFAULT NULL COMMENT 'CPU Dblade background service time, measured in microseconds, not microseconds/visit.',
  diskHDDBackgroundServiceTime  bigint(20) DEFAULT NULL COMMENT 'Disk hdd background service time, measured in microseconds, not microseconds/visit.',
  diskSSDBackgroundServiceTime  bigint(20) DEFAULT NULL COMMENT 'Disk ssd background service time, measured in microseconds, not microseconds/visit.',
  delayWAFLSuspCPWaitTime  bigint(20) DEFAULT NULL COMMENT 'WAFL SUSP CP wait time, measured in microseconds, not microseconds/visit.',
  delayDiskIOWaitTime  bigint(20) DEFAULT NULL COMMENT 'DISK IO wait time, measured in microseconds, not microseconds/visit.',
  collectionPeriod  bigint(20) DEFAULT NULL COMMENT 'Collection Period, measured in milliseconds.',
  PRIMARY KEY (objid,containerid),
  KEY containerkey_index (containerkey)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

CREATE TABLE netapp_performance.sample_current_qos_workload_queue_nblade (
  objid bigint(20) NOT NULL DEFAULT '0' COMMENT 'Qos workload node relationship Id',
  time bigint(20) DEFAULT NULL,
  empty tinyint(1) DEFAULT NULL,
  containerkey varchar(255) COLLATE utf8_bin DEFAULT NULL,
  containerid bigint(20) NOT NULL DEFAULT '0' COMMENT 'Cluster ID',
  readOps float DEFAULT NULL COMMENT 'The workloads read operations that completed during the measurement interval; measured per second.',
  writeOps float DEFAULT NULL COMMENT 'The workloads write operations that completed during the measurement interval; measured per second.',
  otherOps float DEFAULT NULL COMMENT 'The workloads operations (other) that completed during the measurement interval; measured per second.',
  readLatency float DEFAULT NULL COMMENT 'The average read response time for requests that were initiated by the workload.',
  writeLatency  float DEFAULT NULL COMMENT 'The average write response time for requests that were initiated by the workload.',
  otherLatency float DEFAULT NULL COMMENT 'The average response (other) time for requests that were initiated by the workload.',
  readData  float DEFAULT NULL COMMENT 'The data read per second by the workload.',
  writeData  float DEFAULT NULL COMMENT 'The data written per second by the workload.',
  cpuNBladeResidenceTime  bigint(20) DEFAULT NULL COMMENT 'The CPU Nblade residence time (service time + wait time), measured in microseconds.',
  cpuNBladeUtilServiceTime  bigint(20) DEFAULT NULL COMMENT 'The CPU Nblade util service time, measured in microseconds.',
  delayClusterInterconnectWaitTime  bigint(20) DEFAULT NULL COMMENT 'The Cluster interconnect wait time, measured in microseconds.',
  delayQosLimitWaitTime  bigint(20) DEFAULT NULL COMMENT 'The QoS wait time, measured in microseconds.',
  delayNetworkWaitTime  bigint(20) DEFAULT NULL COMMENT 'The network wait time, measured in microseconds.',
  collectionPeriod  bigint(20) DEFAULT NULL COMMENT 'Collection Period, measured in milliseconds.',
  PRIMARY KEY (objid,containerid),
  KEY containerkey_index (containerkey)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;


drop table netapp_performance.sample_current_qos_workload_detail;

CREATE TABLE netapp_performance.sample_current_qos_workload_detail (
  objid BIGINT(20) COMMENT 'Reference ID of qos volume workload detail',
  time BIGINT(20),
  containerkey VARCHAR(255),
  containerid bigint(20) COMMENT 'Vserver container reference ID',
  serviceTime bigint(20) COMMENT 'The workloads average service time per visit to the service center',
  waitTime bigint(20) COMMENT 'The workloads average wait time per visit to the service center.',
  collectionPeriod  bigint(20) DEFAULT NULL COMMENT 'Collection Period, measured in milliseconds.',
  KEY containerkey_index (containerkey),
  PRIMARY KEY (objid, containerid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
  PARTITION BY LIST(containerid) (PARTITION p0 VALUES IN (0));

drop table netapp_performance.sample_current_qos_workload_constituent;
alter table netapp_performance.sample_current_qos_service_center drop column visits;
alter table netapp_performance.sample_current_qos_service_center modify waitTime bigint(20);