-- Copyright (c) 2013 NetApp. All rights reserved

CREATE TABLE netapp_model.storage_switch (
  objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPI:  storage-pool-get-iter.storage-pool-info',
  originid BIGINT(20) NOT NULL,
  uuid VARCHAR(255) NOT NULL COMMENT 'Derived from WWN. Not from ZAPI. DERIVED: {WWN}',
  resourceKey VARCHAR(512)  NOT NULL,
  name VARCHAR(255) NOT NULL COMMENT 'Name of the switch',
  clusterId BIGINT(20) NOT NULL,
  wwn VARCHAR(255) NOT NULL COMMENT 'World Wide Name of the switch',
  vendor VARCHAR(255) NOT NULL COMMENT 'Vendor of the switch. E.g.: \'cisco\' or \'brocade\'',
  firmwareVersion VARCHAR(255) NOT NULL COMMENT 'Firmware version of the switch',
  ipAddress VARCHAR(255) NOT NULL COMMENT 'IP address of the switch',
  model VARCHAR(255) NOT NULL COMMENT 'Model of the switch',
  role ENUM('NOT_MAPPED','UNKNOWN','PRIMARY','SUBORDINATE') DEFAULT NULL COMMENT 'Role of switch in the fabric',
  roleRaw VARCHAR(255) COMMENT 'Role of switch in the fabric',
  status ENUM('NOT_MAPPED','UNKNOWN','OK','ERROR') DEFAULT NULL COMMENT 'Status of switch in the fabric',
  statusRaw VARCHAR(255) COMMENT 'Status of switch in the fabric',
  sensorCount INT(11) DEFAULT NULL COMMENT 'Sensor count',
  powerSupplyCount INT(11) DEFAULT NULL COMMENT 'Power supply count',
  fanCount INT(11) DEFAULT NULL COMMENT 'Fan count',
  checksum INT(11) NOT NULL,
  changeDetailChecksum INT(11) NOT NULL,
  createtime BIGINT(20) NOT NULL,
  updatetime BIGINT(20) NOT NULL,
  objState ENUM('LIVE','DEAD'),
  KEY FK_clusterId (clusterId),
  KEY updatetime_index (updatetime),
  KEY originId_index (originid),
  KEY objState_index (objState, objid),
  CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
  PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;
