CREATE TABLE netapp_model.system_health_alert (
  objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.',
  originid BIGINT(20) NOT NULL,
  uuid VARCHAR(512) NOT NULL COMMENT 'Globally unique ID of the SHM events Derived: {clusterUUID}.alertId.alertingResource',
  resourceKey VARCHAR(512)  NOT NULL,
  alertId  VARCHAR(255) NOT NULL COMMENT 'Alert identification',
  alertingResource VARCHAR(255) NOT NULL COMMENT 'Unique name of resource that generated the alert',
  clusterId BIGINT(20) NOT NULL COMMENT 'Cluseter objid hosting this health monitor',
  nodeId BIGINT(20) NOT NULL COMMENT 'Node objid hosting this health monitor',
  indicationTime TIMESTAMP COMMENT 'Time of alert generation',
  monitor ENUM('NOT_MAPPED', 'NODE_CONNECT', 'SYSTEM_CONNECT', 'SYSTEM') COMMENT 'Type of health monitor',
  monitorRAW VARCHAR(45) COMMENT 'Type of health monitor. Value from ZAPI',
  perceivedSeverity VARCHAR(45) COMMENT 'Severity of alert',
  policy VARCHAR(255) COMMENT 'Policy rule responsible for creating this alert',
  possibleEffect VARCHAR(512) COMMENT 'Possible effect seen due to this problem',
  correctiveActions VARCHAR(512)COMMENT 'Recommended actions to correct the problem reported by alert',
  probableCause  VARCHAR(512) COMMENT 'Probable cause for alert generation',
  probableCauseDescription VARCHAR(512) COMMENT 'Detailed description of probable cause for alert generation',
  subsystem VARCHAR(255) COMMENT 'Type of subsystem being monitored',
  additionalInfo VARCHAR(512) COMMENT 'Additional information from the resource that generated this alert',
  acknowledge TINYINT(1) COMMENT ' Acknowledge the alert condition',
  acknowledger VARCHAR(45) COMMENT 'Person who acknowledged this alert',
  suppress TINYINT(1) COMMENT 'Suppress this alert',
  suppressor VARCHAR(45) COMMENT 'Person who suppressed this alert',
  checksum INT(11) NOT NULL,
  changeDetailChecksum INT(11) NOT NULL,
  createtime BIGINT(20) NOT NULL,
  updatetime BIGINT(20) NOT NULL,
  objState ENUM('LIVE','DEAD'),
  KEY FK_clusterId (clusterId),
  KEY FK_nodeId (nodeId),
  KEY updatetime_index (updatetime),
  KEY originId_index (originid),
  KEY objState_index (objState, objid),
  CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
  CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node(objid),
  PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

