-- SELECT 'DROP TABLE track_table' AS "Statement";
-- DROP TABLE IF EXISTS netapp_performance.track_table;

-- The table tracks the summary tables whose historical data has been recalculated
CREATE TABLE IF NOT EXISTS netapp_performance.track_table (
  table_name VARCHAR(100) NOT NULL,
  updated_on TIMESTAMP DEFAULT CURRENT_TIMESTAMP
);

-- SELECT 'DROP FUNCTION ratio_method' AS "Statement";
DROP FUNCTION IF EXISTS netapp_performance.ratio_method;
DELIMITER $$

CREATE FUNCTION netapp_performance.ratio_method(col1 VARCHAR(50), col2 VARCHAR(50))
  RETURNS VARCHAR(250) DETERMINISTIC
BEGIN
  DECLARE result VARCHAR(250);
  SELECT CONCAT('COALESCE(SUM(sa.', col1, '*sa.', col2, ')/SUM(sa.', col2, '), AVG(sa.', col1, ')) AS ', col1) INTO result;
  RETURN result;
END;

$$
DELIMITER ;

-- Conditionally drop the procedure.
-- SELECT 'DROP PROCEDURE update_summary_aggregate' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_aggregate;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_aggregate()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE from_time BIGINT(20);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name LIKE 'summary_aggregate%' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE cursor_sample_view CURSOR FOR SELECT DISTINCT time - MOD(time, 3600000)  FROM netapp_performance.sample_view; 
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    -- DROP VIEW IF EXISTS sample_view;
	SET @query = CONCAT('CREATE VIEW netapp_performance.sample_view AS SELECT * FROM netapp_performance.', sample_table);  
    PREPARE stmt from @query; 
    EXECUTE stmt; 
    DEALLOCATE PREPARE stmt;

    OPEN cursor_sample_view; 
    FETCH cursor_sample_view INTO from_time; 
	WHILE done = FALSE DO 
      -- SELECT obj_id, from_time; 
      SET @sql = CONCAT('INSERT INTO ', summary_table, '(objid, fromtime, totime, cpReadBlocks, cpReads, totalTransfers, ',
					  '      userReadBlocks,  userReads, userWriteBlock, userWrites) ',
                      '  SELECT sa.objid, ', from_time, ', ', from_time + 3599999, ', '
					  '    AVG(sa.cpReadBlocks) AS cpReadBlocks, AVG(sa.cpReads) AS cpReads, AVG(sa.totalTransfers) AS totalTransfers, AVG(sa.userReadBlocks) AS userReadBlocks, ',
					  '    AVG(sa.userReads) AS userReads, AVG(sa.userWriteBlock) AS userWriteBlock, AVG(sa.userWrites) AS userWrites ',
					  '  FROM ', sample_table, ' sa ',
					  '  WHERE sa.time >= ', from_time, ' AND sa.time < ', from_time + 3599999,
                      '  GROUP BY sa.objid ',
					  '  ON DUPLICATE KEY UPDATE cpReadBlocks = values(cpReadBlocks), cpReads = values(cpReads), ',
                      '    totalTransfers = values(totalTransfers), userReadBlocks = values(userReadBlocks), userReads = values(userReads), ',
                      '    userWriteBlock = values(userWriteBlock), userWrites = values(userWrites)' );
                        
	  -- SELECT @sql;
      PREPARE stmt from @sql;
	  EXECUTE stmt;
      DEALLOCATE PREPARE stmt;
      FETCH cursor_sample_view INTO from_time; 
    END WHILE; 
    CLOSE cursor_sample_view; 
    DROP VIEW sample_view; 
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table);     
	COMMIT;
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$
 
-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- Conditionally drop the procedure.
-- SELECT 'DROP PROCEDURE netapp_performance.update_summary_nfsv3' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_cifsvserver;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_cifsvserver()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name = 'summary_cifsvserver' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE netapp_performance.', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
					  '     SUM(sa.cifs_latency_base) AS cifs_latency_base, ',
						    ratio_method('cifs_latency', 'cifs_ops'), ', ',
                      '     AVG(sa.cifs_ops) AS cifs_ops, ',
						    ratio_method('cifs_read_latency', 'cifs_read_ops'), ', ',
                      '     AVG(sa.cifs_read_ops) AS cifs_read_ops, ',
						    ratio_method('cifs_write_latency', 'cifs_write_ops'), ', ',
                      '     AVG(sa.cifs_write_ops) AS cifs_write_ops ', 
					  '   FROM netapp_performance.', sample_table, ' sa ',
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.cifs_latency_base = b.cifs_latency_base, a.cifs_latency = b.cifs_latency, ',
                      '    a.cifs_ops = b.cifs_ops, a.cifs_read_latency = b.cifs_read_latency, ',
                      '    a.cifs_read_ops = b.cifs_read_ops, a.cifs_write_latency = b.cifs_write_latency, ',
                      '    a.cifs_write_ops = b.cifs_write_ops ', 
                      ' WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table);
    COMMIT;     
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$
 
-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- Conditionally drop the procedure.
-- SELECT 'DROP PROCEDURE update_summary_disk' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_disk;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_disk()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name LIKE 'summary_disk%' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;
  DECLARE CONTINUE HANDLER FOR SQLSTATE '42S21' BEGIN END;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
  	SET @query = CONCAT('SELECT COUNT(*) INTO @table_count FROM information_schema.tables ', 
						'WHERE table_name = \'', sample_table, '\'', 'AND table_schema = \'netapp_performance\' AND table_type = \'BASE TABLE\'');
	PREPARE stmt from @query;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
    IF @table_count = 0 then
       -- SELECT @table_count, sample_table, summary_table;
	   SET @drop_table = CONCAT('DROP TABLE IF EXISTS netapp_performance.', summary_table);
	   PREPARE stmt from @drop_table;
       EXECUTE stmt;
       DEALLOCATE PREPARE stmt;
    ELSE 
      SET @drop_table = CONCAT('ALTER TABLE netapp_performance.', summary_table, ' ADD COLUMN baseForDiskBusy BIGINT(20) UNSIGNED DEFAULT NULL AFTER diskBusy ');
	  PREPARE stmt from @drop_table;
      EXECUTE stmt;
      DEALLOCATE PREPARE stmt;
      
      SET @sql = CONCAT('UPDATE netapp_performance.', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
                      '    AVG(sa.cpReadBlocks) AS cpReadBlocks, ',
					  '    COALESCE(SUM(sa.cpReadLatency*sa.cpReads)/SUM(sa.cpReads), AVG(sa.cpReadLatency)) AS cpReadLatency, ',
                      '    AVG(sa.cpReads) AS cpreads, ',
					  '    COALESCE(SUM(sa.diskBusy*sa.baseForDiskBusy)/SUM(sa.baseForDiskBusy), AVG(sa.diskBusy)) AS diskBusy, ',
					  '    SUM(sa.baseForDiskBusy) AS baseForDiskBusy, ',
					  '    AVG(sa.guaranteedReadBlocks) AS guaranteedReadBlocks, ',
					  '    COALESCE(SUM(sa.guaranteedReadLatency*sa.guaranteedReads)/SUM(sa.guaranteedReads), AVG(sa.guaranteedReadLatency)) AS guaranteedReadLatency, ',
					  '    AVG(sa.guaranteedReads) AS guaranteedReads, ',
					  '    AVG(sa.guaranteedWriteBlocks) AS guaranteedWriteBlocks, ',
					  '    COALESCE(SUM(sa.guaranteedWriteLatency*sa.guaranteedWrites)/SUM(sa.guaranteedWrites), AVG(sa.guaranteedWriteLatency)) AS guaranteedWriteLatency, ',
                      '    AVG(sa.guaranteedWrites) AS guaranteedWrites, ',
                      '    AVG(sa.totalTransfers) AS totalTransfers, ',
                      '    AVG(sa.userReadBlocks) AS userReadBlocks, ',
                      '    COALESCE(SUM(sa.userReadLatency*sa.userReads)/SUM(sa.userReads), AVG(sa.userReadLatency)) AS userReadLatency, ',
					  '    AVG(sa.userReads) AS userReads, ',
                      '    AVG(sa.userWriteBlocks) AS userWriteBlocks, ',
                      '    COALESCE(SUM(sa.userWriteLatency*sa.userWrites)/SUM(sa.userWrites), AVG(sa.userWriteLatency)) AS userWriteLatency, ',
                      '    AVG(sa.userWrites) AS userWrites ',
					  '   FROM netapp_performance.', sample_table, ' sa ',  
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.cpReadBlocks = b.cpReadBlocks, a.cpReadLatency = b.cpReadLatency, a.cpReads = b.cpReads, a.diskBusy = b.diskBusy, ',
                      '    a.baseForDiskBusy = b.baseForDiskBusy, a.guaranteedReadBlocks = b.guaranteedReadBlocks, a.guaranteedReadLatency = b.guaranteedReadLatency, ',
                      '    a.guaranteedReads = b.guaranteedReads, a.guaranteedWriteBlocks = b.guaranteedWriteBlocks, a.guaranteedWriteLatency = b.guaranteedWriteLatency, ',
					  '    a.guaranteedWrites = b.guaranteedWrites, a.totalTransfers = b.totalTransfers, a.userReadBlocks = b.userReadBlocks, a.userReadLatency = b.userReadLatency, ',
                      '    a.userReads = b.userReads, a.userWriteBlocks = b.userWriteBlocks, a.userWriteLatency = b.userWriteLatency, a.userWrites = b.userWrites ',
                      ' WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	  -- SELECT @sql;

      PREPARE stmt from @sql;
      EXECUTE stmt;
      DEALLOCATE PREPARE stmt;
    END IF;
    
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table); 
	COMMIT;
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$

-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- Conditionally drop the procedure.
-- SELECT 'DROP PROCEDURE netapp_performance.update_summary_fcplif' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_fcplif;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_fcplif()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name LIKE 'summary_fcplif%' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE netapp_performance.', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
						    netapp_performance.ratio_method('avgLatency', 'totalOps'), ', ',
					  '     AVG(sa.totalOps) AS totalOps, ',
					  '     AVG(sa.otherOps) AS otherOps, ',
					  '     AVG(sa.readData) AS readData, ',
                      '     AVG(sa.readOps) AS readOps, ',
					  '     AVG(sa.writeData) AS writeData, ',
                      '     AVG(sa.writeOps) AS writeOps ',
					  '   FROM netapp_performance.', sample_table, ' sa ',  
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.avgLatency = b.avgLatency, a.totalOps = b.totalOps, a.otherOps = b.otherOps, ',
					  '    a.readData = b.readData, a.readOps = b.readOps, a.writeData = b.writeData, a.writeOps = b.writeOps ',
                      ' WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table);   
    COMMIT;  
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$

-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- Conditionally drop the procedure.
-- SELECT 'DROP PROCEDURE netapp_performance.update_summary_fcpLifVserver' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_fcpLifVserver;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_fcpLifVserver()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name LIKE 'summary_fcpLifVserver%' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE netapp_performance.', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
						    netapp_performance.ratio_method('avgLatency', 'totalOps'), ', ',
					  '     AVG(sa.totalOps) AS totalOps, ',
					  '     AVG(sa.otherOps) AS otherOps, ',
					  '     AVG(sa.readData) AS readData, ',
                      '     AVG(sa.readOps) AS readOps, ',
					  '     AVG(sa.writeData) AS writeData, ',
                      '     AVG(sa.writeOps) AS writeOps ',
					  '   FROM netapp_performance.', sample_table, ' sa ',  
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.avgLatency = b.avgLatency, a.totalOps = b.totalOps, a.otherOps = b.otherOps, ',
					  '    a.readData = b.readData, a.readOps = b.readOps, a.writeData = b.writeData, a.writeOps = b.writeOps ',
                      ' WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table);  
    COMMIT;   
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$
 
-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- Conditionally drop the procedure.
-- SELECT 'DROP PROCEDURE netapp_performance.update_summary_iscsilif' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_iscsilif;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_iscsilif()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name LIKE 'summary_iscsilif' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE netapp_performance.', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
					  '    SUM(sa.dataInBlocks) AS dataInBlocks, ',
					  '    SUM(sa.dataOutBlocks) AS dataOutBlocks, ',
					  '    AVG(sa.iscsiReadOps) AS iscsiReadOps, ',
					  '    AVG(sa.iscsiWriteOps) AS iscsiWriteOps, ',
					  '    AVG(sa.protocolErrors) AS protocolErrors, ',
                      '    AVG(sa.readData) AS readData, ',
                      '    AVG(sa.writeData) AS writeData, ',
					  '    SUM(sa.cmdTransfered) AS cmdTransfered, ',
					       netapp_performance.ratio_method('avgLatency', 'cmdTransfered'),
					  '   FROM netapp_performance.', sample_table, ' sa ',  
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.dataInBlocks = b.dataInBlocks, a.dataOutBlocks = b.dataOutBlocks, a.iscsiReadOps = b.iscsiReadOps, ',
                      '    a.iscsiWriteOps = b.iscsiWriteOps, a.protocolErrors = b.protocolErrors, a.readData = b.readData, ',
                      '    a.writeData = b.writeData, a.cmdTransfered = b.cmdTransfered, a.avgLatency = b.avgLatency ',
                      ' WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table);
    COMMIT;     
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$
 
-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- Conditionally drop the procedure.
-- SELECT 'DROP PROCEDURE update_summary_iscsilifVserver' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_iscsilifVserver;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_iscsilifVserver()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name LIKE 'summary_iscsilifVserver' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE netapp_performance.', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
					  '    SUM(sa.dataInBlocks) AS dataInBlocks, ',
					  '    SUM(sa.dataOutBlocks) AS dataOutBlocks, ',
					  '    AVG(sa.iscsiReadOps) AS iscsiReadOps, ',
					  '    AVG(sa.iscsiWriteOps) AS iscsiWriteOps, ',
					  '    AVG(sa.protocolErrors) AS protocolErrors, ',
                      '    AVG(sa.readData) AS readData, ',
                      '    AVG(sa.writeData) AS writeData, ',
					  '    SUM(sa.cmdTransfered) AS cmdTransfered, ',
					       netapp_performance.ratio_method('avgLatency', 'cmdTransfered'),
					  '   FROM netapp_performance.', sample_table, ' sa ',  
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.dataInBlocks = b.dataInBlocks, a.dataOutBlocks = b.dataOutBlocks, a.iscsiReadOps = b.iscsiReadOps, ',
                      '    a.iscsiWriteOps = b.iscsiWriteOps, a.protocolErrors = b.protocolErrors, a.readData = b.readData, ',
                      '    a.writeData = b.writeData, a.cmdTransfered = b.cmdTransfered, a.avgLatency = b.avgLatency ',
                      ' WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table);     
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$

-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- Conditionally drop the procedure.
-- SELECT 'DROP PROCEDURE netapp_performance.update_summary_nfsv3' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_nfsv3;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_nfsv3()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name = 'summary_nfsv3' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE ', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
					  '     AVG(sa.nfsv3_ops) AS nfsv3_ops, ',
					  '     AVG(sa.nfsv3_read_ops) AS nfsv3_read_ops, ',
                      '     AVG(sa.nfsv3_write_ops) AS nfsv3_write_ops, ',
					  '     SUM(sa.read_total) AS read_total, ',
                      '     AVG(sa.nfsv3_read_throughput) AS nfsv3_read_throughput, ',
                      '     AVG(sa.nfsv3_throughput) AS nfsv3_throughput, ',
					  '     SUM(sa.write_total) AS write_total, ',
                      '     AVG(sa.nfsv3_write_throughput) AS nfsv3_write_throughput, ',
						    ratio_method('read_avg_latency', 'read_total'), ', ',
						    ratio_method('write_avg_latency', 'write_total'), 
					  '   FROM netapp_performance.', sample_table, ' sa ',
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.nfsv3_ops = b.nfsv3_ops, a.nfsv3_read_ops = b.nfsv3_read_ops, ',
					  '    a.nfsv3_write_ops = b.nfsv3_write_ops, a.read_total = b.read_total, ',
                      '    a.nfsv3_read_throughput = b.nfsv3_read_throughput, a.nfsv3_throughput = b.nfsv3_throughput, ',
                      '    a.write_total = b.write_total, a.nfsv3_write_throughput = b.nfsv3_write_throughput, ',
                      '    a.read_avg_latency = b.read_avg_latency, a.write_avg_latency = b.write_avg_latency ', 
                      ' WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table);
    COMMIT;     
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$
 
-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- Conditionally drop the procedure.
-- SELECT 'DROP PROCEDURE netapp_performance.update_summary_nfsv4' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_nfsv4;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_nfsv4()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name = 'summary_nfsv4' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE ', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
					  '     AVG(sa.nfsv4_ops) AS nfsv4_ops, ',
					  '     SUM(sa.read_total) AS read_total, ',
                      '     AVG(sa.nfsv4_read_throughput) AS nfsv4_read_throughput, ',
                      '     AVG(sa.nfsv4_throughput) AS nfsv4_throughput, ',
					  '     SUM(sa.write_total) AS write_total, ',
                      '     AVG(sa.nfsv4_write_throughput) AS nfsv4_write_throughput, ',
						    ratio_method('read_avg_latency', 'read_total'), ', ',
						    ratio_method('write_avg_latency', 'write_total'), 
					  '   FROM netapp_performance.', sample_table, ' sa ',
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.nfsv4_ops = b.nfsv4_ops, a.read_total = b.read_total, ',
                      '    a.nfsv4_read_throughput = b.nfsv4_read_throughput, a.nfsv4_throughput = b.nfsv4_throughput, ',
                      '    a.write_total = b.write_total, a.nfsv4_write_throughput = b.nfsv4_write_throughput, ',
                      '    a.read_avg_latency = b.read_avg_latency, a.write_avg_latency = b.write_avg_latency ', 
                      ' WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table);
    COMMIT;     
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$
 
-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- Conditionally drop the procedure.
-- SELECT 'DROP PROCEDURE netapp_performance.update_summary_nfsv41' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_nfsv41;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_nfsv41()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name = 'summary_nfsv41' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE ', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
					  '     AVG(sa.nfsv41_ops) AS nfsv41_ops, ',
					  '     SUM(sa.read_total) AS read_total, ',
                      '     AVG(sa.nfsv41_read_throughput) AS nfsv41_read_throughput, ',
                      '     AVG(sa.nfsv41_throughput) AS nfsv41_throughput, ',
					  '     SUM(sa.write_total) AS write_total, ',
                      '     AVG(sa.nfsv41_write_throughput) AS nfsv41_write_throughput, ',
						    ratio_method('read_avg_latency', 'read_total'), ', ',
						    ratio_method('write_avg_latency', 'write_total'), 
					  '   FROM netapp_performance.', sample_table, ' sa ',
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.nfsv41_ops = b.nfsv41_ops, a.read_total = b.read_total, ',
                      '    a.nfsv41_read_throughput = b.nfsv41_read_throughput, a.nfsv41_throughput = b.nfsv41_throughput, ',
                      '    a.write_total = b.write_total, a.nfsv41_write_throughput = b.nfsv41_write_throughput, ',
                      '    a.read_avg_latency = b.read_avg_latency, a.write_avg_latency = b.write_avg_latency ', 
                      ' WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table);
    COMMIT;     
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$
 
-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- Conditionally drop the procedure.
-- SELECT 'DROP PROCEDURE netapp_performance.update_summary_node' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_node;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_node()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name LIKE 'summary_node%' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;
 
  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE netapp_performance.', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
					  '    COALESCE(SUM(sa.cpuBusy*sa.cpuElapsedTime)/SUM(sa.cpuElapsedTime), AVG(sa.cpuBusy)) AS cpuBusy, ',
                      '    COALESCE(SUM(sa.avgProcessorBusy*sa.cpuElapsedTime)/SUM(sa.cpuElapsedTime), AVG(sa.avgProcessorBusy)) AS avgProcessorBusy, ',
                      '    SUM(sa.cpuElapsedTime) AS cpuElapsedTime, ',
					  '    AVG(sa.netDataRecv) AS netDataRecv, ',
					  '    AVG(sa.netDataSent) AS netDataSent, ',
					  '    AVG(sa.readOps) AS readOps, ',
					  '    AVG(sa.writeOps) AS writeOps, ',
					  '    AVG(sa.systemOps) AS systemOps, ',
                      '    AVG(sa.totalOps) AS totalOps, ',
                      '    AVG(sa.otherOps) AS otherOps, ',
                      '    AVG(sa.partnerOps) AS partnerOps, ',
                      '    AVG(sa.cifsOps) AS cifsOps, ',
                      '    AVG(sa.fcpOps) AS fcpOps, ',
					  '    AVG(sa.httpOps) AS httpOps, ',
                      '    AVG(sa.iscsiOps) AS iscsiOps, ',
                      '    AVG(sa.nfsOps) AS nfsOps, ',
					  '    COALESCE(SUM(sa.sysReadLatency*sa.readOps)/SUM(sa.readOps), AVG(sa.sysReadLatency)) AS sysReadLatency, ',
                      '    COALESCE(SUM(sa.sysWriteLatency*sa.writeOps)/SUM(sa.writeOps), AVG(sa.sysWriteLatency)) AS sysWriteLatency, ',
                      '    COALESCE(SUM(sa.sysAvgLatency*sa.totalOps)/SUM(sa.totalOps), AVG(sa.sysAvgLatency)) AS sysAvgLatency ',
					  '   FROM netapp_performance.', sample_table, ' sa ',  
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.cpuBusy = b.cpuBusy, a.avgProcessorBusy = b.avgProcessorBusy, a.cpuElapsedTime = b.cpuElapsedTime, a.netDataRecv = b.netDataRecv, ',
					  '    a.netDataSent = b.netDataSent, a.readOps = b.readOps, a.writeOps = b.writeOps, a.systemOps = b.systemOps, ',
					  '    a.totalOps = b.totalOps, a.otherOps = b.otherOps, a.partnerOps = b.partnerOps, a.cifsOps = b.cifsOps, ',
                      '    a.fcpOps = b.fcpOps, a.httpOps = b.httpOps, a.iscsiOps = b.iscsiOps, a.nfsOps = b.nfsOps, ',
                      '    a.sysReadLatency = b.sysReadLatency, a.sysWriteLatency = b.sysWriteLatency, a.sysAvgLatency = b.sysAvgLatency ',
                      ' WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table); 
	COMMIT;
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$

-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- SELECT 'DROP PROCEDURE update_summary_qos_service_center' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_qos_service_center;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_qos_service_center()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name LIKE 'summary_qos_service_center%' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;
  
  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE netapp_performance.', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
					  '    SUM(sa.waitTime) AS waitTime, ',
			          '    COALESCE(SUM(sa.utilization*sa.cpuElapsedTime*sa.numInstances)/(SUM(sa.cpuElapsedTime)*AVG(sa.numInstances)), AVG(sa.utilization)) AS utilization, '
					  '    SUM(sa.collectionPeriod) AS collectionPeriod, ',
					  '    SUM(sa.cpuElapsedTime) AS cpuElapsedTime, ',
                      '    AVG(sa.numInstances) AS numInstances ',
					  '   FROM netapp_performance.', sample_table, ' sa ', 
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.waitTime = b.waitTime, a.utilization = b.utilization, a.collectionPeriod = b.collectionPeriod, ',
                      '    a.cpuElapsedTime = b.cpuElapsedTime, a.numInstances = b.numInstances ', 
                      'WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table); 
    COMMIT;    
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$
 
-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- SELECT 'DROP PROCEDURE netapp_performance.update_summary_qos_volume_workload' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_qos_volume_workload;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_qos_volume_workload()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name LIKE 'summary_qos_volume_workload%' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE netapp_performance.', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
                           ratio_method('latency', 'ops'), ', ',
					  '    AVG(sa.ops) AS ops, ',
					  '    AVG(sa.totalData) AS totalData, ',
					  '    AVG(sa.readData) AS readData, ',
					       ratio_method('readLatency', 'readOps'), ', ',
					  '    AVG(sa.readOps) AS readOps, ',
                      '    AVG(sa.writeData) AS writeData, ',
                           ratio_method('writeLatency', 'writeOps'), ', ',
                      '    AVG(sa.writeOps) AS writeOps, ',
                           ratio_method('cacheMissRate', 'cacheMissRateBase'), ', ',
					  '    SUM(sa.cacheMissRateBase) AS cacheMissRateBase ',
					  '   FROM netapp_performance.', sample_table, ' sa ',
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.latency = b.latency, a.ops = b.ops, a.totalData = b.totalData, a.readData = b.readData, ',
					  '    a.readLatency = b.readLatency, a.readOps = b.readOps, a.writeData = b.writeData, a.writeLatency = b.writeLatency, ',
					  '    a.writeOps = b.writeOps, a.cacheMissRate = b.cacheMissRate, a.cacheMissRateBase = b.cacheMissRateBase ',
                      'WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table);     
    COMMIT;
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$
 
-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- SELECT 'DROP PROCEDURE netapp_performance.update_summary_qos_workload_detail' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_qos_workload_detail;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_qos_workload_detail()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name LIKE 'summary_qos_workload_detail%' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE netapp_performance.', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
					  '    SUM(sa.serviceTime) AS serviceTime, ',
					  '    SUM(sa.waitTime) AS waitTime, ',
					  '    SUM(sa.collectionPeriod) AS collectionPeriod ',
					  '   FROM netapp_performance.', sample_table, ' sa ',
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.serviceTime = b.serviceTime, a.waitTime = b.waitTime, a.collectionPeriod = b.collectionPeriod ',
                      'WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table);   
    COMMIT;  
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$
 
-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- SELECT 'DROP PROCEDURE netapp_performance.update_summary_qos_workload_queue_dblade' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_qos_workload_queue_dblade;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_qos_workload_queue_dblade()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name LIKE 'summary_qos_workload_queue_dblade%' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE netapp_performance.', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
					  '    SUM(sa.cpuDBladeResidenceTime) AS cpuDBladeResidenceTime, ',
					  '    SUM(sa.cpuDBladeUtilServiceTime) AS cpuDBladeUtilServiceTime, ',
					  '    SUM(sa.cpuDBladeBackgroundServiceTime) AS cpuDBladeBackgroundServiceTime, ',
					  '    SUM(sa.cpuDBladeBackgroundWAFLCPServiceTime) AS cpuDBladeBackgroundWAFLCPServiceTime, ',
                      '    SUM(sa.cpuDBladeBackgroundBackgroundDeleteServiceTime) AS cpuDBladeBackgroundBackgroundDeleteServiceTime, ',
                      '    SUM(sa.cpuDBladeBackgroundRAStripeXServiceTime) AS cpuDBladeBackgroundRAStripeXServiceTime, ',
                      '    SUM(sa.diskHDDBackgroundServiceTime) AS diskHDDBackgroundServiceTime, ',
                      '    SUM(sa.diskHDDBackgroundWAFLCPServiceTime) AS diskHDDBackgroundWAFLCPServiceTime, ',
                      '    SUM(sa.diskHDDBackgroundBackgroundDeleteServiceTime) AS diskHDDBackgroundBackgroundDeleteServiceTime, ',
                      '    SUM(sa.diskHDDBackgroundRAStripeXServiceTime) AS diskHDDBackgroundRAStripeXServiceTime, ',
                      '    SUM(sa.diskSSDBackgroundServiceTime) AS diskSSDBackgroundServiceTime, ',
                      '    SUM(sa.diskSSDBackgroundWAFLCPServiceTime) AS diskSSDBackgroundWAFLCPServiceTime, ',
                      '    SUM(sa.diskSSDBackgroundBackgroundDeleteServiceTime) AS diskSSDBackgroundBackgroundDeleteServiceTime, ',
                      '    SUM(sa.diskSSDBackgroundRAStripeXServiceTime) AS diskSSDBackgroundRAStripeXServiceTime, ',
                      '    SUM(sa.delayWAFLSuspCPWaitTime) AS delayWAFLSuspCPWaitTime, ',
                      '    SUM(sa.delayDiskIOWaitTime) AS delayDiskIOWaitTime, ',
                      '    SUM(sa.delayNvlogTransferWaitTime) AS delayNvlogTransferWaitTime, ',
                           netapp_performance.ratio_method('cacheMissRate', 'cacheMissRateBase'), ', ',
                      '    SUM(sa.collectionPeriod) AS collectionPeriod, ',
					  '    SUM(sa.cacheMissRateBase) AS cacheMissRateBase ',
					  '   FROM netapp_performance.', sample_table, ' sa ', 
					  '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.cpuDBladeResidenceTime = b.cpuDBladeResidenceTime, ',
                      '    a.cpuDBladeUtilServiceTime = b.cpuDBladeUtilServiceTime, ',
                      '    a.cpuDBladeBackgroundServiceTime = b.cpuDBladeBackgroundServiceTime, ',
                      '    a.cpuDBladeBackgroundWAFLCPServiceTime = b.cpuDBladeBackgroundWAFLCPServiceTime, ',
                      '    a.cpuDBladeBackgroundBackgroundDeleteServiceTime = b.cpuDBladeBackgroundBackgroundDeleteServiceTime, ', 
                      '    a.cpuDBladeBackgroundRAStripeXServiceTime = b.cpuDBladeBackgroundRAStripeXServiceTime, ', 
                      '    a.diskHDDBackgroundServiceTime = b.diskHDDBackgroundServiceTime, ',
                      '    a.diskHDDBackgroundWAFLCPServiceTime = b.diskHDDBackgroundWAFLCPServiceTime, ',
                      '    a.diskHDDBackgroundBackgroundDeleteServiceTime = b.diskHDDBackgroundBackgroundDeleteServiceTime, ',
                      '    a.diskHDDBackgroundRAStripeXServiceTime = b.diskHDDBackgroundRAStripeXServiceTime, ',
                      '    a.diskSSDBackgroundServiceTime = b.diskSSDBackgroundServiceTime, ',
                      '    a.diskSSDBackgroundWAFLCPServiceTime = b.diskSSDBackgroundWAFLCPServiceTime, ',
                      '    a.diskSSDBackgroundBackgroundDeleteServiceTime = b.diskSSDBackgroundBackgroundDeleteServiceTime, ',
                      '    a.diskSSDBackgroundRAStripeXServiceTime = b.diskSSDBackgroundRAStripeXServiceTime, ',
                      '    a.delayWAFLSuspCPWaitTime = b.delayWAFLSuspCPWaitTime, ', 
                      '    a.delayDiskIOWaitTime = b.delayDiskIOWaitTime, ', 
                      '    a.delayNvlogTransferWaitTime = b.delayNvlogTransferWaitTime, ',
                      '    a.cacheMissRate = b.cacheMissRate, ',
					  '    a.collectionPeriod = b.collectionPeriod, ',
					  '    a.cacheMissRateBase = b.cacheMissRateBase ',
                      'WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table);     
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$
 
-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- SELECT 'DROP PROCEDURE netapp_performance.update_summary_qos_workload_queue_nblade' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_summary_qos_workload_queue_nblade;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a procedure
CREATE PROCEDURE netapp_performance.update_summary_qos_workload_queue_nblade()
BEGIN
  DECLARE sample_table VARCHAR(100);
  DECLARE summary_table VARCHAR(100);
  DECLARE done BOOL DEFAULT FALSE;

  DECLARE cursor_table CURSOR FOR SELECT REPLACE(table_name, 'summary', 'sample'), table_name FROM information_schema.tables 
								  WHERE table_name LIKE 'summary_qos_workload_queue_nblade%' AND table_schema = 'netapp_performance' AND table_type = 'BASE TABLE'
                                    AND table_name NOT IN (SELECT table_name FROM netapp_performance.track_table);
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cursor_table;
  FETCH cursor_table INTO sample_table, summary_table;
  WHILE done = FALSE DO
    SET @sql = CONCAT('UPDATE netapp_performance.', summary_table, ' a, ',
                      '  (SELECT sa.objid AS objid, (sa.time - MOD(sa.time, 3600000)) AS time, ',
					  '    AVG(sa.readOps) AS readOps, ',
					  '    AVG(sa.writeOps) AS writeOps, ',
					  '    AVG(sa.otherOps) AS otherOps, ',
					       netapp_performance.ratio_method('readLatency', 'readOps'), ', ',
					       netapp_performance.ratio_method('writeLatency', 'writeOps'), ', ',
					       netapp_performance.ratio_method('otherLatency', 'otherOps'), ', ',
					  '    AVG(sa.readData) AS readData, ',
                      '    AVG(sa.writeData) AS writeData, ', 
                      '    SUM(sa.cpuNBladeResidenceTime) AS cpuNBladeResidenceTime, ',
                      '    SUM(sa.cpuNBladeUtilServiceTime) AS cpuNBladeUtilServiceTime, ',
                      '    SUM(sa.delayClusterInterconnectWaitTime) AS delayClusterInterconnectWaitTime, ',
                      '    SUM(sa.delayQosLimitWaitTime) AS delayQosLimitWaitTime, ',
                      '    SUM(sa.delayNetworkWaitTime) AS delayNetworkWaitTime, ',
					  '    SUM(sa.collectionPeriod) AS collectionPeriod ',
					  '   FROM netapp_performance.', sample_table, ' sa ', 
                      '   GROUP BY objid, sa.time - MOD(sa.time, 3600000)) b ',
					  'SET a.readOps = b.readOps, a.writeOps = b.writeOps, a.otherOps = b.otherOps, ',
                      '    a.readLatency = b.readLatency, a.writeLatency = b.writeLatency, a.otherLatency = b.otherLatency, ',
                      '    a.readData = b.readData, a.writeData = b.writeData, a.cpuNBladeResidenceTime = b.cpuNBladeResidenceTime, ', 
                      '    a.cpuNBladeUtilServiceTime = b.cpuNBladeUtilServiceTime, a.delayClusterInterconnectWaitTime = b.delayClusterInterconnectWaitTime, ',
					  '    a.delayQosLimitWaitTime = b.delayQosLimitWaitTime, a.delayNetworkWaitTime = b.delayNetworkWaitTime, a.collectionPeriod = b.collectionPeriod ',
                      'WHERE a.objid = b.objid AND a.fromtime = b.time AND a.totime = b.time + 3599999 ');
                        
	-- SELECT @sql;

    PREPARE stmt from @sql;
    EXECUTE stmt;
    DEALLOCATE PREPARE stmt;
 
	INSERT INTO netapp_performance.track_table(table_name) VALUES(summary_table);
	COMMIT;
	FETCH cursor_table INTO sample_table, summary_table;
  END WHILE;

  CLOSE cursor_table;

END;
$$
 
-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- Conditionally drop the procedure.
-- SELECT 'DROP PROCEDURE update_all_summaries' AS "Statement";
DROP PROCEDURE IF EXISTS netapp_performance.update_all_summaries;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create a main procedure to recalculate all historical summary data
CREATE PROCEDURE netapp_performance.update_all_summaries()
BEGIN
	
  -- SELECT 'CALL netapp_performance.update_summary_aggregate' AS "Statement";
  CALL netapp_performance.update_summary_aggregate();

  -- SELECT 'CALL netapp_performance.update_summary_cifsvserver' AS "Statement";
  CALL netapp_performance.update_summary_cifsvserver();

  -- SELECT 'CALL netapp_performance.update_summary_disk' AS "Statement";  
  CALL netapp_performance.update_summary_disk();
  
  -- SELECT 'CALL netapp_performance.update_summary_fcplif' AS "Statement";
  CALL netapp_performance.update_summary_fcplif();
  
  -- SELECT 'CALL netapp_performance.update_summary_fcpLifVserver' AS "Statement";
  CALL netapp_performance.update_summary_fcpLifVserver();

  -- SELECT 'CALL update_summary_iscsilif' AS "Statement";
  CALL netapp_performance.update_summary_iscsilif();

  -- SELECT 'CALL netapp_performance.update_summary_iscsilifVserver' AS "Statement"; 
  CALL netapp_performance.update_summary_iscsilifVserver();

  -- SELECT 'CALL netapp_performance.update_summary_nfsv3' AS "Statement";
  CALL netapp_performance.update_summary_nfsv3();
  
  -- SELECT 'CALL netapp_performance.update_summary_nfsv4' AS "Statement";
  CALL netapp_performance.update_summary_nfsv4();
  
    -- SELECT 'CALL netapp_performance.update_summary_nfsv41' AS "Statement";
  CALL netapp_performance.update_summary_nfsv41();

  -- SELECT 'CALL netapp_performance.update_summary_node' AS "Statement";
  CALL netapp_performance.update_summary_node();
  
  -- SELECT 'CALL netapp_performance.update_summary_qos_service_center' AS "Statement";
  CALL netapp_performance.update_summary_qos_service_center();
   
  -- SELECT 'CALL netapp_performance.update_summary_qos_volume_workload' AS "Statement";
  CALL netapp_performance.update_summary_qos_volume_workload();

  -- SELECT 'CALL netapp_performance.update_summary_qos_workload_detail' AS "Statement";
  CALL netapp_performance.update_summary_qos_workload_detail();
  
  -- SELECT 'CALL netapp_performance.update_summary_qos_workload_queue_dblade' AS "Statement";
  CALL netapp_performance.update_summary_qos_workload_queue_dblade();
  
  -- SELECT 'CALL netapp_performance.update_summary_qos_workload_queue_nblade' AS "Statement";
  CALL netapp_performance.update_summary_qos_workload_queue_nblade();

  DROP TABLE IF EXISTS netapp_performance.track_table;
  
END;
$$

-- Reset the delimiter back to a semicolon to work again.
DELIMITER ;

-- SELECT 'CALL netapp_performance.update_all_summaries' AS "Statement";
CALL netapp_performance.update_all_summaries();