DROP FUNCTION IF EXISTS netapp_performance.concat_addColumns;
DELIMITER $$

CREATE FUNCTION netapp_performance.concat_addColumns(table_name VARCHAR(255), xmlDoc TEXT)
	returns VARCHAR(3000) deterministic
begin
	DECLARE result VARCHAR(3000);
     DECLARE i INT DEFAULT 1;
     DECLARE columnName VARCHAR(255) DEFAULT '';
     DECLARE columnType VARCHAR(255) DEFAULT '';
     DECLARE comments VARCHAR(1000) DEFAULT '';
     -- concat all "add column"  sql commands
     -- e.g.,
     -- add column c1 float DEFAULT null COMMNET this data for latency,
     -- add column c2 flaot DEFAULT null COMMENT this data for busy,
     REPEAT
          SET columnName = ExtractValue(xmlDoc, '//columnName[$i]');
          SET columnType = ExtractValue(xmlDoc, '//columnType[$i]');
          SET comments = ExtractValue(xmlDoc, '//comments[$i]');
          SET i = i+1;
          IF ((columnName IS NOT NULL) and (columnName != '') and (select 
			            count(*) = 0
			        from
			            information_schema.COLUMNS c
		          where
		              c.table_schema = 'netapp_performance'
                  and c.table_name = table_name
                  and c.column_name = columnName )) THEN
              SET result = CONCAT_WS(',', result, CONCAT('ADD COLUMN ', columnName, ' ', columnType, ' ', comments));
          END IF;
     UNTIL (columnName IS NULL or columnName= '') END REPEAT;
	RETURN CONCAT(result, ';');

End;

$$
DELIMITER ;

DROP PROCEDURE IF EXISTS netapp_performance.add_columns;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create procedure
CREATE PROCEDURE netapp_performance.add_columns(IN table_expression VARCHAR(255), IN xmlDoc TEXT)
BEGIN
     DECLARE done BOOL DEFAULT FALSE;
     DECLARE sqlTXT varchar(3000);
	 DECLARE tableName varchar(255);

     DECLARE cursor_table CURSOR FOR select
					table_name 
			from
                  information_schema.TABLES t
              where
                  t.table_schema = 'netapp_performance'
	              AND (t.table_name RLIKE table_expression);

     DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

     OPEN cursor_table;
     cursor_table_loop : LOOP
            FETCH cursor_table INTO tableName;
            IF done THEN
              LEAVE cursor_table_loop;
            END IF;

            set @sqlTXT = concat_addColumns(tableName, xmlDoc);
            IF @sqlTXT IS NOT NULL and @sqlTXT != '' THEN
              select CONCAT('ALTER TABLE ', 'netapp_performance', '.', tableName, ' ', @sqlTXT) INTO @sql;
              SELECT @sql;
              PREPARE stmt from @sql;
              EXECUTE stmt;
              DEALLOCATE PREPARE stmt;
            END IF;

     END LOOP cursor_table_loop;
     CLOSE cursor_table;
 
END;

$$
DELIMITER ;
