DROP FUNCTION IF EXISTS netapp_performance.concat_changeColumns;
DELIMITER $$

CREATE FUNCTION netapp_performance.concat_changeColumns(table_name VARCHAR(255), xmlDoc TEXT)
	returns VARCHAR(1000) deterministic
begin
	DECLARE result VARCHAR(1000);
     DECLARE i INT DEFAULT 1;
     DECLARE oldColumnName VARCHAR(255) DEFAULT '';
	 DECLARE newColumnName VARCHAR(255) DEFAULT '';
	 DECLARE columnType VARCHAR(255) DEFAULT '';

     -- generate all "change "  sql commands
     -- e.g.,
     -- change old_c1 new_c1 FLOAT,
     -- change old_c2 new_c2 FLAOT,
     REPEAT
          SET oldColumnName = ExtractValue(xmlDoc, '//oldColumnName[$i]');
		  SET newColumnName = ExtractValue(xmlDoc, '//newColumnName[$i]');
		  SET columnType = ExtractValue(xmlDoc, '//columnType[$i]');
          SET i = i+1;
          IF ((oldColumnName IS NOT NULL) and (oldColumnName != '') and (select
						count(*) > 0
				from
						information_schema.COLUMNS c
				where
						c.TABLE_SCHEMA = 'netapp_performance'
						and c.TABLE_NAME = table_name
						and c.COLUMN_NAME = oldColumnName)) THEN
             SET result = CONCAT_WS(',', result, CONCAT('CHANGE ', oldColumnName, ' ', newColumnName,' ',columnType));
          END IF;
     UNTIL (oldColumnName IS NULL or oldColumnName= '') END REPEAT;
	RETURN CONCAT(result, ';');

End;

$$
DELIMITER ;

DROP PROCEDURE IF EXISTS netapp_performance.change_columns;

-- Reset the delimiter to write a procedure.
DELIMITER $$

-- Create procedure
CREATE PROCEDURE netapp_performance.change_columns(IN table_expression VARCHAR(255), IN xmlDoc TEXT)
BEGIN
     DECLARE done BOOL DEFAULT FALSE;
     DECLARE sqlTXT varchar(2000);
	DECLARE tableName varchar(255);

     DECLARE cursor_table CURSOR FOR select
				table_name from
                  information_schema.TABLES t
              where
                  t.table_schema = 'netapp_performance'
				AND (t.table_name RLIKE table_expression);

     DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

     OPEN cursor_table;
     cursor_table_loop : LOOP
            FETCH cursor_table INTO tableName;
            IF done THEN
              LEAVE cursor_table_loop;
            END IF;

            set @sqlTXT = concat_changeColumns(tableName, xmlDoc);
select @sqlTXT;
			IF @sqlTXT IS NOT NULL and @sqlTXT != '' THEN
				SELECT CONCAT ('ALTER TABLE ', 'netapp_performance', '.', tableName, ' ', @sqlTXT) INTO @sql;
				SELECT @sql;
				PREPARE stmt from @sql;
				EXECUTE stmt;
				DEALLOCATE PREPARE stmt;
			END IF;

     END LOOP cursor_table_loop;
     CLOSE cursor_table;

END;

$$
DELIMITER ;