package com.netapp.oci.platform.framework.performance;

import com.netapp.oci.platform.framework.performance.data.Sample;

import java.util.Collection;
import java.sql.Connection;

import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;

/**
 * Strategy to persist performance samples.
 *
 * @author Anton Sergeev
 */
public interface PersistenceStrategy<C> {
    /**
     * Saves new samples to db
     *
     * @param connection - database connection
     * @param samples containing at most a single sample per monitored object (i.e. switch port, or virtual-machine)
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException if failed to save the samples in the DB or if duplicates were found
     */
    void execute(Connection connection, Collection<Sample<C>> samples) throws PerformanceException;

    /**
     * Return string id of the performance package - id is used to create
     * distinct database tables and for logging
     *
     * @return performance package identifier
     */
    String getPerformancePackageId();
}
