/*
 * Copyright (c) 2009 NetApp
 * All rights reserved
 */
package com.netapp.oci.platform.framework.performance.util;

import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;
import com.netapp.oci.platform.common.param.PluginParam;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.InsertSelectSummaryTable;
import com.netapp.oci.platform.framework.performance.tables.SampleTable;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;
import org.apache.log4j.Logger;

import java.sql.Connection;

/**
 * Summarization persistence strategy based on the usage of insert select summary table
 *
 * @author Anton Sergeev
 */
public abstract class InsertSelectSummarizePersistenceStrategy<C, S> extends SummarizePersistenceStrategy<C, S> {
    protected InsertSelectSummarizePersistenceStrategy(Logger logger, PluginParam param, String sampleContainerKey, Long sampleContainerId) {
        super(logger, param, sampleContainerKey, sampleContainerId);
    }

    protected abstract SqlMethod[][] getSummarizationMethods();

    protected void summarize(SampleTable<C> sampleTable, Connection connection) throws PerformanceException {
        InsertSelectSummaryTable<S> summaryTable = (InsertSelectSummaryTable<S>) getSummaryTable(connection);
        try {
            summaryTable.insert(sampleTable, getHourTimeSpan(sampleTable), getSummarizationMethods());
        } catch (TableException ex) {
            throw new PerformanceException("Failed to generate summaries from table " + sampleTable.getFullName(), ex); 
        }
    }
}
