/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.job;

import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.plugin.data.PlatformPlugin;
import com.netapp.oci.platform.plugin.remote.PlatformPluginRemote;
import org.apache.log4j.Logger;
import org.jboss.ejb3.annotation.ResourceAdapter;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import javax.annotation.Resource;
import javax.annotation.security.RunAs;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.EJB;
import javax.ejb.MessageDriven;
import javax.sql.DataSource;
import java.sql.Connection;

@MessageDriven(activationConfig = {
        @ActivationConfigProperty(propertyName = "cronTrigger", propertyValue = "0 0 * * * ?"),
        @ActivationConfigProperty(propertyName = "jobName", propertyValue = "PruneJob")})
@ResourceAdapter("quartz-ra.rar")
@RunAs("admin")
/**
 * The purpose of this job is to prune the expired database entries.
 */
@SuppressWarnings("unused")
public class PruneJob implements Job {

    private static final Logger logger = Logger.getLogger(PruneJob.class);

    @Resource(name = "jdbc/OnaroDataSource",
            mappedName = "java:/onaro_pool",
            type = DataSource.class,
            authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    @EJB
    private PlatformPluginRemote platformPluginRemote;

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        if (logger.isDebugEnabled()) {
            logger.debug("Starting Prune Job");
        }
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            for (PlatformPlugin platformPlugin: platformPluginRemote.getPlugins()) {
                try {
                    platformPlugin.prune(connection, System.currentTimeMillis());
                } catch (Exception e) {
                    logger.error("Exception while pruning '" + platformPlugin + "' plugin", e);
                }
            }
        } finally {
            ConnectionHelper.close(connection);
            if (logger.isDebugEnabled()) {
                logger.debug("Completed Prune Job");
            }
        }
    }
}
