package com.netapp.oci.simulation.time;

/**
 * Represents accelerated flow of time when time flows much faster then normal,
 * basically as fast as CPU allows. Gaps between triggers are not being waited out.
 *
 * @author Anton Sergeev
 */
public class AcceleratedTimeFlow implements TimeFlow {
    /**
     * When this time flow starts
     */
    private long startTime;

    /**
     * When this time flow stops
     */
    private long stopTime;

    /**
     * Time flow offset after start
     */
    private long offset;

    public AcceleratedTimeFlow(long startTime, long stopTime) {
        this.startTime = startTime;
        this.stopTime = stopTime;
    }

    public long getCurrentTimeMillis() {
        return startTime + offset;
    }

    public void sleep(long timeMillis) {
        offset += timeMillis;
    }

    public boolean shouldStop() {
        return getCurrentTimeMillis() > stopTime;
    }

    public boolean isSynchronous() {
        return true;
    }
}
