package com.netapp.oci.simulation.time;

import com.netapp.oci.simulation.SimulationActor;
import com.netapp.oci.simulation.time.schedule.Trigger;

import java.util.HashMap;
import java.util.Map;

/**
 * Component controls the time flow of the simulation
 *
 * @author Anton Sergeev
 */
public class TimeControl {
    private Map<Trigger, SimulationActor> triggerToActor = new HashMap<Trigger, SimulationActor>();

    /**
     * Registers simulation actor and associates given schedule trigger with it
     *
     * @param actor simulation actor
     * @param trigger time trigger
     */
    public void registerActor(SimulationActor actor, Trigger trigger) {
        triggerToActor.put(trigger, actor);
    }

    /**
     * Runs the simulation according to provided time flow
     *
     * @param timeFlow time flow to use during simulation (normal or accelerated)
     */
    public void run(TimeFlow timeFlow) {
        while (!timeFlow.shouldStop()) {
            final long currentTime = timeFlow.getCurrentTimeMillis();
            for (Trigger trigger: triggerToActor.keySet()) {
                if (trigger.itIsTime(currentTime)) {
                    final SimulationActor actor = triggerToActor.get(trigger);
                    if (timeFlow.isSynchronous()) {
                        actor.act(currentTime);
                    } else {
                        new Thread(new Runnable() {
                            public void run() {
                                actor.act(currentTime);
                            }
                        }).start();
                    }
                }
            }
            timeFlow.sleep(1000);
        }
    }
}
