package com.netapp.oci.simulation.time.schedule;

import com.onaro.commons.types.Period;

import java.util.Random;

/**
 * Periodic trigger delayed by fixed or random amount of time
 * 
 * @author Anton Sergeev
 */
public class PeriodicDelayedTrigger extends PeriodicTrigger {
    private int delayedByUnits;

    public PeriodicDelayedTrigger(Period period, int units, int delayedByUnits) {
        super(period, units);
        this.delayedByUnits = delayedByUnits;
    }

    public PeriodicDelayedTrigger(Period period, int units) {
        this(period, units, getRandomDelay(units));
    }

    private static int getRandomDelay(int units) {
        Random random = new Random();

        return random.nextInt(units);
    }

    @Override
    public boolean itIsTime(long time) {
        if (lastTimeTriggered == 0) {
            lastTimeTriggered = time - period.milliseconds() * (units - delayedByUnits);
        }

        return super.itIsTime(time);
    }

    public int getDelayedByUnits() {
        return delayedByUnits;
    }
}
