/**
 * NetApp 2014
 */

package com.netapp.oci.statsutils;

/**
 * This class defines the different types of statistics that MEGA/OCIE
 * currently stores in the database.
 *
 */ 
public enum StatisticType {

    /**
     * Statistic type is unknown.
     */
    UNKNOWN("unknown", 0),

    /**
     * Statistical type of raw.  This type would be the raw collected value
     * from the device in question.
     */
    RAW("raw", 1),

    /**
     * Statistical type of cooked.  A "cooked" statistic is a value that has
     * been calculated based on the requirement of each raw statistic.
     */
    COOKED("cooked", 2),

    /**
     * Statistical type of summary.  Summary statistics are "calculated" statistics
     * that are rollup up into a larger time window.  As an example, if calculated
     * statistics are collected every 5 minutes, then the summary statistics are 
     * rolled up into a single hourly statistic.
     */
    SUMMARY("summary", 3),

    /**
     * Statistical type of aggregate_raw.  Aggregation is the grouping of statistics
     * into a single entity.  In most cases this entity is a container of the 
     * underlying statistic.  For this enumeration, this is the aggregation of the
     * raw counter types.
     */
    AGGREGATE_RAW("aggregate_raw", 4),

    /**
     * Statistical type of aggregate_cooked.  Aggregation is the grouping of 
     * statistics into a single entity.  In most cases this entity is a container of the 
     * underlying statistic.  For this enumeration, this is the aggregation of the
     * raw calculated types.
     */
    AGGREGATE_COOKED("aggregate_cooked", 5);

    /**
     * The name of the enumeration.
     */
    private final String name;
    /** 
     * The value of the enumeration
     */
    private final int value;
    
    /**
     * Default constructor.
     */
    StatisticType( String name, int value ) {
        this.name = name;
        this.value = value;
    }

    /**
     * Get name.
     *
     * @return name as String.
     */
    public String getName() {
        return name;
    }
    
    /**
     * Get value.
     *
     * @return value as int.
     */
    public int getValue() {
        return value;
    }
}

