package com.onaro.sanscreen.acquisition.cli;

import com.onaro.sanscreen.acquisition.cli.commands.Command;
import com.onaro.sanscreen.acquisition.cli.commands.CommandType;

import java.util.Map;

/**
 * Implements the command-line use case (execution of a single, stand-along CLI command)
 *
 * Each command in this use case has to include user and password.
 */
public class CommandLineMode {
    public static void main(final String[] args) {
        try {
            // First argument has to be a command
            if (args.length < 1) {
                throw new IllegalArgumentException("No arguments provided");
            }

            final CLI cli = new CLI();

            final Command command = CommandType.fromCommandString(args[0]).getCommand();
            final Map<String, String> arguments = cli.parseArguments(args, 1);
            cli.checkMandatoryArguments(arguments, true, "user", "password");

            // Extract the user & password from the argument list (we use them in login. No need to pass them on).
            final String user = arguments.remove("user");
            final String password = arguments.remove("password");
            cli.login(user, password);

            // Run the command
            cli.run(command, arguments);
        }
        catch (IllegalArgumentException iae) {
            iae.printStackTrace();

            showHelp();
            System.exit(1);
        }
        catch (Exception e) {
            e.printStackTrace();

            System.exit(1);
        }
    }

    private static void showHelp() {
        System.out.println("Usage:");
        System.out.println("======");
        System.out.println("acqcli <command> [-argname=value ...] -user=<user> -password=<password>");
        System.out.println();

        showCommandHelp();
    }

    static String showCommandHelp() {
        System.out.println("Available commands:");

        final String formatString = "    %-50s\t%s\n";

        for (final CommandType commandType : CommandType.values()) {
            System.out.format(formatString, commandType.getCommand().usageString(), commandType.getCommand().usageDescription());
        }

        return formatString;
    }
}
