package com.onaro.sanscreen.acquisition.cli.commands;

import com.onaro.sanscreen.acquisition.cli.CLI;
import com.onaro.sanscreen.acquisition.sessions.AcquisitionFacadeForClientSession;
import com.onaro.sanscreen.acquisition.sessions.AcquisitionFacadeSession;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceAudit;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceConfiguration;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceNew;

import java.util.Date;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

@SuppressWarnings({"deprecation"})
public class DescDsCommand implements Command {
    @Override
    public String usageString() {
        return "desc -id=<dsid>";
    }

    @Override
    public String usageDescription() {
        return "Describe a data source with the given id (provide all details known)";
    }

    @Override
    public void run(final CLI cli, final Map<String, String> args) throws Exception {
        String formatString;

        cli.checkMandatoryArguments(args, true, "id");
        final long dsid = Long.parseLong(args.get("id"));

        final AcquisitionFacadeSession facadeSession = cli.getSession(AcquisitionFacadeSession.JNDI_NAME);
        final DataSourceNew dataSource = facadeSession.getDsById(dsid);

        System.out.println("Data source details:");
        System.out.println("====================");

        formatString = "%-20s\t%-40s\n";
        System.out.format(formatString, "ID:", dataSource.getId());
        System.out.format(formatString, "Name:", dataSource.getName());
        System.out.format(formatString, "Vendor:", dataSource.getVendor());
        System.out.format(formatString, "Model:", dataSource.getModel());
        System.out.format(formatString, "Type:", dataSource.getDsTypeId());
        System.out.format(formatString, "Status:", dataSource.getStatus());
        System.out.format(formatString, "Status Ex:", dataSource.getStatusExt());
        System.out.println();

        // List property values (sorted by name)
        System.out.println("Properties:");
        System.out.println("===========");

        formatString = "%-40s\t%-50s\n";
        final SortedSet<String> propertyNames = new TreeSet<String>(dataSource.getAttrs().stringPropertyNames());
        for (final String propertyName : propertyNames) {
            final String propertyValue;
            if (!"netappfoundation.password".equals(propertyName)) {
                propertyValue = dataSource.getAttrs().getProperty(propertyName);
            } else {
                // OCIE netappfoundation password is not encrypted, so mask the value
                propertyValue = "********";
            }
            System.out.format(formatString, propertyName, propertyValue);
        }
        System.out.println();

        // List recent audit
        System.out.println("Recent audits:");
        System.out.println("==============");

        formatString = "%-30s\t%-30s\t%-5s\t%-40s\n";

        System.out.format(formatString, "Start", "End", "Times", "Status");
        System.out.format(formatString, "=====", "===", "=====", "======");

        final DataSourceAudit[] audits = facadeSession.getAllDsHistoryById(dsid);
        for (int c = 0 ; c < Math.min(audits.length, 10) ; c++) {
            System.out.format(formatString,
                              new Date(audits[c].getStartTime()),
                              new Date(audits[c].getEndTime()),
                              audits[c].getNumberOfTimes(),
                              audits[c].getStatus());
        }

    }
}
