package com.onaro.sanscreen.acquisition.cli.commands;

import com.onaro.sanscreen.acquisition.cli.CLI;
import com.onaro.sanscreen.server.sessions.ForceAcqSession;

import java.util.Map;
import java.util.Properties;

public class ForceDsCommand implements Command
{
  @Override
  public String usageString()
  {
    return "force -id=<dsid> -type=<(P)artial|(F)Full>";
  }

  @Override
  public String usageDescription()
  {
    return "Force an immediate poll for a data source with the given id";
  }

  @Override
  public void run(final CLI cli, final Map<String, String> args) throws Exception
  {
    //validate args
    if (args.size() < 1 || args.size() > 2 )
      throw new IllegalArgumentException("Invalid arguments usage:\n" + usageDescription() );
    cli.checkMandatoryArguments( args, true, "id" );
    String type = args.get( "type" );
    if( args.size() == 2 )
    {
      //validate the type argument
      cli.checkMandatoryArguments( args, true, "type" );
      if( type == null )
        throw new IllegalArgumentException("Invalid arguments usage:\n" + usageDescription() );

      type = type.toLowerCase();
      if( !( type.equals( "f" ) || type.equals( "full" ) || type.equals( "p" ) || type.equals( "partial" ) ) )
        throw new IllegalArgumentException( "Invalid arguments usage:\n" + usageDescription() );

      //make sure the type is either f or p not full or partial
      type = type.substring( 0, 1 );
    }

    //get args
    long dsid = Long.parseLong( args.get( "id" ) );

    //add optional args to message
    final Properties properties = new Properties();
    if( type != null )
      properties.setProperty( "type", type );

    //execute
    final ForceAcqSession forceAcqSession = cli.getSession( ForceAcqSession.JNDI_NAME );
    forceAcqSession.forceAcquisition( new long[]{dsid}, properties );

    System.out.println( "Data source forced poll" );
  }
}
