package com.onaro.sanscreen.acquisition.cli.commands;

import com.onaro.sanscreen.acquisition.cli.CLI;
import com.onaro.sanscreen.acquisition.sessions.AcquisitionFacadeSession;
import com.onaro.sanscreen.server.dataobject.acquisition.AcquisitionUnit;

import java.util.Collection;
import java.util.Map;

@SuppressWarnings({"deprecation"})
public class LsAuCommand implements Command {
    @Override
    public String usageString() {
        return "lsau";
    }

    @Override
    public String usageDescription() {
        return "Lists all acquisition units currently in the system";
    }

    @Override
    public void run(final CLI cli, final Map<String, String> args) throws Exception {
        if (args.size() > 0)
            throw new IllegalArgumentException("No arguments expected for the 'lsau' command");

        // Get a list of all data sources
        final AcquisitionFacadeSession facadeSession = cli.getSession(AcquisitionFacadeSession.JNDI_NAME);
        final Collection<AcquisitionUnit> acqUnits = facadeSession.getAllAcquisitionUnit();

        // Listing all the existing data sources
        final String formatString = "%-24s\t%-40s\t%-20s\n";

        System.out.format(formatString, "Name", "IP", "Status");
        System.out.format(formatString, "====", "==", "======");

        for (final AcquisitionUnit acqUnit : acqUnits) {
            System.out.format(formatString,
                              acqUnit.getName(),
                              acqUnit.getIp(),
                              acqUnit.getAcqStatus());
        }
    }
}
