package com.onaro.sanscreen.acquisition.cli.commands;

import com.onaro.sanscreen.acquisition.cli.CLI;
import com.onaro.sanscreen.acquisition.sessions.AcquisitionFacadeSession;
import com.onaro.sanscreen.server.dataobject.acquisition.AcquisitionUnitNew;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSource;

import java.util.Map;

@SuppressWarnings({"deprecation"})
public class LsDsCommand implements Command {
    @Override
    public String usageString() {
        return "ls";
    }

    @Override
    public String usageDescription() {
        return "Lists all data sources currently in the system";
    }

    @Override
    public void run(final CLI cli, final Map<String, String> args) throws Exception {
        if (args.size() > 0)
            throw new IllegalArgumentException("No arguments expected for the 'ls' command");

        // Get a list of all data sources
        final AcquisitionFacadeSession facadeSession = cli.getSession(AcquisitionFacadeSession.JNDI_NAME);
        final com.onaro.sanscreen.server.dataobject.acquisition.DataSource[] dataSources = facadeSession.getAllDs();

        // Listing all the existing data sources
        final String formatString = "%-3s\t%-24s\t%-8s\t%-40s\t%-5s\t%-12s\t%-35s\t%-7s\n";

        System.out.format(formatString, "ID", "Name", "Acq Unit", "IP", "Type", "Vendor", "Model", "State");
        System.out.format(formatString, "==", "====", "========", "==", "====", "======", "=====", "=====");

        for (final com.onaro.sanscreen.server.dataobject.acquisition.DataSource dataSource : dataSources) {
            final AcquisitionUnitNew acqUnit = facadeSession.getAcquisitionUnit(dataSource.getAcquisitionUnitName() != null ? dataSource.getAcquisitionUnitName() : "local");

            System.out.format(formatString,
                              dataSource.getId(),
                              dataSource.getName(),
                              dataSource.getAcquisitionUnitName() != null ? dataSource.getAcquisitionUnitName() : "local",
                              dataSource.getIP(),
                              dataSource.getType(),
                              dataSource.getVendor(),
                              dataSource.getModel(),
                              figureOutState(dataSource, acqUnit));
        }
    }

    private String figureOutState(final DataSource dataSource, final AcquisitionUnitNew acqUnit) {
        if (!acqUnit.isActive())
            return "Acq Unit not running";

        if (dataSource.getManual() == 1) {
            // It is supposed to be running
            if (dataSource.isActive())
                return "Running";

            return "Starting";
        }
        else if (dataSource.getManual() == 2) {
            // It is supposed to be running
            if (dataSource.isActive())
                return "Stopping";

            return "Stopped";
        }
        else
            return "Unknown";
    }
}
