package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders;


import com.netapp.oci.netapp.performance.cluster.interfaces.data.ClusterMeasurements;
import com.netapp.oci.netapp.performance.node.interfaces.data.NodeMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * Builder class for processing and reporting cluster metrics.
 */
public class ClusterMetricsBuilder {
	 private final NetAppOCIEPerformancePackage netAppOCIEPerformancePackage;

	    public ClusterMetricsBuilder(final NetAppOCIEPerformancePackage netAppOCIEPerformancePackage) {
	        this.netAppOCIEPerformancePackage = netAppOCIEPerformancePackage;
	    }

	    public Map<Cluster.ClusterKey, Collection<RawSample<ClusterMeasurements>>> processCounterData(
	            final NetAppDefaultStorageContext context,
	            Map<Cluster.ClusterKey, Collection<RawSample<NodeMeasurements>>> nodeMeasurements){
	    	
	    	final List<RawSample<ClusterMeasurements>> rawSamples = new LinkedList<RawSample<ClusterMeasurements>>();
	        final Map<Cluster.ClusterKey, Collection<RawSample<ClusterMeasurements>>> allMeasurementsToReport =
	                new HashMap<Cluster.ClusterKey, Collection<RawSample<ClusterMeasurements>>>(1);
	        allMeasurementsToReport.put(context.getKey(), rawSamples);
	        
	        final Map<String, ClusterMetricsScratchPad> clusterMetricsScratchPadsMap =
	        		new HashMap<String, ClusterMetricsScratchPad>();

	        for(final Cluster.ClusterKey clusterKey : nodeMeasurements.keySet()){	

	        	ClusterMetricsScratchPad clusterMetricsScratchPad = getClusterMetricsScratchPad(clusterMetricsScratchPadsMap, clusterKey);
	        	Collection<RawSample<NodeMeasurements>> allNodeMeasurementsForCluster = nodeMeasurements.get(clusterKey);
	        	ClusterMeasurements clusterMeasurements =  new ClusterMeasurements();
	        	for(RawSample<NodeMeasurements> eachNodeSample : allNodeMeasurementsForCluster){
	        		NodeMeasurements nodeMeasurement =  eachNodeSample.getCounters();
                    if(nodeMeasurement!=null)  {
                        clusterMetricsScratchPad.addClusterOps(nodeMeasurement.getTotalOps());
                        clusterMetricsScratchPad.addClusterThroughput(nodeMeasurement.getOpmSysThroughput());
                        clusterMetricsScratchPad.setClusterMaxNodeUtilization(nodeMeasurement.getOpmSysUtilization());
                        clusterMetricsScratchPad.setClusterMaxAggregateUtilization(nodeMeasurement.getMaxAggregateUtilization());
                    }
	        	}
	        	clusterMeasurements.setClusterOps(clusterMetricsScratchPad.getClusterOps());
        		clusterMeasurements.setClusterThroughput(clusterMetricsScratchPad.getClusterThroughput());
        		clusterMeasurements.setClusterMaxNodeUtilization(clusterMetricsScratchPad.getClusterMaxNodeUtilization());
        		clusterMeasurements.setClusterMaxAggregateUtilization(clusterMetricsScratchPad.getClusterMaxAggregateUtilization());

	        	rawSamples.add(new RawSample<ClusterMeasurements>(context.getKey().getOcieUUID(), clusterMeasurements));
	        }
	        return allMeasurementsToReport;
	    }
	    

	    public void reportPerformance(final Map<Cluster.ClusterKey, Collection<RawSample<ClusterMeasurements>>> clusterMetrics)
	            throws DataSourceErrorException {
	        if(clusterMetrics == null) {
	            return;
	        }
	        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<ClusterMeasurements>>> measurementEntry : clusterMetrics.entrySet()) {
	            final Cluster.ClusterKey key = measurementEntry.getKey();
	            final Collection<RawSample<ClusterMeasurements>> clusterMeasurements = measurementEntry.getValue();
	            netAppOCIEPerformancePackage.reportClusterPerformance(key, clusterMeasurements);
	        }
	    }
	    
	    public ClusterMetricsScratchPad getClusterMetricsScratchPad(Map<String, ClusterMetricsScratchPad> clusterMetricsScratchPads, Cluster.ClusterKey key){
	    	String clusterUUID = key.getOcieUUID();
	    	ClusterMetricsScratchPad clusterMetricsScratchPad = clusterMetricsScratchPads.get(clusterUUID);
	    	if(clusterMetricsScratchPad == null){
	    		clusterMetricsScratchPad = new ClusterMetricsScratchPad();
	    		clusterMetricsScratchPads.put(clusterUUID, clusterMetricsScratchPad);
	    		return clusterMetricsScratchPad;
	    	}
	    	else {
	    		return clusterMetricsScratchPad;
	    	}
	    }
	    
	private static class ClusterMetricsScratchPad {
		private float clusterOps;
		private float clusterThroughput;
		private float clusterMaxNodeUtilization;
		private float clusterMaxAggregateUtilization;
		
		public float getClusterOps() {
			return clusterOps;
		}
		
		public void addClusterOps(float clusterOps) {
			this.clusterOps = MetricsRowScratchPad.add(this.clusterOps, clusterOps);
		}
		
		public float getClusterThroughput() {
			return clusterThroughput;
		}
		
		public void addClusterThroughput(float clusterThroughput) {
			this.clusterThroughput = MetricsRowScratchPad.add(this.clusterThroughput, clusterThroughput);
		}
		
		public float getClusterMaxNodeUtilization() {
			return clusterMaxNodeUtilization;
		}
		
		public void setClusterMaxNodeUtilization(float clusterMaxNodeUtilization) {
			this.clusterMaxNodeUtilization = Math.max(this.clusterMaxNodeUtilization, clusterMaxNodeUtilization);
			
		}
		
		public float getClusterMaxAggregateUtilization() {
			return clusterMaxAggregateUtilization;
		}
		
		public void setClusterMaxAggregateUtilization(
				float clusterMaxAggregateUtilization) {
			this.clusterMaxAggregateUtilization = Math.max(this.clusterMaxAggregateUtilization, clusterMaxAggregateUtilization);
		}
		
	}
}
