package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders;


import com.netapp.oci.netapp.performance.wafl.interfaces.data.WaflMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.metrics.ArchiveCounterMetaData;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.metrics.ArchiveMetricsScratchPad;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.Collection;
import java.util.Map;

/**
 * Builder class for processing and reporting wafl metrics.
 */
public class WaflMetricsBuilder extends ArchiveMetricsBuilderBase<Cluster.ClusterKey,
        WaflMeasurements, WaflMeasurements, WaflMetricsBuilder.WaflCounter> {


    public static enum WaflCounter implements ArchiveCounterMetaData {
    	cp_count,
        total_cp_msecs,
        wafl_cp_blks_read,
        P2_FLUSH("cp_phase_times", 34),;

        private final String onTapCounterName;
        private final int onTapCounterIndex;

        private WaflCounter(final String onTapCounterName, final int onTapCounterIndex) {
            this.onTapCounterName = onTapCounterName;
            this.onTapCounterIndex = onTapCounterIndex;
        }

        private WaflCounter() {
            this.onTapCounterName = this.name();
            this.onTapCounterIndex = -1;
        }

        @Override
        public String getOnTapCounterName() {
            return onTapCounterName;
        }

        @Override
        public int getOnTapCounterIndex() {
            return onTapCounterIndex;
        }
    }

    /**
     * Retrieve the workload UUID form the performance archive and append the node uuid.
     *
     * @param context    The foundation context
     * @param nodeName   Node name from where the archiver was retrieved
     * @param counterRow Raw counter data form the performance archive
     * @return Performance Instance UUID
     */
    @Override
    protected String getPerformanceUUID(final NetAppDefaultStorageContext context,
                                        final String nodeName, final Map<String, Object> counterRow) {
    	
    	return new StringBuilder().append(nodeName).append(':')
                .append("kernel:wafl").toString();
    }

    public WaflMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "wafl", WaflCounter.class);
    }
    
    @Override
    protected String getInventoryUUIDFromPerformanceUUID(final NetAppDefaultStorageContext context,
            final String performanceInstanceUUID, String nodeName, Map<String, Object> counterRow) {
    	Map<String, String> waflPerfUUIDToNodeUUID = context.getWaflPerfUUIDtoNodeUUID();
    	return waflPerfUUIDToNodeUUID.get(performanceInstanceUUID);
    }
    

    @Override
    protected Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context,
                                                           final String perfInstanceUUID, String nodeName, Map<String, Object> counterRow) {
        return context.getKey();
    }

    @Override
    public void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey,
            Collection<RawSample<WaflMeasurements>>> measurements) throws DataSourceErrorException {
        if (measurements == null) {
            return;
        }
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<WaflMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<WaflMeasurements>> waflMeasurements = measurementEntry.getValue();
            reportingPackage.reportWaflPerformance(key, waflMeasurements);
        }
    }

    @Override
    public WaflCounter[] getCountersMetaData(final NetAppDefaultStorageContext context) {
        return WaflCounter.values();
    }

    @Override
    protected WaflMeasurements normalizeMetrics(final ArchiveMetricsScratchPad<WaflCounter> currentScratchPad,
                                                                  final NetAppDefaultStorageContext context) {
        final WaflMeasurements waflMeasurements = new WaflMeasurements();
        waflMeasurements.setCpCount(currentScratchPad.getCounterDelta(WaflCounter.cp_count));
        waflMeasurements.setP2Flush(currentScratchPad.getPercentFromDeltas(WaflCounter.P2_FLUSH, WaflCounter.total_cp_msecs));
        waflMeasurements.setTotalCpMsecs(currentScratchPad.getCounterDelta(WaflCounter.total_cp_msecs));
        waflMeasurements.setWaflCpBlksRead(currentScratchPad.getCounterDelta(WaflCounter.wafl_cp_blks_read));

        return waflMeasurements;
    }

	@Override
	protected WaflMeasurements rawMetrics(
			ArchiveMetricsScratchPad<WaflCounter> currentScratchPad,
			NetAppDefaultStorageContext context) {
		return null;
	}

	@Override
	public void reportRawPerformance(
			ClusterKey clusterKey,
			Map<ClusterKey, Collection<RawSample<WaflMeasurements>>> rawMeasurements)
			throws DataSourceErrorException {
		
	}

}
