package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.common;


import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * This class contains the meta for an archive file.
 * @param <C> Enum with a list of counter object names.
 */
public class ArchiveFileMetaData<C extends Enum<C>> {
    /*
     * Contains information returned from the ZAPI
     */
    private final ArchiveDataSourceInfo archiveDataSourceInfo;
    /*
     * An archive files has several samples for each object. This map contains the counter object name to the sample
     * time that will be processed by acquisition.
     */
    private final Map<C,Long> counterObjectNameToTimestampToProcess;
    /*
     * List of samples to be processed in this file.
     */
    private final Set<Long> timestampsToProcess;

    public ArchiveFileMetaData(final ArchiveDataSourceInfo archiveDataSourceInfo,
                               final Map<C, Long> counterObjectNameToTimestampToProcess) {
        this.archiveDataSourceInfo = archiveDataSourceInfo;
        this.counterObjectNameToTimestampToProcess = Collections.unmodifiableMap(counterObjectNameToTimestampToProcess);
        this.timestampsToProcess = Collections.unmodifiableSet(new HashSet<Long>(counterObjectNameToTimestampToProcess.values()));
    }

    public ArchiveDataSourceInfo getArchiveDataSourceInfo() {
        return archiveDataSourceInfo;
    }

    public Map<C, Long> getCounterObjectNameToTimestampToProcess() {
        return counterObjectNameToTimestampToProcess;
    }

    public Set<Long> getTimestampsToProcess() {
        return this.timestampsToProcess;
    }

}
