package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.impl;

import com.netapp.oci.netapp.performance.aggregate.interfaces.data.AggregateMeasurements;
import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverMeasurements;
import com.netapp.oci.netapp.performance.cluster.interfaces.data.ClusterMeasurements;
import com.netapp.oci.netapp.performance.disk.interfaces.data.DiskMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifMeasurements;
import com.netapp.oci.netapp.performance.fcpport.interfaces.data.FcpPortMeasurements;
import com.netapp.oci.netapp.performance.hostadapter.interfaces.data.HostadapterMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifMeasurements;
import com.netapp.oci.netapp.performance.lun.interfaces.data.LunMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifMeasurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv3Measurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv41Measurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv4Measurements;
import com.netapp.oci.netapp.performance.nic.interfaces.data.NicMeasurements;
import com.netapp.oci.netapp.performance.node.interfaces.data.NodeMeasurements;
import com.netapp.oci.netapp.performance.processor.interfaces.data.ProcessorMeasurements;
import com.netapp.oci.netapp.performance.qosservicecenter.interfaces.data.QosServiceCenterMeasurements;
import com.netapp.oci.netapp.performance.qosworkload.interfaces.data.QosVolumeWorkloadMeasurements;
import com.netapp.oci.netapp.performance.qosworkloaddblade.interfaces.data.QosWorkloadQueueDBladeMeasurements;
import com.netapp.oci.netapp.performance.qosworkloaddetail.interfaces.data.QosWorkloadDetailMeasurements;
import com.netapp.oci.netapp.performance.qosworkloadnblade.interfaces.data.QosWorkloadQueueNBladeMeasurements;
import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeMeasurements;
import com.netapp.oci.netapp.performance.wafl.interfaces.data.WaflMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.*;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.common.BasePerformanceArchive;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.common.CounterObjectsInterface;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.parser.CounterTable;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders.FcpLifVserverMetricsBuilder;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders.IscsiLifVserverMetricsBuilder;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders.LifVserverMetricsBuilder;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders.VolumeVserverMetricsBuilder;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer.VServerKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * Performance archiver implementation for the "opm" preset in OnTap.
 */
public class OPMPerformanceArchive extends BasePerformanceArchive<OPMPerformanceArchive.OPMCounterObjects> {

    private static final Logger LOG = LoggerFactory.getLogger(OPMPerformanceArchive.class);
    protected String dataStoreName = "opm";
    protected String processName = "kernel";

    /**
     * List of object names available in the "opm" preset
     */

    public static enum OPMCounterObjects implements CounterObjectsInterface{
        // Allocate state which represents sample period value. This value is in seconds
        // E.g 60 = 1 min , 300 = 5 min
        workload_queue_dblade(DEFAULT_QOS_PERIOD),
        workload_queue_nblade(DEFAULT_QOS_PERIOD),
        resource(DEFAULT_QOS_PERIOD),
        volume,
        cifs,
        lif,
        nic_common,
        system(DEFAULT_QOS_PERIOD),
        aggregate,
        nfsv3,
        nfsv4,
        nfsv4_1,
        fcp_port,
        fcp_lif,
        iscsi_lif,
        disk,
        lun,
        wafl,
        hostadapter,
        processor(DEFAULT_QOS_PERIOD),
        workload_aggr(DEFAULT_QOS_PERIOD);


        private final String onTapObjectName;
        private final long period;
        
        
        public long getPeriod() {
            return period;
        }
        
        private OPMCounterObjects(String onTapObjectName, long period){
        	this.onTapObjectName = onTapObjectName;
        	this.period = period;
        }
        
        private OPMCounterObjects(long period){
        	this.onTapObjectName = this.name();
        	this.period = period;
        }
        
        private OPMCounterObjects(){
        	this.onTapObjectName = this.name();
        	this.period = DEFAULT_PERIOD_VALUE;
        }
        
        public String getOnTapObjectName(){
        	return onTapObjectName;
        }
    }

    /**
     * Builders to process and report metrics from the OPM archive files.
     */
    private final QosServiceCenterMetricsBuilder qosServiceCenterMetricsBuilder;
    private final QosWorkloadQueueDBladeMetricsBuilder qosWorkloadQueueDBladeMetricsBuilder;
    private final QosWorkloadQueueNBladeMetricsBuilder qosWorkloadQueueNBladeMetricsBuilder;
    private final QosWorkloadMetricsBuilder qosWorkloadMetricsBuilder;
    private final VolumeMetricsBuilder volumeMetricsBuilder;
    private final LifMetricsBuilder lifMetricsBuilder;
    private final FcpPortMetricsBuilder fcpPortMetricsBuilder;
    private final ISCSILIFMetricsBuilder iscsilifMetricsBuilder;
    private final NodeMetricsBuilder nodeMetricsBuilder;
    private final NicCommonMetricsBuilder nicCommonMetricsBuilder;
    private final AggregateMetricsBuilder aggregateMetricsBuilder;
    private final DiskMetricsBuilder diskMetricsBuilder;
    private final Nfsv3MetricsBuilder nfSv3MetricsBuilder;
    private final Nfsv4MetricsBuilder nfSv4MetricsBuilder;
    private final Nfsv41MetricsBuilder nfSv41MetricsBuilder;
    private final CifsVserverMetricsBuilder cifsVserverMetricsBuilder;
    private final FcpLifMetricsBuilder fcpLifMetricsBuilder;
    private final LunMetricsBuilder lunMetricsBuilder;
    private final WaflMetricsBuilder waflMetricsBuilder;
    private final LifVserverMetricsBuilder lifVserverMetricsBuilder;
    private final IscsiLifVserverMetricsBuilder iscsiLifVserverMetricsBuilder;
    private final FcpLifVserverMetricsBuilder fcpLifVserverMetricsBuilder;
    private final VolumeVserverMetricsBuilder volumeVserverMetricsBuilder;
    private final ProcessorMetricsBuilder processorMetricsBuilder;
    private final HostadapterMetricsBuilder hostadapterMetricsBuilder;
    private final ClusterMetricsBuilder clusterMetricsBuilder;
    private DataSourceErrorException acquisitionFailedException;
    private final QosWorkloadDetailMetricsBuilder qosWorkloadDetailMetricsBuilder;


    public OPMPerformanceArchive(final NetAppOCIEPerformancePackage netAppOCIEPerformancePackage,
                                 final String archiveFileDownloadDir) {
        super(netAppOCIEPerformancePackage, archiveFileDownloadDir);

        //Initialize Builders
        qosServiceCenterMetricsBuilder = new QosServiceCenterMetricsBuilder(netAppOCIEPerformancePackage);
        qosWorkloadQueueDBladeMetricsBuilder = new QosWorkloadQueueDBladeMetricsBuilder(netAppOCIEPerformancePackage);
        qosWorkloadQueueNBladeMetricsBuilder = new QosWorkloadQueueNBladeMetricsBuilder(netAppOCIEPerformancePackage);
        qosWorkloadMetricsBuilder = new QosWorkloadMetricsBuilder(netAppOCIEPerformancePackage);
        volumeMetricsBuilder = new VolumeMetricsBuilder(netAppOCIEPerformancePackage);
        lifMetricsBuilder = new LifMetricsBuilder(netAppOCIEPerformancePackage);
        nicCommonMetricsBuilder = new NicCommonMetricsBuilder(netAppOCIEPerformancePackage);
        fcpPortMetricsBuilder = new FcpPortMetricsBuilder(netAppOCIEPerformancePackage);
        iscsilifMetricsBuilder = new ISCSILIFMetricsBuilder(netAppOCIEPerformancePackage);
        nodeMetricsBuilder = new NodeMetricsBuilder(netAppOCIEPerformancePackage);
        aggregateMetricsBuilder = new AggregateMetricsBuilder(netAppOCIEPerformancePackage);
        diskMetricsBuilder = new DiskMetricsBuilder(netAppOCIEPerformancePackage);
        nfSv3MetricsBuilder = new Nfsv3MetricsBuilder(netAppOCIEPerformancePackage);
        nfSv4MetricsBuilder = new Nfsv4MetricsBuilder(netAppOCIEPerformancePackage);
        nfSv41MetricsBuilder = new Nfsv41MetricsBuilder(netAppOCIEPerformancePackage);
        fcpLifMetricsBuilder = new FcpLifMetricsBuilder(netAppOCIEPerformancePackage);
        lunMetricsBuilder = new LunMetricsBuilder(netAppOCIEPerformancePackage);
        cifsVserverMetricsBuilder = new CifsVserverMetricsBuilder(netAppOCIEPerformancePackage);
        waflMetricsBuilder = new WaflMetricsBuilder(netAppOCIEPerformancePackage);
        lifVserverMetricsBuilder = new LifVserverMetricsBuilder(netAppOCIEPerformancePackage);
        iscsiLifVserverMetricsBuilder = new IscsiLifVserverMetricsBuilder(netAppOCIEPerformancePackage);
        fcpLifVserverMetricsBuilder = new FcpLifVserverMetricsBuilder(netAppOCIEPerformancePackage);
        processorMetricsBuilder = new ProcessorMetricsBuilder(netAppOCIEPerformancePackage);
        volumeVserverMetricsBuilder = new VolumeVserverMetricsBuilder(netAppOCIEPerformancePackage);
        hostadapterMetricsBuilder = new HostadapterMetricsBuilder(netAppOCIEPerformancePackage);
        clusterMetricsBuilder = new ClusterMetricsBuilder(netAppOCIEPerformancePackage);
        qosWorkloadDetailMetricsBuilder = new QosWorkloadDetailMetricsBuilder(netAppOCIEPerformancePackage);
    }

    
    @Override
    public List<OPMCounterObjects> getCounterObjects() {
        return Collections.unmodifiableList(Arrays.asList(OPMCounterObjects.values()));
    }

    @Override
    public Long determineArchivePoolTime(final List<Long> archiveTimestamps, final Long fileTimeStamp, final OPMCounterObjects counterObject) {
        if(archiveTimestamps == null || archiveTimestamps.size() == 0) {
            return null;
        }
        //for QoS related counters since these counters are polled in 1 min period. Always return the 5th sample time
        if(counterObject.getPeriod()==60)  {

            final int numberOfSamples = archiveTimestamps.size();
            if (numberOfSamples == 5) {
                return Collections.max(archiveTimestamps);
            } else {
                //Check if the last timestamp is the 5th time stamp.
                final Long newestTimeStamp = Collections.max(archiveTimestamps);
                //The newestTimeStamp should be between the 4th minute and the 5th minute.
                final long forthMinFromFileGeneration = fileTimeStamp + (4 * 60 * 1000);
                final long fifthMinFromFileGeneration = fileTimeStamp + (5 * 60 * 1000);
                if (newestTimeStamp >= (forthMinFromFileGeneration) && newestTimeStamp <= (fifthMinFromFileGeneration)) {
                    return newestTimeStamp;
                } else {
                    return null;
                }
            }
        }
        else {    //for non-qos related counters. These counters are polled in 5 min period and usually there is only 1 sample
            final Long newestTimeStamp = Collections.max(archiveTimestamps);
            return newestTimeStamp;
        }

    }

    @Override
    public void processAndReportPerformance(final NetAppDefaultStorageContext context,
                                            final Map<OPMCounterObjects, List<CounterTable>> objectNameToCounterTableList)
            throws DataSourceErrorException {
        Map<Cluster.ClusterKey, Collection<RawSample<QosWorkloadQueueDBladeMeasurements>>> qosDBladeMetrics = null;
        Map<Cluster.ClusterKey, Collection<RawSample<QosWorkloadQueueNBladeMeasurements>>> qosNBladeMetrics = null;
        Map<Cluster.ClusterKey, Collection<RawSample<QosWorkloadDetailMeasurements>>> qosWorkloadDetailMetrics = null;
        Map<VServerKey, Collection<RawSample<VolumeMeasurements>>> volumeMeasurements = null;
        Map<VServerKey, Collection<RawSample<NetworkLifMeasurements>>> lifMeasurements = null;
        Map<VServerKey, Collection<RawSample<LunMeasurements>>> lunMeasurements = null;
        Map<Cluster.ClusterKey, Collection<RawSample<NicMeasurements>>> nicCommonMeasurements = null;
        Map<Cluster.ClusterKey, Collection<RawSample<NodeMeasurements>>> nodeMeasurements = null;
        Map<Cluster.ClusterKey, Collection<RawSample<AggregateMeasurements>>> aggregateMeasurements = null;
        Map<Cluster.ClusterKey, Collection<RawSample<DiskMeasurements>>> diskMeasurements = null;
        Map<Cluster.ClusterKey, Collection<RawSample<Nfsv3Measurements>>> nfsv3Measurements = null;
        Map<Cluster.ClusterKey, Collection<RawSample<Nfsv4Measurements>>> nfsv4Measurements = null;
        Map<Cluster.ClusterKey, Collection<RawSample<Nfsv41Measurements>>>  nfsv41Measurements = null;
        acquisitionFailedException = null;
        boolean isQosDiscoveryEnable = !context.getQosVolumeWorkloadInstanceUUIDs().isEmpty();
        Cluster.ClusterKey clusterKey = context.getKey();
		Map<Cluster.ClusterKey, Collection<RawSample<FcpPortMeasurements>>> fcpPortMetrics = null;
        Map<VServerKey, Collection<RawSample<IscsiLifMeasurements>>> iscsiLifMetrics = null;
        Map<VServerKey, Collection<RawSample<FcpLifMeasurements>>> fcpLifMetrics = null;

        Map<Cluster.ClusterKey, Collection<RawSample<CifsVserverMeasurements>>> cifsVserverMetrics = null;
        Map<ClusterKey, Collection<RawSample<WaflMeasurements>>> waflMeasurements = null;
        Map<ClusterKey, Collection<RawSample<ProcessorMeasurements>>> processorMeasurements = null;

        Map<Cluster.ClusterKey, Collection<RawSample<HostadapterMeasurements>>> hostadapterMeasurements = null;
        // Process Counter Data
        // if there is a DataSourceErrorException, we should fail poll only after the previous value scratchPads have all been cleared
        if(isQosDiscoveryEnable){
        	try{
        		final List<CounterTable> workloadQueueDBlade = objectNameToCounterTableList.get(OPMCounterObjects.workload_queue_dblade);
        		qosDBladeMetrics = qosWorkloadQueueDBladeMetricsBuilder.processCounterData(context, workloadQueueDBlade);
        		qosWorkloadQueueDBladeMetricsBuilder.reportPerformance(clusterKey, qosDBladeMetrics);
        	}catch(DataSourceErrorException dsee){
        		clearPreviousValues(qosWorkloadQueueDBladeMetricsBuilder, dsee);
        	}
        	try{
        		final List<CounterTable> workloadQueueNBlade = objectNameToCounterTableList.get(OPMCounterObjects.workload_queue_nblade);
        		qosNBladeMetrics = qosWorkloadQueueNBladeMetricsBuilder.processCounterData(context, workloadQueueNBlade);
        		qosWorkloadQueueNBladeMetricsBuilder.reportPerformance(clusterKey, qosNBladeMetrics);
        	}catch(DataSourceErrorException dsee){
        		clearPreviousValues(qosWorkloadQueueNBladeMetricsBuilder, dsee);
        	}
        	try{
        		final List<CounterTable> resourceCounterData = objectNameToCounterTableList.get(OPMCounterObjects.resource);
        		if(qosDBladeMetrics != null || qosDBladeMetrics != null){
        			Map<Cluster.ClusterKey, Collection<RawSample<QosServiceCenterMeasurements>>> qosServiceCenterMetrics =
        					qosServiceCenterMetricsBuilder.processCounterData(context, resourceCounterData, qosDBladeMetrics, qosNBladeMetrics);
        			qosServiceCenterMetricsBuilder.reportPerformance(clusterKey, qosServiceCenterMetrics);
        		}
        	}catch(DataSourceErrorException dsee){
        		clearPreviousValues(qosServiceCenterMetricsBuilder, dsee);
        	}

            //QosWorkloadDetail is only available from FS.1
            if(context.isGreaterThanFullSteam0()) {
                try{
                    final List<CounterTable> workloadDetailData = objectNameToCounterTableList.get(OPMCounterObjects.workload_aggr);
                    qosWorkloadDetailMetrics = qosWorkloadDetailMetricsBuilder.processCounterData(context, workloadDetailData);
                    qosWorkloadDetailMetricsBuilder.reportPerformance(clusterKey, qosWorkloadDetailMetrics);

                }catch(DataSourceErrorException dsee){
                    clearPreviousValues(qosWorkloadDetailMetricsBuilder, dsee);
                }
            }
        	
            final Map<Cluster.ClusterKey, Collection<RawSample<QosVolumeWorkloadMeasurements>>> qosWorkloadMetrics =
                     qosWorkloadMetricsBuilder.processCounterData(context, qosWorkloadQueueDBladeMetricsBuilder.getScratchPad(),
                             qosWorkloadQueueNBladeMetricsBuilder.getScratchPad());
            qosWorkloadMetricsBuilder.reportPerformance(qosWorkloadMetrics);
        }

        // Maintain order, as some of the perf Metrics builder depend on others using global scratchPad

        try {
        	final List<CounterTable> volumeCounterData = objectNameToCounterTableList.get(OPMCounterObjects.volume);
        	volumeMeasurements = volumeMetricsBuilder.processCounterData(context, volumeCounterData);
        	volumeMetricsBuilder.reportPerformance(clusterKey, volumeMeasurements);
        }catch(DataSourceErrorException dsee){
        	clearPreviousValues(volumeMetricsBuilder, dsee);
        }

        try{
        	final List<CounterTable> lifCounterData = objectNameToCounterTableList.get(OPMCounterObjects.lif);
        	lifMeasurements = lifMetricsBuilder.processCounterData(context, lifCounterData);
        	lifMetricsBuilder.reportPerformance(clusterKey, lifMeasurements);
        }catch(DataSourceErrorException dsee){
        	clearPreviousValues(lifMetricsBuilder, dsee);
        }

        try{
        	final List<CounterTable> lunCounterData = objectNameToCounterTableList.get(OPMCounterObjects.lun);
        	lunMeasurements = lunMetricsBuilder.processCounterData(context, lunCounterData);
        	lunMetricsBuilder.reportPerformance(clusterKey, lunMeasurements);
        }catch(DataSourceErrorException dsee){
        	clearPreviousValues(lunMetricsBuilder, dsee);
        }
        
        //Cifs
        try{
            final List<CounterTable> cifsCounterData = objectNameToCounterTableList.get(OPMCounterObjects.cifs);
            cifsVserverMetrics = cifsVserverMetricsBuilder.processCounterData(context, cifsCounterData);
            cifsVserverMetricsBuilder.reportPerformance(clusterKey, cifsVserverMetrics);
        }catch(DataSourceErrorException dsee){
            clearPreviousValues(cifsVserverMetricsBuilder, dsee);
        }

        //nic_common
	    try{
        	final List<CounterTable> nicCommonCounterData = objectNameToCounterTableList.get(OPMCounterObjects.nic_common);
        	nicCommonMeasurements = nicCommonMetricsBuilder.processCounterData(context, nicCommonCounterData);
        	nicCommonMetricsBuilder.reportPerformance(clusterKey, nicCommonMeasurements);
        }catch(DataSourceErrorException dsee){
        	clearPreviousValues(nicCommonMetricsBuilder, dsee);
        }

        // iscsi_lif
        try{
            final List<CounterTable> iscsiLifCounterData = objectNameToCounterTableList.get(OPMCounterObjects.iscsi_lif);
            iscsiLifMetrics = iscsilifMetricsBuilder.processCounterData(context, iscsiLifCounterData);
            iscsilifMetricsBuilder.reportPerformance(clusterKey, iscsiLifMetrics);
        } catch (DataSourceErrorException dsee) {
            clearPreviousValues(iscsilifMetricsBuilder, dsee);
        }

        // fcp_lif
        try{
            final List<CounterTable> fcpLifCounterData = objectNameToCounterTableList.get(OPMCounterObjects.fcp_lif);
            fcpLifMetrics = fcpLifMetricsBuilder.processCounterData(context, fcpLifCounterData);
            fcpLifMetricsBuilder.reportPerformance(clusterKey, fcpLifMetrics);
        } catch (DataSourceErrorException dsee) {
            clearPreviousValues(fcpLifMetricsBuilder, dsee);
        }

        // Fcp_port
        // This is dependent on the fcp_lip cached in globalScratchPad
        try {
            final List<CounterTable> fcpPortCounterData = objectNameToCounterTableList.get(OPMCounterObjects.fcp_port);
            fcpPortMetrics = fcpPortMetricsBuilder.processCounterData(context, fcpPortCounterData);
            fcpPortMetricsBuilder.reportPerformance(clusterKey, fcpPortMetrics);
        } catch (DataSourceErrorException dsee) {
            clearPreviousValues(fcpPortMetricsBuilder, dsee);
        }

        // lif_vserver
        try{
            lifVserverMetricsBuilder.getAndReportPerformance(context, null);
        }catch(DataSourceErrorException dsee){
            LOG.error("Unable to get and report lifVserver from archiver!", dsee);
        }

        // iscsilif_vserver
        try{
            iscsiLifVserverMetricsBuilder.getAndReportPerformance(context, null);
        }catch(DataSourceErrorException dsee){
            LOG.error("Unable to get and report iscsiLifVserver from archiver!", dsee);
        }

        // fcplif_vserver
        try{
            fcpLifVserverMetricsBuilder.getAndReportPerformance(context, null);
        }catch(DataSourceErrorException dsee){
            LOG.error("Unable to get and report fcpLifVserver from archiver!", dsee);
        }

        // volume_vserver
        try{
            volumeVserverMetricsBuilder.getAndReportPerformance(context, null);
        }catch(DataSourceErrorException dsee){
            LOG.error("Unable to get and report iscsiLifVserver from archiver!", dsee);
        }
        

        try{
            final List<CounterTable> nfsv3CounterData = objectNameToCounterTableList.get(OPMCounterObjects.nfsv3);
            nfsv3Measurements = nfSv3MetricsBuilder.processCounterData(context, nfsv3CounterData);
            nfSv3MetricsBuilder.reportPerformance(clusterKey, nfsv3Measurements);
        }catch(DataSourceErrorException dsee){
            clearPreviousValues(nfSv3MetricsBuilder, dsee);
        }


        try{
            final List<CounterTable> nfsv4CounterData = objectNameToCounterTableList.get(OPMCounterObjects.nfsv4);
            nfsv4Measurements = nfSv4MetricsBuilder.processCounterData(context, nfsv4CounterData);
            nfSv4MetricsBuilder.reportPerformance(clusterKey, nfsv4Measurements);
        }catch(DataSourceErrorException dsee){
            clearPreviousValues(nfSv4MetricsBuilder, dsee);
        }

        try{
            final List<CounterTable> nfsv41CounterData = objectNameToCounterTableList.get(OPMCounterObjects.nfsv4_1);
            nfsv41Measurements = nfSv41MetricsBuilder.processCounterData(context, nfsv41CounterData);
            nfSv41MetricsBuilder.reportPerformance(clusterKey, nfsv41Measurements);
        }catch(DataSourceErrorException dsee){
            clearPreviousValues(nfSv41MetricsBuilder, dsee);
        }


        // wafl, processor
        if(context.isGreaterThanFullSteam0()) {
        	try{
        		final List<CounterTable> waflCounterData = objectNameToCounterTableList.get(OPMCounterObjects.wafl);
        		waflMeasurements = waflMetricsBuilder.processCounterData(context, waflCounterData);
        		waflMetricsBuilder.reportPerformance(clusterKey, waflMeasurements);
        	}catch(DataSourceErrorException dsee){
        		clearPreviousValues(waflMetricsBuilder, dsee);
        	}
        	
        	try{
        		final List<CounterTable> processorCounterData = objectNameToCounterTableList.get(OPMCounterObjects.processor);
        		processorMeasurements = processorMetricsBuilder.processCounterData(context, processorCounterData);
        		processorMetricsBuilder.reportPerformance(clusterKey, processorMeasurements);
        	}catch(DataSourceErrorException dsee){
        		clearPreviousValues(processorMetricsBuilder, dsee);
        	}
        	
            //Disk
            try{
            	final List<CounterTable> diskCounterData = objectNameToCounterTableList.get(OPMCounterObjects.disk);
            	diskMeasurements = diskMetricsBuilder.processCounterData(context, diskCounterData);
            	diskMetricsBuilder.reportPerformance(clusterKey, diskMeasurements);
            }catch(DataSourceErrorException dsee){
            	clearPreviousValues(diskMetricsBuilder, dsee);
            }

            //Aggregate synthetic counters are dependent on disk counters. Therefore move AggregateBuilder after DiskBuilder.
            // Aggregate
            try{
                final List<CounterTable> aggregateCounterData = objectNameToCounterTableList.get(OPMCounterObjects.aggregate);
                aggregateMeasurements = aggregateMetricsBuilder.processCounterData(context, aggregateCounterData);
                aggregateMetricsBuilder.reportPerformance(clusterKey, aggregateMeasurements);
            }catch(DataSourceErrorException dsee){
                clearPreviousValues(aggregateMetricsBuilder, dsee);

            }
            
            //hostadapter
    	    try{
            	final List<CounterTable> hostadapterCommonCounterData = objectNameToCounterTableList.get(OPMCounterObjects.hostadapter);
            	hostadapterMeasurements = hostadapterMetricsBuilder.processCounterData(context, hostadapterCommonCounterData);
            	hostadapterMetricsBuilder.reportPerformance(clusterKey, hostadapterMeasurements);
            }catch(DataSourceErrorException dsee){
            	clearPreviousValues(hostadapterMetricsBuilder, dsee);
            }

            //wafl, aggregate, disk and processor should always be before nodeBuilder, since nodeMetricsBuilder uses these scratchPads.
        	//Node
            try{
            	final List<CounterTable> nodeCounterData = objectNameToCounterTableList.get(OPMCounterObjects.system);
            	nodeMeasurements = nodeMetricsBuilder.processCounterData(context, nodeCounterData);
            	nodeMetricsBuilder.reportPerformance(clusterKey, nodeMeasurements);
            }catch(DataSourceErrorException dsee){
            	clearPreviousValues(nodeMetricsBuilder, dsee);
            }

            
          //cluster
            final Map<Cluster.ClusterKey, Collection<RawSample<ClusterMeasurements>>> clusterMetrics =
                    clusterMetricsBuilder.processCounterData(context, nodeMeasurements);
            clusterMetricsBuilder.reportPerformance(clusterMetrics);

        }
        
        
        if(acquisitionFailedException != null){
    		throw acquisitionFailedException;
    	}
    }


    public String getDataStoreName() {
        return dataStoreName;
    }

    public String getProcessName() {
        return processName;
    }


    /**
     * Given a archiver metrics builder, clears the previous value scratchPads and remembers the error that caused failure of acquisition.
     * @param archiverBuilder
     * @param dsee
     */
    private void clearPreviousValues(ArchiveMetricsBuilderBase archiverBuilder,  DataSourceErrorException dsee){
        // we need to remember only the first thrown Exception as that is the root cause error
        // this exception is remembered until all performance previous value scratchPads are cleared and then we fail the poll against this remembered Exception.
        if(acquisitionFailedException == null){
            acquisitionFailedException = dsee;
        }
        // The current poll has failed. We do not want to carry forward stale counter values from previous poll and deviate from the 5-minute sample norm.
        // This is done so that the delta and rated counter values are consistently calculated for uniform time intervals.
        archiverBuilder.clearPreviousValues();
        LOG.info("Cleared counter values cached from previous poll for {}.", archiverBuilder.getClass().getName());

    }

    /**
     * Clears off the cached previous counters values from all performance metric builders.
     * We only need to do this for those builders which read counter values directly from archiver file source and use scratchPads to
     * derive current values (delta from previous values) .
     */
    public void clearAllPreviousValues(){
        qosServiceCenterMetricsBuilder.clearPreviousValues();
        qosWorkloadQueueDBladeMetricsBuilder.clearPreviousValues();
        qosWorkloadQueueNBladeMetricsBuilder.clearPreviousValues();
        volumeMetricsBuilder.clearPreviousValues();
        lifMetricsBuilder.clearPreviousValues();
        cifsVserverMetricsBuilder.clearPreviousValues();
        nicCommonMetricsBuilder.clearPreviousValues();
        nodeMetricsBuilder.clearPreviousValues();
        aggregateMetricsBuilder.clearPreviousValues();
        diskMetricsBuilder.clearPreviousValues();
        nfSv3MetricsBuilder.clearPreviousValues();
        nfSv4MetricsBuilder.clearPreviousValues();
        nfSv41MetricsBuilder.clearPreviousValues();
        fcpPortMetricsBuilder.clearPreviousValues();
        iscsilifMetricsBuilder.clearPreviousValues();
        fcpLifMetricsBuilder.clearPreviousValues();
        lunMetricsBuilder.clearPreviousValues();
        waflMetricsBuilder.clearPreviousValues();
        processorMetricsBuilder.clearPreviousValues();
        hostadapterMetricsBuilder.clearPreviousValues();
        qosWorkloadDetailMetricsBuilder.clearPreviousValues();
    }


	
}
