package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.parser;

import java.util.Arrays;
import java.util.List;

import com.sun.jna.Library;
import com.sun.jna.Pointer;
import com.sun.jna.Structure;
import com.sun.jna.Union;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.ptr.PointerByReference;

/**
 * Native CMAPL library wrapper interface. The native methods in this class are implemented by the cmapl wrapper library
 * (cmaplwrapperlib).
 */
public interface CLibrary extends Library  {

	/**
	 * Represents C enum : cmapl_cnt_data_type_t
	 */
	public static interface CmaplCounterDataType {
        public static final int CMAPL_CNT_4BYTES = 0;
        public static final int CMAPL_CNT_8BYTES = 1;
        public static final int CMAPL_CNT_STRING = 2;
    }

	/**
	 * Represents C Union type : cmapl_cnt_value_t
	 */
	public static class CmaplCntValueT extends Union {

		// C type uint32_t
		public int val_4B;
		// C type uint64_t
		public long val_8B;
		// C type : char*
		public String val_str;
		public CmaplCntValueT() {
			super();
		}

		public CmaplCntValueT(Pointer p) {
			super(p);
			read();
		}

		public static class ByReference extends CmaplCntValueT implements com.sun.jna.Structure.ByReference {
		};
		public static class ByValue extends CmaplCntValueT implements com.sun.jna.Structure.ByValue {
		};
	}

	/**
	 * Represents C type : cmapl_counter_data_t
	 */
	public static class CmaplCounterDataT extends Structure {
		public int value_type;
		// C type : cmapl_cnt_value_t*
		public CmaplCntValueT.ByReference values;
		public int num_values;
		public CmaplCounterDataT() {
			super();
		}

                public CmaplCounterDataT(Pointer p) {
			super(p);
			read();
		}

		protected List<? > getFieldOrder() {
			return Arrays.asList("value_type", "values", "num_values");
		}

                /**
		 * cmapl_cnt_data_type_t C type
		 * @param value_type
		 * @param values
		 * @param num_values
		 */
		public CmaplCounterDataT(int value_type, CmaplCntValueT.ByReference values, int num_values) {
			super();
			this.value_type = value_type;
			this.values = values;
			this.num_values = num_values;
		}

		public static class ByReference extends CmaplCounterDataT implements Structure.ByReference {
		};
		public static class ByValue extends CmaplCounterDataT implements Structure.ByValue {
		};
	}


	/**
	 *	Represents C type : cmapl_wrapper_counter_data_t
	 */
	public static class CmaplWrapperCounterDataT extends Structure {
		public int index;
		// C type : cmapl_counter_data_t*
		public CmaplCounterDataT.ByReference counter_values;
		public int num_of_counters_count;
		public CmaplWrapperCounterDataT() {
			super();
		}
		public CmaplWrapperCounterDataT(Pointer p) {
			super(p);
			read();
		}
		protected List<? > getFieldOrder() {
			return Arrays.asList("index", "counter_values", "num_of_counters_count");
		}
		/**
		 * cmapl_wrapper_counter_data_t C type
		 * @param index
		 * @param counter_values
		 * @param num_of_counters_count
		 */
		public CmaplWrapperCounterDataT(int index, CmaplCounterDataT.ByReference counter_values, int num_of_counters_count) {
			super();
			this.index = index;
			this.counter_values = counter_values;
			this.num_of_counters_count = num_of_counters_count;
		}

		public static class ByReference extends CmaplWrapperCounterDataT implements Structure.ByReference {
		};
		public static class ByValue extends CmaplWrapperCounterDataT implements Structure.ByValue {
		};
	}

	/**
	 *	Represents C type : counter_wrapper_headers
	 */
	public static class CmaplCounterWrapperHeaders extends Structure {
		public int counter_id;
		// C type : char*
		public String counter_name;
		public CmaplCounterWrapperHeaders() {
			super();
		}

		public CmaplCounterWrapperHeaders(Pointer p) {
			super(p);
			read();
		}

		protected List<? > getFieldOrder() {
			return Arrays.asList("counter_id", "counter_name");
		}
		/**
		 * counter_wrapper_headers C type.
		 * @param counter_id
		 * @param counter_name
		 */
		public CmaplCounterWrapperHeaders(int counter_id, String counter_name) {
			super();
			this.counter_id = counter_id;
			this.counter_name = counter_name;
		}
		public static class ByReference extends CmaplCounterWrapperHeaders implements Structure.ByReference {
		};
		public static class ByValue extends CmaplCounterWrapperHeaders implements Structure.ByValue {
		};
	}


	//bool get_timestamp_list(const char* ccma_file_path, uint64_t time_period, char* object, uint64_t **timestamps, uint32_t* num_timestamps, char ** error);
	public boolean get_timestamp_list(String ccma_file_path, long time_period, String current_object, PointerByReference array,
                                      IntByReference size, PointerByReference error);

        //bool write_timestamps_to_file(const char* ccma_file_path, char* output_file_path, uint64_t time_period, char* object);
	public boolean write_timestamps_to_file(String ccma_file_path, String output_file_path, long time_period, String object);

        //free_cmapl_wrapper_timestamps_data(uint64_t** timestamps);
	public void free_cmapl_wrapper_timestamps_data(Pointer timestampListPointer, Pointer error);

	//bool get_counter_values_for_timestamp( const char* ccma_file_path, uint64_t time, uint64_t time_period, char* current_obj_char, cmapl_wrapper_counter_data_t **wrapper_counter_data, uint32_t * counter_data_handle_count, char ** error);
	public boolean get_counter_values_for_timestamp (String ccma_file_path, long time, long time_period, String current_obj_char,
                                                     PointerByReference wrapper_counter_data, IntByReference counter_data_handle_count,
			                                         PointerByReference wrapper_headers, IntByReference header_count, PointerByReference error);

	//bool write_counter_values_for_timestamp_to_file ( const char* ccma_file_path,char *output_file_path, uint64_t time, uint64_t time_period,, char* current_obj_char );
	public boolean write_counter_values_for_timestamp_to_file(String ccma_file_path, String output_file_path, long time,
                                                              long time_period, String current_obj_char);

	//void free_cmapl_wrapper_counter_data (cmapl_wrapper_counter_data_t * counter_data_output, uint32_t instance_count);
	public void free_cmapl_wrapper_counter_data(Pointer counter_data_output, int instance_count, Pointer wrapper_headers,
                                                int header_count, Pointer error);

    //bool get_periods_list(const char* ccma_file_path, char* current_obj, uint64_t** time_period_list, uint32_t* num_periods, char **error);
    public void get_periods_list(String ccma_file_path, String current_obj, PointerByReference time_period_list,
                                 IntByReference num_periods, PointerByReference error);

    //void free_cmapl_wrapper_periods_data(uint64_t * time_period_list, char * error)
    public void free_cmapl_wrapper_periods_data(Pointer time_period_list, Pointer error);
}
