package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.parser;

import java.util.Collections;
import java.util.List;

/**
 * This class models a column of the counter data in a counter row. Each counter column can contain a list of values.
 */
public class CounterColumn {
    private final List<Object> columnData;
    private final String columnName;

    public CounterColumn(final String columnName, final List<Object> counterData) {
        this.columnName = columnName;
        this.columnData = Collections.unmodifiableList(counterData);
    }

    /**
     * This method returns the column value(s). A column can one or more values.
     *
     * @return Column data, a {@link Number} or {@link String} or {@link List<Number>} or {@link List<String>}.
     */
    public Object getColumnData() {
        if ((columnData != null) && (columnData.size() > 0)) {
            return (columnData.size() == 1 ? columnData.get(0) : columnData);
        } else {
            return null;
        }
    }

    /**
     * Returns the column name
     * @return Column Name
     */
    public String getColumnName() {
        return columnName;
    }
}
