package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeAggregate;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModePlex;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeRaidGroup;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Aggregate;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Plex;
import org.apache.log4j.Logger;
import org.jdom.Element;
import java.util.LinkedHashMap;
import java.util.List;

public class AggregateBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(AggregateBuilder.class);

    public AggregateBuilder() {
        super("aggr-get-iter", "aggr-attributes", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"aggr-attributes",}, new String[]{"aggregate-name", "aggregate-uuid",});
            put(new String[]{"aggr-attributes", "nodes",}, new String[]{"node-name",});
            put(new String[]{"aggr-attributes", "aggr-space-attributes",}, new String[]{"hybrid-cache-size-total", "size-total", "size-used", "total-reserved-space", "size-available", "percent-used-capacity",});
            put(new String[]{"aggr-attributes", "aggr-fs-attributes",}, new String[]{"type", "block-type",});
            put(new String[]{"aggr-attributes", "aggr-raid-attributes",}, new String[]{"aggregate-type", "disk-count", "state", "raid-status","raid-type", "is-hybrid", "is-hybrid-enabled", "raid-size", "mirror-status", "has-local-root", "has-partner-root", "uses-shared-disks",});
            put(new String[]{"aggr-attributes", "aggr-raid-attributes", "plexes", "plex-attributes",}, new String[]{"is-online", "plex-name","plex-status","resyncing-percentage"});
            put(new String[]{"aggr-attributes", "aggr-raid-attributes", "plexes", "plex-attributes", "raidgroups", "raidgroup-attributes",}, new String[]{"is-reconstructing", "raidgroup-name",});
            put(new String[]{"aggr-attributes", "aggr-snapshot-attributes",}, new String[]{"size-total", "size-used", "size-available",});
            put(new String[]{"aggr-attributes", "aggr-64bit-upgrade-attributes", "aggr-status-attributes",}, new String[]{"is-64-bit-upgrade-in-progress",});
        }});
    }

    protected void buildIterInstance(final CModeCluster cluster, final Element aggrAttributesElt) {

        // Aggregate attributes
        final String aggrName = BuilderUtil.getChildTextTrim(aggrAttributesElt, "aggregate-name");
        final String aggrUUID = BuilderUtil.getChildTextTrim(aggrAttributesElt, "aggregate-uuid");
        final List<String> nodeNameStrings = BuilderUtil.getListFromGrandChildren(aggrAttributesElt, "nodes", "node-name");
        final Element aggrSpaceElement = aggrAttributesElt.getChild("aggr-space-attributes");
        final Long hybridCacheSizeTotal = BuilderUtil.parseLongFromChild(aggrSpaceElement, "hybrid-cache-size-total");
        final Long sizeTotal = BuilderUtil.parseLongFromChild(aggrSpaceElement, "size-total");
        final Long sizeUsed = BuilderUtil.parseLongFromChild(aggrSpaceElement, "size-used");
        final Long sizeAvailable = BuilderUtil.parseLongFromChild(aggrSpaceElement, "size-available");
        final Long totalReservedSpace = BuilderUtil.parseLongFromChild(aggrSpaceElement, "total-reserved-space");
        final Integer sizeUsedPercent = BuilderUtil.parseIntFromChild(aggrSpaceElement, "percent-used-capacity");

        // FS attributes
        final Element aggrFSAttributesElt = aggrAttributesElt.getChild("aggr-fs-attributes");
        final String aggrTypeVal = BuilderUtil.getChildTextTrim(aggrFSAttributesElt, "type");
        final String blockTypeRaw = BuilderUtil.getChildTextTrim(aggrFSAttributesElt, "block-type");
        final EnumWrapper<Aggregate.BlockType> blockType = new EnumWrapper<Aggregate.BlockType>(blockTypeRaw, Aggregate.BlockType.fromString(blockTypeRaw));

        // RAID attributes
        final Element aggrRAIDAttributesElt = aggrAttributesElt.getChild("aggr-raid-attributes");
        final EnumWrapper<Aggregate.AggregateState> aggrState = BuilderUtil.getEnumWrapperByValue(Aggregate.AggregateState.class, aggrRAIDAttributesElt, "state", Aggregate.AggregateState.NOT_MAPPED);
        final String aggrRAIDStatus = BuilderUtil.getChildTextTrim(aggrRAIDAttributesElt, "raid-status");
        final String aggrRAIDType = BuilderUtil.getChildTextTrim(aggrRAIDAttributesElt, "raid-type");
        final Boolean isHybrid = BuilderUtil.parseBooleanFromChild(aggrRAIDAttributesElt, "is-hybrid");
        final Boolean isHybridEnabled = BuilderUtil.parseBooleanFromChild(aggrRAIDAttributesElt, "is-hybrid-enabled");
        final Long raidSize = BuilderUtil.parseLongFromChild(aggrRAIDAttributesElt, "raid-size");
        final String mirrorStatusRaw = BuilderUtil.getChildTextTrim(aggrRAIDAttributesElt, "mirror-status");
        final EnumWrapper<Aggregate.MirrorStatus> mirrorStatus = new EnumWrapper<Aggregate.MirrorStatus>(mirrorStatusRaw, Aggregate.MirrorStatus.fromString(mirrorStatusRaw));
        final Boolean hasLocalRoot = BuilderUtil.parseBooleanFromChild(aggrRAIDAttributesElt, "has-local-root");
        final Boolean hasPartnerRoot = BuilderUtil.parseBooleanFromChild(aggrRAIDAttributesElt, "has-partner-root");
        final Boolean usesSharedDisks = BuilderUtil.parseBooleanFromChildDefaultFalse(aggrRAIDAttributesElt, "uses-shared-disks");
        final EnumWrapper<Aggregate.AggregateType> aggregateType = BuilderUtil.getEnumWrapperByValue(Aggregate.AggregateType.class, aggrRAIDAttributesElt, "aggregate-type", Aggregate.AggregateType.NOT_MAPPED);


        // snapshot attributes
        final Element aggrSnapshotElt = aggrAttributesElt.getChild("aggr-snapshot-attributes");
        final Long snapshotSizeTotal = BuilderUtil.parseLongFromChild(aggrSnapshotElt, "size-total");
        final Long snapshotSizeUsed = BuilderUtil.parseLongFromChild(aggrSnapshotElt, "size-used");
        final Long sizeSnapshotAvailable = BuilderUtil.parseLongFromChild(aggrSnapshotElt, "size-available");

        // status attributes
        final Element sixtyFourBitUpgradeAttrs = aggrAttributesElt.getChild("aggr-64bit-upgrade-attributes");
        final Boolean is64BitUpgradeInProgress;
        if (sixtyFourBitUpgradeAttrs != null) {
            final Element sixtyFourBitStatusAttrs = sixtyFourBitUpgradeAttrs.getChild("aggr-status-attributes");
            is64BitUpgradeInProgress = BuilderUtil.parseBooleanFromChild(sixtyFourBitStatusAttrs, "is-64-bit-upgrade-in-progress");
        }
        else {
            is64BitUpgradeInProgress = null;
        }

        final CModeAggregate aggregate = cluster.createAggregate(aggrUUID);
        aggregate.setAggrName(aggrName);
        aggregate.setAggrType(aggrTypeVal);
        aggregate.setAggrState(aggrState);
        aggregate.setHybrid(isHybrid);
        aggregate.setHybridEnabled(isHybridEnabled);
        aggregate.setSnapshotSizeTotal(snapshotSizeTotal);
        aggregate.setSnapshotSizeUsed(snapshotSizeUsed);
        aggregate.setSnapShotSizeAvailable(sizeSnapshotAvailable);
        aggregate.setHybridCacheSizeTotal(hybridCacheSizeTotal);
        aggregate.setNodeNameStrings(nodeNameStrings);
        aggregate.setSizeTotal(sizeTotal);
        aggregate.setSizeUsed(sizeUsed);
        aggregate.setSizeAvailable(sizeAvailable);
        aggregate.setSizeUsedPercent(sizeUsedPercent);
        aggregate.setTotalReservedSpace(totalReservedSpace);
        aggregate.setRaidStatus(aggrRAIDStatus);
        aggregate.setRaidType(aggrRAIDType);
        aggregate.setRaidSize(raidSize);
        aggregate.setBlockType(blockType);
        aggregate.setIs64UpgradeInProgress(is64BitUpgradeInProgress);
        aggregate.setMirrorStatus(mirrorStatus);
        aggregate.setHasLocalRoot(hasLocalRoot);
        aggregate.setHasPartnerRoot(hasPartnerRoot);
        aggregate.setUsesSharedDisks(usesSharedDisks);
        aggregate.setAggregateType(aggregateType);
        cluster.getAggregates().add(aggregate);

        // Plexes
        final Element plexElt = aggrRAIDAttributesElt.getChild("plexes");
        if (plexElt != null) {
            @SuppressWarnings({ "unchecked" })
            final List<Element> plexAttributeElts = plexElt.getChildren("plex-attributes");
            for (final Element plexAttributeElt : plexAttributeElts) {
                final String plexName = BuilderUtil.getChildTextTrim(plexAttributeElt, "plex-name");
                final Boolean isPlexOnline = BuilderUtil.parseBooleanFromChild(plexAttributeElt, "is-online");
                final EnumWrapper<Plex.PlexStatus> plexStatus = BuilderUtil.getEnumWrapperByValue(
                        Plex.PlexStatus.class, plexAttributeElt, "plex-status", Plex.PlexStatus.NOT_MAPPED);

                final Integer resyncPercentage = BuilderUtil.parseIntFromChild(plexAttributeElt, "resyncing-percentage");

                final CModePlex plex = aggregate.createPlex(plexName, isPlexOnline);
                plex.setStatus(plexStatus);
                plex.setResyncPercentage(resyncPercentage);
                aggregate.getPlexes().add(plex);

                // RaidGroups
                final Element raidGroupsElt = plexAttributeElt.getChild("raidgroups");
                if (raidGroupsElt != null) {
                    @SuppressWarnings("unchecked")
                    final List<Element> raidGroupElts = raidGroupsElt.getChildren("raidgroup-attributes");
                    for (final Element raidGroupElt : raidGroupElts) {
                        final String raidGroupName = BuilderUtil.getChildTextTrim(raidGroupElt, "raidgroup-name");
                        final Boolean isReconstructing = BuilderUtil.parseBooleanFromChild(raidGroupElt,
                                "is-reconstructing");
                        final CModeRaidGroup raidGroup = plex.createRaidGroup(aggregate.getKey().getUUID(),
                                raidGroupName);
                        raidGroup.setReconstructing(isReconstructing);
                        plex.getRaidGroups().add(raidGroup);
                    }
                }
            }
        }
    }
}
