package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeAggregateSpare;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.LinkedHashMap;

public class AggregateSpareBuilder extends IterBuilderBase {

    private static final Logger logger = Logger.getLogger(AggregateSpareBuilder.class);

    public AggregateSpareBuilder() {
        super("aggr-spare-get-iter", "aggr-spare-disk-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[] {"aggr-spare-disk-info"},
                    new String[] {"disk", "disk-type", "original-owner", "local-usable-data-size", "local-usable-root-size"});
        }});
    }

    protected void buildIterInstance(final CModeCluster cluster, final Element element) {
        CModeAggregateSpare aggrSpare = create(cluster, element);
        if (aggrSpare != null) {
            cluster.getAggregateSpares().add(aggrSpare);
        }
    }

    private CModeAggregateSpare create(CModeCluster cluster, Element e) {
        Long usableDataSize = BuilderUtil.parseLongFromChild(e, "local-usable-data-size");
        Long usableRootSize = BuilderUtil.parseLongFromChild(e, "local-usable-root-size");

        if ((usableDataSize == null) && (usableRootSize == null)) {
            // the data provides no insight and is not worth keeping
            return null;
        }

        String diskName = BuilderUtil.getChildText(e, "disk");
        String diskType = BuilderUtil.getChildText(e, "disk-type");
        String originalOwner = BuilderUtil.getChildText(e, "original-owner");

        CModeAggregateSpare aggregateSpare = new CModeAggregateSpare(cluster)
            .setDiskName(diskName)
            .setDiskType(diskType)
            .setOriginalOwner(originalOwner)
            .setUsableDataSize(usableDataSize)
            .setUsableRootSize(usableRootSize);

        return aggregateSpare;
    }
}
