package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.*;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.RaidGroup;
import org.jdom.Element;

import java.util.LinkedHashMap;
import java.util.List;

/**
 * This class was originally named  DiskAggregateRelationshipBuilder. This class now, build CModeDiskAggregateRelationship
 * and CModeRaidGroup
 */
public class AggregateStatusInfoBuilder extends IterBuilderBase {

    protected AggregateStatusInfoBuilder() {
        super("aggr-status-get-iter", "aggr-status-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"aggr-status-info",}, new String[]{"aggregate-uuid",});
            put(new String[]{"aggr-status-info", "aggr-plex-list", "aggr-plex-info",}, new String[]{"plex-name",});
            put(new String[]{"aggr-status-info", "aggr-plex-list", "aggr-plex-info", "aggr-raidgroup-list", "aggr-raidgroup-info", "aggr-raidgroup-entity-list", "aggr-raidgroup-entity-info", }, new String[]{"rg-raidtype",});
            put(new String[]{"aggr-status-info", "aggr-plex-list", "aggr-plex-info", "aggr-raidgroup-list", "aggr-raidgroup-info", "aggr-disk-list", }, new String[]{"disk","is-cache",});
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element aggregateStatusInfo) {
        final String aggregateUUID = BuilderUtil.getChildTextTrim(aggregateStatusInfo, "aggregate-uuid");
        final Element plexList = aggregateStatusInfo.getChild("aggr-plex-list");
        if (plexList != null) {
            for (final Element plexInfo : (List<Element>) plexList.getChildren()) {
                String plexName = BuilderUtil.getChildTextTrim(plexInfo, "plex-name");
                final Element raidGroupList = plexInfo.getChild("aggr-raidgroup-list");
                if (raidGroupList != null) {
                    for (final Element raidGroupInfo : (List<Element>) raidGroupList.getChildren()) {
                        final Element diskList = raidGroupInfo.getChild("aggr-disk-list");
                        if (diskList != null) {
                            for (final Element diskInfo : (List<Element>) diskList.getChildren()) {
                                final String diskName = BuilderUtil.getChildTextTrim(diskInfo, "disk");
                                final Boolean isCache = BuilderUtil.parseBooleanFromChild(diskInfo, "is-cache");
                                final CModeDiskAggregateRelationship cModeDiskAggregateRelationship =
                                        cluster.createDiskAggregateRelationship(diskName, aggregateUUID);
                                cModeDiskAggregateRelationship.setIsCache(isCache);
                                cluster.getDiskAggregateRelationships().add(cModeDiskAggregateRelationship);
                            }
                        }
                        final CModeAggregate cModeAggregate = cluster.getAggregates().get(new UUIDKey("aggregate", aggregateUUID));
                        if(cModeAggregate != null) {
                            final CModePlex cModePlex = cModeAggregate.getPlexes().get(new CModePlex.PlexKey(aggregateUUID,plexName));
                            if(cModePlex != null) {
                                final Element raidGroupEntityList = raidGroupInfo.getChild("aggr-raidgroup-entity-list");
                                if(raidGroupEntityList != null) {
                                    for (final Element raidGroupEntityInfo : (List<Element>) raidGroupEntityList.getChildren()) {
                                        final String raidGroupName = BuilderUtil.getChildTextTrim(raidGroupEntityInfo, "raidgroup");
                                        final CModeRaidGroup cModeRaidGroup = cModePlex.getRaidGroups().get(new CModeRaidGroup.RaidGroupKey(aggregateUUID, raidGroupName));
                                        if(cModeRaidGroup != null){
                                            EnumWrapper<RaidGroup.RaidType> raidTypeEnumWrapper =
                                                    BuilderUtil.getEnumWrapperByValue(RaidGroup.RaidType.class,
                                                            raidGroupEntityInfo, "rg-raidtype", RaidGroup.RaidType.NOT_MAPPED);
                                            cModeRaidGroup.setRaidType(raidTypeEnumWrapper);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
