package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeClusterPeer;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.ClusterPeer;
import org.jdom.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.LinkedHashMap;
import java.util.List;

public class ClusterPeerBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = LoggerFactory.getLogger(ClusterPeerBuilder.class);

    public ClusterPeerBuilder() {
        super("cluster-peer-get-iter", "cluster-peer-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"cluster-peer-info",}, new String[]{"address-family", "auth-status-admin",
                    "auth-status-operational", "availability", "cluster-name", "cluster-uuid",
                    "ipspace-name", "is-cluster-healthy", "serial-number", "unreachable-local-nodes",});
            put(new String[]{"cluster-peer-info", "active-addresses",}, new String[]{"remote-inet-address",});
            put(new String[]{"cluster-peer-info", "peer-addresses",}, new String[]{"remote-inet-address",});
            put(new String[]{"cluster-peer-info", "remote-cluster-nodes",}, new String[]{"peer-node-name",});
            put(new String[]{"cluster-peer-info", "unreachable-local-nodes",}, new String[]{"peer-node-name",});
        }});
    }

    protected void buildIterInstance(final CModeCluster cluster, final Element clusterPeerInfo) {

        final String remoteClusterName = BuilderUtil.getChildTextTrim(clusterPeerInfo, "cluster-name");
        final String remoteClusterUUUID = BuilderUtil.getChildTextTrim(clusterPeerInfo, "cluster-uuid");
        final String activeAddresses = BuilderUtil.getListAsCSVString(
                BuilderUtil.getListFromGrandChildren(clusterPeerInfo, "active-addresses", "remote-inet-address"));
        final EnumWrapper<ClusterPeer.AddressFamily> addressFamily = BuilderUtil.getEnumWrapperByValue(
                ClusterPeer.AddressFamily.class, clusterPeerInfo, "address-family", ClusterPeer.AddressFamily.NOT_MAPPED);
        final String authStatusAdmin = BuilderUtil.getChildTextTrim(clusterPeerInfo, "auth-status-admin");
        final EnumWrapper<ClusterPeer.AuthStatusOperational> authStatusOperational = BuilderUtil.getEnumWrapperByValue(
                ClusterPeer.AuthStatusOperational.class, clusterPeerInfo, "auth-status-operational",
                ClusterPeer.AuthStatusOperational.NOT_MAPPED);
        final EnumWrapper<ClusterPeer.Availability> availability = BuilderUtil.getEnumWrapperByValue(
                ClusterPeer.Availability.class, clusterPeerInfo, "availability", ClusterPeer.Availability.NOT_MAPPED);
        final String ipSpaceName = BuilderUtil.getChildTextTrim(clusterPeerInfo, "ipspace-name");
        final Boolean isClusterHealthy = BuilderUtil.parseBooleanFromChild(clusterPeerInfo, "is-cluster-healthy");
        final String peerAddresses = BuilderUtil.getListAsCSVString(
                BuilderUtil.getListFromGrandChildren(clusterPeerInfo, "peer-addresses", "remote-inet-address"));
        final String remoteClusterNodes = BuilderUtil.getListAsCSVString(
                BuilderUtil.getListFromGrandChildren(clusterPeerInfo, "remote-cluster-nodes", "peer-node-name"));
        final String serialNumber = BuilderUtil.getChildTextTrim(clusterPeerInfo, "serial-number");
        final String unreachableLocalNodes = BuilderUtil.getListAsCSVString(
                BuilderUtil.getListFromGrandChildren(clusterPeerInfo, "unreachable-local-nodes", "peer-node-name"));

        final CModeClusterPeer clusterPeer = cluster.createClusterPeer(remoteClusterUUUID);
        clusterPeer.setActiveAddresses(activeAddresses);
        clusterPeer.setAddressFamily(addressFamily);
        clusterPeer.setAuthStatusAdmin(authStatusAdmin);
        clusterPeer.setAuthStatusOperational(authStatusOperational);
        clusterPeer.setAvailability(availability);
        clusterPeer.setRemoteClusterName(remoteClusterName);
        clusterPeer.setIpSpaceName(ipSpaceName);
        clusterPeer.setClusterHealthy(isClusterHealthy);
        clusterPeer.setPeerAddresses(peerAddresses);
        clusterPeer.setRemoteClusterNodes(remoteClusterNodes);
        clusterPeer.setSerialNumber(serialNumber);
        clusterPeer.setUnreachableLocalNodes(unreachableLocalNodes);
        cluster.getClusterPeers().add(clusterPeer);
    }
}
