package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapperList;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeExportPolicy;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeExportRule;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPINames;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import org.apache.log4j.Logger;
import org.jdom.Element;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.children;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.elem;
import static com.onaro.sanscreen.acquisition.netappschema.netappdata.ExportRule.AccessProtocol;
import static com.onaro.sanscreen.acquisition.netappschema.netappdata.ExportRule.SecurityFlavor;

public class ExportsBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(ExportsBuilder.class);

    private static final String ZAPI_NAME_EXPORT_POLICY_INFO = "export-policy-info";

    public ExportsBuilder() {
        super("export-policy-get-iter", ZAPI_NAME_EXPORT_POLICY_INFO, getDefaultMaxRecords(),
                elem(ZAPINames.DESIRED_ATTRIBUTES,
                        elem(ZAPI_NAME_EXPORT_POLICY_INFO, children(
                                "policy-id", "policy-name", "vserver"))));
    }

    @Override
    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        super.buildModel(cluster, zapiConnection);
        new ExportRuleBuilder().buildModel(cluster, zapiConnection);
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element policyInfoElt) {
        final String policyName = BuilderUtil.getChildTextTrim(policyInfoElt, "policy-name");
        final Long policyID = BuilderUtil.parseLongFromChildDefault(policyInfoElt, "policy-id", -1);
        final String vServerName = BuilderUtil.getChildTextTrim(policyInfoElt, "vserver");

        final CModeExportPolicy policy = cluster.createExportPolicy(vServerName, policyID);
        policy.setPolicyName(policyName);
        cluster.getExportPolicies().add(policy);
    }

    static final private class ExportRuleBuilder extends IterBuilderBase {

        private static final String ZAPI_NAME_EXPORT_RULE_INFO = "export-rule-info";

        public ExportRuleBuilder() {
            super("export-rule-get-iter", ZAPI_NAME_EXPORT_RULE_INFO, getDefaultMaxRecords(),
                    elem(ZAPINames.DESIRED_ATTRIBUTES,
                            elem(ZAPI_NAME_EXPORT_RULE_INFO, children(
                                    "client-match", "policy-name", "rule-index", "vserver-name",
                                    elem("protocol",
                                            elem("access-protocol")),
                                    elem("ro-rule",
                                            elem("security-flavor")),
                                    elem("rw-rule",
                                            elem("security-flavor")),
                                    elem("super-user-security",
                                            elem("security-flavor"))))));
        }

        @Override
        protected void buildIterInstance(final CModeCluster cluster, final Element ruleInfoElt) {
            final String policyName = BuilderUtil.getChildTextTrim(ruleInfoElt, "policy-name");
            final Integer ruleIndex = BuilderUtil.parseIntFromChild(ruleInfoElt, "rule-index");
            final String vServerName = BuilderUtil.getChildTextTrim(ruleInfoElt, "vserver-name");
            final String clientMatch = BuilderUtil.getChildTextTrim(ruleInfoElt, "client-match");
            final EnumWrapperList<AccessProtocol> protocol = BuilderUtil.getEnumWrapperListByValue(
                    AccessProtocol.class, ruleInfoElt, "protocol", "access-protocol");
            final EnumWrapperList<SecurityFlavor> roRule = BuilderUtil.getEnumWrapperListByValue(
                    SecurityFlavor.class, ruleInfoElt, "ro-rule", "security-flavor");
            final EnumWrapperList<SecurityFlavor> rwRule = BuilderUtil.getEnumWrapperListByValue(
                    SecurityFlavor.class, ruleInfoElt, "rw-rule", "security-flavor");
            final EnumWrapperList<SecurityFlavor> superUserSecurity = BuilderUtil.getEnumWrapperListByValue(
                    SecurityFlavor.class, ruleInfoElt, "super-user-security", "security-flavor");
            final CModeExportRule rule = cluster.createExportRule(vServerName, policyName, ruleIndex);
            rule.setClientMatch(clientMatch);
            rule.getProtocol().set(protocol);
            rule.getRoRule().set(roRule);
            rule.getRwRule().set(rwRule);
            rule.getSuperUserSecurity().set(superUserSecurity);
            cluster.getExportRules().add(rule);
        }
    }
}
