package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCronJobSchedule;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeJobSchedule;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeJobScheduleInterval;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.JobSchedule;
import org.jdom.Element;

import java.util.LinkedHashMap;
import java.util.List;

public class JobScheduleBuilder extends IterBuilderBase {

    @Override
    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        super.buildModel(cluster, zapiConnection);                                  // job-schedule-get-iter
        new CronJobScheduleBuilder().buildModel(cluster, zapiConnection);           // calls job-schedule-cron-get-iter
        new JobScheduleIntervalBuilder().buildModel(cluster, zapiConnection);       // calls job-schedule-interval-get-iter
    }

    protected JobScheduleBuilder() {
        super("job-schedule-get-iter", "job-schedule-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"job-schedule-info",}, new String[]{"job-schedule-description", "job-schedule-name", "job-schedule-type",});
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element jobScheduleInfo) {
        final String jobScheduleDescription = BuilderUtil.getChildTextTrim(jobScheduleInfo, "job-schedule-description");
        final String jobScheduleName = BuilderUtil.getChildTextTrim(jobScheduleInfo, "job-schedule-name");
        final EnumWrapper<JobSchedule.ScheduleType> jobScheduleType = BuilderUtil.getEnumWrapperByValue(JobSchedule.ScheduleType.class, jobScheduleInfo, "job-schedule-type", JobSchedule.ScheduleType.NOT_MAPPED);
        final CModeJobSchedule jobSchedule = cluster.createJobSchedule(jobScheduleName);
        jobSchedule.setJobScheduleType(jobScheduleType);
        jobSchedule.setDescription(jobScheduleDescription);
        cluster.getJobSchedules().add(jobSchedule);
    }

    private static final class CronJobScheduleBuilder extends IterBuilderBase {
        private CronJobScheduleBuilder() {
            super("job-schedule-cron-get-iter", "job-schedule-cron-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
                put(new String[]{"job-schedule-cron-info", "job-schedule-cron-day"}, new String[]{"cron-day",});
                put(new String[]{"job-schedule-cron-info", "job-schedule-cron-day-of-week"}, new String[]{"cron-day-of-week",});
                put(new String[]{"job-schedule-cron-info", "job-schedule-cron-hour"}, new String[]{"cron-hour",});
                put(new String[]{"job-schedule-cron-info", "job-schedule-cron-minute"}, new String[]{"cron-minute",});
                put(new String[]{"job-schedule-cron-info", "job-schedule-cron-month"}, new String[]{"cron-month",});
                put(new String[]{"job-schedule-cron-info",}, new String[]{"job-schedule-name",});
            }});
        }

        protected void buildIterInstance(final CModeCluster cluster, final Element cronInfoElt) {
            final List<String> cronDays = BuilderUtil.getListFromGrandChildren(cronInfoElt, "job-schedule-cron-day", "cron-day-of-month");
            final String cronDaysCSV = BuilderUtil.getListAsCSVString(cronDays);
            final List<String> cronDaysOfWeek = BuilderUtil.getListFromGrandChildren(cronInfoElt, "job-schedule-cron-day-of-week", "cron-day-of-week");
            final String cronDaysOfWeekCSV = BuilderUtil.getListAsCSVString(cronDaysOfWeek);
            final List<String> cronHours = BuilderUtil.getListFromGrandChildren(cronInfoElt, "job-schedule-cron-hour", "cron-hour");
            final String cronHoursCSV = BuilderUtil.getListAsCSVString(cronHours);
            final List<String> cronMinutes = BuilderUtil.getListFromGrandChildren(cronInfoElt, "job-schedule-cron-minute", "cron-minute");
            final String cronMinutesCSV = BuilderUtil.getListAsCSVString(cronMinutes);
            final List<String> cronMonths = BuilderUtil.getListFromGrandChildren(cronInfoElt, "job-schedule-cron-month", "cron-month");
            final String cronMonthsCSV = BuilderUtil.getListAsCSVString(cronMonths);
            final String jobScheduleName = BuilderUtil.getChildTextTrim(cronInfoElt, "job-schedule-name");
            final CModeCronJobSchedule cronJobSchedule = cluster.createCronJobSchedule(jobScheduleName);
            cronJobSchedule.setCronDayOfMonthCSV(cronDaysCSV);
            cronJobSchedule.setCronDayOfWeekCSV(cronDaysOfWeekCSV);
            cronJobSchedule.setCronHourCSV(cronHoursCSV);
            cronJobSchedule.setCronMinuteCSV(cronMinutesCSV);
            cronJobSchedule.setCronMonthCSV(cronMonthsCSV);
            cluster.getCronJobSchedules().add(cronJobSchedule);
        }
    }


    private static final class JobScheduleIntervalBuilder extends IterBuilderBase {
        private JobScheduleIntervalBuilder() {
            super("job-schedule-interval-get-iter", "job-schedule-interval-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
                put(new String[]{"job-schedule-interval-info",}, new String[]{"job-schedule-interval-days", "job-schedule-interval-hours", "job-schedule-interval-minutes", "job-schedule-interval-seconds", "job-schedule-name"});
            }});
        }

        protected void buildIterInstance(final CModeCluster cluster, final Element jobScheduleIntervalElt) {
            final Long intervalDays = BuilderUtil.parseLongFromChild(jobScheduleIntervalElt, "job-schedule-interval-days");
            final Long intervalHours = BuilderUtil.parseLongFromChild(jobScheduleIntervalElt, "job-schedule-interval-hours");
            final Long intervalMinutes = BuilderUtil.parseLongFromChild(jobScheduleIntervalElt, "job-schedule-interval-minutes");
            final Long intervalSeconds = BuilderUtil.parseLongFromChild(jobScheduleIntervalElt, "job-schedule-interval-seconds");
            final String jobScheduleName = BuilderUtil.getChildTextTrim(jobScheduleIntervalElt, "job-schedule-name");
            final CModeJobScheduleInterval jobScheduleInterval = cluster.createJobScheduleInterval(jobScheduleName);
            jobScheduleInterval.setDays(intervalDays);
            jobScheduleInterval.setHours(intervalHours);
            jobScheduleInterval.setMinutes(intervalMinutes);
            jobScheduleInterval.setSeconds(intervalSeconds);
            cluster.getJobScheduleIntervals().add(jobScheduleInterval);
        }
    }
}
