package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeLUN;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPINames;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.LUN;
import org.apache.log4j.Logger;
import org.jdom.Element;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.children;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.elem;

public class LUNBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(LUNBuilder.class);

    private static final String ZAPI_NAME_LUN_INFO = "lun-info";

    public LUNBuilder() {
        super("lun-get-iter", ZAPI_NAME_LUN_INFO, getDefaultMaxRecords(),
                elem(ZAPINames.DESIRED_ATTRIBUTES,
                        elem(ZAPI_NAME_LUN_INFO, children(
                                "alignment", "comment", "is-space-alloc-enabled", "is-space-reservation-enabled", "mapped", "multiprotocol-type", "online", "path", "prefix-size", "qtree", "serial-number", "size", "size-used", "suffix-size", "uuid", "volume", "vserver", "qos-policy-group", "class"))));
    }

    protected void buildIterInstance(final CModeCluster cluster, final Element lunInfoElt) {
        final EnumWrapper<LUN.Alignment> alignment = BuilderUtil.getEnumWrapperByValue(LUN.Alignment.class, lunInfoElt, "alignment", LUN.Alignment.NOT_MAPPED);
        final String comment = BuilderUtil.getChildTextTrim(lunInfoElt, "comment");
        final Boolean isSpaceAllocEnabled = BuilderUtil.parseBooleanFromChild(lunInfoElt, "is-space-alloc-enabled");
        final Boolean isSpaceReservationEnabled = BuilderUtil.parseBooleanFromChild(lunInfoElt, "is-space-reservation-enabled");
        final Boolean isMapped = BuilderUtil.parseBooleanFromChild(lunInfoElt, "mapped");
        final String multiProtocolType = BuilderUtil.getChildTextTrim(lunInfoElt, "multiprotocol-type");
        final Boolean online = BuilderUtil.parseBooleanFromChild(lunInfoElt, "online");
        final String path = BuilderUtil.getChildTextTrim(lunInfoElt, "path");
        final Long prefixSize = BuilderUtil.parseLongFromChild(lunInfoElt, "prefix-size");
        final String qtreeName = BuilderUtil.getChildTextTrim(lunInfoElt, "qtree");
        final String serialNumber = BuilderUtil.getChildTextTrim(lunInfoElt, "serial-number");
        final Long size = BuilderUtil.parseLongFromChild(lunInfoElt, "size");
        final Long sizeUsed = BuilderUtil.parseLongFromChild(lunInfoElt, "size-used");
        final Long suffixSize = BuilderUtil.parseLongFromChild(lunInfoElt, "suffix-size");
        final String uuid = BuilderUtil.getChildTextTrim(lunInfoElt, "uuid");
        final String volumeName = BuilderUtil.getChildTextTrim(lunInfoElt, "volume");
        final String vServerName = BuilderUtil.getChildTextTrim(lunInfoElt, "vserver");
        final String qosPolicyGroupName = BuilderUtil.getChildTextTrim(lunInfoElt, "qos-policy-group");
        final EnumWrapper<LUN.LUNClass> lunClass = BuilderUtil.getEnumWrapperByValue(LUN.LUNClass.class, lunInfoElt, "class", LUN.LUNClass.NOT_MAPPED);
        final CModeLUN lun = cluster.createLUN(uuid);
        lun.setAlignment(alignment);
        lun.setSpaceAllocEnabled(isSpaceAllocEnabled);
        lun.setSpaceReservationEnabled(isSpaceReservationEnabled);
        lun.setMapped(isMapped);
        lun.setMultiprotocolTypeString(multiProtocolType);
        lun.setOnline(online);
        lun.setPath(path);
        lun.setPrefixSize(prefixSize);
        lun.setQTreeName(qtreeName);
        lun.setSerialNumber(serialNumber);
        lun.setSize(size);
        lun.setSizeUsed(sizeUsed);
        lun.setVolumeName(volumeName);
        lun.setVServerName(vServerName);
        lun.setComment(comment);
        lun.setSuffixSize(suffixSize);
        lun.setQosPolicyGroupName(qosPolicyGroupName);
        lun.setLunClass(lunClass);
        cluster.getLUNs().add(lun);
    }
}
