package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeLicenseInterim;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import netapp.manage.NaElement;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.List;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.children;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.elem;

/**
 * Builds instances of {@link com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeLicenseInterim}
 * from <a href="http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/DOT/RrollingrockN/web_ontap_cluster_unsupported/interim/index.html#interim-license-cluster-info">interim-license-cluster-info</a>.
 */
public class LicenseInterimBuilder extends BuilderBase<CModeCluster> {
    private static final Logger logger = Logger.getLogger(LicenseInterimBuilder.class);
    private static final String ZAPI_NAME_INTERIM_LICENSE_CLUSTER_INFO = "interim-license-cluster-info";
    private static final String ZAPI_NAME_LICENSES = "licenses";

    @SuppressWarnings({"unchecked"})
    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        assert cluster.isPreSierraNevada();
        final NaElement request = elem("interim-license-list-get");
        // Manually record our desired attributes for logging purposes
        final NaElement attributes = elem("attributes",
                elem(ZAPI_NAME_LICENSES,
                    elem(ZAPI_NAME_INTERIM_LICENSE_CLUSTER_INFO, children(
                            "code", "description", "expiration-date", "limit", "serial-number", "service"))));
        final Element licenseInfoResultsElem = zapiConnection.invokeElem(request, attributes).getElement();

        final Element licensesElem = licenseInfoResultsElem.getChild(ZAPI_NAME_LICENSES);
        if (licensesElem == null) {
            return;
        }
//        // interim-license-list-get is not an iterator,
//        // LicenseInterimBuilder does not extend IterBuilderBase,
//        // So we must manually record our desired attributes for logging purposes
//        final Element licenseInfoResultselt = zapiConnection.invokeCommand("interim-license-list-get", null).getElement();
//        zapiConnection.recordRequestedAttribute("interim-license-list-get.licenses.interim-license-cluster-info.code");
//        zapiConnection.recordRequestedAttribute("interim-license-list-get.licenses.interim-license-cluster-info.description");
//        zapiConnection.recordRequestedAttribute("interim-license-list-get.licenses.interim-license-cluster-info.expiration-date");
//        zapiConnection.recordRequestedAttribute("interim-license-list-get.licenses.interim-license-cluster-info.limit");
//        zapiConnection.recordRequestedAttribute("interim-license-list-get.licenses.interim-license-cluster-info.serial-number");
//        zapiConnection.recordRequestedAttribute("interim-license-list-get.licenses.interim-license-cluster-info.service");
//
//        final Element licensesElt = licenseInfoResultselt.getChild("licenses");
//        if (licensesElt == null) {
//            return;
//        }

        final List<Element> licenseElems = licensesElem.getChildren("interim-license-cluster-info");
        for (final Element licenseElem : licenseElems) {
            final String code = BuilderUtil.getChildTextTrim(licenseElem, "code");
            final String description = BuilderUtil.getChildTextTrim(licenseElem, "description");
            final String expirationDate = BuilderUtil.getChildTextTrim(licenseElem, "expiration-date");
            final Integer limit = BuilderUtil.parseIntFromChild(licenseElem, "limit");
            final String serialNumber = BuilderUtil.getChildTextTrim(licenseElem, "serial-number");
            final EnumWrapper<CModeLicenseInterim.InterimLicensedFeature> service = BuilderUtil.getEnumWrapperByValue(
                    CModeLicenseInterim.InterimLicensedFeature.class, licenseElem, "service", CModeLicenseInterim.InterimLicensedFeature.NOT_MAPPED);
            if (serialNumber == null || serialNumber.isEmpty()) {
                logger.warn(PackageContext.getLogPrefix() + " - Skipping invalid license " + service.getRawValue()
                        + " " + description);
                continue;
            }
            final CModeLicenseInterim licenseInterim = cluster.createLicenseInterim(service, serialNumber);
            licenseInterim.setCode(code);
            licenseInterim.setDescription(description);
            licenseInterim.setExpirationDate(expirationDate);
            licenseInterim.setLimit(limit);
            cluster.getLicenses().add(licenseInterim);
        }
    }
}
