package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeLunImport;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeVolumeMove;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.LunImport;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VolumeMove;
import org.jdom.Element;
import java.util.LinkedHashMap;

public class LunImportBuilder extends IterBuilderBase {
    public LunImportBuilder() {
        super("lun-import-get-iter", "lun-import-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"lun-import-info",}, new String[]{"administrative-status", "compared-blocks", "estimated-remaining-duration", "failure-reason", "foreign-disk", "import-current-node", "import-home-node", "imported-blocks", "operation-in-progress", "operational-status", "path", "percent-complete", "total-blocks", "vserver"});
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element lunImportInfoElt) {
        final EnumWrapper<LunImport.AdministrativeStatus> administrativeStatus = BuilderUtil.getEnumWrapperByValue(LunImport.AdministrativeStatus.class, lunImportInfoElt, "administrative-status", LunImport.AdministrativeStatus.NOT_MAPPED);
        final Long comparedBlocks = BuilderUtil.parseLongFromChild(lunImportInfoElt, "compared-blocks");
        final Long estimatedRemainingDuration = BuilderUtil.parseLongFromChild(lunImportInfoElt, "estimated-remaining-duration");
        final String failureReason = BuilderUtil.getChildTextTrim(lunImportInfoElt, "failure-reason");
        final String foreignDisk = BuilderUtil.getChildTextTrim(lunImportInfoElt, "foreign-disk");
        final String importCurrentNode = BuilderUtil.getChildTextTrim(lunImportInfoElt, "import-current-node");
        final String importHomeNode = BuilderUtil.getChildTextTrim(lunImportInfoElt, "import-home-node");
        final Long importedBlocks = BuilderUtil.parseLongFromChild(lunImportInfoElt, "imported-blocks");
        final EnumWrapper<LunImport.OperationInProgress> operationInProgress = BuilderUtil.getEnumWrapperByValue(LunImport.OperationInProgress.class, lunImportInfoElt, "operation-in-progress", LunImport.OperationInProgress.NOT_MAPPED);
        final EnumWrapper<LunImport.OperationalStatus> operationalStatus = BuilderUtil.getEnumWrapperByValue(LunImport.OperationalStatus.class, lunImportInfoElt, "operational-status", LunImport.OperationalStatus.NOT_MAPPED);
        final String path = BuilderUtil.getChildTextTrim(lunImportInfoElt, "path");
        final Long percentComplete = BuilderUtil.parseLongFromChild(lunImportInfoElt, "percent-complete");
        final Long totalBlocks = BuilderUtil.parseLongFromChild(lunImportInfoElt, "total-blocks");
        final String vServer = BuilderUtil.getChildTextTrim(lunImportInfoElt, "vserver");
        
        final CModeLunImport lunImport = cluster.createLunImport(vServer, foreignDisk);
        lunImport.setAdministrativeStatus(administrativeStatus);
        lunImport.setComparedBlocks(comparedBlocks);
        lunImport.setEstimatedRemainingDuration(estimatedRemainingDuration);
        lunImport.setFailureReason(failureReason);
        lunImport.setForeignDisk(foreignDisk);
        lunImport.setImportCurrentNode(importCurrentNode);
        lunImport.setImportHomeNode(importHomeNode);
        lunImport.setImportedBlocks(importedBlocks);
        lunImport.setOperationInProgress(operationInProgress);
        lunImport.setOperationalStatus(operationalStatus);
        lunImport.setPath(path);
        lunImport.setPercentComplete(percentComplete);
        lunImport.setTotalBlocks(totalBlocks);
        lunImport.setVServer(vServer);

        cluster.getLunImports().add(lunImport);
    }
}

