package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeNFSService;
import org.jdom.Element;

import java.util.LinkedHashMap;

public class NFSBuilder extends IterBuilderBase {

    public NFSBuilder() {
        super("nfs-service-get-iter", "nfs-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"nfs-info",}, new String[]{"is-nfs-access-enabled", "is-nfsv2-enabled", "is-nfsv3-enabled", "is-nfsv40-enabled", "is-nfsv41-enabled", "vserver", "is-nfsv41-pnfs-enabled", "is-nfsv41-acl-enabled", "is-nfsv40-read-delegation-enabled", "is-nfsv40-write-delegation-enabled", "is-nfsv40-migration-enabled", "is-nfsv40-referrals-enabled", "is-nfsv41-migration-enabled", "is-nfsv41-referrals-enabled",});
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element nfsInfoElt) {
        final Boolean isNFSAccessEnabled = BuilderUtil.parseBooleanFromChild(nfsInfoElt, "is-nfs-access-enabled");
        final Boolean isNFSV2Enabled = BuilderUtil.parseBooleanFromChild(nfsInfoElt, "is-nfsv2-enabled");
        final Boolean isNFSV3Enabled = BuilderUtil.parseBooleanFromChild(nfsInfoElt, "is-nfsv3-enabled");
        final Boolean isNFSV40Enabled = BuilderUtil.parseBooleanFromChild(nfsInfoElt, "is-nfsv40-enabled");
        final Boolean isNFSV41Enabled = BuilderUtil.parseBooleanFromChild(nfsInfoElt, "is-nfsv41-enabled");
        final Boolean isNFS41PnfsEnabled = BuilderUtil.parseBooleanFromChild(nfsInfoElt, "is-nfsv41-pnfs-enabled");
        final Boolean isNFS41AclEnabled = BuilderUtil.parseBooleanFromChild(nfsInfoElt, "is-nfsv41-acl-enabled");
        final Boolean isNFS40ReadDelegationEnabled = BuilderUtil.parseBooleanFromChild(nfsInfoElt, "is-nfsv40-read-delegation-enabled");
        final Boolean isNFS40WriteDelegationEnabled = BuilderUtil.parseBooleanFromChild(nfsInfoElt, "is-nfsv40-write-delegation-enabled");
        final Boolean isNFS40MigrationEnabled = BuilderUtil.parseBooleanFromChild(nfsInfoElt, "is-nfsv40-migration-enabled");
        final Boolean isNFS40ReferralsEnabled = BuilderUtil.parseBooleanFromChild(nfsInfoElt, "is-nfsv40-referrals-enabled");
        final Boolean isNFS41MigrationEnabled = BuilderUtil.parseBooleanFromChild(nfsInfoElt, "is-nfsv41-migration-enabled");
        final Boolean isNFS41ReferralsEnabled = BuilderUtil.parseBooleanFromChild(nfsInfoElt, "is-nfsv41-referrals-enabled");
        final String vServerName = BuilderUtil.getChildTextTrim(nfsInfoElt, "vserver");

        final CModeNFSService nfsService = cluster.createNFSService(vServerName);
        nfsService.setNFSAccessEnabled(isNFSAccessEnabled);
        nfsService.setNFSV2Enabled(isNFSV2Enabled);
        nfsService.setNFSV3Enabled(isNFSV3Enabled);
        nfsService.setNFSV40Enabled(isNFSV40Enabled);
        nfsService.setNFSV41Enabled(isNFSV41Enabled);
        nfsService.setNFS41PnfsEnabled(isNFS41PnfsEnabled);
        nfsService.setNFS41AclEnabled(isNFS41AclEnabled);
        nfsService.setNFS40ReadDelegationEnabled(isNFS40ReadDelegationEnabled);
        nfsService.setNFS40WriteDelegationEnabled(isNFS40WriteDelegationEnabled);
        nfsService.setNFS40MigrationEnabled(isNFS40MigrationEnabled);
        nfsService.setNFS40ReferralsEnabled(isNFS40ReferralsEnabled);
        nfsService.setNFS41MigrationEnabled(isNFS41MigrationEnabled);
        nfsService.setNFS41ReferralsEnabled(isNFS41ReferralsEnabled);
        cluster.getNFSServices().add(nfsService);
    }
}
