package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeLif;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Lif;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.LinkedHashMap;
import java.util.List;

public class NetInterfaceBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(NetInterfaceBuilder.class);

    public NetInterfaceBuilder() {
        super("net-interface-get-iter", "net-interface-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"net-interface-info",}, new String[]{"interface-name",
                                                                  "role",
                                                                  "address",
                                                                  "administrative-status",
                                                                  "operational-status",
                                                                  "dns-domain-name",
                                                                  "current-node",
                                                                  "current-port",
                                                                  "failover-policy",
                                                                  "home-node",
                                                                  "home-port",
                                                                  "is-auto-revert",
                                                                  "is-home",
                                                                  "lif-uuid",
                                                                  "netmask",
                                                                  "netmask-length",
                                                                  "vserver",
                                                                  "failover-group",
                                                                  "routing-group-name",
                                                                  "dns-domain-name",
                                                                  "is-dns-update-enabled",
                                                                  "listen-for-dns-query",});
            put(new String[]{"net-interface-info", "data-protocols",}, new String[]{"data-protocol",});
        }});
    }

    protected void buildIterInstance(final CModeCluster cluster, final Element netIfInfoElt) {
        final String interfaceName = BuilderUtil.getChildTextTrim(netIfInfoElt, "interface-name");
        final String vServerName = BuilderUtil.getChildTextTrim(netIfInfoElt, "vserver");
        final String currentNode = BuilderUtil.getChildTextTrim(netIfInfoElt, "current-node");
        final String currentPort = BuilderUtil.getChildTextTrim(netIfInfoElt, "current-port");
        final EnumWrapper<Lif.FailoverPolicy> failoverPolicy = BuilderUtil.getEnumWrapperByValue(Lif.FailoverPolicy.class, netIfInfoElt, "failover-policy", Lif.FailoverPolicy.NOT_MAPPED);
        final String homeNode = BuilderUtil.getChildTextTrim(netIfInfoElt, "home-node");
        final String homePort = BuilderUtil.getChildTextTrim(netIfInfoElt, "home-port");
        final Boolean isAutoRevert = BuilderUtil.parseBooleanFromChild(netIfInfoElt, "is-auto-revert");
        final Boolean isHome = BuilderUtil.parseBooleanFromChild(netIfInfoElt, "is-home");
        final String lifUUID = BuilderUtil.getChildTextTrim(netIfInfoElt, "lif-uuid");
        final String ipAddressString = BuilderUtil.getChildTextTrim(netIfInfoElt, "address");
        final String netMask = BuilderUtil.getChildTextTrim(netIfInfoElt, "netmask");
        final Integer netmaskLenString = BuilderUtil.parseIntFromChild(netIfInfoElt, "netmask-length");
        final String dnsDomainName = BuilderUtil.getChildTextTrim(netIfInfoElt, "dns-domain-name");
        final String routingGroupName = BuilderUtil.getChildTextTrim(netIfInfoElt, "routing-group-name");
        final EnumWrapper<Lif.Status> operStatus = BuilderUtil.getEnumWrapperByValue(Lif.Status.class, netIfInfoElt, "operational-status", Lif.Status.NOT_MAPPED);
        final EnumWrapper<Lif.Status> adminStatus = BuilderUtil.getEnumWrapperByValue(Lif.Status.class, netIfInfoElt, "administrative-status", Lif.Status.NOT_MAPPED);
        final EnumWrapper<Lif.Role> role = BuilderUtil.getEnumWrapperByValue(Lif.Role.class, netIfInfoElt, "role", Lif.Role.NOT_MAPPED);
        final String failoverGroupName = BuilderUtil.getChildTextTrim(netIfInfoElt, "failover-group");
        final Boolean isDNSUpdateEnabled = BuilderUtil.parseBooleanFromChild(netIfInfoElt, "is-dns-update-enabled");
        final Boolean listenForDNSQuery = BuilderUtil.parseBooleanFromChild(netIfInfoElt, "listen-for-dns-query");
        
        final Element dataProtocolListElt = netIfInfoElt.getChild("data-protocols");
        final StringBuilder protocolListBuilder = new StringBuilder();
        if (dataProtocolListElt != null) {
            @SuppressWarnings({"unchecked"})
            final List<Element> dataProtocolElts = dataProtocolListElt.getChildren("data-protocol");
            for (final Element dataProtocolElt : dataProtocolElts) {
                final String protocol = dataProtocolElt.getTextNormalize();
                if (protocolListBuilder.length() > 0) {
                    protocolListBuilder.append(',');
                }
                protocolListBuilder.append(protocol);
            }
        }
        final String protocolList = protocolListBuilder.toString();
        final CModeLif cModeLif = cluster.createLif(interfaceName, vServerName);
        cModeLif.setLifUUID(lifUUID);
        cModeLif.setCurrentNode(currentNode);
        cModeLif.setCurrentPort(currentPort);
        cModeLif.setHomeNode(homeNode);
        cModeLif.setHomePort(homePort);
        cModeLif.setAutoRevert(isAutoRevert);
        cModeLif.setFailoverPolicy(failoverPolicy);
        cModeLif.setHome(isHome);
        cModeLif.setIpAddress(ipAddressString);
        cModeLif.setNetMask(netMask);
        cModeLif.setNetmaskLen(netmaskLenString);
        cModeLif.setDnsDomainName(dnsDomainName);
        cModeLif.setRoutingGroupName(routingGroupName);
        cModeLif.setOperStatus(operStatus);
        cModeLif.setAdminStatus(adminStatus);
        cModeLif.setRole(role);
        cModeLif.setFailoverGroupName(failoverGroupName);
        cModeLif.setIsDNSUpdateEnabled(isDNSUpdateEnabled);
        cModeLif.setListenForDNSQuery(listenForDNSQuery);
        cModeLif.setDataProtocolList(protocolList);

        cluster.getLifs().add(cModeLif);

    }
}
