package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;


import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeNetworkPortBroadcastDomain;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import org.jdom.Element;

import java.util.LinkedHashMap;

/**
 * Builder class for generating the net-port-broadcast-domain-get-iter ZAPI request, reading the ZAPI response and
 * creating the CModeNetPortBroadcastDomain objects.
 */
public class NetPortBroadcastDomainBuilder extends IterBuilderBase {

    protected NetPortBroadcastDomainBuilder() {
        super("net-port-broadcast-domain-get-iter", "net-port-broadcast-domain-info", getDefaultMaxRecords(),
                new LinkedHashMap<String[], String[]>() {
                    {
                        put(new String[]{"net-port-broadcast-domain-info"},
                                new String[]{"broadcast-domain", "mtu", "ipspace"});
                    }
                });
    }

    /**
     * This method will be called for each net-port-broadcast-domain-info in the ZAPI response. Here we retrieve
     * broadcast domain related attributes from the ZAPI response and create a CModeNetPortBroadcastDomain.
     */
    @Override
    protected void buildIterInstance(CModeCluster cluster, Element netPortBroadcastDomainInfo) {
        final String name = BuilderUtil.getChildTextTrim(netPortBroadcastDomainInfo, "broadcast-domain");
        final Long mtu = BuilderUtil.parseLongFromChild(netPortBroadcastDomainInfo, "mtu");
        final String ipSpaceName = BuilderUtil.getChildTextTrim(netPortBroadcastDomainInfo, "ipspace");
        final CModeNetworkPortBroadcastDomain netPortBroadcastDomain = cluster.createNetPortBroadcastDomain(name,
                ipSpaceName);
        netPortBroadcastDomain.setMtu(mtu);
        cluster.getNetworkPortBroadcastDomains().add(netPortBroadcastDomain);
    }
}
