package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;


import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeNetworkFailoverGroup;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeNetworkFailoverGroupNetworkPortRelationship;
import netapp.manage.NaElement;
import org.jdom.Element;

import java.util.List;

/**
 * Builder class for generating the net-failover-group-get-iter ZAPI request, reading the ZAPI response and creating the
 * CModeNetFailoverGroup objects.
 */
public class NetworkFailoverGroupBuilder extends IterBuilderBase
{
  protected NetworkFailoverGroupBuilder()
  {
    super("net-failover-group-get-iter", "net-failover-group-info", getDefaultMaxRecords(), (NaElement) null);
  }

  /**
   * This method will be called for each net-failover-group-info in the ZAPI response. Here we retrieve failover group
   * related attributes from the ZAPI response and create a CModeNetFailoverGroup.
   */
  @Override
  protected void buildIterInstance(CModeCluster cluster, Element netFailoverGroupInfo)
  {
    final String failoverGroupName = BuilderUtil.getChildTextTrim(netFailoverGroupInfo, "failover-group");
    final String broadcastDomainName = BuilderUtil.getChildTextTrim(netFailoverGroupInfo, "broadcast-domain");
    final String vserver = BuilderUtil.getChildTextTrim(netFailoverGroupInfo, "vserver");
    final CModeNetworkFailoverGroup netFailoverGroup = cluster.createNetworkFailoverGroup(failoverGroupName);
    netFailoverGroup.setNetworkPortBroadcastDomainName(broadcastDomainName);
    netFailoverGroup.setVServer(vserver);
    cluster.getNetworkFailoverGroups().add(netFailoverGroup);

    //Release FS uses the targets attribute
    Element targets = netFailoverGroupInfo.getChild("targets");
    if (targets != null)
      failoverPortRelationships(cluster, netFailoverGroupInfo, targets);
    else
    {
      //Release SN uses the port attribute
      final String port = BuilderUtil.getChildTextTrim(netFailoverGroupInfo, "port");
      final String serverNodeName = BuilderUtil.getChildTextTrim(netFailoverGroupInfo, "node");
      final CModeNetworkFailoverGroupNetworkPortRelationship relationship = cluster.createNetworkFailoverGroupNetworkPortRelationship(serverNodeName, failoverGroupName, port);
      cluster.getNetworkFailoverGroupNetworkPortRelationships().add(relationship);
    }
  }

  /**
   * This is for ONTAP version FS and greater
   *
   * @param cluster
   * @param netFailoverGroupInfo
   * @param targets
   */
  private void failoverPortRelationships(CModeCluster cluster, Element netFailoverGroupInfo, Element targets)
  {
    final String failoverGroupName = BuilderUtil.getChildTextTrim(netFailoverGroupInfo, "failover-group");
    final List<Element> netQualifiedPortNames = targets.getChildren("net-qualified-port-name");
    if (netQualifiedPortNames != null)
    {
      for (final Element netQualifiedPortName : netQualifiedPortNames)
      {
        String serverNodeName_Port = netQualifiedPortName.getValue();
        if (serverNodeName_Port != null)
        {
          serverNodeName_Port = serverNodeName_Port.trim();
          int index = serverNodeName_Port.lastIndexOf(":");
          final String port = serverNodeName_Port.substring(index + 1);
          final String serverNodeName = serverNodeName_Port.substring(0, index );
          final CModeNetworkFailoverGroupNetworkPortRelationship relationship = cluster.createNetworkFailoverGroupNetworkPortRelationship(serverNodeName, failoverGroupName, port.trim());
          cluster.getNetworkFailoverGroupNetworkPortRelationships().add(relationship);
        }
      }
    }
  }
}
