package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeNetworkSubnet;
import org.jdom.Element;

import java.util.LinkedHashMap;
import java.util.List;

public class NetworkSubnetBuilder extends IterBuilderBase{

    public NetworkSubnetBuilder() {
        super("net-subnet-get-iter", "net-subnet-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {
        {
            put(new String[]{"net-subnet-info",}, new String[]{"available-count", "broadcast-domain","gateway", "ipspace", "subnet-name", "subnet", "total-count", "used-count", "force-update-lif-associations"});
            put(new String[]{"net-subnet-info", "ip-ranges"}, new String[]{"ip-range",});
        }
         });
    }


    /**
     * This method will be called for each net-subnet-info in the ZAPI response. Here we retrieve net subnet related
     * attributes from the ZAPI response and create a CModeNetworkSubnet
     */
    @Override
    protected void buildIterInstance(CModeCluster cluster, Element subnetInfo) {
        final Integer availableCount = BuilderUtil.parseIntFromChild(subnetInfo, "available-count");
        final String broadcastDomain = BuilderUtil.getChildTextTrim(subnetInfo, "broadcast-domain");
        final Boolean forceUpdateLifAssociations = BuilderUtil.parseBooleanFromChild(subnetInfo, "force-update-lif-associations");
        final String gateway = BuilderUtil.getChildTextTrim(subnetInfo, "gateway");
        final List<String> ipRanges = BuilderUtil.getListFromChildren(subnetInfo, "ip-ranges", "ip-range");
        final String ipspace = BuilderUtil.getChildTextTrim(subnetInfo, "ipspace");
        final String subnet = BuilderUtil.getChildTextTrim(subnetInfo, "subnet");
        final String subnet_name = BuilderUtil.getChildTextTrim(subnetInfo,"subnet-name");
        final Integer totalCount = BuilderUtil.parseIntFromChild(subnetInfo, "total-count");
        final Integer usedCount = BuilderUtil.parseIntFromChild(subnetInfo, "used-count");

        final CModeNetworkSubnet cmodeSubnet = cluster.createNetworkSubnet(ipspace, subnet_name);
        cmodeSubnet.setAvailableCount(availableCount);
        cmodeSubnet.setBroadcastDomain(broadcastDomain);
        if(forceUpdateLifAssociations!=null)  {
            cmodeSubnet.setForceUpdateLifAssociations(forceUpdateLifAssociations);
        }
        cmodeSubnet.setGateway(gateway);
        cmodeSubnet.setIpRanges(BuilderUtil.getListAsCSVString(ipRanges));
        cmodeSubnet.setIpspace(ipspace);
        cmodeSubnet.setSubnet(subnet);
        cmodeSubnet.setName(subnet_name);
        cmodeSubnet.setTotalCount(totalCount);
        cmodeSubnet.setUsedCount(usedCount);


        cluster.getNetworkSubnets().add(cmodeSubnet);
    }

}
