package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeQTree;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.QTree;
import org.jdom.Element;

import java.util.LinkedHashMap;

public class QTreeBuilder extends IterBuilderBase {

    public QTreeBuilder() {
        super("qtree-list-iter", "qtree-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"qtree-info",}, new String[]{"export-policy", "id", "is-export-policy-inherited", "oplocks", "qtree", "security-style", "status", "volume", "vserver",  });
        }});
    }

    protected void buildIterInstance(final CModeCluster cluster, final Element qtreeInfoElt) {
        final String qtreeID = BuilderUtil.getChildTextTrim(qtreeInfoElt, "id");
        final Boolean oplocksEnabled = BuilderUtil.parseBooleanFromChild(qtreeInfoElt, "oplocks");
        final String qtreeRawName = BuilderUtil.getChildText(qtreeInfoElt, "qtree"); // need to preserve leading, trailing spaces in name
        final String qtreeName = (qtreeRawName == null) ? "" : qtreeRawName;         // need to return empty instead of null for use in keys
        final EnumWrapper<QTree.SecurityStyle> securityStyle = BuilderUtil.getEnumWrapperByValue(QTree.SecurityStyle.class, qtreeInfoElt, "security-style", QTree.SecurityStyle.NOT_MAPPED);
        final EnumWrapper<QTree.Status> status = BuilderUtil.getEnumWrapperByValue(QTree.Status.class, qtreeInfoElt, "status", QTree.Status.NOT_MAPPED);
        final String volumeName = BuilderUtil.getChildTextTrim(qtreeInfoElt, "volume");
        final String vServerName = BuilderUtil.getChildTextTrim(qtreeInfoElt, "vserver");
        final String exportPolicyName = BuilderUtil.getChildTextTrim(qtreeInfoElt, "export-policy");
        final Boolean isExportPolicyInherited = BuilderUtil.parseBooleanFromChild(qtreeInfoElt, "is-export-policy-inherited");
        final CModeQTree qtree = cluster.createQTree(vServerName, volumeName, qtreeID);
        qtree.setQTreeName(qtreeName);
        qtree.setOpLocksEnabled(oplocksEnabled);
        qtree.setSecurityStyle(securityStyle);
        qtree.setStatus(status);
        qtree.setExportPolicyName(exportPolicyName);
        qtree.setExportPolicyInherited(isExportPolicyInherited);

        cluster.getQTrees().add(qtree);
    }
}
