package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeSnapMirrorHistory;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.SnapMirrorHistory;
import org.apache.log4j.Logger;
import org.jdom.Element;
import java.util.LinkedHashMap;

public class SnapMirrorHistoryBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(SnapMirrorHistoryBuilder.class);

    @SuppressWarnings("serial")
    public SnapMirrorHistoryBuilder() {
        super("snapmirror-history-get-iter", "snapmirror-history-info", getDefaultMaxRecords(),
                new LinkedHashMap<String[], String[]>() {
                    {
                        put(new String[] { "snapmirror-history-info", }, new String[] { "additional-info",
                                "destination-location", "destination-volume", "destination-vserver", "end-time",
                                "operation-id", "operation-type", "relationship-group-type", "result", "source-volume",
                                "start-time", "transfer-size" });
                    }
                });
    }

    protected void buildIterInstance(final CModeCluster cluster, final Element snapMirrorHistoryInfo) {
        final String additionalInfo = BuilderUtil.getChildTextTrim(snapMirrorHistoryInfo, "additional-info");
        final String destinationLocation = BuilderUtil.getChildTextTrim(snapMirrorHistoryInfo, "destination-location");
        final String destinationVolume = BuilderUtil.getChildTextTrim(snapMirrorHistoryInfo, "destination-volume");
        final String sourceVolume = BuilderUtil.getChildTextTrim(snapMirrorHistoryInfo, "source-volume");
        final String destinationVServer = BuilderUtil.getChildTextTrim(snapMirrorHistoryInfo, "destination-vserver");
        final Long startTime = BuilderUtil.parseLongFromChild(snapMirrorHistoryInfo, "start-time");
        final Long endTime = BuilderUtil.parseLongFromChild(snapMirrorHistoryInfo, "end-time");          
        final String operationId = BuilderUtil.getChildTextTrim(snapMirrorHistoryInfo, "operation-id");
        final EnumWrapper<SnapMirrorHistory.OperationType> operationType = BuilderUtil.getEnumWrapperByValue(
                SnapMirrorHistory.OperationType.class, snapMirrorHistoryInfo, "operation-type", SnapMirrorHistory.OperationType.NOT_MAPPED);
        final EnumWrapper<SnapMirrorHistory.RelationshipGroupType> relationshipGroupType = BuilderUtil.getEnumWrapperByValue(
                SnapMirrorHistory.RelationshipGroupType.class, snapMirrorHistoryInfo, "relationship-group-type", SnapMirrorHistory.RelationshipGroupType.NOT_MAPPED);
        final EnumWrapper<SnapMirrorHistory.Result> result = BuilderUtil.getEnumWrapperByValue(
                SnapMirrorHistory.Result.class, snapMirrorHistoryInfo, "result", SnapMirrorHistory.Result.NOT_MAPPED);
        final Long transferSize = BuilderUtil.parseLongFromChild(snapMirrorHistoryInfo, "transfer-size"); 
              
        final CModeSnapMirrorHistory snapMirrorHistory = cluster.createSnapMirrorHistory(operationId);
        snapMirrorHistory.setAdditionalInfo(additionalInfo);
        snapMirrorHistory.setDestinationVolume(destinationVolume);
        snapMirrorHistory.setDestinationLocation(destinationLocation);
        snapMirrorHistory.setDestinationVServer(destinationVServer);
        snapMirrorHistory.setSourceVolume(sourceVolume);
        snapMirrorHistory.setStartTime(startTime);
        snapMirrorHistory.setEndTime(endTime);
        snapMirrorHistory.setOperationType(operationType);
        snapMirrorHistory.setRelationshipGroupType(relationshipGroupType);
        snapMirrorHistory.setResult(result);
        snapMirrorHistory.setTransferSize(transferSize);
        
        cluster.getSnapMirrorHistories().add(snapMirrorHistory);
    }
}
