package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeAdapter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.StorageAdapter;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.*;

public class StorageAdapterBuilder extends BuilderBase<CModeCluster> {

    @SuppressWarnings({"UnusedDeclaration"})
    private final static Logger logger = Logger.getLogger(StorageAdapterBuilder.class);

    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        // storage-adapter-get-adapter-list is not an iterator,
        // storage-adapter-get-adapter-info is not an iterator,
        // StorageAdapterBuilder doesn't extend IterBuilderBase,
        // So we must manually record our desired attributes for logging purposes
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-list.adapter-list.adapter-name-elem.adapter-name");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-list.adapter-list.adapter-name-elem.node-name");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-info.adapter-details.adapter-detail-info.adapter-type");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-info.adapter-details.adapter-detail-info.adapter-fc.adapter-fc-info.fc-node-name");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-info.adapter-details.adapter-detail-info.adapter-fc.adapter-fc-info.fc-port-name");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-info.adapter-details.adapter-detail-info.adapter-fc.adapter-fc-info.adapter-model");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-info.adapter-details.adapter-detail-info.adapter-fc.adapter-fc-info.is-in-use");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-info.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.adapter-state");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-info.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.cable-end-identifier");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-info.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.cable-length");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-info.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.cable-manufacturer");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-info.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.cable-part-number");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-info.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.cable-serial-number");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-info.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.attached-serial-number");
        zapiConnection.recordRequestedAttribute("storage-adapter-get-adapter-info.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.cable-technology");


        final Map<String, Set<String>> nodeToListOfAdapterNames = getAllAdapterNamesByNodesFromZapi(zapiConnection);
        if (nodeToListOfAdapterNames == null) {
            return;
        }

        final Map<String, String> args = new LinkedHashMap<String, String>();
        for (final Map.Entry<String, Set<String>> nodeToAdapterNames : nodeToListOfAdapterNames.entrySet()) {
            final String nodeName = nodeToAdapterNames.getKey();
            final Set<String> adapterNames = nodeToAdapterNames.getValue();
            for (final String adapterName : adapterNames) {
                final Element adapterResultsElt = getAdapterInfoFromZapi(zapiConnection, args, nodeName, adapterName);
                modelAdapter(cluster, nodeName, adapterName, adapterResultsElt);
            }
        }
    }

    private void modelAdapter(final CModeCluster cluster, final String nodeName, final String adapterName, final Element adapterResultsElt) {
        final Element adapterDetailsElt = adapterResultsElt.getChild("adapter-details");
        if (adapterDetailsElt != null) {
            final Element adapterDetailInfoElt = adapterDetailsElt.getChild("adapter-detail-info");
            final EnumWrapper<StorageAdapter.Type> adapterType = BuilderUtil.getEnumWrapperByValue(StorageAdapter.Type.class, adapterDetailInfoElt, "adapter-type", StorageAdapter.Type.NOT_MAPPED);

            // Possible values: "ADT_IF_ATA", "ADT_IF_PARALLEL_SCSI", "ADT_IF_SAS", "ADT_IF_FC".
            if ( !StorageAdapter.Type.ADT_IF_FC.equals(adapterType.getEnumValue()) &&
                    ! StorageAdapter.Type.ADT_IF_SAS.equals(adapterType.getEnumValue())) {
                if (logger.isDebugEnabled()) {
                    logger.debug("skipping " + nodeName + " " + adapterName + ", unsupported adapter type " + adapterType);
                }
                return;
            }

            final CModeAdapter adapter = cluster.createCModeAdapter(nodeName, adapterName);
            adapter.setType(adapterType);

            boolean addAdapter = false;


            final Element fcElt = adapterDetailInfoElt.getChild("adapter-fc");

            if (fcElt != null) {
                addAdapter = true;
                setAdapterFCAttributes(adapter, fcElt);
            }

            final Element sasElt = adapterDetailInfoElt.getChild("adapter-sas");

            if (sasElt != null) {
                addAdapter = true;
                setAdapterSASattributes(adapter, sasElt);
            }


            if (addAdapter) {
                cluster.getCModeStorageAdapters().add(adapter);

            }
            else{
                logger.warn("skipping " + nodeName + " " + adapterName + ", no information (adapter-fc or adpater-sas) was found for adapter type " + adapterType);
            }
        }
    }

    private void setAdapterSASattributes(final CModeAdapter adapter, final Element sasElt) {
        final Element sasInfoElt = sasElt.getChild("adapter-sas-info");

        final EnumWrapper<StorageAdapter.State> state = BuilderUtil.getEnumWrapperByValue(StorageAdapter.State.class, sasInfoElt, "adapter-state", StorageAdapter.State.NOT_MAPPED);
        adapter.setAdapterState(state);
        final Element qsfpCableElt = sasInfoElt.getChild("sas-qsfp-cable");

        if (qsfpCableElt != null) {
            final Element qsfpCableInfoElt = qsfpCableElt.getChild("sas-qsfp-cable-info");

            adapter.setCableEndIdentifier(BuilderUtil.getChildTextTrim(qsfpCableInfoElt, "cable-end-identifier"));
            adapter.setCableLength(BuilderUtil.getChildTextTrim(qsfpCableInfoElt, "cable-length"));
            adapter.setCableManufacturer(BuilderUtil.getChildTextTrim(qsfpCableInfoElt, "cable-manufacturer"));
            adapter.setCablePartNumber(BuilderUtil.getChildTextTrim(qsfpCableInfoElt, "cable-part-number"));
            adapter.setCableSerialNumber(BuilderUtil.getChildTextTrim(qsfpCableInfoElt, "cable-serial-number"));
            adapter.setAttachedSerialNumber(BuilderUtil.getChildTextTrim(qsfpCableInfoElt, "attached-serial-number"));
            adapter.setCableTechnology(BuilderUtil.getChildTextTrim(qsfpCableInfoElt, "cable-technology"));
        }
    }

    private void setAdapterFCAttributes(final CModeAdapter adapter, final Element fcElt) {
        final Element fcInfoElt = fcElt.getChild("adapter-fc-info");
        final String fcNodeName = BuilderUtil.getChildTextTrim(fcInfoElt, "fc-node-name");
        final String normalizedNodeWWN = BuilderUtil.normalizeWWN(fcNodeName);
        final String fcPortName = BuilderUtil.getChildTextTrim(fcInfoElt, "fc-port-name");
        final String normalizedPortWWN = BuilderUtil.normalizeWWN(fcPortName);
        final String adapterModel = BuilderUtil.getChildTextTrim(fcInfoElt, "adapter-model");
        final Boolean isInUse = BuilderUtil.parseBooleanFromChild(fcInfoElt, "is-in-use");
        adapter.setNodeWWN(normalizedNodeWWN);
        adapter.setPortWWN(normalizedPortWWN);
        adapter.setModel(adapterModel);
        adapter.setInUse(isInUse);
    }

    private Element getAdapterInfoFromZapi(final ZAPIConnection zapiConnection, final Map<String, String> args, final String nodeName, final String adapterName) throws DataSourceErrorException {
        args.clear();
        args.put("node-name", nodeName);
        args.put("adapter-name", adapterName);

        return zapiConnection.invokeCommand("storage-adapter-get-adapter-info", args).getElement();
    }

    private Map<String, Set<String>> getAllAdapterNamesByNodesFromZapi(final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        final Element storageAdapterListResultsElt = zapiConnection.invokeCommand("storage-adapter-get-adapter-list", null).getElement();

        final Element adapterListElt = storageAdapterListResultsElt.getChild("adapter-list");
        if (adapterListElt == null) {
            return null;
        }
        final Map<String, Set<String>> adapters = new HashMap<String, Set<String>>();
        @SuppressWarnings({"unchecked"})
        final List<Element> adapterNameElts = adapterListElt.getChildren("adapter-name-elem");
        for (final Element adapterNameElt : adapterNameElts) {
            final String nodeName = BuilderUtil.getChildTextTrim(adapterNameElt, "node-name");
            if (!adapters.containsKey(nodeName)) {
                final Set<String> adapterNames = new HashSet<String>();
                adapters.put(nodeName, adapterNames);
            }
            final Set<String> adapterNames = adapters.get(nodeName);

            final String adapterName = BuilderUtil.getChildTextTrim(adapterNameElt, "adapter-name");
            adapterNames.add(adapterName);
        }
        return adapters;
    }
}
