package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeStoragePool;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.StoragePool;
import org.jdom.Element;

import java.util.LinkedHashMap;

public class StoragePoolBuilder extends IterBuilderBase {


    protected StoragePoolBuilder() {
        super("storage-pool-get-iter", "storage-pool-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"storage-pool-info",}, new String[]{"allocation-unit-size", "disk-count", "pool-total-size", "pool-usable-size", "storage-pool", "storage-type","is-healthy",});
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element storagePoolInfo) {
        final Long allocationUnitSize = BuilderUtil.parseLongFromChild(storagePoolInfo, "allocation-unit-size");
        final Long diskCount = BuilderUtil.parseLongFromChild(storagePoolInfo, "disk-count");
        final Long poolTotalSize = BuilderUtil.parseLongFromChild(storagePoolInfo, "pool-total-size");
        final Long poolUsableSize = BuilderUtil.parseLongFromChild(storagePoolInfo, "pool-usable-size");
        final String storagePoolName = BuilderUtil.getChildTextTrim(storagePoolInfo, "storage-pool");
        final EnumWrapper<StoragePool.StorageType> storageType = BuilderUtil.getEnumWrapperByValue(StoragePool.StorageType.class, storagePoolInfo, "storage-type", StoragePool.StorageType.NOT_MAPPED);
        final Boolean isHealthy = BuilderUtil.parseBooleanFromChild(storagePoolInfo, "is-healthy");
        final CModeStoragePool storagePool = cluster.createStoragePool(storagePoolName);
        storagePool.setAllocationUnitSize(allocationUnitSize);
        storagePool.setDiskCount(diskCount);
        storagePool.setHealthy(isHealthy);
        storagePool.setPoolTotalSize(poolTotalSize);
        storagePool.setPoolUsableSize(poolUsableSize);
        storagePool.setStorageType(storageType);
        cluster.getStoragePools().add(storagePool);
    }
}
