package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeNode;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeStorageShelf;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeStorageShelves;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.StorageShelf;
import com.onaro.util.VersionComparator;

import org.jdom.Element;

import java.util.*;

/**
 * @author Iti Behari
 *         Date: 7/17/13
 *
 *   This builder is dependent on nodes being discovered
 */
public class StorageShelfBuilder extends BuilderBase<CModeCluster> {

    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {

        for (final CModeNode cModeNode : cluster.getCModeNodes().values()) {
            final Element responseElt = getStorageShelfListFromZAPI(zapiConnection, cModeNode);
            if (responseElt == null) {
                continue;
            }

            final Element shelfListElt = responseElt.getChild("shelf-list");
            if (shelfListElt == null) {
                continue;
            }
            @SuppressWarnings({"unchecked"})
            final List<Element> shelfInfoEltList = shelfListElt.getChildren("shelf-info");
            for (final Element shelfInfoElt : shelfInfoEltList) {
                buildStorageShelf(cModeNode, shelfInfoElt);
            }
        }

        populateShelfPathsAttributes(cluster, zapiConnection);
    }

    private void buildStorageShelf(final CModeNode cModeNode, final Element shelfInfoElt) {
        final String channelName = BuilderUtil.getChildTextTrim(shelfInfoElt, "channel-name");
        final String shelfUIDRaw = BuilderUtil.getChildTextTrim(shelfInfoElt, "shelf-uid");
       
        final String shelfUID = BuilderUtil.normalizeWWN(shelfUIDRaw);

        final UUIDKey shelfKey = new UUIDKey("storage_shelf", shelfUID);
        final CModeStorageShelves nodeShelves = cModeNode.getStorageShelves();
        if (nodeShelves.containsKey(shelfKey)) {
            final CModeStorageShelf storageShelf = nodeShelves.get(shelfKey);
            final String existingChannelName = storageShelf.getChannelName();
            final int channelCompare = existingChannelName.compareTo(channelName);
            if (channelCompare > 0) {
                setStorageShelfAttributes(shelfInfoElt, channelName, storageShelf);
            }
        } else {
            final CModeStorageShelf storageShelf = cModeNode.createCModeStorageShelf(shelfUID);
            setStorageShelfAttributes(shelfInfoElt, channelName, storageShelf);
            cModeNode.getStorageShelves().add(storageShelf);
        }
    }

    private void setStorageShelfAttributes(final Element shelfInfoElt, final String channelName, final CModeStorageShelf storageShelf) {
        storageShelf.setChannelName(channelName);
        storageShelf.setShelfType(BuilderUtil.getEnumWrapperByValue(CModeStorageShelf.ShelfType.class, shelfInfoElt, "shelf-type", CModeStorageShelf.ShelfType.NOT_MAPPED));
        storageShelf.setFirmwareRevA(BuilderUtil.getChildTextTrim(shelfInfoElt, "firmware-rev-A"));
        storageShelf.setFirmwareRevB(BuilderUtil.getChildTextTrim(shelfInfoElt, "firmware-rev-B"));
        storageShelf.setModule(BuilderUtil.getChildTextTrim(shelfInfoElt, "module"));
        storageShelf.setModuleState(BuilderUtil.getChildTextTrim(shelfInfoElt, "module-state"));
        storageShelf.setReturnedNodeName(BuilderUtil.getChildTextTrim(shelfInfoElt, "node-name"));
        storageShelf.setShelfID(BuilderUtil.getChildTextTrim(shelfInfoElt, "shelf-id"));
        storageShelf.setShelfName(BuilderUtil.getChildTextTrim(shelfInfoElt, "shelf-name"));
        storageShelf.setShelfState(BuilderUtil.getEnumWrapperByValue(StorageShelf.State.class, shelfInfoElt, "shelf-state", StorageShelf.State.NOT_MAPPED));
        storageShelf.setTerminationSwitch(BuilderUtil.getChildTextTrim(shelfInfoElt, "termination-switch"));

        final Integer bayCount;
        final List<String> bayList = new ArrayList<String>();
        final Element bayInfo = shelfInfoElt.getChild("shelf-bay-info");
        if (bayInfo != null) {
            bayCount = BuilderUtil.parseIntFromChild(bayInfo, "bay-count");
            final Element bayListElt = bayInfo.getChild("bay-list");
            if (bayListElt != null) {
                bayList.addAll(BuilderUtil.getListFromChildren(bayListElt, "bay-info", "shelf-bay"));
            }
        } else {
            bayCount = null;
        }
        storageShelf.setBayCount(bayCount);
        storageShelf.setBayList(BuilderUtil.getListAsCSVString(bayList));
    }

    private Element getStorageShelfListFromZAPI(final ZAPIConnection zapiConnection, final CModeNode cModeNode) throws DataSourceErrorException {
        final String nodeName = cModeNode.getKey().getName();
        final Map<String, String> args = new LinkedHashMap<String, String>() {{
            put("node-name", nodeName);
        }};
        final ZAPIConnection.ZapiResponse response = zapiConnection.invokeCommand("storage-shelf-list-info", args);
        return response.getElement();
    }

    private void populateShelfPathsAttributes(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
    		new StorageShelfIterBuilderBase().buildModel(cluster, zapiConnection);
        }

    }
