package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeNode;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeStorageShelfChannel;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.jdom.Element;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Iti Behari
 *         Date: 7/17/13
 */
public class StorageShelfEnvBuilder extends BuilderBase<CModeCluster> {

	 private static final Logger logger = LoggerFactory.getLogger(StorageShelfEnvBuilder.class);

    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        for (final CModeNode cModeNode : cluster.getCModeNodes().values()) {
            final String nodeName = cModeNode.getKey().getName();

            final Element shelfChannelListElt = getStorageChannelListFromZapi(zapiConnection, nodeName);
            if (shelfChannelListElt == null) {
                continue;
            }

            @SuppressWarnings({"unchecked"})
            final List<Element> shelfEnvChannelInfoElts = shelfChannelListElt.getChildren("shelf-environ-channel-info");
            for (final Element shelfEnvChannelInfoElt : shelfEnvChannelInfoElts) {

                final String channelName = BuilderUtil.getChildTextTrim(shelfEnvChannelInfoElt, "channel-name");
                final Boolean isChannelMonitorFailure = BuilderUtil.parseBooleanFromChild(shelfEnvChannelInfoElt, "is-shelf-channel-failure");

                final CModeStorageShelfChannel shelfChannel = cModeNode.createCModeStorageShelfChannel(channelName);
                cModeNode.getStorageShelfChannels().add(shelfChannel);

                final Element shelfEnvListElt = shelfEnvChannelInfoElt.getChild("shelf-environ-shelf-list");
                if (shelfEnvListElt == null) {
                    continue;
                }
                @SuppressWarnings({"unchecked"})
                final List<Element> shelfListElt = shelfEnvListElt.getChildren("shelf-environ-shelf-info");
                if (shelfListElt == null) {
                    continue;
                }
                buildShelfChannel(nodeName, shelfChannel, channelName, isChannelMonitorFailure, shelfListElt);
            }
        }
    }

    private void buildShelfChannel(final String nodeName, final CModeStorageShelfChannel shelfChannel, final String channelName,
                                   final Boolean channelMonitorFailure, final List<Element> shelfListElt) {
        for (final Element shelfEnvInfoElt : shelfListElt) {

            // JSD EnvShelf
            final CModeStorageShelfChannel.CModeStorageShelfEnv shelfEnv = createShelfEnvironmentAndSetAttributes(shelfEnvInfoElt, shelfChannel);
            if(shelfEnv != null) {
                shelfChannel.getStorageShelfEnvs().add(shelfEnv);

                final Element coolingElementListElt = shelfEnvInfoElt.getChild("cooling-element-list");
                if (coolingElementListElt != null) {
                        addCoolingElementsToShelfEnvironment(shelfEnv, coolingElementListElt);
            	}

            	final Element powerSupplyListElt = shelfEnvInfoElt.getChild("power-supply-list");
            	if (powerSupplyListElt != null) {
            		addPowerSuppliesToShelfEnvironment(shelfEnv, powerSupplyListElt);
            	}

            	final Element tempSensorListElt = shelfEnvInfoElt.getChild("temp-sensor-list");
            	if (tempSensorListElt != null) {
            		addTempSensorsToShelfEnvironment(shelfEnv, tempSensorListElt);
            	}

            	final Element currentSensorListElt = shelfEnvInfoElt.getChild("current-sensor-list");
            	if (currentSensorListElt != null) {
            		addCurrentSensorsToShelfEnvironment(shelfEnv, currentSensorListElt);
            	}

            	final Element voltageSensorListElt = shelfEnvInfoElt.getChild("voltage-sensor-list");
            	if (voltageSensorListElt != null) {
            		addVoltageSensorsToShelfEnvironment(shelfEnv, voltageSensorListElt);
            	}

                final Element connectorInformationElt = shelfEnvInfoElt.getChild("connector-information");
                if (connectorInformationElt != null) {
                        addShelfCablesToStorageShelf(nodeName, shelfEnv, channelName, channelMonitorFailure, connectorInformationElt);
                }
            }
            else {
                logger.info("Shelf doesn't have a logical ID");
            }
        }
    }

    private void addShelfCablesToStorageShelf(final String nodeName, final CModeStorageShelfChannel.CModeStorageShelfEnv shelfEnv,
                                              final String channelName, final Boolean isChannelMonitorFailure, final Element connectorInformationElt) {
        final String shelfId = shelfEnv.getKey().getID();
        @SuppressWarnings({"unchecked"})
        final List<Element> connectorInfoList = connectorInformationElt.getChildren("connector-info");

        for (final Element connectorInfoElt : connectorInfoList) {
            final Element sasConnectorList = connectorInfoElt.getChild("sas-connector-list");

            if (sasConnectorList != null) {
                @SuppressWarnings({"unchecked"})
                final List<Element> sasConnectorInfoList = sasConnectorList.getChildren("sas-connector-info");

                for (final Element sasConnectorInfo : sasConnectorInfoList) {
                    final String partNumber = BuilderUtil.getChildTextTrim(sasConnectorInfo, "cable-part-no");
                    final String serialNumber = BuilderUtil.getChildTextTrim(sasConnectorInfo, "cable-serial-no");
                    final Integer connectorNo = BuilderUtil.parseIntFromChild(sasConnectorInfo, "connector-no");

                    final CModeStorageShelfChannel.CModeShelfCable shelfCable =
                            shelfEnv.createAndAddShelfCable(nodeName, shelfId, channelName, partNumber, serialNumber, connectorNo);

                    shelfCable.setAttachedSerialNumber(BuilderUtil.getChildTextTrim(sasConnectorInfo, "attached-serial-no"));
                    shelfCable.setEndIdentifier(BuilderUtil.getChildTextTrim(sasConnectorInfo, "cable-end-identifier"));
                    shelfCable.setLength(BuilderUtil.getChildTextTrim(sasConnectorInfo, "cable-length"));
                    shelfCable.setManufacturer(BuilderUtil.getChildTextTrim(sasConnectorInfo, "cable-manufacturer"));
                    shelfCable.setTechnology(BuilderUtil.getChildTextTrim(sasConnectorInfo, "cable-technology"));
                    shelfCable.setType(BuilderUtil.getChildTextTrim(sasConnectorInfo, "cable-type"));
                    shelfCable.setConnectorDesignator(BuilderUtil.getChildTextTrim(sasConnectorInfo, "connector-designator"));
                    shelfCable.setCableConnected(BuilderUtil.parseBooleanFromChild(sasConnectorInfo, "is-cable-connected"));
                    shelfCable.setConnectorError(BuilderUtil.parseBooleanFromChild(sasConnectorInfo, "is-connector-error"));
                    shelfCable.setShelfChannelFailure(isChannelMonitorFailure);
                    shelfCable.setShelfModule(BuilderUtil.getChildTextTrim(sasConnectorInfo, "shelf-module"));
                    shelfCable.setSwapCount(BuilderUtil.parseIntFromChild(sasConnectorInfo, "swap-count"));
                }
            }
        }
    }

    private CModeStorageShelfChannel.CModeStorageShelfEnv createShelfEnvironmentAndSetAttributes(final Element shelfEnvInfoElt, final CModeStorageShelfChannel shelfChannel) {
        final String shelfID = BuilderUtil.getChildTextTrim(shelfEnvInfoElt, "shelf-id");
        final Element sesInfoElt = shelfEnvInfoElt.getChild("ses-generic-info");
        if (sesInfoElt != null) {
        final String sesLogicalIDRaw = BuilderUtil.getChildTextTrim(sesInfoElt, "ses-logical-id");
        final CModeStorageShelfChannel.CModeStorageShelfEnv shelfEnv = shelfChannel.createStorageShelfEnv(sesLogicalIDRaw);

            shelfEnv.setShelfType(BuilderUtil.getChildTextTrim(shelfEnvInfoElt, "shelf-type"));
            shelfEnv.setShelfStatus(BuilderUtil.getChildTextTrim(shelfEnvInfoElt, "shelf-status"));
            shelfEnv.setSesContactState(BuilderUtil.getChildTextTrim(sesInfoElt, "ses-contact-state"));
            shelfEnv.setSesLogicalID(BuilderUtil.normalizeWWN(sesLogicalIDRaw.substring(2)));  // Strip leading "0x" and normalize
            shelfEnv.setSesProductID(BuilderUtil.getChildTextTrim(sesInfoElt, "ses-product-id"));
            shelfEnv.setSesProductVersion(BuilderUtil.getChildTextTrim(sesInfoElt, "ses-product-revision"));
            shelfEnv.setSesVendorID(BuilderUtil.getChildTextTrim(sesInfoElt, "ses-vendor-id"));
            return shelfEnv;
        }
        else {
            return null;
        }

    }

    private void addCoolingElementsToShelfEnvironment(final CModeStorageShelfChannel.CModeStorageShelfEnv shelfEnv, final Element coolingElementListElt) {
        @SuppressWarnings({"unchecked"})
        final List<Element> coolingElementList = coolingElementListElt.getChildren("cooling-element-info");
        for (final Element coolingElementElt : coolingElementList) {
            final String coolingElementID = BuilderUtil.getChildTextTrim(coolingElementElt, "cooling-element-number");
            final Boolean isNotInstalled = BuilderUtil.parseBooleanFromChild(coolingElementElt, "cooling-element-is-not-installed");
            final Boolean isError = BuilderUtil.parseBooleanFromChild(coolingElementElt, "cooling-element-is-error");
            final CModeStorageShelfChannel.CModeCoolingElement coolingElement = shelfEnv.createCoolingElement(coolingElementID);
            coolingElement.setNotInstalled(isNotInstalled);
            coolingElement.setError(isError);
            shelfEnv.getCoolingElements().add(coolingElement);
        }
    }

    private void addPowerSuppliesToShelfEnvironment(final CModeStorageShelfChannel.CModeStorageShelfEnv shelfEnv, final Element powerSupplyListElt) {
        @SuppressWarnings({"unchecked"})
        final List<Element> powerSupplyList = powerSupplyListElt.getChildren("power-supply-info");
        for (final Element powerSupplyElt : powerSupplyList) {

            final String powerSupplyID = BuilderUtil.getChildTextTrim(powerSupplyElt, "power-supply-element-number");
            final CModeStorageShelfChannel.CModePowerSupply powerSupply = shelfEnv.createPowerSupply(powerSupplyID);

            powerSupply.setNotInstalled(BuilderUtil.parseBooleanFromChild(powerSupplyElt, "power-supply-is-not-installed"));
            powerSupply.setError(BuilderUtil.parseBooleanFromChild(powerSupplyElt, "power-supply-is-error"));

            shelfEnv.getPowerSupplies().add(powerSupply);
        }
    }

    private void addTempSensorsToShelfEnvironment(final CModeStorageShelfChannel.CModeStorageShelfEnv shelfEnv, final Element tempSensorListElt) {
        @SuppressWarnings({"unchecked"})
        final List<Element> tempSensorList = tempSensorListElt.getChildren("temp-sensor-info");
        for (final Element tempSensorElt : tempSensorList) {
            final String sensorID = BuilderUtil.getChildTextTrim(tempSensorElt, "temp-sensor-element-no");

            final CModeStorageShelfChannel.CModeTempSensor tempSensor = shelfEnv.createTempSensor(sensorID);
            tempSensor.setNotInstalled(BuilderUtil.parseBooleanFromChild(tempSensorElt, "temp-sensor-is-not-installed"));
            tempSensor.setCurrentCondition(BuilderUtil.getChildTextTrim(tempSensorElt, "temp-sensor-current-condition"));
            tempSensor.setCurrentTemp(BuilderUtil.parseIntFromChild(tempSensorElt, "temp-sensor-current-temperature"));
            tempSensor.setHiCritical(BuilderUtil.parseIntFromChild(tempSensorElt, "temp-sensor-hi-critical"));
            tempSensor.setHiWarning(BuilderUtil.parseIntFromChild(tempSensorElt, "temp-sensor-hi-warning"));
            tempSensor.setAmbient(BuilderUtil.parseBooleanFromChild(tempSensorElt, "temp-sensor-is-ambient"));
            tempSensor.setError(BuilderUtil.parseBooleanFromChild(tempSensorElt, "temp-sensor-is-error"));
            tempSensor.setLowCritical(BuilderUtil.parseIntFromChild(tempSensorElt, "temp-sensor-low-critical"));
            tempSensor.setLowWarning(BuilderUtil.parseIntFromChild(tempSensorElt, "temp-sensor-low-warning"));
            shelfEnv.getTempSensors().add(tempSensor);
        }
    }

    private void addCurrentSensorsToShelfEnvironment(final CModeStorageShelfChannel.CModeStorageShelfEnv shelfEnv, final Element currentSensorListElt) {
        @SuppressWarnings({"unchecked"})
        final List<Element> currentSensorList = currentSensorListElt.getChildren("current-sensor-info");
        for (final Element currentSensorElt : currentSensorList) {
            final String sensorID = BuilderUtil.getChildTextTrim(currentSensorElt, "current-sensor-no");
            final CModeStorageShelfChannel.CModeSensor sensor = shelfEnv.createSensor(CModeStorageShelfChannel.CModeSensor.Type.current, sensorID);

            sensor.setSensorNotInstalled(BuilderUtil.parseBooleanFromChild(currentSensorElt, "is-sensor-not-installed"));
            sensor.setSensorError(BuilderUtil.parseBooleanFromChild(currentSensorElt, "is-sensor-error"));
            sensor.setSensorCondition(BuilderUtil.getChildTextTrim(currentSensorElt, "sensor-condition"));
            sensor.setSensorValueUnits(BuilderUtil.getChildTextTrim(currentSensorElt, "sensor-reading"));

            shelfEnv.getSensors().add(sensor);
        }
    }

    private void addVoltageSensorsToShelfEnvironment(final CModeStorageShelfChannel.CModeStorageShelfEnv shelfEnv, final Element voltageSensorListElt) {
        @SuppressWarnings({"unchecked"})
        final List<Element> voltageSensorList = voltageSensorListElt.getChildren("voltage-sensor-info");
        for (final Element voltageSensorElt : voltageSensorList) {
            final String sensorID = BuilderUtil.getChildTextTrim(voltageSensorElt, "voltage-sensor-no");
            final CModeStorageShelfChannel.CModeSensor sensor = shelfEnv.createSensor(CModeStorageShelfChannel.CModeSensor.Type.voltage, sensorID);

            sensor.setSensorNotInstalled(BuilderUtil.parseBooleanFromChild(voltageSensorElt, "is-sensor-not-installed"));
            sensor.setSensorError(BuilderUtil.parseBooleanFromChild(voltageSensorElt, "is-sensor-error"));
            sensor.setSensorCondition(BuilderUtil.getChildTextTrim(voltageSensorElt, "sensor-condition"));
            sensor.setSensorValueUnits(BuilderUtil.getChildTextTrim(voltageSensorElt, "sensor-reading"));

            shelfEnv.getSensors().add(sensor);
        }
    }

    private Element getStorageChannelListFromZapi(final ZAPIConnection zapiConnection, final String nodeName) throws DataSourceErrorException {
        final Map<String, String> args = new LinkedHashMap<String, String>() {{
            put("node-name", nodeName);
        }};
        final ZAPIConnection.ZapiResponse response = zapiConnection.invokeCommand("storage-shelf-environment-list-info", args);
        final Element responseElt = response.getElement();
        if(responseElt != null)
            return responseElt.getChild("shelf-environ-channel-list");
        else
            return null;
    }
}
