/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeStorageSwitch;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.StorageSwitch;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.LinkedHashMap;
import java.util.List;

/**
 * @author Al Velella
 *         Date: 10/21/13
 *         Time: 4:27 PM
 */
public class StorageSwitchBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(AggregateBuilder.class);

    public StorageSwitchBuilder() {
        super("storage-switch-get-iter", "storage-switch-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"storage-switch-info",},
                        new String[]{
                                "firmware-version",
                                "name",
                                "ip-address",
                                "model",
                                "role",
                                "status",
                                "vendor",
                                "wwn",});
            put(new String[]{"storage-switch-info", "storage-switch-connectivity-list", "storage-switch-connectivity-info",},
                        new String[]{
                                "peer-port-wwn",});
            put(new String[]{"storage-switch-info", "storage-switch-temperature-sensor-list", "temperature-sensor-info",},
                        new String[]{
                                "name",});
            put(new String[]{"storage-switch-info", "storage-switch-power-supply-unit-list", "storage-switch-power-supply-unit-info",},
                        new String[]{
                                "name",});
            put(new String[]{"storage-switch-info", "storage-switch-fan-list", "storage-switch-fan-info",},
                        new String[]{
                                "name",});
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element switchAttributesElt) {
        // Storage switch attributes
        final String firmwareVersion = BuilderUtil.getChildTextTrim(switchAttributesElt, "firmware-version");
        final String name = BuilderUtil.getChildTextTrim(switchAttributesElt, "name");
        final String ipAddress = BuilderUtil.getChildTextTrim(switchAttributesElt, "ip-address");
        final String model = BuilderUtil.getChildTextTrim(switchAttributesElt, "model");
        final EnumWrapper<StorageSwitch.Role> roleWrapper = BuilderUtil.getEnumWrapperByValue(StorageSwitch.Role.class, switchAttributesElt, "role", StorageSwitch.Role.NOT_MAPPED);
        final EnumWrapper<StorageSwitch.Status> statusWrapper = BuilderUtil.getEnumWrapperByValue(StorageSwitch.Status.class, switchAttributesElt, "status", StorageSwitch.Status.NOT_MAPPED);
        final String vendor = BuilderUtil.getChildTextTrim(switchAttributesElt, "vendor");
        final String wwn = BuilderUtil.normalizeWWN(BuilderUtil.getChildTextTrim(switchAttributesElt, "wwn"));

        final CModeStorageSwitch storageSwitch = cluster.createStorageSwitch(wwn);
        storageSwitch.setFirmwareVersion(firmwareVersion);
        storageSwitch.setName(name);
        storageSwitch.setIpAddress(ipAddress);
        storageSwitch.setModel(model);
        storageSwitch.setRoleWrapper(roleWrapper);
        storageSwitch.setStatusWrapper(statusWrapper);
        storageSwitch.setVendor(vendor);

        final Element connectivityListElt = switchAttributesElt.getChild("storage-switch-connectivity-list");
        if (connectivityListElt != null) {
            @SuppressWarnings({"unchecked"})
            final List<Element> connectivityInfoElts = connectivityListElt.getChildren("storage-switch-connectivity-info");
            for (final Element connectivityInfoElt : connectivityInfoElts) {
                final String peerWwn = BuilderUtil.getChildTextTrim(connectivityInfoElt, "peer-port-wwn");
                if (peerWwn != null) {
                    storageSwitch.addPeerWwn(BuilderUtil.normalizeWWN(peerWwn));
                }
            }
        }

        final Element sensorListElt = switchAttributesElt.getChild("storage-switch-temperature-sensor-list");
        if (sensorListElt != null) {
            @SuppressWarnings({"unchecked"})
            final List<Element> sensorElt = sensorListElt.getChildren("temperature-sensor-info");
            if (sensorElt != null) {
                storageSwitch.setSensorCount(sensorElt.size());
            }
        }

        final Element powerSupplyListElt = switchAttributesElt.getChild("storage-switch-power-supply-unit-list");
        if (powerSupplyListElt != null) {
            @SuppressWarnings({"unchecked"})
            final List<Element> powerSupplyElt = powerSupplyListElt.getChildren("storage-switch-power-supply-unit-info");
            if (powerSupplyElt != null) {
                storageSwitch.setPowerSupplyCount(powerSupplyElt.size());
            }
        }

        final Element fanListElt = switchAttributesElt.getChild("storage-switch-fan-list");
        if (powerSupplyListElt != null) {
            @SuppressWarnings({"unchecked"})
            final List<Element> fanElt = fanListElt.getChildren("storage-switch-fan-info");
            if (fanElt != null) {
                storageSwitch.setFanCount(fanElt.size());
            }
        }

        cluster.getStorageSwitches().add(storageSwitch);
    }
}
