package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeSystemHealthAlert;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPINames;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.SystemHealthAlert;
import org.jdom.Element;

import java.util.List;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.elem;

/**
 * Builder for diagnosis-alert-get-iter ZAPI
 */
public class SystemHealthAlertBuilder extends IterBuilderBase{
    public static final String DIAGNOSIS_ALERT_INFO = "diagnosis-alert-info";
    public static final String MONITOR = "monitor";
    public static final String SUBSYSTEM = "subsystem";
    public static final String ALERT_ID = "alert-id";


    public SystemHealthAlertBuilder() {
        super("diagnosis-alert-get-iter", "diagnosis-alert-info", getDefaultMaxRecords(), null,
                elem(ZAPINames.QUERY,
                        elem(DIAGNOSIS_ALERT_INFO,
                                elem(MONITOR, "system-connect|node-connect"),
                                elem(SUBSYSTEM, "FHM-Switch|metrocluster_node|metrocluster|fhm-bridge|sas_connect"),
                                elem(ALERT_ID, "InterclusterBrokenConnectionAlert|InterconnectAdapterOfflineAlert|RaidDegradedMirrorAggrAlert|RaidLeftBehindAggrAlert" +
                                        "|RaidLeftBehindSpareAlert|StorageFCAdapterFault_Alert|ClusterSeveredAllLinksAlert|NoISLPresentAlert|FabricSwitchFanFail_Alert" +
                                        "|FabricSwitchPowerFail_Alert|FabricSwitchTempCritical_Alert|FabricSwitchUnreachable_Alert" +
                                        "|StorageBridgePortDown_Alert|StorageBridgeTempAboveCritical_Alert"+
                                        "|StorageBridgeTempBelowCritical_Alert|StorageBridgeUnreachable_Alert"+
                                        "|StorageBridgeLinkFailure_Alert|FabricSwitchTempSensorFailed_Alert" +
                                        "|SASAdapterOffline_Alert|NchmTwoNodeMccAUSODisabled_Alert"+
                                        "|AUSOTriggerFailed_Alert")))
         );
    }

    protected void buildIterInstance(final CModeCluster cluster, final Element diagnosidAlertInfoElt) {
        final Boolean acknowledge = BuilderUtil.parseBooleanFromChild(diagnosidAlertInfoElt, "acknowledge");
        final String acknowledger = BuilderUtil.getChildTextTrim(diagnosidAlertInfoElt, "acknowledger");
        final List<String> additionalInfoArray = BuilderUtil.getListFromGrandChildren(diagnosidAlertInfoElt, "additional-info", "string");
        final String alertId = BuilderUtil.getChildTextTrim(diagnosidAlertInfoElt, "alert-id");
        final String alertingResource = BuilderUtil.getChildTextTrim(diagnosidAlertInfoElt, "alerting-resource");
        final String alertingResourceName = BuilderUtil.getChildTextTrim(diagnosidAlertInfoElt, "alerting-resource-name");
        final String correctiveActions = BuilderUtil.getChildTextTrim(diagnosidAlertInfoElt, "corrective-actions");
        final Long indicationTime = BuilderUtil.parseLongFromChild(diagnosidAlertInfoElt, "indication-time");
        final String nodeName =  BuilderUtil.getChildTextTrim(diagnosidAlertInfoElt, "node");
        final String perceivedSeverity = BuilderUtil.getChildTextTrim(diagnosidAlertInfoElt, "perceived-severity");
        final String policy = BuilderUtil.getChildTextTrim(diagnosidAlertInfoElt, "policy");
        final String possibleEffect = BuilderUtil.getChildTextTrim(diagnosidAlertInfoElt, "possible-effect");
        final String probableCause = BuilderUtil.getChildTextTrim(diagnosidAlertInfoElt, "probable-cause");
        final String probableCauseDescription = BuilderUtil.getChildTextTrim(diagnosidAlertInfoElt, "probable-cause-description");
        final String subsystem = BuilderUtil.getChildTextTrim(diagnosidAlertInfoElt, "subsystem");
        final Boolean suppress = BuilderUtil.parseBooleanFromChild(diagnosidAlertInfoElt, "suppress");
        final String suppressor = BuilderUtil.getChildTextTrim(diagnosidAlertInfoElt, "suppressor");

        final EnumWrapper<SystemHealthAlert.MonitorType> monitorTypeEnumWrapper =
                BuilderUtil.getEnumWrapperByValue(SystemHealthAlert.MonitorType.class, diagnosidAlertInfoElt, "monitor", SystemHealthAlert.MonitorType.NOT_MAPPED);
        final StringBuilder additionalInfoBuilder = new StringBuilder();
        boolean first = true;
        for(String str : additionalInfoArray )  {
            if(!first)   {
                additionalInfoBuilder.append(';');
            }
            additionalInfoBuilder.append(str);
            first = false;
        }
        final String additionalInfo = additionalInfoBuilder.toString();

        final CModeSystemHealthAlert alert = cluster.createSystemHealthAlert(alertId, alertingResource);

        alert.setAcknowledge(acknowledge);
        alert.setAcknowledger(acknowledger);
        alert.setAdditionalInfo(additionalInfo);
        alert.setAlertingResourceName(alertingResourceName);
        alert.setCorrectiveActions(correctiveActions);
        alert.setIndicationTime(indicationTime);
        alert.setMonitor(monitorTypeEnumWrapper);
        alert.setPerceivedSeverity(perceivedSeverity);
        alert.setPolicy(policy);
        alert.setPossibleEffect(possibleEffect);
        alert.setProbableCause(probableCause);
        alert.setProbableCauseDescription(probableCauseDescription);
        alert.setSubsystem(subsystem);
        alert.setSuppress(suppress);
        alert.setSuppressor(suppressor);
        alert.setNode(nodeName);

        cluster.getSystemHealthAlerts().add(alert);
    }

}
