package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorType;
import org.apache.log4j.Logger;
import org.jdom.Element;

public class TimezoneBuilder extends BuilderBase<CModeCluster> {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(TimezoneBuilder.class);

    @Override
    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        try {
            final ZAPIConnection.ZapiResponse response = zapiConnection.invokeCommand("clock-get-timezone", null);
            final Element resultsElt = response.getElement();
            final String timezone = BuilderUtil.getChildTextTrim(resultsElt, "timezone");
            final String timezoneUTC = BuilderUtil.getChildTextTrim(resultsElt, "timezone-UTC");
            final String timezoneVersion = BuilderUtil.getChildTextTrim(resultsElt, "timezone-version");
            cluster.setTimezone(timezone);
            cluster.setTimezoneUTC(timezoneUTC);
            cluster.setTimezoneVersion(timezoneVersion);
        } catch (DataSourceErrorException dsee) {

            // Work around a bug in ZAPI: when non admin user reads this ZAPI, ZAPI fails with:
            // user 'guest' does not have write access to this resource
            // If this happens, just ignore it.
            // Allow the poll to succeed even without the timezone info by ignoring this speci
            boolean ignoreError = false;
            if (dsee.getType().equals(DataSourceErrorType.INVALID_LOGIN_CREDENTIALS)) {
                final Throwable cause = dsee.getCause();
                final String errorMessage = cause.getMessage();
                if (errorMessage.contains("Insufficient privileges") && errorMessage.contains("does not have write access to this resource")) {
                    ignoreError = true;
                }
            }

            if (ignoreError) {
                return;
            } else {
                throw dsee;
            }
        }
    }
}
