package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeVServerPeer;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServerPeer;
import org.apache.log4j.Logger;
import org.jdom.Element;
import java.util.LinkedHashMap;

public class VServerPeerBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(VServerPeerBuilder.class);

    public VServerPeerBuilder() {
        super("vserver-peer-get-iter", "vserver-peer-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"vserver-peer-info",}, new String[]{"peer-cluster", "peer-state", "peer-vserver", "peer-vserver-uuid", "vserver", "vserver-uuid"});
            put(new String[]{"vserver-peer-info", "applications",}, new String[]{"vserver-peer-application",});
        }});
    }

    protected void buildIterInstance(final CModeCluster cluster, final Element vServerPeerInfo) {
        final String applications = BuilderUtil.getListAsCSVString(BuilderUtil.getListFromGrandChildren(
                vServerPeerInfo, "applications", "vserver-peer-application"));
        final String peerCluster = BuilderUtil.getChildTextTrim(vServerPeerInfo, "peer-cluster");
        final EnumWrapper<VServerPeer.PeerState> peerState = BuilderUtil.getEnumWrapperByValue(
                VServerPeer.PeerState.class, vServerPeerInfo, "peer-state", VServerPeer.PeerState.NOT_MAPPED);
        final String peerVServer = BuilderUtil.getChildTextTrim(vServerPeerInfo, "peer-vserver");
        final String peerVServerUUID = BuilderUtil.getChildTextTrim(vServerPeerInfo, "peer-vserver-uuid");
        final String vServer = BuilderUtil.getChildTextTrim(vServerPeerInfo, "vserver");
        final String vServerUUID = BuilderUtil.getChildTextTrim(vServerPeerInfo, "vserver-uuid");
        
        final CModeVServerPeer vServerPeer = cluster.createVServerPeer(peerCluster, peerVServer, vServer);
        vServerPeer.setApplications(applications);
        vServerPeer.setPeerClusterName(peerCluster);
        vServerPeer.setPeerState(peerState);
        vServerPeer.setPeerVServerName(peerVServer);
        vServerPeer.setPeerVServerUUID(peerVServerUUID);
        vServerPeer.setVServerName(vServer);
        vServerPeer.setVServerUUID(vServerUUID);
        
        cluster.getVServerPeers().add(vServerPeer);
    }
}
