package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;

/**
 * This builder builds the vserverID to vserverName map for cifs/nfsv3/nfsv4/nfsv41/lif, 
 * the mapping is cached into cluster object. 
 *
 */
public class VServerStatsIDMapBuilder extends BuilderBase<CModeCluster>{

	@Override
	public void buildModel(CModeCluster cluster, ZAPIConnection zapiConnection)	throws DataSourceErrorException {
		CifsBuilder cifsBuilder = new CifsBuilder();
		cifsBuilder.buildModel(cluster, zapiConnection);
        cluster.getCifsVserverIDNameMap().putAll(cifsBuilder.getUUIDNameMap());
		
		Nfsv3Builder nfsv3Builder = new Nfsv3Builder();
		nfsv3Builder.buildModel(cluster, zapiConnection);
        cluster.getNfsv3VserverIDNameMap().putAll(nfsv3Builder.getUUIDNameMap());

		Nfsv4Builder nfsv4Builder = new Nfsv4Builder();
		nfsv4Builder.buildModel(cluster, zapiConnection);
        cluster.getNfsv4VserverIDNameMap().putAll(nfsv4Builder.getUUIDNameMap());

		Nfsv41Builder nfsv41Builder = new Nfsv41Builder();
		nfsv41Builder.buildModel(cluster, zapiConnection);
        cluster.getNfsv41VserverIDNameMap().putAll(nfsv41Builder.getUUIDNameMap());
        
		LifBuilder lifBuilder = new LifBuilder();
		lifBuilder.buildModel(cluster, zapiConnection);
        cluster.getLifVserverIDNameMap().putAll(lifBuilder.getUUIDNameMap());
        
	}

    public static final class Nfsv3Builder extends PerfInstanceBuilder {
        public Nfsv3Builder() {
            super("nfsv3");
        }
    }

    public static final class Nfsv4Builder extends PerfInstanceBuilder {
        public Nfsv4Builder() {
            super("nfsv4");
        }
    }

    public static final class Nfsv41Builder extends PerfInstanceBuilder {
        public Nfsv41Builder() {
            super("nfsv4_1");
        }
    }

    public static final class CifsBuilder extends PerfInstanceBuilder {
        public CifsBuilder() {
            super("cifs:vserver");
        }
    }

    public static final class LifBuilder extends PerfInstanceBuilder {
        public LifBuilder() {
            super("lif:vserver");
        }
    }
    
}
