package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;


import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeSisStatus;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeVolume;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeVolumeClone;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Volume;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.EnumSet;
import java.util.LinkedHashMap;

public class VolumeBuilder extends IterBuilderBase
{
    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(VolumeBuilder.class);

    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        super.buildModel(cluster, zapiConnection);                              // calls volume-get-iter
        new SisBuilder().buildModel(cluster, zapiConnection);                   // calls sis-get-iter
        new VolumeCloneBuilder().buildModel(cluster, zapiConnection);           // calls volume-clone-get-iter
    }

    public VolumeBuilder() {

        super("volume-get-iter", "volume-attributes", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"volume-attributes", "volume-id-attributes",}, new String[]{"name", "uuid", "instance-uuid", "containing-aggregate-uuid", "owning-vserver-uuid", "junction-parent-name", "junction-path", "qos-policy-group-name", "style", "type",});
            put(new String[]{"volume-attributes", "volume-infinitevol-attributes",}, new String[]{"constituent-role", "storage-service", "enable-snapdiff", "is-managed-by-service", "max-data-constituent-size", "max-namespace-constituent-size"});
            put(new String[]{"volume-attributes", "volume-state-attributes",}, new String[]{"state", "is-vserver-root", "is-junction-active", "is-constituent",});
            put(new String[]{"volume-attributes", "volume-security-attributes",}, new String[]{"style",});
            put(new String[]{"volume-attributes", "volume-security-attributes", "volume-security-unix-attributes"}, new String[]{"group-id", "permissions", "user-id"});
            put(new String[]{"volume-attributes", "volume-space-attributes",}, new String[]{"size", "size-total", "size-used", "size-available", "is-space-guarantee-enabled", "overwrite-reserve", "overwrite-reserve-required", "overwrite-reserve-used", "overwrite-reserve-used-actual", "snapshot-reserve-size", "percentage-snapshot-reserve", "percentage-snapshot-reserve-used", "size-available-for-snapshots", "percentage-fractional-reserve", "percentage-size-used", "space-guarantee", "space-mgmt-option-try-first", "size-used-by-snapshots",});
            put(new String[]{"volume-attributes", "volume-sis-attributes",}, new String[]{"is-sis-volume", "deduplication-space-saved", "compression-space-saved", "percentage-deduplication-space-saved", "percentage-compression-space-saved",});
            put(new String[]{"volume-attributes", "volume-mirror-attributes",}, new String[]{"is-data-protection-mirror", "is-load-sharing-mirror", "is-move-mirror", "is-replica-volume",});
            put(new String[]{"volume-attributes", "volume-export-attributes",}, new String[]{"policy",});
            put(new String[]{"volume-attributes", "volume-hybrid-cache-attributes",}, new String[]{"eligibility","caching-policy"});
            put(new String[]{"volume-attributes", "volume-clone-attributes", "volume-clone-parent-attributes"}, new String[]{"name",});
            put(new String[]{"volume-attributes", "volume-directory-attributes",}, new String[]{"i2p-enabled",});
            put(new String[]{"volume-attributes", "volume-snapshot-attributes",}, new String[]{"auto-snapshots-enabled", "snapdir-access-enabled", "snapshot-clone-dependency-enabled", "snapshot-count","snapshot-policy",});
            put(new String[]{"volume-attributes", "volume-autosize-attributes",}, new String[]{"mode", "is-enabled", "maximum-size", "increment-size",});
            put(new String[]{"volume-attributes", "volume-performance-attributes",}, new String[]{"is-atime-update-enabled",});
            put(new String[]{"volume-attributes", "volume-inode-attributes",}, new String[]{"block-type", "files-total", "files-used",});
            put(new String[]{"volume-attributes", "volume-language-attributes",}, new String[]{"language-code", "is-create-ucode-enabled", "is-convert-ucode-enabled",});
            put(new String[]{"volume-attributes", "volume-clone-attributes",}, new String[]{"clone-child-count",});
            put(new String[]{"volume-attributes", "volume-snapshot-autodelete-attributes",}, new String[]{"is-autodelete-enabled", "commitment", "delete-order", "defer-delete", "target-free-space", "trigger", "prefix", "destroy-list",});
            put(new String[]{"volume-attributes", "volume-vm-align-attributes",}, new String[]{"vm-align-sector", "vm-align-suffix",});
            put(new String[]{"volume-attributes", "volume-flexcache-attributes",}, new String[]{"origin", "min-reserve",});
            put(new String[]{"volume-attributes", "volume-qos-attributes",}, new String[]{"policy-group-name",});
        }});
    }

  protected void buildIterInstance(final CModeCluster cluster, final Element volumeAttributesElt) {
        // ID attributes
        final Element volumeIDAttributesElt = volumeAttributesElt.getChild("volume-id-attributes");
        String volumeUUID = BuilderUtil.getChildTextTrim(volumeIDAttributesElt, "uuid");
        final String volumeInstanceUUID = BuilderUtil.getChildTextTrim(volumeIDAttributesElt, "instance-uuid");
        final String volumeName = BuilderUtil.getChildTextTrim(volumeIDAttributesElt, "name");
        final String parentAggrUUID = BuilderUtil.getChildTextTrim(volumeIDAttributesElt, "containing-aggregate-uuid");
        final String owningVServerUUID = BuilderUtil.getChildTextTrim(volumeIDAttributesElt, "owning-vserver-uuid");
        final String junctionParentName = BuilderUtil.getChildTextTrim(volumeIDAttributesElt, "junction-parent-name");

        final EnumWrapper<Volume.VolumeStyle> volumeStyle = BuilderUtil.getEnumWrapperByValue(Volume.VolumeStyle.class, volumeIDAttributesElt, "style", Volume.VolumeStyle.NOT_MAPPED);
        final EnumWrapper<Volume.VolumeType> volumeType = BuilderUtil.getEnumWrapperByValue(Volume.VolumeType.class, volumeIDAttributesElt, "type", Volume.VolumeType.NOT_MAPPED);

        // Infinivol attributes
        final Element infinivolAttributesElt = volumeAttributesElt.getChild("volume-infinitevol-attributes");
        final String storageServiceName;
        final Boolean enableSnapDiff;
        final Boolean isManagedByService;
        final Long maxDataConstituentSize;
        final Long maxNamespaceConstituentSize;
        final EnumWrapper<Volume.ConstituentRole> constituentRoleWrapper;
        if (infinivolAttributesElt != null) {
            storageServiceName = BuilderUtil.getChildTextTrim(infinivolAttributesElt, "storage-service");
            enableSnapDiff = BuilderUtil.parseBooleanFromChild(infinivolAttributesElt, "enable-snapdiff");
            isManagedByService = BuilderUtil.parseBooleanFromChild(infinivolAttributesElt, "is-managed-by-service");
            maxDataConstituentSize = BuilderUtil.parseLongFromChild(infinivolAttributesElt, "max-data-constituent-size");
            maxNamespaceConstituentSize = BuilderUtil.parseLongFromChild(infinivolAttributesElt, "max-namespace-constituent-size");
            constituentRoleWrapper = BuilderUtil.getEnumWrapperByValue(Volume.ConstituentRole.class, infinivolAttributesElt, "constituent-role", Volume.ConstituentRole.NOT_MAPPED);
        } else {
            storageServiceName = null;
            enableSnapDiff = null;
            isManagedByService = null;
            maxDataConstituentSize = null;
            maxNamespaceConstituentSize = null;
            constituentRoleWrapper = new EnumWrapper<Volume.ConstituentRole>(null, null);
        }


        // State attributes
        final Element volumeStateAttributesElt = volumeAttributesElt.getChild("volume-state-attributes");
        final EnumWrapper<Volume.State> state = BuilderUtil.getEnumWrapperByValue(Volume.State.class, volumeStateAttributesElt, "state", Volume.State.NOT_MAPPED);
        final Boolean isVServerRoot = BuilderUtil.parseBooleanFromChildDefaultFalse(volumeStateAttributesElt, "is-vserver-root");
        // BURT 692958 says isJunctionActive should not default to false
        final Boolean isJunctionActive = BuilderUtil.parseBooleanFromChild(volumeStateAttributesElt, "is-junction-active");
        final Boolean isConstituent = BuilderUtil.parseBooleanFromChildDefaultFalse(volumeStateAttributesElt, "is-constituent");
        final String junctionPath = BuilderUtil.getChildTextTrim(volumeIDAttributesElt, "junction-path");
        // Later cluster versions identify policy group name in a child element "volume-qos-elements");
        String qosPolicyGroupName = BuilderUtil.getChildTextTrim(volumeIDAttributesElt, "qos-policy-group-name");

        // Security Attributes
        final Element volumeSecurityAttributesElt = volumeAttributesElt.getChild("volume-security-attributes");
        final EnumWrapper<Volume.SecurityStyle> securityStyle = BuilderUtil.getEnumWrapperByValue(Volume.SecurityStyle.class, volumeSecurityAttributesElt, "style", Volume.SecurityStyle.NOT_MAPPED);

        // Unix Security attributes
        String groupID = null;
        String permissions = null;
        String userID = null;
        if (volumeSecurityAttributesElt != null) {
            final Element unixVolumeSecurityAttrsElt = volumeSecurityAttributesElt.getChild("volume-security-unix-attributes");
            if (unixVolumeSecurityAttrsElt != null) {
                groupID = BuilderUtil.getChildTextTrim(unixVolumeSecurityAttrsElt, "group-id");
                permissions = BuilderUtil.getChildTextTrim(unixVolumeSecurityAttrsElt, "permissions");
                userID = BuilderUtil.getChildTextTrim(unixVolumeSecurityAttrsElt, "user-id");
            }
        }

        // Space attributes
        final Element volumeSpaceAttributesElt = volumeAttributesElt.getChild("volume-space-attributes");
        final Long size = BuilderUtil.parseLongFromChild(volumeSpaceAttributesElt, "size");
        final Long sizeTotal = BuilderUtil.parseLongFromChild(volumeSpaceAttributesElt, "size-total");
        final Long sizeUsed = BuilderUtil.parseLongFromChild(volumeSpaceAttributesElt, "size-used");
        final Long sizeAvail = BuilderUtil.parseLongFromChild(volumeSpaceAttributesElt, "size-available");
        final Boolean isSpaceGuarantee = BuilderUtil.parseBooleanFromChild(volumeSpaceAttributesElt, "is-space-guarantee-enabled");
        final Long overwriteReserve = BuilderUtil.parseLongFromChild(volumeSpaceAttributesElt, "overwrite-reserve");
        final Long overwriteReserveRequired = BuilderUtil.parseLongFromChild(volumeSpaceAttributesElt, "overwrite-reserve-required");
        final Long overwriteReserveUsed = BuilderUtil.parseLongFromChild(volumeSpaceAttributesElt, "overwrite-reserve-used");
        final Long overwriteReserveUsedActual = BuilderUtil.parseLongFromChild(volumeSpaceAttributesElt, "overwrite-reserve-used-actual");
        final Long snapshotReserveSize = BuilderUtil.parseLongFromChild(volumeSpaceAttributesElt, "snapshot-reserve-size");
        final Integer percentSnapshotReserve = BuilderUtil.parseIntFromChild(volumeSpaceAttributesElt, "percentage-snapshot-reserve");
        final Integer percentSnapshotReserveUsed = BuilderUtil.parseIntFromChild(volumeSpaceAttributesElt, "percentage-snapshot-reserve-used");
        final Long sizeAvailableForSnapshots = BuilderUtil.parseLongFromChild(volumeSpaceAttributesElt, "size-available-for-snapshots");
        final Integer percentFractionalReserve = BuilderUtil.parseIntFromChild(volumeSpaceAttributesElt, "percentage-fractional-reserve");
        final Integer percentSizeUsed = BuilderUtil.parseIntFromChild(volumeSpaceAttributesElt, "percentage-size-used");
        final EnumWrapper<Volume.SpaceMgmtOptionTryFirst> tryFirst = BuilderUtil.getEnumWrapperByValue(Volume.SpaceMgmtOptionTryFirst.class, volumeSpaceAttributesElt, "space-mgmt-option-try-first", Volume.SpaceMgmtOptionTryFirst.NOT_MAPPED);
        final Long sizeUsedBySnapshots = BuilderUtil.parseLongFromChild(volumeSpaceAttributesElt, "size-used-by-snapshots");
        final EnumWrapper<Volume.SpaceGuarantee> spaceGuarantee = BuilderUtil.getEnumWrapperByValue(Volume.SpaceGuarantee.class, volumeSpaceAttributesElt, "space-guarantee", Volume.SpaceGuarantee.NOT_MAPPED);

        // SIS attributes
        final Element volumeSISAttributesElt = volumeAttributesElt.getChild("volume-sis-attributes");
        final Boolean isSIS = BuilderUtil.parseBooleanFromChild(volumeSISAttributesElt, "is-sis-volume");
        final Long deduplicationSpaceSaved = BuilderUtil.parseLongFromChild(volumeSISAttributesElt, "deduplication-space-saved");
        final Long compressionSpaceSaved = BuilderUtil.parseLongFromChild(volumeSISAttributesElt, "compression-space-saved");
        final Integer percentageDeduplicationSpaceSaved = BuilderUtil.parseIntFromChild(volumeSISAttributesElt, "percentage-deduplication-space-saved");
        final Integer percentageCompressionSpaceSaved = BuilderUtil.parseIntFromChild(volumeSISAttributesElt, "percentage-compression-space-saved");

        // Mirror Attributes
        final Element volumeMirrorAttributesElt = volumeAttributesElt.getChild("volume-mirror-attributes");
        final Boolean isDataProtectionMirror = BuilderUtil.parseBooleanFromChild(volumeMirrorAttributesElt, "is-data-protection-mirror");
        final Boolean isLoadSharingMirror = BuilderUtil.parseBooleanFromChild(volumeMirrorAttributesElt, "is-load-sharing-mirror");
        final Boolean isMoveMirror = BuilderUtil.parseBooleanFromChild(volumeMirrorAttributesElt, "is-move-mirror");
        final Boolean isReplicaVolume = BuilderUtil.parseBooleanFromChild(volumeMirrorAttributesElt, "is-replica-volume");

        // Exports
        final Element volumeExportPolicyElt = volumeAttributesElt.getChild("volume-export-attributes");  //Optional element
        String exportPolicy = "";
        if (null != volumeExportPolicyElt) {
            exportPolicy = BuilderUtil.getChildTextTrim(volumeExportPolicyElt, "policy");
        }

        // Hybrid cache settings.
        final Element volumeHybridCacheAttributesElt = volumeAttributesElt.getChild("volume-hybrid-cache-attributes");
        final EnumWrapper<Volume.HybridCacheEligibility> hybridCacheEligibility = BuilderUtil.getEnumWrapperByValue(Volume.HybridCacheEligibility.class, volumeHybridCacheAttributesElt, "eligibility", Volume.HybridCacheEligibility.NOT_MAPPED);
        final EnumWrapper<Volume.CachingPolicy> cachingPolicy = BuilderUtil.getEnumWrapperByValue(Volume.CachingPolicy.class, volumeHybridCacheAttributesElt, "caching-policy", Volume.CachingPolicy.NOT_MAPPED);

        // Clones
        final Element cloneAttributesElt = volumeAttributesElt.getChild("volume-clone-attributes");
        String cloneParentName = null;
        if (cloneAttributesElt != null) {
            final Element cloneParentAttributesElt = cloneAttributesElt.getChild("volume-clone-parent-attributes");
            if (cloneParentAttributesElt != null) {
                cloneParentName = BuilderUtil.getChildTextTrim(cloneParentAttributesElt, "name");
            }
        }

        // volume-directory-attributes
        final Element volumeDirectoryAttrsElt = volumeAttributesElt.getChild("volume-directory-attributes");
        final Boolean isI2PEnabled = BuilderUtil.parseBooleanFromChild(volumeDirectoryAttrsElt, "i2p-enabled");

        // volume-snapshot-attributes
        final Element volumeSnapshotElement = volumeAttributesElt.getChild("volume-snapshot-attributes");
        final Boolean autoSnapshotsEnabled = BuilderUtil.parseBooleanFromChild(volumeSnapshotElement, "auto-snapshots-enabled");
        final Boolean snapdirAccessEnabled = BuilderUtil.parseBooleanFromChild(volumeSnapshotElement, "snapdir-access-enabled");
        final Boolean snapshotCloneDependencyEnabled = BuilderUtil.parseBooleanFromChild(volumeSnapshotElement, "snapshot-clone-dependency-enabled");
        final Long snapshotCount = BuilderUtil.parseLongFromChild(volumeSnapshotElement, "snapshot-count");
        final String volumeSnapshotPolicy = BuilderUtil.getChildTextTrim(volumeSnapshotElement, "snapshot-policy");

        // volume-autosize-attributes
        final Element volumeAutosizeAttributes = volumeAttributesElt.getChild("volume-autosize-attributes");
        // "mode" is supported by SN.0 (main), but not RR.1.
        // for RR.1, we use "is-enabled", and map true to GROW and false to OFF.
        // try "mode" first, if not supported then use "is-enabled"
        EnumWrapper<Volume.VolumeAutoSizeMode> autoSizeMode = BuilderUtil.getEnumWrapperByValue(Volume.VolumeAutoSizeMode.class, volumeAutosizeAttributes, "mode", Volume.VolumeAutoSizeMode.NOT_MAPPED);
        final Boolean isEnabled = BuilderUtil.parseBooleanFromChild(volumeAutosizeAttributes, "is-enabled");

        final Long autoSizeMaxSize = BuilderUtil.parseLongFromChild(volumeAutosizeAttributes, "maximum-size");
        final Long autoSizeIncrementSize = BuilderUtil.parseLongFromChild(volumeAutosizeAttributes, "increment-size");

        // volume-performance-attributes
        final Element volumePerfAttributesElt = volumeAttributesElt.getChild("volume-performance-attributes");
        final Boolean isATimeUpdateEnabled = BuilderUtil.parseBooleanFromChild(volumePerfAttributesElt, "is-atime-update-enabled");

        // volume-inode-attributes
        final Element volumeINodeAttrsElt = volumeAttributesElt.getChild("volume-inode-attributes");
        final String blockTypeRaw = BuilderUtil.getChildTextTrim(volumeINodeAttrsElt, "block-type");
        final EnumWrapper<Volume.BlockType> blockType = new EnumWrapper<Volume.BlockType>(blockTypeRaw, Volume.BlockType.fromString(blockTypeRaw));
        final Long inodeFilesTotal = BuilderUtil.parseLongFromChild(volumeINodeAttrsElt, "files-total");
        final Long inodeFilesUsed = BuilderUtil.parseLongFromChild(volumeINodeAttrsElt, "files-used");

        // volume-language-attributes
        final Element volumeLanguageAttrsElt = volumeAttributesElt.getChild("volume-language-attributes");
        final String languageCode = BuilderUtil.getChildTextTrim(volumeLanguageAttrsElt, "language-code");
        final Boolean isCreateUCodeEnabled = BuilderUtil.parseBooleanFromChild(volumeLanguageAttrsElt, "is-create-ucode-enabled");
        final Boolean isConvertUCodeEnabled = BuilderUtil.parseBooleanFromChild(volumeLanguageAttrsElt, "is-convert-ucode-enabled");

        // volume-clone-attributes
        final Element volumeCloneAttributes = volumeAttributesElt.getChild("volume-clone-attributes");
        final Integer cloneCount = BuilderUtil.parseIntFromChild(volumeCloneAttributes, "clone-child-count");

        // volume-snapshot-autodelete-attributes
        final Element snapshotAutoDeleteElt = volumeAttributesElt.getChild("volume-snapshot-autodelete-attributes");
        final Boolean isAutoDeleteEnabled = BuilderUtil.parseBooleanFromChild(snapshotAutoDeleteElt, "is-autodelete-enabled");
        final EnumWrapper<Volume.Commitment> commitment = BuilderUtil.getEnumWrapperByValue(Volume.Commitment.class, snapshotAutoDeleteElt, "commitment", Volume.Commitment.NOT_MAPPED);
        final EnumWrapper<Volume.DeleteOrder> deleteOrder = BuilderUtil.getEnumWrapperByValue(Volume.DeleteOrder.class, snapshotAutoDeleteElt, "delete-order", Volume.DeleteOrder.NOT_MAPPED);
        final EnumWrapper<Volume.DeferDelete> deferDelete = BuilderUtil.getEnumWrapperByValue(Volume.DeferDelete.class, snapshotAutoDeleteElt, "defer-delete", Volume.DeferDelete.NOT_MAPPED);
        final Integer targetFreeSpace = BuilderUtil.parseIntFromChild(snapshotAutoDeleteElt, "target-free-space");
        final EnumWrapper<Volume.Trigger> trigger = BuilderUtil.getEnumWrapperByValue(Volume.Trigger.class, snapshotAutoDeleteElt, "trigger", Volume.Trigger.NOT_MAPPED);
        final String prefix = BuilderUtil.getChildTextTrim(snapshotAutoDeleteElt, "prefix");
        final String destroyListRaw = BuilderUtil.getChildTextTrim(snapshotAutoDeleteElt, "destroy-list");
        final String[] destroyListElts = (destroyListRaw != null) ? destroyListRaw.split(",") : new String[0];
        // TODO: capture & report the raw values
        final EnumSet<Volume.DestroyList> destroyList = EnumSet.noneOf(Volume.DestroyList.class);
        for (final String destroyListEltRaw : destroyListElts) {
            final Volume.DestroyList destroyListElt = BuilderUtil.getEnumByValue(Volume.DestroyList.class, destroyListEltRaw, Volume.DestroyList.NOT_MAPPED);
            destroyList.add(destroyListElt);
        }

        // volume-vm-align-attributes
        final Element vmAlignAttributes = volumeAttributesElt.getChild("volume-vm-align-attributes");
        final Integer vmAlignSector = BuilderUtil.parseIntFromChild(vmAlignAttributes, "vm-align-sector");
        final String vmAlignSuffix = BuilderUtil.getChildTextTrim(vmAlignAttributes, "vm-align-suffix");

        // volume-flexcache-attributes
        final Element volumeFlexCacheElt = volumeAttributesElt.getChild("volume-flexcache-attributes");
        final Long flexCacheMinReserve = BuilderUtil.parseLongFromChild(volumeFlexCacheElt, "min-reserve");
        final String flexCacheOriginVolumeName = BuilderUtil.getChildTextTrim(volumeFlexCacheElt, "origin");

        final Element volumeQosElement = volumeAttributesElt.getChild("volume-qos-attributes");
        if (volumeQosElement != null) {
            qosPolicyGroupName = BuilderUtil.getChildTextTrim(volumeQosElement, "policy-group-name");
        }

        // Create and add the volume
        // Infinivols don't have uuid or instance uuid, use name
        if (volumeUUID == null) {
            volumeUUID = CModeVolume.CREATED_VOLUME_UUID_PREFIX + "." + owningVServerUUID + "." + volumeName;
        }

        final CModeVolume volume = cluster.createVolume(volumeUUID);
        volume.setInstanceUUID(volumeInstanceUUID);
        volume.setVolumeName(volumeName);
        volume.setAggregateUUID(parentAggrUUID);
        volume.setVServerUUID(owningVServerUUID);
        volume.setVolumeType(volumeType);
        volume.setSize(size);
        volume.setSizeTotal(sizeTotal);
        volume.setSizeUsed(sizeUsed);
        volume.setSizeAvail(sizeAvail);
        volume.setVServerRoot(isVServerRoot);
        volume.setState(state);
        volume.setJunctionPath(junctionPath);
        volume.setJunctionActive(isJunctionActive);
        volume.setQosPolicyGroupName(qosPolicyGroupName);
        volume.setVolumeStyle(volumeStyle);
        volume.setSISVolume(isSIS);
        volume.setDataProtectionMirror(isDataProtectionMirror);
        volume.setLoadSharingMirror(isLoadSharingMirror);
        volume.setMoveMirror(isMoveMirror);
        volume.setReplicaVolume(isReplicaVolume);
        volume.setIsSpaceGuarantee(isSpaceGuarantee);
        volume.setVmAlignSector(vmAlignSector);
        volume.setVmAlignSuffix(vmAlignSuffix);
        volume.setOverwriteReserve(overwriteReserve);
        volume.setOverwriteReserveRequired(overwriteReserveRequired);
        volume.setOverwriteReserveUsed(overwriteReserveUsed);
        volume.setOverwriteReserveActualUsed(overwriteReserveUsedActual);
        volume.setSnapshotReserveSize(snapshotReserveSize);
        volume.setPercentSnapshotReserve(percentSnapshotReserve);
        volume.setPercentSnapshotReserveUsed(percentSnapshotReserveUsed);
        volume.setSizeAvailableForSnapshots(sizeAvailableForSnapshots);
        volume.setPercentFractionalReserve(percentFractionalReserve);
        volume.setPercentSizeUsed(percentSizeUsed);
        volume.setExportPolicy(exportPolicy);
        volume.setDeduplicationSpaceSaved(deduplicationSpaceSaved);
        volume.setCompressionSpaceSaved(compressionSpaceSaved);
        volume.setPercentageDeduplicationSpaceSaved(percentageDeduplicationSpaceSaved);
        volume.setPercentageCompressionSpaceSaved(percentageCompressionSpaceSaved);
        volume.setSecurityStyle(securityStyle);
        volume.setHybridCacheEligibility(hybridCacheEligibility);
        volume.setCloneParentName(cloneParentName);
        volume.setSpaceMgmtTryFirst(tryFirst);
        volume.setSizeUsedBySnapshots(sizeUsedBySnapshots);
        volume.setI2pEnabled(isI2PEnabled);
        volume.setAutoSnapshotsEnabled(autoSnapshotsEnabled);
        volume.setSnapdirAccessEnabled(snapdirAccessEnabled);
        volume.setSnapshotCloneDependencyEnabled(snapshotCloneDependencyEnabled);
        volume.setSnapshotCount(snapshotCount);
        volume.setAutoSizeMode(autoSizeMode);
        volume.setMaxAutosizeSize(autoSizeMaxSize);
        volume.setAutosizeIncrementSize(autoSizeIncrementSize);
        volume.setEnabled(isEnabled);
        volume.setAtimeUpdateEnabled(isATimeUpdateEnabled);
        volume.setBlockType(blockType);
        volume.setInodeFilesTotal(inodeFilesTotal);
        volume.setInodeFilesUsed(inodeFilesUsed);
        volume.setLanguageCode(languageCode);
        volume.setCreateUcodeEnabled(isCreateUCodeEnabled);
        volume.setConvertUcodeEnabled(isConvertUCodeEnabled);
        volume.setCloneChildCount(cloneCount);
        volume.setAutoDeleteEnabled(isAutoDeleteEnabled);
        volume.setCommitment(commitment);
        volume.setDeleteOrder(deleteOrder);
        volume.setDeferDelete(deferDelete);
        volume.setTargetFreeSpace(targetFreeSpace);
        volume.setTrigger(trigger);
        volume.setPrefix(prefix);
        volume.setDestroyList(destroyList);
        volume.setDestroyListRaw(destroyListRaw);
        volume.setFlexCacheMinReserve(flexCacheMinReserve);
        volume.setFlexCacheOriginVolumeName(flexCacheOriginVolumeName);
        volume.setSpaceGuarantee(spaceGuarantee);
        volume.setSnapshotPolicy(volumeSnapshotPolicy);
        volume.setStorageServiceName(storageServiceName);
        volume.setEnableSnapDiff(enableSnapDiff);
        volume.setManagedByService(isManagedByService);
        volume.setMaxDataConstituentSize(maxDataConstituentSize);
        volume.setMaxNamespaceConstituentSize(maxNamespaceConstituentSize);
        volume.setConstituentRole(constituentRoleWrapper);
        volume.setConstituent(isConstituent);
        volume.setJunctionParentName(junctionParentName);
        volume.setUnixSecurityGroupID(groupID);
        volume.setUnixSecurityPermissions(permissions);
        volume.setUnixSecurityUserID(userID);
        volume.setCachingPolicyRaw(cachingPolicy.getRawValue());
        volume.setCachingPolicy(cachingPolicy.getEnumValue());
        cluster.getVolumes().add(volume);
    }


    // Get some additional volume info
    private static final class SisBuilder extends IterBuilderBase {
        private SisBuilder() {
            super("sis-get-iter", "sis-status-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
                put(new String[]{"sis-status-info",}, new String[]{"vserver", "path", "policy", "state", "status", "type", "progress", "schedule", "last-op-begin-timestamp", "last-op-end-timestamp", "last-op-error", "last-op-state", "last-op-size", "is-compression-enabled", "is-inline-compression-enabled",});
            }});
        }

        protected void buildIterInstance(final CModeCluster cluster, final Element sisStatusElt) {
            final String vServerName = BuilderUtil.getChildTextTrim(sisStatusElt, "vserver");
            final String volumePath = BuilderUtil.getChildTextTrim(sisStatusElt, "path");
            if (vServerName == null) {
                logger.warn("Skipping sis-status-info with no vserver, path: " + volumePath);
                return;
            }

            final String policyName = BuilderUtil.getChildTextTrim(sisStatusElt, "policy");
            final EnumWrapper<Volume.VolumeSisStatus> status = BuilderUtil.getEnumWrapperByValue(Volume.VolumeSisStatus.class, sisStatusElt, "status", Volume.VolumeSisStatus.NOT_MAPPED);
            final EnumWrapper<Volume.VolumeSisState> state = BuilderUtil.getEnumWrapperByValue(Volume.VolumeSisState.class, sisStatusElt, "state", Volume.VolumeSisState.NOT_MAPPED);
            final EnumWrapper<Volume.VolumeSisType> type = BuilderUtil.getEnumWrapperByValue(Volume.VolumeSisType.class, sisStatusElt, "type", Volume.VolumeSisType.NOT_MAPPED);
            final String progress = BuilderUtil.getChildTextTrim(sisStatusElt, "progress");
            final String schedule = BuilderUtil.getChildTextTrim(sisStatusElt, "schedule");
            final Long lastOpBeginTS = BuilderUtil.parseLongFromChild(sisStatusElt, "last-op-begin-timestamp");
            final Long lastOpEndTS = BuilderUtil.parseLongFromChild(sisStatusElt, "last-op-end-timestamp");
            final String lastOpError = BuilderUtil.getChildTextTrim(sisStatusElt, "last-op-error");
            final EnumWrapper<Volume.VolumeSisOpState> lastOpState = BuilderUtil.getEnumWrapperByValue(Volume.VolumeSisOpState.class, sisStatusElt, "last-op-state", Volume.VolumeSisOpState.NOT_MAPPED);
            final Long lastOpSize = BuilderUtil.parseLongFromChild(sisStatusElt, "last-op-size");
            final Boolean isCompressionEnabled = BuilderUtil.parseBooleanFromChild(sisStatusElt, "is-compression-enabled");
            final Boolean isInlineCompressionEnabled = BuilderUtil.parseBooleanFromChild(sisStatusElt, "is-inline-compression-enabled");

            final CModeSisStatus sisStatus = cluster.createSisStatus(vServerName, volumePath);
            sisStatus.setSisPolicyName(policyName);
            sisStatus.setStatus(status);
            sisStatus.setState(state);
            sisStatus.setType(type);
            sisStatus.setProgress(progress);
            sisStatus.setSchedule(schedule);
            sisStatus.setLastOpBeginTS(lastOpBeginTS);
            sisStatus.setLastOpEndTS(lastOpEndTS);
            sisStatus.setLastOpError(lastOpError);
            sisStatus.setLastOpState(lastOpState);
            sisStatus.setLastOpSize(lastOpSize);
            sisStatus.setCompressionEnabled(isCompressionEnabled);
            sisStatus.setInlineCompressionEnabled(isInlineCompressionEnabled);
            cluster.getSisStatuses().add(sisStatus);
        }
    }

    private static final class VolumeCloneBuilder extends IterBuilderBase {

        private VolumeCloneBuilder() {
            super("volume-clone-get-iter", "volume-clone-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
                put(new String[]{"volume-clone-info",}, new String[]{"vserver", "volume", "split-estimate",});
            }});
        }

        protected void buildIterInstance(final CModeCluster cluster, final Element volumeCloneInfoElt) {
            final String vServerName = BuilderUtil.getChildTextTrim(volumeCloneInfoElt, "vserver");
            final String volumeName = BuilderUtil.getChildTextTrim(volumeCloneInfoElt, "volume");
            final Long cloneSplitEstimate = BuilderUtil.parseLongFromChild(volumeCloneInfoElt, "split-estimate");
            final CModeVolumeClone volumeClone = cluster.createVolumeClone(vServerName, volumeName);
            volumeClone.setCloneSplitEstimate(cloneSplitEstimate);
            cluster.getVolumeClones().add(volumeClone);
        }
    }
}
