package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeVolume;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Volume;
import org.jdom.Element;

import java.util.LinkedHashMap;

public class VolumeRuntimeBuilder extends RuntimeIterBuilder
{
  public VolumeRuntimeBuilder()
  {
    super( "volume-get-iter", "volume-attributes", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>()
    {{
        put(new String[]{"volume-attributes", "volume-id-attributes"}, new String[]{"uuid", "owning-vserver-uuid"} );
        put(new String[]{"volume-attributes", "volume-space-attributes"}, new String[]{"size", "size-total", "size-used", "size-available", "is-space-guarantee-enabled", "overwrite-reserve", "overwrite-reserve-required", "overwrite-reserve-used", "overwrite-reserve-used-actual", "snapshot-reserve-size", "percentage-snapshot-reserve", "percentage-snapshot-reserve-used", "size-available-for-snapshots", "percentage-fractional-reserve", "percentage-size-used", "space-guarantee", "space-mgmt-option-try-first", "size-used-by-snapshots"} );
        put(new String[]{"volume-attributes", "volume-state-attributes"}, new String[]{"state"});
        put(new String[]{"volume-attributes", "volume-sis-attributes"}, new String[]{"deduplication-space-saved", "compression-space-saved", "percentage-deduplication-space-saved", "percentage-compression-space-saved"});
        put(new String[]{"volume-attributes", "volume-autosize-attributes",}, new String[]{"maximum-size", "increment-size"});
        put(new String[]{"volume-attributes", "volume-snapshot-autodelete-attributes",}, new String[]{ "target-free-space"});
        put(new String[]{"volume-attributes", "volume-inode-attributes",}, new String[]{ "files-total", "files-used"});
      }
    });
  }

  @Override
  protected void buildIterInstance(CModeCluster cluster, Element volumeAttributes)
  {
    if( volumeAttributes != null )
    {
      //get volume name
      String volName = null;
      String volumeUUID = null;
      Element volumeIdAttributes = volumeAttributes.getChild( "volume-id-attributes" );
      if( volumeIdAttributes != null )
      {
        Element volNameElt = volumeIdAttributes.getChild( "name" );
        if( volNameElt != null )
          volName = volNameElt.getText();

        //regular volumes have UUIDs
        Element volumeUUIDElt = volumeIdAttributes.getChild( "uuid" );
        if( volumeUUIDElt != null )
          volumeUUID = volumeUUIDElt.getText();

        // Infinivols don't have uuid or instance uuid, use name
        if( volumeUUID == null )
        {
          Element owningVserverUUIDElt = volumeIdAttributes.getChild( "owning-vserver-uuid" );
          if( owningVserverUUIDElt != null )
            volumeUUID = CModeVolume.CREATED_VOLUME_UUID_PREFIX + "." + owningVserverUUIDElt.getText() + "." + volName;
        }
      }

      //make sure we have the valid id's
      if( volName != null && volumeUUID != null )
      {
        final UUIDKey cModeVolumeKey = new UUIDKey( "volume", volumeUUID );
        final CModeVolume volume = cluster.getVolumes().get( cModeVolumeKey );

        //make sure the volume exists
        if( volume != null )
        {
          //set the volume-space-attributes runtime values for this volume
          Element volumeSpaceAttributes = volumeAttributes.getChild( "volume-space-attributes" );
          if( volumeSpaceAttributes != null )
          {
            final Long size = BuilderUtil.parseLongFromChild( volumeSpaceAttributes, "size" );
            final Long sizeTotal = BuilderUtil.parseLongFromChild( volumeSpaceAttributes, "size-total" );
            final Long sizeUsed = BuilderUtil.parseLongFromChild( volumeSpaceAttributes, "size-used" );
            final Long sizeUsedBySnapshots = BuilderUtil.parseLongFromChild( volumeSpaceAttributes, "size-used-by-snapshots" );
            final Long sizeAvailableForSnapshots = BuilderUtil.parseLongFromChild( volumeSpaceAttributes, "size-available-for-snapshots" );
            final Long sizeAvail = BuilderUtil.parseLongFromChild(volumeSpaceAttributes, "size-available");

            final Integer percentSizeUsed = BuilderUtil.parseIntFromChild( volumeSpaceAttributes, "percent-used-capacity" );
            final Integer percentFractionalReserve = BuilderUtil.parseIntFromChild( volumeSpaceAttributes, "percentage-fractional-reserve" );
            final Integer percentSnapshotReserve = BuilderUtil.parseIntFromChild( volumeSpaceAttributes, "percentage-snapshot-reserve" );
            final Integer percentSnapshotReserveUsed = BuilderUtil.parseIntFromChild( volumeSpaceAttributes, "percentage-snapshot-reserve-used" );

            final Long overwriteReserveRequired = BuilderUtil.parseLongFromChild( volumeSpaceAttributes, "overwrite-reserve-required" );
            final Long overwriteReserveUsedActual = BuilderUtil.parseLongFromChild( volumeSpaceAttributes, "overwrite-reserve-used-actual" );

            volume.setSize( size );
            volume.setSizeTotal( sizeTotal );
            volume.setSizeUsed( sizeUsed );
            volume.setSizeAvail( sizeAvail );
            volume.setSizeAvailableForSnapshots( sizeAvailableForSnapshots );
            volume.setSizeUsedBySnapshots( sizeUsedBySnapshots );
            volume.setPercentSnapshotReserve( percentSnapshotReserve );
            volume.setPercentSnapshotReserveUsed( percentSnapshotReserveUsed );
            volume.setPercentSizeUsed( percentSizeUsed );
            volume.setPercentFractionalReserve( percentFractionalReserve );
            volume.setOverwriteReserveRequired( overwriteReserveRequired );
            volume.setOverwriteReserveActualUsed( overwriteReserveUsedActual );
          }

          //set the volume-state-attributes runtime values for this volume
          Element volumeStateAttributes = volumeAttributes.getChild( "volume-state-attributes" );
          if( volumeStateAttributes != null )
          {
            final EnumWrapper<Volume.State> state = BuilderUtil.getEnumWrapperByValue(Volume.State.class, volumeStateAttributes, "state", Volume.State.NOT_MAPPED);
            final Long stateRaw = BuilderUtil.parseLongFromChild( volumeStateAttributes, "state" );

            volume.setState( state );
//              volume.setState( stateRaw ); //add raw in future
          }

          //set the volume-sis-attributes runtime values for this volume
          Element volumeSISAttributes = volumeAttributes.getChild( "volume-sis-attributes" );
          if( volumeSISAttributes != null )
          {
            final Long deduplicationSpaceSaved = BuilderUtil.parseLongFromChild(volumeSISAttributes, "deduplication-space-saved");
            final Long compressionSpaceSaved = BuilderUtil.parseLongFromChild(volumeSISAttributes, "compression-space-saved");
            final Integer percentageDeduplicationSpaceSaved = BuilderUtil.parseIntFromChild(volumeSISAttributes, "percentage-deduplication-space-saved");
            final Integer percentageCompressionSpaceSaved = BuilderUtil.parseIntFromChild(volumeSISAttributes, "percentage-compression-space-saved");

            volume.setDeduplicationSpaceSaved(deduplicationSpaceSaved);
            volume.setCompressionSpaceSaved(compressionSpaceSaved);
            volume.setPercentageDeduplicationSpaceSaved(percentageDeduplicationSpaceSaved);
            volume.setPercentageCompressionSpaceSaved(percentageCompressionSpaceSaved);
          }

          //set the volume-autosize-attributes runtime values for this volume
          final Element volumeAutosizeAttributes = volumeAttributes.getChild("volume-autosize-attributes");
          if( volumeAutosizeAttributes != null )
          {
            final Long autoSizeMaxSize = BuilderUtil.parseLongFromChild(volumeAutosizeAttributes, "maximum-size");
            final Long autoSizeIncrementSize = BuilderUtil.parseLongFromChild(volumeAutosizeAttributes, "increment-size");

            volume.setMaxAutosizeSize(autoSizeMaxSize);
            volume.setAutosizeIncrementSize(autoSizeIncrementSize);
          }

          //set the volume-inode-attributes runtime values for this volume
          final Element volumeINodeAttrs = volumeAttributes.getChild("volume-inode-attributes");
          if( volumeINodeAttrs != null )
          {
            final Long inodeFilesTotal = BuilderUtil.parseLongFromChild(volumeINodeAttrs, "files-total");
            final Long inodeFilesUsed = BuilderUtil.parseLongFromChild(volumeINodeAttrs, "files-used");

            volume.setInodeFilesTotal(inodeFilesTotal);
            volume.setInodeFilesUsed(inodeFilesUsed);
          }

          //set the volume-snapshot-autodelete-attributes runtime values for this volume
          final Element snapshotAutoDelete = volumeAttributes.getChild("volume-snapshot-autodelete-attributes");
          if( snapshotAutoDelete != null )
          {
            final Integer targetFreeSpace = BuilderUtil.parseIntFromChild(snapshotAutoDelete, "target-free-space");
            volume.setTargetFreeSpace(targetFreeSpace);
          }
        }
      }
    }
  }
}
