package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;


import com.netapp.oci.netapp.performance.node.interfaces.data.CIFSNodeMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappCIFSNodeMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class CIFSNodeMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
        CIFSNodeMeasurements,
        CIFSNodeMeasurements,
        NetappCIFSNodeMetrics,
        NetappCIFSNodeMetrics.CIFSNodeCounter,
        NetappCIFSNodeMetrics.CIFSNodeCounter> {

    public CIFSNodeMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "cifs:node", NetappCIFSNodeMetrics.CIFSNodeCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(NetAppDefaultStorageContext context, ZAPIConnection zapiConnection) {
        return new ArrayList<String>(context.getCifsPerfUUIDtoNodeUUID().keySet());
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(NetAppDefaultStorageContext context, String perfInstanceUUID) {
        return context.getKey();    // Parent of each node is the cluster
    }

    @Override
    String getIventoryUUIDFromPerfUUID(NetAppDefaultStorageContext context, String perfInstanceUUID) {
        return context.getCifsPerfUUIDtoNodeUUID().get(perfInstanceUUID);
    }

    @Override
    NetappCIFSNodeMetrics createMetricsFromValues(MetricsRowScratchPad<NetappCIFSNodeMetrics.CIFSNodeCounter> currentValues, MetricsRowScratchPad<NetappCIFSNodeMetrics.CIFSNodeCounter> previousValues) {
        return new NetappCIFSNodeMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, Map<Cluster.ClusterKey, Collection<RawSample<CIFSNodeMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<CIFSNodeMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<CIFSNodeMeasurements>> nodeMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportCIFSNodePerformance(key, nodeMeasurements);
        }
    }
}
