package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.disk.interfaces.data.DiskMeasurements;
import com.netapp.oci.netapp.performance.disk.interfaces.data.DiskRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappDiskMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import org.apache.log4j.Logger;

import java.util.Collection;
import java.util.List;
import java.util.Map;

public class DiskMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
        DiskMeasurements,
        DiskRawMeasurements,
        NetappDiskMetrics,
        NetappDiskMetrics.DiskCounter,
        NetappDiskMetrics.DiskCounter> {
    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(DiskMetricsBuilder.class);

    public DiskMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "disk", NetappDiskMetrics.DiskCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection connection) {
        return context.getDiskUUIDs();
    }

    @Override
    NetappDiskMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappDiskMetrics.DiskCounter> currentValues, final MetricsRowScratchPad<NetappDiskMetrics.DiskCounter> previousValues) {
        return new NetappDiskMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<DiskMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<DiskMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<DiskMeasurements>> diskMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportDiskPerformance(key, diskMeasurements);
        }
    }

    @Override
    void reportRawPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<DiskRawMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<DiskRawMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<DiskRawMeasurements>> diskMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportDiskRawPerformance(key, diskMeasurements);
        }
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return perfInstanceUUID;                    // Same UUID used for inventory and performance
    }
}
