package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappFcpLifVserverMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

public class FcpLifVserverMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
	FcpLifVserverMeasurements,
	FcpLifVserverRawMeasurements,
	NetappFcpLifVserverMetrics,
	NetappFcpLifVserverMetrics.FcpLifVserverCounter,
	NetappFcpLifVserverMetrics.FcpLifVserverCounter> {

	private static final Logger logger = LoggerFactory.getLogger(FcpLifVserverMetricsBuilder.class);

	public FcpLifVserverMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
		super(reportingPackage, "fcp_lif:vserver", NetappFcpLifVserverMetrics.FcpLifVserverCounter.class);
	}

    @Override
	List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
		final Map<String, String> fcpLifNameToVServerUUIDMap = context.getFcpLifNameToVServerUUIDMap();

		// perf-object-get-instances for fcp_lif:vserver uses vserver UUID for ZAPI invocation.
		Set<String> vserverUUIDs = new HashSet<>();
		vserverUUIDs.addAll(fcpLifNameToVServerUUIDMap.values());
		
		logger.debug("Collect fcp_lif:vserver stats for: " + vserverUUIDs);
		return new ArrayList<String>(vserverUUIDs);
	}

	@Override
	ClusterKey getParentInventoryUUIDKey(NetAppDefaultStorageContext context, String perfInstanceUUID) {
		return context.getKey();
	}

	@Override
	String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
		return perfInstanceUUID;
	}

	@Override
	NetappFcpLifVserverMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappFcpLifVserverMetrics.FcpLifVserverCounter> currentValues, final MetricsRowScratchPad<NetappFcpLifVserverMetrics.FcpLifVserverCounter> previousValues) {
		return new NetappFcpLifVserverMetrics(currentValues, previousValues);
	}

	@Override
	void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<FcpLifVserverMeasurements>>> measurements) throws DataSourceErrorException {
		for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<FcpLifVserverMeasurements>>> measurementEntry : measurements.entrySet()) {
			final Cluster.ClusterKey key = measurementEntry.getKey();
			final Collection<RawSample<FcpLifVserverMeasurements>> fcpLifMeasurements = measurementEntry.getValue();
			storagePerformanceReporter.reportFcpLifVserverPerformance(key, fcpLifMeasurements);
		}
	}

	@Override
	void reportRawPerformance(Cluster.ClusterKey clusterKey, 
			final Map<Cluster.ClusterKey, Collection<RawSample<FcpLifVserverRawMeasurements>>> measurements) throws DataSourceErrorException {
		for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<FcpLifVserverRawMeasurements>>> measurementEntry : measurements.entrySet()) {
			final Cluster.ClusterKey key = measurementEntry.getKey();
			final Collection<RawSample<FcpLifVserverRawMeasurements>> fcpLifMeasurements = measurementEntry.getValue();
			storagePerformanceReporter.reportFcpLifVserverRawPerformance(key, fcpLifMeasurements);
		}
	}

}
