package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import com.netapp.oci.netapp.performance.hostadapter.interfaces.data.HostadapterMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetAppHostadapterMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetAppHostadapterMetrics.HostadapterCounter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

public class HostadapterMetricsBuilder extends MetricsBuilderBase<
		Cluster.ClusterKey,
        HostadapterMeasurements,
        HostadapterMeasurements,
        NetAppHostadapterMetrics,
        NetAppHostadapterMetrics.HostadapterCounter,
        NetAppHostadapterMetrics.HostadapterCounter>{

	private static final Logger logger = LoggerFactory.getLogger(HostadapterMetricsBuilder.class);
	
    public HostadapterMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage,"hostadapter", NetAppHostadapterMetrics.HostadapterCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        return context.getStorageAdapterPerfUUIDs();
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();
    }

    @Override
    public String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
    	Map<String, String> nodeNameToUUID = context.getNodeNameToUUIDMap();
    	String[] tokens = perfInstanceUUID.split(":");
    	//Expected tokens should be in the format of "nodeName:kernel:adapterName"
    	if(tokens.length == 3){
        	String nodeName = tokens[0];
        	String adapterName = tokens[2];
        	if(!StringUtils.isEmpty(nodeName) && !StringUtils.isEmpty(adapterName)) {
            	String nodeUUID = nodeNameToUUID.get(nodeName);
            	if(!StringUtils.isEmpty(nodeUUID)){
            		//nodeuuid.adapterName
            		return new StringBuilder(nodeUUID).append(".").append(adapterName).toString();
            	}
        	}
        	logger.warn((new StringBuilder(
    				"Cannot formulate Inventory UUID as one of the required field is null [ node name = "))
    				.append(nodeName).append("adapterName = ").append(adapterName)
    				.append("nodeUUId = ").append(nodeNameToUUID.get(nodeName))
    				.toString());
    	}else{
    		logger.warn((new StringBuilder("Cannot formulate Inventory UUID as performance instance UUId is expected in format \"<nodeName>:kernel:<adapterName>\" but was: ")
    					.append(perfInstanceUUID).toString()));
    	}
    	return null;
    }

	@Override
	NetAppHostadapterMetrics createMetricsFromValues(
			MetricsRowScratchPad<HostadapterCounter> currentValues,
			MetricsRowScratchPad<HostadapterCounter> previousValues) {
		return new NetAppHostadapterMetrics(currentValues, previousValues);
	}

	@Override
	void reportPerformance(
			ClusterKey clusterKey,
			Map<ClusterKey, Collection<RawSample<HostadapterMeasurements>>> measurements)
			throws DataSourceErrorException {
      for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<HostadapterMeasurements>>> measurementEntry : measurements.entrySet()) {
	      final Cluster.ClusterKey key = measurementEntry.getKey();
	      final Collection<RawSample<HostadapterMeasurements>> HostadapterMeasurements = measurementEntry.getValue();
	      storagePerformanceReporter.reportHostadapterPerformance(key, HostadapterMeasurements);
	  }
		
	}

}
