package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifVserverMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifVserverRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappIscsiLifVserverMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

public class IscsiLifVserverMetricsBuilder extends MetricsBuilderBase<ClusterKey,
	IscsiLifVserverMeasurements,
	IscsiLifVserverRawMeasurements,
	NetappIscsiLifVserverMetrics,
	NetappIscsiLifVserverMetrics.IscsiLifVserverCounter,
	NetappIscsiLifVserverMetrics.IscsiLifVserverCounter> {
	
	private static final Logger logger = LoggerFactory.getLogger(ISCSILIFMetricsBuilder.class);

    public IscsiLifVserverMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage,"iscsi_lif:vserver", NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        final Map<String, String> networkLifNameToVServerUUIDMap = context.getNetworkLifNameToVServerUUIDMap();

        // perf-object-get-instances for iscsi_lif:vserver uses vserver UUID for ZAPI invocation.
		Set<String> vserverUUIDs = new HashSet<>();
		vserverUUIDs.addAll(networkLifNameToVServerUUIDMap.values());
		
		logger.debug("Collect iscsi_lif:vserver stats for: " + vserverUUIDs);
		return new ArrayList<String>(vserverUUIDs);
    }

    @Override
    ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return perfInstanceUUID;
    }

    @Override
    NetappIscsiLifVserverMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappIscsiLifVserverMetrics.IscsiLifVserverCounter> currentValues, final MetricsRowScratchPad<NetappIscsiLifVserverMetrics.IscsiLifVserverCounter> previousValues) {
        return new NetappIscsiLifVserverMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(ClusterKey clusterKey, final Map<ClusterKey, Collection<RawSample<IscsiLifVserverMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<ClusterKey, Collection<RawSample<IscsiLifVserverMeasurements>>> measurementEntry : measurements.entrySet()) {
            final ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<IscsiLifVserverMeasurements>> iscsiLifMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportIscsiLifVserverPerformance(key, iscsiLifMeasurements);
        }
    }

    @Override
    void reportRawPerformance(ClusterKey clusterKey, final Map<ClusterKey, Collection<RawSample<IscsiLifVserverRawMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<ClusterKey, Collection<RawSample<IscsiLifVserverRawMeasurements>>> measurementEntry : measurements.entrySet()) {
            final ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<IscsiLifVserverRawMeasurements>> iscsiLifMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportIscsiLifVserverRawPerformance(key, iscsiLifMeasurements);
        }
    }
    
}
