package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.lun.interfaces.data.LunMeasurements;
import com.netapp.oci.netapp.performance.lun.interfaces.data.LunRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLunMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import org.apache.log4j.Logger;

import java.util.Collection;
import java.util.List;
import java.util.Map;

public class LunMetricsBuilder extends MetricsBuilderBase<VServer.VServerKey,
        LunMeasurements,
        LunRawMeasurements,
        NetappLunMetrics,
        NetappLunMetrics.LunCounter,
        NetappLunMetrics.LunCounter> {

    private static final Logger logger = Logger.getLogger(LunMetricsBuilder.class);

    public LunMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "lun", NetappLunMetrics.LunCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection connection) {
        return context.getLunUUIDs();
    }

    @Override
    VServer.VServerKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        final String volumeUUID = context.getLunUUIDtoVolumeUUIDMap().get(perfInstanceUUID);
        if (volumeUUID == null) {
            logger.warn(PackageContext.getLogPrefix() + " - Failed to find volume for lun " + perfInstanceUUID);
            return null;
        }
        final String vServerUUID = context.getVolumeVServerUUIDMap().get(volumeUUID);
        if (vServerUUID == null) {
            logger.warn(PackageContext.getLogPrefix() + " - Failed to find vServer for Volume " + volumeUUID + " for lun " + perfInstanceUUID);
            return null;
        }
        return new VServer.VServerKey(context.getKey().getID(Cluster.ClusterKey.Field.cluster_uuid), vServerUUID);
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return perfInstanceUUID;
    }

    @Override
    NetappLunMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappLunMetrics.LunCounter> currentValues, final MetricsRowScratchPad<NetappLunMetrics.LunCounter> previousValues) {
        return new NetappLunMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<VServer.VServerKey, Collection<RawSample<LunMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<VServer.VServerKey, Collection<RawSample<LunMeasurements>>> measurementEntry : measurements.entrySet()) {
            final VServer.VServerKey key = measurementEntry.getKey();
            final Collection<RawSample<LunMeasurements>> volumeMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportLunPerformance(clusterKey, key, volumeMeasurements);
        }
    }
    
    @Override
    void reportRawPerformance(Cluster.ClusterKey clusterKey, final Map<VServer.VServerKey, Collection<RawSample<LunRawMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<VServer.VServerKey, Collection<RawSample<LunRawMeasurements>>> measurementEntry : measurements.entrySet()) {
            final VServer.VServerKey key = measurementEntry.getKey();
            final Collection<RawSample<LunRawMeasurements>> volumeMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportLunRawPerformance(clusterKey, key, volumeMeasurements);
        }
    }

}
