package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;


import com.netapp.oci.netapp.performance.node.interfaces.data.NFSv3NodeMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNFSv3NodeMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class NFSv3NodeMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
        NFSv3NodeMeasurements,
        NFSv3NodeMeasurements,
        NetappNFSv3NodeMetrics,
        NetappNFSv3NodeMetrics.NFSv3NodeCounter,
        NetappNFSv3NodeMetrics.OCIENFSv3NodeCounter> {


    public NFSv3NodeMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "nfsv3:node", NetappNFSv3NodeMetrics.NFSv3NodeCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        return new ArrayList<String>(context.getNfsv3PerfUUIDtoNodeUUID().keySet());
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();        // Parent of each node is the cluster
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getNfsv3PerfUUIDtoNodeUUID().get(perfInstanceUUID);
    }

    @Override
    NetappNFSv3NodeMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappNFSv3NodeMetrics.NFSv3NodeCounter> currentValues,
                                                   final MetricsRowScratchPad<NetappNFSv3NodeMetrics.NFSv3NodeCounter> previousValues) {
        return new NetappNFSv3NodeMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<NFSv3NodeMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<NFSv3NodeMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<NFSv3NodeMeasurements>> nodeMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportNFSv3NodePerformance(key, nodeMeasurements);
        }
    }
}
