package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;


import com.netapp.oci.netapp.performance.node.interfaces.data.NFSv4NodeMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNFSv4NodeMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class NFSv4NodeMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
        NFSv4NodeMeasurements,
        NFSv4NodeMeasurements,
        NetappNFSv4NodeMetrics,
        NetappNFSv4NodeMetrics.NFSv4NodeCounter,
        NetappNFSv4NodeMetrics.OCIENFSv4NodeCounter> {


    public NFSv4NodeMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "nfsv4:node", NetappNFSv4NodeMetrics.NFSv4NodeCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        return new ArrayList<String>(context.getNfsv4PerfUUIDtoNodeUUID().keySet());
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();        // Parent of each node is the cluster
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context,  final String perfInstanceUUID) {
        return context.getNfsv4PerfUUIDtoNodeUUID().get(perfInstanceUUID);
    }

    @Override
    NetappNFSv4NodeMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappNFSv4NodeMetrics.NFSv4NodeCounter> currentValues,
                                                   final MetricsRowScratchPad<NetappNFSv4NodeMetrics.NFSv4NodeCounter> previousValues) {
        return new NetappNFSv4NodeMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<NFSv4NodeMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<NFSv4NodeMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<NFSv4NodeMeasurements>> nodeMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportNFSv4NodePerformance(key, nodeMeasurements);
        }
    }
}
