package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv4Measurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv4RawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNfsv4Metrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNfsv4Metrics.Nfsv4Counter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class Nfsv4MetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
		Nfsv4Measurements,
		Nfsv4RawMeasurements,
		NetappNfsv4Metrics,
		NetappNfsv4Metrics.Nfsv4Counter,
		NetappNfsv4Metrics.Nfsv4Counter>{

	public Nfsv4MetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
		super(reportingPackage, "nfsv4", NetappNfsv4Metrics.Nfsv4Counter.class);
	}

	@Override
	List<String> getTargetPerfUUIDs(NetAppDefaultStorageContext context, ZAPIConnection zapiConnection) {
		return new ArrayList<String>(context.getNfsv4VserverIDToUUIDMap().keySet());
	}

	@Override
	ClusterKey getParentInventoryUUIDKey(NetAppDefaultStorageContext context, String perfInstanceUUID) {
		return context.getKey();
	}

	@Override
	String getIventoryUUIDFromPerfUUID(NetAppDefaultStorageContext context, String perfInstanceUUID) {
		return context.getNfsv4VserverIDToUUIDMap().get(perfInstanceUUID);
	}

	@Override
	NetappNfsv4Metrics createMetricsFromValues(
			MetricsRowScratchPad<Nfsv4Counter> currentValues,
			MetricsRowScratchPad<Nfsv4Counter> previousValues) {
		return new NetappNfsv4Metrics(currentValues, previousValues);	
	}
	
	@Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<Nfsv4Measurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<Nfsv4Measurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<Nfsv4Measurements>> nfsv4Measurements = measurementEntry.getValue();
            storagePerformanceReporter.reportNfsv4Performance(key, nfsv4Measurements);
        }
    }

	@Override
	void reportRawPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<Nfsv4RawMeasurements>>> measurements) throws DataSourceErrorException {
		for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<Nfsv4RawMeasurements>>> measurementEntry : measurements.entrySet()) {
			final Cluster.ClusterKey key = measurementEntry.getKey();
			final Collection<RawSample<Nfsv4RawMeasurements>> nfsv4Measurements = measurementEntry.getValue();
			storagePerformanceReporter.reportNfsv4RawPerformance(key, nfsv4Measurements);
		}
	}

}
