package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.qosworkload.interfaces.data.QosVolumeWorkloadMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappQosVolumeWorkloadMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class QosVolumeWorkloadMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
                                                                        QosVolumeWorkloadMeasurements,
                                                                        QosVolumeWorkloadMeasurements,
                                                                        NetappQosVolumeWorkloadMetrics,
                                                                        NetappQosVolumeWorkloadMetrics.QosVolumeWorkloadCounter,
                                                                        NetappQosVolumeWorkloadMetrics.OCIEWorkloadCounter> {

    public QosVolumeWorkloadMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "workload", NetappQosVolumeWorkloadMetrics.QosVolumeWorkloadCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection connection) {
    	return context.getQosVolumeWorkloadInstanceUUIDs();
    }

    @Override
    NetappQosVolumeWorkloadMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappQosVolumeWorkloadMetrics.QosVolumeWorkloadCounter> currentValues, final MetricsRowScratchPad<NetappQosVolumeWorkloadMetrics.QosVolumeWorkloadCounter> previousValues) {
        return new NetappQosVolumeWorkloadMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<QosVolumeWorkloadMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<QosVolumeWorkloadMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<QosVolumeWorkloadMeasurements>> workloadMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportQosVolumeWorkloadPerformance(key, workloadMeasurements);
        }
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey().getID(Cluster.ClusterKey.Field.uuid) + '.' + perfInstanceUUID;
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfGlobalUUID) {
        return context.getKey();
    }

    @Override
    protected MetricsRowScratchPad<NetappQosVolumeWorkloadMetrics.QosVolumeWorkloadCounter> getMetricsScratchPad(final Class<NetappQosVolumeWorkloadMetrics.QosVolumeWorkloadCounter> enumClass,
                                                                                                  final String perfInstanceUUID, final String inventoryUUID,
                                                                                                  final long serverPollStartTimeStamp, final long pollStartTimeStamp,
                                                                                                  final long zapiStartTimeStamp, final long zapiResponseTimestamp,
                                                                                                  final MetricsRowScratchPad<NetappQosVolumeWorkloadMetrics.QosVolumeWorkloadCounter> previousValues) {

        final List<String> discoveredReadIOColumnNames = getVolumeWorkloadReadIOColumns();
        return new NetappQosVolumeWorkloadMetrics.WorkloadRowMetricsScratchPad(perfInstanceUUID, inventoryUUID,
                serverPollStartTimeStamp,pollStartTimeStamp, previousValues, discoveredReadIOColumnNames,metricsDebugEnabled);
    }


    private List<String>  getVolumeWorkloadReadIOColumns() {
        final List<String> readIOColumnNames;
        final NetAppDefaultStorageContext context =((NetAppOCIEPerformancePackage)storagePerformanceReporter).getNetAppDefaultStorageContext();
        if (context != null) {
            final List<String> readIOColumnNameCandidates = context.getVolumeWorkloadReadIOColumns();
            readIOColumnNames = (readIOColumnNameCandidates != null)
                    ? readIOColumnNameCandidates
                    : new ArrayList<String>(0);
        } else {
            // No previous foundation poll
            readIOColumnNames = new ArrayList<String>(0);
        }
        return readIOColumnNames;
    }
}
