package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.qosworkloadconstituent.interfaces.data.QosWorkloadConstituentMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappQosWorkloadConstituentMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappQosWorkloadConstituentMetrics.WorkloadConstituentRowMetricsScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class QosWorkloadConstituentMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
        QosWorkloadConstituentMeasurements,
        QosWorkloadConstituentMeasurements,
        NetappQosWorkloadConstituentMetrics,
        NetappQosWorkloadConstituentMetrics.QosWorkloadConstituentCounter,
        NetappQosWorkloadConstituentMetrics.QosWorkloadConstituentCounter> {

    Map<String, QosWorkloadConstituentMeasurements> qosWorkloadConstituentMeasurements;
    private static final Logger LOG = LoggerFactory.getLogger(QosWorkloadConstituentMetricsBuilder.class);    
    
    public QosWorkloadConstituentMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "workload:constituent", NetappQosWorkloadConstituentMetrics.QosWorkloadConstituentCounter.class);
        qosWorkloadConstituentMeasurements = new HashMap<String, QosWorkloadConstituentMeasurements>();
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        return context.getQosWorkloadConstituentUUIDs();
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        String nodeName = perfInstanceUUID.substring(0,perfInstanceUUID.indexOf(";"));
        String workloadUUID = perfInstanceUUID.substring(perfInstanceUUID.indexOf(";")+1, perfInstanceUUID.length());
        String inventoryInstanceUUID = context.getNodeNameToUUIDMap().get(nodeName) + "." + workloadUUID; 
        return inventoryInstanceUUID;
    }

    @Override
    NetappQosWorkloadConstituentMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappQosWorkloadConstituentMetrics.QosWorkloadConstituentCounter> currentValues, final MetricsRowScratchPad<NetappQosWorkloadConstituentMetrics.QosWorkloadConstituentCounter> previousValues) {
        return new NetappQosWorkloadConstituentMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<QosWorkloadConstituentMeasurements>>> measurements) throws DataSourceErrorException {
        // we do not report workload constituent measurements, instead queue dblade and queue nblade measurements derived from constituent workloads are reported.
        // clear previous workload constituent measurements from cache and update it with current measurement sample for use by queue dblade and queue nblade builders.
        qosWorkloadConstituentMeasurements.clear();
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<QosWorkloadConstituentMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Collection<RawSample<QosWorkloadConstituentMeasurements>> workloadConstituentMeasurements = measurementEntry.getValue();
            for(final RawSample<QosWorkloadConstituentMeasurements> sample : workloadConstituentMeasurements){
                qosWorkloadConstituentMeasurements.put(sample.getKey(), sample.getCounters());
            }
            LOG.debug("Qos workload constituent measurements are updated with current poll. Cluster Id: {}", measurementEntry.getKey());
        }
    }

    public Map<String, QosWorkloadConstituentMeasurements> getQosWorkloadConstituentMeasurements(){
        return qosWorkloadConstituentMeasurements;
    }
    
    @Override
    protected WorkloadConstituentRowMetricsScratchPad getMetricsScratchPad(final Class<NetappQosWorkloadConstituentMetrics.QosWorkloadConstituentCounter> enumClass,
                                                                                                                 final String perfInstanceUUID, final String inventoryUUID,
                                                                                                                 final long serverPollStartTimeStamp, final long pollStartTimeStamp,
                                                                                                                 final long zapiStartTimeStamp, final long zapiResponseTimestamp,
                                                                                                                 final MetricsRowScratchPad<NetappQosWorkloadConstituentMetrics.QosWorkloadConstituentCounter> previousValues) {

         // Overriding the MetricRowScratchPad, to include the new column headers
    	List<String> readIOColumns = getVolumeWorkloadReadIOColumns();
        return new WorkloadConstituentRowMetricsScratchPad(perfInstanceUUID, inventoryUUID, serverPollStartTimeStamp,
                pollStartTimeStamp, previousValues, metricsDebugEnabled, readIOColumns);
    	}
    
    protected List<String> getVolumeWorkloadReadIOColumns(){
    	final List<String> volumeWorkloadReadIOColumns;
        final NetAppDefaultStorageContext context =((NetAppOCIEPerformancePackage)storagePerformanceReporter).getNetAppDefaultStorageContext();
        if (context != null) {
        	volumeWorkloadReadIOColumns = context.getVolumeWorkloadReadIOColumns();
        }
        else {
        	volumeWorkloadReadIOColumns = new ArrayList<String>(0);
        }
       return volumeWorkloadReadIOColumns; 
    }
 }
