package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.qosworkloadconstituent.interfaces.data.QosWorkloadConstituentMeasurements;
import com.netapp.oci.netapp.performance.qosworkloaddblade.interfaces.data.QosWorkloadQueueDBladeMeasurements;
import com.netapp.oci.netapp.performance.qosworkloaddetail.interfaces.data.QosWorkloadDetailMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.data.QosWorkloadConstituentData;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.data.QosWorkloadDetailData;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.data.ReAttributionContext;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.impl.ReAttributionServiceFactory;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.interfaces.IReAttributionContext;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.interfaces.IReAttributionService;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.*;
import java.util.Map.Entry;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Builder class for processing and reporting workload queue dblade metrics derived from Qos workload detail 
 * and Qos workload constituent measurements.
 */
public class QosWorkloadQueueDBladeMetricsBuilder {

    private static final Logger LOG = LoggerFactory.getLogger(QosWorkloadQueueDBladeMetricsBuilder.class);
    private final QosWorkloadDetailMetricsBuilder qosWorkloadDetailMetricsBuilder;
    private final QosWorkloadConstituentMetricsBuilder qosWorkloadConstituentMetricsBuilder;

    protected final NetAppOCIEPerformancePackage reportingPackage; 
    
    public QosWorkloadQueueDBladeMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage, final QosWorkloadDetailMetricsBuilder qosWorkloadDetailMetricsBuilder, final QosWorkloadConstituentMetricsBuilder qosWorkloadConstituentMetricsBuilder) {

        this.reportingPackage =   reportingPackage;
        this.qosWorkloadDetailMetricsBuilder =  qosWorkloadDetailMetricsBuilder;
        this.qosWorkloadConstituentMetricsBuilder = qosWorkloadConstituentMetricsBuilder;

    }
    
    /**
     * process Qos workload detail and Qos workload constituent measurements to derive Qos workload queue dblade metrics
     */
    public LinkedList<RawSample<QosWorkloadQueueDBladeMeasurements>> getDbladePerformanceMetrics(NetAppDefaultStorageContext context){
        LinkedList<RawSample<QosWorkloadQueueDBladeMeasurements>> measurementList = new LinkedList<RawSample<QosWorkloadQueueDBladeMeasurements>>();
        Map<String, QosWorkloadQueueDBladeMeasurements> qosWorkloadDBladeMeasurementsMap = new HashMap<String, QosWorkloadQueueDBladeMeasurements>();
        Map<String, Map<String, String>> qosWorkloadDetailMap = context.getQosWorkloadDetailMap();
        Map<String, String> nodeNameToUUIDMap = context.getNodeNameToUUIDMap();
        Map<String, QosWorkloadDetailMeasurements> qosWorkloadDetailMeasurementsMap = qosWorkloadDetailMetricsBuilder.getQosWorkloadDetailMeasurements();
        Map<String, QosWorkloadConstituentMeasurements> qosWorkloadConstituentMeasurementsMap = qosWorkloadConstituentMetricsBuilder.getQosWorkloadConstituentMeasurements();

        for(Entry<String, QosWorkloadDetailMeasurements> eachDetailMeasurement : qosWorkloadDetailMeasurementsMap.entrySet()){
                String inventoryUUID = eachDetailMeasurement.getKey();
                QosWorkloadDetailMeasurements qosWorkloadDetailMeasurement = eachDetailMeasurement.getValue();
                final long detailWaitTime = qosWorkloadDetailMeasurement.getWaitTime();
                final long detailServiceTime = qosWorkloadDetailMeasurement.getServiceTime();
                final Map<String, String> eachQosWorkloadDetail = qosWorkloadDetailMap.get(inventoryUUID);
                if(eachQosWorkloadDetail != null){
                    String nodeUUID = nodeNameToUUIDMap.get(eachQosWorkloadDetail.get("NodeNamePrefix"));
                    String clusterWorkloadUUID = eachQosWorkloadDetail.get("WorkloadUUID");
                    String workloadUUID = clusterWorkloadUUID.substring(clusterWorkloadUUID.indexOf(".") + 1, clusterWorkloadUUID.length());
                    String key = nodeUUID.concat(".").concat(workloadUUID);
                    if (qosWorkloadDBladeMeasurementsMap.get(key) == null) {
                        QosWorkloadQueueDBladeMeasurements qosWorkloadQueueDBladeMeasurements = new QosWorkloadQueueDBladeMeasurements();
                        /*
                         * Getting the WorkloadConstituent to calculate cache_miss_rate counter
                         */
                        QosWorkloadConstituentMeasurements qosWorkloadConstituentMeasurements = qosWorkloadConstituentMeasurementsMap.get(key);

                        if (qosWorkloadConstituentMeasurements != null) {
                            final long sumOfDiskAndHyaHDD = qosWorkloadConstituentMeasurements.getSumDiskAndHyaHDD();
                            final long readIoTypeBase = qosWorkloadConstituentMeasurements.getReadIOTypeBase();

                            float cacheMissRate = (readIoTypeBase == 0 || sumOfDiskAndHyaHDD == 0)
                                    ? 0
                                    : (readIoTypeBase > 0 && sumOfDiskAndHyaHDD > 0)
                                    ? (float) (100d * (double) sumOfDiskAndHyaHDD / (double) readIoTypeBase)
                                    : MetricsRowSample.NULL_FLOAT;

                            qosWorkloadQueueDBladeMeasurements.setCacheMissRate(cacheMissRate)
                                    .setCacheMissRateBase(readIoTypeBase);

                        }
                        qosWorkloadDBladeMeasurementsMap.put(key, qosWorkloadQueueDBladeMeasurements);
                    }
                    QosWorkloadQueueDBladeMeasurements eachMeasurement = qosWorkloadDBladeMeasurementsMap.get(key);
                    String workloadLastGroup = inventoryUUID.substring(inventoryUUID.lastIndexOf(".") + 1, inventoryUUID.length());
                    if(("CPU_dblade").equals(workloadLastGroup)){
                        if(detailServiceTime != MetricsRowSample.NULL_LONG){
                            if(detailWaitTime != MetricsRowSample.NULL_LONG){
                                eachMeasurement.setCpuDBladeResidenceTime(detailServiceTime + detailWaitTime);
                                }
                            eachMeasurement.setCpuDBladeUtilServiceTime(detailServiceTime);
                        }
                    }
                    else if(("CPU_dblade_background").equals(workloadLastGroup)){
                        if(detailServiceTime != MetricsRowSample.NULL_LONG){
                            eachMeasurement.setCpuDBladeBackgroundServiceTime(detailServiceTime);
                        }
                    }
                    else if(("DELAY_CENTER_WAFL_SUSP_CP").equals(workloadLastGroup)){
                        eachMeasurement.setDelayWAFLSuspCPWaitTime(detailWaitTime);
                    }
                    else if(workloadLastGroup != null && workloadLastGroup.startsWith("DELAY_CENTER_DISK_IO")){
                        eachMeasurement.setDelayDiskIOWaitTime(detailWaitTime);
                    }
                    else if(workloadLastGroup != null && workloadLastGroup.startsWith("DISK_HDD")){
                        eachMeasurement.setDiskHDDBackgroundServiceTime(detailServiceTime + eachMeasurement.getDiskHDDBackgroundServiceTime());
                    }
                    else if(workloadLastGroup != null && workloadLastGroup.startsWith("DISK_SSD")){
                        eachMeasurement.setDiskSSDBackgroundServiceTime(detailServiceTime + eachMeasurement.getDiskSSDBackgroundServiceTime());
                    }
                    // avoid overwriting recorded collectionPeriod with collectionPeriod of wrap reset workload detail samples
                    if (qosWorkloadDetailMeasurement.getCollectionPeriod() > eachMeasurement.getCollectionPeriod()){
                        eachMeasurement.setCollectionPeriod(qosWorkloadDetailMeasurement.getCollectionPeriod());
                    }
                }
        }
        // Add Re Attribution Counters
        List<QosWorkloadDetailData> reAttributedWorkloadDetails =
                reAttributeWorkloadDetail(context, qosWorkloadConstituentMeasurementsMap, qosWorkloadDetailMeasurementsMap);
        for (QosWorkloadDetailData reAttributedWorkloadDetail : reAttributedWorkloadDetails) {
            final String workloadDetailUUID = reAttributedWorkloadDetail.getUUID();
            final Map<String, String> eachQosWorkloadDetail = qosWorkloadDetailMap.get(workloadDetailUUID);
            if (eachQosWorkloadDetail != null) {
                String nodeUUID = nodeNameToUUIDMap.get(eachQosWorkloadDetail.get("NodeNamePrefix"));
                String clusterWorkloadUUID = eachQosWorkloadDetail.get("WorkloadUUID");
                String workloadUUID = clusterWorkloadUUID.substring(clusterWorkloadUUID.indexOf(".") + 1, clusterWorkloadUUID.length());
                String key = nodeUUID.concat(".").concat(workloadUUID);
                final QosWorkloadQueueDBladeMeasurements qosWorkloadQueueDBladeMeasurements = qosWorkloadDBladeMeasurementsMap.get(key);
                if (qosWorkloadQueueDBladeMeasurements != null) {
                    String workloadLastGroup = workloadDetailUUID.substring(workloadDetailUUID.lastIndexOf(".") + 1, workloadDetailUUID.length());
                    if ("CPU_dblade_background".equals(workloadLastGroup)) {
                        qosWorkloadQueueDBladeMeasurements.setCpuDBladeBackgroundWAFLCPServiceTime(
                                reAttributedWorkloadDetail.getServiceTimeWAFLCP());
                        qosWorkloadQueueDBladeMeasurements.setCpuDBladeBackgroundBackgroundDeleteServiceTime(
                                reAttributedWorkloadDetail.getServiceTimeBackgroundDelete());
                        qosWorkloadQueueDBladeMeasurements.setCpuDBladeBackgroundRAStripXServiceTime(
                                reAttributedWorkloadDetail.getServiceTimeReadAheadXStripe());
                    } else if (workloadLastGroup != null && workloadLastGroup.startsWith("DISK_HDD")) {
                        qosWorkloadQueueDBladeMeasurements.setDiskHDDBackgroundWAFLCPServiceTime(
                                reAttributedWorkloadDetail.getServiceTimeWAFLCP());
                        qosWorkloadQueueDBladeMeasurements.setDiskHDDBackgroundBackgroundDeleteServiceTime(
                                reAttributedWorkloadDetail.getServiceTimeBackgroundDelete());
                        qosWorkloadQueueDBladeMeasurements.setDiskHDDBackgroundRAStripeXServiceTime(
                                reAttributedWorkloadDetail.getServiceTimeReadAheadXStripe());
                    } else if (workloadLastGroup != null && workloadLastGroup.startsWith("DISK_SSD")) {
                        qosWorkloadQueueDBladeMeasurements.setDiskSSDBackgroundWAFLCPServiceTime(
                                reAttributedWorkloadDetail.getServiceTimeWAFLCP());
                        qosWorkloadQueueDBladeMeasurements.setDiskSSDBackgroundBackgroundDeleteServiceTime(
                                reAttributedWorkloadDetail.getServiceTimeBackgroundDelete());
                        qosWorkloadQueueDBladeMeasurements.setDiskSSDBackgroundRAStripeXServiceTime(
                                reAttributedWorkloadDetail.getServiceTimeReadAheadXStripe());
                    }
                }
            }
        }

        for(Entry<String, QosWorkloadQueueDBladeMeasurements> qosWorkloadDBladeMeasurementByUUID : qosWorkloadDBladeMeasurementsMap.entrySet()){ 
            QosWorkloadQueueDBladeMeasurements qosWorkloadDBladeMeasurement = qosWorkloadDBladeMeasurementByUUID.getValue();
            if(isZeroSupressDetected(qosWorkloadDBladeMeasurement)){
                continue;
            }
            RawSample<QosWorkloadQueueDBladeMeasurements> dBladeRawSample = new RawSample<QosWorkloadQueueDBladeMeasurements>(qosWorkloadDBladeMeasurementByUUID.getKey(), qosWorkloadDBladeMeasurement);
            measurementList.add(dBladeRawSample);
        }
        return measurementList;
    }

    public void getAndReportPerformance(NetAppDefaultStorageContext context) throws DataSourceErrorException {
        Map<Cluster.ClusterKey,Collection<RawSample<QosWorkloadQueueDBladeMeasurements>>> metricsToReport = new HashMap<Cluster.ClusterKey, Collection<RawSample<QosWorkloadQueueDBladeMeasurements>>>();
        LinkedList<RawSample<QosWorkloadQueueDBladeMeasurements>> measurementList = getDbladePerformanceMetrics(context);
        metricsToReport.put(context.getKey(),measurementList);
        reportPerformance(metricsToReport);
    }
    
    void reportPerformance(
            final Map<Cluster.ClusterKey, Collection<RawSample<QosWorkloadQueueDBladeMeasurements>>> measurements)
            throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<QosWorkloadQueueDBladeMeasurements>>> measurementEntry : measurements
                .entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<QosWorkloadQueueDBladeMeasurements>> workloadQueueDBladeMeasurements = measurementEntry
                    .getValue();
            reportingPackage.reportQosWorkloadQueueDBladePerformance(
                    key, workloadQueueDBladeMeasurements);
            LOG.debug("Qos workload queue dblade measurements are reported successfully with current poll stats. Cluster Id: {}", measurementEntry.getKey());
        }
    }
    
    public boolean isZeroSupressDetected(QosWorkloadQueueDBladeMeasurements dbladeMeasurement) {
        return (dbladeMeasurement.getCpuDBladeResidenceTime() == 0L 
                && dbladeMeasurement.getCpuDBladeUtilServiceTime() == 0L
                && dbladeMeasurement.getCpuDBladeBackgroundServiceTime() == 0L
                && dbladeMeasurement.getCpuDBladeBackgroundWAFLCPServiceTime() == 0L
                && dbladeMeasurement.getCpuDBladeBackgroundBackgroundDeleteServiceTime() == 0L
                && dbladeMeasurement.getCpuDBladeBackgroundRAStripXServiceTime() == 0L
                && dbladeMeasurement.getDiskHDDBackgroundServiceTime() == 0L
                && dbladeMeasurement.getDiskHDDBackgroundWAFLCPServiceTime() == 0L
                && dbladeMeasurement.getDiskHDDBackgroundRAStripeXServiceTime() == 0L
                && dbladeMeasurement.getDiskHDDBackgroundBackgroundDeleteServiceTime() == 0L
                && dbladeMeasurement.getDiskSSDBackgroundServiceTime() == 0L
                && dbladeMeasurement.getDiskSSDBackgroundWAFLCPServiceTime() == 0L
                && dbladeMeasurement.getDiskSSDBackgroundBackgroundDeleteServiceTime() == 0L
                && dbladeMeasurement.getDiskSSDBackgroundRAStripeXServiceTime() == 0L
                && dbladeMeasurement.getDelayWAFLSuspCPWaitTime() == 0L
                && dbladeMeasurement.getDelayDiskIOWaitTime() == 0L
                && dbladeMeasurement.getCacheMissRate() == 0f
                && dbladeMeasurement.getCacheMissRateBase() == 0L);
    }

    /**
     * Here we copy Qos Workload Detail and Qos Workload Constituent Data to QosWorkloadDetailData and
     * QosWorkloadConstituentData and call the IReAttributionService to re attribute the workload details. The Map
     * returned contains the re attributed workload details.
     */
    private List<QosWorkloadDetailData> reAttributeWorkloadDetail(
            NetAppDefaultStorageContext context, Map<String,
            QosWorkloadConstituentMeasurements> qosWorkloadConstituentMeasurementsMap,
            Map<String, QosWorkloadDetailMeasurements> qosWorkloadDetailMeasurementsMap) {
        IReAttributionContext reAttributionContext = populateReAttributionContext(context);

        List<QosWorkloadConstituentData> qosWorkloadConstituentDataList = new ArrayList<>();
        for (Entry<String, QosWorkloadConstituentMeasurements> workloadConstituentEntry :
                qosWorkloadConstituentMeasurementsMap.entrySet()) {
            final QosWorkloadConstituentMeasurements workloadConstituentMeasurements = workloadConstituentEntry.getValue();
            if (workloadConstituentMeasurements != null) {
                final QosWorkloadConstituentData qosWorkloadConstituentData =
                        new QosWorkloadConstituentData(workloadConstituentEntry.getKey(),
                                workloadConstituentMeasurements.getDirtyBuffers(),
                                workloadConstituentMeasurements.getFreedBlocks(),
                                workloadConstituentMeasurements.getReadAheadXStripe());
                qosWorkloadConstituentDataList.add(qosWorkloadConstituentData);
            }
        }

        List<QosWorkloadDetailData> qosWorkloadDetailDataList = new ArrayList<>();
        for (Entry<String, QosWorkloadDetailMeasurements> workloadDetailEntry : qosWorkloadDetailMeasurementsMap.entrySet()) {
            final QosWorkloadDetailMeasurements workloadDetailMeasurements = workloadDetailEntry.getValue();
            if (workloadDetailMeasurements != null) {
                final QosWorkloadDetailData qosWorkloadDetailData = new QosWorkloadDetailData(workloadDetailEntry.getKey(),
                        workloadDetailMeasurements.getServiceTime(), workloadDetailMeasurements.getWaitTime());
                qosWorkloadDetailDataList.add(qosWorkloadDetailData);
            }
        }

        final IReAttributionService reAttributionService =
                ReAttributionServiceFactory.getReAttributionService(reAttributionContext);
        reAttributionService.reAttributeQosWorkloadDetails(qosWorkloadDetailDataList, qosWorkloadConstituentDataList);

        return qosWorkloadDetailDataList;
    }

    /**
     * This method reads data from the NetAppDefaultStorageContext and creates IReAttributionContext.
     */
    private IReAttributionContext populateReAttributionContext(NetAppDefaultStorageContext context) {
        final Map<String, Map<IReAttributionContext.QosWorkloadDetailAttributes, String>>
                qosWorkloadDetailMap = new HashMap<String, Map<IReAttributionContext.QosWorkloadDetailAttributes, String>>();
        final Map<String, Map<IReAttributionContext.QosWorkloadNodeRelationshipAttributes, String>>
                workloadNodeRelationshipMap = new HashMap<String, Map<IReAttributionContext.QosWorkloadNodeRelationshipAttributes, String>>();
        final Map<String, Map<IReAttributionContext.AggregateAttributes, String>>
                aggregateMap = new HashMap<String, Map<IReAttributionContext.AggregateAttributes, String>>();
        IReAttributionContext reAttributionContext = new ReAttributionContext(context.getKey().getOcieUUID(),
                aggregateMap,workloadNodeRelationshipMap, qosWorkloadDetailMap);

        for(Entry<String, Map<String, String>> workloadEntry : context.getQosVolumeWorkloadUUIDToAggregateMap().entrySet()) {
            String workloadUUID = workloadEntry.getKey();
            final Map<String, String> aggregate = workloadEntry.getValue();
            Map<IReAttributionContext.AggregateAttributes, String> aggregateAttributesMap = new HashMap<>(2);
            aggregateAttributesMap.put(IReAttributionContext.AggregateAttributes.AGGREGATE_UUID,
                    aggregate.get("AggregateUUID"));
            aggregateAttributesMap.put(IReAttributionContext.AggregateAttributes.AGGREGATE_NAME,
                    aggregate.get("AggregateName"));
            aggregateMap.put(workloadUUID, aggregateAttributesMap);
        }

        for (Entry<String, Map<String, String>> workloadDetailEntry : context.getQosWorkloadDetailMap().entrySet()) {
            String workloadDetailUUID = workloadDetailEntry.getKey();
            final Map<String, String> workloadDetailEntryValue = workloadDetailEntry.getValue();
            Map<IReAttributionContext.QosWorkloadDetailAttributes, String>
                    nodeAggregateAndWorkloadMap = new HashMap<>(3);
            nodeAggregateAndWorkloadMap.put(IReAttributionContext.QosWorkloadDetailAttributes.NODE_NAME,
                    workloadDetailEntryValue.get("NodeNamePrefix"));
            nodeAggregateAndWorkloadMap.put(IReAttributionContext.QosWorkloadDetailAttributes.AGGREGATE_UUID,
                    workloadDetailEntryValue.get("AggregateUUIDSuffix"));
            nodeAggregateAndWorkloadMap.put(IReAttributionContext.QosWorkloadDetailAttributes.WORKLOAD_UUID,
                    workloadDetailEntryValue.get("WorkloadUUID"));
            qosWorkloadDetailMap.put(workloadDetailUUID, nodeAggregateAndWorkloadMap);
        }

        for (Entry<String, Map<String, String>> workloadNodeRelEntry : context.getQosWorkloadConstituentMap().entrySet()) {
            final String nodeWorkloadRelUUID = workloadNodeRelEntry.getKey();
            final Map<String, String> nodeWorkloadRelEntry = workloadNodeRelEntry.getValue();
            Map<IReAttributionContext.QosWorkloadNodeRelationshipAttributes, String>
                    workloadNodeRelationshipAttributesMap = new HashMap<>();
            workloadNodeRelationshipAttributesMap.put(
                    IReAttributionContext.QosWorkloadNodeRelationshipAttributes.WORKLOAD_UUID,
                    nodeWorkloadRelEntry.get("WorkloadUUID"));
            workloadNodeRelationshipAttributesMap.put(
                    IReAttributionContext.QosWorkloadNodeRelationshipAttributes.WORKLOAD_NAME,
                    nodeWorkloadRelEntry.get("WorkloadName"));
            workloadNodeRelationshipAttributesMap.put(
                    IReAttributionContext.QosWorkloadNodeRelationshipAttributes.NODE_NAME,
                    nodeWorkloadRelEntry.get("WorkloadConstituentNodeName"));
            workloadNodeRelationshipMap.put(nodeWorkloadRelUUID, workloadNodeRelationshipAttributesMap);
        }

        return reAttributionContext;
    }

}
