package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeMeasurements;
import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappVolumeMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.framework.datasource.config.PackageConfiguration;
import com.onaro.sanscreen.acquisition.framework.recordings.RecordingSession;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppStoragePerformanceReporter;
import org.apache.log4j.Logger;

import java.util.Collection;
import java.util.List;
import java.util.Map;

public class VolumeMetricsBuilder extends MetricsBuilderBase<VServer.VServerKey,
        VolumeMeasurements,
        VolumeRawMeasurements,
        NetappVolumeMetrics,
        NetappVolumeMetrics.VolumeCounter,
        NetappVolumeMetrics.VolumeCounter> {

    private static final Logger logger = Logger.getLogger(VolumeMetricsBuilder.class);

    public VolumeMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "volume", NetappVolumeMetrics.VolumeCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection connection) {
        return context.getVolumeInstanceUUIDs();
    }

    @Override
    NetappVolumeMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappVolumeMetrics.VolumeCounter> currentValues, final MetricsRowScratchPad<NetappVolumeMetrics.VolumeCounter> previousValues) {
        return new NetappVolumeMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(final Cluster.ClusterKey clusterKey, final Map<VServer.VServerKey, Collection<RawSample<VolumeMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<VServer.VServerKey, Collection<RawSample<VolumeMeasurements>>> measurementEntry : measurements.entrySet()) {
            final VServer.VServerKey key = measurementEntry.getKey();
            final Collection<RawSample<VolumeMeasurements>> volumeMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportVolumePerformance(clusterKey, key, volumeMeasurements);
        }
    }

    @Override
    void reportRawPerformance(final Cluster.ClusterKey clusterKey, final Map<VServer.VServerKey, Collection<RawSample<VolumeRawMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<VServer.VServerKey, Collection<RawSample<VolumeRawMeasurements>>> measurementEntry : measurements.entrySet()) {
            final VServer.VServerKey key = measurementEntry.getKey();
            final Collection<RawSample<VolumeRawMeasurements>> volumeMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportVolumeRawPerformance(clusterKey, key, volumeMeasurements);
        }
    }
    
    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return perfInstanceUUID;
    }

    @Override
    VServer.VServerKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfGlobalUUID) {
        // Given the Volume's performance UUID, find the parent VServer's foundation UUID.
        // Need to substitute volume global UUID used for perf query with instance UUID used for foundation reporting
        final String instanceUUID = getIventoryUUIDFromPerfUUID(context, perfGlobalUUID);
        final String vServerUUID = context.getVolumeVServerUUIDMap().get(instanceUUID);
        if (vServerUUID == null) {
            logger.warn(PackageContext.getLogPrefix() + " - Failed to find vServer for Volume " + instanceUUID);
            return null;
        }
        return new VServer.VServerKey(context.getKey().getOcieUUID(), vServerUUID);
    }
}
