/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.aggregate.interfaces.data.WaflHyaAggrMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappWaflHyaAggrMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.config.PackageConfiguration;
import com.onaro.sanscreen.acquisition.framework.recordings.RecordingSession;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppStoragePerformanceReporter;

import java.util.Collection;
import java.util.List;
import java.util.Map;


public class WaflHyaAggrMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
        WaflHyaAggrMeasurements,
        WaflHyaAggrMeasurements,
        NetappWaflHyaAggrMetrics,
        NetappWaflHyaAggrMetrics.WaflHyaAggrCounter,
        NetappWaflHyaAggrMetrics.WaflHyaAggrCounter> {

    public WaflHyaAggrMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "wafl_hya_per_aggr", NetappWaflHyaAggrMetrics.WaflHyaAggrCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        return context.getHybridAggrUUIDs();
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getAggregatePerfUUIDToUUID().get(perfInstanceUUID);
    }

    @Override
    NetappWaflHyaAggrMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappWaflHyaAggrMetrics.WaflHyaAggrCounter> currentValues,
                                                     final MetricsRowScratchPad<NetappWaflHyaAggrMetrics.WaflHyaAggrCounter> previousValues) {
        return new NetappWaflHyaAggrMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<WaflHyaAggrMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<WaflHyaAggrMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<WaflHyaAggrMeasurements>> waflHyaAggrMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportWaflHyaAggrPerformance(key, waflHyaAggrMeasurements);
        }
    }
}
