package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import java.util.Map;

/**
 * This merger is used for merging scratchpads using performance uuid.Scratch pads with same performance uuid will be
 * merged together.
 *
 * @param <T> he type of the merge result scratchpad. This is a Enum with the list of counters in the result scratchpad.
 */
public abstract class MergeByPerformanceUUID<T extends Enum<T>> implements MergeOperator<T> {
    /**
     * Class modeling the enum with the list of counters in the result scratchpad.
     */
    final Class<T> resultsEnumClass;

    public MergeByPerformanceUUID(Class<T> resultsEnumClass) {
        this.resultsEnumClass = resultsEnumClass;
    }

    /**
     * Merge mergeResults with mergeWithScratchPad using performance uuid. This is a full outer merge
     *
     */
    @Override
    public void mergeScratchPad(final Map<String, MetricsRowScratchPad<T>> mergeResults, String mergeWithCounterObjectName,
                               final Map<String, ? extends MetricsRowScratchPad<? extends Enum<?>>> mergeWithScratchPad) {
        for (Map.Entry<String, ? extends MetricsRowScratchPad<? extends Enum<?>>> counterInstances : mergeWithScratchPad.entrySet()) {
            final String performanceUUID = retrievePerfUUID(counterInstances, mergeWithCounterObjectName );
            final MetricsRowScratchPad<? extends Enum<?>> scratchPad = counterInstances.getValue();

            if (!mergeResults.containsKey(performanceUUID)) {
                final MetricsRowScratchPad<T> prevScratchPad;
                if (scratchPad.previousValues != null) {
                    prevScratchPad = new MetricsRowScratchPad<>(resultsEnumClass, scratchPad.previousValues.getPerfInstanceUUID(),
                            scratchPad.previousValues.getInventoryUUID(), scratchPad.previousValues.getServerPollStartTimeStamp(),
                            scratchPad.previousValues.getPollStartTimeStamp(), null, scratchPad.previousValues.metricsDebugEnabled);
                } else {
                    prevScratchPad = null;
                }
                final MetricsRowScratchPad<T> currScratchPad = new MetricsRowScratchPad<>(resultsEnumClass,
                        scratchPad.getPerfInstanceUUID(), scratchPad.getInventoryUUID(), scratchPad.getServerPollStartTimeStamp(),
                        scratchPad.getPollStartTimeStamp(), prevScratchPad, scratchPad.metricsDebugEnabled);
                mergeResults.put(performanceUUID, currScratchPad);
            }
            MetricsRowScratchPad<T> mergeToResult = mergeResults.get(performanceUUID);
            mergeToResult = mergeCounters(mergeToResult, mergeWithCounterObjectName, scratchPad);
            mergeResults.put(performanceUUID, mergeToResult);
        }
    }

    /**
     * Returns the performanceUUID of the counterObject name from the Map.Entry value
     * @param counterInstances
     * @param counterObjectName
     * @return
     */
    public String retrievePerfUUID(Map.Entry<String, ? extends MetricsRowScratchPad<? extends Enum<?>>> counterInstances, String counterObjectName){
        return counterInstances.getKey();
    }

}