package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;


import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetAppHostadapterMetrics.HostadapterCounter.*;

import java.math.BigInteger;
import java.util.Map;

import com.netapp.oci.netapp.performance.hostadapter.interfaces.data.HostadapterMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

public class NetAppHostadapterMetrics extends MetricsRowSample<HostadapterMeasurements, HostadapterMeasurements, NetAppHostadapterMetrics.HostadapterCounter, NetAppHostadapterMetrics.HostadapterCounter>{

    public static enum HostadapterCounter{
    	bytes_read,
    	bytes_written,
    	max_link_data_rate
    }

    public NetAppHostadapterMetrics(MetricsRowScratchPad<HostadapterCounter> currentValues, MetricsRowScratchPad<HostadapterCounter> previousValues) {
        super(HostadapterCounter.class, currentValues, previousValues);
    }

    @Override
    float getNormalizedFloatMetric(HostadapterCounter counterName) {    
        final float normalizedMetric;
        
        switch (counterName) {
        case bytes_read:
        case bytes_written:
            normalizedMetric = currentValues.getCounterRatePerSec(counterName);
            break;
        default:
            normalizedMetric = NULL_FLOAT;
            break;
        }
        return normalizedMetric;
    }

    @Override
    public HostadapterMeasurements normalizeMetrics(NetAppDefaultStorageContext context) {
        Map<HostadapterCounter, BigInteger> counterValues = getCurrentValues().getCounterValues();
        return createHostadapterMeasurements(counterValues, context);
    }
          
    protected HostadapterMeasurements createHostadapterMeasurements(Map<HostadapterCounter, BigInteger> counters, NetAppDefaultStorageContext context){
        HostadapterMeasurements hostadapterMeasurements = new HostadapterMeasurements();
        
        if(counters.containsKey(bytes_read)){
            hostadapterMeasurements.setBytesRead(getNormalizedFloatMetric(bytes_read));
        }
        if(counters.containsKey(bytes_written)){
            hostadapterMeasurements.setBytesWritten(getNormalizedFloatMetric(bytes_written));
        }
        if(counters.containsKey(max_link_data_rate)){
        	//Since max link data rate for hostadapter is constant and this data is not available as configuration, we need the raw value here.
    		BigInteger maxLinkDataRate = currentValues.getCounterValues().get(HostadapterCounter.max_link_data_rate);
    		hostadapterMeasurements.setMaxLinkDataRate((maxLinkDataRate != null && maxLinkDataRate.signum() >= 0) ? maxLinkDataRate.longValue(): MetricsRowSample.NULL_LONG);
        }

        return hostadapterMeasurements;
    }
    
 }
