package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import com.netapp.oci.netapp.performance.aggregate.interfaces.data.AggregateMeasurements;
import com.netapp.oci.netapp.performance.aggregate.interfaces.data.AggregateRawMeasurements;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappAggregateMetrics.AggregateCounter.*;

public class NetappAggregateMetrics extends MetricsRowSample<AggregateMeasurements, AggregateRawMeasurements, NetappAggregateMetrics.AggregateCounter, NetappAggregateMetrics.AggregateCounter>{

	public static enum AggregateCounter {
        cp_read_blocks,
        cp_reads,
        total_transfers,
        user_read_blocks,
        user_reads,
        user_write_blocks,
        user_writes,
        total_transfers_hdd,
        total_transfers_ssd,
        user_read_blocks_hdd,
        user_read_blocks_ssd,
        user_reads_hdd,
        user_reads_ssd,
        user_write_blocks_hdd,
        user_write_blocks_ssd,
        user_writes_hdd,
        user_writes_ssd
    }

    private NetAppOCIEPerformancePackage reportingPackage;

    public NetappAggregateMetrics(MetricsRowScratchPad<AggregateCounter> currentValues, MetricsRowScratchPad<AggregateCounter> previousValues, NetAppOCIEPerformancePackage reportingPackage) {
		super(AggregateCounter.class, currentValues, previousValues);
        this.reportingPackage = reportingPackage;
	}

	@Override
	float getNormalizedFloatMetric(AggregateCounter counterName) {
        final float normalizedMetric;
        switch (counterName) {
        	case cp_read_blocks:  
        	case cp_reads:
        	case total_transfers:
        	case user_read_blocks:
        	case user_reads:
        	case user_write_blocks:
        	case user_writes:
            case total_transfers_hdd:
            case total_transfers_ssd:
            case user_read_blocks_hdd:
            case user_read_blocks_ssd:
            case user_reads_hdd:
            case user_reads_ssd:
            case user_write_blocks_hdd:
            case user_write_blocks_ssd:
            case user_writes_hdd:
            case user_writes_ssd:
                normalizedMetric = currentValues.getCounterRatePerSec(counterName);
                break;
            default:
                normalizedMetric = NULL_FLOAT;
                break;
        }
        
        return normalizedMetric;
	}

	@Override
	public AggregateMeasurements normalizeMetrics(NetAppDefaultStorageContext context) {

        float userReadBlocks = getNormalizedFloatMetric(user_read_blocks);
        float cpReadBlocks =  getNormalizedFloatMetric(cp_read_blocks);
        float userWriteBlocks = getNormalizedFloatMetric(user_write_blocks);

        float opmAggrUserReadLatency = MetricsRowSample.NULL_FLOAT;
        float opmAggrUserWriteLatency = MetricsRowSample.NULL_FLOAT;
        float opmAggrCpReadLatency = MetricsRowSample.NULL_FLOAT;
        float opmAggrUtilization = MetricsRowSample.NULL_FLOAT;
        float diskUserReads = MetricsRowSample.NULL_FLOAT;
        float diskUserWrites = MetricsRowSample.NULL_FLOAT;
        float diskCpReads = MetricsRowSample.NULL_FLOAT;

        Map<String, List<String>> aggrUUIDToDiskUUIDList = context.getAggregateUUIDToDiskUUIDList();
        //for SN, PerfInstanceUUID is in the form of NodeKernelPerfUUID, so should be using inventoryUUID here
        List<String> diskUUIDList = aggrUUIDToDiskUUIDList.get(currentValues.getInventoryUUID());
        if(diskUUIDList!=null && !diskUUIDList.isEmpty())  {
            List<NetappDiskMetrics.DiskCounter> numeratorCounterNamesLatency = new ArrayList<>();
            numeratorCounterNamesLatency.add(NetappDiskMetrics.DiskCounter.user_read_latency);
            numeratorCounterNamesLatency.add(NetappDiskMetrics.DiskCounter.user_write_latency);
            numeratorCounterNamesLatency.add(NetappDiskMetrics.DiskCounter.cp_read_latency);
            List<NetappDiskMetrics.DiskCounter> denominatorCounterNamesLatency = new ArrayList<>();
            denominatorCounterNamesLatency.add(NetappDiskMetrics.DiskCounter.user_reads);
            denominatorCounterNamesLatency.add(NetappDiskMetrics.DiskCounter.user_writes);
            denominatorCounterNamesLatency.add(NetappDiskMetrics.DiskCounter.cp_reads);
            Map<NetappDiskMetrics.DiskCounter,MetricsRowScratchPad<NetappDiskMetrics.DiskCounter>> opmLatencyMap = reportingPackage.getGlobalScratchPad().getScratchPadsForMaximumLatencyOps(diskUUIDList, "disk", numeratorCounterNamesLatency, denominatorCounterNamesLatency);
            MetricsRowScratchPad<NetappDiskMetrics.DiskCounter> opmAggrUserReadLatencyPad = opmLatencyMap.get(NetappDiskMetrics.DiskCounter.user_read_latency);
            MetricsRowScratchPad<NetappDiskMetrics.DiskCounter> opmAggrUserWriteLatencyPad = opmLatencyMap.get(NetappDiskMetrics.DiskCounter.user_write_latency);
            MetricsRowScratchPad<NetappDiskMetrics.DiskCounter> opmAggrCpReadLatencyPad = opmLatencyMap.get(NetappDiskMetrics.DiskCounter.cp_read_latency);
            if(opmAggrUserReadLatencyPad!=null) {
                opmAggrUserReadLatency = opmAggrUserReadLatencyPad.getLatencyPerOp(NetappDiskMetrics.DiskCounter.user_read_latency, NetappDiskMetrics.DiskCounter.user_reads);
                diskUserReads = opmAggrUserReadLatencyPad.getCounterRatePerSec(NetappDiskMetrics.DiskCounter.user_reads);
            }
            if(opmAggrUserWriteLatencyPad!=null) {
                opmAggrUserWriteLatency = opmAggrUserWriteLatencyPad.getLatencyPerOp(NetappDiskMetrics.DiskCounter.user_write_latency, NetappDiskMetrics.DiskCounter.user_writes);
                diskUserWrites = opmAggrUserWriteLatencyPad.getCounterRatePerSec(NetappDiskMetrics.DiskCounter.user_writes);
            }
            if(opmAggrCpReadLatencyPad!=null) {
                opmAggrCpReadLatency = opmAggrCpReadLatencyPad.getLatencyPerOp(NetappDiskMetrics.DiskCounter.cp_read_latency, NetappDiskMetrics.DiskCounter.cp_reads);
                diskCpReads = opmAggrUserWriteLatencyPad.getCounterRatePerSec(NetappDiskMetrics.DiskCounter.cp_reads);
            }

            List<NetappDiskMetrics.DiskCounter> numeratorCounterNamesUtil = new ArrayList<>();
            numeratorCounterNamesUtil.add(NetappDiskMetrics.DiskCounter.disk_busy);
            List<NetappDiskMetrics.DiskCounter> denominatorCounterNamesUtil = new ArrayList<>();
            denominatorCounterNamesUtil.add(NetappDiskMetrics.DiskCounter.base_for_disk_busy);
            Map<NetappDiskMetrics.DiskCounter,Float> opmUtilMap = reportingPackage.getGlobalScratchPad().getMaximumOfPercentDeltas(diskUUIDList, "disk", numeratorCounterNamesUtil, denominatorCounterNamesUtil);
            opmAggrUtilization = opmUtilMap.get(NetappDiskMetrics.DiskCounter.disk_busy);
        }

        List<Float>  latencyList = new ArrayList<>();
        latencyList.add(opmAggrUserReadLatency);
        latencyList.add(opmAggrUserWriteLatency);
        latencyList.add(opmAggrCpReadLatency);
        List<Float>  opList = new ArrayList<>();
        opList.add(diskUserReads);
        opList.add(diskUserWrites);
        opList.add(diskCpReads);


        final AggregateMeasurements m = new AggregateMeasurements();
        m.setCpReadBlocks(cpReadBlocks);
        m.setCpReads(getNormalizedFloatMetric(cp_reads));
        m.setTotalTransfers(getNormalizedFloatMetric(total_transfers));
        m.setUserReadBlocks(userReadBlocks);
        m.setUserReads(getNormalizedFloatMetric(user_reads));
        m.setUserWriteBlocks(userWriteBlocks);
        m.setUserWrites(getNormalizedFloatMetric(user_writes));
        m.setTotalTransfersHdd(getNormalizedFloatMetric(total_transfers_hdd));
        m.setTotalTransfersSsd(getNormalizedFloatMetric(total_transfers_ssd));
        m.setUserReadBlocksHdd(getNormalizedFloatMetric(user_read_blocks_hdd));
        m.setUserReadBlocksSsd(getNormalizedFloatMetric(user_read_blocks_ssd));
        m.setUserReadsHdd(getNormalizedFloatMetric(user_reads_hdd));
        m.setUserReadsSsd(getNormalizedFloatMetric(user_reads_ssd));
        m.setUserWriteBlocksHdd(getNormalizedFloatMetric(user_write_blocks_hdd));
        m.setUserWriteBlocksSsd(getNormalizedFloatMetric(user_write_blocks_ssd));
        m.setUserWritesHdd(getNormalizedFloatMetric(user_writes_hdd));
        m.setUserWritesSsd(getNormalizedFloatMetric(user_writes_ssd));
        m.setOpmAggrThroughput(MetricsRowScratchPad.add(userReadBlocks, userWriteBlocks, cpReadBlocks) * 4096); //blocks to bytes
        m.setOpmAggrUserReadLatency(opmAggrUserReadLatency);
        m.setOpmAggrUserWriteLatency(opmAggrUserWriteLatency);
        m.setOpmAggrCpReadLatency(opmAggrCpReadLatency);
        m.setOpmAggrUtilization(opmAggrUtilization);
        m.setOpmAggrLatency(MetricsRowScratchPad.averageLatency(latencyList, opList));
        return m;
	}
	
	@Override
	public AggregateRawMeasurements rawMetrics(NetAppDefaultStorageContext context){
        Map<AggregateCounter,BigInteger> counterValues = getCurrentValues().getCounterValues();
        return createAggregateRawMeasurements(counterValues);
	}
	
	protected static AggregateRawMeasurements createAggregateRawMeasurements(Map<AggregateCounter,BigInteger> counters){
		AggregateRawMeasurements aggregateRawMeasurements = new AggregateRawMeasurements();
		aggregateRawMeasurements.setCpReadBlocks(counters.get(cp_read_blocks));
		aggregateRawMeasurements.setCpReads(counters.get(cp_reads));
		aggregateRawMeasurements.setTotalTransfers(counters.get(total_transfers));
		aggregateRawMeasurements.setUserReadBlocks(counters.get(user_read_blocks));
		aggregateRawMeasurements.setUserReads(counters.get(user_reads));
		aggregateRawMeasurements.setUserWriteBlocks(counters.get(user_write_blocks));
		aggregateRawMeasurements.setUserWrites(counters.get(user_writes));
        aggregateRawMeasurements.setTotalTransfersHdd(counters.get(total_transfers_hdd));
        aggregateRawMeasurements.setTotalTransfersSsd(counters.get(total_transfers_ssd));
        aggregateRawMeasurements.setUserReadBlocksHdd(counters.get(user_read_blocks_hdd));
        aggregateRawMeasurements.setUserReadBlocksSsd(counters.get(user_read_blocks_ssd));
        aggregateRawMeasurements.setUserReadsHdd(counters.get(user_reads_hdd));
        aggregateRawMeasurements.setUserReadsSsd(counters.get(user_reads_ssd));
        aggregateRawMeasurements.setUserWriteBlocksHdd(counters.get(user_write_blocks_hdd));
        aggregateRawMeasurements.setUserWriteBlocksSsd(counters.get(user_write_blocks_ssd));
        aggregateRawMeasurements.setUserWritesHdd(counters.get(user_writes_hdd));
        aggregateRawMeasurements.setUserWritesSsd(counters.get(user_writes_ssd));
		return aggregateRawMeasurements;
	}
}
