package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import java.math.BigInteger;
import java.util.Map;

import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverMeasurements;
import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverRawMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

public class NetappCifsVserverMetrics extends MetricsRowSample<CifsVserverMeasurements, CifsVserverRawMeasurements, NetappCifsVserverMetrics.CifsVserverCounter, 
	NetappCifsVserverMetrics.CifsVserverCounter> {

	public static enum CifsVserverCounter {
		cifs_latency,
		cifs_latency_base,
		cifs_ops,
		cifs_read_latency,
		cifs_read_ops,
		cifs_write_latency,
		cifs_write_ops,
		cifs_read_size_histo,
		cifs_write_size_histo
	}

	public NetappCifsVserverMetrics(
			MetricsRowScratchPad<CifsVserverCounter> currentValues,
			MetricsRowScratchPad<CifsVserverCounter> previousValues) {
		super(CifsVserverCounter.class, currentValues, previousValues);
	}

	CifsVserverCounter getDenominatorForNumerator(final CifsVserverCounter numeratorName) {
	    final CifsVserverCounter denominatorName;
	    switch (numeratorName) {
	    case cifs_latency:
	        denominatorName = CifsVserverCounter.cifs_ops;  
            break;
	    case cifs_read_latency:
	        denominatorName = CifsVserverCounter.cifs_read_ops;  
            break;
	    case cifs_write_latency:
	        denominatorName = CifsVserverCounter.cifs_write_ops;  
	        break;
	    default:
	        denominatorName = null;
	    }
	    return denominatorName;
	}

	@Override
	float getNormalizedFloatMetric(CifsVserverCounter counterName) {
	    final float normalizedMetric;

	    switch (counterName) {
	    case cifs_read_ops:
	    case cifs_write_ops:
	    case cifs_ops:
	        normalizedMetric = currentValues.getCounterRatePerSec(counterName);
	        break;
	    case cifs_read_latency:
	    case cifs_write_latency:
	    case cifs_latency:
	        normalizedMetric = currentValues.getLatencyPerOp(counterName, getDenominatorForNumerator(counterName));
	        break;
	    default:
	        normalizedMetric = NULL_FLOAT;
	        break;
	    }
	    return normalizedMetric;
	}
	
	   @Override
	    public CifsVserverMeasurements normalizeMetrics(NetAppDefaultStorageContext context) {
	       
	        final CifsVserverMeasurements m = new CifsVserverMeasurements();
	        m.setCifsLatencyBase(currentValues.getCounterDelta(CifsVserverCounter.cifs_latency_base));
	        m.setCifsLatency(getNormalizedFloatMetric(CifsVserverCounter.cifs_latency));
	        m.setCifsOps(getNormalizedFloatMetric(CifsVserverCounter.cifs_ops));
	        m.setCifsReadLatency(getNormalizedFloatMetric(CifsVserverCounter.cifs_read_latency));
	        m.setCifsReadOps(getNormalizedFloatMetric(CifsVserverCounter.cifs_read_ops));
	        m.setCifsWriteLatency(getNormalizedFloatMetric(CifsVserverCounter.cifs_write_latency));
	        m.setCifsWriteOps(getNormalizedFloatMetric(CifsVserverCounter.cifs_write_ops));
	        return m;
	    }
	    
	@Override
	public CifsVserverRawMeasurements rawMetrics(NetAppDefaultStorageContext context){
        final CifsVserverRawMeasurements m = new CifsVserverRawMeasurements();
        Map<CifsVserverCounter, String> rawValue = getCurrentValues().getRawValues();
        m.setCifsReadSizeHisto(rawValue.get(CifsVserverCounter.cifs_read_size_histo));
        m.setCifsWriteSizeHisto(rawValue.get(CifsVserverCounter.cifs_write_size_histo));
        
        Map<CifsVserverCounter,BigInteger> counterValues = getCurrentValues().getCounterValues();
        m.setCifsLatency(counterValues.get(CifsVserverCounter.cifs_latency));
        m.setCifsLatencyBase(counterValues.get(CifsVserverCounter.cifs_latency_base));
        m.setCifsOps(counterValues.get(CifsVserverCounter.cifs_ops));
        m.setCifsReadLatency(counterValues.get(CifsVserverCounter.cifs_read_latency));
        m.setCifsReadOps(counterValues.get(CifsVserverCounter.cifs_read_ops));
        m.setCifsWriteLatency(counterValues.get(CifsVserverCounter.cifs_write_latency));
        m.setCifsWriteOps(counterValues.get(CifsVserverCounter.cifs_write_ops));
        return m;
	}

 
	
}
