package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifVserverMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifVserverRawMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.math.BigInteger;
import java.util.Map;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.*;

public class NetappIscsiLifVserverMetrics extends MetricsRowSample<IscsiLifVserverMeasurements, IscsiLifVserverRawMeasurements, NetappIscsiLifVserverMetrics.IscsiLifVserverCounter,
	NetappIscsiLifVserverMetrics.IscsiLifVserverCounter> {

	public static enum IscsiLifVserverCounter {
		data_in_blocks,
		data_out_blocks,
		iscsi_read_ops,
		iscsi_write_ops,
		protocol_errors,
		avg_latency,
		avg_other_latency,
		avg_read_latency,
		avg_write_latency,
		iscsi_other_ops,
		read_data,
		write_data,
		cmd_transfered
	}

	public NetappIscsiLifVserverMetrics(final MetricsRowScratchPad<IscsiLifVserverCounter> currentValues, final MetricsRowScratchPad<IscsiLifVserverCounter> previousValues) {
		super(IscsiLifVserverCounter.class, currentValues, previousValues);
	}

	IscsiLifVserverCounter getDenominatorForNumerator(final IscsiLifVserverCounter numeratorName) {
		final IscsiLifVserverCounter denominatorName;
		switch (numeratorName) {
		case avg_latency:
			denominatorName = cmd_transfered;
			break;
		case avg_other_latency:
			denominatorName = iscsi_other_ops;
			break;
		case avg_read_latency:
			denominatorName = iscsi_read_ops;
			break;
		case avg_write_latency:
			denominatorName = iscsi_write_ops;
			break;
		default:
			denominatorName = null;
		}

		return denominatorName;
	}

	@Override
	float getNormalizedFloatMetric(final IscsiLifVserverCounter counterName) {
		final float normalizedMetric;
		switch (counterName) {
			case iscsi_read_ops:
			case iscsi_write_ops:
			case iscsi_other_ops:
			case protocol_errors:
			case read_data:
			case write_data:
				normalizedMetric = currentValues.getCounterRatePerSec(counterName);
				break;
			case avg_latency:
				normalizedMetric = currentValues.getLatencyPerOp(counterName, getDenominatorForNumerator(counterName));
				break;
			case avg_other_latency:
				normalizedMetric = currentValues.getLatencyPerOp(counterName, getDenominatorForNumerator(counterName));
				break;
			case avg_read_latency:
				normalizedMetric = currentValues.getLatencyPerOp(counterName, getDenominatorForNumerator(counterName));
				break;
			case avg_write_latency:
				normalizedMetric = currentValues.getLatencyPerOp(counterName, getDenominatorForNumerator(counterName));
				break;
			default:
				normalizedMetric = NULL_FLOAT;
		}
		return normalizedMetric;
	}

	@Override
	public IscsiLifVserverMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
        final Map<IscsiLifVserverCounter,BigInteger> counterValues = getCurrentValues().getCounterValues();
		final IscsiLifVserverMeasurements ilvMeasurements = new IscsiLifVserverMeasurements();
		ilvMeasurements.setDataInBlocks(currentValues.getCounterDelta(data_in_blocks));
		ilvMeasurements.setDataOutBlocks(currentValues.getCounterDelta(data_out_blocks));
		ilvMeasurements.setIscsiReadOps(getNormalizedFloatMetric(iscsi_read_ops));
		ilvMeasurements.setIscsiWriteOps(getNormalizedFloatMetric(iscsi_write_ops));
		ilvMeasurements.setProtocolErrors(getNormalizedFloatMetric(protocol_errors));
		ilvMeasurements.setReadData(getNormalizedFloatMetric(read_data));
		ilvMeasurements.setWriteData(getNormalizedFloatMetric(write_data));
		ilvMeasurements.setCmdTransfered(currentValues.getCounterDelta(cmd_transfered));
		ilvMeasurements.setAvgLatency(getNormalizedFloatMetric(avg_latency));
		ilvMeasurements.setAvgOtherLatency(getNormalizedFloatMetric(avg_other_latency));
		ilvMeasurements.setAvgReadLatency(getNormalizedFloatMetric(avg_read_latency));
		ilvMeasurements.setAvgWriteLatency(getNormalizedFloatMetric(avg_write_latency));
		ilvMeasurements.setIscsiOtherOps(getNormalizedFloatMetric(iscsi_other_ops));
		ilvMeasurements.setOpmIscsiLifOps(MetricsRowScratchPad.add(ilvMeasurements.getIscsiReadOps(),
				ilvMeasurements.getIscsiWriteOps(), ilvMeasurements.getIscsiOtherOps()));
		ilvMeasurements.setOpmIscsiLifThroughput(MetricsRowScratchPad.add(ilvMeasurements.getReadData(),
				ilvMeasurements.getWriteData()));
		return ilvMeasurements;
	}

	@Override
	public IscsiLifVserverRawMeasurements rawMetrics(final NetAppDefaultStorageContext context) {
		Map<IscsiLifVserverCounter,BigInteger> counterValues = getCurrentValues().getCounterValues();
		IscsiLifVserverRawMeasurements ilvRawMeasurements = new IscsiLifVserverRawMeasurements();
		ilvRawMeasurements .setAvgLatency(counterValues.get(avg_latency));
		ilvRawMeasurements.setAvgOtherLatency(counterValues.get(avg_other_latency));
		ilvRawMeasurements.setAvgReadLatency(counterValues.get(avg_read_latency));
		ilvRawMeasurements.setAvgWriteLatency(counterValues.get(avg_write_latency));
		ilvRawMeasurements.setIscsiOtherOps(counterValues.get(iscsi_other_ops));
		ilvRawMeasurements.setIscsiReadOps(counterValues.get(iscsi_read_ops));
		ilvRawMeasurements.setIscsiWriteOps(counterValues.get(iscsi_write_ops));
		ilvRawMeasurements.setDataInBlocks(counterValues.get(data_in_blocks));
		ilvRawMeasurements.setReadData(counterValues.get(read_data));
		ilvRawMeasurements.setDataOutBlocks(counterValues.get(data_out_blocks));
		ilvRawMeasurements.setWriteData(counterValues.get(write_data));
		ilvRawMeasurements.setCmdTransfered(counterValues.get(cmd_transfered));
		ilvRawMeasurements.setOpmIscsiLifOps(MetricsRowScratchPad.add(ilvRawMeasurements.getIscsiOtherOps(),
				ilvRawMeasurements.getIscsiReadOps(), ilvRawMeasurements.getIscsiWriteOps()));
		ilvRawMeasurements.setOpmIscsiLifThroughput(MetricsRowScratchPad.add(ilvRawMeasurements.getReadData(),
				ilvRawMeasurements.getWriteData()));
		return ilvRawMeasurements;
	}
}
