package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;


import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv3Measurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv3RawMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.math.BigInteger;
import java.util.Map;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNfsv3Metrics.Nfsv3Counter.*;

public class NetappNfsv3Metrics extends MetricsRowSample<Nfsv3Measurements, Nfsv3RawMeasurements, NetappNfsv3Metrics.Nfsv3Counter, NetappNfsv3Metrics.Nfsv3Counter>{

	public static enum Nfsv3Counter {
		nfsv3_ops,
		nfsv3_read_ops,
		nfsv3_write_ops,
		read_total,
		nfsv3_read_throughput,
		nfsv3_throughput,
		write_total,
		nfsv3_write_throughput,
		read_avg_latency,
		write_avg_latency
	}
	
    public NetappNfsv3Metrics(MetricsRowScratchPad<Nfsv3Counter> currentValues, MetricsRowScratchPad<Nfsv3Counter> previousValues) {
		super(Nfsv3Counter.class, currentValues, previousValues);
	}
	
    Nfsv3Counter getDenominatorForNumerator(final Nfsv3Counter numeratorName) {
        final Nfsv3Counter denominatorName;
        switch (numeratorName) {
            case read_avg_latency:
                denominatorName = Nfsv3Counter.read_total;
                break;
            case write_avg_latency:
                denominatorName = Nfsv3Counter.write_total;
                break;   
            default:
                denominatorName = null;
        }
        return denominatorName;
    }
    
	@Override
	float getNormalizedFloatMetric(Nfsv3Counter counterName) {
	    final float normalizedMetric;
	    
	    switch (counterName) {
	    case nfsv3_ops:
	    case nfsv3_read_ops:
	    case nfsv3_write_ops:
	    case nfsv3_read_throughput:
	    case nfsv3_throughput:
	    case nfsv3_write_throughput:
	        normalizedMetric = currentValues.getCounterRatePerSec(counterName);
	        break;
	    case read_avg_latency:
	    case write_avg_latency:
	        normalizedMetric = currentValues.getLatencyPerOp(counterName, getDenominatorForNumerator(counterName));
	        break;
	    default:
	        normalizedMetric = NULL_FLOAT;
	        break;
	    }
	    return normalizedMetric;
	}

	@Override
	public Nfsv3Measurements normalizeMetrics(NetAppDefaultStorageContext context) {
	    final Nfsv3Measurements m = new Nfsv3Measurements();
	    m.setNfsv3Ops(getNormalizedFloatMetric(nfsv3_ops));
        m.setNfsv3ReadOps(getNormalizedFloatMetric(nfsv3_read_ops));
        m.setNfsv3WriteOps(getNormalizedFloatMetric(nfsv3_write_ops));
        m.setReadTotal(currentValues.getCounterDeltaFinal(read_total));
        m.setNfsv3ReadThroughput(getNormalizedFloatMetric(nfsv3_read_throughput));
        m.setNfsv3Throughput(getNormalizedFloatMetric(nfsv3_throughput));
        m.setWriteTotal(currentValues.getCounterDeltaFinal(write_total));
        m.setNfsv3WriteThroughput(getNormalizedFloatMetric(nfsv3_write_throughput));
        m.setReadAvgLatency(getNormalizedFloatMetric(read_avg_latency));
        m.setWriteAvgLatency(getNormalizedFloatMetric(write_avg_latency));
		return m;
	}

	
	@Override
    public Nfsv3RawMeasurements rawMetrics(NetAppDefaultStorageContext context){
        final Nfsv3RawMeasurements m = new Nfsv3RawMeasurements();
        Map<Nfsv3Counter, String> rawValue = getCurrentValues().getRawValues();

        Map<Nfsv3Counter,BigInteger> counterValues = getCurrentValues().getCounterValues();
        m.setNfsv3Ops(counterValues.get(nfsv3_ops));
        m.setNfsv3ReadOps(counterValues.get(nfsv3_read_ops));
        m.setReadTotal(counterValues.get(read_total));
        m.setNfsv3ReadThroughput(counterValues.get(nfsv3_read_throughput));
        m.setNfsv3Throughput(counterValues.get(nfsv3_throughput));
        m.setNfsv3WriteOps(counterValues.get(nfsv3_write_ops));
        m.setNfsv3WriteThroughput(counterValues.get(nfsv3_write_throughput));
        m.setReadAvgLatency(counterValues.get(read_avg_latency));
        m.setWriteAvgLatency(counterValues.get(write_avg_latency));
        m.setWriteTotal(counterValues.get(write_total));
        return m;
    }
			
}
