package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv41Measurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv41RawMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.math.BigInteger;
import java.util.Map;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNfsv41Metrics.Nfsv41Counter.*;

public class NetappNfsv41Metrics extends MetricsRowSample<Nfsv41Measurements, Nfsv41RawMeasurements, NetappNfsv41Metrics.Nfsv41Counter, NetappNfsv41Metrics.Nfsv41Counter>{
	public static enum Nfsv41Counter {
		read_avg_latency,
		read_total,
		nfs41_read_throughput,
		nfs41_throughput,
		write_avg_latency,
		write_total,
		nfs41_write_throughput,
		nfsv4_1_ops
	}

    public NetappNfsv41Metrics(MetricsRowScratchPad<Nfsv41Counter> currentValues, MetricsRowScratchPad<Nfsv41Counter> previousValues) {
		super(Nfsv41Counter.class, currentValues, previousValues);
	}
	
    Nfsv41Counter getDenominatorForNumerator(final Nfsv41Counter numeratorName) {
        final Nfsv41Counter denominatorName;
        switch (numeratorName) {
            case read_avg_latency:
                denominatorName = read_total;
                break;
            case write_avg_latency:
                denominatorName = write_total;
                break;   
            default:
                denominatorName = null;
        }
        return denominatorName;
    }
    
	@Override
	float getNormalizedFloatMetric(Nfsv41Counter counterName) {
	    final float normalizedMetric;
        
        switch (counterName) {
        case nfsv4_1_ops:
        case nfs41_read_throughput:
        case nfs41_throughput:
        case nfs41_write_throughput:
            normalizedMetric = currentValues.getCounterRatePerSec(counterName);
            break;
        case read_avg_latency:
        case write_avg_latency:
            normalizedMetric = currentValues.getLatencyPerOp(counterName, getDenominatorForNumerator(counterName));
            break;
        default:
            normalizedMetric = NULL_FLOAT;
            break;
        }
        return normalizedMetric;
	}

	@Override
	public Nfsv41Measurements normalizeMetrics(NetAppDefaultStorageContext context) {
	    final Nfsv41Measurements m = new Nfsv41Measurements();
        m.setNfsv41Ops(getNormalizedFloatMetric(nfsv4_1_ops));
        m.setReadTotal(currentValues.getCounterDeltaFinal(read_total));
        m.setNfsv41ReadThroughput(getNormalizedFloatMetric(nfs41_read_throughput));
        m.setNfsv41Throughput(getNormalizedFloatMetric(nfs41_throughput));
        m.setWriteTotal(currentValues.getCounterDeltaFinal(write_total));
        m.setNfsv41WriteThroughput(getNormalizedFloatMetric(nfs41_write_throughput));
        m.setReadAvgLatency(getNormalizedFloatMetric(read_avg_latency));
        m.setWriteAvgLatency(getNormalizedFloatMetric(write_avg_latency));
        return m;
	};

	@Override
	public Nfsv41RawMeasurements rawMetrics(NetAppDefaultStorageContext context){
        final Nfsv41RawMeasurements m = new Nfsv41RawMeasurements();
        Map<Nfsv41Counter, String> rawValue = getCurrentValues().getRawValues();

        Map<Nfsv41Counter,BigInteger> counterValues = getCurrentValues().getCounterValues();
        m.setNfs41ReadThroughput(counterValues.get(nfs41_read_throughput));
        m.setNfs41Throughput(counterValues.get(nfs41_throughput));
        m.setNfs41WriteThroughput(counterValues.get(nfs41_write_throughput));
        m.setNfsv41Ops(counterValues.get(nfsv4_1_ops));
        m.setReadAvgLatency(counterValues.get(read_avg_latency));
        m.setReadTotal(counterValues.get(read_total));
        m.setWriteAvgLatency(counterValues.get(write_avg_latency));
        m.setWriteTotal(counterValues.get(write_total));
        return m;
	}
	
}
