package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;


import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNicCommonMetrics.NicCommonCounter.*;

import java.math.BigInteger;
import java.util.Map;

import com.netapp.oci.netapp.performance.nic.interfaces.data.NicMeasurements;
import com.netapp.oci.netapp.performance.nic.interfaces.data.NicRawMeasurements;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.NetworkPort.Speed;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

public class NetappNicCommonMetrics extends MetricsRowSample<NicMeasurements, NicRawMeasurements, NetappNicCommonMetrics.NicCommonCounter, NetappNicCommonMetrics.NicCommonCounter>{
	private static final long MEGABIT_1 = 1000000;

    public static enum NicCommonCounter{
        rx_bytes_rate,
        rx_errors_rate,
        tx_bytes_rate,
        tx_errors_rate,
        rx_bytes_per_sec, //legacy counter in SierraNevada
        rx_errors_per_min, //legacy counter in SierraNevada
        tx_bytes_per_sec, //legacy counter in SierraNevada
        tx_errors_per_min //legacy counter in SierraNevada
    }

    public NetappNicCommonMetrics(MetricsRowScratchPad<NicCommonCounter> currentValues, MetricsRowScratchPad<NicCommonCounter> previousValues) {
        super(NicCommonCounter.class, currentValues, previousValues);
    }

    @Override
    float getNormalizedFloatMetric(NicCommonCounter counterName) {    
        final float normalizedMetric;
        
        switch (counterName) {
        case rx_bytes_rate:
        case rx_errors_rate:
        case tx_bytes_rate:
        case tx_errors_rate:
            normalizedMetric = currentValues.getCounterRatePerSec(counterName);
            break;
        default:
            normalizedMetric = NULL_FLOAT;
            break;
        }
        return normalizedMetric;
    }

    @Override
    public NicMeasurements normalizeMetrics(NetAppDefaultStorageContext context) {
        Map<NicCommonCounter, BigInteger> counterValues = getCurrentValues().getCounterValues();
        return createNicMeasurements(counterValues, context);
    }
    
    @Override
    public NicRawMeasurements rawMetrics(NetAppDefaultStorageContext context){
        Map<NicCommonCounter,BigInteger> counterValues = getCurrentValues().getCounterValues();
        return createNicRawMeasurements(counterValues);
    }
    
    protected static NicRawMeasurements createNicRawMeasurements(Map<NicCommonCounter,BigInteger> counters){
        NicRawMeasurements nicRawMeasurements = new NicRawMeasurements();
        
        if(counters.containsKey(rx_bytes_rate)){
            nicRawMeasurements.setRxBytesRate(counters.get(rx_bytes_rate));
        }
        else{
            nicRawMeasurements.setRxBytesRate(counters.get(rx_bytes_per_sec));
        }
        
        if(counters.containsKey(rx_errors_rate)){
            nicRawMeasurements.setRxErrorsRate(counters.get(rx_errors_rate));
        }
        else{
            nicRawMeasurements.setRxErrorsRate(counters.get(rx_errors_per_min).divide(BigInteger.valueOf(60)));
        }
        
        if(counters.containsKey(tx_bytes_rate)){
            nicRawMeasurements.setTxBytesRate(counters.get(tx_bytes_rate));
        }
        else{
            nicRawMeasurements.setTxBytesRate(counters.get(tx_bytes_per_sec));
        }
        
        if(counters.containsKey(tx_errors_rate)){
            nicRawMeasurements.setTxErrorsRate(counters.get(tx_errors_rate));
        }
        else{
            nicRawMeasurements.setTxErrorsRate(counters.get(tx_errors_per_min).divide(BigInteger.valueOf(60)));
        }
        
        nicRawMeasurements.setOpmNicTotalBytePersec(MetricsRowScratchPad.add(nicRawMeasurements.getRxBytesRate(),nicRawMeasurements.getTxBytesRate()));
        
        nicRawMeasurements.setOpmNicTotalErrorRate(MetricsRowScratchPad.add(nicRawMeasurements.getRxErrorsRate(),nicRawMeasurements.getTxErrorsRate()));
        
        return nicRawMeasurements;
    }
  
        
    //rx_bytes_per_sec, //legacy counter in SierraNevada
    //rx_errors_per_min, //legacy counter in SierraNevada
    //tx_bytes_per_sec, //legacy counter in SierraNevada
    //tx_errors_per_min //legacy counter in SierraNevada
    protected NicMeasurements createNicMeasurements(Map<NicCommonCounter, BigInteger> counters, NetAppDefaultStorageContext context){
        NicMeasurements nicMeasurements = new NicMeasurements();
        
        if(counters.containsKey(rx_bytes_rate)){
            nicMeasurements.setRxBytesRate(getNormalizedFloatMetric(rx_bytes_rate));
        }
        else{
            nicMeasurements.setRxBytesRate(getNormalizedFloatMetric(rx_bytes_per_sec));
        }
        
        if(counters.containsKey(rx_errors_rate)){
            nicMeasurements.setRxErrorsRate(getNormalizedFloatMetric(rx_errors_rate));
        }
        else{
            nicMeasurements.setRxErrorsRate(getNormalizedFloatMetric(rx_errors_per_min) / 60);
        }
        
        if(counters.containsKey(tx_bytes_rate)){
            nicMeasurements.setTxBytesRate(getNormalizedFloatMetric(tx_bytes_rate));
        }
        else{
            nicMeasurements.setTxBytesRate(getNormalizedFloatMetric(tx_bytes_per_sec)); 
        }
        
        if(counters.containsKey(tx_errors_rate)){
            nicMeasurements.setTxErrorsRate(getNormalizedFloatMetric(tx_errors_rate));
        }
        else{
            nicMeasurements.setTxErrorsRate(getNormalizedFloatMetric(tx_errors_per_min) / 60);
        }
        
        nicMeasurements.setOpmNicTotalBytePerSec(MetricsRowScratchPad.add(nicMeasurements.getRxBytesRate() , nicMeasurements.getTxBytesRate()));
        
        nicMeasurements.setOpmNicTotalErrorRate(MetricsRowScratchPad.add(nicMeasurements.getRxErrorsRate(), nicMeasurements.getTxErrorsRate()));
        
        nicMeasurements.setOpmNicUtilization(calculateOpmNicUtilization(nicMeasurements.getOpmNicTotalBytePerSec(), context.getIpPortToLinkSpeedMap().get(currentValues.getInventoryUUID())));
        
        return nicMeasurements;
    }
    
    /**
     * 
     * @param float opmNicTotalBytePerSec
     * @param float linkSpeed
     * @return float nicUtilixation (opm_total_byte_per_sec*8(converting bytes to bits) / link_speed) * 100
     */
    private float calculateOpmNicUtilization(float opmNicTotalBytePerSec, Speed linkSpeed){
    		
    	if(null == linkSpeed)
    		return MetricsRowSample.NULL_FLOAT;
    	long linkSpeesInBitsPerSec = getLinkSpeesInBitsPerSec(linkSpeed);
    	if(linkSpeesInBitsPerSec > 0){
    		// multiplying by 8 to convert Bytes/sec to Bits/sec and 100 to return utilization in 0 to 100 % range
    		return (float)(100 * ((8*(double)opmNicTotalBytePerSec)/linkSpeesInBitsPerSec));
    	}else{
    		// if we have 0 or negative values(error handling), then utilization cannot be calibrated, hence setting to null representation for float
    		return MetricsRowSample.NULL_FLOAT;
    	}
    }
    
    /**
     *  Converting Enum NetworkPort.Speed to BigIntiger equivalent
     * @param linkSpeed
     * @return BigInteger Link speed in  Bits/Sec
     */
    private long getLinkSpeesInBitsPerSec(Speed linkSpeed){
    	long linkSpeesInBitsPerSec =0;
    	if(linkSpeed.compareTo(Speed.MEGABIT_10) == 0) linkSpeesInBitsPerSec = 10 * MEGABIT_1; // 10 Mega Bits per second
    	else if(linkSpeed.compareTo(Speed.MEGABIT_100) == 0) linkSpeesInBitsPerSec = 100 * MEGABIT_1; // 100 Mega Bit per second
    	else if(linkSpeed.compareTo(Speed.GIGABIT_1) == 0) linkSpeesInBitsPerSec = 1000 * MEGABIT_1; // 1 Giga Bits per second
    	else if(linkSpeed.compareTo(Speed.GIGABIT_10) == 0) linkSpeesInBitsPerSec = 10000 * MEGABIT_1; // 10 Giga Bits per second
    	return linkSpeesInBitsPerSec;
    }
}
