package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import com.netapp.oci.netapp.performance.qosservicecenter.interfaces.data.QosServiceCenterMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import org.apache.log4j.Logger;

import java.math.BigInteger;

public class NetappQosServiceCenterMetrics extends MetricsRowSample<QosServiceCenterMeasurements, QosServiceCenterMeasurements, NetappQosServiceCenterMetrics.ServiceCenterCounter,        // raw
                                                                                                  NetappQosServiceCenterMetrics.OCIEServiceCenterCounter> {      // final
    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(NetappQosServiceCenterMetrics.class);
    private NetAppDefaultStorageContext context;

    /**
     * Columns to be collected via ZAPI for non-DELAY Service Centers
     */
    public static enum ServiceCenterCounter {
        busy_percent,
        cpu_elapsed_time,
        num_instances,
    }


    /**
     * Columns to be reported to OCIE, includes columns collected via ZAPI for non-DELAY Service Centers,
     * as well as summations of columns for child workload details
     */
    public static enum OCIEServiceCenterCounter {
        busy_percent,          // non-delay center directly from resource
        cpu_elapsed_time,      // non-delay center directly from resource
        wait_time,             // delay center summation of child workload detail
    }


    public NetappQosServiceCenterMetrics(final MetricsRowScratchPad<ServiceCenterCounter> currentValues, final MetricsRowScratchPad<ServiceCenterCounter> previousValues) {
        super(ServiceCenterCounter.class, currentValues, previousValues);
    }


    /**
     * Get the normalized float metric for report to OCIE.   This is NOT an override, because set of reported
     * metrics is derived from but not equal to the set of collected metrics
     *
     * @param counterName     The enum constant for the desired counter
     *
     * @return The measurement
     */
    float getNormalizedFloatMetric(final OCIEServiceCenterCounter counterName) {

        final float normalizedMetric;
        switch (counterName) {
            case busy_percent:
                normalizedMetric = getBusyPercent();
                break;

            default:
                normalizedMetric = NULL_FLOAT;
                break;
        }
        return normalizedMetric;
    }


    private float getBusyPercent() {
        final BigInteger numOfInstances = currentValues.getCounterValues().get(ServiceCenterCounter.num_instances);
        if (numOfInstances == null || numOfInstances.intValue() == 0) {
            return NULL_FLOAT;
        }
        final float unnormalizedPercent = currentValues.getPercentFromDeltas(ServiceCenterCounter.busy_percent,
                ServiceCenterCounter.cpu_elapsed_time);
        final float normalizedPercent = unnormalizedPercent / numOfInstances.intValue();
        currentValues.setFinalValueForDebug(ServiceCenterCounter.busy_percent, normalizedPercent);
        return normalizedPercent;
    }


    @Override
    public QosServiceCenterMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
        this.context = context;
        final BigInteger numOfInstancesBigInt = currentValues.getCounterValues().get(ServiceCenterCounter.num_instances);
        Long numOfInstances = MetricsRowSample.NULL_LONG;
        if (numOfInstancesBigInt != null) {
            numOfInstances = numOfInstancesBigInt.longValue();
        }
        return new QosServiceCenterMeasurements(
                NULL_LONG,         // Wait time not collected directly from ZAPI, aggregated from child workload details
                getNormalizedFloatMetric(OCIEServiceCenterCounter.busy_percent),
                currentValues.getCounterDeltaFinal(ServiceCenterCounter.cpu_elapsed_time),
                numOfInstances,
                currentValues.getCollectionPeriod()
        );
    }
}
