package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import com.netapp.oci.netapp.performance.qosworkloaddetail.interfaces.data.QosWorkloadDetailMeasurements;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;


import java.math.BigInteger;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class NetappQosWorkloadDetailMetrics extends MetricsRowSample<QosWorkloadDetailMeasurements, QosWorkloadDetailMeasurements, NetappQosWorkloadDetailMetrics.QosWorkloadDetailCounter,
                                                                                                    NetappQosWorkloadDetailMetrics.QosWorkloadDetailCounter> {

    private static final Logger LOG = LoggerFactory.getLogger(NetappQosWorkloadDetailMetrics.class);
    private final boolean isZeroSupressionEnabled;
    
    public static enum QosWorkloadDetailCounter {
        service_time,
        wait_time,
        visits
    }
    
    public NetappQosWorkloadDetailMetrics(final MetricsRowScratchPad<QosWorkloadDetailCounter> currentValues, final MetricsRowScratchPad<QosWorkloadDetailCounter> previousValues) {
        super(QosWorkloadDetailCounter.class, currentValues, previousValues);
        final String zeroSuppressionEnabledString = PackageContext.getPackage().getConfiguration().getProperty("zeroSuppressionEnabled", "true");
        isZeroSupressionEnabled = Boolean.parseBoolean(zeroSuppressionEnabledString);
    }
    
    
    /**
     * Normalized this metrics instance against the previous metrics instance, and wrap in the
     * measurements class expected by the server
     * @param context in case other data from the context is required to normalize data
     * @return normalized metric values wrapped in the class expected by the server
     */
    @Override
    public QosWorkloadDetailMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
        final BigInteger visitsEstimate = currentValues.counterValues.get(QosWorkloadDetailCounter.visits);
        final Long deltaVisitsEstimate = currentValues.getCounterDelta(QosWorkloadDetailCounter.visits);
        if (isZeroSupressionEnabled) {
            if ((visitsEstimate != null && visitsEstimate.longValue() == 0L) || (deltaVisitsEstimate != null && deltaVisitsEstimate == 0L)) {
                    currentValues.setZeroSuppressDetected(true);
                }
        }

        final QosWorkloadDetailMeasurements m = new QosWorkloadDetailMeasurements();
        m.setServiceTime(currentValues.getCounterDelta(QosWorkloadDetailCounter.service_time));
        m.setWaitTime(currentValues.getCounterDelta(QosWorkloadDetailCounter.wait_time));
        m.setCollectionPeriod(currentValues.getCollectionPeriod());
        return m;
    }
    
    /**
     * Get the normalized integer metric value for the given enum (metric identifyer), set of current values, and set of previous values.
     * Some normalizations require more than one counter value, so the entire sets for current and previous are required.
     *
     * @param counterName     the name/id/enum of the counter to normalize
     *
     * @return the normalized counter value, -1 if normalization is not possible or required
     */
    @Override
    float getNormalizedFloatMetric(QosWorkloadDetailCounter counterName) {
        return -1;
    }    
}
