package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;


import com.netapp.oci.netapp.performance.node.interfaces.data.SMB1NodeMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappSMB1NodeMetrics.SMB1NodeCounter.*;

public class NetappSMB1NodeMetrics extends MetricsRowSample<SMB1NodeMeasurements, SMB1NodeMeasurements, NetappSMB1NodeMetrics.SMB1NodeCounter,
        NetappSMB1NodeMetrics.SMB1NodeCounter>{

    public static enum SMB1NodeCounter {
        latency,
        latency_base,
        read_latency,
        read_latency_base,
        write_latency,
        write_latency_base,
        ops,
        read_ops,
        write_ops
    }

    public NetappSMB1NodeMetrics(final MetricsRowScratchPad<SMB1NodeCounter> currentValues, final MetricsRowScratchPad<SMB1NodeCounter> previousValues) {
        super(SMB1NodeCounter.class, currentValues, previousValues);
    }

    @Override
    float getNormalizedFloatMetric(final SMB1NodeCounter counterName) {
        final float normalizedMetric;
        switch (counterName) {
            case ops:
            case read_ops:
            case write_ops:
                normalizedMetric = currentValues.getCounterRatePerSec(counterName);
                break;

            case latency:
            case read_latency:
            case write_latency:
                normalizedMetric = currentValues.getLatencyPerOp(counterName,
                        getDenominatorForNumerator(counterName));
                break;

            default:
                normalizedMetric = NULL_FLOAT;
        }
        return normalizedMetric;
    }

    @Override
    SMB1NodeCounter getDenominatorForNumerator(final SMB1NodeCounter numeratorName) {
        final SMB1NodeCounter denominatorName;
        switch (numeratorName) {
            case latency:
                denominatorName = latency_base;
                break;

            case read_latency:
                denominatorName = read_latency_base;
                break;

            case write_latency:
                denominatorName = write_latency_base;
                break;

            default:
                denominatorName = null;
        }
        return denominatorName;
    }

    @Override
    public SMB1NodeMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
        final SMB1NodeMeasurements smb1NodeMeasurements = new SMB1NodeMeasurements();
        smb1NodeMeasurements.setLatency(getNormalizedFloatMetric(latency));
        smb1NodeMeasurements.setOps(getNormalizedFloatMetric(ops));
        smb1NodeMeasurements.setReadLatency(getNormalizedFloatMetric(read_latency));
        smb1NodeMeasurements.setReadOps(getNormalizedFloatMetric(read_ops));
        smb1NodeMeasurements.setWriteLatency(getNormalizedFloatMetric(write_latency));
        smb1NodeMeasurements.setWriteOps(getNormalizedFloatMetric(write_ops));
        return smb1NodeMeasurements;
    }
}
