package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappSMB2NodeMetrics.SMB2NodeCounter.*;


import com.netapp.oci.netapp.performance.node.interfaces.data.SMB2NodeMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

public class NetappSMB2NodeMetrics extends MetricsRowSample<SMB2NodeMeasurements, SMB2NodeMeasurements, NetappSMB2NodeMetrics.SMB2NodeCounter,
        NetappSMB2NodeMetrics.SMB2NodeCounter>{

    public static enum SMB2NodeCounter {
        latency,
        latency_base,
        read_latency,
        read_latency_base,
        write_latency,
        write_latency_base,
        ops,
        read_ops,
        write_ops
    }

    public NetappSMB2NodeMetrics(final MetricsRowScratchPad<SMB2NodeCounter> currentValues, final MetricsRowScratchPad<SMB2NodeCounter> previousValues) {
        super(SMB2NodeCounter.class, currentValues, previousValues);
    }

    @Override
    float getNormalizedFloatMetric(final SMB2NodeCounter counterName) {
        final float normalizedMetric;
        switch (counterName) {
            case ops:
            case read_ops:
            case write_ops:
                normalizedMetric = currentValues.getCounterRatePerSec(counterName);
                break;

            case latency:
            case read_latency:
            case write_latency:
                normalizedMetric = currentValues.getLatencyPerOp(counterName,
                        getDenominatorForNumerator(counterName));
                break;

            default:
                normalizedMetric = NULL_FLOAT;
        }
        return normalizedMetric;
    }

    @Override
    SMB2NodeCounter getDenominatorForNumerator(final SMB2NodeCounter numeratorName) {
        final SMB2NodeCounter denominatorName;
        switch (numeratorName) {
            case latency:
                denominatorName = latency_base;
                break;

            case read_latency:
                denominatorName = read_latency_base;
                break;

            case write_latency:
                denominatorName = write_latency_base;
                break;

            default:
                denominatorName = null;
        }
        return denominatorName;
    }

    @Override
    public SMB2NodeMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
        final SMB2NodeMeasurements smb2NodeMeasurements = new SMB2NodeMeasurements();
        smb2NodeMeasurements.setLatency(getNormalizedFloatMetric(latency));
        smb2NodeMeasurements.setOps(getNormalizedFloatMetric(ops));
        smb2NodeMeasurements.setReadLatency(getNormalizedFloatMetric(read_latency));
        smb2NodeMeasurements.setReadOps(getNormalizedFloatMetric(read_ops));
        smb2NodeMeasurements.setWriteLatency(getNormalizedFloatMetric(write_latency));
        smb2NodeMeasurements.setWriteOps(getNormalizedFloatMetric(write_ops));
        return smb2NodeMeasurements;
    }
}
