package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import com.netapp.oci.netapp.performance.volumenode.interfaces.data.VolumeNodeMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

public class NetappVolumeNodeMetrics extends MetricsRowSample<VolumeNodeMeasurements, VolumeNodeMeasurements, NetappVolumeNodeMetrics.VolumeNodeCounter,
                                                                              NetappVolumeNodeMetrics.VolumeNodeCounter> {

    public static enum VolumeNodeCounter {
        avg_latency,
        read_data,
        read_latency,
        read_ops,
        total_ops,
        write_data,
        write_latency,
        write_ops, }

    public NetappVolumeNodeMetrics(final MetricsRowScratchPad<VolumeNodeCounter> currentValues, final MetricsRowScratchPad<VolumeNodeCounter> previousValues) {
        super(VolumeNodeCounter.class, currentValues, previousValues);
    }

    VolumeNodeCounter getDenominatorForNumerator(final VolumeNodeCounter numeratorName) {
        final VolumeNodeCounter denominatorName;
        switch (numeratorName) {
            case avg_latency:
                denominatorName = VolumeNodeCounter.total_ops;
                break;
            case read_latency:
                denominatorName = VolumeNodeCounter.read_ops;
                break;
            case write_latency:
                denominatorName = VolumeNodeCounter.write_ops;
                break;
            default:
                denominatorName = null;
                break;
        }
        return denominatorName;
    }

    @Override
    float getNormalizedFloatMetric(final VolumeNodeCounter counterName) {
        final float normalizedMetric;
        switch (counterName) {
            case avg_latency:
            case read_latency:
            case write_latency:
                normalizedMetric = currentValues.getLatencyPerOp(counterName,
                                                                 getDenominatorForNumerator(counterName));
                break;

            case read_data:
            case write_data:
            case read_ops:
            case total_ops:
            case write_ops:
                normalizedMetric = currentValues.getCounterRatePerSec(counterName);
                break;

            default:
                normalizedMetric = NULL_FLOAT;
                break;
        }
        return normalizedMetric;
    }    

    @Override
    public VolumeNodeMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
        // Server side VolumeMeasurements has no setters, only giant constructor
        return new VolumeNodeMeasurements(getNormalizedFloatMetric(VolumeNodeCounter.avg_latency),
                                      getNormalizedFloatMetric(VolumeNodeCounter.read_data),
                                      getNormalizedFloatMetric(VolumeNodeCounter.read_latency),
                                      getNormalizedFloatMetric(VolumeNodeCounter.read_ops),
                                      getNormalizedFloatMetric(VolumeNodeCounter.total_ops),
                                      getNormalizedFloatMetric(VolumeNodeCounter.write_data),
                                      getNormalizedFloatMetric(VolumeNodeCounter.write_latency),
                                      getNormalizedFloatMetric(VolumeNodeCounter.write_ops));
    }
}
