package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeMeasurements;
import com.netapp.oci.netapp.performance.vserver.interfaces.data.VolumeVserverMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappVolumeVserverMetrics.VServerVolumeCounters.*;

public class NetappVolumeVserverMetrics extends MetricsRowSample<VolumeVserverMeasurements, VolumeVserverMeasurements, NetappVolumeVserverMetrics.VServerVolumeCounters,
        NetappVolumeVserverMetrics.VServerVolumeCounters> {

    public static enum VServerVolumeCounters {
        avg_latency,
        cifs_other_latency,
        cifs_other_ops,
        cifs_read_data,
        cifs_read_latency,
        cifs_read_ops,
        cifs_write_data,
        cifs_write_latency,
        cifs_write_ops,
        fcp_other_latency,
        fcp_other_ops,
        fcp_read_data,
        fcp_read_latency,
        fcp_read_ops,
        fcp_write_data,
        fcp_write_latency,
        fcp_write_ops,
        iscsi_other_latency,
        iscsi_other_ops,
        iscsi_read_data,
        iscsi_read_latency,
        iscsi_read_ops,
        iscsi_write_data,
        iscsi_write_latency,
        iscsi_write_ops,
        nfs_other_latency,
        nfs_other_ops,
        nfs_read_data,
        nfs_read_latency,
        nfs_read_ops,
        nfs_write_data,
        nfs_write_latency,
        nfs_write_ops,
        other_latency,
        other_ops,
        read_data,
        read_latency,
        read_ops,
        san_other_latency,
        san_other_ops,
        san_read_data,
        san_read_latency,
        san_read_ops,
        san_write_data,
        san_write_latency,
        san_write_ops,
        total_ops,
        write_data,
        write_latency,
        write_ops
    }

    public NetappVolumeVserverMetrics(final MetricsRowScratchPad<VServerVolumeCounters> currentValues, final MetricsRowScratchPad<VServerVolumeCounters> previousValues) {
        super(VServerVolumeCounters.class, currentValues, previousValues);
    }

    @Override
    public VolumeVserverMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {

        // VserverMeasurements is only partially populated here with vserver specific metrics.
        // Other metrics from other target object names (fcp lif, iscsi lif) are merged in later.
        final VolumeVserverMeasurements m = new VolumeVserverMeasurements();
        m.setAvgLatency(getNormalizedFloatMetric(avg_latency));
        m.setCifsOtherLatency(getNormalizedFloatMetric(cifs_other_latency));
        m.setCifsOtherOps(getNormalizedFloatMetric(cifs_other_ops));
        m.setCifsReadData(getNormalizedFloatMetric(cifs_read_data));
        m.setCifsReadLatency(getNormalizedFloatMetric(cifs_read_latency));
        m.setCifsReadOps(getNormalizedFloatMetric(cifs_read_ops));
        m.setCifsWriteData(getNormalizedFloatMetric(cifs_write_data));
        m.setCifsWriteLatency(getNormalizedFloatMetric(cifs_write_latency));
        m.setCifsWriteOps(getNormalizedFloatMetric(cifs_write_ops));
        m.setFcpOtherLatency(getNormalizedFloatMetric(fcp_other_latency));
        m.setFcpOtherOps(getNormalizedFloatMetric(fcp_other_ops));
        m.setFcpReadData(getNormalizedFloatMetric(fcp_read_data));
        m.setFcpReadLatency(getNormalizedFloatMetric(fcp_read_latency));
        m.setFcpReadOps(getNormalizedFloatMetric(fcp_read_ops));
        m.setFcpWriteData(getNormalizedFloatMetric(fcp_write_data));
        m.setFcpWriteLatency(getNormalizedFloatMetric(fcp_write_latency));
        m.setFcpWriteOps(getNormalizedFloatMetric(fcp_write_ops));
        m.setIscsiOtherLatency(getNormalizedFloatMetric(iscsi_other_latency));
        m.setIscsiOtherOps(getNormalizedFloatMetric(iscsi_other_ops));
        m.setIscsiReadData(getNormalizedFloatMetric(iscsi_read_data));
        m.setIscsiReadLatency(getNormalizedFloatMetric(iscsi_read_latency));
        m.setIscsiReadOps(getNormalizedFloatMetric(iscsi_read_ops));
        m.setIscsiWriteData(getNormalizedFloatMetric(iscsi_write_data));
        m.setIscsiWriteLatency(getNormalizedFloatMetric(iscsi_write_latency));
        m.setIscsiWriteOps(getNormalizedFloatMetric(iscsi_write_ops));
        m.setNfsOtherLatency(getNormalizedFloatMetric(nfs_other_latency));
        m.setNfsOtherOps(getNormalizedFloatMetric(nfs_other_ops));
        m.setNfsReadData(getNormalizedFloatMetric(nfs_read_data));
        m.setNfsReadLatency(getNormalizedFloatMetric(nfs_read_latency));
        m.setNfsReadOps(getNormalizedFloatMetric(nfs_read_ops));
        m.setNfsWriteData(getNormalizedFloatMetric(nfs_write_data));
        m.setNfsWriteLatency(getNormalizedFloatMetric(nfs_write_latency));
        m.setNfsWriteOps(getNormalizedFloatMetric(nfs_write_ops));
        m.setOtherLatency(getNormalizedFloatMetric(other_latency));
        m.setOtherOps(getNormalizedFloatMetric(other_ops));
        m.setReadData(getNormalizedFloatMetric(read_data));
        m.setReadLatency(getNormalizedFloatMetric(read_latency));
        m.setReadOps(getNormalizedFloatMetric(read_ops));
        m.setSanOtherLatency(getNormalizedFloatMetric(san_other_latency));
        m.setSanOtherOps(getNormalizedFloatMetric(san_other_ops));
        m.setSanReadData(getNormalizedFloatMetric(san_read_data));
        m.setSanReadLatency(getNormalizedFloatMetric(san_read_latency));
        m.setSanReadOps(getNormalizedFloatMetric(san_read_ops));
        m.setSanWriteData(getNormalizedFloatMetric(san_write_data));
        m.setSanWriteLatency(getNormalizedFloatMetric(san_write_latency));
        m.setSanWriteOps(getNormalizedFloatMetric(san_write_ops));
        m.setTotalOps(getNormalizedFloatMetric(total_ops));
        m.setWriteData(getNormalizedFloatMetric(write_data));
        m.setWriteLatency(getNormalizedFloatMetric(write_latency));
        m.setWriteOps(getNormalizedFloatMetric(write_ops));
        return m;
    }

    VServerVolumeCounters getDenominatorForNumerator(final VServerVolumeCounters numeratorName) {
        final VServerVolumeCounters denominatorName;
        switch (numeratorName) {

            case avg_latency:
                denominatorName = total_ops;
                break;
            case cifs_other_latency:
                denominatorName = cifs_other_ops;
                break;
            case cifs_read_latency:
                denominatorName = cifs_read_ops;
                break;
            case cifs_write_latency:
                denominatorName = cifs_write_ops;
                break;
            case fcp_other_latency:
                denominatorName = fcp_other_ops;
                break;
            case fcp_read_latency:
                denominatorName = fcp_read_ops;
                break;
            case fcp_write_latency:
                denominatorName = fcp_write_ops;
                break;
            case iscsi_other_latency:
                denominatorName = iscsi_other_ops;
                break;
            case iscsi_read_latency:
                denominatorName = iscsi_read_ops;
                break;
            case iscsi_write_latency:
                denominatorName = iscsi_write_ops;
                break;
            case nfs_other_latency:
                denominatorName = nfs_other_ops;
                break;
            case nfs_read_latency:
                denominatorName = nfs_read_ops;
                break;
            case nfs_write_latency:
                denominatorName = nfs_write_ops;
                break;
            case other_latency:
                denominatorName = other_ops;
                break;
            case read_latency:
                denominatorName = read_ops;
                break;
            case san_other_latency:
                denominatorName = san_other_ops;
                break;
            case san_read_latency:
                denominatorName = san_read_ops;
                break;
            case san_write_latency:
                denominatorName = san_write_ops;
                break;
            case write_latency:
                denominatorName = write_ops;
                break;
            default:
                denominatorName = null;
                break;
        }
        return denominatorName;
    }

    @Override
    float getNormalizedFloatMetric(final VServerVolumeCounters counterName) {

        final float normalizedMetric;
        switch (counterName) {
            case avg_latency:
            case cifs_other_latency:
            case cifs_read_latency:
            case cifs_write_latency:
            case fcp_other_latency:
            case fcp_read_latency:
            case fcp_write_latency:
            case iscsi_other_latency:
            case iscsi_read_latency:
            case iscsi_write_latency:
            case nfs_other_latency:
            case nfs_read_latency:
            case nfs_write_latency:
            case other_latency:
            case read_latency:
            case san_other_latency:
            case san_read_latency:
            case san_write_latency:
            case write_latency:
                normalizedMetric = currentValues.getLatencyPerOp(counterName, getDenominatorForNumerator(counterName));
                break;

            case cifs_read_data:
            case cifs_write_data:
            case fcp_read_data:
            case fcp_write_data:
            case iscsi_read_data:
            case iscsi_write_data:
            case nfs_read_data:
            case nfs_write_data:
            case read_data:
            case san_read_data:
            case san_write_data:
            case write_data:
            case cifs_other_ops:
            case cifs_read_ops:
            case cifs_write_ops:
            case fcp_other_ops:
            case fcp_read_ops:
            case fcp_write_ops:
            case iscsi_other_ops:
            case iscsi_read_ops:
            case iscsi_write_ops:
            case nfs_other_ops:
            case nfs_read_ops:
            case nfs_write_ops:
            case other_ops:
            case read_ops:
            case san_other_ops:
            case san_read_ops:
            case san_write_ops:
            case total_ops:
            case write_ops:
                normalizedMetric = currentValues.getCounterRatePerSec(counterName);
                break;

            default:
                normalizedMetric = NULL_FLOAT;
        }
        return normalizedMetric;
    }
}
