/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import com.netapp.oci.netapp.performance.aggregate.interfaces.data.WaflHyaAggrMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappWaflHyaAggrMetrics.WaflHyaAggrCounter.*;


public class NetappWaflHyaAggrMetrics extends MetricsRowSample<WaflHyaAggrMeasurements, WaflHyaAggrMeasurements, NetappWaflHyaAggrMetrics.WaflHyaAggrCounter, NetappWaflHyaAggrMetrics.WaflHyaAggrCounter>  {

    public enum WaflHyaAggrCounter {
        hya_read_hit_latency_average,
        hya_read_hit_latency_count,
        hya_read_miss_latency_average,
        hya_read_miss_latency_count,
        hya_write_hdd_latency_average,
        hya_write_hdd_latency_count,
        hya_write_ssd_latency_average,
        hya_write_ssd_latency_count,
        read_miss_blks_rate,
        read_ops_replaced,
        read_ops_replaced_percent,
        read_ops_total,
        ssd_read_cached,
        ssd_total,
        ssd_total_used,
        ssd_write_cached,
        write_blks_replaced,
        est_write_blks_total
    }

    public NetappWaflHyaAggrMetrics(final MetricsRowScratchPad<WaflHyaAggrCounter> currentValues, final MetricsRowScratchPad<WaflHyaAggrCounter> previousValues) {
        super(WaflHyaAggrCounter.class, currentValues, previousValues);
    }


    @Override
    float getNormalizedFloatMetric(final WaflHyaAggrCounter counterName) {
        final float normalizedMetric;
        switch (counterName) {
            case hya_read_hit_latency_average:
                normalizedMetric = currentValues.getLatencyPerOp(hya_read_hit_latency_average, hya_read_hit_latency_count);
                break;

            case hya_read_miss_latency_average:
                normalizedMetric = currentValues.getLatencyPerOp(hya_read_miss_latency_average, hya_read_miss_latency_count);
                break;

            case hya_write_hdd_latency_average:
                normalizedMetric = currentValues.getLatencyPerOp(hya_write_hdd_latency_average, hya_write_hdd_latency_count);
                break;

            case hya_write_ssd_latency_average:
                normalizedMetric = currentValues.getLatencyPerOp(hya_write_ssd_latency_average, hya_write_ssd_latency_count);
                break;

            case read_ops_replaced_percent:
                normalizedMetric = currentValues.getPercentFromDeltas(read_ops_replaced_percent, read_ops_total);
                break;

            case write_blks_replaced:
                normalizedMetric = currentValues.getPercentFromDeltas(write_blks_replaced, est_write_blks_total);
                break;

            case read_miss_blks_rate:
            case read_ops_replaced:
            case ssd_read_cached:
            case ssd_total:
            case ssd_total_used:
            case ssd_write_cached:
                normalizedMetric = currentValues.getCounterRatePerSec(counterName);
                break;

            default:
                normalizedMetric = NULL_FLOAT;
                break;
        }
        return normalizedMetric;
    }

    @Override
    public WaflHyaAggrMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
        final WaflHyaAggrMeasurements waflHyaAggrMeasurements = new WaflHyaAggrMeasurements();
        waflHyaAggrMeasurements.setHyaReadHitLatencyAverage(getNormalizedFloatMetric(hya_read_hit_latency_average));
        waflHyaAggrMeasurements.setHyaReadMissLatencyAverage(getNormalizedFloatMetric(hya_read_miss_latency_average));
        waflHyaAggrMeasurements.setHyaWriteHddLatencyAverage(getNormalizedFloatMetric(hya_write_hdd_latency_average));
        waflHyaAggrMeasurements.setHyaWriteSsdLatencyAverage(getNormalizedFloatMetric(hya_write_ssd_latency_average));
        waflHyaAggrMeasurements.setReadMissBlksRate(getNormalizedFloatMetric(read_miss_blks_rate));
        waflHyaAggrMeasurements.setReadOpsReplaced(getNormalizedFloatMetric(read_ops_replaced));
        waflHyaAggrMeasurements.setReadOpsReplacedPercent(getNormalizedFloatMetric(read_ops_replaced_percent));
        waflHyaAggrMeasurements.setSsdReadCached(getNormalizedFloatMetric(ssd_read_cached));
        waflHyaAggrMeasurements.setSsdTotal(getNormalizedFloatMetric(ssd_total));
        waflHyaAggrMeasurements.setSsdTotalUsed(getNormalizedFloatMetric(ssd_total_used));
        waflHyaAggrMeasurements.setSsdWriteCached(getNormalizedFloatMetric(ssd_write_cached));
        waflHyaAggrMeasurements.setWritesBlksReplaced(getNormalizedFloatMetric(write_blks_replaced));
        return waflHyaAggrMeasurements;
    }
}
