package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model;

/**
 * Contains a raw String value and a corresponding Enum.
 *
 * @param <T> the mapped Enum &lt;T extends Enum&lt;T&gt;&gt;
 */
public class EnumWrapper<T extends Enum<T>> {
    private final String rawValue;
    private final T enumValue;

    /**
     * Returns a EnumWrapper instance
     *
     * @param rawValue  the raw String value
     * @param enumValue the mapped Enum
     */
    public EnumWrapper(final String rawValue, final T enumValue) {
        this.rawValue = rawValue;
        this.enumValue = enumValue;
    }

    public String getRawValue() {
        return rawValue;
    }

    public T getEnumValue() {
        return enumValue;
    }

    public String toString() {
        // toString must never return null
        return (enumValue != null) ? enumValue.name() : (rawValue != null) ? rawValue : "";
    }

    public static String toStringOrNull(final EnumWrapper enumWrapper) {
        return enumWrapper == null ? null : enumWrapper.toString();
    }
}
