package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeAggregateSpare extends BaseCRCXMLNode<CModeCluster> {

	private AggregateSpareKey key;

	private String originalOwner;
	private String diskName;
	private String diskType;

	private Long usableDataSize;
	private Long usableRootSize;

    public CModeAggregateSpare(final CModeCluster parent) {
        super(parent, CModeAggregateSpare.class.getSimpleName());
    }

	public Long getUsableDataSize() {
		return usableDataSize;
	}

	public CModeAggregateSpare setUsableDataSize(final Long usableDataSize) {
		this.usableDataSize = usableDataSize;
		return this;
	}

	public Long getUsableRootSize() {
		return usableRootSize;
	}

	public CModeAggregateSpare setUsableRootSize(final Long usableRootSize) {
		this.usableRootSize = usableRootSize;
		return this;
	}

	public String getOriginalOwner() {
		return originalOwner;
	}

	public CModeAggregateSpare setOriginalOwner(final String originalOwner) {
		this.originalOwner = originalOwner;
		return this;
	}

	public String getDiskName() {
		return diskName;
	}

	public CModeAggregateSpare setDiskName(final String diskName) {
		this.diskName = diskName;
		return this;
	}

	public String getDiskType() {
		return diskType;
	}

	public CModeAggregateSpare setDiskType(final String diskType) {
		this.diskType = diskType;
		return this;
	}

	@Override
	public AggregateSpareKey getKey() {
		if (key == null) {
	        key = new AggregateSpareKey(diskName);
		}
		return key;
	}

    @Override
	public String toString() {
        StringBuilder sb = new StringBuilder(128);
        sb.append(super.toString())
            .append("[diskName=").append(diskName)
            .append(", diskType=").append(diskType)
            .append(", originalOwner=").append(originalOwner)
            .append(", usableDataSize=").append(usableDataSize)
            .append(", usableRootSize=").append(usableRootSize)
            .append(']');
        return sb.toString();
    }

    @Override
	protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addAttributeNoCRC(writer, "ResourceKey", getKey());
        addAttributeNoCRC(writer, "UUID", key.getOcieUUID());
        addAttributeNoCRC(writer, "DiskName", diskName);
        addAttributeNoCRC(writer, "DiskType", diskType);
        addAttributeNoCRC(writer, "OriginalOwner", originalOwner);
        addAttributeNoCRC(writer, "UsableDataSize", usableDataSize);
        addAttributeNoCRC(writer, "UsableRootSize", usableRootSize);
    }

    public final static class AggregateSpareKey extends ModelKeyBase<CModeAggregateSpare.AggregateSpareKey.Field> {

        public static enum Field {
        	disk_name,
        }

        public AggregateSpareKey(final String diskName) {
            super("aggregate_spare_key", new EnumMap<Field, String>(Field.class) {{
                put(Field.disk_name, diskName);
            }});
        }
    }

}