package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.CIFSShareACL;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeCIFSACL extends BaseCRCXMLNode<CModeCluster> {

    public final static class ACLKey extends ModelKeyBase<ACLKey.Field> {
        public static enum Field {
            vserver_name,
            share_name,
            user_or_group,
        }

        public ACLKey(final String vServerName, final String shareName, final String userOrGroup) {
            super("cifs_share_acl",
                  new EnumMap<Field, String>(Field.class) {{
                        put(Field.vserver_name, vServerName);
                        put(Field.share_name, shareName);
                        put(Field.user_or_group, userOrGroup);
                  }});
        }
    }

    private final ACLKey key;

    private EnumWrapper<CIFSShareACL.Permission> permissionWrapper;


    CModeCIFSACL(final CModeCluster parent, final String vServerName, final String shareName, final String userOrGroup) {
        super(parent, CModeCIFSACL.class.getSimpleName());
        this.key = new ACLKey(vServerName, shareName, userOrGroup);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public ACLKey getKey() {
        return key;
    }

    public EnumWrapper<CIFSShareACL.Permission> getPermissionWrapper() {
        return permissionWrapper;
    }

    public void setPermissionWrapper(final EnumWrapper<CIFSShareACL.Permission> permissionWrapper) {
        this.permissionWrapper = permissionWrapper;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "vServerName", key.getID(ACLKey.Field.vserver_name));
        addAttributeNoCRC(writer, "shareName", key.getID(ACLKey.Field.share_name));
        addAttributeNoCRC(writer, "userOrGroup", key.getID(ACLKey.Field.user_or_group));
        addAttributeNoCRC(writer, "Permission", permissionWrapper.getRawValue());
    }
}
