package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;

public class CModeCIFSServer extends BaseCRCXMLNode<CModeCluster> {

    private final UUIDKey key;
    private String vServerName;
    private String domain;
    private String domainWorkgroup;
    private EnumWrapper<VServer.CifsAuthStyle> authSytle;
    private EnumWrapper<VServer.CifsAdminStatus> adminStatus;

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    CModeCIFSServer(final CModeCluster parent, final String name) {
        super(parent, CModeCIFSServer.class.getSimpleName());
        this.key = new UUIDKey("cifs_server", name);
    }

    public String getName() {
        return key.getID();
    }

    public String getVServerName() {
        return vServerName;
    }

    public void setVServerName(final String vServerName) {
        this.vServerName = vServerName;
    }

    public String getDomain() {
        return domain;
    }

    public void setDomain(final String domain) {
        this.domain = domain;
    }

    public String getDomainWorkgroup() {
        return domainWorkgroup;
    }

    public void setDomainWorkgroup(final String domainWorkgroup) {
        this.domainWorkgroup = domainWorkgroup;
    }

    public EnumWrapper<VServer.CifsAuthStyle> getAuthSytle() {
        return authSytle;
    }

    public void setAuthSytle(final EnumWrapper<VServer.CifsAuthStyle> authSytle) {
        this.authSytle = authSytle;
    }

    public EnumWrapper<VServer.CifsAdminStatus> getAdminStatus() {
        return adminStatus;
    }

    public void setAdminStatus(final EnumWrapper<VServer.CifsAdminStatus> adminStatus) {
        this.adminStatus = adminStatus;
    }


    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "Name", key.getID());
        addAttributeNoCRC(writer, "VServerName", vServerName);
        addAttributeNoCRC(writer, "Domain", domain);
        addAttributeNoCRC(writer, "DomainWorkgroup", domainWorkgroup);
        addAttributeNoCRC(writer, "AuthStyle", authSytle.getEnumValue());
        addAttributeNoCRC(writer, "AuthStyleRaw", authSytle.getRawValue());
        addAttributeNoCRC(writer, "AdminStatus", adminStatus.getEnumValue());
        addAttributeNoCRC(writer, "AdminStatusRaw", adminStatus.getRawValue());
    }
}
