package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapperList;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.CIFSShare;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeCIFSShare extends BaseCRCXMLNode<CModeCluster> {

    public final static class CIFSShareKey extends ModelKeyBase<CIFSShareKey.Field> {
        public static enum Field {
            cifs_server_name,
            cifs_share_name
        }

        public CIFSShareKey(final String cifsServerName, final String cifsShareName) {
            super("cifs_share", new EnumMap<Field, String>(Field.class) {{
                put(Field.cifs_server_name, cifsServerName);
                put(Field.cifs_share_name, cifsShareName);
            }});
        }
    }


    private final CIFSShareKey key;
    private String path;
    private String comment;
    private final EnumWrapperList<CIFSShare.ShareProperty> shareProperties = new EnumWrapperList<CIFSShare.ShareProperty>();


    CModeCIFSShare(final CModeCluster parent, final String cifsServerName, final String cifsShareName) {
        super(parent, CModeCIFSShare.class.getSimpleName());
        this.key = new CIFSShareKey(cifsServerName, cifsShareName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public CIFSShareKey getKey() {
        return key;
    }

    public String getCIFSServerName() {
        return key.getID(CIFSShareKey.Field.cifs_server_name);
    }

    public String getPath() {
        return path;
    }

    public String getCIFSShareName() {
        return key.getID(CIFSShareKey.Field.cifs_share_name);
    }

    public String getComment() {
        return comment;
    }

    public EnumWrapperList<CIFSShare.ShareProperty> getShareProperties() {
        return shareProperties;
    }

    public void setPath(final String path) {
        this.path = path;
    }

    public void setComment(final String comment) {
        this.comment = comment;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", key.getID());
        addAttributeNoCRC(writer, "CifsServerName", getCIFSServerName());
        addAttributeNoCRC(writer, "CifsShareName", getCIFSShareName());
        addAttributeNoCRC(writer, "Path", path);
        addAttributeNoCRC(writer, "Comment", comment);
    }
}
