package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation.BuilderModel;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation.BuilderUtil;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.util.VersionComparator;
import org.apache.log4j.Logger;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CModeCluster extends BaseCRCXMLNode<MarshallingCache> implements BuilderModel
{

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(CModeCluster.class);

    private String managementIP;            // <operator configured target management IP>


    // cluster-identity-info from cluster-identity-get
    private final UUIDKey key;              // cluster-uuid
    private String name;                    // cluster-name
    private String serialNum;               // cluster-serial-number
    private String contact;                 // cluster-contact
    private String location;                // cluster-location

    // version from system-get-version
    private String versionString;           // version

    // version-tuple from system-get-version
    private Integer generation;             // generation
    private Integer major;                  // major
    private Integer minor;                  // minor

    // clock-get-timezone
    private String timezone;                // timezone
    private String timezoneUTC;             // timezone-UTC
    private String timezoneVersion;         // timezone-version

    //attribute for metrcluster from metrocluster-is-configured
    private boolean isMetrocluster;   //a value indicating if the cluster is configured for MetroCluster.
    
    //attributes for metrocluster from metrocluster-get
    private EnumWrapper<Cluster.MtConfigurationState> mtConfigurationState;    //state of the local cluster config: metrocluster-info.local-configuration-state
    private String mtConfigurationStateRaw; //Raw enum value from ZAPI
    private EnumWrapper<Cluster.MtMode> mtMode;  //Mode of operation of local cluster: metrocluster-get. metrocluster-info.local-mode
    private String mtModeRaw; // Raw enum value from ZAPI
    private String mtDrPartnerClusterID;       //partner cluster on the metrocluster. Null if this node is not in a metrocluster
    private EnumWrapper<Cluster.MtConfigurationState> mtRemoteConfigurationState;    //state of the remote cluster config: metrocluster-info.remote-configuration-state
    private String mtRemoteConfigurationStateRaw; //Raw enum value from ZAPI
    private EnumWrapper<Cluster.MtMode> mtRemoteMode;  //Mode of operation of remote cluster: metrocluster-get. metrocluster-info.remote-mode
    private String mtRemoteModeRaw; // Raw enum value from ZAPI
    private String mtConfigurationType;       //metrocluster configuration type

    // Some CPU metrics are type "array", a single counter name returns a CSV
    // of sub-column values.   We must dynamically discover the sub-column
    // names in order to interpret the counters correctly in performance.
    private List<String> cpuColumns;
    
    private List<String> waflCpPhaseTimesColumns;

    // Some Volume Workload metrics are type "array", a single counter name returns a CSV
    // of sub-column values.   We must dynamically discover the sub-column
    // names in order to interpret the counters correctly in performance.
    private List<String> volumeWorkloadReadIOColumns;
    // Node performance UUID maps
    final private Map<String, String> cifsNodeUUIDNameMap = new HashMap<String, String>();
    final private Map<String, String> smb1NodeUUIDNameMap = new HashMap<String, String>();
    final private Map<String, String> smb2NodeUUIDNameMap = new HashMap<String, String>();
    final private Map<String, String> nfsv3NodeUUIDNameMap = new HashMap<String, String>();
    final private Map<String, String> nfsv4NodeUUIDNameMap = new HashMap<String, String>();
    final private Map<String, String> nfsv4_1NodeUUIDNameMap = new HashMap<String, String>();

    // vserverID and vserverName map for nfs/cifs/lif 
    final private Map<String, String> cifsVserverIDNameMap = new HashMap<String, String>();
    final private Map<String, String> nfsv3VserverIDNameMap = new HashMap<String, String>();
    final private Map<String, String> nfsv4VserverIDNameMap = new HashMap<String, String>();
    final private Map<String, String> nfsv41VserverIDNameMap = new HashMap<String, String>();
    final private Map<String, String> lifVserverIDNameMap = new HashMap<String, String>();
    
    // node-version-detail-info from system-get-version
    Map<String, String> nodeVersions;       // node-version-detail-info.node-uuid, version

    private boolean isLessThanSierraNevada = false;

    /**
     * Overall system health as determined by the diagnosis framework from diagnosis-status-get
     */
    private EnumWrapper<Cluster.HmStatus> diagnosisStatus;

    /**
     * Is Spare Low from aggr-check-spare-low
     */
    private Boolean isSpareLow;

    // Children
    private final CModeNodes nodes;
    private final CModeNodeDetails nodeDetails;
    private final CModeNodeFailovers nodeFailovers;
    private final CModeNodeMetroclusters nodeMetroclusters;
    private final CModeAggregates aggregates;
    private final CModeAggregateSpares aggregateSpares;
    private final CModeMetroClusterOperations metroClusterOperations;
    private final CModeStorageSwitches storageSwitches;
    private final CModeStoragePools storagePools;
    private final CModeStoragePoolAvailableSizes storagePoolAvailableSizes;
    private final CModeStoragePoolAggregates storagePoolAggregates;
    private final CModeDisks disks;
    private final CModeDiskAggregateRelationships diskAggregateRelationships;
    private final CModeNetworkFailoverGroupNetworkPortRelationships networkFailoverGroupNetworkPortRelationships;
    private final CModeVServers vServers;
    private final CModeVolumes volumes;
    private final CModeVolumeClones volumeClones;
    private final CModeVolumeMoves volumeMoves;
    private final CModeVolumeFlexCaches volumeFlexCaches;
    private final CModeVolumeFootprints volumeFootprints;
    private final CModeQuotaStatuses quotaStatuses;
    private final CModeSnapshots snapshots;
    private final CModeSisStatuses sisStatuses;
    private final CModeQTrees qtrees;
    private final CModeLUNs luns;
    private final CModeLunImports lunImports;
    private final CModeAdapterStatuses adapterStatuses;
    private final CModeAdapters adapters;
    private final CModeNetInterfaces ports;
    private final CModeNetworkIpSpaces networkIpSpaces;
    private final CModeNetworkSubnets networkSubnets;
    private final CModeNetworkPortBroadcastDomains networkPortBroadcastDomains;
    private final CModeNetworkFailoverGroups networkFailoverGroups;
    private final CModeLifs lifs;
    private final CModeFcpServiceInfos fcpServiceInfos;
    private final CModeFcpLifs fcpLifs;
    private final CModeFcpAdapters fcpAdapters;
    private final CModeIGroups iGroups;
    private final CModePortSets portSets;
    private final CModeLUNMaps lunMaps;
    private final CModeCIFSServers cifsServers;
    private final CModeCIFSShares cifsShares;
    private final CModeCIFSACLs cifsShareACLs;
    private final CModeCIFSOptions cifsOptions;
    private final CModeExportRules exportRules;
    private final CModeExportPolicies exportPolicies;
    private final CModeNFSServices nfsServices;
    private final CModeISCSIServices iscsiServices;
    private final CModeISCSIPortalGroups iscsiPortalGroups;
    private final CModeISCSIPortalGroupToLIFs iscsiPortalGroupToLIFs;
    private final CModeISCSISessions iscsiSessions;
    private final CModeISCSISecurityEntries iscsiSecurityEntries;
    private final CModeLicenses licenses;
    private final CModeQuotas quotas;
    private final CModeSnapMirrors snapMirrors;
    private final CModeSnapMirrorPolicies snapMirrorPolicies;
    private final CModeVServerPeers vServerPeers;
    private final CModeSnapMirrorHistories snapMirrorHistories;
    private final CModeVServerDNSInfos dnsInfos;
    private final CModeVServerDDNSInfos ddnsInfos;
    private final CModeVServerNISInfos nisInfos;
    private final CModeVServerLDAPInfos ldapInfos;
    private final CModeVServerCIFSSecurityInfos cifsSecInfos;
    private final CModeVServerNameMappings vServerNameMappings;
    private final CModeQosVolumeWorkloads qosVolumeWorkloads;
    private final CModeQosServiceCenters qosServiceCenters;
    // private final CModeQosWorkloadDetails qosWorkloadDetails;
    private final CModeJobSchedules jobSchedules;
    private final CModeCronJobSchedules cronJobSchedules;
    private final CModeJobScheduleIntervals jobScheduleIntervals;
    private final CModeSisPolicies sisPolicies;
    private final CModeSnapshotPolicies snapshotPolicies;
    private final CModeQosPolicyGroups qosPolicyGroups;
    private final CModeNetRoutes netRoutes;
    private final CModeKerberos kerberos;
    private final CModeStorageServices storageServices;
    private final CModeApplicationRecords applicationRecords;
    private Boolean haConfigured;
    private final CModeSystemHealthAlerts systemHealthAlerts;
    private Boolean sslFipsEnabled;
    private final CModeClusterPeers clusterPeers;

    CModeCluster(final CModeClusterRoot parent, final String clusterUUID) {
        super(parent, CModeCluster.class.getSimpleName());
        this.key = new UUIDKey("cluster", clusterUUID);
        this.nodes = new CModeNodes(this);
        this.nodeDetails = new CModeNodeDetails(this);
        this.nodeFailovers = new CModeNodeFailovers(this);
        this.nodeMetroclusters = new CModeNodeMetroclusters(this);
        this.aggregates = new CModeAggregates(this);
        this.metroClusterOperations = new CModeMetroClusterOperations(this);
        this.aggregateSpares = new CModeAggregateSpares(this);
        this.storageSwitches = new CModeStorageSwitches(this);
        this.storagePools = new CModeStoragePools(this);
        this.storagePoolAvailableSizes = new CModeStoragePoolAvailableSizes(this);
        this.storagePoolAggregates = new CModeStoragePoolAggregates(this);
        this.disks = new CModeDisks(this);
        this.diskAggregateRelationships = new CModeDiskAggregateRelationships(this);
        this.networkFailoverGroupNetworkPortRelationships = new CModeNetworkFailoverGroupNetworkPortRelationships(this);
        this.vServers = new CModeVServers(this);
        this.volumes = new CModeVolumes(this);
        this.volumeClones = new CModeVolumeClones(this);
        this.volumeMoves = new CModeVolumeMoves(this);
        this.volumeFlexCaches = new CModeVolumeFlexCaches(this);
        this.volumeFootprints = new CModeVolumeFootprints(this);
        this.quotaStatuses = new CModeQuotaStatuses(this);
        this.snapshots = new CModeSnapshots(this);
        this.sisStatuses = new CModeSisStatuses(this);
        this.qtrees = new CModeQTrees(this);
        this.luns = new CModeLUNs(this);
        this.lunImports = new CModeLunImports(this);
        this.adapterStatuses = new CModeAdapterStatuses(this);
        this.adapters = new CModeAdapters(this);
        this.lifs = new CModeLifs(this);
        this.fcpServiceInfos = new CModeFcpServiceInfos(this);
        this.fcpLifs = new CModeFcpLifs(this);
        this.ports = new CModeNetInterfaces(this);
        this.networkIpSpaces = new CModeNetworkIpSpaces(this);
        this.networkSubnets = new CModeNetworkSubnets(this);
        this.networkPortBroadcastDomains = new CModeNetworkPortBroadcastDomains(this);
        this.networkFailoverGroups = new CModeNetworkFailoverGroups(this);
        this.fcpAdapters = new CModeFcpAdapters(this);
        this.iGroups = new CModeIGroups(this);
        this.portSets = new CModePortSets(this);
        this.lunMaps = new CModeLUNMaps(this);
        this.cifsServers = new CModeCIFSServers(this);
        this.cifsShares = new CModeCIFSShares(this);
        this.cifsShareACLs = new CModeCIFSACLs(this);
        this.cifsOptions = new CModeCIFSOptions(this);
        this.exportRules = new CModeExportRules(this);
        this.nfsServices = new CModeNFSServices(this);
        this.exportPolicies = new CModeExportPolicies(this);
        this.iscsiServices = new CModeISCSIServices(this);
        this.iscsiPortalGroups = new CModeISCSIPortalGroups(this);
        this.iscsiPortalGroupToLIFs = new CModeISCSIPortalGroupToLIFs(this);
        this.iscsiSessions = new CModeISCSISessions(this);
        this.iscsiSecurityEntries = new CModeISCSISecurityEntries(this);
        this.licenses = new CModeLicenses(this);
        this.quotas = new CModeQuotas(this);
        this.snapMirrors = new CModeSnapMirrors(this);
        this.snapMirrorPolicies = new CModeSnapMirrorPolicies(this);
        this.snapMirrorHistories = new CModeSnapMirrorHistories(this);
        this.vServerPeers = new CModeVServerPeers(this);
        this.dnsInfos = new CModeVServerDNSInfos(this);
        this.ddnsInfos = new CModeVServerDDNSInfos(this);
        this.nisInfos = new CModeVServerNISInfos(this);
        this.ldapInfos = new CModeVServerLDAPInfos(this);
        this.cifsSecInfos = new CModeVServerCIFSSecurityInfos(this);
        this.vServerNameMappings = new CModeVServerNameMappings(this);
        this.qosVolumeWorkloads = new CModeQosVolumeWorkloads(this);
        this.qosServiceCenters = new CModeQosServiceCenters(this);
        this.jobSchedules = new CModeJobSchedules(this);
        this.cronJobSchedules = new CModeCronJobSchedules(this);
        this.jobScheduleIntervals = new CModeJobScheduleIntervals(this);
        this.sisPolicies = new CModeSisPolicies(this);
        this.snapshotPolicies = new CModeSnapshotPolicies(this);
        this.qosPolicyGroups = new CModeQosPolicyGroups(this);
        this.netRoutes = new CModeNetRoutes(this);
        this.kerberos = new CModeKerberos(this);
        this.storageServices = new CModeStorageServices(this);
        this.applicationRecords = new CModeApplicationRecords(this);
        this.systemHealthAlerts = new CModeSystemHealthAlerts(this);
        this.clusterPeers = new CModeClusterPeers(this);
    }

  public CModeNetworkFailoverGroupNetworkPortRelationships getNetworkFailoverGroupNetworkPortRelationships() {
    return networkFailoverGroupNetworkPortRelationships;
  }

  public String getClusterUUID() {
        return key.getID();
    }

    public String getClusterName() {
        return name;
    }

    public void setClusterName(final String name) {
        this.name = name;
    }

    public String getManagementIP() {
        return managementIP;
    }

    public void setManagementIP(final String managementIP) {
        this.managementIP = managementIP;
    }

    public String getSerialNum() {
        return serialNum;
    }

    public void setSerialNum(final String serialNum) {
        this.serialNum = serialNum;
    }

    public String getVersionString() {
        return versionString;
    }

    public void setVersionString(final String versionString) {
        this.versionString = versionString;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public boolean isPreSierraNevada() {
        final VersionComparator versionComparator = new VersionComparator();
        final String clusterVersion = getVersionString();
        @SuppressWarnings({"UnnecessaryLocalVariable"})
        final boolean lessThanSN = (versionComparator.compare(clusterVersion, Cluster.SIERRA_NEVADA) < 0);     // RR is less than SN
        return lessThanSN;
    }

    public Integer getGeneration() {
        return generation;
    }

    public void setGeneration(final Integer generation) {
        this.generation = generation;
    }

    public Integer getMajor() {
        return major;
    }

    public void setMajor(final Integer major) {
        this.major = major;
    }

    public Integer getMinor() {
        return minor;
    }

    public void setMinor(final Integer minor) {
        this.minor = minor;
    }

    public String getTimezone() {
        return timezone;
    }

    public void setTimezone(final String timezone) {
        this.timezone = timezone;
    }

    public String getTimezoneUTC() {
        return timezoneUTC;
    }

    public void setTimezoneUTC(final String timezoneUTC) {
        this.timezoneUTC = timezoneUTC;
    }

    public String getTimezoneVersion() {
        return timezoneVersion;
    }

    public void setTimezoneVersion(final String timezoneVersion) {
        this.timezoneVersion = timezoneVersion;
    }

    public List<String> getCpuColumns() {
        return cpuColumns;
    }

    public void setCpuColumns(final List<String> cpuColumns) {
        this.cpuColumns = cpuColumns;
    }
    
    public List<String> getWaflCpPhaseTimesColumns(){
    	return waflCpPhaseTimesColumns;
    }
    
    public void setWaflCpPhaseTimesColumns(final List<String> waflCpPhaseTimesColumns){
    	this.waflCpPhaseTimesColumns = waflCpPhaseTimesColumns;
    }

    public List<String> getVolumeWorkloadReadIOColumns() {
        return volumeWorkloadReadIOColumns;
    }

    public void setVolumeWorkloadReadIOColumns(final List<String> volumeWorkloadReadIOColumns) {
        this.volumeWorkloadReadIOColumns = volumeWorkloadReadIOColumns;
    }

    public String getContact() {
        return contact;
    }

    public void setContact(final String contact) {
        this.contact = contact;
    }

    public String getLocation() {
        return location;
    }

    public void setLocation(final String location) {
        this.location = location;
    }

    public void setHaConfigured(final Boolean haConfigured) {
        this.haConfigured = haConfigured;
    }

    public Boolean getHaConfigured() {
        return haConfigured;
    }

    public Map<String, String> getCModeNodeVersions() {
        return nodeVersions;
    }

    public void setCModeNodeVersions(final Map<String, String> nodeVersions) {
        this.nodeVersions = nodeVersions;
    }

    public CModeVServer createVServer(final String uuID) {
        return new CModeVServer(this, uuID);
    }

    public CModeVServers getVServers() {
        return vServers;
    }

    public CModeVolume createVolume(final String instanceUUID) {
        return new CModeVolume(this, instanceUUID);
    }

    public CModeVolumes getVolumes() {
        return volumes;
    }

    public CModeVolumeClone createVolumeClone(final String vServerName, final String volumeName) {
        return new CModeVolumeClone(this, vServerName, volumeName);
    }

    public CModeVolumeClones getVolumeClones() {
        return volumeClones;
    }

    public CModeVolumeFlexCache createVolumeFlexCache(final String vServerName, final String cacheVolumeName) {
        return new CModeVolumeFlexCache(this, vServerName, cacheVolumeName);
    }

    public CModeVolumeMove createVolumeMove(final String jobUUID) {
        return new CModeVolumeMove(this, jobUUID);
    }

    public CModeVolumeMoves getVolumeMoves() {
        return volumeMoves;
    }

    public CModeVolumeFlexCaches getVolumeFlexCaches() {
        return volumeFlexCaches;
    }

    public CModeVolumeFootprints getVolumeFootprints() {
        return volumeFootprints;
    }

    public CModeVolumeFootprint createVolumeFootprint(final String vServerName, final String volumeName) {
        return new CModeVolumeFootprint(this, vServerName, volumeName);
    }

    public CModeQuotaStatus createAndAddQuotaStatus(final String vserverName, final String volName){
        final CModeQuotaStatus cModeQuotaStatus = new CModeQuotaStatus(this, vserverName, volName);
        quotaStatuses.add(cModeQuotaStatus);
        return cModeQuotaStatus;
    }

    public CModeQuotaStatuses getQuotaStatuses(){
        return quotaStatuses;
    }

    public CModeSisStatus createSisStatus(final String vServerName, final String volumePath) {
        return new CModeSisStatus(this, vServerName, volumePath);
    }

    public CModeSnapshots getSnapshots() {
        return snapshots;
    }

    public CModeSnapshot createSnapshot(final String vServerName, final String volumeName, final String instanceUUID) {
        return new CModeSnapshot(this, vServerName, volumeName, instanceUUID);
    }

    public CModeSisStatuses getSisStatuses() {
        return this.sisStatuses;
    }

    public CModeAggregate createAggregate(final String uuid) {
        return new CModeAggregate(this, uuid);
    }

    public CModeAggregates getAggregates() {
        return aggregates;
    }

    public CModeAggregateSpares getAggregateSpares() {
        return aggregateSpares;
    }
    
    public CModeMetroClusterOperation createMetroClusterOperation (final String uuid) {
        return new CModeMetroClusterOperation(this, uuid);
    }

    public CModeMetroClusterOperations  getMetroClusterOperations() {
        return metroClusterOperations;
    }

    public CModeNodeDetail createNodeDetail(final String nodeUUID) {
        return new CModeNodeDetail(this, nodeUUID);
    }

    public CModeNodeDetails getNodeDetails() {
        return nodeDetails;
    }

    public CModeNode createNode(final String systemName) {
        return new CModeNode(this, systemName);
    }

    public CModeNodes getCModeNodes() {
        return nodes;
    }

    public CModeNodeFailover createNodeFailover(final String nodeUUID) {
        return new CModeNodeFailover(this, nodeUUID);
    }

    public CModeNodeFailovers getNodeFailovers() {
        return nodeFailovers;
    }

    public CModeNodeMetrocluster createNodeMetrocluster(final String nodeName) {
        return new CModeNodeMetrocluster(this, nodeName);
    }

    public CModeNodeMetroclusters getNodeMetroclusters() {
        return nodeMetroclusters;
    }

    public CModeDisk createDisk(final String diskUUID) {
        return new CModeDisk(this, diskUUID);
    }

    public CModeDisks getDisks() {
        return disks;
    }

    public CModeDiskAggregateRelationship createDiskAggregateRelationship(final String diskName, final String aggregateUUID) {
        return new CModeDiskAggregateRelationship(this, diskName, aggregateUUID);
    }

    public CModeNetworkFailoverGroupNetworkPortRelationship createNetworkFailoverGroupNetworkPortRelationship(String serverNodeName, final String networkFailoverGroupName, final String networkPortName) {
      return new CModeNetworkFailoverGroupNetworkPortRelationship(this, serverNodeName, networkFailoverGroupName, networkPortName);
    }

    public CModeDiskAggregateRelationships getDiskAggregateRelationships() {
        return diskAggregateRelationships;
    }

    public CModeStoragePool createStoragePool(final String storagePoolName) {
        return new CModeStoragePool(this, storagePoolName);
    }

    public CModeStoragePoolAvailableSize createStoragePoolAvailableSize(final String nodeName, final String storagePoolName) {
        return new CModeStoragePoolAvailableSize(this, nodeName, storagePoolName);
    }

    public CModeStoragePoolAggregate createStoragePoolAggregate(final String storagePoolName, final String aggregateName) {
        return new CModeStoragePoolAggregate(this, storagePoolName, aggregateName);
    }

    public CModeStorageSwitches getStorageSwitches() {
        return storageSwitches;
    }

    public CModeStorageSwitch createStorageSwitch(final String wwn) {
        return new CModeStorageSwitch(this, wwn);
    }

    public CModeStoragePools getStoragePools() {
        return storagePools;
    }

    public CModeStoragePoolAvailableSizes getStoragePoolAvailableSizes() {
        return storagePoolAvailableSizes;
    }

    public CModeStoragePoolAggregates getStoragePoolAggregates() {
        return storagePoolAggregates;
    }

    public CModeQTree createQTree(final String vServerName, final String volumeName, final String qTreeID) {
        return new CModeQTree(this, vServerName, volumeName, qTreeID);
    }

    public CModeQTrees getQTrees() {
        return qtrees;
    }

    public CModeLUN createLUN(final String uuid) {
        return new CModeLUN(this, uuid);
    }

    public CModeLunImport createLunImport(final String vServerName, final String foreignDisk) {
        return new CModeLunImport(this, vServerName, foreignDisk);
    }
    
    public CModeLUNs getLUNs() {
        return luns;
    }
    
    public CModeLunImports getLunImports() {
        return lunImports;
    }

    public CModeAdapterStatus createAdapterStatus(final String adapterName, final String nodeName) {
        return new CModeAdapterStatus(this, adapterName, nodeName);
    }

    public CModeAdapterStatuses getAdapterStatuses() {
        return adapterStatuses;
    }

    public CModeLif createLif(final String interfaceName, final String vServerName) {
        return new CModeLif(this, interfaceName, vServerName);
    }

    public CModeLifs getLifs() {
        return lifs;
    }

    public CModeFcpServiceInfo createFcpServiceInfo(final String vServerName) {
        return new CModeFcpServiceInfo(this, vServerName);
    }

    public CModeFcpServiceInfos getFcpServiceInfos() {
        return fcpServiceInfos;
    }

    public CModeFcpLif createFcpLif(final String interfaceName, final String vServerName) {
        return new CModeFcpLif(this, interfaceName, vServerName);
    }

    public CModeFcpLifs getFcpLifs() {
        return fcpLifs;
    }

    public CModeNetInterface createNetInterface(final String nodeName, final String portName) {
        return new CModeNetInterface(this, nodeName, portName);
    }

    public CModeNetInterfaces getNetInterfaces() {
        return ports;
    }

    public CModeNetworkIPSpace createNetworkIPSpace(final String uuid) {
        return new CModeNetworkIPSpace(this, uuid);
    }

    public CModeNetworkSubnet createNetworkSubnet(final String ipspaceName, final String name) {
        return new CModeNetworkSubnet(this, ipspaceName, name);
    }

    public CModeNetworkIpSpaces getNetworkIPSpaces() {
        return networkIpSpaces;
    }

    public CModeNetworkSubnets getNetworkSubnets() {
        return networkSubnets;
    }

    public CModeNetworkPortBroadcastDomain createNetPortBroadcastDomain(final String broadcastDomainName,
                                                                        final String ipSpaceName) {
        return new CModeNetworkPortBroadcastDomain(this, broadcastDomainName, ipSpaceName);
    }

    public CModeNetworkPortBroadcastDomains getNetworkPortBroadcastDomains() {
        return networkPortBroadcastDomains;
    }

    public CModeNetworkFailoverGroup createNetworkFailoverGroup(final String failoverGroupName) {
        return new CModeNetworkFailoverGroup(this, failoverGroupName);
    }

    public CModeNetworkFailoverGroups getNetworkFailoverGroups() {
        return networkFailoverGroups;
    }

    public CModeFcpAdapter createFcpAdapter(final String nodeName, final String adapterName) {
        return new CModeFcpAdapter(this, nodeName, adapterName);
    }

    public CModeFcpAdapters getFcpAdapters() {
        return fcpAdapters;
    }

    public CModeIGroup createCModeIGroup(final String iGroupUUID) {
        return new CModeIGroup(this, iGroupUUID);
    }

    public CModeIGroups getIGroups() {
        return iGroups;
    }

    public CModePortSet createCModePortSet(final String vServerName, final String portSetName) {
        return new CModePortSet(this, vServerName, portSetName);
    }

    public CModePortSets getPortSets() {
        return portSets;
    }

    public CModeLUNMap createCModeLUNMap(final String lunUUID, final String iGroupUUID) {
        return new CModeLUNMap(this, lunUUID, iGroupUUID);
    }

    public CModeLUNMaps getLUNMaps() {
        return lunMaps;
    }

    public CModeAdapter createCModeAdapter(final String nodeName, final String adapterName) {
        return new CModeAdapter(this, nodeName, adapterName);
    }

    public CModeAdapters getCModeStorageAdapters() {
        return adapters;
    }

    public CModeCIFSServers getCifsServers() {
        return cifsServers;
    }

    public CModeCIFSServer createCifsServer(final String cifsServerName) {
        return new CModeCIFSServer(this, cifsServerName);
    }

    public CModeCIFSShares getCifsShares() {
        return cifsShares;
    }

    public CModeCIFSShare createCifsShare(final String cifsServerName, final String cifsShareName) {
        return new CModeCIFSShare(this, cifsServerName, cifsShareName);
    }

    public CModeCIFSACLs getCIFSShareACLs() {
        return cifsShareACLs;
    }

    public CModeCIFSACL createCIFsShareACL(final String vServerName, final String shareName, final String userOrGroup) {
        return new CModeCIFSACL(this, vServerName, shareName, userOrGroup);
    }


    public CModeCIFSOption createAndAddCIFSOption(final String vServerName, final String defaultUnixUser, final String defaultUnixGroup, final Boolean isExportPolicyEnabled) {
        final CModeCIFSOption cModeCIFSOption = new CModeCIFSOption(this, vServerName, defaultUnixUser, defaultUnixGroup, isExportPolicyEnabled);
        cifsOptions.add(cModeCIFSOption);
        return cModeCIFSOption;
    }

    public CModeCIFSOptions getCifsOptions() {
        return cifsOptions;
    }

    public CModeExportRules getExportRules() {
        return exportRules;
    }

    public CModeExportRule createExportRule(final String vserverName, final String policyName, final Integer ruleIndex) {
        return new CModeExportRule(this, vserverName, policyName, ruleIndex);
    }

    public CModeExportPolicies getExportPolicies() {
        return exportPolicies;
    }

    public CModeExportPolicy createExportPolicy(final String vServerName, final Long policyID) {
        return new CModeExportPolicy(this, vServerName, policyID);
    }

    public CModeNFSServices getNFSServices() {
        return nfsServices;
    }

    public CModeNFSService createNFSService(final String vServerName) {
        return new CModeNFSService(this, vServerName);
    }

    public CModeISCSIServices getIscsiServices() {
        return iscsiServices;
    }

    public CModeISCSIService createCModeISCSIService(final String iqn) {
        return new CModeISCSIService(this, iqn);
    }

    public CModeISCSIPortalGroups getISCSIPortalGroups() {
        return iscsiPortalGroups;
    }

    public CModeISCSIPortalGroup createISCSIPortalGroup(final String vServerName, final String portalGroupName) {
        return new CModeISCSIPortalGroup(this, vServerName, portalGroupName);
    }

    public CModeISCSIPortalGroupToLIFs getISCSIPortalGroupToLIFs() {
        return iscsiPortalGroupToLIFs;
    }

    public CModeISCSIPortalGroupToLIF createISCSIPortalGroupToLIF(final String vServerName, final String interfaceName, final String portalGroupName) {
        return new CModeISCSIPortalGroupToLIF(this, vServerName, interfaceName, portalGroupName);
    }

    public CModeISCSISession createISCSISession(final String vServerName, final String portalGroupName, final Integer targetSessionID) {
        return new CModeISCSISession(this, vServerName, portalGroupName, targetSessionID);
    }

    public CModeISCSISessions getISCSISessions() {
        return iscsiSessions;
    }

    public CModeISCSISecurityEntry createISCSISecurityEntry(final String vServerName, final String initiator) {
        return new CModeISCSISecurityEntry(this, vServerName, initiator);
    }

    public CModeISCSISecurityEntries getISCSISecurityEntries() {
        return iscsiSecurityEntries;
    }

    public CModeLicenseInterim createLicenseInterim(final EnumWrapper<CModeLicenseInterim.InterimLicensedFeature> service, final String serialNumber) {
        return new CModeLicenseInterim(this, service, serialNumber);
    }

    public CModeLicenseV2 createLicenseV2(final EnumWrapper<CModeLicenseV2.LicensedPackage> _package, final String serialNumber) {
        return new CModeLicenseV2(this, _package, serialNumber);
    }

    public CModeLicenses getLicenses() {
        return licenses;
    }

    public CModeQuota createQuota(final String vServerName, final String volumeName, final String qtreeName, final String quotaTarget, final String userHash) {
        return new CModeQuota(this, vServerName, volumeName, qtreeName, quotaTarget, userHash);
    }

    public CModeQuotas getQuotas() {
        return quotas;
    }

    public CModeSnapMirror createSnapMirror(final String sourceLocation, final String destinationLocation) {
        return new CModeSnapMirror(this, sourceLocation, destinationLocation);
    }

    public CModeSnapMirrors getSnapMirrors() {
        return snapMirrors;
    }

    public CModeSnapMirrorPolicy createSnapMirrorPolicy(final String vServerName, final String policyName) {
        return new CModeSnapMirrorPolicy(this, vServerName, policyName);
    }

    public CModeSnapMirrorPolicies getSnapMirrorPolicies() {
        return snapMirrorPolicies;
    }
    
    public CModeSnapMirrorHistory createSnapMirrorHistory(final String operationId) {
        return new CModeSnapMirrorHistory(this, operationId);
    }

    public CModeSnapMirrorHistories getSnapMirrorHistories() {
        return snapMirrorHistories;
    }

    public CModeVServerPeer createVServerPeer(final String peerClusterName, final String peerVServerName, final String vServerName) {
        return new CModeVServerPeer(this, peerClusterName, peerVServerName, vServerName);
    }

    public CModeVServerPeers getVServerPeers() {
        return vServerPeers;
    }

    public CModeVServerDNSInfo createVServerDNSInfo(final String vServerName) {
        return new CModeVServerDNSInfo(this, vServerName);
    }

    public CModeVServerDNSInfos getVServerDNSInfos() {
        return dnsInfos;
    }
    
    public CModeVServerDDNSInfo createVServerDDNSInfo(final String vServerName) {
        return new CModeVServerDDNSInfo(this, vServerName);
    }

    public CModeVServerDDNSInfos getVServerDDNSInfos() {
        return ddnsInfos;
    }

    public CModeVServerNISInfo createVServerNISInfo(final String vServerName, final String nisDomainName) {
        return new CModeVServerNISInfo(this, vServerName, nisDomainName);
    }

    public CModeVServerNISInfos getVServerNISInfos() {
        return nisInfos;
    }

    public CModeVServerLDAPInfo createVServerLDAPInfo(final String vServerName) {
        return new CModeVServerLDAPInfo(this, vServerName);
    }

    public CModeVServerLDAPInfos getVServerLDAPInfos() {
        return ldapInfos;
    }

    public CModeVServerNameMapping createVServerNameMapping(final String vServerName, final Integer position, final String direction) {
        return new CModeVServerNameMapping(this, vServerName, position, direction);
    }

    public CModeVServerNameMappings getVServerNameMappings() {
        return vServerNameMappings;
    }

    public CModeJobSchedule createJobSchedule(final String scheduleName) {
        return new CModeJobSchedule(this, scheduleName);
    }

    public CModeJobSchedules getJobSchedules() {
        return jobSchedules;
    }

    public CModeCronJobSchedule createCronJobSchedule(final String scheduleName) {
        return new CModeCronJobSchedule(this, scheduleName);
    }

    public CModeCronJobSchedules getCronJobSchedules() {
        return cronJobSchedules;
    }

    public CModeJobScheduleInterval createJobScheduleInterval(final String scheduleName) {
        return new CModeJobScheduleInterval(this, scheduleName);
    }

    public CModeJobScheduleIntervals getJobScheduleIntervals() {
        return jobScheduleIntervals;
    }

    public CModeQosVolumeWorkload createQosVolumeWorkload(final String qosWorkloadUUID) {
        return new CModeQosVolumeWorkload(this, qosWorkloadUUID);
    }

    public CModeQosVolumeWorkloads getQosVolumeWorkloads() {
        return qosVolumeWorkloads;
    }

    public CModeQosServiceCenter createQosServiceCenter(final String qosServiceCenterUUID) {
        return new CModeQosServiceCenter(this, qosServiceCenterUUID);
    }

    public CModeQosServiceCenters getQosServiceCenters() {
        return qosServiceCenters;
    }

    public CModeQosWorkloadDetail createQosWorkloadDetail(final String workloadDetailUUID) {
        return new CModeQosWorkloadDetail(this, workloadDetailUUID);
    }

    public CModeSisPolicy createSisPolicy(final String vServerName, final String policyName) {
        return new CModeSisPolicy(this, vServerName, policyName);
    }

    public CModeSisPolicies getSisPolicies() {
        return sisPolicies;
    }

    public CModeSnapshotPolicy createSnapshotPolicy(final String vServerName, final String policyName) {
        return new CModeSnapshotPolicy(this, vServerName, policyName);
    }

    public CModeSnapshotPolicies getSnapshotPolicies() {
        return snapshotPolicies;
    }

    public CModeQosPolicyGroup createQosPolicyGroup(final String qosPolicyGroupUUID) {
        return new CModeQosPolicyGroup(this, qosPolicyGroupUUID);
    }

    public CModeQosPolicyGroups getQosPolicyGroups() {
        return qosPolicyGroups;
    }

    public CModeSystemHealthAlerts getSystemHealthAlerts() {
        return systemHealthAlerts;
    }

    public CModeSystemHealthAlert createSystemHealthAlert(final String alertId, final String alertingResource)  {
        return new CModeSystemHealthAlert(this, alertId, alertingResource);
    }

    public CModeNetRoute createNetRoute(final String vServerName, final String routingGroupName, final String destinationAddress) {
        return new CModeNetRoute(this, vServerName, routingGroupName, destinationAddress);
    }

    public CModeNetRoutes getNetRoutes() {
        return netRoutes;
    }

    public CModeKerbero createKerbero(final String vServerName) {
        return new CModeKerbero(this, vServerName);
    }

    public CModeKerberos getKerberos() {
        return kerberos;
    }


    public boolean isLessThanSierraNevada() {
        return isLessThanSierraNevada;
    }

    public void setLessThanSierraNevada(final boolean lessThanSierraNevada) {
        isLessThanSierraNevada = lessThanSierraNevada;
    }

    public EnumWrapper<Cluster.HmStatus> getDiagnosisStatus() {
        return diagnosisStatus;
    }

    public void setDiagnosisStatus(final EnumWrapper<Cluster.HmStatus> diagnosisStatus) {
        this.diagnosisStatus = diagnosisStatus;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isSpareLow() {
        return isSpareLow;
    }

    public void setSpareLow(final Boolean spareLow) {
        isSpareLow = spareLow;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    public EnumWrapper<Cluster.MtConfigurationState> getMtConfigurationState() {
        return mtConfigurationState;
    }

    public void setMtConfigurationState(EnumWrapper<Cluster.MtConfigurationState> mtConfigurationState) {
        this.mtConfigurationState = mtConfigurationState;
    }

    public String getMtConfigurationStateRaw() {
        return mtConfigurationStateRaw;
    }

    public void setMtConfigurationStateRaw(String mtConfigurationStateRaw) {
        this.mtConfigurationStateRaw = mtConfigurationStateRaw;
    }

    public EnumWrapper<Cluster.MtConfigurationState> getMtRemoteConfigurationState() {
        return mtRemoteConfigurationState;
    }

    public void setMtRemoteConfigurationState(EnumWrapper<Cluster.MtConfigurationState> mtRemoteConfigurationState) {
        this.mtRemoteConfigurationState = mtRemoteConfigurationState;
    }

    public String getMtRemoteConfigurationStateRaw() {
        return mtRemoteConfigurationStateRaw;
    }

    public void setMtRemoteConfigurationStateRaw(String mtRemoteConfigurationStateRaw) {
        this.mtRemoteConfigurationStateRaw = mtRemoteConfigurationStateRaw;
    }

    public EnumWrapper<Cluster.MtMode> getMtMode() {
        return mtMode;
    }

    public void setMtMode(EnumWrapper<Cluster.MtMode> mtMode) {
        this.mtMode = mtMode;
    }

    public String getMtModeRaw() {
        return mtModeRaw;
    }

    public void setMtModeRaw(String mtModeRaw) {
        this.mtModeRaw = mtModeRaw;
    }

    public EnumWrapper<Cluster.MtMode> getMtRemoteMode() {
        return mtRemoteMode;
    }

    public void setMtRemoteMode(EnumWrapper<Cluster.MtMode> mtRemoteMode) {
        this.mtRemoteMode = mtRemoteMode;
    }

    public String getMtRemoteModeRaw() {
        return mtRemoteModeRaw;
    }

    public void setMtRemoteModeRaw(String mtRemoteModeRaw) {
        this.mtRemoteModeRaw = mtRemoteModeRaw;
    }

    public String getMtConfigurationType() {
        return mtConfigurationType;
    }

    public void setMtConfigurationType(String mtConfigurationType) {
        this.mtConfigurationType = mtConfigurationType;
    }

    public boolean getIsMetrocluster() {
        return isMetrocluster;
    }
    
    public void setIsMetrocluster(final boolean isMetrocluster) {
        this.isMetrocluster = isMetrocluster;
    }

    public Boolean getSslFipsEnabled () {
        return sslFipsEnabled ;
    }

    public void setSslFipsEnabled(final Boolean sslFipsEnabled) {
        this.sslFipsEnabled = sslFipsEnabled;
    }
    
    public String getMtDrPartnerClusterID() {
        return mtDrPartnerClusterID;
    }

    public void setMtDrPartnerClusterID(String mtDrPartnerClusterID) {
        this.mtDrPartnerClusterID = mtDrPartnerClusterID;
    }

    public CModeStorageService createStorageService(final String vServerName, final String volumeName, final Long storageServiceID) {
        return new CModeStorageService(this, vServerName, volumeName, storageServiceID);
    }

    public CModeStorageServices getStorageServices() {
        return storageServices;
    }

    public CModeApplicationRecord createAndAddApplicationRecord(final String recordName, final String recordValue, final String modifiedFrom, final Long lastModifiedTimestamp) {
        final CModeApplicationRecord record = new CModeApplicationRecord(this, recordName, recordValue, modifiedFrom, lastModifiedTimestamp);
        applicationRecords.add(record);
        return record;
    }

    public CModeApplicationRecords getApplicationRecords() {
        return applicationRecords;
    }
    public Map<String, String> getCifsNodeUUIDNameMap() {
        return cifsNodeUUIDNameMap;
    }

    public Map<String, String> getSmb1NodeUUIDNameMap() {
        return smb1NodeUUIDNameMap;
    }

    public Map<String, String> getSmb2NodeUUIDNameMap() {
        return smb2NodeUUIDNameMap;
    }

    public Map<String, String> getNfsv3NodeUUIDNameMap() {
        return nfsv3NodeUUIDNameMap;
    }

    public Map<String, String> getNfsv4NodeUUIDNameMap() {
        return nfsv4NodeUUIDNameMap;
    }

    public Map<String, String> getNfsv4_1NodeUUIDNameMap() {
        return nfsv4_1NodeUUIDNameMap;
    }

    public Map<String, String> getCifsVserverIDNameMap() {
        return cifsVserverIDNameMap;
    }
 
    public Map<String, String> getNfsv3VserverIDNameMap() {
        return nfsv3VserverIDNameMap;
    }

    public Map<String, String> getLifVserverIDNameMap() {
        return lifVserverIDNameMap;
    }
    
    public Map<String, String> getNfsv4VserverIDNameMap() {
        return nfsv4VserverIDNameMap;
    }

    public Map<String, String> getNfsv41VserverIDNameMap() {
        return nfsv41VserverIDNameMap;
    }

    public CModeVServerCIFSSecurityInfo createVServerCIFSSecurityInfo(final String vServerName) {
        return new CModeVServerCIFSSecurityInfo(this, vServerName);
    }

    public CModeVServerCIFSSecurityInfos getVServerCIFSSecInfos() {
        return cifsSecInfos;
    }

    public CModeClusterPeer createClusterPeer(final String peerClusterUUID) {
        return new CModeClusterPeer(this, peerClusterUUID);
    }

    public CModeClusterPeers getClusterPeers() {
        return clusterPeers;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", getKey().getUUID());
        addAttributeNoCRC(writer, "Name", name);
        addAttributeNoCRC(writer, "SerialNumber", serialNum);
        addAttributeNoCRC(writer, "ManagementIP", managementIP);
        addAttributeNoCRC(writer, "Version", versionString);
        addAttributeNoCRC(writer, "VersionGeneration", generation);
        addAttributeNoCRC(writer, "VersionMajor", major);
        addAttributeNoCRC(writer, "VersionMinor", minor);
        addAttributeNoCRC(writer, "Contact", contact);
        addAttributeNoCRC(writer, "Location", location);
        addAttributeNoCRC(writer, "DiagnosisStatus", getDiagnosisStatus().getEnumValue());
        addAttributeNoCRC(writer, "DiagnosisStatusRaw", getDiagnosisStatus().getRawValue());
        addAttributeNoCRC(writer, "Timezone", timezone);
        addAttributeNoCRC(writer, "TimezoneUTC", timezoneUTC);
        addAttributeNoCRC(writer, "TimezoneVersion", timezoneVersion);
        addAttributeNoCRC(writer, "CPUColumns", BuilderUtil.getListAsCSVString(cpuColumns));
        addAttributeNoCRC(writer, "WaflCpPhaseTimesColumns", BuilderUtil.getListAsCSVString(waflCpPhaseTimesColumns));;
        addAttributeNoCRC(writer, "HaConfigured", haConfigured);
        addAttributeNoCRC(writer, "IsMetrocluster", isMetrocluster);
        addAttributeNoCRC(writer, "SslFipsEnabled", sslFipsEnabled);
        addAttributeNoCRC(writer, "MtConfigurationState", getMtConfigurationState().getEnumValue());
        addAttributeNoCRC(writer, "MtConfigurationStateRaw", getMtConfigurationState().getRawValue());
        addAttributeNoCRC(writer, "MtMode", getMtMode().getEnumValue());
        addAttributeNoCRC(writer, "MtModeRaw", getMtMode().getRawValue());
        addAttributeNoCRC(writer, "MtDrPartnerClusterID", mtDrPartnerClusterID);
        addAttributeNoCRC(writer, "MtRemoteConfigurationState", getMtRemoteConfigurationState().getEnumValue());
        addAttributeNoCRC(writer, "MtRemoteConfigurationStateRaw", getMtRemoteConfigurationState().getRawValue());
        addAttributeNoCRC(writer, "MtRemoteMode", getMtRemoteMode().getEnumValue());
        addAttributeNoCRC(writer, "MtRemoteModeRaw", getMtRemoteMode().getRawValue());
        addAttributeNoCRC(writer, "MtConfigurationType", mtConfigurationType);
    }

    @Override
    protected boolean hasChildren() {
        // we're talking to something
        return true;
    }

    @SuppressWarnings("unchecked")
    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        super.marshallChildrenToFileAndCRC(cache, writer, tabs);
        // marshall Nodes Version from system-get-version
        final String xmlNodeName = "CModeNodeVersions";
        addTabulation(writer, tabs);
        writer.append("<").append(xmlNodeName).append(">").append(lineSeperator);
        for (final Map.Entry<String, String> nodeVersionEntry : nodeVersions.entrySet()) {
            final String nodeUUID = nodeVersionEntry.getKey();
            final String nodeVersion = nodeVersionEntry.getValue();
            addTabulation(writer, tabs + 1);
            writer.append("<CModeNodeVersion UUID=\"");
            writer.append(nodeUUID);
            writer.append("\" Version=\"");
            writer.append(nodeVersion);
            writer.append("\" />").append(lineSeperator);
        }
        addTabulation(writer, tabs);
        writer.append("</").append(xmlNodeName).append(">").append(lineSeperator);

        // children martial themselves
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeNodes.class.getSimpleName(), nodes.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeNodeDetails.class.getSimpleName(), nodeDetails.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeNodeFailovers.class.getSimpleName(), nodeFailovers.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeAdapterStatuses.class.getSimpleName(), adapterStatuses.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeAdapters.class.getSimpleName(), adapters.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeNetInterfaces.class.getSimpleName(), ports.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeNetworkIpSpaces.class.getSimpleName(), networkIpSpaces.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeNetworkSubnets.class.getSimpleName(), networkSubnets.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeNetworkPortBroadcastDomains.class.getSimpleName(), networkPortBroadcastDomains.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeNetworkFailoverGroups.class.getSimpleName(), networkFailoverGroups.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeLifs.class.getSimpleName(), lifs.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeAggregates.class.getSimpleName(), aggregates.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeAggregateSpares.class.getSimpleName(), aggregateSpares.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeMetroClusterOperations.class.getSimpleName(), metroClusterOperations.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeStorageSwitch.class.getSimpleName(), storageSwitches.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeStoragePools.class.getSimpleName(), storagePools.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeStoragePoolAvailableSizes.class.getSimpleName(), storagePoolAvailableSizes.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeStoragePoolAggregates.class.getSimpleName(), storagePoolAggregates.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeDisks.class.getSimpleName(), disks.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeDiskAggregateRelationships.class.getSimpleName(), diskAggregateRelationships.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeVServers.class.getSimpleName(), vServers.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeVServerNameMappings.class.getSimpleName(), vServerNameMappings.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeVolumes.class.getSimpleName(), volumes.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeVolumeClones.class.getSimpleName(), volumeClones.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeVolumeMoves.class.getSimpleName(), volumeMoves.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeVolumeFlexCaches.class.getSimpleName(), volumeFlexCaches.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeVolumeFootprints.class.getSimpleName(), volumeFootprints.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeQuotaStatuses.class.getSimpleName(), quotaStatuses.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeSnapshots.class.getSimpleName(), snapshots.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeSisStatuses.class.getSimpleName(), sisStatuses.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeQTrees.class.getSimpleName(), qtrees.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeQuotas.class.getSimpleName(), quotas.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeLUNs.class.getSimpleName(), luns.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeFcpServiceInfos.class.getSimpleName(), fcpServiceInfos.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeFcpAdapters.class.getSimpleName(), fcpAdapters.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeFcpLifs.class.getSimpleName(), fcpLifs.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeIGroups.class.getSimpleName(), iGroups.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModePortSets.class.getSimpleName(), portSets.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeLUNMaps.class.getSimpleName(), lunMaps.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeExportPolicies.class.getSimpleName(), exportPolicies.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeExportRules.class.getSimpleName(), exportRules.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeNFSServices.class.getSimpleName(), nfsServices.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeISCSIServices.class.getSimpleName(), iscsiServices.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeISCSIPortalGroups.class.getSimpleName(), iscsiPortalGroups.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeISCSIPortalGroupToLIFs.class.getSimpleName(), iscsiPortalGroupToLIFs.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeISCSISessions.class.getSimpleName(), iscsiSessions.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeISCSISecurityEntries.class.getSimpleName(), iscsiSecurityEntries.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeLicenses.class.getSimpleName(), licenses.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeSnapMirrors.class.getSimpleName(), snapMirrors.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeSnapMirrorPolicies.class.getSimpleName(), snapMirrorPolicies.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeSnapMirrorHistories.class.getSimpleName(), snapMirrorHistories.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeVServerPeers.class.getSimpleName(), vServerPeers.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeVServerDNSInfos.class.getSimpleName(), dnsInfos.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeVServerNISInfos.class.getSimpleName(), nisInfos.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeVServerLDAPInfos.class.getSimpleName(), ldapInfos.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeQosVolumeWorkloads.class.getSimpleName(), qosVolumeWorkloads.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeQosServiceCenters.class.getSimpleName(), qosServiceCenters.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeJobSchedules.class.getSimpleName(), jobSchedules.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeCronJobSchedules.class.getSimpleName(), cronJobSchedules.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeJobScheduleIntervals.class.getSimpleName(), jobScheduleIntervals.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeSisPolicies.class.getSimpleName(), sisPolicies.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeSnapshotPolicies.class.getSimpleName(), snapshotPolicies.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeCIFSServers.class.getSimpleName(), cifsServers.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeCIFSShares.class.getSimpleName(), cifsShareACLs.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeQosPolicyGroups.class.getSimpleName(), qosPolicyGroups.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeNetRoutes.class.getSimpleName(), netRoutes.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeKerberos.class.getSimpleName(), kerberos.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeStorageServices.class.getSimpleName(), storageServices.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeNodeMetroclusters.class.getSimpleName(), nodeMetroclusters.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeLunImports.class.getSimpleName(), lunImports.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeNetworkFailoverGroupNetworkPortRelationships.class.getSimpleName(), networkFailoverGroupNetworkPortRelationships.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeClusterPeers.class.getSimpleName(), clusterPeers.values(), tabs);
    }


}
