package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Disk;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;

public class CModeDisk extends BaseCRCXMLNode<CModeCluster> {

    private final UUIDKey diskKey;

    private String diskName;
    private String shelf;
    private String bay;
    private String vendor;
    private String model;
    private String serialNum;
    private Long rpm;
    private String firmwareRev;
    private Long bytesPerSector;
    private Long capacitySectors;
    private Long rightSizeSectors;
    private String homeNodeID;
    private String homeNodeName;
    private Boolean isFailed;
    private String ownerNodeID;
    private String ownerNodeName;
    private Long poolID;
    private String reservedByNodeIDString;
    private String activeNodeName;
    private EnumWrapper<Disk.DiskType> diskType;
    private EnumWrapper<Disk.DiskContainerType> containerType;
    private EnumWrapper<Disk.DiskInterfaceType> effectiveDiskType;
    private Long physicalBlocks;
    private Long usedBlocks;
    private EnumWrapper<Disk.RaidPosition> position;
    private String sparePool;
    private String aggregateName;
    private String storagePoolName;
    private String checkSumType;
    private String plexName;
    private String raidGroupName;
    private Boolean isMediaScrubbing;
    private Boolean isZeroed;
    private Boolean isZeroing;
    private Boolean isVirtual;
    private String checksumCompatibility;
    private Boolean isOffline;
    private Boolean isPreFailed;
    private Boolean isReconstructing;
    private Boolean isReplacing;
    private Boolean isShared;
    private Integer zeroingPercentComplete;
    private Boolean isInFDR;
    private String failedReason;
    private Long usableDataSize;
    private Long usableRootSize;
    private Boolean isLocalAttach;
    private Integer percentSparesConsumed;
    private String drHomeNodeID;
    private String drHomeNodeName;
    private Integer percentRatedLifeUsed;

    private EnumWrapper<Disk.PartitioningType> partitioningType;


    private final CModeDiskPaths diskPaths;

    CModeDisk(final CModeCluster parent, final String diskUUID) {
        super(parent, CModeDisk.class.getSimpleName());
        this.diskKey = new UUIDKey("disk", diskUUID);
        this.diskPaths = new CModeDiskPaths(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return diskKey;
    }

    public String getDiskName() {
        return diskName;
    }

    public String getShelf() {
        return shelf;
    }

    public String getBay() {
        return bay;
    }

    public String getVendor() {
        return vendor;
    }

    public String getModel() {
        return model;
    }

    public String getSerialNum() {
        return serialNum;
    }

    public Long getRPM() {
        return rpm;
    }

    public String getFirmwareRev() {
        return firmwareRev;
    }

    public String getHomeNodeName() {
        return homeNodeName;
    }

    public Boolean isFailed() {
        return isFailed;
    }

    public EnumWrapper<Disk.DiskContainerType> getContainerType() {
        return containerType;
    }

    public Long getPhysicalBlocks() {
        return physicalBlocks;
    }

    public Long getUsedBlocks() {
        return usedBlocks;
    }

    public EnumWrapper<Disk.RaidPosition> getPosition() {
        return position;
    }

    public String getAggregateName() {
        return aggregateName;
    }

    public String getPlexName() {
        return plexName;
    }

    public String getRaidGroupName() {
        return raidGroupName;
    }

    public EnumWrapper<Disk.DiskInterfaceType> getEffectiveDiskType() {
        return effectiveDiskType;
    }

    public Long getBytesPerSector() {
        return bytesPerSector;
    }

    public Boolean isVirtual() {
        return isVirtual;
    }

    public CModeDiskPath createCModeDiskPath(final String diskUUID, final String initiatorPortName, final String targetWWPN) {
        return new CModeDiskPath(this, diskUUID, initiatorPortName, targetWWPN);
    }

    public CModeDiskPaths getDiskPaths() {
        return diskPaths;
    }

    public void setDiskName(final String diskName) {
        this.diskName = diskName;
    }

    public void setShelf(final String shelf) {
        this.shelf = shelf;
    }

    public void setBay(final String bay) {
        this.bay = bay;
    }

    public void setVendor(final String vendor) {
        this.vendor = vendor;
    }

    public void setModel(final String model) {
        this.model = model;
    }

    public void setSerialNum(final String serialNum) {
        this.serialNum = serialNum;
    }

    public void setRpm(final Long rpm) {
        this.rpm = rpm;
    }

    public void setFirmwareRev(final String firmwareRev) {
        this.firmwareRev = firmwareRev;
    }

    public void setBytesPerSector(final Long bytesPerSector) {
        this.bytesPerSector = bytesPerSector;
    }

    public void setCapacitySectors(final Long capacitySectors) {
        this.capacitySectors = capacitySectors;
    }

    public Long getCapacitySectors() {
        return capacitySectors;
    }

    public Long getRightSizeSectors() {
        return rightSizeSectors;
    }

    public void setRightSizeSectors(final Long rightSizeSectors) {
        this.rightSizeSectors = rightSizeSectors;
    }

    public void setHomeNodeID(final String homeNodeID) {
        this.homeNodeID = homeNodeID;
    }

    public void setHomeNodeName(final String homeNodeName) {
        this.homeNodeName = homeNodeName;
    }

    public void setFailed(final Boolean failed) {
        isFailed = failed;
    }

    public void setOwnerNodeID(final String ownerNodeID) {
        this.ownerNodeID = ownerNodeID;
    }

    public void setOwnerNodeName(final String ownerNodeName) {
        this.ownerNodeName = ownerNodeName;
    }

    public void setPoolID(final Long poolID) {
        this.poolID = poolID;
    }

    public void setReservedByNodeIDString(final String reservedByNodeIDString) {
        this.reservedByNodeIDString = reservedByNodeIDString;
    }

    public void setActiveNodeName(final String activeNodeName) {
        this.activeNodeName = activeNodeName;
    }

    public String getActiveNodeName() {
        return activeNodeName;
    }

    public void setContainerType(final EnumWrapper<Disk.DiskContainerType> containerType) {
        this.containerType = containerType;
    }

    public void setEffectiveDiskType(final EnumWrapper<Disk.DiskInterfaceType> effectiveDiskType) {
        this.effectiveDiskType = effectiveDiskType;
    }

    public void setPhysicalBlocks(final Long physicalBlocks) {
        this.physicalBlocks = physicalBlocks;
    }

    public void setUsedBlocks(final Long usedBlocks) {
        this.usedBlocks = usedBlocks;
    }

    public void setPosition(final EnumWrapper<Disk.RaidPosition> position) {
        this.position = position;
    }

    public void setSparePool(final String sparePool) {
        this.sparePool = sparePool;
    }

    public void setAggregateName(final String aggregateName) {
        this.aggregateName = aggregateName;
    }

    public void setCheckSumType(final String checkSumType) {
        this.checkSumType = checkSumType;
    }

    public void setPlexName(final String plexName) {
        this.plexName = plexName;
    }

    public void setRaidGroupName(final String raidGroupName) {
        this.raidGroupName = raidGroupName;
    }

    public void setMediaScrubbing(final Boolean mediaScrubbing) {
        isMediaScrubbing = mediaScrubbing;
    }

    public void setZeroed(final Boolean zeroed) {
        isZeroed = zeroed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getHomeNodeID() {
        return homeNodeID;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getOwnerNodeID() {
        return ownerNodeID;
    }

    public String getOwnerNodeName() {
        return ownerNodeName;
    }

    public Long getPoolID() {
        return poolID;
    }

    public Boolean isMediaScrubbing() {
        return isMediaScrubbing;
    }

    public Boolean isZeroed() {
        return isZeroed;
    }

    public void setVirtual(final Boolean virtual) {
        isVirtual = virtual;
    }

    public String getChecksumCompatibility() {
        return checksumCompatibility;
    }

    public void setChecksumCompatibility(final String checksumCompatibility) {
        this.checksumCompatibility = checksumCompatibility;
    }

    public Boolean isOffline() {
        return isOffline;
    }

    public void setOffline(final Boolean offline) {
        isOffline = offline;
    }

    public Boolean isPreFailed() {
        return isPreFailed;
    }

    public void setPreFailed(final Boolean preFailed) {
        isPreFailed = preFailed;
    }

    public Boolean isReconstructing() {
        return isReconstructing;
    }

    public void setReconstructing(final Boolean reconstructing) {
        isReconstructing = reconstructing;
    }

    public Boolean isReplacing() {
        return isReplacing;
    }

    public void setReplacing(final Boolean replacing) {
        isReplacing = replacing;
    }

    public Boolean isZeroing() {
        return isZeroing;
    }

    public void setZeroing(final Boolean zeroing) {
        isZeroing = zeroing;
    }

    public Integer getZeroingPercentComplete() {
        return zeroingPercentComplete;
    }

    public void setZeroingPercentComplete(final Integer zeroingPercentComplete) {
        this.zeroingPercentComplete = zeroingPercentComplete;
    }

    public Boolean isInFDR() {
        return isInFDR;
    }

    public void setInFDR(final Boolean inFDR) {
        isInFDR = inFDR;
    }

    public String getFailedReason() {
        return failedReason;
    }

    public void setFailedReason(final String failedReason) {
        this.failedReason = failedReason;
    }

    public EnumWrapper<Disk.DiskType> getDiskType() {
        return diskType;
    }

    public void setDiskType(final EnumWrapper<Disk.DiskType> diskType) {
        this.diskType = diskType;
    }

    public Boolean getShared() {
        return isShared;
    }

    public void setShared(Boolean shared) {
        isShared = shared;
    }

    public String getStoragePoolName() {
        return storagePoolName;
    }

    public void setStoragePoolName(String storagePoolName) {
        this.storagePoolName = storagePoolName;
    }

    public Long getUsableDataSize() {
		return usableDataSize;
	}

	public void setUsableDataSize(Long usableDataSize) {
		this.usableDataSize = usableDataSize;
	}

	public Long getUsableRootSize() {
		return usableRootSize;
	}

	public void setUsableRootSize(Long usableRootSize) {
		this.usableRootSize = usableRootSize;
	}

    public Boolean getIsLocalAttach() {
        return isLocalAttach;
    }

    public void setIsLocalAttach(Boolean isLocalAttach) {
        this.isLocalAttach = isLocalAttach;
    }

    public Integer getPercentSparesConsumed() {
        return percentSparesConsumed;
    }

    public void setPercentSparesConsumed(Integer percentSparesConsumed) {
        this.percentSparesConsumed = percentSparesConsumed;
    }

    public Integer getPercentRatedLifeUsed() { return percentRatedLifeUsed; }

    public void setPercentRatedLifeUsed(Integer percentRatedLifeUsed) {
        this.percentRatedLifeUsed = percentRatedLifeUsed;
    }



    public EnumWrapper<Disk.PartitioningType> getPartitioningType() {
        return partitioningType;
    }

    public void setPartitioningType(EnumWrapper<Disk.PartitioningType> partitioningType) {
        this.partitioningType = partitioningType;
    }


    public String getDrHomeNodeID() {
        return drHomeNodeID;
    }

    public void setDrHomeNodeID(String drHomeNodeID) {
        this.drHomeNodeID = drHomeNodeID;
    }

    public String getDrHomeNodeName() {
        return drHomeNodeName;
    }

    public void setDrHomeNodeName(String drHomeNodeName) {
        this.drHomeNodeName = drHomeNodeName;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "Name", diskName);
        addAttributeNoCRC(writer, "UUID", diskKey.getUUID());
        addAttributeNoCRC(writer, "Shelf", shelf);
        addAttributeNoCRC(writer, "Bay", bay);
        addAttributeNoCRC(writer, "Vendor", vendor);
        addAttributeNoCRC(writer, "Model", model);
        addAttributeNoCRC(writer, "SerialNum", serialNum);
        addAttributeNoCRC(writer, "Type", diskType);
        addAttributeNoCRC(writer, "RPM", rpm);
        addAttributeNoCRC(writer, "FirmwareRev", firmwareRev);
        addAttributeNoCRC(writer, "BytesPerSector", bytesPerSector);
        addAttributeNoCRC(writer, "CapacitySectors", capacitySectors);
        addAttributeNoCRC(writer, "RightSizeSectors", rightSizeSectors);
        addAttributeNoCRC(writer, "HomeNodeID", homeNodeID);
        addAttributeNoCRC(writer, "HomeNodeName", homeNodeName);
        addAttributeNoCRC(writer, "IsFailed", isFailed);
        addAttributeNoCRC(writer, "OwnerNodeID", ownerNodeID);
        addAttributeNoCRC(writer, "OwnerNodeName", ownerNodeName);
        addAttributeNoCRC(writer, "PoolID", poolID);
        addAttributeNoCRC(writer, "ReservedByNodeID", reservedByNodeIDString);
        addAttributeNoCRC(writer, "ActiveNodeName", activeNodeName);
        addAttributeNoCRC(writer, "ContainerType", EnumWrapper.toStringOrNull(containerType));
        addAttributeNoCRC(writer, "EffectiveDiskType", EnumWrapper.toStringOrNull(effectiveDiskType));
        addAttributeNoCRC(writer, "PhysicalBlocks", physicalBlocks);
        addAttributeNoCRC(writer, "UsedBlocks", usedBlocks);
        addAttributeNoCRC(writer, "Position", EnumWrapper.toStringOrNull(position));
        addAttributeNoCRC(writer, "SharePool", sparePool);
        addAttributeNoCRC(writer, "AggregateName", aggregateName);
        addAttributeNoCRC(writer, "ChecksumCompatibility", checksumCompatibility);
        addAttributeNoCRC(writer, "CheckSumType", checkSumType);
        addAttributeNoCRC(writer, "PlexName", plexName);
        addAttributeNoCRC(writer, "RaidGroupName", raidGroupName);
        addAttributeNoCRC(writer, "StoragePoolName", storagePoolName);
        addAttributeNoCRC(writer, "IsMediaScrubbing", isMediaScrubbing);
        addAttributeNoCRC(writer, "IsOffline", isOffline);
        addAttributeNoCRC(writer, "IsPreFailed", isPreFailed);
        addAttributeNoCRC(writer, "IsReconstructing", isReconstructing);
        addAttributeNoCRC(writer, "IsReplacing", isReplacing);
        addAttributeNoCRC(writer, "IsShared", isShared);
        addAttributeNoCRC(writer, "IsZeroed", isZeroed);
        addAttributeNoCRC(writer, "IsZeroing", isZeroing);
        addAttributeNoCRC(writer, "IsVirtual", isVirtual);
        addAttributeNoCRC(writer, "ZeroingPercentComplete", zeroingPercentComplete);
        addAttributeNoCRC(writer, "IsInFDR", isInFDR);
        addAttributeNoCRC(writer, "FailedReason", failedReason);
        addAttributeNoCRC(writer, "UsableDataSize", usableDataSize);
        addAttributeNoCRC(writer, "UsableRootSize", usableRootSize);
        addAttributeNoCRC(writer, "IsLocalAttach", isLocalAttach);
        addAttributeNoCRC(writer, "PercentSparesConsumed", percentSparesConsumed);
        addAttributeNoCRC(writer, "PercentRatedLifeUsed", percentRatedLifeUsed);
        addAttributeNoCRC(writer, "PartitioningType", partitioningType.getEnumValue());
        addAttributeNoCRC(writer, "PartitioningTypeRaw", partitioningType.getRawValue());
        addAttributeNoCRC(writer, "DrHomeNodeId", drHomeNodeID);
        addAttributeNoCRC(writer, "DrHomeNodeName", drHomeNodeName);
    }
}
