package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeDiskAggregateRelationship extends BaseCRCXMLNode<CModeCluster> {

    private final DiskAggregateRelationshipKey diskAggregateRelationshipKey;
    private final String diskName;
    private final String aggregateUUID;

    private Boolean isCache;

    protected CModeDiskAggregateRelationship(final CModeCluster parent, final String diskName, final String aggregateUUID) {
        super(parent, CModeDiskAggregateRelationship.class.getSimpleName());
        this.aggregateUUID = aggregateUUID;
        this.diskName = diskName;
        this.diskAggregateRelationshipKey = new DiskAggregateRelationshipKey(aggregateUUID, diskName);
    }


    public static final class DiskAggregateRelationshipKey extends ModelKeyBase<DiskAggregateRelationshipKey.Field> {
        public static enum Field {
            disk_name,
            aggregate_uuid
        }

        public DiskAggregateRelationshipKey(final String diskName, final String aggregateUUID) {
            super("disk_aggregate_relationship", new EnumMap<Field, String>(Field.class) {{
                put(Field.disk_name, diskName);
                put(Field.aggregate_uuid, aggregateUUID);
            }});
        }
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public DiskAggregateRelationshipKey getKey() {
        return diskAggregateRelationshipKey;
    }

    public String getDiskName() {
        return diskName;
    }

    public String getAggregateUUID() {
        return aggregateUUID;
    }

    public Boolean getIsCache() {
        return isCache;
    }

    public void setIsCache(Boolean cache) {
        isCache = cache;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(cache, writer);
        addAttributeNoCRC(writer, "DiskName", diskName);
        addAttributeNoCRC(writer, "AggregateUUID", aggregateUUID);
        addAttributeNoCRC(writer, "IsCache", isCache);
    }
}