package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeExportPolicy extends BaseCRCXMLNode<CModeCluster> {

    private final CModeExportPolicyKey key;
    private final Long policyID;
    private String policyName;


    CModeExportPolicy(final CModeCluster parent, final String vServerName, final Long policyID) {
        super(parent, CModeExportPolicy.class.getSimpleName());
        this.key = new CModeExportPolicyKey(vServerName, String.valueOf(policyID));
        this.policyID = policyID;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public CModeExportPolicyKey getKey() {
        return key;
    }

    public String getVServerName() {
        return key.getID(CModeExportPolicyKey.Field.vserver_name);
    }

    public String getName() {
        return policyName;
    }

    public Long getPolicyID() {
        return policyID;
    }

    public void setPolicyName(final String policyName) {
        this.policyName = policyName;
    }

    public static final class CModeExportPolicyKey extends ModelKeyBase<CModeExportPolicyKey.Field> {
        public static enum Field {
            vserver_name,
            policy_id
        }

        public CModeExportPolicyKey(final String vServerName, final String policyID) {
            super("export_policy", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.policy_id, policyID);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "VServerName", key.getID(CModeExportPolicyKey.Field.vserver_name));
        addAttributeNoCRC(writer, "PolicyID", key.getID(CModeExportPolicyKey.Field.policy_id));
        addAttributeNoCRC(writer, "PolicyName", policyName);
    }
}
