package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.FcpPort;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeFcpAdapter extends BaseCRCXMLNode<CModeCluster> {

    private final FCPKey key;
    private Integer maxSpeed;
    private String nodeWWN;
    private String portWWN;
    private EnumWrapper<FcpPort.PhysicalProtocol> physicalProtocol;
    private EnumWrapper<FcpPort.Speed> speed;
    private EnumWrapper<FcpPort.State> state;
    private String switchPort;
    private Integer dataLinkRate;
    private Boolean fabricEstablished;
    private String infoName;
    private String firmwareRev;

    private EnumWrapper<FcpPort.MediaType> mediaType;

    CModeFcpAdapter(final CModeCluster parent, final String nodeName, final String adapterName) {
        super(parent, CModeFcpAdapter.class.getSimpleName());
        this.key = new FCPKey(nodeName, adapterName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public FCPKey getKey() {
        return key;
    }

    public Integer getMaxSpeed() {
        return maxSpeed;
    }

    public String getNodeWWN() {
        return nodeWWN;
    }

    public String getPortWWN() {
        return portWWN;
    }

    public EnumWrapper<FcpPort.PhysicalProtocol> getPhysicalProtocol() {
        return physicalProtocol;
    }

    public EnumWrapper<FcpPort.Speed> getSpeed() {
        return speed;
    }

    public EnumWrapper<FcpPort.State> getState() {
        return state;
    }

    public String getSwitchPort() {
        return switchPort;
    }

    public Integer getDataLinkRate() {
        return dataLinkRate;
    }

    public EnumWrapper<FcpPort.MediaType> getMediaType() {
        return mediaType;
    }

    public void setMaxSpeed(final Integer maxSpeed) {
        this.maxSpeed = maxSpeed;
    }

    public void setNodeWWN(final String nodeWWN) {
        this.nodeWWN = nodeWWN;
    }

    public void setPortWWN(final String portWWN) {
        this.portWWN = portWWN;
    }

    public void setPhysicalProtocol(final EnumWrapper<FcpPort.PhysicalProtocol> physicalProtocol) {
        this.physicalProtocol = physicalProtocol;
    }

    public void setSpeed(final EnumWrapper<FcpPort.Speed> speed) {
        this.speed = speed;
    }

    public void setState(final EnumWrapper<FcpPort.State> state) {
        this.state = state;
    }

    public void setSwitchPort(final String switchPort) {
        this.switchPort = switchPort;
    }

    public void setDataLinkRate(final Integer dataLinkRate) {
        this.dataLinkRate = dataLinkRate;
    }

    public void setMediaType(final EnumWrapper<FcpPort.MediaType> mediaType) {
        this.mediaType = mediaType;
    }

    public Boolean getFabricEstablished() {
        return fabricEstablished;
    }

    public void setFabricEstablished(Boolean fabricEstablished) {
        this.fabricEstablished = fabricEstablished;
    }

    public String getInfoName() {
        return infoName;
    }

    public void setInfoName(String infoName) {
        this.infoName = infoName;
    }

    public String getFirmwareRev() {
        return firmwareRev;
    }

    public void setFirmwareRev(String firmwareRev) {
        this.firmwareRev = firmwareRev;
    }

    public final static class FCPKey extends ModelKeyBase<FCPKey.Field> {
        public static enum Field {
            node_name,
            adapter_name
        }

        public FCPKey(final String nodeName, final String adapterName) {
            super("fcp_adapter", new EnumMap<Field, String>(Field.class) {{
                put(Field.node_name, nodeName);
                put(Field.adapter_name, adapterName);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "AdapterName", key.getID(FCPKey.Field.adapter_name));
        addAttributeNoCRC(writer, "InfoName", infoName);
        addAttributeNoCRC(writer, "FabricEstablished", fabricEstablished);
        addAttributeNoCRC(writer, "FirmwareRev", firmwareRev);
        addAttributeNoCRC(writer, "Node", key.getID(FCPKey.Field.node_name));
        addAttributeNoCRC(writer, "NodeName", nodeWWN);
        addAttributeNoCRC(writer, "PortName", portWWN);
        addAttributeNoCRC(writer, "State", state.getEnumValue());
        addAttributeNoCRC(writer, "StateRaw", state.getRawValue());
        addAttributeNoCRC(writer, "Speed", speed.getEnumValue());
        addAttributeNoCRC(writer, "SpeedRaw", speed.getRawValue());
        addAttributeNoCRC(writer, "MaxSpeed", maxSpeed);
        addAttributeNoCRC(writer, "PhysicalProtocol", physicalProtocol.getEnumValue());
        addAttributeNoCRC(writer, "PhysicalProtocolRaw", physicalProtocol.getRawValue());
        addAttributeNoCRC(writer, "SwitchPort", switchPort);
        addAttributeNoCRC(writer, "DataLinkRate", dataLinkRate);
        addAttributeNoCRC(writer, "MediaType", mediaType.getEnumValue());
        addAttributeNoCRC(writer, "MediaTypeRaw", mediaType.getRawValue());
    }
}
