package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeISCSISession extends BaseCRCXMLNode<CModeCluster> {

    private final CModeISCSISessionKey key;
    private final CModeISCSISessionConnections cModeISCSISessionConnections;
    private final int targetSessionID;
    private Integer maxConnections;
    private String initiatorNodeName;
    private String initiatorAliasName;
    private String initiatorSessionID;

    CModeISCSISession(final CModeCluster cluster, final String vServerName, final String portalGroupName, final Integer targetSessionID) {
        super(cluster, CModeISCSISession.class.getSimpleName());
        this.key = new CModeISCSISessionKey(vServerName, portalGroupName, String.valueOf(targetSessionID));
        this.targetSessionID = targetSessionID;
        this.cModeISCSISessionConnections = new CModeISCSISessionConnections(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public CModeISCSISessionKey getKey() {
        return key;
    }

    public String getVServerName() {
        return key.getID(CModeISCSISessionKey.Field.vserver_name);
    }

    public String getPortalGroupName() {
        return key.getID(CModeISCSISessionKey.Field.portal_group_name);
    }

    public int getTargetSessionID() {
        return targetSessionID;
    }

    public Integer getMaxConnections() {
        return maxConnections;
    }

    public String getInitiatorNodeName() {
        return initiatorNodeName;
    }

    public String getInitiatorAliasName() {
        return initiatorAliasName;
    }

    public String getInitiatorSessionID() {
        return initiatorSessionID;
    }

    public void setMaxConnections(final Integer maxConnections) {
        this.maxConnections = maxConnections;
    }

    public void setInitiatorNodeName(final String initiatorNodeName) {
        this.initiatorNodeName = initiatorNodeName;
    }

    public void setInitiatorAliasName(final String initiatorAliasName) {
        this.initiatorAliasName = initiatorAliasName;
    }

    public void setInitiatorSessionID(final String initiatorSessionID) {
        this.initiatorSessionID = initiatorSessionID;
    }

    public CModeISCSISessionConnections getCModeISCSISessionConnections() {
        return cModeISCSISessionConnections;
    }

    public CModeISCSISessionConnection createCModeISCSISessionConnection(final Integer connectionID) {
        return new CModeISCSISessionConnection(this, connectionID);
    }

    public static final class CModeISCSISessionKey extends ModelKeyBase<CModeISCSISessionKey.Field> {
        public static enum Field {
            vserver_name,
            portal_group_name,
            target_session_id
        }

        public CModeISCSISessionKey(final String vServerName, final String portalGroupName, final String targetSessionID) {
            super("iscsi_session", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.portal_group_name, portalGroupName);
                put(Field.target_session_id, targetSessionID);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "VServerName", key.getID(CModeISCSISessionKey.Field.vserver_name));
        addAttributeNoCRC(writer, "PortalGroupName", key.getID(CModeISCSISessionKey.Field.portal_group_name));
        addAttributeNoCRC(writer, "TargetSessionID", key.getID(CModeISCSISessionKey.Field.target_session_id));
        addAttributeNoCRC(writer, "MaxConnections", maxConnections);
        addAttributeNoCRC(writer, "InitiatorNodeName", initiatorNodeName);
        addAttributeNoCRC(writer, "InitiatorAliasName", initiatorAliasName);
        addAttributeNoCRC(writer, "InitiatorSessionID", initiatorSessionID);
    }

    @Override
    protected boolean hasChildren() {
        return !cModeISCSISessionConnections.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        super.marshallChildrenToFileAndCRC(cache, writer, tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeISCSISessionConnections.class.getSimpleName(), cModeISCSISessionConnections.values(), tabs); 
    }

}
