package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.EnumMap;
import java.util.List;

public class CModeLUNMap extends BaseCRCXMLNode<CModeCluster> {

    private final LUNMapKey key;
    private Integer lunID;
    private String vServerName;
    private String owningNodeName;
    private String path;
    private List<String> reportingNodeNames = new ArrayList<String>();

    CModeLUNMap(final CModeCluster parent, final String lunUUID, final String iGroupUUID) {
        super(parent, CModeLUNMap.class.getSimpleName());
        this.key = new LUNMapKey(iGroupUUID, lunUUID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public LUNMapKey getKey() {
        return this.key;
    }

    public String getVServerName() {
        return vServerName;
    }

    public Integer getLunID() {
        return lunID;
    }

    public String getOwningNodeName() {
        return owningNodeName;
    }
    
    public String getPath() {
        return path;
    }
    
    public List<String> getReportingNodeNames() {
        return reportingNodeNames;
    }

    public void setLunID(final Integer lunID) {
        this.lunID = lunID;
    }

    public void setOwningNodeName(final String owningNodeName) {
        this.owningNodeName = owningNodeName;
    }
    
    public void setVServerName(final String vServerName) {
        this.vServerName = vServerName;
    }

    public void setPath(final String path) {
        this.path = path;
    }

    public void setReportingNodeNames(final List<String> reportingNodeNames) {
        this.reportingNodeNames = reportingNodeNames;
    }
    
    public final static class LUNMapKey extends ModelKeyBase<LUNMapKey.Field> {
        public static enum Field {
            igroup_uuid,
            lun_uuid
        }

        public LUNMapKey(final String iGroupUUID, final String lunUUID) {
            super("lun_map", new EnumMap<Field, String>(Field.class) {{
                put(Field.igroup_uuid, iGroupUUID);
                put(Field.lun_uuid, lunUUID);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "IGroupUUID", key.getID(LUNMapKey.Field.igroup_uuid));
        addAttributeNoCRC(writer, "LunUUID", key.getID(LUNMapKey.Field.lun_uuid));
        addAttributeNoCRC(writer, "LunID", lunID);
        addAttributeNoCRC(writer, "VServerName", vServerName);
        addAttributeNoCRC(writer, "OwningNodeName", owningNodeName);
        addAttributeNoCRC(writer, "Path", path);
    }
    
    @Override
    protected boolean hasChildren() {
        return !reportingNodeNames.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        final String xmlNodeName = "ReportingNodes";
        addTabulation(writer, tabs);
        writer.append("<").append(xmlNodeName).append(">").append(lineSeperator);
        for (final String nodeName : reportingNodeNames) {
            addTabulation(writer, tabs + 1);
            writer.append("<ReportingNode>");
            writer.append(nodeName);
            writer.append("</ReportingNode>").append(lineSeperator);
        }
        addTabulation(writer, tabs);
        writer.append("</").append(xmlNodeName).append(">").append(lineSeperator);
    }
}
