package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.License;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Node;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;
import java.util.Map;

public abstract class CModeLicense extends BaseCRCXMLNode<CModeCluster> {

    private final CModeLicenseKey key;

    CModeLicense(final CModeCluster cluster, final String name, final String serviceName, final String serialNumber) {
        super(cluster, name);
        this.key = new CModeLicenseKey(serviceName, serialNumber);
    }

    /**
     * Transfer this license's properties to the specified license as specified in
     * <a href="https://rally1.rallydev.com/#/4379996341d/detail/userstory/6155985603">US5844</a>.
     *
     * @param license The target odata license.
     */
    public abstract void toLicense(License license, Map<String, Node> nodesByName);

    @Override
    @SuppressWarnings("unchecked")
    public CModeLicenseKey getKey() {
        return key;
    }

    public String getSerialNumber() {
        return key.getID(CModeLicenseKey.Field.serial_number);
    }

    public String getServiceName() {
        return key.getID(CModeLicenseKey.Field.service_name);
    }

    public static final class CModeLicenseKey extends ModelKeyBase<CModeLicenseKey.Field> {
        public static enum Field {
            service_name,
            serial_number
        }

        public CModeLicenseKey(final String serviceName, final String serialNumber) {
            super("license", new EnumMap<Field, String>(Field.class) {{
                put(Field.service_name, serviceName);
                put(Field.serial_number, serialNumber);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "ServiceName", getServiceName());
        addAttributeNoCRC(writer, "SerialNumber", getSerialNumber());
    }
}
