package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.License;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Node;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.Map;

/**
 * Interim license information in Rollingrock (RR).
 */
public class CModeLicenseInterim extends CModeLicense {

    /**
     * Name of the licensed package.
     */
    private final EnumWrapper<InterimLicensedFeature> licensedFeature;
    /**
     * License code for the specified service/feature. 14 Uppercase Alpha only characters.
     */
    private String code;
    /**
     * Description of the licensed service/feature.
     */
    private String description;
    /**
     * Cluster license expiration date.
     */
    private String expirationDate;
    /**
     * A feature specific data field most commonly interpreted as a node-count.
     */
    private Integer limit;

    CModeLicenseInterim(final CModeCluster cluster, final EnumWrapper<InterimLicensedFeature> licensedFeature, final String serialNumber) {
        super(cluster, CModeLicenseInterim.class.getSimpleName(), licensedFeature.getRawValue(), serialNumber);
        this.licensedFeature = licensedFeature;
    }

    public EnumWrapper<InterimLicensedFeature> getLicensedFeature() {
        return licensedFeature;
    }

    public String getCode() {
        return code;
    }

    public void setCode(final String code) {
        this.code = code;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(final String description) {
        this.description = description;
    }

    public String getExpirationDate() {
        return expirationDate;
    }

    public void setExpirationDate(final String expirationDate) {
        this.expirationDate = expirationDate;
    }

    public Integer getLimit() {
        return limit;
    }

    public void setLimit(final Integer limit) {
        this.limit = limit;
    }

    /**
     * Mapping of
     * <a href="http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/DOT/RrollingrockN/web_ontap_cluster_unsupported/interim/index.html#interim-licensed-feature">interim-licensed-feature</a>
     */
    public static enum InterimLicensedFeature {
        NOT_MAPPED,
        BASE,
        MIRROR,
        FLEXVOL_HPO,
        CIFS,
        SNAPRESTORE,
        FLEXCLONE,
        NFS,
        SNAPMIRROR_DP,
        ISCSI,
        STRIPED_VOLUME,
        FCP,
        SNAPMANAGER_SUITE,
        SPA,
        INSIGHT_BALANCE,
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "LicensedFeature", getLicensedFeature().getEnumValue());
        addAttributeNoCRC(writer, "LicensedFeatureRaw", getLicensedFeature().getRawValue());
        addAttributeNoCRC(writer, "Code", getCode());
        addAttributeNoCRC(writer, "Description", getDescription());
        addAttributeNoCRC(writer, "ExpirationDate", getExpirationDate());
        addAttributeNoCRC(writer, "Limit", getLimit());
    }

    @Override
    public void toLicense(final License license, final Map<String, Node> nodesByName) {
        license.setOwner(getParent().getClusterName());
        license.setOwnerType(License.OwnerType.CLUSTER);
        license.setDescription(getDescription());
    }

}
