package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;


import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import java.io.IOException;

/**
 * Data object for holding the results form the net-ipspaces-get-iter ZAPI.
 */
public class CModeNetworkIPSpace extends BaseCRCXMLNode<CModeCluster> {
    static public final String CREATED_NETWORKIPSPACE_UUID_PREFIX = "NETAPP_OCIE_NETWORKIPSPACE_UUID";
    
    /**
     * Key using the uuid of the IPSpace.
     */
    private final UUIDKey ipSpaceKey;
    /**
     * The ID of the IPSpace
     */
    private Long id;
    /**
     * IPSpace name
     */
    private String ipSpaceName;


    protected CModeNetworkIPSpace(CModeCluster parent, String uuid) {
        super(parent, CModeNetworkIPSpace.class.getSimpleName());
        ipSpaceKey = new UUIDKey("ip_space", uuid);
    }

    @Override
    public UUIDKey getKey() {
        return ipSpaceKey;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getIPSpaceName() {
        return ipSpaceName;
    }

    public void setIPSpaceName(String name) {
        this.ipSpaceName = name;
    }

    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer)
            throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", ipSpaceKey.getUUID());
        addAttributeNoCRC(writer, "ID", id);
        addAttributeNoCRC(writer, "Name", ipSpaceName);
    }
}
