package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;


import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

/**
 * Data object for holding the results form the net-port-broadcast-domain-get-iter ZAPI.
 */
public class CModeNetworkPortBroadcastDomain extends BaseCRCXMLNode<CModeCluster> {

    /**
     * The name of the broadcast domain.
     */
    private final String broadcastDomainName;
    /**
     * Maximum Transmission Unit (MTU) of the broadcast domain.
     */
    private Long mtu;
    /**
     * IP Space name that the broadcast domain belongs to.
     */
    private final String ipSpaceName;
    /**
     * Key using the broadcast domain name and the cluster uuid.
     */
    private final NetworkPortBroadcastDomainKey broadcastDomainKey;

    protected CModeNetworkPortBroadcastDomain(CModeCluster parent, String broadcastDomainName, String ipSpaceName) {
        super(parent, CModeNetworkPortBroadcastDomain.class.getSimpleName());
        this.broadcastDomainName = broadcastDomainName;
        this.ipSpaceName = ipSpaceName;
        this.broadcastDomainKey = new NetworkPortBroadcastDomainKey(parent.getClusterUUID(), broadcastDomainName, ipSpaceName);
    }

    @Override
    public NetworkPortBroadcastDomainKey getKey() {
        return broadcastDomainKey;
    }

    public Long getMtu() {
        return mtu;
    }

    public void setMtu(Long mtu) {
        this.mtu = mtu;
    }

    public String getIpSpaceName() {
        return ipSpaceName;
    }

    public String getBroadcastDomainName() {
        return broadcastDomainName;
    }

    /**
     * OnTap does not provide a UUID for broadcast domain. Create a new key using the cluster uuid, broadcast
     * domain name & ip space name.
     */
    public static final class NetworkPortBroadcastDomainKey extends ModelKeyBase<NetworkPortBroadcastDomainKey.Field> {
        public static enum Field {
            cluster_uuid,
            broadcast_domain_name,
            ip_space_name
        }

        public NetworkPortBroadcastDomainKey(final String clusterUUID, final String broadcastDomainName, final String ipSpaceName) {
            super("broadcast_domain", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.ip_space_name, ipSpaceName);
                put(Field.broadcast_domain_name, broadcastDomainName);
            }});
        }
    }

    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer)
            throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "Name", this.broadcastDomainName);
        addAttributeNoCRC(writer, "MTU", mtu);
        addAttributeNoCRC(writer, "IPSpaceName", ipSpaceName);
    }
}
