package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Node;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;

public class CModeNodeDetail extends BaseCRCXMLNode<CModeCluster> {

    // node-details-info from system-node-get-iter, system-node-get
    private final UUIDKey nodeKey;              // node-uuid
    private String nodeName;                        // node
    private String nodeVendor;                      // node-vendor
    private String nodeModel;                       // node-model
    private String nodeSerialNum;                   // node-serial-number
    private Long nodeUptime;                        // node-uptime
    private String nodeLocation;                    // node-location
    private Boolean envOverTemperature;             // env-over-temperature
    private String envFailedFanMessage;             // env-failed-fan-message
    private String envFailedPowerSupplyMessage;     // env-failed-power-supply-message
    private EnumWrapper<Node.BatteryState> nvramBatteryStatus; // nvram-battery-status
    private String nvramID;                         // node-nvram-id
    private String cpuFirmwareRelease;              // cpu-firmware-release
    private Integer failedFanCount;                 // env-failed-fan-count
    private Integer failedPowerSupplyCount;         // env-failed-power-supply-count
    private String nodeOwner;                       // node-owner
    private Boolean isHealthy;                      // is-node-healthy
    private String productVersion;                  // product-version
    private Boolean isEpsilonNode;                  // is-epsilon-node
    private Boolean isAllFlashOptimized;            // is-all-flash-optimized

    CModeNodeDetail(final CModeCluster parent, final String nodeUUID) {
        super(parent, CModeNodeDetail.class.getSimpleName());
        this.nodeKey = new UUIDKey("node_detail", nodeUUID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return nodeKey;
    }

    public void setNodeNodeName(final String nodeName) {
        this.nodeName = nodeName;
    }

    public String getNodeNodeName() {
        return nodeName;
    }

    public void setNodeVendor(final String nodeVendor) {
        this.nodeVendor = nodeVendor;
    }

    public String getNodeVendor() {
        return nodeVendor;
    }

    public void setNodeModel(final String nodeModel) {
        this.nodeModel = nodeModel;
    }

    public String getNodeModel() {
        return nodeModel;
    }

    public void setNodeSerialNum(final String nodeSerialNum) {
        this.nodeSerialNum = nodeSerialNum;
    }

    public String getNodeSerialNum() {
        return nodeSerialNum;
    }

    public void setNodeUptime(final Long nodeUptime) {
        this.nodeUptime = nodeUptime;
    }

    public Long getNodeUptime() {
        return nodeUptime;
    }

    public void setNodeLocation(final String nodeLocation) {
        this.nodeLocation = nodeLocation;
    }

    public String getNodeLocation() {
        return nodeLocation;
    }

    public Boolean isEnvOverTemperature() {
        return envOverTemperature;
    }

    public void setEnvOverTemperature(final Boolean envOverTemperature) {
        this.envOverTemperature = envOverTemperature;
    }

    public String getEnvFailedFanMessage() {
        return envFailedFanMessage;
    }

    public void setEnvFailedFanMessage(final String envFailedFanMessage) {
        this.envFailedFanMessage = envFailedFanMessage;
    }

    public String getEnvFailedPowerSupplyMessage() {
        return envFailedPowerSupplyMessage;
    }

    public void setEnvFailedPowerSupplyMessage(final String envFailedPowerSupplyMessage) {
        this.envFailedPowerSupplyMessage = envFailedPowerSupplyMessage;
    }

    public EnumWrapper<Node.BatteryState> getNvramBatteryStatus() {
        return nvramBatteryStatus;
    }

    public void setNvramBatteryStatus(final EnumWrapper<Node.BatteryState> nvramBatteryStatus) {
        this.nvramBatteryStatus = nvramBatteryStatus;
    }

    public String getNvramID() {
        return nvramID;
    }

    public void setNvramID(final String nvramID) {
        this.nvramID = nvramID;
    }

    public String getCpuFirmwareRelease() {
        return cpuFirmwareRelease;
    }

    public void setCpuFirmwareRelease(final String cpuFirmwareRelease) {
        this.cpuFirmwareRelease = cpuFirmwareRelease;
    }

    public Integer getFailedFanCount() {
        return failedFanCount;
    }

    public void setFailedFanCount(final Integer failedFanCount) {
        this.failedFanCount = failedFanCount;
    }

    public Integer getFailedPowerSupplyCount() {
        return failedPowerSupplyCount;
    }

    public void setFailedPowerSupplyCount(final Integer failedPowerSupplyCount) {
        this.failedPowerSupplyCount = failedPowerSupplyCount;
    }

    public String getNodeOwner() {
        return nodeOwner;
    }

    public void setNodeOwner(final String nodeOwner) {
        this.nodeOwner = nodeOwner;
    }

    public Boolean isHealthy() {
        return isHealthy;
    }

    public void setHealthy(final Boolean healthy) {
        isHealthy = healthy;
    }

    public void setIsEpsilonNode(final Boolean epsilonNode) {
        this.isEpsilonNode = epsilonNode;
    }

    public Boolean getIsEpsilonNode() {
        return isEpsilonNode;
    }

    public String getProductVersion() {
        return productVersion;
    }

    public void setProductVersion(final String productVersion) {
        this.productVersion = productVersion;
    }

    public Boolean getAllFlashOptimized() {
        return isAllFlashOptimized;
    }

    public void setAllFlashOptimized(Boolean allFlashOptimized) {
        isAllFlashOptimized = allFlashOptimized;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", nodeKey.getUUID());
        addAttributeNoCRC(writer, "Name", nodeName);
        addAttributeNoCRC(writer, "Model", nodeModel);
        addAttributeNoCRC(writer, "SerialNumber", nodeSerialNum);
        addAttributeNoCRC(writer, "Uptime", nodeUptime);
        addAttributeNoCRC(writer, "Vendor", nodeVendor);
        addAttributeNoCRC(writer, "Location", nodeLocation);
        addAttributeNoCRC(writer, "EnvOverTemperature", envOverTemperature);
        addAttributeNoCRC(writer, "EnvFailedFanMessage", envFailedFanMessage);
        addAttributeNoCRC(writer, "EnvFailedPowerSupplyMessage", envFailedPowerSupplyMessage);
        addAttributeNoCRC(writer, "NVRAMBatteryStatus", nvramBatteryStatus.getEnumValue());
        addAttributeNoCRC(writer, "NVRAMBatteryStatusRaw", nvramBatteryStatus.getRawValue());
        addAttributeNoCRC(writer, "NVRAMID", nvramID);
        addAttributeNoCRC(writer, "CPUFirmwareRelease", cpuFirmwareRelease);
        addAttributeNoCRC(writer, "FailedFanCount", failedFanCount);
        addAttributeNoCRC(writer, "FailedPowerSupplyCount", failedPowerSupplyCount);
        addAttributeNoCRC(writer, "Owner", nodeOwner);
        addAttributeNoCRC(writer, "Healthy", isHealthy);
        addAttributeNoCRC(writer, "IsEpsilonNode", isEpsilonNode);
        addAttributeNoCRC(writer, "IsAllFlashOptimized", isAllFlashOptimized);
        addAttributeNoCRC(writer, "Version", productVersion);
    }

}
