package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Portset;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;
import java.util.List;

public class CModePortSet extends BaseCRCXMLNode<CModeCluster> {

    private final PortSetKey key;
    private EnumWrapper<Portset.Type> type;
    private List<String> initiatorGroupNames;
    private List<String> portsetPortNames;

    CModePortSet(final CModeCluster cluster, final String vServerName, final String portSetName) {
        super(cluster, CModePortSet.class.getSimpleName());
        this.key = new PortSetKey(vServerName, portSetName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public PortSetKey getKey() {
        return key;
    }

    public EnumWrapper<Portset.Type> getType() {
        return type;
    }

    public List<String> getInitiatorGroupNames() {
        return initiatorGroupNames;
    }

    public List<String> getPortsetPortNames() {
        return portsetPortNames;
    }

    public void setType(final EnumWrapper<Portset.Type> type) {
        this.type = type;
    }

    public void setInitiatorGroupNames(final List<String> initiatorGroupNames) {
        this.initiatorGroupNames = initiatorGroupNames;
    }

    public void setPortSetPortNames(final List<String> portsetPortNames) {
        this.portsetPortNames = portsetPortNames;
    }

    public final static class PortSetKey extends ModelKeyBase<PortSetKey.Field> {
        public static enum Field {
            vserver_name,
            portset_name
        }

        public PortSetKey(final String vServerName, final String portSetName) {
            super("port_set", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.portset_name, portSetName);
            }}, EnumSet.of(Field.portset_name));                      // These EnumSet<E> emptyAbleFields fields are allowed to be empty but not null
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "VServerName", key.getID(PortSetKey.Field.vserver_name));
        addAttributeNoCRC(writer, "Name", key.getID(PortSetKey.Field.portset_name));
        addAttributeNoCRC(writer, "Type", type.getEnumValue());
        addAttributeNoCRC(writer, "TypeRaw", type.getRawValue());
    }

    @Override
    protected boolean hasChildren() {
        return !initiatorGroupNames.isEmpty() || !portsetPortNames.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        final String igroupNames = "InitiatorGroupNames";
        addTabulation(writer, tabs);
        writer.append("<").append(igroupNames).append(">").append(lineSeperator);
        for (final String initiatorGroupName : initiatorGroupNames) {
            addTabulation(writer, tabs + 1);
            writer.append("<InitiatorGroupName>");
            writer.append(initiatorGroupName);
            writer.append("</InitiatorGroupName>").append(lineSeperator);
        }
        addTabulation(writer, tabs);
        writer.append("</").append(igroupNames).append(">").append(lineSeperator);

        final String portsetNames = "PortsetPortNames";
        addTabulation(writer, tabs);
        writer.append("<").append(portsetNames).append(">").append(lineSeperator);
        for (final String portsetPortName : portsetPortNames) {
            addTabulation(writer, tabs + 1);
            writer.append("<PortsetPortName>");
            writer.append(portsetPortName);
            writer.append("</PortsetPortName>").append(lineSeperator);
        }
        addTabulation(writer, tabs);
        writer.append("</").append(portsetNames).append(">").append(lineSeperator);
    }
}
