package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

/**
 * This object represents a physical CPU inside of a Node
 */
public class CModeProcessor extends BaseCRCXMLNode<CModeNode> {

    private final ProcessorKey key;
    private final Integer processorNumber;

    CModeProcessor(final CModeNode node, final Integer processorNumber) {
        super(node, CModeProcessor.class.getSimpleName());
        this.key = new ProcessorKey(node.getKey().getUUID(), String.valueOf(processorNumber));
        this.processorNumber = processorNumber;
    }



    @Override
    @SuppressWarnings({"unchecked"})
    public ProcessorKey getKey() {
        return key;
    }

    public Integer getProcessorNumber() {
        return processorNumber;
    }

     public final static class ProcessorKey extends ModelKeyBase<ProcessorKey.Field> {
        public static enum Field {
            node_name,
            processor_number
        }

        public ProcessorKey(final String nodeName, final String processorNumber) {
            super("processor", new EnumMap<Field, String>(Field.class) {{
                put(Field.node_name, nodeName);
                put(Field.processor_number, processorNumber);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "NodeName", key.getID());
        addAttributeNoCRC(writer, "ProcessorNumber", processorNumber);
    }
}
