package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

public class CModeQosServiceCenter extends BaseCRCXMLNode<CModeCluster> {

    @SuppressWarnings({"UnusedDeclaration"})
    public static Set<String> FILTER_RESOURCE_NAMES = new HashSet<String>(){{
        add("CPU_cifs");
        add("CPU_cluster");
        add("CPU_dnscache");
        add("CPU_nwk_exclusive");
        add("CPU_nwk_exempt");
        add("CPU_nwk_legacy");
        add("CPU_protocol");
        add("CPU_ssan_exempt");
        add("CPU_target");
        // add("CPU_exempt");       // DE4985 / BURT_756607 we must collect CPU_exempt service centers so
        add("CPU_kahuna");          // we can collect CPU_exempt workload details and subtract them from CPU_dblade
        add("CPU_raid_exempt");     // workload details
        add("CPU_raid");
        add("CPU_sm_exempt");
        add("CPU_storage");
        add("CPU_wafl_exempt");
        add("CPU_wafl_xcleaner");
        add("DELAY_CENTER_WAFL_SUSP_OTHER");
       // add("DISK_HDD_OTHER");  //OPM wants DISK_HDD_OTHER. Hence starting to collect.
       // add("DISK_SSD_OTHER"); // SN re-attribution needs this
        add("CPU_hostOS");
        add("CPU_idle");
    }};

    private final UUIDKey key;
    private String name;
    private String type;
    private String parentResourceUUID;

    CModeQosServiceCenter(final CModeCluster parent, final String serviceCenterUUID) {
        super(parent, CModeQosServiceCenter.class.getSimpleName());
        this.key = new UUIDKey("qos_service_center", serviceCenterUUID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }
    
    public String getType() {
        return type;
    }

    public void setType(final String type) {
        this.type = type;
    }

    public String getParentResourceUUID() {
        return parentResourceUUID;
    }
    
    public void setParentResourceUUID(final String parentResourceUUID) {
        this.parentResourceUUID = parentResourceUUID;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", key.getID());
        addAttributeNoCRC(writer, "Name", name);
        addAttributeNoCRC(writer, "Type", type);
        addAttributeNoCRC(writer, "ParentResourceUUID", parentResourceUUID);
    }

}
