package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumSet;

public class CModeQosWorkloadDetail extends BaseCRCXMLNode<CModeCluster> {

    // Workload Detail Names and UUIDs are constructed algorithmically
    private final UUIDKey key;            // uuid

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum RESOURCE_NAME {
        CPU_dblade,
        CPU_dblade_background,                  // not in rolling rock
        CPU_nblade,
        CPU_exempt,                             // DE4985 / BURT_756607 we must collect CPU_exempt workload details so
        DELAY_CENTER_CLUSTER_INTERCONNECT,      // we can subtract them from CPU_dblade workload details
        DELAY_CENTER_NETWORK,
        DELAY_CENTER_QOS_LIMIT,                 // not in rolling rock
        DELAY_CENTER_WAFL_SUSP_CP,
        DELAY_CENTER_DISK_IO_OTHER,
        DELAY_CENTER_NVLOG_TRANSFER,
        DISK_HDD_OTHER,
        DISK_SSD_OTHER,
        // DISK_HDD_<aggr>                      // dynamic based on aggregates
        // DISK_SSD_<aggr>                      // Not in rolling rock, dynamic based on aggregates
        // DELAY_CENTER_DISK_IO_<aggr>          // Not seen yet, dynamic based on aggregates
    }


    public static EnumSet<RESOURCE_NAME> getNBladeResourceNames() {
        final EnumSet<RESOURCE_NAME> nBladeResourceNames = EnumSet.noneOf(RESOURCE_NAME.class);
        nBladeResourceNames.add(RESOURCE_NAME.CPU_nblade);
        nBladeResourceNames.add(RESOURCE_NAME.DELAY_CENTER_CLUSTER_INTERCONNECT);
        nBladeResourceNames.add(RESOURCE_NAME.DELAY_CENTER_NETWORK);
        nBladeResourceNames.add(RESOURCE_NAME.DELAY_CENTER_QOS_LIMIT);
        return nBladeResourceNames;
    }

    public static EnumSet<RESOURCE_NAME> getDBladeResourceNames() {
        final EnumSet<RESOURCE_NAME> dBladeResourceNames = EnumSet.noneOf(RESOURCE_NAME.class);
        dBladeResourceNames.add(RESOURCE_NAME.CPU_dblade);
        dBladeResourceNames.add(RESOURCE_NAME.CPU_exempt);
        dBladeResourceNames.add(RESOURCE_NAME.CPU_dblade_background);
        dBladeResourceNames.add(RESOURCE_NAME.DELAY_CENTER_WAFL_SUSP_CP);
        dBladeResourceNames.add(RESOURCE_NAME.DELAY_CENTER_NVLOG_TRANSFER);
        dBladeResourceNames.add(RESOURCE_NAME.DISK_HDD_OTHER);
        dBladeResourceNames.add(RESOURCE_NAME.DISK_SSD_OTHER);
        // Also DISK_HDD_<aggr>
        // Also DISK_SSD_<aggr>
        // Also DELAY_CENTER_DISK_IO_<aggr>
        return dBladeResourceNames;
    }

    private String workloadDetailName;              // name
    
    CModeQosWorkloadDetail(final CModeCluster parent, final String uuid) {
        super(parent, CModeQosWorkloadDetail.class.getSimpleName());
        this.key = new UUIDKey("qos_workload_detail", uuid);
    }
    
    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    public String getName() {
        return workloadDetailName;
    }
    
    public void setName(final String name) {
        this.workloadDetailName = name;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "UUID", key.getID());
        addRequiredAttributeAndCRC(writer, "Name", workloadDetailName);
    }
}
