package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.TreeQuota;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;

public class CModeQuota extends BaseCRCXMLNode<CModeCluster> {

    // Values are Strings not numbers because "-" is a valid value meaning "unlimited",
    // even for used.
    private final CModeQuotaKey key;
    private final CModeQuotaUsers quotaUsers;
    private String diskLimit;
    private String diskUsed;
    private String fileLimit;
    private String fileUsed;
    private TreeQuota.QuotaType quotaType;
    private String softDiskLimit;
    private String softFileLimit;
    private String threshold;

    CModeQuota(final CModeCluster cluster, final String vServerName, final String volumeName, final String qtreeName,
               final String quotaTarget, final String userHash) {
        super(cluster, CModeQuota.class.getSimpleName());
        this.key = new CModeQuotaKey(vServerName, volumeName, qtreeName, quotaTarget, userHash);
        this.quotaUsers = new CModeQuotaUsers(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public CModeQuotaKey getKey() {
        return key;
    }

    public String getDiskLimit() {
        return diskLimit;
    }

    public String getDiskUsed() {
        return diskUsed;
    }

    public String getFileLimit() {
        return fileLimit;
    }

    public String getFileUsed() {
        return fileUsed;
    }

    public String getQTreeID() {
        return key.getID(CModeQuotaKey.Field.qtree_id);
    }

    public String getQuotaTarget() {
        return key.getID(CModeQuotaKey.Field.quota_target);
    }

    public TreeQuota.QuotaType getQuotaType() {
        return quotaType;
    }

    public String getSoftDiskLimit() {
        return softDiskLimit;
    }

    public String getSoftFileLimit() {
        return softFileLimit;
    }

    public String getThreshold() {
        return threshold;
    }

    public String getVolumeName() {
        return key.getID(CModeQuotaKey.Field.volume_name);
    }

    public String getVServerName() {
        return key.getID(CModeQuotaKey.Field.vserver_name);
    }

    public void setDiskLimit(final String diskLimit) {
        this.diskLimit = diskLimit;
    }

    public void setDiskUsed(final String diskUsed) {
        this.diskUsed = diskUsed;
    }

    public void setFileLimit(final String fileLimit) {
        this.fileLimit = fileLimit;
    }

    public void setFileUsed(final String fileUsed) {
        this.fileUsed = fileUsed;
    }

    public void setQuotaType(final TreeQuota.QuotaType quotaType) {
        this.quotaType = quotaType;
    }

    public void setSoftDiskLimit(final String softDiskLimit) {
        this.softDiskLimit = softDiskLimit;
    }

    public void setSoftFileLimit(final String softFileLimit) {
        this.softFileLimit = softFileLimit;
    }

    public void setThreshold(final String threshold) {
        this.threshold = threshold;
    }

    public String getQuotaUserHash() {
        return key.getID(CModeQuotaKey.Field.quota_user_hash);
    }

    public CModeQuotaUsers getQuotaUsers() {
        return quotaUsers;
    }

    public CModeQuotaUser createQuotaUser(final String quotaUserID) {
        return new CModeQuotaUser(this, quotaUserID);
    }

    public static final class CModeQuotaKey extends ModelKeyBase<CModeQuotaKey.Field> {

        public static enum Field {
            vserver_name,
            volume_name,
            qtree_id,
            quota_target,
            quota_user_hash
        }

        public CModeQuotaKey(final String vServerName, final String volumeName, final String qTreeName, final String quotaTarget,
                             final String userHash) {
            super("quota", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.volume_name, volumeName);
                put(Field.qtree_id, qTreeName);
                put(Field.quota_target, quotaTarget);
                put(Field.quota_user_hash, userHash);
            }}, EnumSet.of(Field.qtree_id,
                           Field.quota_target));
        }

    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "VServerName", getVServerName());
        addAttributeNoCRC(writer, "VolumeName", getVolumeName());
        addAttributeNoCRC(writer, "QTreeID", getQTreeID());
        addAttributeNoCRC(writer, "QuotaTarget", getQuotaTarget());
        addAttributeNoCRC(writer, "DiskLimit", diskLimit);
        addAttributeNoCRC(writer, "FileLimit", fileLimit);
        addAttributeNoCRC(writer, "QuotaType", quotaType);
        addAttributeNoCRC(writer, "SoftDiskLimit", softDiskLimit);
        addAttributeNoCRC(writer, "SoftFileLimit", softFileLimit);
        addAttributeNoCRC(writer, "Threshold", threshold);
    }

    @Override
    protected boolean hasChildren() {
        return !quotaUsers.isEmpty();
    }

    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        super.marshallChildrenToFileAndCRC(cache, writer, tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeQuotaUsers.class.getSimpleName(), quotaUsers.values(), tabs);
    }

}
