package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.RaidGroup;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeRaidGroup extends BaseCRCXMLNode<CModePlex> {

    private final RaidGroupKey key;
    private Boolean isReconstructing;
    private EnumWrapper<RaidGroup.RaidType> raidType;

    CModeRaidGroup(final CModePlex parent, final String aggrUUID, final String name) {
        super(parent, CModeRaidGroup.class.getSimpleName());
        this.key = new RaidGroupKey(aggrUUID, name);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public RaidGroupKey getKey() {
        return key;
    }

    public Boolean isReconstructing() {
        return isReconstructing;
    }

    public void setReconstructing(final Boolean reconstructing) {
        isReconstructing = reconstructing;
    }

    public EnumWrapper<RaidGroup.RaidType> getRaidType() {
        return raidType;
    }

    public void setRaidType(EnumWrapper<RaidGroup.RaidType> raidType) {
        this.raidType = raidType;
    }

    public final static class RaidGroupKey extends ModelKeyBase<RaidGroupKey.Field> {
        public static enum Field {
            aggregate_uuid,
            raidgroup_name
        }

        public RaidGroupKey(final String aggrUUID, final String rgName) {
            super("raidgroup", new EnumMap<Field, String>(Field.class) {{
                put(Field.aggregate_uuid, aggrUUID);
                put(Field.raidgroup_name, rgName);
            }});
        }
    }

    protected void marshallAttributesToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(cache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "AggregateUUID", key.getID(RaidGroupKey.Field.aggregate_uuid));
        addAttributeNoCRC(writer, "RaidgroupName", key.getID(RaidGroupKey.Field.raidgroup_name));
        addAttributeNoCRC(writer, "RaidType", raidType);
        addAttributeNoCRC(writer, "IsReconstructing", isReconstructing);
    }
}
