package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.SisPolicy;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeSisPolicy extends BaseCRCXMLNode<CModeCluster> {

    private final SisPolicyKey policyKey;
    private Boolean enabled;
    private String comment;
    private String duration;
    private String schedule;
    private EnumWrapper<SisPolicy.QosPolicy> qosPolicy;

    CModeSisPolicy(final CModeCluster parent, final String vServerName, final String policyName) {
        super(parent, CModeSisPolicy.class.getSimpleName());
        policyKey = new SisPolicyKey(vServerName, policyName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public SisPolicyKey getKey() {
        return policyKey;
    }

    public Boolean isEnabled() {
        return enabled;
    }

    public void setEnabled(final Boolean enabled) {
        this.enabled = enabled;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(final String comment) {
        this.comment = comment;
    }

    public String getDuration() {
        return duration;
    }

    public void setDuration(final String duration) {
        this.duration = duration;
    }

    public String getSchedule() {
        return schedule;
    }

    public void setSchedule(final String schedule) {
        this.schedule = schedule;
    }

    public EnumWrapper<SisPolicy.QosPolicy> getQosPolicy() {
        return qosPolicy;
    }

    public void setQosPolicy(final EnumWrapper<SisPolicy.QosPolicy> qosPolicy) {
        this.qosPolicy = qosPolicy;
    }

    public static final class SisPolicyKey extends ModelKeyBase<SisPolicyKey.Field> {
        public static enum Field {
            vserver_name,
            sis_policy_name
        }

        public SisPolicyKey(final String vServerName, final String policyName) {
            super("sis_policy", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.sis_policy_name, policyName);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "VServerName", policyKey.getID(SisPolicyKey.Field.vserver_name));
        addAttributeNoCRC(writer, "PolicyName", policyKey.getID(SisPolicyKey.Field.sis_policy_name));
        addAttributeNoCRC(writer, "Comment", comment);
        addAttributeNoCRC(writer, "Duration", duration);
        addAttributeNoCRC(writer, "Schedule", schedule);
        addAttributeNoCRC(writer, "QOSPolicy", EnumWrapper.toStringOrNull(qosPolicy));
    }
}
