package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.SnapMirrorPolicy;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

/**
 * Name	        Range	    Type	    Description
 * comment		            string      A human readable description associated with the SnapMirror policy.
 *                          optional
 *
 * policy-name		        sm-policy   The name of the SnapMirror policy
 *                          optional
 *
 * total-keep	[0..2^32-1]	integer     Total Retention Count for All Rules in the Policy
 *                          optional
 *
 * transfer-priority		sm-transfer-priority-enum   Specifies the priority at which a SnapMirror transfer runs.
 *                          optional                    'normal' : By default every transfer in the system has this priority. These transfers are scheduled before most 'low' priority transfers.
 *                                                      'low': These transfers have least priority and usually are scheduled after most 'normal' priority transfers.
 *
 * vserver-name		        vserver-name    Specifies the vserver for the SnapMirror policy
 *                          optional
 */
public class CModeSnapMirrorPolicy extends BaseCRCXMLNode<CModeCluster> {

    private final PolicyKey policyKey;
    private String comment;
    private Long totalKeep;
    EnumWrapper<SnapMirrorPolicy.TransferPriority> transferPriority;
    private String tries;
    private EnumWrapper<SnapMirrorPolicy.Restart> restart;
    private Boolean ignoreAccessTime;
    private Boolean createSnapshot;
    private Boolean isNetworkCompressionEnabled;
    private EnumWrapper<SnapMirrorPolicy.Type> type;

    private final CModeSnapMirrorRules rules;

    CModeSnapMirrorPolicy(final CModeCluster parent, final String vServerName, final String policyName) {
        super(parent, CModeSnapMirrorPolicy.class.getSimpleName());
        this.policyKey = new PolicyKey(vServerName, policyName);
        this.rules = new CModeSnapMirrorRules(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public PolicyKey getKey() {
        return policyKey;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(final String comment) {
        this.comment = comment;
    }

    public Long getTotalKeep() {
        return totalKeep;
    }

    public void setTotalKeep(final Long totalKeep) {
        this.totalKeep = totalKeep;
    }

    public EnumWrapper<SnapMirrorPolicy.TransferPriority> getTransferPriority() {
        return transferPriority;
    }

    public void setTransferPriority(final EnumWrapper<SnapMirrorPolicy.TransferPriority> transferPriority) {
        this.transferPriority = transferPriority;
    }

    public CModeSnapMirrorRule createCModeSnapMirrorRule(final int index) {
        return new CModeSnapMirrorRule(this, index);
    }

    public CModeSnapMirrorRules getCmodeSnapMirrorRules() {
        return rules;
    }

    public String getTries() {
        return tries;
    }

    public void setTries(String tries) {
        this.tries = tries;
    }

    public EnumWrapper<SnapMirrorPolicy.Restart> getRestart() {
        return restart;
    }

    public void setRestart(EnumWrapper<SnapMirrorPolicy.Restart> restart) {
        this.restart = restart;
    }

    public Boolean getIgnoreAccessTime() {
        return ignoreAccessTime;
    }

    public void setIgnoreAccessTime(Boolean ignoreAccessTime) {
        this.ignoreAccessTime = ignoreAccessTime;
    }

    public Boolean getCreateSnapshot() {
        return createSnapshot;
    }

    public void setCreateSnapshot(Boolean createSnapshot) {
        this.createSnapshot = createSnapshot;
    }

    public Boolean getNetworkCompressionEnabled() {
        return isNetworkCompressionEnabled;
    }

    public void setNetworkCompressionEnabled(Boolean networkCompressionEnabled) {
        isNetworkCompressionEnabled = networkCompressionEnabled;
    }

    public EnumWrapper<SnapMirrorPolicy.Type> getType() {
        return type;
    }

    public void setType(EnumWrapper<SnapMirrorPolicy.Type> type) {
        this.type = type;
    }

    public final static class PolicyKey extends ModelKeyBase<PolicyKey.Field> {
        public static enum Field {
            vserver_name,
            snapmirror_policy_name
        }

        public PolicyKey(final String vServerName, final String policyName) {
            super("snapmirror_policy", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.snapmirror_policy_name, policyName);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", policyKey.getID());
        addAttributeNoCRC(writer, "VServerName", policyKey.getID(PolicyKey.Field.vserver_name));
        addAttributeNoCRC(writer, "PolicyName", policyKey.getID(PolicyKey.Field.snapmirror_policy_name));
        addAttributeNoCRC(writer, "Comment", comment);
        addAttributeNoCRC(writer, "CreateSnapshot", createSnapshot);
        addAttributeNoCRC(writer, "IgnoreAccessTime", ignoreAccessTime);
        addAttributeNoCRC(writer, "IsNetworkCompressionEnabled", isNetworkCompressionEnabled);
        addAttributeNoCRC(writer, "Restart", restart.getEnumValue());
        addAttributeNoCRC(writer, "TotalKeep", totalKeep);
        addAttributeNoCRC(writer, "TransferPriority", transferPriority);
        addAttributeNoCRC(writer, "Tries", tries);
        addAttributeNoCRC(writer, "Type", type.getEnumValue());
    }

    protected boolean hasChildren() {
        return !rules.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeSnapMirrorRules.class.getSimpleName(), rules.values(), tabs);
    }
}
