package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeSnapshot extends BaseCRCXMLNode<CModeCluster> {

    private final SnapshotKey snapshotKey;
    private Long accessTime;
    private Boolean isBusy;
    private Long cumulativePercentageOfTotalBlocks;
    private String dependency;
    private String name;
    private Integer percentageOfTotalBlocks;
    private String versionUUID;

    CModeSnapshot(final CModeCluster parent, final String vServerName, final String volumeName, final String instanceUUID) {
        super(parent, CModeSnapshot.class.getSimpleName());
        this.snapshotKey = new SnapshotKey(vServerName, volumeName, instanceUUID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public SnapshotKey getKey() {
        return snapshotKey;
    }

    public Long getAccessTime() {
        return accessTime;
    }

    public void setAccessTime(final Long accessTime) {
        this.accessTime = accessTime;
    }

    public String getVServerName() {
        return snapshotKey.getID(SnapshotKey.Field.vserver_name);
    }

    public Boolean isBusy() {
        return isBusy;
    }

    public void setBusy(final Boolean busy) {
        isBusy = busy;
    }

    public Long getCumulativePercentageOfTotalBlocks() {
        return cumulativePercentageOfTotalBlocks;
    }

    public void setCumulativePercentageOfTotalBlocks(final Long cumulativePercentageOfTotalBlocks) {
        this.cumulativePercentageOfTotalBlocks = cumulativePercentageOfTotalBlocks;
    }

    public String getDependency() {
        return dependency;
    }

    public void setDependency(final String dependency) {
        this.dependency = dependency;
    }

    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public Integer getPercentageOfTotalBlocks() {
        return percentageOfTotalBlocks;
    }

    public void setPercentageOfTotalBlocks(final Integer percentageOfTotalBlocks) {
        this.percentageOfTotalBlocks = percentageOfTotalBlocks;
    }

    public String getInstanceUUID() {
        return snapshotKey.getID(SnapshotKey.Field.snapshot_instance_uuid);
    }

    public String getVolumeName() {
        return snapshotKey.getID(SnapshotKey.Field.volume_name);
    }


    public String getVersionUUID() {
        return versionUUID;
    }

    public void setVersionUUID(final String versionUUID) {
        this.versionUUID = versionUUID;
    }

    public final static class SnapshotKey extends ModelKeyBase<SnapshotKey.Field> {
        public static enum Field {
            vserver_name,
            volume_name,
            snapshot_instance_uuid
        }

        public SnapshotKey(final String vServerName, final String volumeName, final String snapshotInstanceUUID) {
            super("snapshot", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.volume_name, volumeName);
                put(Field.snapshot_instance_uuid, snapshotInstanceUUID);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "VolumeName", getVolumeName());
        addAttributeNoCRC(writer, "InstanceUUID", getInstanceUUID());
        addAttributeNoCRC(writer, "VServerName", getVServerName());
        addAttributeNoCRC(writer, "VersionUUID", versionUUID);
        addAttributeNoCRC(writer, "Name", name);
        addAttributeNoCRC(writer, "IsBusy", isBusy);
        addAttributeNoCRC(writer, "CreationTime", accessTime);
        addAttributeNoCRC(writer, "Dependency", accessTime);
        addAttributeNoCRC(writer, "CumulativePercentageOfTotalBlocks", cumulativePercentageOfTotalBlocks);
        addAttributeNoCRC(writer, "PercentageOfTotalBlocks", percentageOfTotalBlocks);
    }

}
