package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.StoragePool;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeStoragePool extends BaseCRCXMLNode<CModeCluster> {

    private final StoragePoolNameKey storagePoolKey;
    private final String storagePoolName;
    private Long allocationUnitSize;
    private Long diskCount;
    private Long poolTotalSize;
    private Long poolUsableSize;
    private Boolean isHealthy;
    private EnumWrapper<StoragePool.StorageType> storageType;


    protected CModeStoragePool(final CModeCluster parent, final String storagePoolName) {
        super(parent, CModeStoragePool.class.getSimpleName());
        this.storagePoolName = storagePoolName;
        this.storagePoolKey = new StoragePoolNameKey(parent.getClusterUUID(), storagePoolName);
    }



    public Long getAllocationUnitSize() {
        return allocationUnitSize;
    }

    public String getStoragePoolName() {
        return storagePoolName;
    }

    public void setAllocationUnitSize(final Long allocationUnitSize) {
        this.allocationUnitSize = allocationUnitSize;
    }

    public Long getDiskCount() {
        return diskCount;
    }

    public void setDiskCount(final Long diskCount) {
        this.diskCount = diskCount;
    }

    public Long getPoolTotalSize() {
        return poolTotalSize;
    }

    public void setPoolTotalSize(final Long poolTotalSize) {
        this.poolTotalSize = poolTotalSize;
    }

    public Long getPoolUsableSize() {
        return poolUsableSize;
    }

    public void setPoolUsableSize(final Long poolUsableSize) {
        this.poolUsableSize = poolUsableSize;
    }

    public EnumWrapper<StoragePool.StorageType> getStorageType() {

        return storageType;
    }

    public void setStorageType(final EnumWrapper<StoragePool.StorageType> storageType) {
        this.storageType = storageType;
    }

    public Boolean getHealthy() {
        return isHealthy;
    }

    public void setHealthy(Boolean healthy) {
        isHealthy = healthy;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public StoragePoolNameKey getKey() {
        return storagePoolKey;
    }

    public static final class StoragePoolNameKey extends ModelKeyBase<StoragePoolNameKey.Field> {
        public static enum Field {
            cluster_uuid,
            storage_pool_name
        }

        public StoragePoolNameKey(final String clusterUUID, final String storagePoolName) {
            super("storage_pool", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.storage_pool_name, storagePoolName);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(cache, writer);
        addAttributeNoCRC(writer, "Name", storagePoolName);
        addAttributeNoCRC(writer, "AllocationUnitSize", allocationUnitSize);
        addAttributeNoCRC(writer, "DiskCount", diskCount);
        addAttributeNoCRC(writer, "IsHealthy", isHealthy);
        addAttributeNoCRC(writer, "poolTotalSize", poolTotalSize);
        addAttributeNoCRC(writer, "poolUsableSize", poolUsableSize);
        addAttributeNoCRC(writer, "storageType", storageType);
        addAttributeNoCRC(writer, "storageTypeRaw", storageType.getRawValue());
    }
}
