package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeStoragePoolAggregate extends BaseCRCXMLNode<CModeCluster> {

    private final StoragePoolAggregateKey storagePoolAggregateKey;
    private final String storagePoolName;
    private final String aggregateName;
    private Long allocatedUnitCount;
    private Long usedCapacity;

    protected CModeStoragePoolAggregate(final CModeCluster parent, final String storagePoolName, final String aggregateName) {
        super(parent, CModeStoragePoolAggregate.class.getSimpleName());
        this.aggregateName= aggregateName;
        this.storagePoolName = storagePoolName;
        this.storagePoolAggregateKey = new StoragePoolAggregateKey(parent.getClusterUUID(), storagePoolName, aggregateName);
    }


    public static final class StoragePoolAggregateKey extends ModelKeyBase<StoragePoolAggregateKey.Field> {
        public static enum Field {
            cluster_uuid,
            storage_pool_name,
            aggregate_name
        }

        public StoragePoolAggregateKey(final String clusterUUID, final String storagePoolName, final String aggregateName) {
            super("storage_pool_aggregate_relationship", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.storage_pool_name, storagePoolName);
                put(Field.aggregate_name, aggregateName);
            }});
        }
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public StoragePoolAggregateKey getKey() {
        return storagePoolAggregateKey;
    }


    public String getStoragePoolName() {
        return storagePoolName;
    }

    public String getAggregateName() {
        return aggregateName;
    }

    public Long getAllocatedUnitCount() {
        return allocatedUnitCount;
    }

    public void setAllocatedUnitCount(final Long allocatedUnitCount) {
        this.allocatedUnitCount = allocatedUnitCount;
    }

    public Long getUsedCapacity() {
        return usedCapacity;
    }

    public void setUsedCapacity(final Long usedCapacity) {
        this.usedCapacity = usedCapacity;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(cache, writer);
        addAttributeNoCRC(writer, "StoragePoolName", storagePoolName);
        addAttributeNoCRC(writer, "AggregateName", aggregateName);
        addAttributeNoCRC(writer, "UsedCapacity", usedCapacity);
        addAttributeNoCRC(writer, "AllocatedUnitCount", allocatedUnitCount);
    }
}