package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.StoragePool;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeStoragePoolAvailableSize extends BaseCRCXMLNode<CModeCluster> {

    private final StoragePoolAvailableSizeKey storagePoolAvailableSizeKey;
    private final String spareNodeName;
    private final String storagePoolName;
    private Long allocationUnitCount;
    private Long allocationUnitSize;
    private Long availableSize;
    private String syncMirrorPool;
    private EnumWrapper<StoragePool.StorageType> storageType;


    protected CModeStoragePoolAvailableSize(final CModeCluster parent,  final String nodeName, final String storagePoolName) {
        super(parent, CModeStoragePoolAvailableSize.class.getSimpleName());
        this.spareNodeName = nodeName;
        this.storagePoolName = storagePoolName;
        this.storagePoolAvailableSizeKey = new StoragePoolAvailableSizeKey(parent.getClusterUUID(), nodeName, storagePoolName);
    }


    public static final class StoragePoolAvailableSizeKey extends ModelKeyBase<StoragePoolAvailableSizeKey.Field> {
        public static enum Field {
            cluster_uuid,
            node_name,
            storage_pool_name
        }

        public StoragePoolAvailableSizeKey(final String clusterUUID, final String nodeName, final String storagePoolName) {
            super("storage_pool_available", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.node_name, nodeName);
                put(Field.storage_pool_name, storagePoolName);
            }});
        }
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public StoragePoolAvailableSizeKey getKey() {
        return storagePoolAvailableSizeKey;
    }

    public Long getAllocationUnitCount() {
        return allocationUnitCount;
    }

    public void setAllocationUnitCount(final Long allocationUnitCount) {
        this.allocationUnitCount = allocationUnitCount;
    }

    public Long getAllocationUnitSize() {
        return allocationUnitSize;
    }

    public void setAllocationUnitSize(final Long allocationUnitSize) {
        this.allocationUnitSize = allocationUnitSize;
    }

    public Long getAvailableSize() {
        return availableSize;
    }

    public void setAvailableSize(final Long availableSize) {
        this.availableSize = availableSize;
    }

    public String getSyncMirrorPool() {
        return syncMirrorPool;

    }

    public void setSyncMirrorPool(final String syncMirrorPool) {
        this.syncMirrorPool = syncMirrorPool;
    }

    public EnumWrapper<StoragePool.StorageType> getStorageType() {
        return storageType;
    }

    public void setStorageType(final EnumWrapper<StoragePool.StorageType> storageType) {
        this.storageType = storageType;
    }

    @SuppressWarnings({"unused declaration"})
    public StoragePoolAvailableSizeKey getStoragePoolAvailableSizeKey() {
        return storagePoolAvailableSizeKey;
    }

    public String getStoragePoolName() {
        return storagePoolName;
    }

    public String getSpareNodeName() {
        return spareNodeName;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(cache, writer);
        addAttributeNoCRC(writer, "SpareNodeName", spareNodeName);
        addAttributeNoCRC(writer, "StoragePoolName", storagePoolName);
        addAttributeNoCRC(writer, "AllocationUnitCount", allocationUnitCount);
        addAttributeNoCRC(writer, "AllocationUnitSize", allocationUnitSize);
        addAttributeNoCRC(writer, "AvailableSize", availableSize);
        addAttributeNoCRC(writer, "SyncMirrorPool", syncMirrorPool);
        addAttributeNoCRC(writer, "storageType", storageType);
        addAttributeNoCRC(writer, "storageTypeRaw", storageType.getRawValue());
    }
}