package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeStorageService extends BaseCRCXMLNode<CModeCluster> {

	private final ServiceKey key;
    private final Long storageServiceID;
    private Long constituentGroupID;
    private String storageService;
    private String storageServiceType;

    public CModeStorageService(final CModeCluster parent, final String vServerName, final String volumeName, final Long storageServiceID) {
        super(parent, CModeStorageService.class.getSimpleName());
        this.key = new ServiceKey(vServerName, volumeName, String.valueOf(storageServiceID));
        this.storageServiceID = storageServiceID;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public ServiceKey getKey() {
        return key;
    }

    public Long getConstituentGroupID() {
        return constituentGroupID;
    }

    public void setConstituentGroupID(final Long constituentGroupID) {
        this.constituentGroupID = constituentGroupID;
    }

    public String getStorageService() {
        return storageService;
    }

    public void setStorageService(final String storageService) {
        this.storageService = storageService;
    }

    public Long getStorageServiceID() {
        return storageServiceID;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getStorageServiceType() {
        return storageServiceType;
    }

    public void setStorageServiceType(final String storageServiceType) {
        this.storageServiceType = storageServiceType;
    }

    public final static class ServiceKey extends ModelKeyBase<ServiceKey.Field> {
        public static enum Field {
            vserver_name,
            volume_name,
            storage_service_id
        }

        public ServiceKey(final String vServerName, final String volumeName, final String storageServiceID) {
            super("storage_service", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.volume_name, volumeName);
                put(Field.storage_service_id, storageServiceID);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "VServerName", key.getID(ServiceKey.Field.vserver_name));
        addAttributeNoCRC(writer, "VolumeName", key.getID(ServiceKey.Field.volume_name));
        addAttributeNoCRC(writer, "ConstituentGroupID", constituentGroupID);
        addAttributeNoCRC(writer, "StorageService", storageService);
        addAttributeNoCRC(writer, "StorageServiceID", key.getID(ServiceKey.Field.storage_service_id));
        addAttributeNoCRC(writer, "StorageServiceType", storageServiceType);
    }
}