package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.*;
import org.apache.commons.lang3.StringUtils;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;

public class CModeStorageShelfChannel extends BaseCRCXMLNode<CModeNode> {

    private final UUIDKey key;

    private final CModeStorageShelfEnvs storageShelfEnvs;

    public CModeStorageShelfChannel(final CModeNode parent, final String shelfUID) {
        super(parent, CModeStorageShelfChannel.class.getSimpleName());
        this.key = new UUIDKey("storage_shelf_channel", shelfUID);
        this.storageShelfEnvs = new CModeStorageShelfEnvs(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    public CModeStorageShelfEnv createStorageShelfEnv(final String sesLogicalIDRaw) {
        return new CModeStorageShelfEnv(this, sesLogicalIDRaw);
    }

    public CModeStorageShelfEnvs getStorageShelfEnvs() {
        return this.storageShelfEnvs;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ChannelName", key.getName());
    }

    @Override
    protected boolean hasChildren() {
        return !storageShelfEnvs.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        super.marshallChildrenToFileAndCRC(cache, writer, tabs);
        // children martial themselves
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeStorageShelfEnvs.class.getSimpleName(), storageShelfEnvs.values(), tabs);
    }


    public static final class CModeStorageShelfEnvs extends NodeMap<CModeStorageShelfChannel, UUIDKey, CModeStorageShelfEnv> {
        CModeStorageShelfEnvs(final CModeStorageShelfChannel modelParent) {
            super(modelParent);
        }

        public boolean add(final CModeStorageShelfEnv shelfEnv) {
            return (put(shelfEnv.getKey(), shelfEnv) == null);
        }
    }


    public static final class CModeStorageShelfEnv extends BaseCRCXMLNode<CModeStorageShelfChannel> {

        private final UUIDKey key;

        private String shelfType;
        private String shelfStatus;
        private String sesContactState;
        private String sesLogicalID;
        private String sesProductID;
        private String sesProductVersion;
        private String sesVendorID;

        private final CModeCoolingElements coolingElements;
        private final CModePowerSupplies powerSupplies;
        private final CModeTempSensors tempSensors;
        private final CModeSensors sensors;
        private final CModeShelfCables shelfCables;

        CModeStorageShelfEnv(final CModeStorageShelfChannel parent, final String sesLogicalIDRaw) {
            super(parent, CModeStorageShelfEnv.class.getSimpleName());
            this.key = new UUIDKey("shelf", sesLogicalIDRaw );
            this.coolingElements = new CModeCoolingElements(this);
            this.powerSupplies = new CModePowerSupplies(this);
            this.tempSensors = new CModeTempSensors(this);
            this.sensors = new CModeSensors(this);
            this.shelfCables = new CModeShelfCables(this);
        }

        @Override
        @SuppressWarnings({"unchecked"})
        public UUIDKey getKey() {
            return key;
        }

        public void setShelfType(final String shelfType) {
            this.shelfType = shelfType;
        }

        public void setShelfStatus(final String shelfStatus) {
            this.shelfStatus = shelfStatus;
        }

        public String getSesContactState() {
            return sesContactState;
        }

        public void setSesContactState(final String sesContactState) {
            this.sesContactState = sesContactState;
        }

        public String getSesLogicalID() {
            return sesLogicalID;
        }

        public void setSesLogicalID(final String sesLogicalID) {
            this.sesLogicalID = sesLogicalID;
        }

        public String getSesProductID() {
            return sesProductID;
        }

        public void setSesProductID(final String sesProductID) {
            this.sesProductID = sesProductID;
        }

        public String getSesProductVersion() {
            return sesProductVersion;
        }

        public void setSesProductVersion(final String sesProductVersion) {
            this.sesProductVersion = sesProductVersion;
        }

        public String getSesVendorID() {
            return sesVendorID;
        }

        public void setSesVendorID(final String sesVendorID) {
            this.sesVendorID = sesVendorID;
        }

        public CModeCoolingElement createCoolingElement(final String coolingElementNo) {
            return new CModeCoolingElement(this, coolingElementNo);
        }

        public CModeCoolingElements getCoolingElements() {
            return coolingElements;
        }

        public CModePowerSupply createPowerSupply(final String powerSupplyNo) {
            return new CModePowerSupply(this, powerSupplyNo);
        }

        public CModePowerSupplies getPowerSupplies() {
            return powerSupplies;
        }

        public CModeTempSensor createTempSensor(final String sensorNo) {
            return new CModeTempSensor(this, sensorNo);
        }

        public CModeTempSensors getTempSensors() {
            return tempSensors;
        }

        public CModeSensor createSensor(final CModeSensor.Type type, final String sensorNo) {
            return new CModeSensor(this, type, sensorNo);
        }

        public CModeSensors getSensors() {
            return sensors;
        }

        public CModeShelfCable createAndAddShelfCable(final String nodeName, final String shelfID, final String channelName,
                                                      final String partNumber, final String serialNumber, final Integer connectorNumber) {
            final CModeShelfCable cable = new CModeShelfCable(this, nodeName, shelfID, channelName, partNumber, serialNumber, connectorNumber);
            shelfCables.add(cable);
            return cable;
        }

        public CModeShelfCables getShelfCables() {
            return shelfCables;
        }

        @Override
        protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
            super.marshallAttributes(marshallingCache, writer);
            addAttributeNoCRC(writer, "ShelfID", key.getID());
            addAttributeNoCRC(writer, "ShelfType", shelfType);
            addAttributeNoCRC(writer, "ShelfStatus", shelfStatus);
            addAttributeNoCRC(writer, "SesContactState", sesContactState);
            addAttributeNoCRC(writer, "SesLogicalID", sesLogicalID);
            addAttributeNoCRC(writer, "SesProductID", sesProductID);
            addAttributeNoCRC(writer, "SesProductVersion", sesProductVersion);
            addAttributeNoCRC(writer, "SesVendorID", sesVendorID);
        }

        @Override
        protected boolean hasChildren() {
            return !tempSensors.isEmpty();
        }

        @Override
        protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
            super.marshallChildrenToFileAndCRC(cache, writer, tabs);
            // children martial themselves
            addAllChildrenUnderNodeAndCRC(cache, writer, CModeCoolingElements.class.getSimpleName(), coolingElements.values(), tabs);
            addAllChildrenUnderNodeAndCRC(cache, writer, CModePowerSupplies.class.getSimpleName(), powerSupplies.values(), tabs);
            addAllChildrenUnderNodeAndCRC(cache, writer, CModeTempSensors.class.getSimpleName(), tempSensors.values(), tabs);
            addAllChildrenUnderNodeAndCRC(cache, writer, CModeSensors.class.getSimpleName(), sensors.values(), tabs);
            addAllChildrenUnderNodeAndCRC(cache, writer, CModeShelfCables.class.getSimpleName(), shelfCables.values(), tabs);
        }
    }

    public static final class CModeCoolingElements extends NodeMap<CModeStorageShelfEnv, UUIDKey, CModeCoolingElement> {
        CModeCoolingElements(final CModeStorageShelfEnv modelParent) {
            super(modelParent);
        }

        public boolean add(final CModeCoolingElement coolingElement) {
            return (put(coolingElement.getKey(), coolingElement) == null);
        }
    }

    public static final class CModeCoolingElement extends BaseCRCXMLNode<CModeStorageShelfEnv> {

        private final UUIDKey key;
        private Boolean isNotInstalled;
        private Boolean isError;

        CModeCoolingElement(final CModeStorageShelfEnv parent, final String coolingElementID) {
            super(parent, CModeCoolingElement.class.getSimpleName());
            this.key = new UUIDKey("cooling_element", coolingElementID);
        }

        @Override
        @SuppressWarnings({"unchecked"})
        public UUIDKey getKey() {
            return key;
        }

        public Boolean isNotInstalled() {
            return isNotInstalled;
        }

        public void setNotInstalled(final Boolean notInstalled) {
            isNotInstalled = notInstalled;
        }

        public Boolean isError() {
            return isError;
        }

        public void setError(final Boolean error) {
            isError = error;
        }

        @Override
        protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
            super.marshallAttributes(marshallingCache, writer);
            addAttributeNoCRC(writer, "CoolingElementNo", key.getID());
            addAttributeNoCRC(writer, "IsNotInstalled", isNotInstalled);
            addAttributeNoCRC(writer, "IsError", isError);
        }
    }

    public static final class CModePowerSupplies extends NodeMap<CModeStorageShelfEnv, UUIDKey, CModePowerSupply> {
        CModePowerSupplies(final CModeStorageShelfEnv modelParent) {
            super(modelParent);
        }

        public boolean add(final CModePowerSupply powerSupply) {
            return (put(powerSupply.getKey(), powerSupply) == null);
        }
    }

    public static final class CModePowerSupply extends BaseCRCXMLNode<CModeStorageShelfEnv> {

        private final UUIDKey key;
        private Boolean isNotInstalled;
        private Boolean isError;

        CModePowerSupply(final CModeStorageShelfEnv parent, final String powerSupplyNo) {
            super(parent, CModeTempSensor.class.getSimpleName());
            this.key = new UUIDKey("power_supply", powerSupplyNo);
        }

        @Override
        @SuppressWarnings({"unchecked"})
        public UUIDKey getKey() {
            return key;
        }

        public Boolean isNotInstalled() {
            return isNotInstalled;
        }

        public void setNotInstalled(final Boolean notInstalled) {
            isNotInstalled = notInstalled;
        }

        public Boolean isError() {
            return isError;
        }

        public void setError(final Boolean error) {
            isError = error;
        }


        @Override
        protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
            super.marshallAttributes(marshallingCache, writer);
            addAttributeNoCRC(writer, "PowerSupplyNo", key.getID());
            addAttributeNoCRC(writer, "IsError", isError);
            addAttributeNoCRC(writer, "IsNotInstalled", isNotInstalled);
        }
    }

    public static final class CModeTempSensors extends NodeMap<CModeStorageShelfEnv, UUIDKey, CModeTempSensor> {
        CModeTempSensors(final CModeStorageShelfEnv modelParent) {
            super(modelParent);
        }

        public boolean add(final CModeTempSensor sensor) {
            return (put(sensor.getKey(), sensor) == null);
        }
    }

    public static final class CModeTempSensor extends BaseCRCXMLNode<CModeStorageShelfEnv> {

        private final UUIDKey key;
        private Boolean isNotInstalled;
        private String currentCondition;
        private Integer currentTemp;
        private Integer hiCritical;
        private Integer hiWarning;
        private Boolean isAmbient;
        private Boolean isError;
        private Integer lowCritical;
        private Integer lowWarning;

        CModeTempSensor(final CModeStorageShelfEnv parent, final String sensorNo) {
            super(parent, CModeTempSensor.class.getSimpleName());
            this.key = new UUIDKey("temp_sensor", sensorNo);
        }

        @Override
        @SuppressWarnings({"unchecked"})
        public UUIDKey getKey() {
            return key;
        }

        public Boolean isNotInstalled() {
            return isNotInstalled;
        }

        public void setNotInstalled(final Boolean notInstalled) {
            isNotInstalled = notInstalled;
        }

        public void setCurrentCondition(final String currentCondition) {
            this.currentCondition = currentCondition;
        }


        public void setCurrentTemp(final Integer currentTemp) {
            this.currentTemp = currentTemp;
        }

        public void setHiCritical(final Integer hiCritical) {
            this.hiCritical = hiCritical;
        }

        public void setHiWarning(final Integer hiWarning) {
            this.hiWarning = hiWarning;
        }

        public void setAmbient(final Boolean ambient) {
            isAmbient = ambient;
        }

        public Boolean isError() {
            return isError;
        }

        public void setError(final Boolean error) {
            isError = error;
        }

        public void setLowCritical(final Integer lowCritical) {
            this.lowCritical = lowCritical;
        }

        public void setLowWarning(final Integer lowWarning) {
            this.lowWarning = lowWarning;
        }

        @Override
        protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
            super.marshallAttributes(marshallingCache, writer);
            addAttributeNoCRC(writer, "SensorNo", key.getID());
            addAttributeNoCRC(writer, "IsNotInstalled", isNotInstalled);
            addAttributeNoCRC(writer, "CurrentCondition", currentCondition);
            addAttributeNoCRC(writer, "CurrentTemp", currentTemp);
            addAttributeNoCRC(writer, "HiCritical", hiCritical);
            addAttributeNoCRC(writer, "HiWarning", hiWarning);
            addAttributeNoCRC(writer, "IsAmbient", isAmbient);
            addAttributeNoCRC(writer, "IsError", isError);
            addAttributeNoCRC(writer, "LowCritical", lowCritical);
            addAttributeNoCRC(writer, "LowWarning", lowWarning);
        }
    }


    public static final class CModeSensors extends NodeMap<CModeStorageShelfEnv, CModeSensor.SensorKey, CModeSensor> {
        CModeSensors(final CModeStorageShelfEnv modelParent) {
            super(modelParent);
        }

        public boolean add(final CModeSensor sensor) {
            return (put(sensor.getKey(), sensor) == null);
        }
    }

    public static final class CModeSensor extends BaseCRCXMLNode<CModeStorageShelfEnv> {

        public final static class SensorKey extends ModelKeyBase<SensorKey.Field> {
            public static enum Field {
                sensor_type,
                sensor_no,
            }

            public SensorKey(final String sensorType, final String sensorNo) {
                super("net_route", new EnumMap<Field, String>(Field.class) {{
                    put(Field.sensor_type, sensorType);
                    put(Field.sensor_no, sensorNo);
                }});
            }
        }

        public static enum Type {
            voltage,
            current,
        }

        private final SensorKey key;
        private final Type type;
        private Boolean sensorError;
        private Boolean isSensorNotInstalled;
        private String sensorCondition;
        private String sensorValueUnits;

        CModeSensor(final CModeStorageShelfEnv parent, final Type type, final String sensorNo) {
            super(parent, CModeSensor.class.getSimpleName());
            this.key = new SensorKey(type.name(), sensorNo);
            this.type = type;
        }

        @Override
        @SuppressWarnings({"unchecked"})
        public SensorKey getKey() {
            return key;
        }

        public Type getType() {
            return type;
        }

        public Boolean isSensorNotInstalled() {
            return isSensorNotInstalled;
        }

        public void setSensorNotInstalled(final Boolean sensorNotInstalled) {
            isSensorNotInstalled = sensorNotInstalled;
        }

        public Boolean isSensorError() {
            return sensorError;
        }

        public void setSensorError(final Boolean sensorError) {
            this.sensorError = sensorError;
        }

        public void setSensorCondition(final String sensorCondition) {
            this.sensorCondition = sensorCondition;
        }

        public void setSensorValueUnits(final String sensorValueUnits) {
            this.sensorValueUnits = sensorValueUnits;
        }

        @Override
        protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
            super.marshallAttributes(marshallingCache, writer);
            addAttributeNoCRC(writer, "SensorID", key.getID());
            addAttributeNoCRC(writer, "SensorType", type);
            addAttributeNoCRC(writer, "IsSensorNotInstalled", isSensorNotInstalled);
            addAttributeNoCRC(writer, "SensorError", sensorError);
            addAttributeNoCRC(writer, "SensorCondition", sensorCondition);
            addAttributeNoCRC(writer, "SensorValueUnits", sensorValueUnits);
        }
    }

    public static final class CModeShelfCables extends NodeMap<CModeStorageShelfEnv, CModeShelfCable.ShelfCableKey, CModeShelfCable> {
        CModeShelfCables(final CModeStorageShelfEnv modelParent) {
            super(modelParent);
        }

        public boolean add(final CModeShelfCable shelfCable) {
            return (put(shelfCable.getKey(), shelfCable) == null);
        }
    }

    @SuppressWarnings("UnusedDeclaration")
    public static final class CModeShelfCable extends BaseCRCXMLNode<CModeStorageShelfEnv> {
        private final ShelfCableKey key;
        private String shelfId;
        private String channelName;
        private Boolean isShelfChannelFailure;
        private String endIdentifier;
        private String length;
        private String manufacturer;
        private String partNumber;
        private String serialNumber;
        private String attachedSerialNumber;
        private String technology;
        private String type;
        private String connectorDesignator;
        private Integer connectorNumber;
        private Boolean isCableConnected;
        private Boolean isConnectorError;
        private String shelfModule;
        private Integer swapCount;

        CModeShelfCable(final CModeStorageShelfEnv parent, final String nodeName, final String shelfId, final String channelName,
                        final String partNumber, final String serialNumber, final Integer connectorNumber) {
            super(parent, CModeShelfCable.class.getSimpleName());
            this.partNumber = partNumber;
            this.serialNumber = serialNumber;
            this.shelfId = shelfId;
            this.channelName = channelName;
            this.connectorNumber = connectorNumber;
            this.key = new ShelfCableKey(nodeName, shelfId, channelName, partNumber == null? StringUtils.EMPTY: partNumber,
                    serialNumber == null ? StringUtils.EMPTY: serialNumber, String.valueOf(connectorNumber));
        }

        public final static class ShelfCableKey extends ModelKeyBase<ShelfCableKey.Field> {
            public static enum Field {
                node_name,
                shelf_id,
                channel_name,
                part_number,
                serial_number,
                connector_number
            }

            public ShelfCableKey(final String nodeName, final String shelfId, final String channelName, final String partNumber,
                                 final String serialNumber, final String connectorNumber) {
                super("shelf_cable", new EnumMap<Field, String>(Field.class) {{
                    put(Field.node_name, nodeName);
                    put(Field.shelf_id, shelfId);
                    put(Field.channel_name, channelName);
                    put(Field.part_number, partNumber);
                    put(Field.serial_number, serialNumber);
                    put(Field.connector_number, connectorNumber);

                }}, EnumSet.of(Field.part_number, Field.serial_number));
            }
        }

        public static enum Type {
            voltage,
            current,
        }

        @SuppressWarnings("unchecked")
        public ShelfCableKey getKey() {
            return key;
        }

        public String getShelfNodeName() {
            return key.getID(ShelfCableKey.Field.node_name);
        }

        public String getEndIdentifier() {
            return endIdentifier;
        }

        public void setEndIdentifier(final String endIdentifier) {
            this.endIdentifier = endIdentifier;
        }

        public String getLength() {
            return length;
        }

        public void setLength(final String length) {
            this.length = length;
        }

        public String getManufacturer() {
            return manufacturer;
        }

        public void setManufacturer(final String manufacturer) {
            this.manufacturer = manufacturer;
        }

        public String getPartNumber() {
            return partNumber;
        }

        public void setPartNumber(final String partNumber) {
            this.partNumber = partNumber;
        }

        public String getSerialNumber() {
            return serialNumber;
        }

        public void setSerialNumber(final String serialNumber) {
            this.serialNumber = serialNumber;
        }

        public String getAttachedSerialNumber() {
            return attachedSerialNumber;
        }

        public void setAttachedSerialNumber(final String attachedSerialNumber) {
            this.attachedSerialNumber = attachedSerialNumber;
        }

        public String getTechnology() {
            return technology;
        }

        public void setTechnology(final String technology) {
            this.technology = technology;
        }

        public String getType() {
            return type;
        }

        public void setType(final String type) {
            this.type = type;
        }

        public String getConnectorDesignator() {
            return connectorDesignator;
        }

        public void setConnectorDesignator(final String connectorDesignator) {
            this.connectorDesignator = connectorDesignator;
        }

        public Integer getConnectorNumber() {
            return connectorNumber;
        }

        public void setConnectorNumber(final Integer connectorNumber) {
            this.connectorNumber = connectorNumber;
        }

        public Boolean getCableConnected() {
            return isCableConnected;
        }

        public void setCableConnected(final Boolean cableConnected) {
            isCableConnected = cableConnected;
        }

        public Boolean getConnectorError() {
            return isConnectorError;
        }

        public void setConnectorError(final Boolean connectorError) {
            isConnectorError = connectorError;
        }

        public Integer getSwapCount() {
            return swapCount;
        }

        public void setSwapCount(final Integer swapCount) {
            this.swapCount = swapCount;
        }


        public String getShelfId() {
            return shelfId;
        }

        public void setShelfId(final String shelfId) {
            this.shelfId = shelfId;
        }

        public String getChannelName() {
            return channelName;
        }

        public void setChannelName(final String channelName) {
            this.channelName = channelName;
        }

        public Boolean getShelfChannelFailure() {
            return isShelfChannelFailure;
        }

        public void setShelfChannelFailure(final Boolean shelfChannelFailure) {
            isShelfChannelFailure = shelfChannelFailure;
        }

        public String getShelfModule() {
            return shelfModule;
        }

        public void setShelfModule(final String shelfModule) {
            this.shelfModule = shelfModule;
        }

        @Override
        protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
            super.marshallAttributes(marshallingCache, writer);
            addAttributeNoCRC(writer, "key", key.getOcieUUID());
            addAttributeNoCRC(writer, "ShelfId", shelfId);
            addAttributeNoCRC(writer, "ChannelName", channelName);
            addAttributeNoCRC(writer, "SwapCount", swapCount);
            addAttributeNoCRC(writer, "ShelfModule", shelfModule);
            addAttributeNoCRC(writer, "IsConnectorError", isConnectorError);
            addAttributeNoCRC(writer, "IsCableConnected", isCableConnected);
            addAttributeNoCRC(writer, "ConnectorNumber", connectorNumber);
            addAttributeNoCRC(writer, "ConnectorDesignator", connectorDesignator);
            addAttributeNoCRC(writer, "Type", type);
            addAttributeNoCRC(writer, "Technology", technology);
            addAttributeNoCRC(writer, "SerialNumber", serialNumber);
            addAttributeNoCRC(writer, "AttachedSerialNumber", attachedSerialNumber);
            addAttributeNoCRC(writer, "PartNumber", partNumber);
            addAttributeNoCRC(writer, "Manufacturer", manufacturer);
            addAttributeNoCRC(writer, "Length", length);
            addAttributeNoCRC(writer, "EndIdentifier", endIdentifier);
            addAttributeNoCRC(writer, "IsShelfChannelFailure", isShelfChannelFailure);
        }
    }
}



