package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import org.apache.log4j.Logger;

import java.io.IOException;

public class CModeVServerDNSInfo extends BaseCRCXMLNode<CModeCluster> {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(CModeVServerDNSInfo.class);

    private final UUIDKey key;
    private EnumWrapper<VServer.DNSState> dnsState;
    private String domains;
    private String nameServers;

    CModeVServerDNSInfo(final CModeCluster parent, final String vServerName) {
        super(parent, CModeVServerDNSInfo.class.getSimpleName());
        this.key = new UUIDKey("dns_info", vServerName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    public EnumWrapper<VServer.DNSState> getDnsState() {
        return dnsState;
    }

    public void setDnsState(final EnumWrapper<VServer.DNSState> dnsState) {
        this.dnsState = dnsState;
    }

    public void setDomainNames(final String domainNames) {
        domains = domainNames;
    }

    public String getDomains() {
        return domains;
    }

    public void setNameServers(final String nameServerIPs) {
        nameServers = nameServerIPs;
    }

    public String getNameServers() {
        return nameServers;
    }

    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "VServerName", key.getName());
        addAttributeNoCRC(writer, "DNSState", dnsState.getEnumValue());
        addAttributeNoCRC(writer, "DNSStateRaw", dnsState.getRawValue());
        addAttributeNoCRC(writer, "Domains", domains);
        addAttributeNoCRC(writer, "NameServers", nameServers);
    }
}
