package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;

public class CModeVServerNISInfo extends BaseCRCXMLNode<CModeCluster> {

    private final NISInfoKey key;

    private Boolean isActive;
    private String nisServerIPs;

    CModeVServerNISInfo(final CModeCluster parent, final String vServerName, final String nisDomainName) {
        super(parent, CModeVServerNISInfo.class.getSimpleName());
        this.key = new NISInfoKey(vServerName, nisDomainName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public NISInfoKey getKey() {
        return key;
    }

    public Boolean isActive() {
        return isActive;
    }

    public void setActive(final Boolean active) {
        isActive = active;
    }


    public void setNisServerIPs(final String nisServerIPs) {
        this.nisServerIPs = nisServerIPs;
    }

    public String getNisServerIPs() {
        return nisServerIPs;
    }

    public final static class NISInfoKey extends ModelKeyBase<NISInfoKey.Field> {
        public static enum Field {
            vserver_name,
            nis_domain_name
        }

        public NISInfoKey(final String vServerName, final String nisDomainName) {
            super("nis_domain", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.nis_domain_name, nisDomainName);
            }}, EnumSet.of(Field.nis_domain_name));                      // These EnumSet<E> emptyAbleFields fields are allowed to be empty but not null
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "VServerName", key.getID(NISInfoKey.Field.vserver_name));
        addAttributeNoCRC(writer, "NISDomain", key.getID(NISInfoKey.Field.nis_domain_name));
        addAttributeNoCRC(writer, "IsActive", isActive);
        addAttributeNoCRC(writer, "NISServerIPs", nisServerIPs);
    }
}
