package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;


import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServerNameMapping;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeVServerNameMapping extends BaseCRCXMLNode<CModeCluster> {

    public final static class MappingKey extends ModelKeyBase<MappingKey.Field> {
        public static enum Field {
            vserver_name,
            position,
            direction
        }

        public MappingKey(final String vServerName, final String position, final String direction) {
            super("vserver_name_mapping", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.position, position);
                put(Field.direction, direction);
            }});
        }
    }

    private final MappingKey key;
    private final Integer position;

    private EnumWrapper<VServerNameMapping.Direction> directionWrapper;
    private String pattern;
    private String replacement;

    CModeVServerNameMapping(final CModeCluster parent, final String vServerName, final Integer position, final String direction) {
        super(parent, CModeVServerNameMapping.class.getSimpleName());
        this.key = new MappingKey(vServerName, String.valueOf(position), direction);
        this.position = position;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public MappingKey getKey() {
        return key;
    }

    public Integer getPosition() {
        return position;
    }

    public EnumWrapper<VServerNameMapping.Direction> getDirectionWrapper() {
        return directionWrapper;
    }

    public void setDirectionWrapper(final EnumWrapper<VServerNameMapping.Direction> directionWrapper) {
        this.directionWrapper = directionWrapper;
    }

    public String getPattern() {
        return pattern;
    }

    public void setPattern(final String pattern) {
        this.pattern = pattern;
    }

    public String getReplacement() {
        return replacement;
    }

    public void setReplacement(final String replacement) {
        this.replacement = replacement;
    }



    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addAttributeNoCRC(writer, "VServerName", key.getID(MappingKey.Field.vserver_name));
        addAttributeNoCRC(writer, "Position", position);
        addAttributeNoCRC(writer, "Direction", directionWrapper.getRawValue());
        addAttributeNoCRC(writer, "Pattern", pattern);
        addAttributeNoCRC(writer, "Replacement", replacement);
    }
}