package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Volume;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;

public class CModeVolume extends BaseCRCXMLNode<CModeCluster> {
    static public final String CREATED_VOLUME_UUID_PREFIX = "NETAPP_OCIE_VOLUME_UUID";

    // volume-id-attributes from volume-get-iter
    private final UUIDKey key;                              // uuid
    private String instanceUUID = null;                     // instance-uuid
    private String volumeName = null;                       // name
    private String aggregateUUID = null;                    // containing-aggregate-uuid
    private String vServerUUID = null;                      // owning-vserver-uuid
    private String junctionPath = null;                     // junction-path
    private EnumWrapper<Volume.VolumeStyle> volumeStyle = null; // style
    private EnumWrapper<Volume.VolumeType> volumeType = null; // type
    /**
     * The name of the parent volume that contains the junction inode of this volume.
     * The junction parent volume must belong to the same vserver that owns this volume.
     */
    private String junctionParentName = null;               // junction-parent-name
    private String qosPolicyGroupName = null;               // qos-policy-group-name


    // volume-infinivol-attributes from volume-get-iter
    private String storageServiceName;
    private Boolean enableSnapDiff;
    private Boolean isManagedByService;
    private Long maxDataConstituentSize;
    private Long maxNamespaceConstituentSize;
    private EnumWrapper<Volume.ConstituentRole> constituentRole;


    // volume-state-attributes from volume-get-iter
    private Boolean isVServerRoot = null;                   // is-node-root
    private EnumWrapper<Volume.State> state = null;         // state
    private Boolean isJunctionActive = null;                // is-junction-active
    private Boolean isConstituent;                          // is-constituent

    // volume-sis-attributes from volume-get-iter
    private Boolean isSISVolume = null;                     // is-sis-volume
    private Long deduplicationSpaceSaved = null;            // deduplication-space-saved
    private Long compressionSpaceSaved = null;              // compression-space-saved
    private Integer percentageDeduplicationSpaceSaved = null; // percentage-deduplication-space-saved
    private Integer percentageCompressionSpaceSaved = null; // percentage-compression-space-saved

    // volume-mirror-attributes from volume-get-iter
    private Boolean isDataProtectionMirror = null;          // is-data-protection-mirror
    private Boolean isLoadSharingMirror = null;             // is-load-sharing-mirror
    private Boolean isMoveMirror = null;                    // is-move-mirror
    private Boolean isReplicaVolume = null;                 // is-replica-volume

    // volume-space-attributes from volume-get-iter
    private Long size = null;                               // size
    private Long sizeTotal = null;                          // size-total
    private Long sizeUsed = null;                           // size-used
    private Long sizeAvail = null;                          // size-available
    private Long overwriteReserve = null;                   // overwrite-reserve
    private Long overwriteReserveRequired = null;           // overwrite-reserve-required
    private Long overwriteReserveUsed = null;               // overwrite-reserve-used
    private Long overwriteReserveActualUsed = null;         // overwrite-reserve-used-actual
    private Long snapshotReserveSize = null;                // snapshot-reserve-size
    private Integer percentSnapshotReserve = null;          // percentage-snapshot-reserve
    private Integer percentSnapshotReserveUsed = null;      // percentage-snapshot-reserve-used
    private Long sizeAvailableForSnapshots = null;          // size-available-for-snapshots
    private Integer percentFractionalReserve = null;        // percentage-fractional-reserve
    private Integer percentSizeUsed = null;                 // percentage-size-used
    private Integer percentSizeAvail = null;                // 100 - percentage-size-used
    private Boolean isSpaceGuarantee = null;                // is-space-guarantee-enabled
    private EnumWrapper<Volume.SpaceMgmtOptionTryFirst> spaceMgmtTryFirst = null;// space-mgmt-option-try-first
    private Long sizeUsedBySnapshots = null;                // size-used-by-snapshots
    private EnumWrapper<Volume.SpaceGuarantee> spaceGuarantee = null; // space-guarantee

    // volume-security-attributes from volume-get-iter
    private EnumWrapper<Volume.SecurityStyle> securityStyle = null;      // style
    private String groupID = null;
    private String permissions = null;
    private String userID = null;


    // volume-hybrid-cache-attributes from volume-get-iter
    private EnumWrapper<Volume.HybridCacheEligibility> hybridCacheEligibility = null; // eligibility

    //volume-clone-attributes, volume-clone-parent-attributes from volume-get-iter
    private String cloneParentName = null;                  // name

    // volume-vm-align-attributes
    private Integer vmAlignSector = null;                   // vm-align-sector
    private String vmAlignSuffix = null;                    // vm-align-suffix

    // volume-export-attributes
    private String exportPolicy = null;                     // policy

    // volume-directory-attributes
    private Boolean i2pEnabled = null;                      // i2p-enabled

    // volume-snapshot-attributes
    private Boolean autoSnapshotsEnabled = null;            // auto-snapshots-enabled
    private Boolean snapdirAccessEnabled = null;            // snapdir-access-enabled
    private Boolean snapshotCloneDependencyEnabled = null;  // snapshot-clone-dependency-enabled
    private Long snapshotCount = null;                      // snapshot-count
    private String snapshotPolicy = null;                   // snapshot-policy

    // volume-autosize-attributes
    private EnumWrapper<Volume.VolumeAutoSizeMode> autoSizeMode = null;  // mode
    private Long maxAutosizeSize = null;                    // maximum-size
    private Long autosizeIncrementSize = null;              // increment-size
    private Boolean isEnabled;                              // is-enabled

    // volume-performance-attributes
    private Boolean isAtimeUpdateEnabled = null;            // is-atime-update-enabled

    // volume-inode-attributes
    private EnumWrapper<Volume.BlockType> blockType = null; // block-type
    private Long inodeFilesTotal = null;                    // files-total
    private Long inodeFilesUsed = null;                     // files-used

    // volume-language-attributes
    private String languageCode = null;                     // language-code
    private Boolean isCreateUcodeEnabled = null;            // is-create-ucode-enabled
    private Boolean isConvertUcodeEnabled = null;           // is-convert-ucode-enabled

    // volume-clone-attributes
    private Integer cloneChildCount = null;                 // clone-child-count

    // volume-snapshot-autodelete-attributes
    private Boolean isAutoDeleteEnabled = null;             // is-autodelete-enabled
    private EnumWrapper<Volume.Commitment> commitment = null; // commitment
    private EnumWrapper<Volume.DeleteOrder> deleteOrder = null; // delete-order
    private EnumWrapper<Volume.DeferDelete> deferDelete = null; // defer-delete
    private Integer targetFreeSpace = null;                 // target-free-space
    private EnumWrapper<Volume.Trigger> trigger = null;     // trigger
    private String prefix = null;                           // prefix
    private EnumSet<Volume.DestroyList> destroyList = EnumSet.noneOf(Volume.DestroyList.class);
    // destroy-list
    private String destroyListRaw = null;                   // destroy-list

    // volume-flexcache-attributes
    private Long flexCacheMinReserve = null;                // min-reserve
    private String flexCacheOriginVolumeName = null;        // origin

    // volume-attributes.volume-hybrid-cache-attributes.caching-policy
    private String cachingPolicyRaw = null;
    private Volume.CachingPolicy cachingPolicy = Volume.CachingPolicy.NOT_MAPPED;

    CModeVolume(final CModeCluster parent, final String uuid) {
        super(parent, CModeVolume.class.getSimpleName());
        this.key = new UUIDKey("volume", uuid);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    public String getInstanceUUID() {
        return instanceUUID;
    }

    public void setInstanceUUID(final String instanceUUID) {
        this.instanceUUID = instanceUUID;
    }

    public String getVolumeName() {
        return volumeName;
    }

    public void setVolumeName(final String volumeName) {
        this.volumeName = volumeName;
    }

    public String getAggregateUUID() {
        return aggregateUUID;
    }

    public void setAggregateUUID(final String aggregateUUID) {
        this.aggregateUUID = aggregateUUID;
    }

    public String getVServerUUID() {
        return vServerUUID;
    }

    public void setVServerUUID(final String vServerUUID) {
        this.vServerUUID = vServerUUID;
    }

    public String getJunctionPath() {
        return junctionPath;
    }

    public void setJunctionPath(final String junctionPath) {
        this.junctionPath = junctionPath;
    }

    public EnumWrapper<Volume.VolumeStyle> getVolumeStyle() {
        return volumeStyle;
    }

    public void setVolumeStyle(final EnumWrapper<Volume.VolumeStyle> volumeStyle) {
        this.volumeStyle = volumeStyle;
    }

    public EnumWrapper<Volume.VolumeType> getVolumeType() {
        return volumeType;
    }

    public void setVolumeType(final EnumWrapper<Volume.VolumeType> volumeType) {
        this.volumeType = volumeType;
    }

    public String getJunctionParentName() {
        return junctionParentName;
    }

    public void setJunctionParentName(final String junctionParentName) {
        this.junctionParentName = junctionParentName;
    }

    public String getQosPolicyGroupName() {
        return qosPolicyGroupName;
    }

    public void setQosPolicyGroupName(final String qosPolicyGroupName) {
        this.qosPolicyGroupName = qosPolicyGroupName;
    }

    public String getStorageServiceName() {
        return storageServiceName;
    }

    public void setStorageServiceName(final String storageServiceName) {
        this.storageServiceName = storageServiceName;
    }

    public Boolean isEnableSnapDiff() {
        return enableSnapDiff;
    }

    public void setEnableSnapDiff(final Boolean enableSnapDiff) {
        this.enableSnapDiff = enableSnapDiff;
    }

    public Boolean isManagedByService() {
        return isManagedByService;
    }

    public void setManagedByService(final Boolean managedByService) {
        isManagedByService = managedByService;
    }

    public Long getMaxDataConstituentSize() {
        return maxDataConstituentSize;
    }

    public void setMaxDataConstituentSize(final Long maxDataConstituentSize) {
        this.maxDataConstituentSize = maxDataConstituentSize;
    }

    public Long getMaxNamespaceConstituentSize() {
        return maxNamespaceConstituentSize;
    }

    public void setMaxNamespaceConstituentSize(final Long maxNamespaceConstituentSize) {
        this.maxNamespaceConstituentSize = maxNamespaceConstituentSize;
    }

    public EnumWrapper<Volume.ConstituentRole> getConstituentRole() {
        return constituentRole;
    }

    public void setConstituentRole(final EnumWrapper<Volume.ConstituentRole> constituentRole) {
        this.constituentRole = constituentRole;
    }

    public Boolean getConstituent() {
        return isConstituent;
    }

    public void setConstituent(final Boolean constituent) {
        isConstituent = constituent;
    }

    public Boolean isVServerRoot() {
        return isVServerRoot;
    }

    public void setVServerRoot(final Boolean VServerRoot) {
        isVServerRoot = VServerRoot;
    }

    public EnumWrapper<Volume.State> getState() {
        return state;
    }

    public void setState(final EnumWrapper<Volume.State> state) {
        this.state = state;
    }

    public Boolean isJunctionActive() {
        return isJunctionActive;
    }

    public void setJunctionActive(final Boolean junctionActive) {
        isJunctionActive = junctionActive;
    }

    public Boolean isSISVolume() {
        return isSISVolume;
    }

    public void setSISVolume(final Boolean SISVolume) {
        isSISVolume = SISVolume;
    }

    public Long getDeduplicationSpaceSaved() {
        return deduplicationSpaceSaved;
    }

    public void setDeduplicationSpaceSaved(final Long deduplicationSpaceSaved) {
        this.deduplicationSpaceSaved = deduplicationSpaceSaved;
    }

    public Long getCompressionSpaceSaved() {
        return compressionSpaceSaved;
    }

    public void setCompressionSpaceSaved(final Long compressionSpaceSaved) {
        this.compressionSpaceSaved = compressionSpaceSaved;
    }

    public Integer getPercentageDeduplicationSpaceSaved() {
        return percentageDeduplicationSpaceSaved;
    }

    public void setPercentageDeduplicationSpaceSaved(final Integer percentageDeduplicationSpaceSaved) {
        this.percentageDeduplicationSpaceSaved = percentageDeduplicationSpaceSaved;
    }

    public Integer getPercentageCompressionSpaceSaved() {
        return percentageCompressionSpaceSaved;
    }

    public void setPercentageCompressionSpaceSaved(final Integer percentageCompressionSpaceSaved) {
        this.percentageCompressionSpaceSaved = percentageCompressionSpaceSaved;
    }

    public Boolean isDataProtectionMirror() {
        return isDataProtectionMirror;
    }

    public void setDataProtectionMirror(final Boolean dataProtectionMirror) {
        isDataProtectionMirror = dataProtectionMirror;
    }

    public Boolean isLoadSharingMirror() {
        return isLoadSharingMirror;
    }

    public void setLoadSharingMirror(final Boolean loadSharingMirror) {
        isLoadSharingMirror = loadSharingMirror;
    }

    public Boolean isMoveMirror() {
        return isMoveMirror;
    }

    public void setMoveMirror(final Boolean moveMirror) {
        isMoveMirror = moveMirror;
    }

    public Boolean isReplicaVolume() {
        return isReplicaVolume;
    }

    public void setReplicaVolume(final Boolean replicaVolume) {
        isReplicaVolume = replicaVolume;
    }

    public Long getSize() {
        return size;
    }

    public void setSize(final Long size) {
        this.size = size;
    }

    public Long getSizeTotal() {
        return sizeTotal;
    }

    public void setSizeTotal(final Long sizeTotal) {
        this.sizeTotal = sizeTotal;
    }

    public Long getSizeUsed() {
        return sizeUsed;
    }

    public void setSizeUsed(final Long sizeUsed) {
        this.sizeUsed = sizeUsed;
    }

    public Long getSizeAvail() {
        return sizeAvail;
    }

    public void setSizeAvail(final Long sizeAvail) {
        this.sizeAvail = sizeAvail;
    }

    public Long getOverwriteReserve() {
        return overwriteReserve;
    }

    public void setOverwriteReserve(final Long overwriteReserve) {
        this.overwriteReserve = overwriteReserve;
    }

    public Long getOverwriteReserveRequired() {
        return overwriteReserveRequired;
    }

    public void setOverwriteReserveRequired(final Long overwriteReserveRequired) {
        this.overwriteReserveRequired = overwriteReserveRequired;
    }

    public Long getOverwriteReserveUsed() {
        return overwriteReserveUsed;
    }

    public void setOverwriteReserveUsed(final Long overwriteReserveUsed) {
        this.overwriteReserveUsed = overwriteReserveUsed;
    }

    public Long getOverwriteReserveActualUsed() {
        return overwriteReserveActualUsed;
    }

    public void setOverwriteReserveActualUsed(final Long overwriteReserveActualUsed) {
        this.overwriteReserveActualUsed = overwriteReserveActualUsed;
    }

    public Long getSnapshotReserveSize() {
        return snapshotReserveSize;
    }

    public void setSnapshotReserveSize(final Long snapshotReserveSize) {
        this.snapshotReserveSize = snapshotReserveSize;
    }

    public Integer getPercentSnapshotReserve() {
        return percentSnapshotReserve;
    }

    public void setPercentSnapshotReserve(final Integer percentSnapshotReserve) {
        this.percentSnapshotReserve = percentSnapshotReserve;
    }

    public Integer getPercentSnapshotReserveUsed() {
        return percentSnapshotReserveUsed;
    }

    public void setPercentSnapshotReserveUsed(final Integer percentSnapshotReserveUsed) {
        this.percentSnapshotReserveUsed = percentSnapshotReserveUsed;
    }

    public Long getSizeAvailableForSnapshots() {
        return sizeAvailableForSnapshots;
    }

    public void setSizeAvailableForSnapshots(final Long sizeAvailableForSnapshots) {
        this.sizeAvailableForSnapshots = sizeAvailableForSnapshots;
    }

    public Integer getPercentFractionalReserve() {
        return percentFractionalReserve;
    }

    public void setPercentFractionalReserve(final Integer percentFractionalReserve) {
        this.percentFractionalReserve = percentFractionalReserve;
    }

    public Integer getPercentSizeUsed() {
        return percentSizeUsed;
    }

    public void setPercentSizeUsed(final Integer percentSizeUsed) {
        this.percentSizeUsed = percentSizeUsed;
        if (percentSizeUsed != null) {
            this.percentSizeAvail = 100 - percentSizeUsed;
        }
    }

    //    public Integer getPercentSizeAvail() {
    //        return percentSizeAvail;
    //    }

    public Boolean isSpaceGuarantee() {
        return isSpaceGuarantee;
    }

    public void setIsSpaceGuarantee(final Boolean spaceGuarantee) {
        isSpaceGuarantee = spaceGuarantee;
    }

    public EnumWrapper<Volume.SpaceMgmtOptionTryFirst> getSpaceMgmtTryFirst() {
        return spaceMgmtTryFirst;
    }

    public void setSpaceMgmtTryFirst(final EnumWrapper<Volume.SpaceMgmtOptionTryFirst> spaceMgmtTryFirst) {
        this.spaceMgmtTryFirst = spaceMgmtTryFirst;
    }

    public Long getSizeUsedBySnapshots() {
        return sizeUsedBySnapshots;
    }

    public void setSizeUsedBySnapshots(final Long sizeUsedBySnapshots) {
        this.sizeUsedBySnapshots = sizeUsedBySnapshots;
    }

    public EnumWrapper<Volume.SpaceGuarantee> getSpaceGuarantee() {
        return spaceGuarantee;
    }

    public void setSpaceGuarantee(final EnumWrapper<Volume.SpaceGuarantee> spaceGuarantee) {
        this.spaceGuarantee = spaceGuarantee;
    }

    public EnumWrapper<Volume.SecurityStyle> getSecurityStyle() {
        return securityStyle;
    }

    public void setSecurityStyle(final EnumWrapper<Volume.SecurityStyle> securityStyle) {
        this.securityStyle = securityStyle;
    }

    public String getUnixSecurityPermissions() {
        return permissions;
    }

    public void setUnixSecurityPermissions(final String permissions) {
        this.permissions = permissions;
    }

    public String getUnixSecurityGroupID() {
        return groupID;
    }

    public void setUnixSecurityGroupID(final String groupID) {
        this.groupID = groupID;
    }

    public String getUnixSecurityUserID() {
        return userID;
    }

    public void setUnixSecurityUserID(final String userID) {
        this.userID = userID;
    }

    public EnumWrapper<Volume.HybridCacheEligibility> getHybridCacheEligibility() {
        return hybridCacheEligibility;
    }

    public void setHybridCacheEligibility(final EnumWrapper<Volume.HybridCacheEligibility> hybridCacheEligibility) {
        this.hybridCacheEligibility = hybridCacheEligibility;
    }

    public String getCloneParentName() {
        return cloneParentName;
    }

    public void setCloneParentName(final String cloneParentName) {
        this.cloneParentName = cloneParentName;
    }

    public Integer getVmAlignSector() {
        return vmAlignSector;
    }

    public void setVmAlignSector(final Integer vmAlignSector) {
        this.vmAlignSector = vmAlignSector;
    }

    public String getVmAlignSuffix() {
        return vmAlignSuffix;
    }

    public void setVmAlignSuffix(final String vmAlignSuffix) {
        this.vmAlignSuffix = vmAlignSuffix;
    }

    public String getExportPolicy() {
        return exportPolicy;
    }

    public void setExportPolicy(final String exportPolicy) {
        this.exportPolicy = exportPolicy;
    }

    public Boolean isI2pEnabled() {
        return i2pEnabled;
    }

    public void setI2pEnabled(final Boolean i2pEnabled) {
        this.i2pEnabled = i2pEnabled;
    }

    public Boolean isAutoSnapshotsEnabled() {
        return autoSnapshotsEnabled;
    }

    public void setAutoSnapshotsEnabled(final Boolean autoSnapshotsEnabled) {
        this.autoSnapshotsEnabled = autoSnapshotsEnabled;
    }

    public Boolean isSnapdirAccessEnabled() {
        return snapdirAccessEnabled;
    }

    public void setSnapdirAccessEnabled(final Boolean snapdirAccessEnabled) {
        this.snapdirAccessEnabled = snapdirAccessEnabled;
    }

    public Boolean isSnapshotCloneDependencyEnabled() {
        return snapshotCloneDependencyEnabled;
    }

    public void setSnapshotCloneDependencyEnabled(final Boolean snapshotCloneDependencyEnabled) {
        this.snapshotCloneDependencyEnabled = snapshotCloneDependencyEnabled;
    }

    public Long getSnapshotCount() {
        return snapshotCount;
    }

    public void setSnapshotCount(final Long snapshotCount) {
        this.snapshotCount = snapshotCount;
    }

    public String getSnapshotPolicy() {
        return snapshotPolicy;
    }

    public void setSnapshotPolicy(final String snapshotPolicy) {
        this.snapshotPolicy = snapshotPolicy;
    }

    public EnumWrapper<Volume.VolumeAutoSizeMode> getAutoSizeMode() {
        return autoSizeMode;
    }

    public void setAutoSizeMode(final EnumWrapper<Volume.VolumeAutoSizeMode> autoSizeMode) {
        this.autoSizeMode = autoSizeMode;
    }

    public Long getMaxAutosizeSize() {
        return maxAutosizeSize;
    }

    public void setMaxAutosizeSize(final Long maxAutosizeSize) {
        this.maxAutosizeSize = maxAutosizeSize;
    }

    public Long getAutosizeIncrementSize() {
        return autosizeIncrementSize;
    }

    public void setAutosizeIncrementSize(final Long autosizeIncrementSize) {
        this.autosizeIncrementSize = autosizeIncrementSize;
    }

    public Boolean getEnabled() {
        return isEnabled;
    }

    public void setEnabled(final Boolean enabled) {
        isEnabled = enabled;
    }

    public Boolean isAtimeUpdateEnabled() {
        return isAtimeUpdateEnabled;
    }

    public void setAtimeUpdateEnabled(final Boolean atimeUpdateEnabled) {
        isAtimeUpdateEnabled = atimeUpdateEnabled;
    }

    public EnumWrapper<Volume.BlockType> getBlockType() {
        return blockType;
    }

    public void setBlockType(final EnumWrapper<Volume.BlockType> blockType) {
        this.blockType = blockType;
    }

    public Long getInodeFilesTotal() {
        return inodeFilesTotal;
    }

    public void setInodeFilesTotal(final Long inodeFilesTotal) {
        this.inodeFilesTotal = inodeFilesTotal;
    }

    public Long getInodeFilesUsed() {
        return inodeFilesUsed;
    }

    public void setInodeFilesUsed(final Long inodeFilesUsed) {
        this.inodeFilesUsed = inodeFilesUsed;
    }

    public String getLanguageCode() {
        return languageCode;
    }

    public void setLanguageCode(final String languageCode) {
        this.languageCode = languageCode;
    }

    public Boolean isCreateUcodeEnabled() {
        return isCreateUcodeEnabled;
    }

    public void setCreateUcodeEnabled(final Boolean createUcodeEnabled) {
        isCreateUcodeEnabled = createUcodeEnabled;
    }

    public Boolean isConvertUcodeEnabled() {
        return isConvertUcodeEnabled;
    }

    public void setConvertUcodeEnabled(final Boolean convertUcodeEnabled) {
        isConvertUcodeEnabled = convertUcodeEnabled;
    }

    public Integer getCloneChildCount() {
        return cloneChildCount;
    }

    public void setCloneChildCount(final Integer cloneChildCount) {
        this.cloneChildCount = cloneChildCount;
    }

    public Boolean isAutoDeleteEnabled() {
        return isAutoDeleteEnabled;
    }

    public void setAutoDeleteEnabled(final Boolean autoDeleteEnabled) {
        isAutoDeleteEnabled = autoDeleteEnabled;
    }

    public EnumWrapper<Volume.Commitment> getCommitment() {
        return commitment;
    }

    public void setCommitment(final EnumWrapper<Volume.Commitment> commitment) {
        this.commitment = commitment;
    }

    public EnumWrapper<Volume.DeleteOrder> getDeleteOrder() {
        return deleteOrder;
    }

    public void setDeleteOrder(final EnumWrapper<Volume.DeleteOrder> deleteOrder) {
        this.deleteOrder = deleteOrder;
    }

    public EnumWrapper<Volume.DeferDelete> getDeferDelete() {
        return deferDelete;
    }

    public void setDeferDelete(final EnumWrapper<Volume.DeferDelete> deferDelete) {
        this.deferDelete = deferDelete;
    }

    public Integer getTargetFreeSpace() {
        return targetFreeSpace;
    }

    public void setTargetFreeSpace(final Integer targetFreeSpace) {
        this.targetFreeSpace = targetFreeSpace;
    }

    public EnumWrapper<Volume.Trigger> getTrigger() {
        return trigger;
    }

    public void setTrigger(final EnumWrapper<Volume.Trigger> trigger) {
        this.trigger = trigger;
    }

    public String getPrefix() {
        return prefix;
    }

    public void setPrefix(final String prefix) {
        this.prefix = prefix;
    }

    public EnumSet<Volume.DestroyList> getDestroyList() {
        return destroyList;
    }

    public void setDestroyList(final EnumSet<Volume.DestroyList> destroyList) {
        this.destroyList = destroyList;
    }

    public String getDestroyListRaw() {
        return destroyListRaw;
    }

    public void setDestroyListRaw(final String destroyListRaw) {
        this.destroyListRaw = destroyListRaw;
    }

    public boolean isClone() {
        return cloneParentName != null;
    }

    public Long getFlexCacheMinReserve() {
        return flexCacheMinReserve;
    }

    public void setFlexCacheMinReserve(final Long flexCacheMinReserve) {
        this.flexCacheMinReserve = flexCacheMinReserve;
    }

    public String getFlexCacheOriginVolumeName() {
        return flexCacheOriginVolumeName;
    }

    public void setFlexCacheOriginVolumeName(final String flexCacheOriginVolumeName) {
        this.flexCacheOriginVolumeName = flexCacheOriginVolumeName;
    }

    public Volume.CachingPolicy getCachingPolicy() { return cachingPolicy; }
    public void setCachingPolicy(Volume.CachingPolicy  cachingPolicy) { this.cachingPolicy = cachingPolicy; }

    public String getCachingPolicyRaw() { return cachingPolicyRaw; }
    public void setCachingPolicyRaw(String  cachingPolicyRaw) { this.cachingPolicyRaw = cachingPolicyRaw; }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "FQID", key.getID());
        addAttributeNoCRC(writer, "UUID", key.getID());
        addAttributeNoCRC(writer, "Name", volumeName);
        addAttributeNoCRC(writer, "InstanceUUID", instanceUUID);
        addAttributeNoCRC(writer, "AggregateUUID", aggregateUUID);
        addAttributeNoCRC(writer, "VServerUUID", vServerUUID);
        addAttributeNoCRC(writer, "VolType", volumeType.getEnumValue());
        addAttributeNoCRC(writer, "VolTypeRaw", volumeType.getRawValue());

        addAttributeNoCRC(writer, "StorageServiceName", storageServiceName);
        addAttributeNoCRC(writer, "EnableSnapDiff", enableSnapDiff);
        addAttributeNoCRC(writer, "IsManagedByService", isManagedByService);
        addAttributeNoCRC(writer, "MaxDataConstituentSize", maxDataConstituentSize);
        addAttributeNoCRC(writer, "MaxNamespaceConstituentSize", maxNamespaceConstituentSize);
        addAttributeNoCRC(writer, "ConstituentRole", constituentRole.getRawValue());
        addAttributeNoCRC(writer, "IsConstituent", isConstituent);

        addAttributeNoCRC(writer, "Size", size);
        addAttributeNoCRC(writer, "SizeTotal", sizeTotal);
        addAttributeNoCRC(writer, "SizeUsed", sizeUsed);
        addAttributeNoCRC(writer, "SizeAvailable", sizeAvail);
        addAttributeNoCRC(writer, "IsVserverRoot", isVServerRoot);
        addAttributeNoCRC(writer, "State", state.getEnumValue());
        addAttributeNoCRC(writer, "StateRaw", state.getRawValue());
        addAttributeNoCRC(writer, "JunctionPath", junctionPath);
        addAttributeNoCRC(writer, "IsJunctionActive", isJunctionActive);
        addAttributeNoCRC(writer, "JunctionParentName", junctionParentName);
        addAttributeNoCRC(writer, "QosPolicyGroupName", qosPolicyGroupName);
        addAttributeNoCRC(writer, "Style", volumeStyle.getEnumValue());
        addAttributeNoCRC(writer, "StyleRaw", volumeStyle.getRawValue());
        addAttributeNoCRC(writer, "IsSisVolume", isSISVolume);
        addAttributeNoCRC(writer, "IsDataProtectionMirror", isDataProtectionMirror);
        addAttributeNoCRC(writer, "IsLoadSharingMirror", isLoadSharingMirror);
        addAttributeNoCRC(writer, "IsMoveMirror", isMoveMirror);
        addAttributeNoCRC(writer, "IsReplicaVolume", isReplicaVolume);
        addAttributeNoCRC(writer, "IsSpaceGuaranteeEnabled", isSpaceGuarantee);
        addAttributeNoCRC(writer, "VMAlignSector", vmAlignSector);
        addAttributeNoCRC(writer, "VMAlignSuffix", vmAlignSuffix);
        addAttributeNoCRC(writer, "OverwriteReserve", overwriteReserve);
        addAttributeNoCRC(writer, "OverwriteReserveRequired", overwriteReserveRequired);
        addAttributeNoCRC(writer, "OverwriteReserveUsed", overwriteReserveUsed);
        addAttributeNoCRC(writer, "OverwriteReserveActualUsed", overwriteReserveActualUsed);
        addAttributeNoCRC(writer, "SnapshotReserveSize", snapshotReserveSize);
        addAttributeNoCRC(writer, "PercentageSnapshotReserve", percentSnapshotReserve);
        addAttributeNoCRC(writer, "PercentageSnapshotReserveUsed", percentSnapshotReserveUsed);
        addAttributeNoCRC(writer, "SizeAvailableForSnapshot", sizeAvailableForSnapshots);
        addAttributeNoCRC(writer, "PercentageFractionalReserve", percentFractionalReserve);
        addAttributeNoCRC(writer, "PercentageSizeUsed", percentSizeUsed);
        addAttributeNoCRC(writer, "PercentageSizeAvail", percentSizeAvail);
        addAttributeNoCRC(writer, "ExportPolicy", exportPolicy);
        addAttributeNoCRC(writer, "DeduplicationSpaceSaved", deduplicationSpaceSaved);
        addAttributeNoCRC(writer, "CompressionSpaceSaved", compressionSpaceSaved);
        addAttributeNoCRC(writer, "PercentageDeduplicationSpaceSaved", percentageDeduplicationSpaceSaved);
        addAttributeNoCRC(writer, "PercentageCompressionSpaceSaved", percentageCompressionSpaceSaved);
        addAttributeNoCRC(writer, "SecurityStyle", securityStyle.getEnumValue());
        addAttributeNoCRC(writer, "SecurityStyleRaw", securityStyle.getRawValue());
        addAttributeNoCRC(writer, "SecurityGroupID", groupID);
        addAttributeNoCRC(writer, "SecurityPermissions", permissions);
        addAttributeNoCRC(writer, "SecurityUserID", userID);
        addAttributeNoCRC(writer, "HybridCacheEligibility", hybridCacheEligibility.getEnumValue());
        addAttributeNoCRC(writer, "HybridCacheEligibilityRaw", hybridCacheEligibility.getRawValue());
        addAttributeNoCRC(writer, "CloneParentName", cloneParentName);
        addAttributeNoCRC(writer, "SpaceMgmtOptionTryFirst", spaceMgmtTryFirst.getEnumValue());
        addAttributeNoCRC(writer, "SpaceMgmtOptionTryFirstRaw", spaceMgmtTryFirst.getRawValue());
        addAttributeNoCRC(writer, "SizeUsedBySnapshots", sizeUsedBySnapshots);
        addAttributeNoCRC(writer, "SpaceGuarantee", spaceGuarantee.getEnumValue());
        addAttributeNoCRC(writer, "SpaceGuaranteeRaw", spaceGuarantee.getRawValue());
        addAttributeNoCRC(writer, "I2PEnabled", i2pEnabled);
        addAttributeNoCRC(writer, "AutoSnapshotsEnabled", autoSnapshotsEnabled);
        addAttributeNoCRC(writer, "SnapdirAccessEnabled", snapdirAccessEnabled);
        addAttributeNoCRC(writer, "IsSnapshotCloneDependencyEnabled", snapshotCloneDependencyEnabled);
        addAttributeNoCRC(writer, "SnapshotCount", snapshotCount);
        addAttributeNoCRC(writer, "SnapshotPolicy", snapshotPolicy);
        addAttributeNoCRC(writer, "AutosizeMode", autoSizeMode.getEnumValue());
        addAttributeNoCRC(writer, "AutosizeModeRaw", autoSizeMode.getRawValue());
        addAttributeNoCRC(writer, "AutosizeMaxSize", maxAutosizeSize);
        addAttributeNoCRC(writer, "IsEnabled", isEnabled);
        addAttributeNoCRC(writer, "AutosizeIncrementSize", autosizeIncrementSize);
        addAttributeNoCRC(writer, "IsATimeUpdateEnabled", isAtimeUpdateEnabled);
        addAttributeNoCRC(writer, "BlockType", Volume.BlockType.toXML(blockType.getEnumValue()));
        addAttributeNoCRC(writer, "BlockTypeRaw", blockType.getRawValue());
        addAttributeNoCRC(writer, "InodeFilesTotal", inodeFilesTotal);
        addAttributeNoCRC(writer, "InodeFilesUsed", inodeFilesUsed);
        addAttributeNoCRC(writer, "LanguageCode", languageCode);
        addAttributeNoCRC(writer, "IsCreateUCodeEnabled", isCreateUcodeEnabled);
        addAttributeNoCRC(writer, "IsConvertUCodeEnabled", isConvertUcodeEnabled);
        addAttributeNoCRC(writer, "CloneChildCount", cloneChildCount);
        addAttributeNoCRC(writer, "IsSnapshotAutoDeleteEnabled", isAutoDeleteEnabled);
        addAttributeNoCRC(writer, "SnapshotCommitment", commitment.getEnumValue());
        addAttributeNoCRC(writer, "SnapshotCommitmentRaw", commitment.getRawValue());
        addAttributeNoCRC(writer, "SnapshotDeleteOrder", deleteOrder.getEnumValue());
        addAttributeNoCRC(writer, "SnapshotDeleteOrderRaw", deleteOrder.getRawValue());
        addAttributeNoCRC(writer, "SnapshotDeferDelete", deferDelete.getEnumValue());
        addAttributeNoCRC(writer, "SnapshotDeferDeleteRaw", deferDelete.getRawValue());
        addAttributeNoCRC(writer, "SnapshotDeleteTargetFreeSpace", targetFreeSpace);
        addAttributeNoCRC(writer, "SnapshotTrigger", trigger.getEnumValue());
        addAttributeNoCRC(writer, "SnapshotTriggerRaw", trigger.getRawValue());
        addAttributeNoCRC(writer, "SnapshotPrefix", prefix);
        addAttributeNoCRC(writer, "SnapshotDestroyList", enumSetToStringList(destroyList));
        addAttributeNoCRC(writer, "SnapshotDestroyListRaw", destroyListRaw);
        addAttributeNoCRC(writer, "FlexCacheMinReserve", flexCacheMinReserve);
        addAttributeNoCRC(writer, "FlexCacheOriginVolumeName", flexCacheOriginVolumeName);
        addAttributeNoCRC(writer, "CachingPolicy", cachingPolicy);
        addAttributeNoCRC(writer, "CachingPolicyRaw", cachingPolicyRaw);
    }

    public static String enumSetToStringList(final EnumSet<?> enumSet) {
        final StringBuilder stringBuffer = new StringBuilder();
        String separator = "";
        for (final Object e : enumSet) {
            stringBuffer.append(separator).append(e);
            separator = ",";
        }
        return stringBuffer.toString();
    }

    public static final class VolumeNameKey extends ModelKeyBase<VolumeNameKey.Field> {
        public static enum Field {
            vserver_name,
            volume_name
        }

        public VolumeNameKey(final String vServerName, final String volumeName) {
            super("volume", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.volume_name, volumeName);
            }});
        }
    }
}

