package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.data;


import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.interfaces.IReAttributionContext;

import java.util.Map;

/**
 * Re Attribution context contains the foundation data required for the re attribution service.
 */
public class ReAttributionContext implements IReAttributionContext {

    /**
     * UUID of the cluster.
     */
    private String clusterUUID;
    /**
     * Map of workload node relationship uuid (node_uuid.workload_uuid) to {@link QosWorkloadNodeRelationshipAttributes}
     *
     */
     private Map<String, Map<QosWorkloadNodeRelationshipAttributes, String>> qosWorkloadNodeRelationshipMap;

    /**
     * Map of workload detail UUID (cluster_uuid.workload_detail_uuid) to {@link QosWorkloadDetailAttributes}
     */
    private Map<String, Map<QosWorkloadDetailAttributes, String>> qosWorkloadDetailMap;

    /**
     * Map of volume workload to aggregate - cluster_uuid.workload_uuid to aggregate_uuid, aggregate_name
     */
    private Map<String, Map<AggregateAttributes, String>> qosVolumeWorkloadUUIDToAggregateMap;

    /**
     * Re Attribution context contains the foundation data required for the re attribution service.
     *
     * @param clusterUUID UUID of the Cluster,
     * @param qosVolumeWorkloadUUIDToAggregateMap   Map of volume workload to aggregate - cluster_uuid.workload_uuid to aggregate_uuid
     * @param qosWorkloadNodeRelationshipMap   Map of workload node relationship uuid (node_uuid.workload_uuid) to {@link QosWorkloadNodeRelationshipAttributes}
     * @param qosWorkloadDetailMap  Map of workload detail UUID (cluster_uuid.workload_detail_uuid) to {@link QosWorkloadDetailAttributes}
     */
    public ReAttributionContext(String clusterUUID,
                                Map<String, Map<AggregateAttributes,String>> qosVolumeWorkloadUUIDToAggregateMap,
                                Map<String, Map<QosWorkloadNodeRelationshipAttributes, String>> qosWorkloadNodeRelationshipMap,
                                Map<String, Map<QosWorkloadDetailAttributes, String>> qosWorkloadDetailMap) {
        this.clusterUUID = clusterUUID;
        this.qosWorkloadNodeRelationshipMap = qosWorkloadNodeRelationshipMap;
        this.qosWorkloadDetailMap = qosWorkloadDetailMap;
        this.qosVolumeWorkloadUUIDToAggregateMap = qosVolumeWorkloadUUIDToAggregateMap;
    }

    @Override
    public String getClusterUUID() {
        return this.clusterUUID;
    }

    /**
     * Map of volume workload to aggregate - cluster_uuid.workload_uuid to aggregate_uuid
     *
     * @return Map of volume workload to aggregate.
     */
    @Override
    public Map<String, Map<AggregateAttributes, String>> getQosVolumeWorkloadUUIDToAggregateUUIDMap() {
        return qosVolumeWorkloadUUIDToAggregateMap;
    }


    /**
     * Map of workload node relationship uuid (node_uuid.workload_uuid) to {@link QosWorkloadNodeRelationshipAttributes}
     *
     * @return Map of workload node relationship uuid to {@link QosWorkloadNodeRelationshipAttributes}
     */
    @Override
    public Map<String, Map<QosWorkloadNodeRelationshipAttributes, String>> getQosWorkloadNodeRelationshipMap() {
        return qosWorkloadNodeRelationshipMap;
    }

    /**
     * Map of workload detail UUID (cluster_uuid.workload_detail_uuid) to {@link QosWorkloadDetailAttributes}
     *
     * @return Map of workload detail UUID  to {@link QosWorkloadDetailAttributes}
     */
    @Override
    public Map<String, Map<QosWorkloadDetailAttributes, String>> getQosWorkloadDetailMap() {
        return qosWorkloadDetailMap;
    }
}
