package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.impl;


import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.data.QosWorkloadConstituentData;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.data.QosWorkloadDetailData;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.interfaces.IReAttributionContext;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.interfaces.IReAttributionService;

import java.util.List;

/**
 * Re Attribution service for re attributing volume workload details. For the complete algorithm refer
 * https://wikid.netapp.com/w/MEG/OCIE/Reattributing_Deferred_Workloads/Design
 *
 */
public class ReAttributeDeferredWorkServiceImpl implements IReAttributionService {
    private final IReAttributionContext reAttributionContext;

    /**
     * Create new re attribution service with the context passed.
     */
    public ReAttributeDeferredWorkServiceImpl(IReAttributionContext attributionContext) {
        reAttributionContext = attributionContext;
    }

    /**
     * Re-attributes the QOS workload details. After re-attribution the ${@link QosWorkloadDetailData} will be populated
     * with serviceTimeWAFLCP, serviceTimeBackgroundDelete & serviceTimeReadAheadXStripe. The original serviceTime
     * will not be modified. The caller will have to add them later.
     * <p/>
     * Refer https://wikid.netapp.com/w/MEG/OCIE/Reattributing_Deferred_Workloads/Design
     *
     * @param qosWorkloadDetailData      Map of workload detail uuid and qos workload details to re-attribute.
     * @param qosWorkloadConstituentData Map of node workload relationship uuid and qos workload constituents
     */
    @Override
    public void reAttributeQosWorkloadDetails(List<QosWorkloadDetailData> qosWorkloadDetailData,
                                              List<QosWorkloadConstituentData> qosWorkloadConstituentData) {
        ReAttributeDeferredWorkServiceStep1 step1 = new ReAttributeDeferredWorkServiceStep1(this.reAttributionContext,
                qosWorkloadDetailData);

        ReAttributeDeferredWorkServiceStep2 step2 = new ReAttributeDeferredWorkServiceStep2(this.reAttributionContext,
                qosWorkloadConstituentData);
        ReAttributeDeferredWorkServiceStep3 step3 = new ReAttributeDeferredWorkServiceStep3(this.reAttributionContext,
                qosWorkloadConstituentData, qosWorkloadDetailData, step1, step2);


    }


}