package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders;

import java.math.BigInteger;
import java.util.Collection;
import java.util.Map;

import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.LifMetricsBuilder;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.LifMetricsBuilder.LifCounter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.GlobalScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

/**
 * This class extends {@link SyntheticBuilderBase}. It uses counter definition at {@link LifMetricsBuilder#LifCounter}
 * to aggregate counter values to vserver level.
 *
 */
public class LifVserverMetricsBuilder extends SyntheticBuilderBase<NetworkLifVserverMeasurements, NetworkLifVserverRawMeasurements, Cluster.ClusterKey, LifMetricsBuilder.LifCounter> {

    public LifVserverMetricsBuilder(NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage);
    }
	
	@Override
	protected ClusterKey getParentInventoryUUIDKey(NetAppDefaultStorageContext context, String inventoryUUID,
		MetricsRowScratchPad<LifCounter> metricsRowScratchPad) {
		return context.getKey();
	}

	/**
	 * Get the reporting object inventoryUUID. In this case it's vserver's UUID.
	 */
	@Override
	protected String getInventoryUUID(NetAppDefaultStorageContext context, String vserverId, MetricsRowScratchPad<LifCounter> metricsRowScratchPad) {
		return context.getLifVserverIDToUUIDMap().get(vserverId);
	}
	
	/**
	 * Generate scratch pads by aggregating lif counter values to vserver level. 
	 */
	@Override
	protected Map<String, MetricsRowScratchPad<LifCounter>> generateMetricsRowScratchPads(
		NetAppDefaultStorageContext context, ZAPIConnection zapiConnection,	GlobalScratchPad globalScratchPad) {
        return globalScratchPad.merge(LifCounter.class, "lif", LifCounter.vserver_id);
	}

	@Override
	protected NetworkLifVserverMeasurements normalizeMetrics(
		MetricsRowScratchPad<LifCounter> currentScratchPad,	NetAppDefaultStorageContext context) {
		
		NetworkLifVserverMeasurements lifMeasurements = new NetworkLifVserverMeasurements();
		lifMeasurements.setRecvData(currentScratchPad.getCounterRatePerSec(LifCounter.recv_data));
		lifMeasurements.setRecvErrors(currentScratchPad.getCounterRatePerSec(LifCounter.recv_errors));
		lifMeasurements.setRecvPacket(currentScratchPad.getCounterRatePerSec(LifCounter.recv_packet));
		lifMeasurements.setSentData(currentScratchPad.getCounterRatePerSec(LifCounter.sent_data));
		lifMeasurements.setSentErrors(currentScratchPad.getCounterRatePerSec(LifCounter.sent_errors));
		lifMeasurements.setSentPacket(currentScratchPad.getCounterRatePerSec(LifCounter.sent_packet));
		lifMeasurements.setOpmLifPackets(MetricsRowScratchPad.add(lifMeasurements.getRecvPacket(), lifMeasurements.getSentPacket()));
		lifMeasurements.setOpmLifThroughput(MetricsRowScratchPad.add(lifMeasurements.getRecvData(), lifMeasurements.getSentData()));
	
		return lifMeasurements;
	}

	@Override
	public void reportPerformance(ClusterKey clusterKey,
		Map<ClusterKey, Collection<RawSample<NetworkLifVserverMeasurements>>> measurements)	throws DataSourceErrorException {

		for (final Map.Entry<ClusterKey, Collection<RawSample<NetworkLifVserverMeasurements>>> measurementEntry : measurements.entrySet()) {
			final ClusterKey key = measurementEntry.getKey();
			final Collection<RawSample<NetworkLifVserverMeasurements>> lifMeasurements = measurementEntry.getValue();
			reportingPackage.reportLifVserverPerformance(key, lifMeasurements);
		}
		
	}

	@Override
	protected NetworkLifVserverRawMeasurements rawMetrics(MetricsRowScratchPad<LifCounter> currentScratchPad,
		NetAppDefaultStorageContext context) {

		final NetworkLifVserverRawMeasurements m = new NetworkLifVserverRawMeasurements();
        Map<LifCounter,BigInteger> counterValue = currentScratchPad.getCounterValues();
        m.setRecvData(counterValue.get(LifCounter.recv_data));
        m.setRecvErrors(counterValue.get(LifCounter.recv_errors));
        m.setRecvPacket(counterValue.get(LifCounter.recv_packet));        
        m.setSentData(counterValue.get(LifCounter.sent_data));
        m.setSentErrors(counterValue.get(LifCounter.sent_errors));
        m.setSentPacket(counterValue.get(LifCounter.sent_packet));
        m.setOpmLifPackets(MetricsRowScratchPad.add(m.getRecvPacket(), m.getSentPacket()));
        m.setOpmLifThroughput(MetricsRowScratchPad.add(m.getRecvData(), m.getSentData()));
        
        return m;
	}

	@Override
	public void reportRawPerformance(ClusterKey clusterKey, 
		Map<ClusterKey, Collection<RawSample<NetworkLifVserverRawMeasurements>>> rawMeasurements)
		throws DataSourceErrorException {

		for (final Map.Entry<ClusterKey, Collection<RawSample<NetworkLifVserverRawMeasurements>>> measurementEntry : rawMeasurements.entrySet()) { 
            final ClusterKey key = measurementEntry.getKey(); 
            final Collection<RawSample<NetworkLifVserverRawMeasurements>> lifMeasurements = measurementEntry.getValue(); 
            reportingPackage.reportLifVserverRawPerformance(key, lifMeasurements); 
        } 		
	}

}
