package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders;

import java.math.BigInteger;
import java.util.Collection;
import java.util.Map;

import com.netapp.oci.netapp.performance.vserver.interfaces.data.VolumeVserverMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.VolumeMetricsBuilder;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.VolumeMetricsBuilder.VolumeCounter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.GlobalScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

/**
* This class extends {@link SyntheticBuilderBase}. It uses counter definition at {@link com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.VolumeMetricsBuilder.VolumeCounter}
* to aggregate counter values to vserver level.
*
*/
public class VolumeVserverMetricsBuilder extends SyntheticBuilderBase<VolumeVserverMeasurements, VolumeVserverMeasurements, Cluster.ClusterKey, VolumeMetricsBuilder.VolumeCounter> {

    public VolumeVserverMetricsBuilder(NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage);
    }

    @Override
    protected ClusterKey getParentInventoryUUIDKey(NetAppDefaultStorageContext context, String inventoryUUID,
                                                   MetricsRowScratchPad<VolumeCounter> metricsRowScratchPad) {
        return context.getKey();
    }

    /**
     * Get the reporting object inventoryUUID. In this case it's vserver's UUID.
     */
    @Override
    protected String getInventoryUUID(NetAppDefaultStorageContext context, String vserverUuid, MetricsRowScratchPad<VolumeCounter> metricsRowScratchPad) {
        return vserverUuid;
    }

    /**
     * Generate scratch pads by aggregating volume counter values to vserver level.
     */
    @Override
    protected Map<String, MetricsRowScratchPad<VolumeCounter>> generateMetricsRowScratchPads(
            NetAppDefaultStorageContext context, ZAPIConnection zapiConnection,	GlobalScratchPad globalScratchPad) {
        return globalScratchPad.merge(VolumeCounter.class, "volume", VolumeCounter.vserver_uuid);
    }

    @Override
    protected VolumeVserverMeasurements normalizeMetrics(
            MetricsRowScratchPad<VolumeCounter> currentScratchPad,	NetAppDefaultStorageContext context) {

        final VolumeVserverMeasurements m = new VolumeVserverMeasurements();
        m.setAvgLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.avg_latency, VolumeCounter.total_ops));
        m.setCifsOtherLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.cifs_other_latency, VolumeCounter.cifs_other_ops));
        m.setCifsOtherOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.cifs_other_ops));
        m.setCifsReadLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.cifs_read_latency, VolumeCounter.cifs_read_ops));
        m.setCifsReadOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.cifs_read_ops));
        m.setCifsWriteLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.cifs_write_latency, VolumeCounter.cifs_write_ops));
        m.setCifsWriteOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.cifs_write_ops));
        m.setFcpOtherLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.fcp_other_latency, VolumeCounter.fcp_other_ops));
        m.setFcpOtherOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.fcp_other_ops));
        m.setFcpReadLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.fcp_read_latency, VolumeCounter.fcp_read_ops));
        m.setFcpReadOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.fcp_read_ops));
        m.setFcpWriteLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.fcp_write_latency, VolumeCounter.fcp_write_ops));
        m.setFcpWriteOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.fcp_write_ops));
        m.setIscsiOtherLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.iscsi_other_latency, VolumeCounter.iscsi_other_ops));
        m.setIscsiOtherOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.iscsi_other_ops));
        m.setIscsiReadLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.iscsi_read_latency, VolumeCounter.iscsi_read_ops));
        m.setIscsiReadOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.iscsi_read_ops));
        m.setIscsiWriteLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.iscsi_write_latency, VolumeCounter.iscsi_write_ops));
        m.setIscsiWriteOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.iscsi_write_ops));
        m.setNfsOtherLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.nfs_other_latency, VolumeCounter.nfs_other_ops));
        m.setNfsOtherOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.nfs_other_ops));
        m.setNfsReadLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.nfs_read_latency, VolumeCounter.nfs_read_ops));
        m.setNfsReadOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.nfs_read_ops));
        m.setNfsWriteLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.nfs_write_latency, VolumeCounter.nfs_write_ops));
        m.setNfsWriteOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.nfs_write_ops));
        m.setOtherLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.other_latency, VolumeCounter.other_ops));
        m.setOtherOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.other_ops));
        m.setReadLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.read_latency, VolumeCounter.read_ops));
        m.setReadOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.read_ops));
        m.setSanOtherLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.san_other_latency, VolumeCounter.san_other_ops));
        m.setSanOtherOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.san_other_ops));
        m.setSanReadLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.san_read_latency, VolumeCounter.san_read_ops));
        m.setSanReadOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.san_read_ops));
        m.setSanWriteLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.san_write_latency, VolumeCounter.san_write_ops));
        m.setSanWriteOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.san_write_ops));
        m.setTotalOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.total_ops));
        m.setWriteLatency(currentScratchPad.getLatencyPerOp(VolumeCounter.write_latency, VolumeCounter.write_ops));
        m.setWriteOps(currentScratchPad.getCounterRatePerSec(VolumeCounter.write_ops));
        m.setOpmVolNfsOps(MetricsRowScratchPad.add(m.getNfsReadOps(), m.getNfsWriteOps(), m.getNfsOtherOps()));
        m.setOpmVolCifsOps(MetricsRowScratchPad.add(m.getCifsReadOps(), m.getCifsWriteOps(), m.getCifsOtherOps()));
        m.setOpmVolIscsiOps(MetricsRowScratchPad.add(m.getIscsiReadOps(), m.getIscsiWriteOps(), m.getIscsiOtherOps()));
        m.setOpmVolFcpOps(MetricsRowScratchPad.add(m.getFcpReadOps(),m.getFcpWriteOps(), m.getFcpOtherOps()));
        return m;
    }

    @Override
    public void reportPerformance(ClusterKey clusterKey, Map<ClusterKey, Collection<RawSample<VolumeVserverMeasurements>>> measurements) throws DataSourceErrorException {

        for (final Map.Entry<ClusterKey, Collection<RawSample<VolumeVserverMeasurements>>> measurementEntry : measurements.entrySet()) {
            final ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<VolumeVserverMeasurements>> volumeVserverMeasurements = measurementEntry.getValue();
            reportingPackage.reportVolumeVserverPerformance(key, volumeVserverMeasurements);
        }
    }

    @Override
    protected VolumeVserverMeasurements rawMetrics(MetricsRowScratchPad<VolumeCounter> currentScratchPad, NetAppDefaultStorageContext context) {
        return null;
    }

    @Override
    public void reportRawPerformance(ClusterKey clusterKey, Map<ClusterKey, Collection<RawSample<VolumeVserverMeasurements>>> rawMeasurements) throws DataSourceErrorException {
        // There are no raw metrics collected for VolumeVserver
    }
}
