package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.mergers;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.*;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MergeByPerformanceUUID;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders.VserverMetricsBuilder.VserverCounter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders.VserverMetricsBuilder;

import java.math.BigInteger;
import java.util.EnumMap;
import java.util.Map;

/**
 *  This Merger merges (nfsv3, mfsv4, nfs4v_1, cifs, iscsi_lif and fcp_lip) at Vserverl level into
 *  Map<String, MetricsRowScratchPad<VserverMetricsBuilder.VserverCounter> , where String here is a unque key
 *  representing vserverUUID
 */
public class VserverMetricsMerger extends MergeByPerformanceUUID<VserverMetricsBuilder.VserverCounter> {

    public VserverMetricsMerger() {
        super(VserverMetricsBuilder.VserverCounter.class);
    }

    /**
     * For fcp_lif and Iscsi_lif the performanceUUD is not vserverUUID, so handle them separately so that merge
     * happens on vserverUUID and not its performacenUUID. This is done in the super class.
     * @param counterInstances
     * @param counterObjectName
     * @return
     */
    @Override
    public String retrievePerfUUID(Map.Entry<String, ? extends MetricsRowScratchPad<? extends Enum<?>>> counterInstances, String counterObjectName){
        final String uuid;
        switch(counterObjectName) {
            case "fcp_lif":
                uuid = counterInstances.getValue().getRawValues().get(FcpLifMetricsBuilder.FcpLIFCounter.vserver_uuid);
                break;
            case "iscsi_lif":
                uuid = counterInstances.getValue().getRawValues().get(ISCSILIFMetricsBuilder.ISCSILIFCounter.vserver_uuid);
                break;
            default:
                uuid = counterInstances.getKey();
        }

        return uuid;
    }

    /**
     * Construct Result ScratchPad by partially merging them for each counter Object type
     * @param resultScratchPad The scratch pad to be updated with the merge results.
     * @param mergeWithCounterObjectName Name of the counter object to merge with.
     * @param mergeWithScratchPad Scratchpad to merge with.
     * @return
     */
    @Override
    public MetricsRowScratchPad<VserverCounter> mergeCounters(MetricsRowScratchPad<VserverCounter> resultScratchPad,
                                                          String mergeWithCounterObjectName,
                                                          MetricsRowScratchPad<? extends Enum<?>> mergeWithScratchPad) {
        switch (mergeWithCounterObjectName) {
            case "nfsv3":
                MetricsRowScratchPad<Nfsv3MetricsBuilder.NFSv3Counter> nfsV3ScratchPad =
                        (MetricsRowScratchPad<Nfsv3MetricsBuilder.NFSv3Counter>) mergeWithScratchPad;
                final EnumMap<Nfsv3MetricsBuilder.NFSv3Counter, BigInteger> nfsV3Counters = nfsV3ScratchPad.getCounterValues();
                //Merge Current
                mergeNFSv3ScratchPad(resultScratchPad, nfsV3Counters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && nfsV3ScratchPad.isPreviousPollValuesAvailable()) {
                    //Merge Previous
                    mergeNFSv3ScratchPad(resultScratchPad.getPreviousScratchPad(),
                            nfsV3ScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
            case "nfsv4":
                MetricsRowScratchPad<Nfsv4MetricsBuilder.NFSv4Counter> nfsV4ScratchPad =
                        (MetricsRowScratchPad<Nfsv4MetricsBuilder.NFSv4Counter>) mergeWithScratchPad;
                final EnumMap<Nfsv4MetricsBuilder.NFSv4Counter, BigInteger> nfsV4Counters = nfsV4ScratchPad.getCounterValues();
                mergeNFSv4ScratchPad(resultScratchPad, nfsV4Counters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && nfsV4ScratchPad.isPreviousPollValuesAvailable()) {
                    mergeNFSv4ScratchPad(resultScratchPad.getPreviousScratchPad(),
                            nfsV4ScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
            case "nfsv4_1":
                MetricsRowScratchPad<Nfsv41MetricsBuilder.NFSv41Counter> nfsV41ScratchPad =
                        (MetricsRowScratchPad<Nfsv41MetricsBuilder.NFSv41Counter>) mergeWithScratchPad;
                final EnumMap<Nfsv41MetricsBuilder.NFSv41Counter, BigInteger> nfsV41Counters = nfsV41ScratchPad.getCounterValues();
                mergeNFSv41ScratchPad(resultScratchPad, nfsV41Counters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && nfsV41ScratchPad.isPreviousPollValuesAvailable()) {
                    mergeNFSv41ScratchPad(resultScratchPad.getPreviousScratchPad(),
                            nfsV41ScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
            case "cifs":
                MetricsRowScratchPad<CifsVserverMetricsBuilder.CifsVserverCounter> cifsScratchPad =
                        (MetricsRowScratchPad<CifsVserverMetricsBuilder.CifsVserverCounter>) mergeWithScratchPad;
                final EnumMap<CifsVserverMetricsBuilder.CifsVserverCounter, BigInteger> cifsCounters = cifsScratchPad.getCounterValues();
                mergeCifsScratchPad(resultScratchPad, cifsCounters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && cifsScratchPad.isPreviousPollValuesAvailable()) {
                    mergeCifsScratchPad(resultScratchPad.getPreviousScratchPad(),
                            cifsScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
            case "iscsi_lif":
                MetricsRowScratchPad<ISCSILIFMetricsBuilder.ISCSILIFCounter> iscsiLifScratchPad =
                        (MetricsRowScratchPad<ISCSILIFMetricsBuilder.ISCSILIFCounter>) mergeWithScratchPad;
                final EnumMap<ISCSILIFMetricsBuilder.ISCSILIFCounter, BigInteger> iscsiLifCounters = iscsiLifScratchPad.getCounterValues();
                mergeIscsiLifScratchPad(resultScratchPad, iscsiLifCounters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && iscsiLifScratchPad.isPreviousPollValuesAvailable()) {
                    mergeIscsiLifScratchPad(resultScratchPad.getPreviousScratchPad(),
                            iscsiLifScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
            case "fcp_lif":
                MetricsRowScratchPad<FcpLifMetricsBuilder.FcpLIFCounter> fcpLifScratchPad =
                        (MetricsRowScratchPad<FcpLifMetricsBuilder.FcpLIFCounter>) mergeWithScratchPad;
                final EnumMap<FcpLifMetricsBuilder.FcpLIFCounter, BigInteger> fcpLifCounters = fcpLifScratchPad.getCounterValues();
                mergeFcpLifScratchPad(resultScratchPad, fcpLifCounters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && fcpLifScratchPad.isPreviousPollValuesAvailable()) {
                    mergeFcpLifScratchPad(resultScratchPad.getPreviousScratchPad(),
                            fcpLifScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
        }
        return resultScratchPad;
    }

    private void mergeNFSv41ScratchPad(MetricsRowScratchPad<VserverCounter> resultScratchPad, EnumMap<Nfsv41MetricsBuilder.NFSv41Counter, BigInteger> nfsV41Counters) {
        //Add Ops
        resultScratchPad.addToCounter(VserverCounter.opm_ops, nfsV41Counters.get(Nfsv41MetricsBuilder.NFSv41Counter.nfsv4_1_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_read_ops, nfsV41Counters.get(Nfsv41MetricsBuilder.NFSv41Counter.read_total));
        resultScratchPad.addToCounter(VserverCounter.opm_write_ops, nfsV41Counters.get(Nfsv41MetricsBuilder.NFSv41Counter.write_total));
        //Add Throughput
        resultScratchPad.addToCounter(VserverCounter.opm_throughput, nfsV41Counters.get(Nfsv41MetricsBuilder.NFSv41Counter.nfs41_throughput));
        resultScratchPad.addToCounter(VserverCounter.opm_read_throughput, nfsV41Counters.get(Nfsv41MetricsBuilder.NFSv41Counter.nfs41_read_throughput));
        resultScratchPad.addToCounter(VserverCounter.opm_write_throughput, nfsV41Counters.get(Nfsv41MetricsBuilder.NFSv41Counter.nfs41_write_throughput));
        //Add Latency
        resultScratchPad.addToCounter(VserverCounter.opm_read_latency, nfsV41Counters.get(Nfsv41MetricsBuilder.NFSv41Counter.read_avg_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_write_latency, nfsV41Counters.get(Nfsv41MetricsBuilder.NFSv41Counter.write_avg_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_latency, nfsV41Counters.get(Nfsv41MetricsBuilder.NFSv41Counter.latency));
    }

    private void mergeNFSv4ScratchPad(MetricsRowScratchPad<VserverCounter> resultScratchPad, EnumMap<Nfsv4MetricsBuilder.NFSv4Counter, BigInteger> nfsV4Counters) {
        //Add Ops
        resultScratchPad.addToCounter(VserverCounter.opm_ops, nfsV4Counters.get(Nfsv4MetricsBuilder.NFSv4Counter.nfsv4_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_read_ops, nfsV4Counters.get(Nfsv4MetricsBuilder.NFSv4Counter.read_total));
        resultScratchPad.addToCounter(VserverCounter.opm_write_ops, nfsV4Counters.get(Nfsv4MetricsBuilder.NFSv4Counter.write_total));
        //Add Throughput
        resultScratchPad.addToCounter(VserverCounter.opm_throughput, nfsV4Counters.get(Nfsv4MetricsBuilder.NFSv4Counter.nfs4_throughput));
        resultScratchPad.addToCounter(VserverCounter.opm_read_throughput, nfsV4Counters.get(Nfsv4MetricsBuilder.NFSv4Counter.nfs4_read_throughput));
        resultScratchPad.addToCounter(VserverCounter.opm_write_throughput, nfsV4Counters.get(Nfsv4MetricsBuilder.NFSv4Counter.nfs4_write_throughput));
        //Add Latency
        resultScratchPad.addToCounter(VserverCounter.opm_read_latency, nfsV4Counters.get(Nfsv4MetricsBuilder.NFSv4Counter.read_avg_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_write_latency, nfsV4Counters.get(Nfsv4MetricsBuilder.NFSv4Counter.write_avg_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_latency, nfsV4Counters.get(Nfsv4MetricsBuilder.NFSv4Counter.latency));
    }

    private void mergeNFSv3ScratchPad(MetricsRowScratchPad<VserverCounter> resultScratchPad, EnumMap<Nfsv3MetricsBuilder.NFSv3Counter, BigInteger> nfsV3Counters) {
        //Add OPS
        resultScratchPad.addToCounter(VserverCounter.opm_ops, nfsV3Counters.get(Nfsv3MetricsBuilder.NFSv3Counter.nfsv3_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_read_ops, nfsV3Counters.get(Nfsv3MetricsBuilder.NFSv3Counter.nfsv3_read_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_write_ops, nfsV3Counters.get(Nfsv3MetricsBuilder.NFSv3Counter.nfsv3_write_ops));
        //Add Throughput
        resultScratchPad.addToCounter(VserverCounter.opm_throughput, nfsV3Counters.get(Nfsv3MetricsBuilder.NFSv3Counter.nfsv3_throughput));
        resultScratchPad.addToCounter(VserverCounter.opm_read_throughput, nfsV3Counters.get(Nfsv3MetricsBuilder.NFSv3Counter.nfsv3_read_throughput));
        resultScratchPad.addToCounter(VserverCounter.opm_write_throughput, nfsV3Counters.get(Nfsv3MetricsBuilder.NFSv3Counter.nfsv3_write_throughput));
        //Add Latency
        resultScratchPad.addToCounter(VserverCounter.opm_read_latency, nfsV3Counters.get(Nfsv3MetricsBuilder.NFSv3Counter.read_avg_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_write_latency, nfsV3Counters.get(Nfsv3MetricsBuilder.NFSv3Counter.write_avg_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_latency, nfsV3Counters.get(Nfsv3MetricsBuilder.NFSv3Counter.latency));
    }

    private void mergeCifsScratchPad(MetricsRowScratchPad<VserverCounter> resultScratchPad, EnumMap<CifsVserverMetricsBuilder.CifsVserverCounter, BigInteger> cifsCounters) {
        //Add OPS
        resultScratchPad.addToCounter(VserverCounter.opm_ops, cifsCounters.get(CifsVserverMetricsBuilder.CifsVserverCounter.cifs_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_read_ops, cifsCounters.get(CifsVserverMetricsBuilder.CifsVserverCounter.cifs_read_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_write_ops, cifsCounters.get(CifsVserverMetricsBuilder.CifsVserverCounter.cifs_write_ops));
        //Add Throughput
        resultScratchPad.addToCounter(VserverCounter.opm_throughput, MetricsRowScratchPad.add(cifsCounters.get(CifsVserverMetricsBuilder.CifsVserverCounter.read_data),
                                                                                                cifsCounters.get(CifsVserverMetricsBuilder.CifsVserverCounter.write_data)));
        resultScratchPad.addToCounter(VserverCounter.opm_read_throughput, cifsCounters.get(CifsVserverMetricsBuilder.CifsVserverCounter.read_data));
        resultScratchPad.addToCounter(VserverCounter.opm_write_throughput, cifsCounters.get(CifsVserverMetricsBuilder.CifsVserverCounter.write_data));
        //Add Latency
        resultScratchPad.addToCounter(VserverCounter.opm_read_latency, cifsCounters.get(CifsVserverMetricsBuilder.CifsVserverCounter.cifs_read_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_write_latency, cifsCounters.get(CifsVserverMetricsBuilder.CifsVserverCounter.cifs_write_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_latency, cifsCounters.get(CifsVserverMetricsBuilder.CifsVserverCounter.cifs_latency));
    }

    private void mergeIscsiLifScratchPad(MetricsRowScratchPad<VserverCounter> resultScratchPad, EnumMap<ISCSILIFMetricsBuilder.ISCSILIFCounter, BigInteger> iscsiLifCounters) {
        //Add OPS
        resultScratchPad.addToCounter(VserverCounter.opm_ops, MetricsRowScratchPad.add(
                iscsiLifCounters.get(ISCSILIFMetricsBuilder.ISCSILIFCounter.iscsi_other_ops), iscsiLifCounters.get(ISCSILIFMetricsBuilder.ISCSILIFCounter.iscsi_read_ops),
                iscsiLifCounters.get(ISCSILIFMetricsBuilder.ISCSILIFCounter.iscsi_write_ops)));
        resultScratchPad.addToCounter(VserverCounter.opm_read_ops, iscsiLifCounters.get(ISCSILIFMetricsBuilder.ISCSILIFCounter.iscsi_read_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_write_ops, iscsiLifCounters.get(ISCSILIFMetricsBuilder.ISCSILIFCounter.iscsi_write_ops));
        //Add Throughput
        resultScratchPad.addToCounter(VserverCounter.opm_throughput, MetricsRowScratchPad.add(
                iscsiLifCounters.get(ISCSILIFMetricsBuilder.ISCSILIFCounter.read_data), iscsiLifCounters.get(ISCSILIFMetricsBuilder.ISCSILIFCounter.write_data)));
        resultScratchPad.addToCounter(VserverCounter.opm_read_throughput, iscsiLifCounters.get(ISCSILIFMetricsBuilder.ISCSILIFCounter.read_data));
        resultScratchPad.addToCounter(VserverCounter.opm_write_throughput, iscsiLifCounters.get(ISCSILIFMetricsBuilder.ISCSILIFCounter.write_data));
        //Add Latency
        resultScratchPad.addToCounter(VserverCounter.opm_read_latency, iscsiLifCounters.get(ISCSILIFMetricsBuilder.ISCSILIFCounter.avg_read_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_write_latency, iscsiLifCounters.get(ISCSILIFMetricsBuilder.ISCSILIFCounter.avg_write_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_latency, iscsiLifCounters.get(ISCSILIFMetricsBuilder.ISCSILIFCounter.avg_latency));
    }

    private void mergeFcpLifScratchPad(MetricsRowScratchPad<VserverCounter> resultScratchPad, EnumMap<FcpLifMetricsBuilder.FcpLIFCounter, BigInteger> fcpLifCounters) {
        //Add OPS
        resultScratchPad.addToCounter(VserverCounter.opm_ops, fcpLifCounters.get(FcpLifMetricsBuilder.FcpLIFCounter.total_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_read_ops, fcpLifCounters.get(FcpLifMetricsBuilder.FcpLIFCounter.read_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_write_ops, fcpLifCounters.get(FcpLifMetricsBuilder.FcpLIFCounter.write_ops));
        //Add Throughput
        resultScratchPad.addToCounter(VserverCounter.opm_throughput, MetricsRowScratchPad.add(fcpLifCounters.get(FcpLifMetricsBuilder.FcpLIFCounter.read_data),
                fcpLifCounters.get(FcpLifMetricsBuilder.FcpLIFCounter.write_data)));
        resultScratchPad.addToCounter(VserverCounter.opm_read_throughput, fcpLifCounters.get(FcpLifMetricsBuilder.FcpLIFCounter.read_data));
        resultScratchPad.addToCounter(VserverCounter.opm_write_throughput, fcpLifCounters.get(FcpLifMetricsBuilder.FcpLIFCounter.write_data));
        //Add Latency
        resultScratchPad.addToCounter(VserverCounter.opm_read_latency, fcpLifCounters.get(FcpLifMetricsBuilder.FcpLIFCounter.avg_read_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_write_latency, fcpLifCounters.get(FcpLifMetricsBuilder.FcpLIFCounter.avg_write_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_latency, fcpLifCounters.get(FcpLifMetricsBuilder.FcpLIFCounter.avg_latency));
    }
}