package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transform.ocie;

import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeLunImport;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Disk;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.LunImport;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Node;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer;

public class LunImportTransformer {

    private static final Logger logger = LoggerFactory.getLogger(LunImportTransformer.class);

    private LunImportTransformer() {
    }

    public static void transform(final CModeCluster cModeCluster, final Cluster cluster,
            final Map<String, VServer.VServerKey> vServerKeysByName, final Map<String, Node> nodesByName,
            final Map<String, Disk.DiskKey> diskKeyBySerialNumber) {
        for (final CModeLunImport cModeLunImport : cModeCluster.getLunImports().values()) {
            final String vServerName = cModeLunImport.getVServer();
            final VServer.VServerKey vServerKey = vServerKeysByName.get(vServerName);
            if (vServerKey == null) {
                logger.info("{}  - Failed to find vServer key by name: {} for LUN import.", new Object[] {
                        PackageContext.getLogPrefix(), vServerName });
                continue;
            }
            final VServer vServer = cluster.getVServers().get(vServerKey);
            if (vServer == null) {
                logger.info("{} - Failed to find vServer by key: {} for LUN import.",
                        new Object[] { PackageContext.getLogPrefix(), vServerKey.getResourceKey() });
                continue;
            }

            final String foreignDisk = cModeLunImport.getKey().getID(
                    CModeLunImport.CModeLunImportKey.Field.foreign_disk);
            final LunImport lunImport = cluster.createLunImport(vServerKey, foreignDisk);

            BeanUtils.copyProperties(cModeLunImport, lunImport, new String[] { "administrativeStatus",
                    "administrativeStatusRaw", "foreignDisk", "importCurrentNodeKey", "importHomeNodeKey",
                    "operationInProgress", "operationInProgressRaw", "operationalStatus", "operationalStatusRaw",
                    "vserverId" });

            lunImport.setAdministrativeStatus(cModeLunImport.getAdministrativeStatus().getEnumValue());
            lunImport.setAdministrativeStatusRaw(cModeLunImport.getAdministrativeStatus().getRawValue());
            lunImport.setForeignDisk(diskKeyBySerialNumber.get(cModeLunImport.getForeignDisk()));

            String importCurrentNode = cModeLunImport.getImportCurrentNode();
            if (nodesByName.containsKey(importCurrentNode)) {
                lunImport.setImportCurrentNodeKey(nodesByName.get(importCurrentNode).getKey());
            }

            String importHomeNode = cModeLunImport.getImportHomeNode();
            if (nodesByName.containsKey(importHomeNode)) {
                lunImport.setImportHomeNodeKey(nodesByName.get(importHomeNode).getKey());
            }

            lunImport.setOperationInProgress(cModeLunImport.getOperationInProgress().getEnumValue());
            lunImport.setOperationInProgressRaw(cModeLunImport.getOperationInProgress().getRawValue());
            lunImport.setOperationalStatus(cModeLunImport.getOperationalStatus().getEnumValue());
            lunImport.setOperationalStatusRaw(cModeLunImport.getOperationalStatus().getRawValue());
            lunImport.setVserverId(vServerKey);

            cluster.getLunImports().add(lunImport);
        }
    }

}
