package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transform.ocie;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeNetworkSubnet;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.NetworkIpSpace;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.NetworkPortBroadcastDomain;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.NetworkSubnet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.util.Map;

/**
 * Transform cModeNetworkSubnet to NetworkSubnet
 */
public class NetworkSubnetTransform {
    private static final Logger logger = LoggerFactory.getLogger(NetworkSubnetTransform.class);

    public NetworkSubnetTransform() {

    }

    public static void transform(final CModeCluster cModeCluster, final Cluster cluster, Map<String, NetworkIpSpace.NetworkIpSpaceKey> ipSpaceNameToKeyMap,
                                 Map<String, NetworkPortBroadcastDomain.NetworkPortBroadcastDomainKey> broadcastDomainNameToKeyMap) {
        for(final CModeNetworkSubnet cModeNetworkSubnet : cModeCluster.getNetworkSubnets().values())  {
            String name = cModeNetworkSubnet.getName();
            NetworkIpSpace.NetworkIpSpaceKey ipspaceKey =  ipSpaceNameToKeyMap.get(cModeNetworkSubnet.getIpspace());
            NetworkPortBroadcastDomain.NetworkPortBroadcastDomainKey broadcastDomainKey = broadcastDomainNameToKeyMap.get(cModeNetworkSubnet.getBroadcastDomain());
            String broadcastDomainUUID = (broadcastDomainKey==null)? null:broadcastDomainKey.getOcieUUID();
            if(ipspaceKey!=null)  {
                final NetworkSubnet subnet = cluster.createNetworkSubnet(ipspaceKey.getOcieUUID(), name);

                BeanUtils.copyProperties(cModeNetworkSubnet, subnet, "broadcastDomainUUID");
                subnet.setBroadcastDomainUUID(broadcastDomainUUID);

                cluster.getNetworkSubnets().add(subnet);
            } else  {
                logger.info(PackageContext.getLogPrefix() + " - Failed to find ip space key by {}", cModeNetworkSubnet.getIpspace());
            }
        }

    }

}
