package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transform.ocie;


import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeSnapMirrorHistory;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeVServerPeer;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.SnapMirror;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.SnapMirrorHistory;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServerPeer;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Volume;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Volume.VolumeKey;
import com.onaro.util.VersionComparator;


public class SnapMirrorHistoryTransformer {
    private static final Logger logger = LoggerFactory.getLogger(SnapMirrorHistoryTransformer.class);
            
    private SnapMirrorHistoryTransformer() {
    }

    public static void transform(final CModeCluster cModeCluster, final Cluster cluster,
            final Map<String, VServer.VServerKey> vServerKeysByName,
            Map<VServer.VServerKey, Map<String, Volume.VolumeKey>> volumeKeysByNameByVServerKey) {      
        for (CModeSnapMirrorHistory cModeSnapMirrorHistory : cModeCluster.getSnapMirrorHistories().values()) {
            boolean isVServerSnapMirror = SnapMirrorHistory.RelationshipGroupType.VSERVER.equals(cModeSnapMirrorHistory
                    .getRelationshipGroupType().getEnumValue())
                    && cModeSnapMirrorHistory.getDestinationVolume() == null
                    && cModeSnapMirrorHistory.getSourceVolume() == null;
            SnapMirrorHistory snapMirrorHistory = cluster.createSnapMirrorHistory(cModeSnapMirrorHistory.getOperationId());
            
            VServer.VServerKey vServerKey = vServerKeysByName.get(cModeSnapMirrorHistory.getDestinationVServer());
            if (vServerKey == null) {
                logger.info(PackageContext.getLogPrefix() + " - Failed to find vServerKey for vServerName: " + cModeSnapMirrorHistory.getDestinationVServer()
                        + " in SnapMirrorHistoryTransformer.");
                continue;
            }
            
            SnapMirror.SnapMirrorKey snapMirrorKey;
            
            if (isVServerSnapMirror) {
                snapMirrorKey = new SnapMirror.SnapMirrorKey(cluster.getClusterUUID(), vServerKey.getZapiUUID(),
                        cModeSnapMirrorHistory.getDestinationLocation());
                snapMirrorHistory.setSnapMirrorUUID(vServerKey.getZapiUUID());
            }
            else {
                Map<String, Volume.VolumeKey> volumeKeysByName = volumeKeysByNameByVServerKey.get(vServerKey);
                if (volumeKeysByName == null) {
                    logger.info(PackageContext.getLogPrefix() + " - Failed to find volumeKeysByName for vServerKey: " + vServerKey
                            + " in SnapMirrorHistoryTransformer.");
                    continue;
                }
                
                VolumeKey volumeKey = volumeKeysByName.get(cModeSnapMirrorHistory.getDestinationVolume());
                if (volumeKey == null) {
                    logger.info(PackageContext.getLogPrefix() + " - Failed to find volumeKey for volumeName: " + cModeSnapMirrorHistory.getDestinationVolume()
                            + " in SnapMirrorHistoryTransformer.");
                    continue;
                }
                
                snapMirrorKey = new SnapMirror.SnapMirrorKey(cluster.getClusterUUID(), vServerKey.getZapiUUID(),
                        volumeKey.getVolumeInstanceUUID(), cModeSnapMirrorHistory.getDestinationLocation());
                snapMirrorHistory.setSnapMirrorUUID(volumeKey.getVolumeInstanceUUID());
            }
            
            if (!cluster.getSnapMirrors().containsKey(snapMirrorKey)) {
                logger.info(PackageContext.getLogPrefix() + " - Failed to find snapMirror for snapMirrorKey: " + snapMirrorKey
                        + " in SnapMirrorHistoryTransformer.");
                continue;
            }
            
           BeanUtils.copyProperties(cModeSnapMirrorHistory, snapMirrorHistory, new String[] { "operationType", "OperationTypeRaw", "restult", "resultRaw" });
           snapMirrorHistory.setOperationType(cModeSnapMirrorHistory.getOperationType().getEnumValue());
           snapMirrorHistory.setOperationTypeRaw(cModeSnapMirrorHistory.getOperationType().getRawValue());
           snapMirrorHistory.setResult(cModeSnapMirrorHistory.getResult().getEnumValue());
           snapMirrorHistory.setResultRaw(cModeSnapMirrorHistory.getResult().getRawValue());

           cluster.getSnapMirrorHistories().add(snapMirrorHistory);
        }
    }

}

