package com.onaro.sanscreen.acquisition.diskmodelutils;

import com.netapp.acquisition.aquisitionserverside.interfaces.data.DiskModelDescription;
import com.netapp.acquisition.aquisitionserverside.interfaces.data.DiskModelUtil;

import com.googlecode.sardine.DavResource;
import com.googlecode.sardine.Sardine;
import com.googlecode.sardine.impl.SardineImpl;
import org.apache.commons.io.IOUtils;
import org.apache.http.client.HttpClient;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.AbstractHttpClient;
import org.apache.http.impl.client.DefaultHttpClient;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Document;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.List;

public abstract class DiskModelPublisher {

    private static String USERNAME = "acq_spinningrust!";
    private static String PASSWORD = "dshjki34t2h3h@#%!@";
    private static String HOST = "10.254.112.82";// this is the internal ip of "gate.onaro.com"; only the internal supports webdav
    private static String PROTOCOL = "https";
    private static int PORT = 443;

    private static String READ_ONLY_PATH = "/diskmodels/";
    private static String PUBLISH_PATH = "/acq_diskmodels_rw/";
    private static String DISK_MODEL_NAME_FORMAT = "disk_models.%1$d.txt";
    private static SimpleDateFormat BACKUP_TIME_FORMAT_EXTENSION = new SimpleDateFormat("_yyyy_MM_dd_HH_mm_SSS");
    private final Sardine webDav;

    public DiskModelPublisher() {
        final DefaultHttpClient defaultHttpClient = new DefaultHttpClient();
        final HttpClient trustedHttpClient = wrapClient(defaultHttpClient);
        webDav = new SardineImpl((AbstractHttpClient) trustedHttpClient, USERNAME, PASSWORD);
    }

    abstract public boolean confirm();

    public void remove(final String fileName) throws IOException {
        remove(getPublishURL(fileName));
    }

    public void remove(final URL url) throws IOException {
        webDav.delete(url.toString());
    }

    public void publishExisting(final URL url) throws Exception {
        // run disk model merger on the file to validate it
        // publishNew the different versions of the file
        final String content = webDavGet(url);
        final InputStream is = IOUtils.toInputStream(content);

        final DiskModelMerger merger = new DiskModelMerger(new InputStream[]{is});
        final Map<Integer, File> results = merger.merge(false);
        for (final Map.Entry<Integer, File> entry : results.entrySet()) {
            final Integer version = entry.getKey();
            final String fileName = String.format(DISK_MODEL_NAME_FORMAT, version);
            final String publishedContent = getPublishedFile(fileName);
            final String newContent = getContent(new FileInputStream(entry.getValue()));
            final boolean same = compare(publishedContent, newContent);

            if (!same) {
                if (confirm()) {
                    publish(is, fileName);
                }
                else {
                    System.out.println("the disk model was NOT published!");
                }
            }
            else {
                System.out.println("the disk model is the same and does not need to be published");
            }
        }
    }

    public void publishNew(final File diskModelFile) throws Exception {
        // run disk model merger on the file to validate it
        // publishNew the different versions of the file

        final DiskModelMerger merger = new DiskModelMerger(new File[]{diskModelFile});
        final Map<Integer, File> results = merger.merge(false);
        for (final Map.Entry<Integer, File> entry : results.entrySet()) {
            final Integer version = entry.getKey();
            final String fileName = String.format(DISK_MODEL_NAME_FORMAT, version);
            final String publishedContent = getPublishedFile(fileName);
            boolean same = false;
            if (publishedContent != null){
                final String newContent = getContent(new FileInputStream(entry.getValue()));
                same = compare(publishedContent, newContent);
            }

            if (!same) {
                if (confirm()) {
                    backupAndPublish(entry.getValue(), fileName);
                }
                else {
                    System.out.println("the disk model was NOT published!");
                }
            }
            else {
                System.out.println("the disk model is the same and does not need to be published");
            }
        }
    }

    public List<DavResource> getExistingFiles() throws IOException {
        final List<DavResource> fileList = new LinkedList<DavResource>();
        final URL publishDir = getPublishURL("");
        final List<DavResource> davResources = webDav.list(publishDir.toString());
        for (final DavResource davResource : davResources) {
            if ("application/octetstream".equalsIgnoreCase(davResource.getContentType())) {
                fileList.add(davResource);
            }
        }
        return fileList;
    }

    private String getContent(final InputStream inputStream) throws IOException {
        return getContent(inputStream, null);
    }

    private String getContent(final InputStream inputStream, final Long contentLength) throws IOException {
        final StringWriter content = (contentLength == null) ? new StringWriter() : new StringWriter(contentLength.intValue());
        IOUtils.copy(inputStream, content);
        return content.toString();
    }

    /**
     * @param oldContent
     * @param newContent
     *
     * @return true if the contents are the same. Otherwise false.
     *
     * @throws IOException
     */
    private boolean compare(final String oldContent, final String newContent) throws IOException {
        if (oldContent == null) {
            return false;
        }
        final SortedMap<String, DiskModelDescription> oldModels = DiskModelUtil.parseDiskModelDescription(oldContent);
        final SortedMap<String, DiskModelDescription> newModels = DiskModelUtil.parseDiskModelDescription(newContent);

        final SortedMap<String, DiskModelDescription> removedModels = getRemovedModels(oldModels, newModels);
        final SortedMap<String, DiskModelDescription> addedModels = getAddedModels(oldModels, newModels);
        final SortedMap<String, DiskModelDescription[]> modifiedModels = getModifiedModels(oldModels, newModels);
        final String newLine = "\n";
        writeRemoved(removedModels);
        writeAdded(addedModels);
        writeModified(modifiedModels);
        return removedModels.isEmpty() && addedModels.isEmpty() && modifiedModels.isEmpty();
    }

    private void writeRemoved(SortedMap<String, DiskModelDescription> map) throws IOException {
        System.out.println("******************************    REMOVED    ******************************");
        System.out.println();
        if (map.isEmpty()) {
            System.out.println("none");
        }
        else {
            writeDiskModelDescriptions(map);
        }
        System.out.println();
    }

    private void writeAdded(SortedMap<String, DiskModelDescription> map) throws IOException {
        System.out.println("******************************     ADDED     ******************************");
        if (map.isEmpty()) {
            System.out.println("none");
        }
        else {
            writeDiskModelDescriptions(map);
        }
        System.out.println();
    }

    private void writeModified(SortedMap<String, DiskModelDescription[]> map) throws IOException {
        System.out.println("******************************    MODIFIED    ******************************");
        if (map.isEmpty()) {
            System.out.println("none");
        }
        else {
            for (Map.Entry<String, DiskModelDescription[]> entry : map.entrySet()) {
                System.out.println("old: ");
                writeDiskModelDescription(entry.getValue()[0]);
                System.out.println("new: ");
                writeDiskModelDescription(entry.getValue()[1]);
                System.out.println();
            }
        }
        System.out.println();
    }

    private void writeDiskModelDescriptions(final SortedMap<String, DiskModelDescription> map) throws IOException {
        for (final DiskModelDescription diskModelDescription : map.values()) {
            writeDiskModelDescription(diskModelDescription);
        }
    }

    private void writeDiskModelDescription(final DiskModelDescription diskModelDescription) throws IOException {
        System.out.println(diskModelDescription.toConfigCSV());
    }

    private SortedMap<String, DiskModelDescription[]> getModifiedModels(final SortedMap<String, DiskModelDescription> oldModels, final SortedMap<String, DiskModelDescription> newModels) {

        final SortedMap<String, DiskModelDescription[]> modifiedModels = new TreeMap<String, DiskModelDescription[]>();
        for (final Map.Entry<String, DiskModelDescription> entry : newModels.entrySet()) {
            final DiskModelDescription oldModel = oldModels.get(entry.getKey());
            if (oldModel == null) {
                continue;
            }
            final DiskModelDescription newModel = newModels.get(entry.getKey());
            if (!oldModel.equals(newModel)) {
                modifiedModels.put(entry.getKey(), new DiskModelDescription[]{oldModel, newModel});
            }
        }
        return modifiedModels;
    }

    private SortedMap<String, DiskModelDescription> getAddedModels(final SortedMap<String, DiskModelDescription> oldModels, final SortedMap<String, DiskModelDescription> newModels) {
        final SortedMap<String, DiskModelDescription> addedModels = new TreeMap<String, DiskModelDescription>(newModels);
        addedModels.keySet().removeAll(oldModels.keySet());
        return addedModels;
    }

    private SortedMap<String, DiskModelDescription> getRemovedModels(final SortedMap<String, DiskModelDescription> oldModels, final SortedMap<String, DiskModelDescription> newModels) {
        final SortedMap<String, DiskModelDescription> removedModels = new TreeMap<String, DiskModelDescription>(oldModels);
        removedModels.keySet().removeAll(newModels.keySet());
        return removedModels;
    }

    private URL getPublishURL(final String fileName) throws MalformedURLException {
        return new URL(PROTOCOL, HOST, PORT, PUBLISH_PATH + fileName);
    }

    private URL getReadOnlyURL(final String fileName) throws MalformedURLException {
        return new URL(PROTOCOL, HOST, PORT, READ_ONLY_PATH + fileName);

    }

    public final String backupIfExists(final String fileName) throws IOException {
        final URL url = getPublishURL(fileName);
        final String sourceUrl = url.toString();
        if (!webDav.exists(sourceUrl)) {
            return null;
        }
        final String destinationUrl = url.toString() + BACKUP_TIME_FORMAT_EXTENSION.format(new Date()) + ".bak";
        final String content = getPublishedFile(fileName);
        InputStream is = null;
        try {
            is = IOUtils.toInputStream(content);
            webDav.put(destinationUrl, is);
            return destinationUrl;
        }
        finally {
            if (is != null) {
                is.close();

            }
        }
    }

    public String getPublishedFile(final String fileName) throws IOException {
        final URL url = getPublishURL(fileName);
        return webDavGet(url);
    }

    public String webDavGet(URL url) throws IOException {
        final InputStream is;
        try {
            is = webDav.get(url.toString());
        }
        catch (IOException e) {
            return null;
        }
        try {
            return getContent(is);
        }
        finally {
            is.close();
        }
    }

    private void backupAndPublish(final File file, final String publishedFileName) throws IOException {
        final String backupName = backupIfExists(publishedFileName);
        if (backupName != null) {
            System.out.println("file " + publishedFileName + " backed up to " + backupName);
        }
        final InputStream is = new FileInputStream(file);
        try {
            publish(is, publishedFileName);
        }
        finally {
            is.close();
        }

    }

    private void publish(final InputStream is, final String publishedFileName) throws IOException {
        final String url = getPublishURL(publishedFileName).toString();
        webDav.put(url, is);
        System.out.println("published " + url);
    }

    public static void main(final String[] args) {
        if (args.length != 1) {
            usage();
            System.exit(1);
        }
        try {
            final DiskModelPublisher publish = new DiskModelPublisher() {
                @Override
                public boolean confirm() {
                    return true;
                }
            };
            publish.publishNew(new File(args[0]));
        }
        catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }

    }

    public static void usage() {
        System.out.println("\nUsage: <command> disk_models_file");
        System.out.println("\tArguments: are one file of disk model descriptions.");
        System.out.println("\tProduces: error messages, or publishes 6.0 formats to gate.onaro.com");
    }

    public static HttpClient wrapClient(HttpClient base) {
        try {
            SSLContext ctx = SSLContext.getInstance("TLS");
            X509TrustManager tm = new X509TrustManager() {
                public void checkClientTrusted(X509Certificate[] xcs, String string) throws CertificateException {
                }

                public void checkServerTrusted(X509Certificate[] xcs, String string) throws CertificateException {
                }

                public X509Certificate[] getAcceptedIssuers() {
                    return null;
                }
            };
            ctx.init(null, new TrustManager[]{tm}, null);
            SSLSocketFactory ssf = new SSLSocketFactory(ctx);
            ssf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
            ClientConnectionManager ccm = base.getConnectionManager();
            SchemeRegistry sr = ccm.getSchemeRegistry();
            sr.register(new Scheme("https", ssf, 443));
            return new DefaultHttpClient(ccm, base.getParams());
        }
        catch (Exception ex) {
            return null;
        }
    }

    public static class GUI extends JPanel {
        final DiskModelPublisher publisher;
        final Document log = new DefaultStyledDocument();
        File file = null;

        public GUI() {
            super(new BorderLayout());

            publisher = new DiskModelPublisher() {
                @Override
                public boolean confirm() {
                    final int n = JOptionPane.showConfirmDialog(GUI.this,
                                                                "Do you want to publish the disk model to gate.onaro.com?",
                                                                "Publish the changes?",
                                                                JOptionPane.YES_NO_OPTION);
                    return (n == JOptionPane.YES_OPTION);

                }
            };

            final File bannerFile = new File("resource/acqteam.jpg");
            final ImageIcon banner = new ImageIcon(bannerFile.getAbsolutePath());
            final JLabel label = new JLabel();
            label.setIcon(banner);
            add(label, BorderLayout.PAGE_START);

            final JPanel publishPanel = getPublishPanel();
            final JPanel managePanel = getManagePanel();
            final JTabbedPane tabbedPane = new JTabbedPane();
            tabbedPane.addTab("Publish New Disk Model", publishPanel);
            tabbedPane.addTab("Manage Disk Models", managePanel);

            add(tabbedPane, BorderLayout.CENTER);
            this.setPreferredSize(new Dimension(900, 500));

            redirectSystemStreams();
        }

        private JTextArea getLogPanel(final Document document) {
            final JTextArea log = new JTextArea(document);
            log.setEditable(false);
            return log;
        }

        private JTable getTable(final DefaultTableModel tableModel, final JTextArea log, final JButton removeButton, final JButton publishButton) {
            final JTable table = new JTable();
            table.setModel(tableModel);
            table.setShowGrid(true);
            table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            table.getColumnModel().getColumn(1).setPreferredWidth(10);
            table.getColumnModel().getColumn(2).setPreferredWidth(20);

            final JPopupMenu menu = getPopupMenu(table, tableModel, log, removeButton, publishButton);

            table.addMouseListener(new MouseAdapter() {
                @Override
                public void mouseReleased(MouseEvent e) {
                    int i = table.rowAtPoint(e.getPoint());
                    if (i >= 0 && i < table.getRowCount()) {
                        int row = e.getY() / table.getRowHeight();
                        table.clearSelection();
                        table.setRowSelectionInterval(row, row);
                        removeButton.setEnabled(true);
                        publishButton.setEnabled(true);
                        if (SwingUtilities.isRightMouseButton(e)) {
                            menu.show(e.getComponent(), e.getX(), e.getY());
                        }
                    }
                }
            });

            return table;
        }

        private JPopupMenu getPopupMenu(JTable table, DefaultTableModel tableModel, final JTextArea logPanel, final JButton removeButton, final JButton publishButton) {
            final JPopupMenu menu = new JPopupMenu();
            final JMenuItem delete = new JMenuItem("remove");
            delete.addActionListener(getRemoveAction(table, tableModel, logPanel, removeButton, publishButton));
            final JMenuItem publish = new JMenuItem("publish");
            publish.addActionListener(getPublishExistingAction(table, logPanel, removeButton, publishButton));
            menu.add(publish);
            menu.add(delete);
            return menu;
        }

        private ActionListener getPublishExistingAction(final JTable table, final JTextArea log, final JButton removeButton, final JButton publishButton) {
            return new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {

                    final Thread c = new Thread() {
                        @Override
                        public void run() {
                            try {
                                final int row = table.getSelectedRow();
                                final URL url = new URL((String) table.getValueAt(row, 0));
                                final String title = "Publish existing file?";
                                final String msg = "Publish file: " + url.getFile();
                                final int n = JOptionPane.showConfirmDialog(GUI.this, msg, title, JOptionPane.YES_NO_OPTION);
                                if (JOptionPane.OK_OPTION == n) {
                                    enableButtons(false, removeButton, publishButton);
                                    publisher.publishExisting(url);
                                }
                            }
                            catch (Exception e1) {
                                log.append(e1.getMessage());
                                log.append("\n");
                            }
                            finally {
                                enableButtons(true, removeButton, publishButton);
                            }
                        }
                    };
                    c.start();
                }
            };
        }

        private void enableButtons(final boolean enable, final JButton... buttons) {
            for (final JButton button : buttons) {
                button.setEnabled(enable);
            }
        }

        private ActionListener getPublishNewAction(final JTextArea log, final JButton publishButton) {
            return new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {

                    final Thread c = new Thread() {
                        @Override
                        public void run() {
                            try {

                                final String fileName = file.getCanonicalPath();
                                final String title = "Publish file?";
                                final String msg = "Publish file: " + fileName;
                                final int n = JOptionPane.showConfirmDialog(GUI.this, msg, title, JOptionPane.YES_NO_OPTION);
                                if (JOptionPane.OK_OPTION == n) {
                                    enableButtons(false, publishButton);
                                    publisher.publishNew(file);
                                }
                            }
                            catch (Exception e1) {
                                log.append(e1.getMessage());
                                log.append("\n");
                            }
                            finally {
                                enableButtons(true, publishButton);
                            }
                        }
                    };
                    c.start();
                }
            };
        }

        private ActionListener getRemoveAction(final JTable table, final DefaultTableModel tableModel, final JTextArea logPanel, final JButton removeButton, final JButton publishButton) {

            return new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {

                    final int r = table.getSelectedRow();
                    final String url = (String) table.getValueAt(r, 0);

                    final String title = "Remove file?";
                    final String msg = "Remove file: " + url;
                    final int n = JOptionPane.showConfirmDialog(GUI.this, msg, title, JOptionPane.YES_NO_OPTION);
                    if (JOptionPane.OK_OPTION == n) {
                        final Thread c = new Thread() {
                            @Override
                            public void run() {
                                try {
                                    enableButtons(false, removeButton, publishButton);
                                    tableModel.removeRow(r);
                                    publisher.remove(new URL(url));
                                }
                                catch (IOException e1) {
                                    logPanel.append(e1.getMessage());
                                    logPanel.append("\n");
                                }
                                finally {
                                    enableButtons(true, removeButton, publishButton);
                                }
                            }
                        };
                        c.start();
                    }
                }
            };
        }

        private ActionListener getRefreshAction(final DefaultTableModel tableModel, final JButton removeButton, final JButton publishButton) {
            return new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    final Thread c = new Thread() {
                        @Override
                        public void run() {
                            try {
                                enableButtons(false, removeButton, publishButton);
                                tableModel.setRowCount(0);
                                final List<DavResource> davResources = publisher.getExistingFiles();
                                for (final DavResource davResource : davResources) {
                                    tableModel.addRow(new Object[]{davResource.getHref().toString(), davResource.getContentLength().toString(), davResource.getModified().toString()});
                                }
                            }
                            catch (Exception e1) {
                                e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                            }

                        }
                    };
                    c.start();
                }
            };
        }

        public JPanel getPublishPanel() {
            final JPanel headerPanel = new JPanel(new BorderLayout());
            final JButton selectButton = new JButton("Select File");
            final JTextField filePathField = new JTextField();
            filePathField.setEditable(false);
            final JButton publishButton = new JButton("Publish");
            publishButton.setEnabled(false);
            final JTextArea logArea = getLogPanel(log);

            selectButton.addActionListener(getSelectFileActionLister(filePathField, publishButton));
            publishButton.addActionListener(getPublishNewAction(logArea, publishButton));
            headerPanel.add(selectButton, BorderLayout.LINE_START);
            headerPanel.add(filePathField, BorderLayout.CENTER);
            headerPanel.add(publishButton, BorderLayout.LINE_END);

            final JPanel panel = new JPanel(new BorderLayout());
            panel.add(headerPanel, BorderLayout.PAGE_START);
            panel.add(new JScrollPane(logArea), BorderLayout.CENTER);
            return panel;
        }

        private ActionListener getSelectFileActionLister(final JTextField filePathField, final JButton publishButton) {
            return new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    final JFileChooser fc = new JFileChooser();
                    final int retVal = fc.showOpenDialog(GUI.this);
                    if (retVal == JFileChooser.APPROVE_OPTION) {
                        file = fc.getSelectedFile();
                        try {
                            filePathField.setText(file.getCanonicalPath());
                        }
                        catch (IOException e1) {
                            filePathField.setText(file.getAbsolutePath());
                        }
                        publishButton.setEnabled(true);
                    }
                }
            };
        }

        public JPanel getManagePanel() {

            final JPanel buttonPanel = new JPanel(new FlowLayout());
            final JButton refreshButton = new JButton("Refresh");
            final JButton removeButton = new JButton("Remove");
            final JButton publishButton = new JButton("Publish");
            final DefaultTableModel tableModel = new DefaultTableModel(new Object[]{"Name", "Size (Bytes)", "Modified Date"}, 0);
            final JTextArea logPanel = getLogPanel(log);
            final JTable table = getTable(tableModel, logPanel, removeButton, publishButton);

            refreshButton.addActionListener(getRefreshAction(tableModel, removeButton, publishButton));
            removeButton.addActionListener(getRemoveAction(table, tableModel, logPanel, removeButton, publishButton));
            removeButton.setEnabled(false);
            publishButton.addActionListener(getPublishExistingAction(table, logPanel, removeButton, publishButton));
            publishButton.setEnabled(false);
            buttonPanel.add(refreshButton);
            buttonPanel.add(publishButton);
            buttonPanel.add(removeButton);

            final JPanel headerPanel = new JPanel(new BorderLayout());
            headerPanel.add(buttonPanel, BorderLayout.PAGE_START);
            headerPanel.add(buttonPanel, BorderLayout.PAGE_END);

            table.setShowGrid(true);
            final JScrollPane scrollPane = new JScrollPane(table, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

            final JPanel contentPanel = new JPanel(new BorderLayout());
            contentPanel.add(headerPanel, BorderLayout.PAGE_START);
            contentPanel.add(scrollPane, BorderLayout.CENTER);
            contentPanel.setPreferredSize(new Dimension(500, 150));

            final JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, contentPanel, new JScrollPane(logPanel));
            final JPanel panel = new JPanel(new BorderLayout());
            panel.add(splitPane, BorderLayout.CENTER);
            return panel;
        }

        private void redirectSystemStreams() {
            OutputStream out = new OutputStream() {
                @Override
                public void write(final int b) throws IOException {
                    updateTextPane(String.valueOf((char) b));
                }

                @Override
                public void write(byte[] b, int off, int len) throws IOException {
                    updateTextPane(new String(b, off, len));
                }

                @Override
                public void write(byte[] b) throws IOException {
                    write(b, 0, b.length);
                }
            };

            System.setOut(new PrintStream(out, true));
            System.setErr(new PrintStream(out, true));
        }

        private void updateTextPane(final String text) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    try {
                        log.insertString(log.getLength(), text, null);
                        log.insertString(log.getLength(), "\n", null);
                    }
                    catch (BadLocationException e) {
                        throw new RuntimeException(e);
                    }
                }
            });
        }

        public static void main(String[] args) {
            final JFrame frame = new JFrame("Disk Model Publisher");
            final GUI p = new GUI();
            frame.setContentPane(p);
            frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            frame.setSize(500, 500);
            frame.pack();
            frame.setVisible(true);
        }

    }
}
