package com.onaro.sanscreen.acquisition.foundation.datasource;

import com.onaro.sanscreen.acquisition.framework.FrameworkException;
import com.onaro.sanscreen.acquisition.framework.datasource.BaseDataSource;
import com.onaro.sanscreen.acquisition.schema.odata2.OData2Root;

import java.io.Serializable;

/**
 * Interface defining what a foundation data source can report on.
 */
public interface FoundationUpdatesListener {
    /**
     * Called by data source when it finished creating the XML. This will also update the
     * status of the data source to "Success"
     *
     * @param dataSource
     * @param originatorKey
     * @param originatorDisplayName
     * @param odataRoot The generated XML
     * @param odataXML The generated XML
     * @param changed Indicates if the framework believes this data is changed from the last report
     * @param context Context object containing device specific data being passed arround by foundation to other packages
     *
     * @throws FrameworkException when could not update data to the server
     */
    public void reportOData2(final BaseDataSource dataSource,
                            final String originatorKey,
                            final String originatorDisplayName,
                            final OData2Root odataRoot,
                            final String odataXML,
                            final boolean changed,
                            final Serializable context) throws FrameworkException;

    /**
     * Updates the management URL for this data source.
     *
     * @param dataSource
     * @param manageURL
     */
    public void reportManageURL(final BaseDataSource dataSource, final String manageURL);
}
