package com.onaro.sanscreen.acquisition.foundation.standalone;

import com.onaro.sanscreen.acquisition.foundation.datasource.BaseFoundationPackage;
import com.onaro.sanscreen.acquisition.foundation.datasource.config.FoundationConfiguration;
import com.onaro.sanscreen.acquisition.framework.FrameworkException;
import com.onaro.sanscreen.acquisition.framework.datasource.BaseDataSource;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.PollingPackage;
import com.onaro.sanscreen.acquisition.framework.mgmt.ServiceManager;
import com.onaro.sanscreen.acquisition.framework.standalone.Standalone;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import java.io.IOException;

/**
 * Standalone class to run foundation polls
 */
public class FoundationStandalone extends Standalone {
    private final static Logger logger = Logger.getLogger(Standalone.class);

    public FoundationStandalone() {
        super();
    }

    /**
     * Creates an instance of the taget datasource, then run methods: start, run and stop
     *
     * @param pollDesc the polling description
     * @throws FrameworkException throws when hits framework errors
     * @throws DataSourceErrorException throws when data source is invalid
     */
    public void runPackage(final PollDescription pollDesc) throws FrameworkException, DataSourceErrorException {
        // Setup a datasource with only one package configured
        final BaseDataSource dataSource = new BaseDataSource(pollDesc.getConfiguration());

        final FoundationConfiguration foundationConfiguration = pollDesc.getConfiguration().getPackageConfiguration(BaseFoundationPackage.PACKAGE);

        final FoundationStandaloneListener listener = new FoundationStandaloneListener();

        // Instantiate the package given a different listener!
        final PollingPackage aPackage = foundationConfiguration.instantiatePackage(dataSource, listener, listener, listener, listener);
        dataSource.registerPackage(aPackage);

        // Now execute the foundation package
        dataSource.start();
        dataSource.run(BaseFoundationPackage.PACKAGE);
        dataSource.stop();
    }

    /**
     * Main class, run it as stand-alone application, usually for recording purposes.
     * @param args command line arguments
     */
    public static void main(final String[] args) {
        // We need to bootstrap the logger (with no log level)
        initLogger(Level.INFO);

        try {
            ServiceManager.getInstance().startForStandalone();
            ServiceManager.getInstance().start();

            final FoundationStandalone s = new FoundationStandalone();

            // Create an Instance of the Data-Source and run it
            final PollDescription pollDesc = s.initConfiguration(args);
            if (pollDesc != null)
                s.runPackage(pollDesc);

            ServiceManager.getInstance().stop();
        }
        catch (DataSourceErrorException e) {
            logger.error("ERROR: " + e.getMessage(), e);
        }
        catch (FrameworkException e) {
            logger.error("ERROR: " + e.getMessage(), e);
        }
        catch (IOException e) {
            logger.error("ERROR: " + e.getMessage(), e);
        }
    }
}
