package com.onaro.sanscreen.acquisition.framework.datasource;

import com.onaro.sanscreen.acquisition.framework.datasource.config.PackageConfiguration;
import com.onaro.sanscreen.acquisition.framework.recordings.RecordingSession;

import java.io.File;

public abstract class BasePackage<C extends PackageConfiguration> implements Package<C>, BasePackageMBean {
    protected String logPrefix;
    protected final BaseDataSource dataSource;
    protected final C configuration;

    private final RecordingSession recordingSession;

    protected BasePackage(final BaseDataSource dataSource, final C configuration) {
        if (dataSource == null) {
            throw new IllegalArgumentException("Data source is null");
        }

        this.dataSource = dataSource;

        if (configuration == null) {
            throw new IllegalArgumentException("Configuration is null");
        }

        this.configuration = configuration;

        recordingSession = new RecordingSession(configuration);

        this.logPrefix = "[" + getPackageName() + "] " + getConfiguration().getDataSourceName();
    }

    @Override
    public final String getPackageName() {
        return configuration.getPackageName();
    }

    @Override
    public String[] getPropertyDescriptions() {
        return configuration.getPropertyDescriptions();
    }

    public final boolean isCrawling() {
        return configuration.isCrawling();
    }

    @Override
    public boolean isRecording() {
        return configuration.isRecording();
    }

    @Override
    public void setRecording(final boolean recording) {
        configuration.setRecording(recording);
    }

    @Override
    public boolean isSimulating() {
        return configuration.isSimulating();
    }

    @Override
    public C getConfiguration() {
        return configuration;
    }

    public final File getBinDirectory() {
        return configuration.getBinDirectory();
    }

    @Override
    public BaseDataSource getDataSource() {
        return dataSource;
    }

    @Override
    public RecordingSession getRecordingSession() {
        return recordingSession;
    }

    @Override
    public final String getModel() {
        return configuration.getModel();
    }

    @Override
    public final String getVendor() {
        return configuration.getVendor();
    }

    @Override
    public String getLogPrefix() {
        return logPrefix;
    }

    @Override
    public final boolean equals(Object o) {
        if (this == o) {
            return true;
        }

        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        final BasePackage that = (BasePackage) o;

        if (!getConfiguration().getDataSourceName().equals(that.getConfiguration().getDataSourceName())) {
            return false;
        }

        return getPackageName().equals(that.getPackageName());
    }

    @Override
    public final int hashCode() {
        return dataSource.getDataSourceName().hashCode() * 29 + getPackageName().hashCode();
    }

    @Override
    public String toString() {
        return dataSource.getDataSourceName() + "/" + getPackageName();
    }
}
