package com.onaro.sanscreen.acquisition.framework.datasource.config;

import com.onaro.sanscreen.acquisition.framework.FrameworkException;
import com.onaro.sanscreen.acquisition.framework.datasource.PollingPackage;
import com.onaro.sanscreen.acquisition.framework.datasource.metadata.PackageMetaData;

import java.util.Properties;

public abstract class BasePollingPackageConfiguration<PP extends PollingPackage> extends BasePackageConfiguration<PP> implements PollingPackageConfiguration<PP>{
    // Mandatory properties
    private int pollTimeMillis;
    private int pollStartTimeMins;

    protected BasePollingPackageConfiguration(final DataSourceConfiguration configuration,
                                              final PackageMetaData metaData,
                                              final String dataSourceName,
                                              final long dataSourceID,
                                              final String packageAuditName,
                                              final Properties props,
                                              final boolean defaultForRecording) throws FrameworkException {
        super(configuration, metaData, dataSourceName, dataSourceID, packageAuditName, props, defaultForRecording);

        final String poll = (String) props.remove("poll");
        if (poll == null)
            throw new IllegalArgumentException("Missing mandatory attribute for polling package " + metaData.getName());

        pollTimeMillis = Integer.parseInt(poll) * 1000;

        String startTime = (String) props.remove("poll.starttime");
        if (startTime == null)
            startTime = "-1";

        pollStartTimeMins = Integer.parseInt(startTime);
    }

    public int getPollTimeMillis() {
        return pollTimeMillis;
    }

    public int getPollStartTimeMins() {
        return pollStartTimeMins;
    }
}
