package com.onaro.sanscreen.acquisition.framework.datasource.config;

import com.onaro.sanscreen.acquisition.framework.FrameworkException;
import com.onaro.sanscreen.acquisition.framework.datasource.BaseDataSource;
import com.onaro.sanscreen.acquisition.framework.datasource.Package;
import com.onaro.sanscreen.acquisition.framework.datasource.metadata.DataSourceMetaData;
import org.apache.log4j.Logger;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Represents the configuration of a data source (types, properties, etc...)
 */
public class DataSourceConfiguration {
    private final static Logger logger = Logger.getLogger(DataSourceConfiguration.class);

    private final DataSourceMetaData metaData;
    private final String dataSourceName;
    private final long dataSourceID;
    private Map<String, PackageConfiguration> configurations = new LinkedHashMap<String, PackageConfiguration>();

    public DataSourceConfiguration(final DataSourceMetaData metaData, final String dataSourceName, final long dataSourceID) {
        this.metaData = metaData;
        this.dataSourceName = dataSourceName;
        this.dataSourceID = dataSourceID;
    }

    public void registerPackageConfiguraiton(final PackageConfiguration packageConfiguration) {
        this.configurations.put(packageConfiguration.getPackageName(), packageConfiguration);
    }

    public final String getMetaDataName() {
        return metaData.getName();
    }

    public final String getDataSourceName() {
        return dataSourceName;
    }

    public final long getDataSourceID() {
        return dataSourceID;
    }

    public final long getDataSourceType() {
        return metaData.getDataSourceType();
    }

    public boolean isPackageRegistered(final String packageName) {
        return configurations.containsKey(packageName);
    }

    public final <PC extends PackageConfiguration> PC getPackageConfiguration(final String packageName) {
        //noinspection unchecked
        final PC packageConfiguration = (PC) configurations.get(packageName);
        assert packageConfiguration != null;

        return packageConfiguration;
    }

    public final Map<String, PackageConfiguration> getPackageConfigurations() {
        return configurations;
    }

    public BaseDataSource instantiateDataSource() throws FrameworkException {
        final BaseDataSource dataSource = new BaseDataSource(this);

        // Instantiate all the enabled package implementations given for this data source.
        for (final PackageConfiguration packageConfiguration : getPackageConfigurations().values()) {
            if ( !packageConfiguration.isEnabled())
                continue;

            if (logger.isDebugEnabled())
                logger.debug(getDataSourceName() + " - Instantiating package " + packageConfiguration.getPackageName());

            final Package aPackage = packageConfiguration.instantiatePackage(dataSource);
            dataSource.registerPackage(aPackage);
        }

        return dataSource;
    }
}

