package com.onaro.sanscreen.acquisition.framework.datasource.metadata;

import com.onaro.sanscreen.acquisition.framework.FrameworkException;
import org.jdom.Element;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

/**
 * This class represents the list of vendor-models configuration for this data source.
 */
class VendorModelSection implements Serializable {
    private final static long serialVersionUID = 0L;

    private List<Vendor> vendors = new LinkedList<Vendor>();

    @SuppressWarnings({"unchecked"})
    public VendorModelSection(final DataSourceMetaData packageMetaData, final Element root) throws FrameworkException {
        final Element vendorModels = root.getChild("vendorsModels");
        if (vendorModels == null)
            throw new FrameworkException("Missing vendor-model section for data source type " + packageMetaData.getName());

        // Iterate over all the vendors
        for (final Element vendorElement : (List<Element>)vendorModels.getChildren("vendor")) {
            final Vendor vendor = new Vendor(vendorElement.getAttributeValue("description"));
            vendors.add(vendor);

            // Iterate over all the models defined for this vendor
            for (final Element modelElement : (List<Element>)vendorElement.getChildren("model"))
                vendor.addModel(modelElement.getAttributeValue("description"));
        }
    }

    public List<Vendor> getVendors() {
        return vendors;
    }

    public static class Vendor {
        private String vendorName;
        private List<String> models = new LinkedList<String>();

        public Vendor(final String vendorName) {
            this.vendorName = vendorName;
        }

        public void addModel(final String model) {
            models.add(model);
        }

        public String getVendorName() {
            return vendorName;
        }

        public List<String> getModels() {
            return models;
        }
    }
}
