package com.onaro.sanscreen.acquisition.framework.mgmt;

import com.netapp.oci.profiler.*;
import org.apache.log4j.Logger;

import java.io.File;
import java.io.IOException;
import java.util.Timer;

public class AcqProfilerService extends BaseProfilerService {

    private static final Logger logger = Logger.getLogger(AcqProfilerService.class);

    /**
     * Prefix for the output file.
     */
    private static final String PROFILE_OUTPUT_PREFIX = "Acquisition";

    /**
     * Default sampling interval (in seconds) for the CPU and memory profiler task.
     */
    private static final String DEFAULT_SAMPLE_INTERVAL = "60";

    /**
     * System property that specifies the sampling interval (in seconds)
     */
    private static final String SAMPLE_INTERVAL_SYS_PROPERTY = "profiler.interval";

    public AcqProfilerService() throws ProfilerServiceException {
        try {
            final String[] columnHeaders = new String[] {
                    CpuMemoryProfiler.CPU_USAGE,
                    CpuMemoryProfiler.USED_HEAP_SIZE,
                    CpuMemoryProfiler.CURRENT_HEAP_SIZE,
                    CpuMemoryProfiler.MAX_HEAP_SIZE,
                    "Package Poll Start",
                    "Package Poll Finish",                    
            };
            final String acqHomeDir = System.getProperty("acq.home");
            File outputFile = new File(acqHomeDir + File.separator + "log" + File.separator + PROFILE_OUTPUT_PREFIX + "_" + System.currentTimeMillis() + ".csv");
            setDataStore(new CsvFileDataStore(outputFile, columnHeaders));
            logger.info("AcqProfilerService - Initialized!");
        } catch (IOException e) {
            throw new ProfilerServiceException(e);
        }
    }


    @Override
    public void start() {
        final Timer timer = new Timer("Acq CPU Memory Profiler");
        final ProfilerTimerTask task = new ProfilerTimerTask(this, new Profiler[] {
                new CpuMemoryProfiler(),
            }
        );
        final int timePeriodInSecs = Integer.parseInt(System.getProperty(SAMPLE_INTERVAL_SYS_PROPERTY, DEFAULT_SAMPLE_INTERVAL));
        timer.scheduleAtFixedRate(task, getProfilingStartTime(), timePeriodInSecs * 1000);
    }
}
