package com.onaro.sanscreen.acquisition.framework.mgmt;

/**
 * Allows the framework to listen to service status events. This will allow us to report status about the health of
 * the acquisition services back to the {@link ServiceManager}.
 */
public interface ServiceStatusListener {
    /**
     * The service reports an error it encountered. This error is recoverable and the acquisition can continue
     * its operation. The {@link ServiceManager} will try to report this error to the acquisition unit status
     * view.
     *
     * Note: the implementation of the distribution of this message is synchronous!!! Implementors of {@link ServiceStatusListener}
     * should not block or hold the method for too long.
     *  
     * @param serviceName
     * @param status
     * @param cause Exception causing this error (null if no exception is available)
     */
    public void error(final String serviceName, final String status, final Throwable cause);

    /**
     * The service encountered an unrecoverable error and the acquisition needs to shut down!
     *
     * Note: the implementation of the distribution of this message is synchronous!!! Implementors of {@link ServiceStatusListener}
     * should not block or hold the method for too long.
     *
     * @param serviceName
     * @param status
     * @param cause Exception causing this error (null if no exception is available)
     */
    public void fatal(final String serviceName, final String status, final Throwable cause);
}

