package com.onaro.sanscreen.acquisition.netappfoundation.standalone;

import com.onaro.sanscreen.acquisition.framework.FrameworkException;
import com.onaro.sanscreen.acquisition.framework.datasource.*;
import com.onaro.sanscreen.acquisition.netappfoundation.datasource.NetAppClockSynchronizer;
import com.onaro.sanscreen.acquisition.netappfoundation.datasource.NetAppFoundationUpdatesListener;
import com.onaro.sanscreen.acquisition.netappfoundation.datasource.NetAppOriginatorProvider;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.StorageRoot;
import org.apache.log4j.Logger;

import java.io.File;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 * Main listener for foundation events.
 */
public class NetAppFoundationStandaloneListener implements NetAppFoundationUpdatesListener, DataSourceSampleListener, NetAppOriginatorProvider, DataSourceLifeCycleListener, NetAppClockSynchronizer {
    private final static Logger logger = Logger.getLogger(NetAppFoundationStandaloneListener.class);

    protected Long nextOriginatorID = (long) 0;
    protected Map<String, Long> allocatedOriginators = new HashMap<String, Long>(20);
    protected boolean success = true;

    public NetAppFoundationStandaloneListener() {
    }

    public void dataSourceStarting(final BaseDataSource dataSource) throws FrameworkException {
        logger.info("Data Source starting");
    }

    public void dataSourceStopping(final BaseDataSource dataSource) throws FrameworkException {
        logger.info("Data Source stopping");
    }

    public void dataSourceStatusChanged(final BaseDataSource dataSource, final DataSourceStatus oldStatus, final DataSourceStatus newStatus, final String message) {
        logger.info("Data Source status changed from " + oldStatus + " to " + newStatus);
    }

    public void startingSample(final BaseDataSource dataSource, final PollingPackage pollType) {
        logger.info("Starting sample");
    }

    public void sampleSuccessfull(final BaseDataSource baseDataSource, final PollingPackage aPackage) throws FrameworkException {
        logger.info("Sample successful");
        success = true;
    }

    public void finishedSample(final BaseDataSource dataSource, final PollingPackage aPackage) throws FrameworkException {
        logger.info("Finished sample");
    }

    public void sampleFailed(final BaseDataSource dataSource, final PollingPackage aPackage, final DataSourceErrorException e) {
        logger.error("Sample failed: " + e.getMessage());
        logger.debug("Error stack", e);
        success = false;
    }

    public void sampleFailed(final BaseDataSource dataSource, final PollingPackage aPackage, final Throwable e) {
        logger.error("Sample failed: " + e.getMessage(), e);
        logger.debug("Error stack", e);
        success = false;
    }

    public long getOriginatorID(final Long datasourceID, final String originatorKey, final String originatorDisplayName) throws FrameworkException {
        // In Unit-Test we have to send the same originatorID for the old-oData as well as the new-oData.
        // otherwise we'll get a mismatch between the samples.
        Long originatorID = allocatedOriginators.get(originatorKey);
        if (originatorID == null) {
            originatorID = nextOriginatorID++;
            allocatedOriginators.put(originatorKey, originatorID);
        }

        return originatorID;
    }

    public void deleteOriginatorID(final Long originatorID, final String originatorKey) throws FrameworkException {
        allocatedOriginators.remove(originatorKey);
    }

    public String[] getOriginatorCache() {
        // Do nothing...
        return new String[0];
    }

    public void cleanOriginatorCache() {
        // Do nothing...
    }

    /**
     * Called by the datasource the first time it polls to set the Management URL for the datasource itself.
     *
     * @param manageURL url
     */
    public void reportManageURL(final BaseDataSource dataSource, final String manageURL) {
        // Do nothing
    }

    public void recordingSessionDone(final BaseDataSource dataSource,
                                       final PollingPackage aPackage,
                                       final File recordingSession,
                                       final boolean shouldReport) throws FrameworkException {
        logger.info("Reported Recording Session!");
    }

    @Override
    public void reportNetAppData(BaseDataSource dataSource, String originatorKey, String originatorDisplayName, StorageRoot netAppRoot, final File netappDataFile, boolean changed, Serializable context) throws FrameworkException {
        logger.info("Sample successful!");
    }

    public boolean isSuccess() {
        return success;
    }

    @Override
    public long getServerTime() throws FrameworkException {
        return System.currentTimeMillis();
    }
}
