package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEValue;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

/**
 * @author Iti Behari
 *         Date: 7/9/13
 */
@OCIETable("application_record")
@OCIEResourceType("application_record")
public class ApplicationRecord extends BaseCRCXMLNode<Cluster> {

    public static final class ApplicationRecordKey extends KeyBase<ApplicationRecordKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "cluster-identity-get",
                    zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                    sourceZapi = "cluster-application-record-get-iter",
                    zapiResponsePath = "netapp.results.attributes.cluster-application-record-info.record-name"
            )
            record_name
        }

        public ApplicationRecordKey(final String clusterUUID, final String applicationRecordName) {
            super("application_record", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.record_name, applicationRecordName);
            }});
        }

        public String getOcieUUID() {
            return getFQID();
        }
    }

    private final ApplicationRecordKey key;

    @OCIEZapiSource(
            sourceZapi = "cluster-application-record-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-application-record-info.record-name"
    )
    @OCIEValue(
            ocieColumn = "recordName"
    )
    private String recordName;

    @OCIEZapiSource(
            sourceZapi = "cluster-application-record-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-application-record-info.record-value"
    )
    @OCIEValue(
            ocieColumn = "recordValue"
    )
    private String recordValue;

    @OCIEZapiSource(
            sourceZapi = "cluster-application-record-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-application-record-info.modified-from"
    )
    @OCIEValue(
            ocieColumn = "modifiedFrom"
    )
    private String modifiedFrom;

    @OCIEZapiSource(
            sourceZapi = "cluster-application-record-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-application-record-info.last-modified-timestamp"
    )
    @OCIEValue(
            ocieColumn = "lastModifiedTimestamp"
    )
    private Long lastModifiedTimestamp;

    ApplicationRecord(final Cluster parentCluster, final String applicationRecordName) {
        super(parentCluster, ApplicationRecord.class.getSimpleName());
        this.key = new ApplicationRecordKey(parentCluster.getClusterUUID(), applicationRecordName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public ApplicationRecordKey getKey() {
        return key;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addAttributeNoCRC(writer, "RecordName", recordName);
        addAttributeNoCRC(writer, "RecordValue", recordValue);
        addAttributeNoCRC(writer, "ModifiedFrom", modifiedFrom);
        addAttributeNoCRC(writer, "LastModifiedTimestamp", lastModifiedTimestamp);
    }


    @SuppressWarnings({"UnusedDeclaration"})
    public String getRecordName() {
        return recordName;
    }

    public void setRecordName(final String recordName) {
        this.recordName = recordName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getRecordValue() {
        return recordValue;
    }

    public void setRecordValue(final String recordValue) {
        this.recordValue = recordValue;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getModifiedFrom() {
        return modifiedFrom;
    }

    public void setModifiedFrom(final String modifiedFrom) {
        this.modifiedFrom = modifiedFrom;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getLastModifiedTimestamp() {
        return lastModifiedTimestamp;
    }

    public void setLastModifiedTimestamp(final Long lastModifiedTimestamp) {
        this.lastModifiedTimestamp = lastModifiedTimestamp;
    }


}

