package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.*;

import java.io.IOException;
import java.util.EnumMap;


@OCIETable("cluster_peer")
@OCIEResourceType("cluster_peer")
public class ClusterPeer extends BaseCRCXMLNode<Cluster> {

    public final static class ClusterPeerKey extends KeyBase<ClusterPeerKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "cluster-peer-get-iter",
                    zapiResponsePath = "netapp.results.attributes.cluster-peer-info.cluster-uuid"
            )
            remote_cluster_uuid,
        }

        public ClusterPeerKey(final String remoteClusterUUID) {
            super("cluster_peer", new EnumMap<Field, String>(Field.class) {{
                put(Field.remote_cluster_uuid, remoteClusterUUID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getID(Field.remote_cluster_uuid);
        }
    }

    private final ClusterPeerKey key;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.active-addresses"
    )
    @OCIEValue(
            ocieColumn = "activeAddresses"
    )
    private String activeAddresses;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.address-family"
    )
    @OCIEValue(
            ocieColumn = "addressFamily"
    )
    private AddressFamily addressFamily;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.address-family"
    )
    @OCIEValue(
            ocieColumn = "addressFamilyRaw"
    )
    private String addressFamilyRaw;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.auth-status-admin"
    )
    @OCIEValue(
            ocieColumn = "authStatusAdmin"
    )
    private String authStatusAdmin;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.auth-status-operational"
    )
    @OCIEValue(
            ocieColumn = "authStatusOperational"
    )
    private AuthStatusOperational authStatusOperational;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.auth-status-operational"
    )
    @OCIEValue(
            ocieColumn = "authStatusOperationalRaw"
    )
    private String authStatusOperationalRaw;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.availability"
    )
    @OCIEValue(
            ocieColumn = "availability"
    )
    private Availability availability;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.availability"
    )
    @OCIEValue(
            ocieColumn = "availabilityRaw"
    )
    private String availabilityRaw;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.cluster-name"
    )
    @OCIEValue(
            ocieColumn = "remoteClusterName"
    )
    private String remoteClusterName;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.cluster-uuid"
    )
    @OCIEValue(
            ocieColumn = "remoteClusterUUID"
    )
    private final String remoteClusterUUID;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.ipspace-name"
    )
    @OCIEValue(
            ocieColumn = "networkIpSpaceId"
    )
    private NetworkIpSpace.NetworkIpSpaceKey ipSpaceKey;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.is-cluster-healthy"
    )
    @OCIEValue(
            ocieColumn = "isClusterHealthy"
    )
    private Boolean isClusterHealthy;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.peer-addresses"
    )
    @OCIEValue(
            ocieColumn = "peerAddresses"
    )
    private String peerAddresses;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.remote-cluster-nodes"
    )
    @OCIEValue(
            ocieColumn = "remoteClusterNodes"
    )
    private String remoteClusterNodes;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.serial-number"
    )
    @OCIEValue(
            ocieColumn = "serialNumber"
    )
    private String serialNumber;

    @OCIEZapiSource(
            sourceZapi = "cluster-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.cluster-peer-info.unreachable-local-nodes"
    )
    @OCIEValue(
            ocieColumn = "unreachableLocalNodes"
    )
    private String unreachableLocalNodes;

    ClusterPeer(final Cluster parent, final String remoteClusterUUID) {
        super(parent, ClusterPeer.class.getSimpleName());
        this.key = new ClusterPeerKey(remoteClusterUUID);
        this.remoteClusterUUID = remoteClusterUUID;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum AddressFamily {
        NOT_MAPPED,
        IPV4,
        IPV6,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Availability {
        NOT_MAPPED,
        UNAVAILABLE,
        AVAILABLE,
        PARTIAL,
        PENDING,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum AuthStatusOperational {
        NOT_MAPPED,
        OK,
        ABSENT,
        PENDING,
        EXPIRED,
        REVOKED,
        DECLINED,
        REFUSED,
        OK_AND_OFFER,
        ABSENT_BUT_OFFER,
        REVOKED_BUT_OFFER,
        INTENT_MISMATCH,
        KEY_MISMATCH,
        INCAPABLE,
    }

    public void setActiveAddresses(String activeAddresses) {
        this.activeAddresses = activeAddresses;
    }

    public void setAddressFamily(AddressFamily addressFamily) {
        this.addressFamily = addressFamily;
    }

    public void setAddressFamilyRaw(String addressFamilyRaw) {
        this.addressFamilyRaw = addressFamilyRaw;
    }

    public void setAuthStatusAdmin(String authStatusAdmin) {
        this.authStatusAdmin = authStatusAdmin;
    }

    public void setAuthStatusOperational(AuthStatusOperational authStatusOperational) {
        this.authStatusOperational = authStatusOperational;
    }

    public void setAuthStatusOperationalRaw(String authStatusOperationalRaw) {
        this.authStatusOperationalRaw = authStatusOperationalRaw;
    }

    public void setAvailability(Availability availability) {
        this.availability = availability;
    }

    public void setAvailabilityRaw(String availabilityRaw) {
        this.availabilityRaw = availabilityRaw;
    }

    public void setRemoteClusterName(String remoteClusterName) {
        this.remoteClusterName = remoteClusterName;
    }

    public void setIpSpaceKey(NetworkIpSpace.NetworkIpSpaceKey ipSpaceKey) {
        this.ipSpaceKey = ipSpaceKey;
    }

    public void setClusterHealthy(Boolean clusterHealthy) {
        isClusterHealthy = clusterHealthy;
    }

    public void setPeerAddresses(String peerAddresses) {
        this.peerAddresses = peerAddresses;
    }

    public void setRemoteClusterNodes(String remoteClusterNodes) {
        this.remoteClusterNodes = remoteClusterNodes;
    }

    public void setSerialNumber(String serialNumber) {
        this.serialNumber = serialNumber;
    }

    public void setUnreachableLocalNodes(String unreachableLocalNodes) {
        this.unreachableLocalNodes = unreachableLocalNodes;
    }

    public ClusterPeerKey getKey() {
        return key;
    }

    public String getActiveAddresses() {
        return activeAddresses;
    }

    public AddressFamily getAddressFamily() {
        return addressFamily;
    }

    public String getAddressFamilyRaw() {
        return addressFamilyRaw;
    }

    public String getAuthStatusAdmin() {
        return authStatusAdmin;
    }

    public AuthStatusOperational getAuthStatusOperational() {
        return authStatusOperational;
    }

    public String getAuthStatusOperationalRaw() {
        return authStatusOperationalRaw;
    }

    public Availability getAvailability() {
        return availability;
    }

    public String getAvailabilityRaw() {
        return availabilityRaw;
    }

    public String getRemoteClusterName() {
        return remoteClusterName;
    }

    public String getRemoteClusterUUID() {
        return remoteClusterUUID;
    }

    public NetworkIpSpace.NetworkIpSpaceKey getIpSpaceKey() {
        return ipSpaceKey;
    }

    public Boolean getClusterHealthy() {
        return isClusterHealthy;
    }

    public String getPeerAddresses() {
        return peerAddresses;
    }

    public String getRemoteClusterNodes() {
        return remoteClusterNodes;
    }

    public String getSerialNumber() {
        return serialNumber;
    }

    public String getUnreachableLocalNodes() {
        return unreachableLocalNodes;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "ActiveAddresses", activeAddresses);
        addRequiredAttributeAndCRC(writer, "AddressFamily", addressFamily);
        addRequiredAttributeAndCRC(writer, "AddressFamilyRaw", addressFamilyRaw);
        addRequiredAttributeAndCRC(writer, "AuthStatusAdmin", authStatusAdmin);
        addRequiredAttributeAndCRC(writer, "AuthStatusOperational", authStatusOperational);
        addRequiredAttributeAndCRC(writer, "AuthStatusOperationalRaw", authStatusOperationalRaw);
        addRequiredAttributeAndCRC(writer, "Availability", availability);
        addRequiredAttributeAndCRC(writer, "AvailabilityRaw", availabilityRaw);
        addRequiredAttributeAndCRC(writer, "RemoteClusterName", remoteClusterName);
        addRequiredAttributeAndCRC(writer, "RemoteClusterUUID", remoteClusterUUID);
        addRequiredAttributeAndCRC(writer, "IpSpaceUUID", ipSpaceKey != null ? ipSpaceKey.getOcieUUID() : null);
        addRequiredAttributeAndCRC(writer, "IsClusterHealthy", isClusterHealthy);
        addRequiredAttributeAndCRC(writer, "PeerAddresses", peerAddresses);
        addRequiredAttributeAndCRC(writer, "RemoteClusterNodes", remoteClusterNodes);
        addRequiredAttributeAndCRC(writer, "SerialNumber", serialNumber);
        addRequiredAttributeAndCRC(writer, "UnreachableLocalNodes", unreachableLocalNodes);
    }
}
