package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("disk_aggregate_relationship")
@OCIEResourceType("disk_aggregate_relationship")
public class DiskAggregateRelationship extends BaseCRCXMLNode<Cluster> {

    public final static class DiskAggregateRelationshipKey extends KeyBase<DiskAggregateRelationshipKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "aggr-status-get-iter",
                    zapiResponsePath = "netapp.results.attributes.aggr-status-info.aggr-plex-list.aggr-plex-info.aggr-raidgroup-list.aggr-raidgroup-info.aggr-disk-list.aggr-disk-info.disk"
            )
            disk_uuid,

            @OCIEZapiSource(
                    sourceZapi = "aggr-status-get-iter",
                    zapiResponsePath = "netapp.results.attributes.aggr-status-info.aggregate-uuid"
            )
            aggregate_uuid
        }

        public DiskAggregateRelationshipKey(final String diskUUID, final String aggregateUUID) {
            super("disk_aggregate_relationship",
                    new EnumMap<Field, String>(Field.class) {{
                        put(Field.disk_uuid, diskUUID);
                        put(Field.aggregate_uuid, aggregateUUID);
                    }});
        }

        @Override
        @OCIEUUID(uuidDescr = "disk_uuid.aggregate_uuid")
        public String getOcieUUID() {
            final String diskUUID = getID(Field.disk_uuid);
            final String aggregateUUID = getID(Field.aggregate_uuid);
            return diskUUID + '.' + aggregateUUID;
        }
    }

    @OCIEZapiSource(
            sourceZapi = "aggr-status-get-iter",
            zapiResponsePath = "netapp.results.attributes.aggr-status-info.aggr-plex-list.aggr-plex-info.aggr-raidgroup-list.aggr-raidgroup-info.aggr-disk-list.aggr-disk-info.disk"
    )
    @OCIEValue(
            ocieColumn = "diskId"
    )
    private final Disk.DiskKey diskKey;

    @OCIEZapiSource(
            sourceZapi = "aggr-status-get-iter",
            zapiResponsePath = "netapp.results.attributes.aggr-status-info.aggregate-uuid"
    )
    @OCIEValue(
            ocieColumn = "aggregateId"
    )
    private final Aggregate.AggregateKey aggregateKey;


    @OCIEZapiSource(
            sourceZapi = "aggr-status-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.aggr-status-info.aggr-plex-list.aggr-raidgroup-info.aggr-disk-info.is-cache"
    )
    @OCIEValue(
            ocieColumn = "isCache"
    )
    private Boolean isCache;



    private final DiskAggregateRelationshipKey key;


    protected DiskAggregateRelationship(final Cluster cluster, final Disk.DiskKey diskKey, final Aggregate.AggregateKey aggregateKey) {
        super(cluster, DiskAggregateRelationship.class.getSimpleName());
        this.key = new DiskAggregateRelationshipKey(diskKey.getOcieUUID(), aggregateKey.getOcieUUID());
        this.diskKey = diskKey;
        this.aggregateKey = aggregateKey;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public DiskAggregateRelationshipKey getKey() {
        return this.key;
    }

    @SuppressWarnings({"unused declaration"})
    public Disk.DiskKey getDiskKey() {
        return diskKey;
    }

    @SuppressWarnings({"unused declaration"})
    public Aggregate.AggregateKey getAggregateKey() {
        return aggregateKey;
    }

    public Boolean getIsCache() {
        return isCache;
    }

    public void setIsCache(Boolean cache) {
        isCache = cache;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(cache, writer);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "DiskUUID", diskKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "AggregateUUID", aggregateKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "IsCache", isCache);
    }
}
