package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import java.io.IOException;
import java.util.EnumMap;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEUUID;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

@SuppressWarnings("serial")
public class DoNotDelete extends BaseCRCXMLNode<Cluster>{
    public final static class DoNotDeleteKey extends KeyBase<DoNotDeleteKey.Field> {
        public static enum Field {
            uuid
        }

        public DoNotDeleteKey(final String uuid) {
            super("doNotDelete",
                  new EnumMap<Field, String>(Field.class) {{
                        put(Field.uuid, uuid);
                  }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getID(Field.uuid);
        }
    }
    
    private final DoNotDeleteKey key;
    private String objectTagName;
    private String objectKey;
    
    private final AlternateKeys alternateKeys;
    
    DoNotDelete(final Cluster parentCluster, final String uuid) {
        super(parentCluster, DoNotDelete.class.getSimpleName());
        this.key = new DoNotDeleteKey(uuid);
        this.alternateKeys = new AlternateKeys(this);
    }
    
    @Override
    @SuppressWarnings({"unchecked"})
    public DoNotDeleteKey getKey() {
        return key;
    }
    
    public String getObjectTagName() {
        return objectTagName;
    }
    
    public void setObjectTagName(String objectTagName) {
        this.objectTagName = objectTagName;
    }
    
    public String getObjectKey() {
        return objectKey;
    }

    public void setObjectKey(String objectKey) {
        this.objectKey = objectKey;
    }
    
    public AlternateKey createAlternateKey(String uuid) {
        return new AlternateKey(this, uuid);
    }
    
    public AlternateKeys getAlternateKeys() {
        return alternateKeys;
    }
    
    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ObjectTagName", objectTagName);
        addRequiredAttributeAndCRC(writer, "ObjectKey", objectKey);
    }
    
    @Override
    protected boolean hasChildren() {
        return !alternateKeys.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, "AlternateKeys", alternateKeys.values(), tabs);   
    }
    
}
