package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("export_policy")
@OCIEResourceType("export_policy")
public class ExportPolicy extends BaseCRCXMLNode<VServer> {

    public static final class ExportPolicyKey extends KeyBase<ExportPolicyKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "export-policy-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.export-policy-info.policy-id"
            )
            uuid
        }

        public ExportPolicyKey(final String clusterUUID, final String exportPolicyID) {
            super("export_policy", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.uuid, exportPolicyID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.uuid")
        public String getOcieUUID() {
            return getFQID();
        }
    }

    private final ExportPolicyKey key;

    @OCIEZapiSource(
        sourceZapi = "export-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.export-policy-info.policy-name"
    )
    @OCIEValue(
        ocieColumn = "name"
    )
    private String name;

    // Children
    private final ExportRules exportRules;

    ExportPolicy(final VServer parentVServer, final Long policyID) {
        super(parentVServer, ExportPolicy.class.getSimpleName());
        final String clusterUUID = parentVServer.getKey().getID(VServer.VServerKey.Field.cluster_uuid);
        this.key = new ExportPolicyKey(clusterUUID, String.valueOf(policyID));
        this.exportRules = new ExportRules(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public ExportPolicyKey getKey() {
        return key;
    }

    public String getPolicyID() {
        return key.getID(ExportPolicyKey.Field.uuid);
    }

    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public ExportRules getExportRules() {
        return exportRules;
    }

    public ExportRule createExportRule(final VServer.VServerKey vserverKey, final String ruleIndex) {
        return new ExportRule(this, vserverKey, ruleIndex);
    }



    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws SchemaException, IOException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", name);
        addRequiredAttributeAndCRC(writer, "PolicyID", key.getID(ExportPolicyKey.Field.uuid));
    }

    @Override
    protected boolean hasChildren() {
        return !exportRules.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(marshallingCache, writer, "ExportRules", exportRules.values(), tabs);
    }
}
