package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("fcp_lif")
@OCIEResourceType("fcp_lif")
public class FcpLif extends BaseCRCXMLNode<VServer> {

    public static final class LifKey extends KeyBase<LifKey.Field> {
        final String lifType;
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

// Can't have 2 :( but here the lif_uuid is compound with 2 fields
//            @OCIEZapiSource(
//                sourceZapi = "vserver-get-iter",
//                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
//            )
            @OCIEZapiSource(
                sourceZapi = "net-interface-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.net-interface-info.lif-uuid"
            )
            lif_uuid
        }

        public LifKey(final String lifType, final String clusterUUID, final String vServerUUID, final String lifUUID) {
            super(lifType, new EnumMap<Field, String>(Field.class){{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);       // Awkwardness between vserver_uuid & lif_uuid that is synthetic, compound, and also contains vServer UUID
                put(Field.lif_uuid, lifUUID);
            }});
            this.lifType = lifType;
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.uuid")
        public String getOcieUUID() {
            return getID(Field.lif_uuid);
        }

        @Override
        public String getResourceKey() {
            return getID(Field.cluster_uuid) + ":type=" + lifType + ",lif_uuid=" + getID(Field.lif_uuid);
        }
    }

    private final LifKey key;

    @OCIEZapiSource(
        sourceZapi = "fcp-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-interface-info.interface-name"
    )
    @OCIEValue(
        ocieColumn = "interfaceName"
    )
    private String interfaceName;

    @OCIEZapiSource(
        sourceZapi = "fcp-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-interface-info.current-node"
    )
    @OCIEValue(
        ocieColumn = "currentNodeId"
    )
    private Node.NodeKey currentNodeKey;

    @OCIEZapiSource(
        sourceZapi = "fcp-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-interface-info.current-port"
    )
    @OCIEValue(
        ocieColumn = "currentPortId"
    )
    private FcpPort.FcpPortKey currentPortKey;

    @OCIEZapiSource(
        sourceZapi = "fcp-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-interface-info.home-node"
    )
    @OCIEValue(
        ocieColumn = "homeNodeId"
    )
    private Node.NodeKey homeNodeKey;

    @OCIEZapiSource(
        sourceZapi = "fcp-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-interface-info.home-port"
    )
    @OCIEValue(
        ocieColumn = "homePortId"
    )
    private FcpPort.FcpPortKey homePortKey;

    @OCIEZapiSource(
        sourceZapi = "net-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-interface-info.is-auto-revert"
    )
    @OCIEValue(
        ocieColumn = "isAutoRevert"
    )
    private Boolean isAutoRevert;


    @OCIEZapiSource(
        sourceZapi = "net-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-interface-info.is-home"
    )
    @OCIEValue(
        ocieColumn = "isHome"
    )
    private Boolean isHome;

    @OCIEZapiSource(
        sourceZapi = "fcp-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-interface-info.node-name"
    )
    @OCIEValue(
        ocieColumn = "nodeName"
    )
    private String nodeName;

    @OCIEZapiSource(
        sourceZapi = "fcp-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-interface-info.port-name"
    )
    @OCIEValue(
        ocieColumn = "portName"
    )
    private String portName;

    @OCIEZapiSource(
        sourceZapi = "net-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-interface-info.operational-status"
    )
    @OCIEValue(
        ocieColumn = "operStatus",
        notes = "derived enum"
    )
    private Lif.Status operStatus;

    @OCIEZapiSource(
        sourceZapi = "net-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-interface-info.operational-status"
    )
    @OCIEValue(
        ocieColumn = "operStatusRaw"
    )
    private String operStatusRaw;


    @OCIEZapiSource(
        sourceZapi = "net-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-interface-info.administrative-status"
    )
    @OCIEValue(
        ocieColumn = "adminStatus"
    )
    private Lif.Status adminStatus;

    @OCIEZapiSource(
        sourceZapi = "net-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-interface-info.administrative-status"
    )
    @OCIEValue(
        ocieColumn = "adminStatusRaw",
        notes = "derived enum"
    )
    private String adminStatusRaw;

    @OCIEZapiSource(
        sourceZapi = "net-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-interface-info.role"
    )
    @OCIEValue(
        ocieColumn = "role"
    )
    private Lif.Role role;

    @OCIEZapiSource(
        sourceZapi = "net-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-interface-info.role"
    )
    @OCIEValue(
        ocieColumn = "roleRaw"
    )
    private String roleRaw;

    @OCIEZapiSource(
        sourceZapi = "net-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-interface-info.data-protocols.data-protocol"
    )
    @OCIEValue(
        ocieColumn = "dataProtocols",
        notes = "CSV list"
    )
    private String dataProtocols;

    @OCIEZapiSource(
        sourceZapi = "net-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-interface-info.failover-group"
    )
    @OCIEValue(
        ocieColumn = "failoverGroup"
    )
    private String failoverGroupName;

    FcpLif(final VServer parent, final String lifName) {
        super(parent, "FCPLIF");
        final String clusterUUID = parent.getKey().getID(VServer.VServerKey.Field.cluster_uuid);
        final String vServerUUID = parent.getKey().getID(VServer.VServerKey.Field.uuid);
        this.key = new LifKey("fcp_lif", clusterUUID, vServerUUID, lifName);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getFailoverGroupName()
    {
      return failoverGroupName;
    }

    public void setFailoverGroupName(String failoverGroupName)
    {
      this.failoverGroupName = failoverGroupName;
    }

  @Override
    @SuppressWarnings({"unchecked"})
    public LifKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Node.NodeKey getCurrentNodeKey() {
        return currentNodeKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public FcpPort.FcpPortKey getCurrentPortKey() {
        return currentPortKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Node.NodeKey getHomeNodeKey() {
        return homeNodeKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public FcpPort.FcpPortKey getHomePortKey() {
        return homePortKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getAutoRevert() {
        return isAutoRevert;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getHome() {
        return isHome;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getPortWWN() {
        return portName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getNodeWWN() {
        return nodeName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Lif.Status getOperStatus() {
        return operStatus;
    }

    @SuppressWarnings("UnusedDeclaration")
    public String getOperStatusRaw() {
        return operStatusRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Lif.Status getAdminStatus() {
        return adminStatus;
    }

    @SuppressWarnings("UnusedDeclaration")
    public String getAdminStatusRaw() {
        return adminStatusRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Lif.Role getRole() {
        return role;
    }

    @SuppressWarnings("UnusedDeclaration")
    public String getRoleRaw() {
        return roleRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getDataProtocols() {
        return dataProtocols;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getInterfaceName() {
        return interfaceName;
    }

    public void setInterfaceName(final String interfaceName) {
        this.interfaceName = interfaceName;
    }

    public void setCurrentNodeKey(final Node.NodeKey currentNodeKey) {
        this.currentNodeKey = currentNodeKey;
    }

    public void setCurrentPortKey(final FcpPort.FcpPortKey currentPortKey) {
        this.currentPortKey = currentPortKey;
    }

    public void setHomeNodeKey(final Node.NodeKey homeNodeKey) {
        this.homeNodeKey = homeNodeKey;
    }

    public void setHomePortKey(final FcpPort.FcpPortKey homePortKey) {
        this.homePortKey = homePortKey;
    }

    public void setAutoRevert(final Boolean autoRevert) {
        isAutoRevert = autoRevert;
    }

    public void setHome(final Boolean home) {
        isHome = home;
    }

    public void setNodeName(final String nodeName) {
        this.nodeName = nodeName;
    }

    public void setPortName(final String portName) {
        this.portName = portName;
    }

    public void setOperStatus(final Lif.Status operStatus) {
        this.operStatus = operStatus;
    }

    public void setOperStatusRaw(final String operStatusRaw) {
        this.operStatusRaw = operStatusRaw;
    }

    public void setAdminStatus(final Lif.Status adminStatus) {
        this.adminStatus = adminStatus;
    }

    public void setAdminStatusRaw(final String adminStatusRaw) {
        this.adminStatusRaw = adminStatusRaw;
    }

    public void setRole(final Lif.Role role) {
        this.role = role;
    }

    public void setRoleRaw(final String roleRaw) {
        this.roleRaw = roleRaw;
    }

    public void setDataProtocols(final String dataProtocols) {
        this.dataProtocols = dataProtocols;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", interfaceName);
        addRequiredAttributeAndCRC(writer, "CurrentNodeUUID", KeyBase.getOcieUUIDOrNull(currentNodeKey));
        addRequiredAttributeAndCRC(writer, "CurrentPortUUID", KeyBase.getOcieUUIDOrNull(currentPortKey));
        addRequiredAttributeAndCRC(writer, "HomeNodeUUID", KeyBase.getOcieUUIDOrNull(homeNodeKey));
        addRequiredAttributeAndCRC(writer, "HomePortUUID", KeyBase.getOcieUUIDOrNull(homePortKey));
        addRequiredAttributeAndCRC(writer, "IsAutoRevert", isAutoRevert);
        addRequiredAttributeAndCRC(writer, "IsHome", isHome);
        addRequiredAttributeAndCRC(writer, "NodeName", nodeName);
        addRequiredAttributeAndCRC(writer, "PortName", portName);
        addRequiredAttributeAndCRC(writer, "OperationalStatus", operStatus);
        addRequiredAttributeAndCRC(writer, "OperationalStatusRaw", operStatusRaw);
        addRequiredAttributeAndCRC(writer, "AdministrativeStatus", adminStatus);
        addRequiredAttributeAndCRC(writer, "AdministrativeStatusRaw", adminStatusRaw);
        addRequiredAttributeAndCRC(writer, "Role", role);
        addRequiredAttributeAndCRC(writer, "RoleRaw", roleRaw);
        addRequiredAttributeAndCRC(writer, "DataProtocols", dataProtocols);
        addRequiredAttributeAndCRC(writer, "FailoverGroup",  failoverGroupName );
    }
}
