package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("fcp_port")
@OCIEResourceType("fcp_port")
public class FcpPort extends BaseCRCXMLNode<Node> {

    public final static class FcpPortKey extends KeyBase<FcpPortKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "system-node-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.node-details-info.node-uuid"
            )
            cluster_node_uuid,

            @OCIEZapiSource(
                sourceZapi = "fcp-adapter-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.node-name"
            )
            node_name,                  // this is the node WWN, resource key wants the label to be "node_name"

            @OCIEZapiSource(
                sourceZapi = "fcp-adapter-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.port-name"
            )
            port_name,                  // this is the port WWN, resource key wants the label to be "port_name"

            @OCIEZapiSource(
                sourceZapi = "fcp-adapter-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.adapter"
            )
            adapter
        }

        public FcpPortKey(final String clusterUUID, final String nodeUUID, final String nodeWWN, final String portWWN, final String adapter) {
            super("fcp_port", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.cluster_node_uuid, nodeUUID);
                put(Field.node_name, nodeWWN);
                put(Field.port_name, portWWN);
                put(Field.adapter, adapter);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "adapter.port_name.cluster_node_uuid")
        public String getOcieUUID() {
            return getID(Field.adapter) + '.' + getID(Field.port_name) + '.' + getID(Field.cluster_node_uuid);
        }
    }

    private final FcpPortKey key;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.node-name"
    )
    @OCIEValue(
        ocieColumn = "nodeName"
    )
    private String nodeWWN;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.max-speed"
    )
    @OCIEValue(
        ocieColumn = "maxSpeed"
    )
    private Integer maxSpeed;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.physical-protocol"
    )
    @OCIEValue(
        ocieColumn = "physicalProtocol",
        notes = "derived enum"
    )
    private PhysicalProtocol physicalProtocol;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.physical-protocol"
    )
    @OCIEValue(
        ocieColumn = "physicalProtocolRaw"
    )
    private String physicalProtocolRaw;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.speed"
    )
    @OCIEValue(
        ocieColumn = "speed",
        notes = "derived enum"
    )
    private Speed speed;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.speed"
    )
    @OCIEValue(
        ocieColumn = "speed"
    )
    private String speedRaw;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.state"
    )
    @OCIEValue(
        ocieColumn = "state",
        notes = "derived enum"
    )
    private State state;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.state"
    )
    @OCIEValue(
        ocieColumn = "state"
    )
    private String stateRaw;

    @OCIEZapiSource(
        sourceZapi = "fc-config-list-info",
        zapiResponsePath = "netapp.results.attributes-list.fc-config.adapter-status"
    )
    @OCIEValue(
        ocieColumn = "status",
        notes = "derived enum"
    )
    private Status status;

    @OCIEZapiSource(
        sourceZapi = "fc-config-list-info",
        zapiResponsePath = "netapp.results.attributes-list.fc-config.adapter-status"
    )
    @OCIEValue(
        ocieColumn = "status"
    )
    private String statusRaw;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.switch-port"
    )
    @OCIEValue(
        ocieColumn = "switchPort"
    )
    private String switchPort;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.data-link-rate"
    )
    @OCIEValue(
        ocieColumn = "dataLinkRate"
    )
    private Integer dataLinkRate;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.media-type"
    )
    @OCIEValue(
        ocieColumn = "mediaType",
        notes = "derived enum"
    )
    private MediaType mediaType;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.media-type"
    )
    @OCIEValue(
        ocieColumn = "mediaTypeRaw"
    )
    private String mediaTypeRaw;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.fabric-established"
    )
    @OCIEValue(
        ocieColumn = "fabricEstablished"
    )
    private Boolean fabricEstablished;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.firmware-rev"
    )
    @OCIEValue(
        ocieColumn = "firmwareRev"
    )
    private String firmwareRev;

    @OCIEZapiSource(
        sourceZapi = "fcp-adapter-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.fcp-config-adapter-info.info-name"
    )
    @OCIEValue(
        ocieColumn = "infoName"
    )
    private String infoName;

    FcpPort(final Node parentNode, final String nodeWWN, final String portWWN, final String adapterName) {
        super(parentNode, "FcpPort");
        final Node.NodeKey nodeKey = parentNode.getKey();
        final String clusterUUID = nodeKey.getID(Node.NodeKey.Field.cluster_uuid);
        final String nodeUUID = nodeKey.getID(Node.NodeKey.Field.uuid);
        this.key = new FcpPortKey(clusterUUID, nodeUUID, nodeWWN, portWWN, adapterName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public FcpPortKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getNodeWWN() {
        return nodeWWN;
    }

    public String getName() {
        return key.getID(FcpPortKey.Field.adapter);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getMaxSpeed() {
        return maxSpeed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public PhysicalProtocol getPhysicalProtocol() {
        return physicalProtocol;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getPhysicalProtocolRaw() {
        return physicalProtocolRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Speed getSpeed() {
        return speed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getSpeedRaw() {
        return speedRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public State getState() {
        return state;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getStateRaw() {
        return stateRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Status getStatus() {
        return status;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getStatusRaw() {
        return statusRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getSwitchPort() {
        return switchPort;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getDataLinkRate() {
        return dataLinkRate;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public MediaType getMediaType() {
        return mediaType;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getMediaTypeRaw() {
        return mediaTypeRaw;
    }

    public void setNodeWWN(final String nodeWWN) {
        this.nodeWWN = nodeWWN;
    }

    public void setMaxSpeed(final Integer maxSpeed) {
        this.maxSpeed = maxSpeed;
    }

    public void setPhysicalProtocol(final PhysicalProtocol physicalProtocol) {
        this.physicalProtocol = physicalProtocol;
    }

    public void setPhysicalProtocolRaw(final String physicalProtocolRaw) {
        this.physicalProtocolRaw = physicalProtocolRaw;
    }

    public void setSpeed(final Speed speed) {
        this.speed = speed;
    }

    public void setSpeedRaw(final String speedRaw) {
        this.speedRaw = speedRaw;
    }

    public void setState(final State state) {
        this.state = state;
    }

    public void setStateRaw(final String stateRaw) {
        this.stateRaw = stateRaw;
    }

    public void setStatus(final Status status) {
        this.status = status;
    }

    public void setStatusRaw(final String statusRaw) {
        this.statusRaw = statusRaw;
    }

    public void setSwitchPort(final String switchPort) {
        this.switchPort = switchPort;
    }

    public void setDataLinkRate(final Integer dataLinkRate) {
        this.dataLinkRate = dataLinkRate;
    }

    public void setMediaType(final MediaType mediaType) {
        this.mediaType = mediaType;
    }

    public void setMediaTypeRaw(final String mediaTypeRaw) {
        this.mediaTypeRaw = mediaTypeRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getFabricEstablished() {
        return fabricEstablished;
    }

    public void setFabricEstablished(Boolean fabricEstablished) {
        this.fabricEstablished = fabricEstablished;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getFirmwareRev() {
        return firmwareRev;
    }

    public void setFirmwareRev(String firmwareRev) {
        this.firmwareRev = firmwareRev;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getInfoName() {
        return infoName;
    }

    public void setInfoName(String infoName) {
        this.infoName = infoName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum PhysicalProtocol {
        NOT_MAPPED,
        FIBRE_CHANNEL,
        ETHERNET,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Speed {
        NOT_MAPPED,
        AUTO,
        GIGABIT_1,
        GIGABIT_2,
        GIGABIT_4,
        GIGABIT_8,
        GIGABIT_10,
        GIGABIT_16,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum State {
        NOT_MAPPED,
        STARTUP("startup"),
        UNINITIALIZED("uninitialized"),
        INITIALIZING_FIRMWARE("initializing firmware"),
        LINK_NOT_CONNECTED("link not connected"),
        WAITING_FOR_LINK_UP("waiting for link up"),
        ONLINE("online"),
        LINK_DISCONNECTED("link disconnected"),
        RESETTING("resetting"),
        OFFLINE("offline"),
        OFFLINED_BY_USER_OR_SYSTEM("offlined by user/system");

        private final String stringValue;

        private State() {
            this(null);
        }

        private State(final String stringValue) {
            this.stringValue = stringValue;
        }

        public String stringValue() {
            return (stringValue == null ? name() : stringValue);
        }

        public static State fromString(final String stringVal) {
            if (stringVal == null) {
                return NOT_MAPPED;
            }
            for (final State candidateState : State.values()) {
                if (candidateState.stringValue().equalsIgnoreCase(stringVal)) {
                    return candidateState;
                }
            }
            return NOT_MAPPED;
        }

    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Status {
        NOT_MAPPED,
        OFFLINE,
        ONLINE,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum MediaType {
        NOT_MAPPED,
        PTP,
        LOOP,
        AUTO
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "NodeName", nodeWWN);
        addRequiredAttributeAndCRC(writer, "PortName", key.getID(FcpPortKey.Field.port_name));
        addRequiredAttributeAndCRC(writer, "Adapter", getName());
        addRequiredAttributeAndCRC(writer, "FabricEstablished", fabricEstablished);
        addRequiredAttributeAndCRC(writer, "FirmwareRev", firmwareRev);
        addRequiredAttributeAndCRC(writer, "InfoName", infoName);
        addRequiredAttributeAndCRC(writer, "MaxSpeed", maxSpeed);
        addRequiredAttributeAndCRC(writer, "PhysicalProtocol", physicalProtocol);
        addRequiredAttributeAndCRC(writer, "PhysicalProtocolRaw", physicalProtocolRaw);
        addRequiredAttributeAndCRC(writer, "Speed", speed);
        addRequiredAttributeAndCRC(writer, "SpeedRaw", speedRaw);
        addRequiredAttributeAndCRC(writer, "State", state);
        addRequiredAttributeAndCRC(writer, "StateRaw", stateRaw);
        addRequiredAttributeAndCRC(writer, "Status", status);
        addRequiredAttributeAndCRC(writer, "StatusRaw", statusRaw);
        addRequiredAttributeAndCRC(writer, "SwitchPort", switchPort);
        addRequiredAttributeAndCRC(writer, "DataLinkRate", dataLinkRate);
        addRequiredAttributeAndCRC(writer, "MediaType", mediaType);
        addRequiredAttributeAndCRC(writer, "MediaTypeRaw", mediaTypeRaw);
    }
}
