package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("flash_device")
@OCIEResourceType("flash_device")
public class FlashDevice extends BaseCRCXMLNode<Node> {

    private static final String TYPE_NAME = "flash_device";

    public static final class FlashDeviceKey extends KeyBase<FlashDeviceKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                    sourceZapi = "flash-device-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.flash-device-info.serial-number"
            )
            serial_number,
        }

        public FlashDeviceKey(final String clusterUUID, final String serialNumber) {
            super(TYPE_NAME, new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.serial_number, serialNumber);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "serial_number")
        public String getOcieUUID() {
            return getID(Field.serial_number);
        }
    }

    private final FlashDeviceKey flashDeviceKey;

    /** PCI-e slot number of the flash device. */
    @OCIEZapiSource(
        sourceZapi = "flash-device-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.flash-device-info.slot-number"
    )
    @OCIEValue(
        ocieColumn = "slotNumber"
    )
    private Integer slotNumber = null;

    /** Model name of the flash device in XXXXXX-XX format. */
    @OCIEZapiSource(
        sourceZapi = "flash-device-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.flash-device-info.model-name"
    )
    @OCIEValue(
        ocieColumn = "modelName"
    )
    private String modelName = null;

    /** Firmware revision of FPGA on the flash device. */
    @OCIEZapiSource(
        sourceZapi = "flash-device-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.flash-device-info.firmware-revision"
    )
    @OCIEValue(
        ocieColumn = "firmwareRevision"
    )
    private String firmwareRevision = null;

    /** Advertised capacity of the device, in gigabytes. */
    @OCIEZapiSource(
        sourceZapi = "flash-device-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.flash-device-info.capacity"
    )
    @OCIEValue(
        ocieColumn = "capacityGB"
    )
    private Integer capacityGB = null;

    /** Percentage of device capacity that is currently online [0..100]. */
    @OCIEZapiSource(
        sourceZapi = "flash-device-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.flash-device-info.percent-online"
    )
    @OCIEValue(
        ocieColumn = "percentOnline"
    )
    private Integer percentOnline = null;

    /** The current status of the device. */
    @OCIEZapiSource(
        sourceZapi = "flash-device-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.flash-device-info.status"
    )
    @OCIEValue(
        ocieColumn = "status"
    )
    private FlashDevice.Status status = null;

    /** The current status of the device. Possible values: "online", "offline_failed", or "offline_threshold". */
    @OCIEZapiSource(
        sourceZapi = "flash-device-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.flash-device-info.status"
    )
    @OCIEValue(
        ocieColumn = "status"
    )
    private String statusRaw = null;

    /** Total number of hours the device was operational. */
    @OCIEZapiSource(
        sourceZapi = "flash-device-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.flash-device-info.service-time"
    )
    @OCIEValue(
        ocieColumn = "serviceTime"
    )
    private Integer serviceTime = null;

    /** The name of the threshold profile being used for this flash device. */
    @OCIEZapiSource(
        sourceZapi = "flash-device-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.flash-device-info.threshold-profile"
    )
    @OCIEValue(
        ocieColumn = "thresholdProfile"
    )
    private String thresholdProfile = null;

    FlashDevice(final Node parent, final String serialNumber) {
        super(parent, FlashDevice.class.getSimpleName());
        final String clusterUUID = parent.getParent().getKey().getID(Cluster.ClusterKey.Field.uuid);
        this.flashDeviceKey = new FlashDeviceKey(clusterUUID, serialNumber);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public FlashDeviceKey getKey() {
        return this.flashDeviceKey;
    }

    public String getSerialNumber() {
        return flashDeviceKey.getID(FlashDeviceKey.Field.serial_number);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getSlotNumber() {
        return slotNumber;
    }

    public void setSlotNumber(final Integer slotNumber) {
        this.slotNumber = slotNumber;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getModelName() {
        return modelName;
    }

    public void setModelName(final String modelName) {
        this.modelName = modelName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getFirmwareRevision() {
        return firmwareRevision;
    }

    public void setFirmwareRevision(final String firmwareRevision) {
        this.firmwareRevision = firmwareRevision;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getCapacityGB() {
        return capacityGB;
    }

    public void setCapacityGB(final Integer capacityGB) {
        this.capacityGB = capacityGB;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getPercentOnline() {
        return percentOnline;
    }

    public void setPercentOnline(final Integer percentOnline) {
        this.percentOnline = percentOnline;
    }

    public Status getStatus() {
        return status;
    }

    public void setStatus(final Status status) {
        this.status = status;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getStatusRaw() {
        return statusRaw;
    }

    public void setStatusRaw(final String statusRaw) {
        this.statusRaw = statusRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getServiceTime() {
        return serviceTime;
    }

    public void setServiceTime(final Integer serviceTime) {
        this.serviceTime = serviceTime;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getThresholdProfile() {
        return thresholdProfile;
    }

    public void setThresholdProfile(final String thresholdProfile) {
        this.thresholdProfile = thresholdProfile;
    }


    @SuppressWarnings({"UnusedDeclaration"})
    public enum Status {
        NOT_MAPPED,
        ONLINE,
        OFFLINE_FAILED,
        OFFLINE_THRESHOLD
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", flashDeviceKey);
        addRequiredAttributeAndCRC(writer, "UUID", flashDeviceKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "SlotNumber", slotNumber);
        addRequiredAttributeAndCRC(writer, "SerialNumber", getSerialNumber());
        addRequiredAttributeAndCRC(writer, "ModelName", modelName);
        addRequiredAttributeAndCRC(writer, "FirmwareRevision", firmwareRevision);
        addRequiredAttributeAndCRC(writer, "CapacityGB", capacityGB);
        addRequiredAttributeAndCRC(writer, "PercentOnline", percentOnline);
        addRequiredAttributeAndCRC(writer, "Status", status);
        addRequiredAttributeAndCRC(writer, "StatusRaw", statusRaw);
        addRequiredAttributeAndCRC(writer, "ServiceTime", serviceTime);
        addRequiredAttributeAndCRC(writer, "ThresholdProfile", thresholdProfile);
    }

}
