package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("igroup_initiator")
@OCIEResourceType("igroup_initiator")
public class IGroupInitiator extends BaseCRCXMLNode<IGroup> {

    public final static class InitiatorKey extends KeyBase<InitiatorKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "cluster-identity-get",
                    zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                    sourceZapi = "vserver-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                    sourceZapi = "igroup-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.initiator-group-info.initiator-group-uuid"
            )
            igroup_uuid,


            @OCIEZapiSource(
                    sourceZapi = "igroup-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.initiator-group-info.initiators.initiator-info.initiator-name"
            )
            @OCIEValue(
                    ocieColumn = "initiatorName"
            )
            name
        }

        public InitiatorKey(final String clusterUUID, final String vServerUUID, final String iGroupUUID, final String initiatorName) {
            super("igroup_initiator", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.igroup_uuid, iGroupUUID);
                put(Field.name, initiatorName);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.name")
        public String getOcieUUID() {
            return getFQID();
        }

    }

    private final InitiatorKey key;

    IGroupInitiator(final IGroup parent, final String vServerUUID, final String iGroupUUID, final String initiatorName) {
        super(parent, "IgroupInitiator");
        this.key = new InitiatorKey(parent.getKey().getID(IGroup.IGroupKey.Field.cluster_uuid), vServerUUID, iGroupUUID, initiatorName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public InitiatorKey getKey() {
        return key;
    }



    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "Name", key.getID(InitiatorKey.Field.name));
    }
}
