package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("iscsi_portal_group_lif")
@OCIEResourceType("iscsi_portal_group_lif")
public class ISCSIPortalGroupToLIF extends BaseCRCXMLNode<ISCSIPortalGroup> {

    public static final class ISCSIPortalGroupToLifKey extends KeyBase<ISCSIPortalGroupToLifKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "iscsi-tpgroup-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.iscsi-tpgroup-list-entry-info.tpgroup-name"
            )
            iscsi_portal_group_name,

            // JSD ?
            @OCIEZapiSource(
                sourceZapi = "iscsi-interface-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.iscsi-interface-list-entry-info.interface-name"
            )
            lif_uuid
        }

        public ISCSIPortalGroupToLifKey(final String clusterUUID, final String vServerUUID, final String iscsiPortalGroupName, final String lifUUID) {
            super("iscsi_portal_group_lif", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.iscsi_portal_group_name, iscsiPortalGroupName);
                put(Field.lif_uuid, lifUUID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.vserver_uuid.iscsi_portal_group_name.lif_uuid")
        public String getOcieUUID() {
            return getFQID();
        }
    }

    private final ISCSIPortalGroupToLifKey key;


    @OCIEZapiSource(
            sourceZapi = "iscsi-interface-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.iscsi-interface-list-entry-info.interface-name"
    )
    @OCIEValue(
            ocieColumn = "interfaceName"
    )
    private Lif.LifKey lifKey;

    @OCIEZapiSource(
        sourceZapi = "iscsi-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.iscsi-interface-list-entry-info.interface-name"
    )
    @OCIEValue(
        ocieColumn = "lifId"
    )
    private String interfaceName;

    @OCIEZapiSource(
        sourceZapi = "iscsi-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.iscsi-interface-list-entry-info.ip-address"
    )
    @OCIEValue(
        ocieColumn = "ipAddress"
    )
    private String ipAddress;

    @OCIEZapiSource(
        sourceZapi = "iscsi-interface-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.iscsi-interface-list-entry-info.ip-port"
    )
    @OCIEValue(
        ocieColumn = "ipPort"
    )
    private String ipPort;

    ISCSIPortalGroupToLIF(final ISCSIPortalGroup parent, final String lifUUID) {
        super(parent, "ISCSIPortalGroupLIF");
        final ISCSIPortalGroup.ISCSIPortalGroupKey parentKey = parent.getKey();
        final String clusterUUID = parentKey.getID(ISCSIPortalGroup.ISCSIPortalGroupKey.Field.cluster_uuid);
        final String vServerUUID = parentKey.getID(ISCSIPortalGroup.ISCSIPortalGroupKey.Field.vserver_uuid);
        final String portalGroupName = parentKey.getID(ISCSIPortalGroup.ISCSIPortalGroupKey.Field.portal_group_name);
        this.key = new ISCSIPortalGroupToLifKey(clusterUUID, vServerUUID, portalGroupName, lifUUID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public ISCSIPortalGroupToLifKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLifUUID() {
        return key.getID(ISCSIPortalGroupToLifKey.Field.lif_uuid);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getInterfaceName() {
        return interfaceName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getIpAddress() {
        return ipAddress;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getIpPort() {
        return ipPort;
    }

    public void setInterfaceName(final String interfaceName) {
        this.interfaceName = interfaceName;
    }

    public void setIpAddress(final String ipAddress) {
        this.ipAddress = ipAddress;
    }

    public void setIpPort(final String ipPort) {
        this.ipPort = ipPort;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Lif.LifKey getLifKey() {
        return lifKey;
    }

    public void setLifKey(final Lif.LifKey lifKey) {
        this.lifKey = lifKey;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "LIFUUID", lifKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "InterfaceName", interfaceName);
        addRequiredAttributeAndCRC(writer, "IPAddress", ipAddress);
        addRequiredAttributeAndCRC(writer, "IPPort", ipPort);
    }
}
