package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("job_schedule")
@OCIEResourceType("job_schedule")
public class JobSchedule extends BaseCRCXMLNode<Cluster> {

    public final static class JobScheduleKey extends KeyBase<JobScheduleKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "job-schedule-get-iter",
                zapiResponsePath = "netapp.results.attributes.job-schedule-info.job-schedule-name"
            )
            name
        }

        public JobScheduleKey(final String clusterUUID, final String jobScheduleName) {
            super("ontap_job_schedule", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.name, jobScheduleName);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.name")
        public String getOcieUUID() {
            return getFQID();
        }
    }


    private final JobScheduleKey scheduleKey;

    @OCIEZapiSource(
        sourceZapi = "job-schedule-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.job-schedule-info.job-schedule-type"
    )
    @OCIEValue(
        ocieColumn = "scheduleType"
    )
    private ScheduleType scheduleType;

    @OCIEZapiSource(
        sourceZapi = "job-schedule-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.job-schedule-info.job-schedule-type"
    )
    @OCIEValue(
        ocieColumn = "scheduleTypeRaw"
    )
    private String scheduleTypeRaw;

    @OCIEZapiSource(
        sourceZapi = "job-schedule-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.job-schedule-info.job-schedule-description"
    )
    @OCIEValue(
        ocieColumn = "description"
    )
    private String description;

    @OCIEZapiSource(
        sourceZapi = "job-schedule-cron-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.job-schedule-cron-info.job-schedule-cron-day.cron-day-of-month"
    )
    @OCIEValue(
        ocieColumn = "cronDayOfMonth"
    )
    private String cronDayOfMonth;

    @OCIEZapiSource(
        sourceZapi = "job-schedule-cron-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.job-schedule-cron-info.job-schedule-cron-day.cron-day-of-month"
    )
    @OCIEValue(
        ocieColumn = "cronMinute"
    )
    private String cronDayOfWeek;

    @OCIEZapiSource(
            sourceZapi = "job-schedule-cron-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.job-schedule-cron-info.job-schedule-cron-hour.cron-hour"
    )
    @OCIEValue(
            ocieColumn = "cronHour"
    )
    private String cronHour;

    @OCIEZapiSource(
        sourceZapi = "job-schedule-cron-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.job-schedule-cron-info.job-schedule-cron-minute.cron-minute"
    )
    @OCIEValue(
        ocieColumn = "cronMinute"
    )
    private String cronMinute;

    @OCIEZapiSource(
        sourceZapi = "job-schedule-cron-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.job-schedule-cron-info.job-schedule-cron-month.cron-month"
    )
    @OCIEValue(
        ocieColumn = "cronMonth"
    )
    private String cronMonth;


    @OCIEZapiSource(
        sourceZapi = "job-schedule-interval-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.job-schedule-interval-info.job-schedule-interval-days"
    )
    @OCIEValue(
        ocieColumn = "intervalDays"
    )
    private Long intervalDays;

    @OCIEZapiSource(
        sourceZapi = "job-schedule-interval-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.job-schedule-interval-info.job-schedule-interval-hours"
    )
    @OCIEValue(
        ocieColumn = "intervalHours"
    )
    private Long intervalHours;

    @OCIEZapiSource(
        sourceZapi = "job-schedule-interval-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.job-schedule-interval-info.job-schedule-interval-minutes"
    )
    @OCIEValue(
        ocieColumn = "intervalMinutes"
    )
    private Long intervalMinutes;

    @OCIEZapiSource(
        sourceZapi = "job-schedule-interval-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.job-schedule-interval-info.job-schedule-interval-seconds"
    )
    @OCIEValue(
        ocieColumn = "intervalSeconds"
    )
    private Long intervalSeconds;

    JobSchedule(final Cluster parent, final String jobScheduleName) {
        super(parent, "JobSchedule");
        this.scheduleKey = new JobScheduleKey(parent.getClusterUUID(), jobScheduleName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public JobScheduleKey getKey() {
        return scheduleKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public ScheduleType getScheduleType() {
        return scheduleType;
    }

    public void setScheduleType(final ScheduleType scheduleType) {
        this.scheduleType = scheduleType;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getScheduleTypeRaw() {
        return scheduleTypeRaw;
    }

    public void setScheduleTypeRaw(final String scheduleTypeRaw) {
        this.scheduleTypeRaw = scheduleTypeRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getDescription() {
        return description;
    }

    public void setDescription(final String description) {
        this.description = description;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getCronDayOfMonth() {
        return cronDayOfMonth;
    }

    public void setCronDayOfMonth(final String cronDayOfMonth) {
        this.cronDayOfMonth = cronDayOfMonth;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getCronDayOfWeek() {
        return cronDayOfWeek;
    }

    public void setCronDayOfWeek(final String cronDayOfWeek) {
        this.cronDayOfWeek = cronDayOfWeek;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getCronHour() {
        return cronHour;
    }

    public void setCronHour(final String cronHour) {
        this.cronHour = cronHour;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getCronMinute() {
        return cronMinute;
    }

    public void setCronMinute(final String cronMinute) {
        this.cronMinute = cronMinute;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getCronMonth() {
        return cronMonth;
    }

    public void setCronMonth(final String cronMonth) {
        this.cronMonth = cronMonth;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getIntervalDays() {
        return intervalDays;
    }

    public void setIntervalDays(final Long intervalDays) {
        this.intervalDays = intervalDays;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getIntervalHours() {
        return intervalHours;
    }

    public void setIntervalHours(final Long intervalHours) {
        this.intervalHours = intervalHours;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getIntervalMinutes() {
        return intervalMinutes;
    }

    public void setIntervalMinutes(final Long intervalMinutes) {
        this.intervalMinutes = intervalMinutes;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getIntervalSeconds() {
        return intervalSeconds;
    }

    public void setIntervalSeconds(final Long intervalSeconds) {
        this.intervalSeconds = intervalSeconds;
    }


    @SuppressWarnings({"UnusedDeclaration"})
    public static enum ScheduleType {
        NOT_MAPPED,
        CRON,
        INTERVAL,
        BUILTIN,
        UNKNOWN,
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", scheduleKey);
        addRequiredAttributeAndCRC(writer, "UUID", scheduleKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", scheduleKey.getID(JobScheduleKey.Field.name));
        addRequiredAttributeAndCRC(writer, "ScheduleType", scheduleType);
        addRequiredAttributeAndCRC(writer, "ScheduleTypeRaw", scheduleTypeRaw);
        addRequiredAttributeAndCRC(writer, "Description", description);
        addRequiredAttributeAndCRC(writer, "CronDayOfMonth", cronDayOfMonth);
        addRequiredAttributeAndCRC(writer, "CronDayOfWeek", cronDayOfWeek);
        addRequiredAttributeAndCRC(writer, "CronHour", cronHour);
        addRequiredAttributeAndCRC(writer, "CronMinute", cronMinute);
        addRequiredAttributeAndCRC(writer, "CronMonth", cronMonth);
        addRequiredAttributeAndCRC(writer, "IntervalDays", intervalDays);
        addRequiredAttributeAndCRC(writer, "IntervalHours", intervalHours);
        addRequiredAttributeAndCRC(writer, "IntervalMinutes", intervalMinutes);
        addRequiredAttributeAndCRC(writer, "IntervalSeconds", intervalSeconds);
    }
}
