package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Node.NodeKey;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("lun_map")
@OCIEResourceType("lun_map")
public class LUNMap extends BaseCRCXMLNode<VServer> {

    public final static class LUNMapKey extends KeyBase<LUNMapKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "lun-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.lun-info.uuid"
            )
            lun_uuid,

            @OCIEZapiSource(
                sourceZapi = "igroup-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.initiator-group-info.initiator-group-uuid"
            )
            igroup_uuid
        }

        public LUNMapKey(final String clusterUUID, final String lunUUID, final String iGroupUUID) {
            super("lun_map", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.lun_uuid, lunUUID);
                put(Field.igroup_uuid, iGroupUUID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "igroup_uuid.lun_uuid")
        public String getOcieUUID() {
            return getID(Field.igroup_uuid) + '.' + getID(Field.lun_uuid);
        }
    }

    private final LUNMapKey key;

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.uuid"
    )
    @OCIEValue(
            ocieColumn = "lunId"
    )
    private Integer lunID;

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.path"
    )
    @OCIEValue(
        ocieColumn = "path"
    )
    private String path;
    
    @OCIEZapiSource(sourceZapi = "lun-get-iter", zapiResponsePath = "netapp.results.attributes-list.lun-info.node")
    @OCIEValue(ocieColumn = "owningNodeId")
    private Node.NodeKey owningNodeKey;

    private LUNMapReportingNodes lunMapReportingNodes;
    
    LUNMap(final VServer parent, final String lunUUID, final String iGroupUUID) {
        super(parent, "LUNMap");
        this.key = new LUNMapKey(parent.getKey().getID(VServer.VServerKey.Field.cluster_uuid), lunUUID, iGroupUUID);
        this.lunMapReportingNodes = new LUNMapReportingNodes(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public LUNMapKey getKey() {
        return key;
    }

    public Integer getLunID() {
        return lunID;
    }

    public String getPath() {
        return path;
    }
    
    public LUNMapReportingNodes getLunMapReportingNodes() {
    	return lunMapReportingNodes;
    }
    
    public Node.NodeKey getOwningNodeKey() {
        return owningNodeKey;
    }

    public void setLunID(final Integer lunID) {
        this.lunID = lunID;
    }

    public void setPath(final String path) {
        this.path = path;
    }

    public void setOwningNodeKey(final Node.NodeKey owningNodeKey) {
        this.owningNodeKey = owningNodeKey;
    }
    
    public void setLunMapReportingNodes(LUNMapReportingNodes lunMapReportingNodes) {
    	this.lunMapReportingNodes = lunMapReportingNodes;
    }
    
    public LUNMapReportingNode createLUNMapReportingNode(LUNMap.LUNMapKey lunMapKey, Node.NodeKey nodeKey) {
        return new LUNMapReportingNode(this, lunMapKey, nodeKey);
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "LUNUUID", key.getID(LUNMapKey.Field.lun_uuid));
        addRequiredAttributeAndCRC(writer, "IgroupUUID", key.getID(LUNMapKey.Field.igroup_uuid));
        addRequiredAttributeAndCRC(writer, "LUN", lunID);
        addRequiredAttributeAndCRC(writer, "Path", path);
        addRequiredAttributeAndCRC(writer, "OwningNodeUUID", KeyBase.getOcieUUIDOrNull(owningNodeKey));
    }
    
    @Override
    protected boolean hasChildren() {
        return !lunMapReportingNodes.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer,
            final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, LUNMapReportingNodes.class.getSimpleName(),
                lunMapReportingNodes.values(), tabs);
    }
}
