package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import java.io.IOException;
import java.util.EnumMap;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEUUID;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEValue;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

@SuppressWarnings("serial")
@OCIETable("lun_map_reporting_node")
@OCIEResourceType("lun_map_reporting_node")
public class LUNMapReportingNode extends BaseCRCXMLNode<LUNMap> {
	   public final static class LUNMapReportingNodeKey extends KeyBase<LUNMapReportingNodeKey.Field> {
	        public static enum Field {
	            lun_map_uuid,
	            node_uuid
	        }

	        public LUNMapReportingNodeKey(final String lunMapUUID, final String nodeUUID) {
	            super("lun_map_node_",
	                    new EnumMap<Field, String>(Field.class) {{
	                        put(Field.lun_map_uuid, lunMapUUID);
	                        put(Field.node_uuid, nodeUUID);
	                    }});
	        }

	        @Override
	        @OCIEUUID(uuidDescr = "lun_map_uuid.node_uuid")
	        public String getOcieUUID() {
	            final String lunUUID = getID(Field.lun_map_uuid);
	            final String nodeUUID = getID(Field.node_uuid);
	            return lunUUID + '.' + nodeUUID;
	        }
	    }

        @OCIEZapiSource(
            sourceZapi = "lun-get-iter,igroup-get-iter",
            notes = "Derived from igroup_uuid.lun_uuid"
        )
        @OCIEValue(
            ocieColumn = "lunMapId"
        )
        private final LUNMap.LUNMapKey lunMapKey;

        @OCIEZapiSource(
            sourceZapi = "lun-map-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.lun-map-info.reporting-nodes.node_name"
        )
        @OCIEValue(
            ocieColumn = "nodeId"
        )
	    private final Node.NodeKey nodeKey;

	    private final LUNMapReportingNodeKey key;


	    protected LUNMapReportingNode(final LUNMap lunMap, final LUNMap.LUNMapKey lunMapKey, final Node.NodeKey nodeKey) {
	        super(lunMap, LUNMapReportingNode.class.getSimpleName());
	        this.key = new LUNMapReportingNodeKey(lunMapKey.getOcieUUID(), nodeKey.getOcieUUID());
	        this.lunMapKey = lunMapKey;
	        this.nodeKey = nodeKey;
	    }

	    @Override
	    @SuppressWarnings({"unchecked"})
	    public LUNMapReportingNodeKey getKey() {
	        return this.key;
	    }

	    public LUNMap.LUNMapKey getLunMapKey() {
	        return lunMapKey;
	    }

	    public Node.NodeKey getNodeKey() {
	        return nodeKey;
	    }

	    @Override
	    protected void marshallAttributesToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer) throws IOException, SchemaException {
	        super.marshallAttributesToFileAndCRC(cache, writer);
	        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
	        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
	        addRequiredAttributeAndCRC(writer, "LUNMapUUID", lunMapKey.getOcieUUID());
	        addRequiredAttributeAndCRC(writer, "ReportingingNodeUUID", nodeKey.getOcieUUID());
	    }
}
