package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer.VServerKey;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import java.io.IOException;
import java.util.EnumMap;


@OCIETable("lun_import")
@OCIEResourceType("lun_import")
public class LunImport extends BaseCRCXMLNode<Cluster> {

    public final static class LunImportKey extends KeyBase<LunImportKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "vserver-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
                )
            vserver_uuid,
                
            @OCIEZapiSource(
                sourceZapi = "lun-import-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.lun-import-info.foreign-disk"
            )
            foreign_disk
        }

        public LunImportKey(final String vserverUUID, final String foreignDisk) {
            super("lunImport", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_uuid, vserverUUID);
                put(Field.foreign_disk, foreignDisk);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "vserver_uuid.foreign_disk")
        public String getOcieUUID() {
            return getID(Field.vserver_uuid) + "." + getID(Field.foreign_disk);
        }
    }

    private LunImportKey key;                                 

    @OCIEZapiSource(
        sourceZapi = "lun-import-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.administrative-status"
    )
    @OCIEValue(
        ocieColumn = "administrativeStatus"
    )
    private AdministrativeStatus administrativeStatus;          

    @OCIEZapiSource(
        sourceZapi = "lun-import-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.administrative-status"
    )
    @OCIEValue(
        ocieColumn = "administrativeStatusRaw"
    )
    private String administrativeStatusRaw;

    @OCIEZapiSource(
        sourceZapi = "lun-import-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.compared-blocks"
    )
    @OCIEValue(
        ocieColumn = "comparedBlocks"
    )
    private Long comparedBlocks;                       

    @OCIEZapiSource(
        sourceZapi = "lun-import-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.estimated-remaining-duration"
    )
    @OCIEValue(
        ocieColumn = "estimatedRemainingDuration"
    )
    private Long estimatedRemainingDuration;  

    @OCIEZapiSource(
        sourceZapi = "lun-import-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.failure-reason"
    )
    @OCIEValue(
        ocieColumn = "failureReason"
    )
    private String failureReason;                                   
    
    @OCIEZapiSource(
        sourceZapi = "lun-import-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.foreign-disk"
    )
    @OCIEValue(
        ocieColumn = "foreignDisk"
    )
    private Disk.DiskKey foreignDiskKey;                           

    @OCIEZapiSource(
        sourceZapi = "lun-get-import-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.import-home-node"
    )
    @OCIEValue(
        ocieColumn = "importHomeNodeId"
    )
    private Node.NodeKey importHomeNodeKey;

    @OCIEZapiSource(
        sourceZapi = "lun-import-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.import-current-node"
    )
    @OCIEValue(
        ocieColumn = "importCurrentNodeId"
    )
    private Node.NodeKey importCurrentNodeKey;                                       

    @OCIEZapiSource(
        sourceZapi = "lun-get-import-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.importedkv-blocks"
    )
    @OCIEValue(
        ocieColumn = "importedBlocks"
    )
    private Long importedBlocks;
    
    @OCIEZapiSource(
        sourceZapi = "lun-import-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.operation-in-progress"
    )
    @OCIEValue(
        ocieColumn = "operationInProgress"
    )
    private OperationInProgress operationInProgress;
    
    @OCIEZapiSource(
        sourceZapi = "lun-import-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.operation-in-progress"
    )
    @OCIEValue(
        ocieColumn = "operationInProgressRaw"
    )
    private String operationInProgressRaw;

    @OCIEZapiSource(
        sourceZapi = "lun-import-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.operational-status"
    )
    @OCIEValue(
        ocieColumn = "operationalStatus"
    )
    private OperationalStatus operationalStatus;
    
    @OCIEZapiSource(
        sourceZapi = "lun-import-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.operational-status"
    )
    @OCIEValue(
        ocieColumn = "operationalStatusRaw"
    )
    private String operationalStatusRaw;
        
    @OCIEZapiSource(
        sourceZapi = "lun-get-import-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.path"
    )
    @OCIEValue(
        ocieColumn = "path"
    )
    private String path;        

    @OCIEZapiSource(
        sourceZapi = "lun-get-import-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.percent-complete"
    )
    @OCIEValue(
        ocieColumn = "percentComplete"
    )
    private Long percentComplete;                          

    @OCIEZapiSource(
        sourceZapi = "lun-get-import-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.total-blocks"
    )
    @OCIEValue(
        ocieColumn = "totalBlocks"
    )
    private Long totalBlocks;

    @OCIEZapiSource(
        sourceZapi = "lun-get-import-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-import-info.vserver"
    )
    @OCIEValue(
        ocieColumn = "vserverId"
    )
    private VServer.VServerKey vServerKey;    
   
    LunImport(final Cluster parent, final VServer.VServerKey vServerKey, final String foreignDisk) {
        super(parent, LunImport.class.getSimpleName());
        this.key = new LunImportKey(vServerKey.getID(VServer.VServerKey.Field.uuid), foreignDisk);
    }

    public AdministrativeStatus getAdministrativeStatus() {
        return administrativeStatus;
    }
    
    public void setAdministrativeStatus(AdministrativeStatus administrativeStatus) {
        this.administrativeStatus = administrativeStatus;
    }
    
    public String getAdministrativeStatusRaw() {
        return administrativeStatusRaw;
    }
    
    public void setAdministrativeStatusRaw(String administrativeStatusRaw) {
        this.administrativeStatusRaw = administrativeStatusRaw;
    }
    
    public Long getComparedBlocks() {
        return comparedBlocks;
    }
    
    public void setComparedBlocks(Long comparedBlocks) {
        this.comparedBlocks = comparedBlocks;
    }
    
    public Long getEstimatedRemainingDuration() {
        return estimatedRemainingDuration;
    }
    
    public void setEstimatedRemainingDuration(Long estimatedRemainingDuration) {
        this.estimatedRemainingDuration = estimatedRemainingDuration;
    }
    
    public String getFailureReason() {
        return failureReason;
    }
    
    public void setFailureReason(String failureReason) {
        this.failureReason = failureReason;
    }
    
    public Disk.DiskKey getForeignDisk() {
        return foreignDiskKey;
    }
    
    public void setForeignDisk(Disk.DiskKey foreignDiskKey) {
        this.foreignDiskKey = foreignDiskKey;
    }

    public Node.NodeKey getImportHomeNodeKey() {
        return importHomeNodeKey;
    }
    
    public void setImportHomeNodeKey(Node.NodeKey importHomeNodeKey) {
        this.importHomeNodeKey = importHomeNodeKey;
    }
    
    public Node.NodeKey getImportCurrentNodeKey() {
        return importCurrentNodeKey;
    }
    
    public void setImportCurrentNodeKey(Node.NodeKey importCurrentNodeKey) {
        this.importCurrentNodeKey = importCurrentNodeKey;
    }
    
    public Long getImportedBlocks() {
        return importedBlocks;
    }
 
    public void setImportedBlocks(Long importedBlocks) {
        this.importedBlocks = importedBlocks;
    }

    public OperationInProgress getOperationInProgress() {
        return operationInProgress;
    }
    
    public void setOperationInProgress(OperationInProgress operationInProgress) {
        this.operationInProgress = operationInProgress;
    }
    
    public String getOperationInProgressRaw() {
        return operationInProgressRaw;
    }

    public void setOperationInProgressRaw(String operationInProgressRaw) {
        this.operationInProgressRaw = operationInProgressRaw;
    }
    
    public OperationalStatus getOperationalStatus() {
        return operationalStatus;
    }
    
    public void setOperationalStatus(OperationalStatus operationalStatus) {
        this.operationalStatus = operationalStatus;
    }
    
    public String getOperationalStatusRaw() {
        return operationalStatusRaw;
    }
    
    public void setOperationalStatusRaw(String operationalStatusRaw) {
        this.operationalStatusRaw = operationalStatusRaw;
    }
    
    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;
    }

    public Long getPercentComplete() {
        return percentComplete;
    }
    
    public void setPercentComplete(Long percentComplete) {
        this.percentComplete = percentComplete;
    }
    
    public Long getTotalBlocks() {
        return totalBlocks;
    }

    public void setTotalBlocks(Long totalBlocks) {
        this.totalBlocks = totalBlocks;
    }
    
    public VServer.VServerKey getVServerKey() {
        return vServerKey;
    }
    
    public void setVserverId(VServer.VServerKey vServerKey) {
        this.vServerKey = vServerKey;
    }
    
    public void setKey(LunImportKey key) {
        this.key = key;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public LunImportKey getKey() {
        return key;
    }  

    public static enum AdministrativeStatus {
        NOT_MAPPED,
        STARTED,
        STOPPED,
        PAUSED 
    }
    
    public static enum OperationInProgress {
        NOT_MAPPED,
        IMPORT,
        VERIFY 
    }
    
    public static enum OperationalStatus {
        NOT_MAPPED,
        IN_PROGRESS,
        FAILED,
        COMPLETED,
        STOPPED,
        PAUSED
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "AdministrativeStatus", administrativeStatus);
        addRequiredAttributeAndCRC(writer, "AdministrativeStatusRaw", administrativeStatusRaw);
        addRequiredAttributeAndCRC(writer, "ComparedBlocks", comparedBlocks);       
        addRequiredAttributeAndCRC(writer, "EstimatedRemainingDuration", estimatedRemainingDuration);
        addRequiredAttributeAndCRC(writer, "FailureReason", failureReason);
        addRequiredAttributeAndCRC(writer, "ForeignDiskUUID", KeyBase.getOcieUUIDOrNull(foreignDiskKey));
        addRequiredAttributeAndCRC(writer, "ImportCurrentNodeUUID", KeyBase.getOcieUUIDOrNull(importCurrentNodeKey));
        addRequiredAttributeAndCRC(writer, "ImportHomeNodeUUID", KeyBase.getOcieUUIDOrNull(importHomeNodeKey));
        addRequiredAttributeAndCRC(writer, "ImportedBlocks", importedBlocks);
        addRequiredAttributeAndCRC(writer, "OperationInProgress", operationInProgress);
        addRequiredAttributeAndCRC(writer, "OperationInProgressRaw", operationInProgressRaw);
        addRequiredAttributeAndCRC(writer, "OperationalStatus", operationalStatus);
        addRequiredAttributeAndCRC(writer, "OperationalStatusRaw", operationalStatusRaw);
        addRequiredAttributeAndCRC(writer, "Path", path);               
        addRequiredAttributeAndCRC(writer, "PercentComplete", percentComplete);
        addRequiredAttributeAndCRC(writer, "TotalBlocks", totalBlocks);
        addRequiredAttributeAndCRC(writer, "VserverUUID", VServerKey.getOCIEVServerUUIDOrNull(vServerKey));
    }
}
