package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;

@OCIETable("nis_domain")
@OCIEResourceType("nis_domain")
public class NISDomain extends BaseCRCXMLNode<VServer> {

    @SuppressWarnings({"UnusedDeclaration"})
    private static Logger logger = Logger.getLogger(NISDomain.class);

    public final static class NISDomainKey extends KeyBase<NISDomainKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "nis-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.nis-domain-config-info.nis-domain"
            )
            nis_domain_name
        }

        public NISDomainKey(final String clusterUUID, final String vServerUUID, final String nisDomainName) {
            super("nis_domain", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.nis_domain_name, nisDomainName);
            }}, EnumSet.of(Field.nis_domain_name));                      // These EnumSet<E> emptyAbleFields fields are allowed to be empty but not null
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.vserver_uuid.nis_domain_name")
        public String getOcieUUID() {
            return getFQID();
        }
    }

    @OCIEValue(
        ocieColumn = "resourceKey"
    )
    private final NISDomainKey key;

    @OCIEZapiSource(
        sourceZapi = "nis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.nis-domain-config-info.is-active"
    )
    @OCIEValue(
        ocieColumn = "isActive"
    )
    private Boolean isActive;

    @OCIEZapiSource(
        sourceZapi = "nis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.nis-domain-config-info.nis-servers.ip-address"
    )
    @OCIEValue(
        ocieColumn = "nisServers"
    )
    private String nisServerIPs;

    NISDomain(final VServer parent, final String clusterUUID, final String vServerUUID, final String domainName) {
        super(parent, "NisDomain");
        this.key = new NISDomainKey(clusterUUID, vServerUUID, domainName);
    }


    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getActive() {
        return isActive;
    }

    public void setActive(final Boolean active) {
        isActive = active;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getNisServerIPs() {
        return nisServerIPs;
    }

    public void setNisServerIPs(final String nisServerIPs) {
        this.nisServerIPs = nisServerIPs;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getClusterUUID() {
        return key.getID(NISDomainKey.Field.cluster_uuid);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getVServerUUID() {
        return key.getID(NISDomainKey.Field.vserver_uuid);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getDomainName() {
        return key.getID(NISDomainKey.Field.nis_domain_name);
    }


    @Override
    public NISDomainKey getKey() {
        return key;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "NisDomain", key.getID(NISDomainKey.Field.nis_domain_name));
        addAttributeNoCRC(writer, "IsActive", isActive);
        addAttributeNoCRC(writer, "NisServers", nisServerIPs);
    }

}
