package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import java.io.IOException;
import java.util.EnumMap;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.*;

/**
 * IP Address Spaces (IPspace) are a collection of network ports and routing
 * tables to segregate network traffic to provide logically-isolated storage.
 * This is advantageous for storage administration in that multiple, potentially
 * adversarial, users can share the same hardware yet be unaware of, and unable
 * to access, storage provisioned for others.
 */
@OCIETable("network_ip_space")
@OCIEResourceType("network_ip_space")
public class NetworkIpSpace extends BaseCRCXMLNode<Cluster> {

    public static final class NetworkIpSpaceKey extends KeyBase<NetworkIpSpaceKey.Field> {

        /**
         * YYYYMMDD
         */
        private static final long serialVersionUID = 20140318L;

        public enum Field {

            @OCIEZapiSource(
                    sourceZapi = "net-ipspaces-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.net-ipspaces-info.uuid"
            )
            UUID,
        }

        @SuppressWarnings("serial")
        public NetworkIpSpaceKey(final String uuid) {
            super("network_ip_space",
                    new EnumMap<Field, String>(Field.class) {{
                        put(Field.UUID, uuid);
                    }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getID(Field.UUID);
        }
    }

    @OCIEZapiSource(
            sourceZapi = "net-ipspaces-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-ipspaces-info.id"
    )
    @OCIEValue(
            ocieColumn = "networkIpSpaceId"
    )
    private Long id;

    @OCIEZapiSource(
            sourceZapi = "net-ipspaces-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-ipspaces-info.ipSpaceName"
    )
    @OCIEValue(
            ocieColumn = "name"
    )
    private String ipSpaceName;

    private final NetworkIpSpaceKey ipSpaceKey;

    public NetworkIpSpace(final Cluster parent, final String ipSpaceUUID) {
        super(parent, "NetworkIpSpace");
        ipSpaceKey = new NetworkIpSpaceKey(ipSpaceUUID);
    }

    @Override
    public NetworkIpSpaceKey getKey() {
        return ipSpaceKey;
    }

    public Long getId() {
        return id;
    }

    public NetworkIpSpace setId(Long id) {
        this.id = id;
        return this;
    }

    public String getIPSpaceName() {
        return ipSpaceName;
    }

    public NetworkIpSpace setIPSpaceName(String ipSpace) {
        this.ipSpaceName = ipSpace;
        return this;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", ipSpaceKey);
        addRequiredAttributeAndCRC(writer, "UUID", ipSpaceKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", ipSpaceName);
        addRequiredAttributeAndCRC(writer, "NetworkIpSpaceId", id);
    }
}
