package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;


@OCIETable("network_port")
@OCIEResourceType("network_port")
public class NetworkPort extends BaseCRCXMLNode<Node> {

    public final static class PortKey extends KeyBase<PortKey.Field> {
        private final String macAddress;

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "system-node-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.node-details-info.node-uuid"
            )
            cluster_node_uuid,

            @OCIEZapiSource(
                sourceZapi = "net-port-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.net-port-info.port"
            )
            port,
        }

        public PortKey(final String clusterUUID, final String nodeUUID, final String portName, final String macAddress) {
            super("network_port", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.cluster_node_uuid, nodeUUID);
                put(Field.port, portName);
            }});
            this.macAddress = macAddress;
        }

        public String getPortName() {
            return getID(Field.port);
        }

        @Override
        @OCIEUUID(uuidDescr = "port.mac_address.cluster_node_uuid")
        public String getOcieUUID() {
            return getID(PortKey.Field.port) + '.' + macAddress + '.' + getID(PortKey.Field.cluster_node_uuid);
        }

    }

    @OCIEValue(
        ocieColumn = "resourceKey"
    )
    private final PortKey key;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.port-type"
    )
    @OCIEValue(
        ocieColumn = "portType"
    )
    private final PortType portType;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.port-type"
    )
    @OCIEValue(
        ocieColumn = "portTypeRaw"
    )
    private String portTypeRaw;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.role"
    )
    @OCIEValue(
        ocieColumn = "portRole"
    )
    private Role portRole;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.role"
    )
    @OCIEValue(
        ocieColumn = "roleRaw"
    )
    private String roleRaw;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.link-status"
    )
    @OCIEValue(
        ocieColumn = "linkStatus"
    )
    private LinkStatus linkStatus;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.link-status"
    )
    @OCIEValue(
        ocieColumn = "linkStatusRaw"
    )
    private String linkStatusRaw;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.operationalSpeed"
    )
    @OCIEValue(
        ocieColumn = "operationalSpeed"
    )
    private Speed operSpeed;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.operational-speed"
    )
    @OCIEValue(
        ocieColumn = "operationalSpeedRaw"
    )
    private String operSpeedRaw;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.administrative-speed"
    )
    @OCIEValue(
        ocieColumn = "adminSpeed"
    )
    private Speed adminSpeed;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.administrative-speed"
    )
    @OCIEValue(
        ocieColumn = "adminSpeedRaw"
    )
    private String adminSpeedRaw;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.mtu"
    )
    @OCIEValue(
        ocieColumn = "mtu"
    )
    private Long mtu;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.vlan-id"
    )
    @OCIEValue(
        ocieColumn = "vlanIdentifier"
    )
    private Long vlanID;


    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.vlan-node"
    )
    // Contributes to vlanPortId, but is not directly represented in its own column.
    private Node.NodeKey vlanNode;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.vlan-port"
    )
    @OCIEValue(
        ocieColumn = "vlanPortId"
    )
    private String vlanPort;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.ifgrp-node"
    )
    // Contributes to ifgrpPortId, but is not directly represented in its own column.
    private Node.NodeKey ifGroupNode;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.ifgrp-port"
    )
    @OCIEValue(
        ocieColumn = "ifgrpPortId"
    )
    private String ifGroupPort;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.operational-duplex"
    )
    @OCIEValue(
        ocieColumn = "operationalDuplex"
    )
    private OperationalDuplex operationalDuplex;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.operational-duplex"
    )
    @OCIEValue(
        ocieColumn = "operationalDuplexRaw"
    )
    private String operationalDuplexRaw;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.ipspace"
    )
    @OCIEValue(
        ocieColumn = "networkIpSpaceId"
    )
    private NetworkIpSpace.NetworkIpSpaceKey networkIpSpaceKey;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.broadcast-domain-name"
    )
    @OCIEValue(
        ocieColumn = "networkPortBroadcastDomainId"
    )
    private NetworkPortBroadcastDomain.NetworkPortBroadcastDomainKey netPortBroadcastDomainKey;



    NetworkPort(final Node parent, final String portName, final String macAddress, final PortType portType) {
        super(parent, NetworkPort.class.getSimpleName());
        this.portType = portType;
        final Node.NodeKey nodeKey = parent.getKey();
        final String clusterUUID = nodeKey.getID(Node.NodeKey.Field.cluster_uuid);
        final String nodeUUID = nodeKey.getID(Node.NodeKey.Field.uuid);
        this.key = new PortKey(clusterUUID, nodeUUID, portName, macAddress);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public PortKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public PortType getPortType() {
        return portType;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Role getPortRole() {
        return portRole;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public LinkStatus getLinkStatus() {
        return linkStatus;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Speed getOperSpeed() {
        return operSpeed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Speed getAdminSpeed() {
        return adminSpeed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getMtu() {
        return mtu;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getVlanID() {
        return vlanID;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getVlanPort() {
        return vlanPort;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getIfGroupPort() {
        return ifGroupPort;
    }

    public void setPortRole(final Role portRole) {
        this.portRole = portRole;
    }

    public void setLinkStatus(final LinkStatus linkStatus) {
        this.linkStatus = linkStatus;
    }

    public void setOperSpeed(final Speed operSpeed) {
        this.operSpeed = operSpeed;
    }

    public void setAdminSpeed(final Speed adminSpeed) {
        this.adminSpeed = adminSpeed;
    }

    public void setMtu(final Long mtu) {
        this.mtu = mtu;
    }

    public void setVlanID(final Long vlanID) {
        this.vlanID = vlanID;
    }

    public void setVlanPort(final String vlanPort) {
        this.vlanPort = vlanPort;
    }

    public void setIfGroupPort(final String ifGroupPort) {
        this.ifGroupPort = ifGroupPort;
    }

    public void setOperationalDuplex(final OperationalDuplex operationalDuplex) {
        this.operationalDuplex = operationalDuplex;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getPortTypeRaw() {
        return portTypeRaw;
    }

    public void setPortTypeRaw(final String portTypeRaw) {
        this.portTypeRaw = portTypeRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getRoleRaw() {
        return roleRaw;
    }

    public void setRoleRaw(final String roleRaw) {
        this.roleRaw = roleRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLinkStatusRaw() {
        return linkStatusRaw;
    }

    public void setLinkStatusRaw(final String linkStatusRaw) {
        this.linkStatusRaw = linkStatusRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getOperSpeedRaw() {
        return operSpeedRaw;
    }

    public void setOperSpeedRaw(final String operSpeedRaw) {
        this.operSpeedRaw = operSpeedRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getAdminSpeedRaw() {
        return adminSpeedRaw;
    }

    public void setAdminSpeedRaw(final String adminSpeedRaw) {
        this.adminSpeedRaw = adminSpeedRaw;
    }

    public void setOperationalDuplexRaw(final String operationalDuplexRaw) {
        this.operationalDuplexRaw = operationalDuplexRaw;
    }

    public Node.NodeKey getVlanNode() {
        return vlanNode;
    }

    public void setVlanNode(final Node.NodeKey vlanNode) {
        this.vlanNode = vlanNode;
    }

    public Node.NodeKey getIfGroupNode() {
        return ifGroupNode;
    }

    public void setIfGroupNode(final Node.NodeKey ifGroupNode) {
        this.ifGroupNode = ifGroupNode;
    }

    public NetworkIpSpace.NetworkIpSpaceKey getNetworkIpSpaceKey() {
        return networkIpSpaceKey;
    }

    public void setNetworkIpSpaceKey(NetworkIpSpace.NetworkIpSpaceKey networkIpSpaceKey) {
        this.networkIpSpaceKey = networkIpSpaceKey;
    }

    public NetworkPortBroadcastDomain.NetworkPortBroadcastDomainKey getNetPortBroadcastDomainKey() {
        return netPortBroadcastDomainKey;
    }

    public void setNetPortBroadcastDomainKey(NetworkPortBroadcastDomain.NetworkPortBroadcastDomainKey netPortBroadcastDomainKey) {
        this.netPortBroadcastDomainKey = netPortBroadcastDomainKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum LinkStatus {
        NOT_MAPPED,
        UNKNOWN,
        UP,
        DOWN, }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Speed {
        NOT_MAPPED,
        UNDEF,
        AUTO,
        MEGABIT_10,
        MEGABIT_100,
        GIGABIT_1,
        GIGABIT_10, }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Role {
        NOT_MAPPED,
        UNDEF,
        CLUSTER,
        DATA,
        NODE_MGMT,
        INTERCLUSTER, }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum PortType {
        NOT_MAPPED,
        PHYSICAL,
        IF_GROUP,
        VLAN,
        UNDEF, }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum OperationalDuplex {
        NOT_MAPPED,
        UNDEF,
        AUTO,
        HALF,
        FULL
    }


    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", key.getID(PortKey.Field.port));
        addRequiredAttributeAndCRC(writer, "MACAddress", key.macAddress);
        addRequiredAttributeAndCRC(writer, "AdministrativeSpeed", adminSpeed);
        addRequiredAttributeAndCRC(writer, "AdministrativeSpeedRaw", adminSpeedRaw);
        addRequiredAttributeAndCRC(writer, "OperationalSpeed", operSpeed);
        addRequiredAttributeAndCRC(writer, "OperationalSpeedRaw", operSpeedRaw);
        addRequiredAttributeAndCRC(writer, "LinkStatus", linkStatus);
        addRequiredAttributeAndCRC(writer, "LinkStatusRaw", linkStatusRaw);
        addRequiredAttributeAndCRC(writer, "Role", portRole);
        addRequiredAttributeAndCRC(writer, "RoleRaw", roleRaw);
        addRequiredAttributeAndCRC(writer, "PortType", portType);
        addRequiredAttributeAndCRC(writer, "PortTypeRaw", portTypeRaw);
        addRequiredAttributeAndCRC(writer, "VLANIdentifier", vlanID);
        addRequiredAttributeAndCRC(writer, "MTU", mtu);
        addRequiredAttributeAndCRC(writer, "OperationalDuplex", operationalDuplex);
        addRequiredAttributeAndCRC(writer, "OperationalDuplexRaw", operationalDuplexRaw);
        if(netPortBroadcastDomainKey != null) {
            addRequiredAttributeAndCRC(writer, "NetworkPortBroadcastDomainUUID", netPortBroadcastDomainKey.getOcieUUID());
        }
        if(networkIpSpaceKey != null) {
            addRequiredAttributeAndCRC(writer, "NetworkIpSpaceUUID", networkIpSpaceKey.getOcieUUID());
        }
        // Don't marshall vlan, ifgroup, they come later in a relationship table
    }
}
