package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEValue;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;

@OCIETable("network_port")
@OCIEResourceType("network_port")
public class NetworkPortRelationship extends BaseCRCXMLNode<Cluster> {

    private final NetworkPort.PortKey sourceNetworkPortKey;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.ifgrp-port"
    )
    @OCIEValue(
        ocieColumn = "ifgrpPortId"
    )
    private NetworkPort.PortKey ifGroupPortKey;

    @OCIEZapiSource(
        sourceZapi = "net-port-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.net-port-info.vlan-port"
    )
    @OCIEValue(
        ocieColumn = "vlanPortId"
    )
    private NetworkPort.PortKey vlanPortKey;

    NetworkPortRelationship(final Cluster cluster, final NetworkPort.PortKey sourceNetworkPortKey) {
        super(cluster, "NetworkPortRelationship");
        this.sourceNetworkPortKey = sourceNetworkPortKey;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public NetworkPort.PortKey getKey() {
        return sourceNetworkPortKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public NetworkPort.PortKey getIfGroupPortKey() {
        return ifGroupPortKey;
    }

    public void setIfGroupPortKey(final NetworkPort.PortKey ifGroupPortKey) {
        this.ifGroupPortKey = ifGroupPortKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public NetworkPort.PortKey getVlanPortKey() {
        return vlanPortKey;
    }

    public void setVlanPortKey(final NetworkPort.PortKey vlanPortKey) {
        this.vlanPortKey = vlanPortKey;
    }


    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
//        addRequiredAttributeAndCRC(writer, "ResourceKey", sourceNetworkPortKey.getTailIDs());
        addRequiredAttributeAndCRC(writer, "NetworkPortUUID", sourceNetworkPortKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "VLANPortUUID", KeyBase.getOcieUUIDOrNull(vlanPortKey));
        addRequiredAttributeAndCRC(writer, "IFGRPPortUUID", KeyBase.getOcieUUIDOrNull(ifGroupPortKey));
    }
}
