package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("network_subnet")
@OCIEResourceType("network_subnet")
public class NetworkSubnet extends BaseCRCXMLNode<Cluster> {
    public final static class NetworkSubnetKey extends KeyBase<NetworkSubnetKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "net-subnet-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.net-subnet-info.ipspace"
            )
            @OCIEValue(
                    ocieColumn = "ipspaceId"
            )
            ip_space_uuid,

            @OCIEZapiSource(
                    sourceZapi = "net-subnet-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.net-subnet-info.subnet-name"
            )
            @OCIEValue(
                    ocieColumn = "subnetName"
            )

            subnetName,
        }

        public NetworkSubnetKey(final String ipspaceUUID, final String subnetName) {
            super("network_subnet", new EnumMap<Field, String>(Field.class) {{
                put(Field.ip_space_uuid, ipspaceUUID);
                put(Field.subnetName, subnetName);
            }});
        }

        public String getNetworkSubnetName() {
            return getID(Field.subnetName);
        }

        public String getIpspaceUUID() {
            return getID(Field.ip_space_uuid);
        }

        @Override
        @OCIEUUID(uuidDescr = "ip_space_uuid.name")
        public String getOcieUUID() {
            return getID(Field.ip_space_uuid) + "." + getID(Field.subnetName);
        }
    }

    @OCIEZapiSource(
            sourceZapi = "net-subnet-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-subnet-info.subnet"
    )
    @OCIEValue(
            ocieColumn = "subnet"
    )
    private String subnet;

    @OCIEZapiSource(
            sourceZapi = "net-subnet-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-subnet-info.available-count"
    )
    @OCIEValue(
            ocieColumn = "availableCount"
    )
    private int availableCount;

    @OCIEZapiSource(
            sourceZapi = "net-subnet-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-subnet-info.broadcast-domain-name"
    )
    @OCIEValue(
            ocieColumn = "broadcastDomainId"
    )
    private String broadcastDomainUUID;

    @OCIEZapiSource(
            sourceZapi = "net-subnet-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-subnet-info.force-update-lif-associations"
    )
    @OCIEValue(
            ocieColumn = "forceUpdateLifAssociations"
    )
    private Boolean forceUpdateLifAssociations;

    @OCIEZapiSource(
            sourceZapi = "net-subnet-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-subnet-info.gateway"
    )
    @OCIEValue(
            ocieColumn = "gateway"
    )
    private String gateway;

    @OCIEZapiSource(
            sourceZapi = "net-subnet-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-subnet-info.ip-ranges"
    )
    @OCIEValue(
            ocieColumn = "ipRanges"
    )
    private String ipRanges;

    @OCIEZapiSource(
            sourceZapi = "net-subnet-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-subnet-info.ipspace"
    )
    @OCIEValue(
            ocieColumn = "ipspaceId"
    )
    private String ipspaceUUID;

    @OCIEZapiSource(
            sourceZapi = "net-subnet-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-subnet-info.total-count"
    )
    @OCIEValue(
            ocieColumn = "totalCount"
    )
    private int totalCount;

    @OCIEZapiSource(
            sourceZapi = "net-subnet-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-subnet-info.used-count"
    )
    @OCIEValue(
            ocieColumn = "usedCount"
    )
    private int usedCount;

    private final NetworkSubnetKey subnetKey;

    @Override
    public NetworkSubnetKey getKey() {
        return subnetKey;
    }

    public NetworkSubnet(final Cluster parent, final String ipSpaceUUID, final String subnetName) {
        super(parent, NetworkSubnet.class.getSimpleName());
        subnetKey = new NetworkSubnetKey(ipSpaceUUID, subnetName);
    }

    public String getSubnet() {
        return subnet;
    }

    public void setSubnet(String subnet) {
        this.subnet = subnet;
    }

    public int getAvailableCount() {
        return availableCount;
    }

    public void setAvailableCount(int availableCount) {
        this.availableCount = availableCount;
    }

    public String getBroadcastDomainUUID() {
        return broadcastDomainUUID;
    }

    public void setBroadcastDomainUUID(String broadcastDomainUUID) {
        this.broadcastDomainUUID = broadcastDomainUUID;
    }

    public Boolean getForceUpdateLifAssociations() {
        return forceUpdateLifAssociations;
    }

    public void setForceUpdateLifAssociations(Boolean forceUpdateLifAssociations) {
        this.forceUpdateLifAssociations = forceUpdateLifAssociations;
    }

    public String getGateway() {
        return gateway;
    }

    public void setGateway(String gateway) {
        this.gateway = gateway;
    }

    public String getIpRanges() {
        return ipRanges;
    }

    public void setIpRanges(String ipRanges) {
        this.ipRanges = ipRanges;
    }

    public String getIpspaceUUID() {
        return ipspaceUUID;
    }

    public void setIpspaceUUID(String ipspaceUUID) {
        this.ipspaceUUID = ipspaceUUID;
    }

    public int getTotalCount() {
        return totalCount;
    }

    public void setTotalCount(int totalCount) {
        this.totalCount = totalCount;
    }

    public int getUsedCount() {
        return usedCount;
    }

    public void setUsedCount(int usedCount) {
        this.usedCount = usedCount;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", subnetKey);
        addRequiredAttributeAndCRC(writer, "UUID", subnetKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "IpspaceUUID", subnetKey.getIpspaceUUID());
        addRequiredAttributeAndCRC(writer, "SubnetName", subnetKey.getNetworkSubnetName());
        addRequiredAttributeAndCRC(writer, "Subnet", subnet);
        addRequiredAttributeAndCRC(writer, "BroadcastDomainUUID", broadcastDomainUUID);
        addRequiredAttributeAndCRC(writer, "AvailableCount", availableCount);
        addRequiredAttributeAndCRC(writer, "ForceUpdateLifAssociations", forceUpdateLifAssociations);
        addRequiredAttributeAndCRC(writer, "Gateway", gateway);
        addRequiredAttributeAndCRC(writer, "Ipranges", ipRanges);
        addRequiredAttributeAndCRC(writer, "TotalCount", totalCount);
        addRequiredAttributeAndCRC(writer, "UsedCount", usedCount);
    }


}
