package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("node")
@OCIEResourceType("cluster_node")
public class Node extends BaseCRCXMLNode<Cluster> {

    public final static class NodeKey extends KeyBase<NodeKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "system-node-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.node-details-info.node-uuid"
            )
            uuid
        }

        public NodeKey(final String clusterUUID, final String nodeUUID) {
            super("cluster_node", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.uuid, nodeUUID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getID(Field.uuid);
        }
    }

    private final NodeKey nodeKey;

    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.node"
    )
    @OCIEValue(
        ocieColumn = "name"
    )
    private String name;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.node-model"
    )
    @OCIEValue(
        ocieColumn = "model"
    )
    private String model;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.node-serial-number"
    )
    @OCIEValue(
        ocieColumn = "serialNumber"
    )
    private String serialNum;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.node-uptime"
    )
    @OCIEValue(
        ocieColumn = "uptime"
    )
    private Long uptime;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.node-vendor"
    )
    @OCIEValue(
        ocieColumn = "vendor"
    )
    private String vendor;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.node-location"
    )
    @OCIEValue(
        ocieColumn = "location"
    )
    private String location;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.env-over-temperature"
    )
    @OCIEValue(
        ocieColumn = "isOverTemperature"
    )
    private Boolean envOverTemperature;

    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.env-failed-fan-message"
    )
    @OCIEValue(
        ocieColumn = "envFailedFanMessage"
    )
    private String envFailedFanMessage;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.env-failed-power-supply-message"
    )
    @OCIEValue(
        ocieColumn = "envFailedPowerSupplyMessage"
    )
    private String envFailedPowerSupplyMessage;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.nvram-battery-status"
    )
    @OCIEValue(
        ocieColumn = "envFailedPowerSupplyMessage",
        notes = "enum derived from envFailedPowerSupplyMessageRaw"
    )
    private BatteryState nvramBatteryStatus;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.nvram-battery-status"
    )
    @OCIEValue(
        ocieColumn = "envFailedPowerSupplyMessageRaw"
    )
    private String nvramBatteryStatusRaw;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.node-owner"
    )
    @OCIEValue(
        ocieColumn = "owner"
    )
    private String owner;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.is-node-healthy"
    )
    @OCIEValue(
        ocieColumn = "isNodeHealthy"
    )
    private Boolean isHealthy;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.env-failed-fan-count"
    )
    @OCIEValue(
        ocieColumn = "failedFanCount"
    )
    private Integer failedFanCount;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.failed-power-supply-count"
    )
    @OCIEValue(
        ocieColumn = "failedPowerSupplyCount"
    )
    private Integer failedPowerSupplyCount;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.node-nvram-id"
    )
    @OCIEValue(
        ocieColumn = "nvramId"
    )
    private String nvramID;


    @OCIEZapiSource(
        sourceZapi = "system-node-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.node-details-info.cpu-firmware-release"
    )
    @OCIEValue(
        ocieColumn = "cpuFirmwareRelease"
    )
    private String cpuFirmwareRelease;


    @OCIEZapiSource(
        sourceZapi = "system-get-version",
        zapiResponsePath = "netapp.results.node-version-details.node-version-detail-info.version"
    )
    @OCIEValue(
        ocieColumn = "version",
        notes = "or cluster.version when node version breakdown not acquired"
    )
    private String version;


    @OCIEZapiSource(
        sourceZapi = "system-get-version",
        zapiResponsePath = "netapp.results.node-version-details.node-version-detail-info.version"

    )
    @OCIEValue(
        ocieColumn = "versionGeneration",
        notes = "or cluster.versionGeneration when node version breakdown not acquired"
    )
    private Integer versionGeneration = null;


    @OCIEZapiSource(
        sourceZapi = "system-get-version",
        zapiResponsePath = "netapp.results.node-version-details.node-version-detail-info.version"
    )
    @OCIEValue(
        ocieColumn = "versionMajor",
        notes = "or cluster.versionMajor when node version breakdown not acquired"
    )
    private Integer versionMajor = null;


    @OCIEZapiSource(
        sourceZapi = "system-get-version",
        zapiResponsePath = "netapp.results.node-version-details.node-version-detail-info.version"

    )
    @OCIEValue(
        ocieColumn = "versionMinor",
        notes = "or cluster.versionMinor when node version breakdown not acquired"
    )
    private Integer versionMinor = null;


    @OCIEZapiSource(
        sourceZapi = "system-get-node-info-iter",
        zapiResponsePath = "netapp.results.attributes-list.system-info.cpu-processor-type"
    )
    @OCIEValue(
        ocieColumn = "cpuProcessorType"
    )
    private String cpuType;


    @OCIEZapiSource(
        sourceZapi = "system-get-node-info-iter",
        zapiResponsePath = "netapp.results.attributes-list.system-info.cpu-processor-id"
    )
    @OCIEValue(
        ocieColumn = "cpuProcessorId"
    )
    private String cpuID;


    @OCIEZapiSource(
        sourceZapi = "system-get-node-info-iter",
        zapiResponsePath = "netapp.results.attributes-list.system-info.number-of-processors"
    )
    @OCIEValue(
        ocieColumn = "numberOfProcessors"
    )
    private Integer cpuCount;


    @OCIEZapiSource(
        sourceZapi = "system-get-node-info-iter",
        zapiResponsePath = "netapp.results.attributes-list.system-info.memory-size"
    )
    @OCIEValue(
        ocieColumn = "memorySizeMB"
    )
    private Integer memSize;


    @OCIEZapiSource(
        sourceZapi = "system-get-node-info-iter",
        zapiResponsePath = "netapp.results.attributes-list.system-info.prod-type"
    )
    @OCIEValue(
        ocieColumn = "productType"
    )
    private String productType;                 // (not seen but doc'ed, maybe try system-machine-type?)


    @OCIEZapiSource(
        sourceZapi = "cf-status",
        zapiArgs = "node",
        zapiResponsePath = "netapp.results.is-enabled"
    )
    @OCIEValue(
        ocieColumn = "isFailoverEnabled"
    )
    private Boolean isFailoverEnabled;


    @OCIEZapiSource(
        sourceZapi = "cf-status",
        zapiArgs = "node",
        zapiResponsePath = "netapp.results.state"
    )
    @OCIEValue(
        ocieColumn = "failoverState"
    )
    private String failoverState;


    @OCIEZapiSource(
        sourceZapi = "cf-status",
        zapiArgs = "node",
        zapiResponsePath = "netapp.results.takeover-possible"
    )
    @OCIEValue(
        ocieColumn = "isTakeOverPossible"
    )
    private Boolean isTakeoverPossible;


    @OCIEZapiSource(
        sourceZapi = "cf-status",
        zapiArgs = "node",
        zapiResponsePath = "netapp.results.partner-firmware-state"
    )
    @OCIEValue(
        ocieColumn = "partnerFirmwareState"
    )
    private String partnerFirmwareState;


    @OCIEZapiSource(
        sourceZapi = "cf-status",
        zapiArgs = "node",
        zapiResponsePath = "netapp.results.local-firmware-state"
    )
    @OCIEValue(
        ocieColumn = "localFirmwareState"
    )
    private String localFirmwareState;


    @OCIEZapiSource(
        sourceZapi = "cf-status",
        zapiArgs = "node",
        zapiResponsePath = "netapp.results.is-interconnect-up"
    )
    @OCIEValue(
        ocieColumn = "isInterconnectUp"
    )
    private Boolean isInterconnectUp;


    @OCIEZapiSource(
        sourceZapi = "cf-status",
        zapiArgs = "node",
        zapiResponsePath = "netapp.results.interconnect-links"
    )
    @OCIEValue(
        ocieColumn = "interconnectLinks"
    )
    private String interconnectLinks;


    @OCIEZapiSource(
        sourceZapi = "cf-status",
        zapiArgs = "node",
        zapiResponsePath = "netapp.results.giveback-state"
    )
    @OCIEValue(
        ocieColumn = "giveBackState"
    )
    private String giveBackState;


    @OCIEZapiSource(
        sourceZapi = "cf-status",
        zapiArgs = "node",
        zapiResponsePath = "netapp.results.interconnect-type"
    )
    @OCIEValue(
        ocieColumn = "interconnectType"
    )
    private String interconnectType;


    @OCIEZapiSource(
        sourceZapi = "cf-status",
        zapiArgs = "node",
        zapiResponsePath = "netapp.results.current-mode"
    )
    @OCIEValue(
        ocieColumn = "currentMode"
    )
    private String currentMode;


    @OCIEZapiSource(
        sourceZapi = "cf-status",
        zapiArgs = "node",
        zapiResponsePath = "netapp.results.takeover-failure-reason"
    )
    @OCIEValue(
        ocieColumn = "takeoverFailureReason"
    )
    private String takeoverFailureReason;

    @OCIEZapiSource(
            sourceZapi = "system-node-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.node-details-info.is-epsilon-node"
    )
    @OCIEValue(
            ocieColumn = "isEpsilonNode"
    )
    private Boolean isEpsilonNode;

    @OCIEZapiSource(
            sourceZapi = "system-node-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.node-details-info.is-all-flash-optimized"
    )
    @OCIEValue(
            ocieColumn = "isAllFlashOptimized"
    )
    private Boolean isAllFlashOptimized;

    @OCIEZapiSource(
        sourceZapi = "cf-status",
        zapiArgs = "node",
        zapiResponsePath = "netapp.results.takeover-by-partner-not-possible-reason.takeover-info.reason"
    )
    @OCIEValue(
        ocieColumn = "takeoverByPartnerNotPossibleReason",
        notes = "CSV list of reasons"
    )
    private String takeoverByPartnerNotPossibleReasons;


    @OCIEZapiSource(
        sourceZapi = "cf-status",
        zapiArgs = "node",
        zapiResponsePath = "netapp.results.takeover-of-partner-not-possible-reason.takeover-info.reason"
    )
    @OCIEValue(
        ocieColumn = "takeoverOfPartnerNotPossibleReason",
        notes = "CSV list of reasons"
    )
    private String takeoverOfPartnerNotPossibleReasons;

    // aggregated from disks
    @OCIEZapiSource(
        sourceZapi = "derived"
    )
    @OCIEValue(
        ocieColumn = "rawDiskBytesTotal",
        notes = "Sum of all disks hosted by this node regardless of state or role.",
        defaultValue = "0"
    )
    private Long rawDiskSizeTotal = 0L;


    @OCIEZapiSource(
        sourceZapi = "derived"
    )
    @OCIEValue(
        ocieColumn = "rawDiskBytesUsed",
        notes = "Sum of all disks hosted by this node regardless of state or role.",
        defaultValue = "0"
    )
    private Long rawDiskBytesUsed = 0L;


    // aggregated from Aggregates
    @OCIEZapiSource(
        sourceZapi = "derived"
    )
    @OCIEValue(
        ocieColumn = "aggregateBytesTotal",
        notes = "Sum of all aggregates (total + snapshot) hosted by this node.",
        defaultValue = "0"
    )
    private Long aggregateBytesTotal = 0L;

    @OCIEZapiSource(
        sourceZapi = "derived"
    )
    @OCIEValue(
        ocieColumn = "aggregateBytesUsed",
        notes = "Sum of all aggregates (used + snapshot used) hosted by this node.",
        defaultValue = "0"
    )
    private Long aggregateBytesUsed = 0L;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-node-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.metrocluster-node-info.dr-partner-uuid"
    )
    @OCIEValue(
            ocieColumn = "mtDrPartnerNodeUUID"
    )
    private String mtDrPartnerNodeUUID;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-node-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.metrocluster-node-info.dr-operation-state"
    )
    @OCIEValue(
            ocieColumn = "metroclusterOperationState"
    )
    private MtDrOperationState mtDrOperationState;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-node-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.metrocluster-node-info.dr-operation-state"
    )
    @OCIEValue(
            ocieColumn = "mtDrOperationStateRaw"
    )
    private String mtDrOperationStateRaw;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-node-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.metrocluster-node-info.automatic-uso"
    )
    @OCIEValue(
            ocieColumn = "mtAutomaticUso"
    )
    private Boolean mtAutomaticUso;

    // children
    private final Aggregates aggregates;
    private final NetworkPorts networkPorts;
    private final FcpPorts fcpPorts;
    private final StorageAdapters adapters;
    private final Processors processors;
    private final QosServiceCenters qosServiceCenters;
    private final QosServiceCenterRelationships qosServiceCenterRelationships;
    private final ServiceProcessors serviceProcessors;
    private final FlashDevices flashDevices;

    Node(final Cluster parent, final String uuid) {
        super(parent, "Node");
        this.nodeKey = new NodeKey(parent.getClusterUUID(), uuid);
        this.aggregates = new Aggregates(this);
        this.networkPorts = new NetworkPorts(this);
        this.fcpPorts = new FcpPorts(this);
        this.adapters = new StorageAdapters(this);
        this.processors = new Processors(this);
        this.qosServiceCenters = new QosServiceCenters(this);
        this.qosServiceCenterRelationships = new QosServiceCenterRelationships(this);
        this.serviceProcessors = new ServiceProcessors(this);
        this.flashDevices = new FlashDevices(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public NodeKey getKey() {
        return nodeKey;
    }

    public String getNodeUUID() {
        return nodeKey.getID(NodeKey.Field.uuid);
    }

    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    @SuppressWarnings({"unused declaration"})
    public String getModel() {
        return model;
    }

    public void setModel(final String model) {
        this.model = model;
    }

    @SuppressWarnings({"unused declaration"})
    public String getSerialNum() {
        return serialNum;
    }

    public void setSerialNum(final String serialNum) {
        this.serialNum = serialNum;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getUptime() {
        return uptime;
    }

    public void setUptime(final Long uptime) {
        this.uptime = uptime;
    }

    @SuppressWarnings({"unused declaration"})
    public String getVendor() {
        return vendor;
    }

    public void setVendor(final String vendor) {
        this.vendor = vendor;
    }

    @SuppressWarnings({"unused declaration"})
    public String getLocation() {
        return location;
    }

    public void setLocation(final String location) {
        this.location = location;
    }

    @SuppressWarnings({"unused declaration"})
    public Boolean isEnvOverTemperature() {
        return envOverTemperature;
    }

    public void setEnvOverTemperature(final Boolean envOverTemperature) {
        this.envOverTemperature = envOverTemperature;
    }

    @SuppressWarnings({"unused declaration"})
    public String getEnvFailedFanMessage() {
        return envFailedFanMessage;
    }

    public void setEnvFailedFanMessage(final String envFailedFanMessage) {
        this.envFailedFanMessage = envFailedFanMessage;
    }

    @SuppressWarnings({"unused declaration"})
    public String getEnvFailedPowerSupplyMessage() {
        return envFailedPowerSupplyMessage;
    }

    public void setEnvFailedPowerSupplyMessage(final String envFailedPowerSupplyMessage) {
        this.envFailedPowerSupplyMessage = envFailedPowerSupplyMessage;
    }

    @SuppressWarnings({"unused declaration"})
    public BatteryState getNvramBatteryStatus() {
        return nvramBatteryStatus;
    }

    public void setNvramBatteryStatus(final BatteryState nvramBatteryStatus) {
        this.nvramBatteryStatus = nvramBatteryStatus;
    }

    public void setNvramBatteryStatusRaw(final String nvramBatteryStatusRaw) {
        this.nvramBatteryStatusRaw = nvramBatteryStatusRaw;
    }

    @SuppressWarnings({"unused declaration"})
    public String getVersion() {
        return version;
    }

    public void setVersion(final String version) {
        this.version = version;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getVersionGeneration() {
        return versionGeneration;
    }

    public void setVersionGeneration(final Integer versionGeneration) {
        this.versionGeneration = versionGeneration;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getVersionMajor() {
        return versionMajor;
    }

    public void setVersionMajor(final Integer versionMajor) {
        this.versionMajor = versionMajor;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getVersionMinor() {
        return versionMinor;
    }

    public void setVersionMinor(final Integer versionMinor) {
        this.versionMinor = versionMinor;
    }

    @SuppressWarnings({"unused declaration"})
    public String getCpuType() {
        return cpuType;
    }

    public void setCpuType(final String cpuType) {
        this.cpuType = cpuType;
    }

    @SuppressWarnings({"unused declaration"})
    public String getCpuID() {
        return cpuID;
    }

    public void setCpuID(final String cpuID) {
        this.cpuID = cpuID;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getCpuCount() {
        return cpuCount;
    }

    public void setCpuCount(final Integer cpuCount) {
        this.cpuCount = cpuCount;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getMemSize() {
        return memSize;
    }

    public void setMemSize(final Integer memSize) {
        this.memSize = memSize;
    }

    @SuppressWarnings({"unused declaration"})
    public String getProductType() {
        return productType;
    }

    public void setProductType(final String productType) {
        this.productType = productType;
    }

    public String getMtDrPartnerNodeUUID() {
        return mtDrPartnerNodeUUID;
    }

    public void setMtDrPartnerNodeUUID(final String mtDrPartnerNodeId) {
        this.mtDrPartnerNodeUUID = mtDrPartnerNodeId;
    }

    public MtDrOperationState getMtDrOperationState() {
        return mtDrOperationState;
    }

    public void setMtDrOperationState(final MtDrOperationState mtDrOperationState) {
        this.mtDrOperationState = mtDrOperationState;
    }

    public String getMtDrOperationStateRaw() {
        return mtDrOperationStateRaw;
    }

    public void setMtDrOperationStateRaw(final String mtDrOperationStateRaw) {
        this.mtDrOperationStateRaw = mtDrOperationStateRaw;
    }

    public void setAllFlashOptimized(Boolean allFlashOptimized) {
        isAllFlashOptimized = allFlashOptimized;
    }

    public Boolean getMtAutomaticUso() {
        return mtAutomaticUso;
    }

    public void setMtAutomaticUso(Boolean mtAutomaticUso) {
        this.mtAutomaticUso = mtAutomaticUso;
    }

    /**
     *  State of DR operation. See
     * http://http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/DOT/mainN/web_ontap_cluster/metrocluster/index.html#metrocluster-node-info
     */
    public static enum MtDrOperationState {
        NOT_MAPPED,
        NORMAL,
        SWITCHOVER_IN_PROGRESS,
        SWITCHOVER_FAILED,
        SWITCHOVER_COMPLETED,
        HEAL_AGGRS_IN_PROGRESS,
        HEAL_AGGRS_FAILED,
        HEAL_AGGRS_COMPLETED,
        HEAL_ROOTS_FAILED,
        HEAL_ROOTS_COMPLETED,
        SWITCHBACK_IN_PROGRESS,
        SWTICHBACK_FAILED,
    }


//    @SuppressWarnings({"unused declaration"})
//    public String getSystemID() {
//        return systemID;
//    }
//
//    public void setSystemID(final String systemID) {
//        this.systemID = systemID;
//    }

//    @SuppressWarnings({"unused declaration"})
//    public String getPartnerID() {
//        return partnerID;
//    }
//
//    public void setPartnerID(final String partnerID) {
//        this.partnerID = partnerID;
//    }

//    @SuppressWarnings({"unused declaration"})
//    public String getPartnerName() {
//        return partnerName;
//    }
//
//    public void setPartnerName(final String partnerName) {
//        this.partnerName = partnerName;
//    }

    @SuppressWarnings({"unused declaration"})
    public String getFailoverState() {
        return failoverState;
    }

    public void setFailoverState(final String failoverState) {
        this.failoverState = failoverState;
    }

    @SuppressWarnings({"unused declaration"})
    public Boolean isFailoverEnabled() {
        return isFailoverEnabled;
    }

    public void setFailoverEnabled(final Boolean failoverEnabled) {
        isFailoverEnabled = failoverEnabled;
    }

    @SuppressWarnings({"unused declaration"})
    public Boolean isTakeoverPossible() {
        return isTakeoverPossible;
    }

    public void setTakeoverPossible(final Boolean takeoverPossible) {
        isTakeoverPossible = takeoverPossible;
    }

    @SuppressWarnings({"unused declaration"})
    public String getPartnerFirmwareState() {
        return partnerFirmwareState;
    }

    public void setPartnerFirmwareState(final String partnerFirmwareState) {
        this.partnerFirmwareState = partnerFirmwareState;
    }

    @SuppressWarnings({"unused declaration"})
    public String getLocalFirmwareState() {
        return localFirmwareState;
    }

    public void setLocalFirmwareState(final String localFirmwareState) {
        this.localFirmwareState = localFirmwareState;
    }

    @SuppressWarnings({"unused declaration"})
    public Boolean isInterconnectUp() {
        return isInterconnectUp;
    }

    public void setInterconnectUp(final Boolean interconnectUp) {
        isInterconnectUp = interconnectUp;
    }

    @SuppressWarnings({"unused declaration"})
    public String getInterconnectLinks() {
        return interconnectLinks;
    }

    public void setInterconnectLinks(final String interconnectLinks) {
        this.interconnectLinks = interconnectLinks;
    }

    @SuppressWarnings({"unused declaration"})
    public String getGiveBackState() {
        return giveBackState;
    }

    public void setGiveBackState(final String giveBackState) {
        this.giveBackState = giveBackState;
    }

//    @SuppressWarnings({"unused declaration"})
//    public String getInterconnectName() {
//        return interconnectName;
//    }
//
//    public void setInterconnectName(final String interconnectName) {
//        this.interconnectName = interconnectName;
//    }

    @SuppressWarnings({"unused declaration"})
    public String getInterconnectType() {
        return interconnectType;
    }

    public void setInterconnectType(final String interconnectType) {
        this.interconnectType = interconnectType;
    }

    @SuppressWarnings({"unused declaration"})
    public String getCurrentMode() {
        return currentMode;
    }

    public void setCurrentMode(final String currentMode) {
        this.currentMode = currentMode;
    }

    @SuppressWarnings({"unused declaration"})
    public String getTakeoverByPartnerNotPossibleReasons() {
        return takeoverByPartnerNotPossibleReasons;
    }

    public void setTakeoverByPartnerNotPossibleReasons(final String takeoverByPartnerNotPossibleReasons) {
        this.takeoverByPartnerNotPossibleReasons = takeoverByPartnerNotPossibleReasons;
    }

    @SuppressWarnings({"unused declaration"})
    public String getTakeoverOfPartnerNotPossibleReasons() {
        return takeoverOfPartnerNotPossibleReasons;
    }

    public void setTakeoverOfPartnerNotPossibleReasons(final String takeoverOfPartnerNotPossibleReasons) {
        this.takeoverOfPartnerNotPossibleReasons = takeoverOfPartnerNotPossibleReasons;
    }

    @SuppressWarnings({"unused declaration"})
    public String getTakeoverFailureReason() {
        return takeoverFailureReason;
    }

    public void setTakeoverFailureReason(final String takeoverFailureReason) {
        this.takeoverFailureReason = takeoverFailureReason;
    }

    @SuppressWarnings({"unused declaration"})
    public String getOwner() {
        return owner;
    }

    public void setOwner(final String owner) {
        this.owner = owner;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getFailedFanCount() {
        return failedFanCount;
    }

    public void setFailedFanCount(final Integer failedFanCount) {
        this.failedFanCount = failedFanCount;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getFailedPowerSupplyCount() {
        return failedPowerSupplyCount;
    }

    public void setFailedPowerSupplyCount(final Integer failedPowerSupplyCount) {
        this.failedPowerSupplyCount = failedPowerSupplyCount;
    }

    @SuppressWarnings({"unused declaration"})
    public String getNvramID() {
        return nvramID;
    }

    public void setNvramID(final String nvramID) {
        this.nvramID = nvramID;
    }

    @SuppressWarnings({"unused declaration"})
    public String getCpuFirmwareRelease() {
        return cpuFirmwareRelease;
    }

    public void setCpuFirmwareRelease(final String cpuFirmwareRelease) {
        this.cpuFirmwareRelease = cpuFirmwareRelease;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getRawDiskSizeTotal() {
        return rawDiskSizeTotal;
    }

    public void addToDiskSizeTotal(final Long diskSizeTotal) {
        if (diskSizeTotal != null) {
            this.rawDiskSizeTotal += diskSizeTotal;
        }
    }

    @SuppressWarnings({"unused declaration"})
    public Long getRawDiskBytesUsed() {
        return rawDiskBytesUsed;
    }

    public void addToDiskSizeUsed(final Long diskSizeUsed) {
        if (diskSizeUsed != null) {
            this.rawDiskBytesUsed += diskSizeUsed;
        }
    }

    @SuppressWarnings({"unused declaration"})
    public Long getAggregateBytesTotal() {
        return aggregateBytesTotal;
    }

    public void addToAggregateBytesTotal(final Long aggregateTotalBytes) {
        if (aggregateTotalBytes != null) {
            this.aggregateBytesTotal += aggregateTotalBytes;
        }
    }

    @SuppressWarnings({"unused declaration"})
    public Long getAggregateBytesUsed() {
        return aggregateBytesUsed;
    }

    public void addToAggregateBytesUsed(final Long aggregateUsedBytes) {
        if (aggregateUsedBytes != null) {
            this.aggregateBytesUsed += aggregateUsedBytes;
        }
    }

    @SuppressWarnings({"unused declaration"})
    public Boolean isHealthy() {
        return isHealthy;
    }

    public void setHealthy(final Boolean healthy) {
        isHealthy = healthy;
    }

    public void setIsEpsilonNode(final Boolean epsilonNode) {
        this.isEpsilonNode = epsilonNode;
    }

    @SuppressWarnings({"unused declaration"})
    public Boolean getIsEpsilonNode() {
        return isEpsilonNode;
    }

    @SuppressWarnings({"unused declaration"})
    public static enum BatteryState {
        // TODO use these instead
        NOT_MAPPED,
        BATTERY_OK,
        BATTERY_PARTIALLY_DISCHARGED,
        BATTERY_FULLY_DISCHARGED,
        BATTERY_NOT_PRESENT,
        BATTERY_NEAR_END_OF_LIFE,
        BATTERY_AT_END_OF_LIFE,
        BATTERY_UNKNOWN,
        BATTERY_OVER_CHARGED,
        BATTERY_FULLY_CHARGED
    }

    public Aggregate createAggregate(final String uuid) {
        return new Aggregate(this, uuid);
    }

    public Aggregates getAggregates() {
        return aggregates;
    }
    
    public Aggregate getAggregateByAggregateKey(Aggregate.AggregateKey aggrKey){
    	return aggregates.get(aggrKey);
    }

    public NetworkPort createPort(final String portName, final String macAddress, final NetworkPort.PortType portType) {
        return new NetworkPort(this, portName, macAddress, portType);
    }

    public NetworkPorts getNetworkPorts() {
        return networkPorts;
    }

    public FcpPort createFcpPort(final String nodeWWN, final String portWWN, final String adapterName) {
        return new FcpPort(this, nodeWWN, portWWN, adapterName);
    }

    public FcpPorts getFcpPorts() {
        return fcpPorts;
    }

    public StorageAdapter createStorageAdapter(final String adapterName) {
        return new StorageAdapter(this, adapterName);
    }

    public StorageAdapters getStorageAdapters() {
        return adapters;
    }

    public Processors getProcessors() {
        return processors;
    }

    public Processor createProcessor(final Integer processorNumber) {
        return new Processor(this, processorNumber);
    }

    public QosServiceCenters getQosServiceCenters() {
        return qosServiceCenters;
    }

    public QosServiceCenter createQosServiceCenter(final String uuid) {
        return new QosServiceCenter(this, uuid);
    }

    public QosServiceCenterRelationships getQosServiceCenterRelationships() {
        return qosServiceCenterRelationships;
    }

    public ServiceProcessor createServiceProcessor(final String macAddress) {
        return new ServiceProcessor(this, macAddress);
    }

    public ServiceProcessors getServiceProcessors() {
        return serviceProcessors;
    }

    public QosServiceCenterRelationship createQosServiceCenterRelationship(final QosServiceCenter.QosResourceKey qosServiceCenterKey,
                                                                           final QosServiceCenter.QosResourceKey parentQosServiceCenterKey) {
        return new QosServiceCenterRelationship(this, qosServiceCenterKey, parentQosServiceCenterKey);
    }

    public FlashDevices getFlashDevices() {
        return flashDevices;
    }

    public FlashDevice createFlashDevice(final String serialNumber) {
        return new FlashDevice(this, serialNumber);
    }




    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", nodeKey);
        addRequiredAttributeAndCRC(writer, "UUID", nodeKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", name);
        addRequiredAttributeAndCRC(writer, "Model", model);
        addRequiredAttributeAndCRC(writer, "SerialNumber", serialNum);
        addRequiredAttributeAndCRC(writer, "Uptime", uptime);
        addRequiredAttributeAndCRC(writer, "Version", version);
        addRequiredAttributeAndCRC(writer, "VersionGeneration", versionGeneration);
        addRequiredAttributeAndCRC(writer, "VersionMajor", versionMajor);
        addRequiredAttributeAndCRC(writer, "VersionMinor", versionMinor);
        addRequiredAttributeAndCRC(writer, "Vendor", vendor);
        addRequiredAttributeAndCRC(writer, "Location", location);
        addRequiredAttributeAndCRC(writer, "FailoverState", failoverState);
        addRequiredAttributeAndCRC(writer, "CPUProcessorType", cpuType);
        addRequiredAttributeAndCRC(writer, "CPUProcessorId", cpuID);
        addRequiredAttributeAndCRC(writer, "NumberOfProcessors", cpuCount);
        addRequiredAttributeAndCRC(writer, "MemorySizeMB", memSize);
        addRequiredAttributeAndCRC(writer, "ProductType", productType);
        addRequiredAttributeAndCRC(writer, "EnvFailedFanMessage", envFailedFanMessage);
        addRequiredAttributeAndCRC(writer, "EnvFailedPowerSupplyMessage", envFailedPowerSupplyMessage);
        addRequiredAttributeAndCRC(writer, "NVRAMBatteryStatus", nvramBatteryStatus);
        addRequiredAttributeAndCRC(writer, "NVRAMBatteryStatusRaw", nvramBatteryStatusRaw);
        addRequiredAttributeAndCRC(writer, "NVRAMId", nvramID);
        addRequiredAttributeAndCRC(writer, "CPUFirmwareRelease", cpuFirmwareRelease);
        addRequiredAttributeAndCRC(writer, "IsOverTemperature", envOverTemperature);
        addRequiredAttributeAndCRC(writer, "FailedFanCount", failedFanCount);
        addRequiredAttributeAndCRC(writer, "FailedPowerSupplyCount", failedPowerSupplyCount);
        addRequiredAttributeAndCRC(writer, "RawDiskBytesTotal", rawDiskSizeTotal);
        addRequiredAttributeAndCRC(writer, "RawDiskBytesUsed", rawDiskBytesUsed);
        addRequiredAttributeAndCRC(writer, "IsFailoverEnabled", isFailoverEnabled);
        addRequiredAttributeAndCRC(writer, "IsTakeOverPossible", isTakeoverPossible);
        addRequiredAttributeAndCRC(writer, "PartnerFirmwareState", partnerFirmwareState);
        addRequiredAttributeAndCRC(writer, "LocalFirmwareState", localFirmwareState);
        addRequiredAttributeAndCRC(writer, "InterconnectLinks", interconnectLinks);
        addRequiredAttributeAndCRC(writer, "GiveBackState", giveBackState);
//        addRequiredAttributeAndCRC(writer, "InterconnectName", interconnectName);
        addRequiredAttributeAndCRC(writer, "InterconnectType", interconnectType);
        addRequiredAttributeAndCRC(writer, "IsInterconnectUp", isInterconnectUp);
        addRequiredAttributeAndCRC(writer, "CurrentMode", currentMode);        
        addRequiredAttributeAndCRC(writer, "TakeoverByPartnerNotPossibleReason", takeoverByPartnerNotPossibleReasons);
        addRequiredAttributeAndCRC(writer, "TakeoverOfPartnerNotPossibleReason", takeoverOfPartnerNotPossibleReasons);
        addRequiredAttributeAndCRC(writer, "TakeoverFailureReason", takeoverFailureReason);
        addRequiredAttributeAndCRC(writer, "Owner", owner);
        addRequiredAttributeAndCRC(writer, "IsNodeHealthy", isHealthy);
        addRequiredAttributeAndCRC(writer, "IsEpsilonNode", isEpsilonNode);
        addRequiredAttributeAndCRC(writer, "IsAllFlashOptimized", isAllFlashOptimized);
        addRequiredAttributeAndCRC(writer, "AggregateBytesTotal", aggregateBytesTotal);
        addRequiredAttributeAndCRC(writer, "AggregateBytesUsed", aggregateBytesUsed);
        addRequiredAttributeAndCRC(writer, "MtDrPartnerNodeUUID", mtDrPartnerNodeUUID);
        addRequiredAttributeAndCRC(writer, "MtDrOperationState", getMtDrOperationState());
        addRequiredAttributeAndCRC(writer, "MtDrOperationStateRaw", getMtDrOperationStateRaw());
        addRequiredAttributeAndCRC(writer, "MtAutomaticUso", getMtAutomaticUso());
    }

    @Override
    protected boolean hasChildren() {
        return !aggregates.isEmpty() || !networkPorts.isEmpty() || !fcpPorts.isEmpty() || !adapters.isEmpty() || !processors.isEmpty() || !qosServiceCenterRelationships.isEmpty() || !serviceProcessors.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, "ServiceProcessors", serviceProcessors.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "Processors", processors.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "Aggregates", aggregates.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "NetworkPorts", networkPorts.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "FcpPorts", fcpPorts.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "StorageAdapters", adapters.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "QosServiceCenters", qosServiceCenters.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "QosServiceCenterRelationships", qosServiceCenterRelationships.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "FlashDevices", flashDevices.values(), tabs);
    }
}