package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEUUID;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("portset_to_igroup")
@OCIEResourceType("portset_to_igroup")
public class PortsetInitiatorGroup extends BaseCRCXMLNode<Portset> {

    public final static class IGroupKey extends KeyBase<IGroupKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "igroup-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.initiator-group-info.initiator-group-uuid"
            )
            uuid
        }

        public IGroupKey(final String clusterUUID, final String uuid) {
            super("igroup", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.uuid, uuid);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getID(Field.uuid);
        }
    }

    private final IGroupKey key;

    PortsetInitiatorGroup(final Portset parent, final String iGroupUUID) {
        super(parent, "PortsetIgroupRef");
        this.key = new IGroupKey(parent.getKey().getID(Portset.PortSetKey.Field.cluster_uuid), iGroupUUID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public IGroupKey getKey() {
        return key;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        //addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "IgroupUUID", key.getOcieUUID());
    }
}
