package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("portset_port")
@OCIEResourceType("portset_port")
public class PortsetPort extends BaseCRCXMLNode<Portset> {

    public final static class PortsetPortKey extends KeyBase<PortsetPortKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "portset-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.portset-info.portset-port-info.portset-port-name"
            )
            port_name
        }

        public PortsetPortKey(final String clusterUUID, final String portName) {
            super("portset_port", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.port_name, portName);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.port_name")
        public String getOcieUUID() {
            return getFQID();
        }
    }

    private final PortsetPortKey key;

    PortsetPort(final Portset parent, final String portName) {
        super(parent, "PortsetPort");
        this.key = new PortsetPortKey(parent.getKey().getID(Portset.PortSetKey.Field.cluster_uuid), portName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public PortsetPortKey getKey() {
        return key;
    }

    //fix of DE5013
    @OCIEZapiSource(
            sourceZapi = "portset-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.portset-info.portset-port-info.portset-port-name"
    )
    @OCIEValue(
            ocieColumn = "fcpLifId"
    )
    private String fcpLifId = null;

    @OCIEZapiSource(
            sourceZapi = "portset-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.portset-info.portset-port-info.portset-port-name"
    )
    @OCIEValue(
            ocieColumn = "iSCSIPortalGroupId"
    )
    private String iScsiPortalGroupId = null;

    public String getFcpPortId() {
         return fcpLifId;
    }

    public void setFcpLifId(String fcpId) {
        this.fcpLifId = fcpId;
    }

    public String getIScsiPortalGroupId() {
        return iScsiPortalGroupId;
    }

    public void setIScsiPortalGroupId(String iscsiId) {
        this.iScsiPortalGroupId = iscsiId;
    }
    //end of DE5013

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        // addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        // addRequiredAttributeAndCRC(writer, "UUID", key.getTailIDs());
        addRequiredAttributeAndCRC(writer, "Name", key.getID(PortsetPortKey.Field.port_name));
        addRequiredAttributeAndCRC(writer, "FcpLifUUID", fcpLifId);
        addRequiredAttributeAndCRC(writer, "IScsiPortalGroupUUID", iScsiPortalGroupId);
    }

}
