package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEUUID;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

/**
 * This object represents a physical CPU inside of a Node
 */
@OCIETable("processor")
@OCIEResourceType("processor")
public class Processor extends BaseCRCXMLNode<Node> {

    public final static class ProcessorKey extends KeyBase<ProcessorKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "system-node-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.node-details-info.node-uuid"
            )
            node_uuid,

            @OCIEZapiSource(
                sourceZapi = "system-get-node-info-iter",
                zapiResponsePath = "netapp.results.attributes-list.system-info.number-of-processors"
            )
            processor_num
        }

        public ProcessorKey(final String clusterUUID, final String nodeUUID, final String processorNum) {
            super("processor", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.node_uuid, nodeUUID);
                put(Field.processor_num, processorNum);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "node_uuid.processor_num")
        public String getOcieUUID() {
            return getID(Field.node_uuid) + '.' + getID(Field.processor_num);
        }
    }

    private final ProcessorKey key;

    Processor(final Node parent, final Integer processorNumber) {
        super(parent, Processor.class.getSimpleName());
        final Node.NodeKey parentKey = parent.getKey();
        final String clusterUUID = parentKey.getID(Node.NodeKey.Field.cluster_uuid);
        final String nodeUUID = parentKey.getID(Node.NodeKey.Field.uuid);
        this.key = new ProcessorKey(clusterUUID, nodeUUID, String.valueOf(processorNumber));
    }



    @Override
    @SuppressWarnings({"unchecked"})
    public ProcessorKey getKey() {
        return key;
    }

    public String getProcessorNumber() {
        return key.getID(ProcessorKey.Field.processor_num);
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "ProcessorNumber", getProcessorNumber());
    }
}
