package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("qtree")
@OCIEResourceType("qtree")
public class QTree extends BaseCRCXMLNode<Volume> {

    public final static class QTreeKey extends KeyBase<QTreeKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "volume-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.uuid"
            )
            volume_uuid,

            @OCIEZapiSource(
                sourceZapi = "qtree-list-iter",
                zapiResponsePath = "netapp.results.attributes-list.qtree-info.id"
            )
            qtree_id
        }

        public QTreeKey(final String clusterUUID, final String vServerUUID, final String volumeUUID, final String qtreeID) {
            super("qtree", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.volume_uuid, volumeUUID);
                put(Field.qtree_id, qtreeID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "qtree_id.volume_uuid")
        public String getOcieUUID() {
            return getID(Field.qtree_id) + '.' + getID(Field.volume_uuid);
        }
    }

    // qtree-info from qtree-list-iter
    private final QTreeKey key;                         // UUID

    @OCIEZapiSource(
        sourceZapi = "qtree-list-iter",
        zapiResponsePath = "netapp.results.attributes-list.qtree-info.qtree"
    )
    @OCIEValue(
        ocieColumn = "name"
    )
    private String qTreeName;                           // qtree

    @OCIEZapiSource(
        sourceZapi = "qtree-list-iter",
        zapiResponsePath = "netapp.results.attributes-list.qtree-info.status"
    )
    @OCIEValue(
        ocieColumn = "status",
        notes = "derived enum"
    )
    private Status status;                              // status

    @OCIEZapiSource(
        sourceZapi = "qtree-list-iter",
        zapiResponsePath = "netapp.results.attributes-list.qtree-info.status"
    )
    @OCIEValue(
        ocieColumn = "statusRaw"
    )
    private String statusRaw;                           // status

    @OCIEZapiSource(
        sourceZapi = "qtree-list-iter",
        zapiResponsePath = "netapp.results.attributes-list.qtree-info.security-style"
    )
    @OCIEValue(
        ocieColumn = "securityStyle",
        notes = "derived enum"
    )
    private SecurityStyle securityStyle;                // security-style

    @OCIEZapiSource(
        sourceZapi = "qtree-list-iter",
        zapiResponsePath = "netapp.results.attributes-list.qtree-info.security-style"
    )
    @OCIEValue(
        ocieColumn = "securityStyleRaw"
    )
    private String securityStyleRaw;                    // security-style

    @OCIEZapiSource(
        sourceZapi = "qtree-list-iter",
        zapiResponsePath = "netapp.results.attributes-list.qtree-info.oplocks"
    )
    @OCIEValue(
        ocieColumn = "oplocks"
    )
    private Boolean opLocks;                            // oplocks


    @OCIEZapiSource(
            sourceZapi = "qtree-list-iter",
            zapiResponsePath = "netapp.results.attributes-list.qtree-info.export-policy"
    )
    @OCIEValue(
            ocieColumn = "exportPolicyId"
    )
    private ExportPolicy.ExportPolicyKey exportPolicyKey; // export-policy or Volume's export policy when is-export-policy-inherited


    @OCIEZapiSource(
            sourceZapi = "qtree-list-iter",
            zapiResponsePath = "netapp.results.attributes-list.qtree-info.is-export-policy-inherited"
    )
    @OCIEValue(
            ocieColumn = "isExportPolicyInherited"
    )
    private Boolean isExportPolicyInherited;


    QTree(final Volume parent, final String qTreeID, final String volumeUUID) {
        super(parent, "Qtree");
        final VServer vServer = parent.getParent();
        final VServer.VServerKey vServerKey = vServer.getKey();
        final String clusterUUID = vServerKey.getID(VServer.VServerKey.Field.cluster_uuid);
        final String vServerUUID = vServerKey.getZapiUUID();
        this.key = new QTreeKey(clusterUUID, vServerUUID, volumeUUID, qTreeID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public QTreeKey getKey() {
        return key;
    }

    public String getName() {
        return qTreeName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getqTreeName() {
        return qTreeName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Status getStatus() {
        return status;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public SecurityStyle getSecurityStyle() {
        return securityStyle;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getOpLocks() {
        return opLocks;
    }

    public void setqTreeName(final String qTreeName) {
        this.qTreeName = qTreeName;
    }

    public void setStatus(final Status status) {
        this.status = status;
    }

    public void setSecurityStyle(final SecurityStyle securityStyle) {
        this.securityStyle = securityStyle;
    }

    public void setOpLocks(final Boolean opLocks) {
        this.opLocks = opLocks;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getStatusRaw() {
        return statusRaw;
    }

    public void setStatusRaw(final String statusRaw) {
        this.statusRaw = statusRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getSecurityStyleRaw() {
        return securityStyleRaw;
    }

    public void setSecurityStyleRaw(final String securityStyleRaw) {
        this.securityStyleRaw = securityStyleRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public ExportPolicy.ExportPolicyKey getExportPolicyKey() {
        return exportPolicyKey;
    }

    public void setExportPolicyKey(ExportPolicy.ExportPolicyKey exportPolicyKey) {
        this.exportPolicyKey = exportPolicyKey;
    }

    public Boolean getExportPolicyInherited() {
        return isExportPolicyInherited;
    }

    public void setExportPolicyInherited(Boolean exportPolicyInherited) {
        isExportPolicyInherited = exportPolicyInherited;
    }

    public String getQTreeID() {
        return key.getID(QTreeKey.Field.qtree_id);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Status {
        NOT_MAPPED,
        SNAPVAULTED,
        SNAPMIRRORED,
        NORMAL,
        READONLY,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum SecurityStyle {
        NOT_MAPPED,
        UNIX,
        NTFS,
        MIXED,
    }

    protected void marshallAttributesToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(cache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "QtreeIdentifier", getQTreeID());
        addRequiredAttributeAndCRC(writer, "Name", qTreeName);
        addRequiredAttributeAndCRC(writer, "Status", status);
        addRequiredAttributeAndCRC(writer, "StatusRaw", statusRaw);
        addRequiredAttributeAndCRC(writer, "SecurityStyle", securityStyle);
        addRequiredAttributeAndCRC(writer, "SecurityStyleRaw", securityStyleRaw);
        addRequiredAttributeAndCRC(writer, "Oplocks", opLocks);
        addRequiredAttributeAndCRC(writer, "ExportPolicyUUID", KeyBase.getOcieUUIDOrNull(exportPolicyKey));
        addRequiredAttributeAndCRC(writer, "IsExportPolicyInherited", isExportPolicyInherited);
    }
}
