package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("qos_service_center_relationship")
@OCIEResourceType("qos_service_center_relationship")
public class QosServiceCenterRelationship extends BaseCRCXMLNode<Node> {

    private final RelationshipKey key;
    private final QosServiceCenter.QosResourceKey qosServiceCenterKey;
    private final QosServiceCenter.QosResourceKey parentQosServiceCenterKey;

    QosServiceCenterRelationship(final Node parent, final QosServiceCenter.QosResourceKey qosServiceCenterKey, final QosServiceCenter.QosResourceKey parentQosServiceCenterKey) {
        super(parent, "QosServiceCenterRelationship");
        final String clusterUUID = parent.getKey().getID(Node.NodeKey.Field.cluster_uuid);
        this.key = new RelationshipKey(clusterUUID, qosServiceCenterKey.getOcieUUID(), parentQosServiceCenterKey.getOcieUUID());
        this.qosServiceCenterKey = qosServiceCenterKey;
        this.parentQosServiceCenterKey = parentQosServiceCenterKey;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public RelationshipKey getKey() {
        return key;
    }

    public final static class RelationshipKey extends KeyBase<RelationshipKey.Field> {

        public static enum Field {
            cluster_uuid,
            parent_qos_service_center_uuid,
            qos_service_center_uuid
        }

        public RelationshipKey(final String clusterUUID, final String qosServiceCenterUUID, final String parentQosServiceCenterUUID) {
            super("qos_service_center_relationship", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.parent_qos_service_center_uuid, parentQosServiceCenterUUID);
                put(Field.qos_service_center_uuid, qosServiceCenterUUID);
            }});
        }

        public String getOcieUUID() {
            return getFQID();
        }
    }

    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        // addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "QosServiceCenterUUID", qosServiceCenterKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "ParentQosServiceCenterUUID", parentQosServiceCenterKey.getOcieUUID());
    }    
}
