package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("qos_service_center")
@OCIEResourceType("qos_service_center")
public class QosVolumeWorkload extends BaseCRCXMLNode<Cluster> {

    /**
     * Globally unique key based on uuid.
     */
    public final static class QosVolumeWorkloadKey extends KeyBase<QosVolumeWorkloadKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "qos-workload-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.qos-workload-info.workload-uuid"
            )
            uuid
        }

        public QosVolumeWorkloadKey(final String clusterUUID, final String uuid) {
            super("qos_volume_workload", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.uuid, uuid);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getFQID();
        }
    }

    private final QosVolumeWorkloadKey key;

    @OCIEZapiSource(
        sourceZapi = "qos-workload-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.qos-workload-info.workload-name"
    )
    @OCIEValue(
        ocieColumn = "name"
    )
    private String name;

    @OCIEZapiSource(
        sourceZapi = "qos-workload-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.qos-workload-info.workload-class"
    )
    @OCIEValue(
        ocieColumn = "workloadClass"
    )
    private QosWorkloadClassType qosWorkloadClassType;

    @OCIEZapiSource(
            sourceZapi = "qos-workload-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.qos-workload-info.workload-class"
    )
    @OCIEValue(
            ocieColumn = "workloadClassRaw"
    )
    private String qosWorkloadClassTypeRaw;

    @OCIEZapiSource(
            sourceZapi = "qos-workload-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.qos-workload-info.vserver"
    )
    @OCIEValue(
            ocieColumn = "vserverId"
    )
    private VServer.VServerKey vServerKey;

    @OCIEZapiSource(
            sourceZapi = "qos-workload-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.qos-workload-info.volume"
    )
    @OCIEValue(
            ocieColumn = "volumeId"
    )
    private Volume.VolumeKey volumeKey;

    protected QosVolumeWorkload(final Cluster parent, final String uuid) {
        super(parent, QosVolumeWorkload.class.getSimpleName());
        this.key = new QosVolumeWorkloadKey(parent.getClusterUUID(), uuid);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public QosVolumeWorkloadKey getKey() {
        return key;
    }

    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    @SuppressWarnings("UnusedDeclaration")
    public QosWorkloadClassType getQosWorkloadClassType(){
        return qosWorkloadClassType;
    }

    public void setQosWorkloadClassType(final QosWorkloadClassType qosWorkloadClassType){
        this.qosWorkloadClassType = qosWorkloadClassType;
    }

    @SuppressWarnings("UnusedDeclaration")
    public String getQosWorkloadClassTypeRaw() {
        return qosWorkloadClassTypeRaw;
    }

    public void setQosWorkloadClassTypeRaw(final String qosWorkloadClassRaw) {
        this.qosWorkloadClassTypeRaw = qosWorkloadClassRaw;
    }

    public VServer.VServerKey getvServerKey() {
        return vServerKey;
    }

    public void setvServerKey(final VServer.VServerKey vServerKey) {
        this.vServerKey = vServerKey;
    }

    @SuppressWarnings("UnusedDeclaration")
    public VServer.VServerKey getVServerKey() {
        return vServerKey;
    }

    public void setVServerKey(final VServer.VServerKey vServerKey) {
        this.vServerKey = vServerKey;
    }

    @SuppressWarnings("UnusedDeclaration")
    public Volume.VolumeKey getVolumeKey(){
        return volumeKey;
    }

    public void setVolumeKey(final Volume.VolumeKey volumeKey){
        this.volumeKey = volumeKey;
    }

    @SuppressWarnings("UnusedDeclaration")
    public static enum QosWorkloadClassType{
        NOT_MAPPED,
        PRESET,
        USER_DEFINED,
        UNDEFINED,
        AUTOVOLUME,
        SYSTEM_DEFINED;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", name);
        addRequiredAttributeAndCRC(writer, "VserverUUID", VServer.VServerKey.getOCIEVServerUUIDOrNull(vServerKey));
        addRequiredAttributeAndCRC(writer, "VolumeUUID", Volume.VolumeKey.getVolumeInstanceUUIDOrNull(volumeKey));
        addRequiredAttributeAndCRC(writer, "WorkloadClass", qosWorkloadClassType);
        addRequiredAttributeAndCRC(writer, "WorkloadClassRaw", qosWorkloadClassTypeRaw);
    }
}
