package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@OCIETable("qos_workload_detail")
@OCIEResourceType("qos_workload_detail")
public class QosWorkloadDetail extends BaseCRCXMLNode<Cluster> {

    /**
     * Globally unique key based on uuid.
     */
    public final static class QosWorkloadDetailKey extends KeyBase<QosWorkloadDetailKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "cluster-identity-get",
                    zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "perf-object-instance-list-info-iter",
                zapiArgs = "workload_detail",
                zapiResponsePath = "netapp.results.attributes-list.instance-info.uuid"
            )
            uuid
        }

        public QosWorkloadDetailKey(final String clusterUUID, final String uuid) {
            super("qos_workload_detail", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.uuid, uuid);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getFQID();
        }
    }

    private final QosWorkloadDetailKey key;

    @OCIEZapiSource(
        sourceZapi = "perf-object-instance-list-info-iter",
        zapiArgs = "workload_detail",
        zapiResponsePath = "netapp.results.attributes-list.instance-info.name"
    )
    @OCIEValue(
            ocieColumn = "name"
    )
    private String name;

    @OCIEZapiSource(
            sourceZapi = "derived"
    )
    @OCIEValue(
            ocieColumn = "workloadId"
    )
    private QosVolumeWorkload.QosVolumeWorkloadKey qosVolumeWorkloadKey;

    @OCIEZapiSource(
            sourceZapi = "derived"
    )
    @OCIEValue(
            ocieColumn = "serviceCenterId"
    )
    private QosServiceCenter.QosResourceKey qosServiceCenterKey;
    /**
     * Some workload details UUID contains a node name e.g. ${node_name}:kernel:_WAFL_CP.CPU_dblade.
     * This field is not written to the XML. It is used in the workload details post processing.
     */
    private String nodeNamePrefix;
    /**
     * Some workload details UUID are suffixed with aggregate uuid e.g. ${node_name}:kernel:_WAFL_CP.DISK_HDD_${aggregate}_${aggregate uuid}.
     * This field is not written to the XML. It is used in the workload details post processing.
     */
    private String aggregateUUIDSuffix;

    protected QosWorkloadDetail(final Cluster parent, final String uuid) {
        super(parent, QosWorkloadDetail.class.getSimpleName());
        this.key = new QosWorkloadDetailKey(parent.getClusterUUID(), uuid);
    }
    
    @Override
    @SuppressWarnings({"unchecked"})
    public QosWorkloadDetailKey getKey() {
        return key;
    }

    public void setName(final String name){
        this.name = name;
    }
    
    public String getName(){
        return name;
    }
    
    public void setQosVolumeWorkloadKey(final QosVolumeWorkload.QosVolumeWorkloadKey qosVolumeWorkloadKey){
        this.qosVolumeWorkloadKey = qosVolumeWorkloadKey;
    }

    @SuppressWarnings("UnusedDeclaration")
    public QosVolumeWorkload.QosVolumeWorkloadKey getQosVolumeWorkloadKey(){
        return qosVolumeWorkloadKey;
    }
    
    public void setQosResourceKey(final QosServiceCenter.QosResourceKey qosServiceCenterKey){
        this.qosServiceCenterKey = qosServiceCenterKey;
    }

    public QosServiceCenter.QosResourceKey getQosResourceKey() {
        return qosServiceCenterKey;
    }

    public String getNodeNamePrefix() {
        return nodeNamePrefix;
    }

    public void setNodeNamePrefix(String nodeNamePrefix) {
        this.nodeNamePrefix = nodeNamePrefix;
    }

    public String getAggregateUUIDSuffix() {
        return aggregateUUIDSuffix;
    }

    public void setAggregateUUIDSuffix(String aggregateUUIDSuffix) {
        this.aggregateUUIDSuffix = aggregateUUIDSuffix;
    }
  
   @Override
    protected void marshallAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
	   QosVolumeWorkloads qosVolumeWorkloads = getParent().getQosVolumeWorkloads();
	   QosVolumeWorkload qosVolumeWorkload = qosVolumeWorkloads.get(qosVolumeWorkloadKey);
	   final Pattern WORKLOAD__DETAIL_UUID_REGULAR_EXP = Pattern.compile("^.*" + ":kernel:" + "(.*?)" + "\\." + "(.*)$");
	   final Matcher uuidMatcher = WORKLOAD__DETAIL_UUID_REGULAR_EXP.matcher(getKey().getOcieUUID());
	   if(qosVolumeWorkload.getVolumeKey() == null && (uuidMatcher.matches()) && ((uuidMatcher.group(2).startsWith("DISK_HDD") || uuidMatcher.group(2).startsWith("DISK_HDD_OTHER"))  || (uuidMatcher.group(2).startsWith("DISK_SSD") || uuidMatcher.group(2).startsWith("DISK_SSD_OTHER")))){

		   addTabulation(writer, tabs);
		   writer.append("<").append(getNodeName()).append(' ');
		   marshallAttributesToFileAndCRC(cache, writer);


		   writer.append("/>").append(lineSeperator);
	   }
   }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addAttributeNoCRC(writer, "ResourceKey", key);
        addAttributeNoCRC(writer, "UUID", key.getOcieUUID());
        addAttributeNoCRC(writer, "Name", name);
        addAttributeNoCRC(writer, "QosVolumeWorkloadUUID", KeyBase.getOcieUUIDOrNull(qosVolumeWorkloadKey));
        addAttributeNoCRC(writer, "QosServiceCenterUUID", KeyBase.getOcieUUIDOrNull(qosServiceCenterKey));
    }
}
