package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("qos_workload_node_relationship")
@OCIEResourceType("qos_workload_node_relationship")
/**
 * This class represents a workload at the node level. Each workload is associated with all the nodes in the cluster.
 */
public class QosWorkloadNodeRelationship extends BaseCRCXMLNode<Cluster> {

    /**
     * Globally unique key based on uuid.
     */
    public final static class QosWorkloadRelationshipKey extends KeyBase<QosWorkloadRelationshipKey.Field> {
        public static enum Field {

            @OCIEZapiSource(
                    sourceZapi = "system-node-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.node-details-info.node-uuid"
            )
            node_uuid,

            @OCIEZapiSource(
                    sourceZapi = "qos-workload-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.qos-workload-info.workload-uuid"
            )
            workload_uuid
        }

        public QosWorkloadRelationshipKey(final String nodeUUID, final String workloadUUID) {
            super("qos_workload_node_relationship", new EnumMap<Field, String>(Field.class) {{
                put(Field.node_uuid, nodeUUID);
                put(Field.workload_uuid, workloadUUID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "node_uuid.workload_uuid")
        public String getOcieUUID() {
            return getFQID();
        }
    }

    private final QosWorkloadRelationshipKey key;
    @OCIEZapiSource(
            sourceZapi = "system-node-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.node-details-info.node"
    )
    @OCIEValue(
            ocieColumn = "nodeId"
    )
    private final Node.NodeKey nodeKey;

    @OCIEZapiSource(
            sourceZapi = "qos-workload-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.qos-workload-info.workload-uuid"
    )
    @OCIEValue(
            ocieColumn = "workloadId"
    )
    private final QosVolumeWorkload.QosVolumeWorkloadKey qosWorkloadKey;

    protected QosWorkloadNodeRelationship(final Cluster parent, final Node.NodeKey nodeKey, final QosVolumeWorkload.QosVolumeWorkloadKey qosWorkloadKey) {
        super(parent, QosWorkloadNodeRelationship.class.getSimpleName());
        this.key = new QosWorkloadRelationshipKey(nodeKey.getID(Node.NodeKey.Field.uuid), qosWorkloadKey.getID(QosVolumeWorkload.QosVolumeWorkloadKey.Field.uuid));
        this.nodeKey = nodeKey;
        this.qosWorkloadKey = qosWorkloadKey;
    }

    public QosWorkloadRelationshipKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Node.NodeKey getNodeKey() {
        return nodeKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public QosVolumeWorkload.QosVolumeWorkloadKey getQosWorkloadKey() {
        return qosWorkloadKey;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addAttributeNoCRC(writer, "ResourceKey", key);
        addAttributeNoCRC(writer, "UUID", key.getOcieUUID());
        addAttributeNoCRC(writer, "NodeUUID", nodeKey.getOcieUUID());
        addAttributeNoCRC(writer, "QosWorkloadUUID", qosWorkloadKey.getOcieUUID());
    }
}
