package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.*;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("routing_group")
@OCIEResourceType("routing_group")
public class RoutingGroup extends BaseCRCXMLNode<VServer> {

    public final static class RoutingGroupKey extends KeyBase<RoutingGroupKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "net-routing-group-route-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.routing-group-route-info.routing-group"
            )
            routing_group_name
        }

        public RoutingGroupKey(final String clusterUUID, final String vServerUUID, final String groupName) {
            super("routing_group", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.routing_group_name, groupName);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.vserver_uuid.routing_group_name")
        public String getOcieUUID() {
            return getFQID();       // OCIE UUID already includes cluster UUID
        }
    }

    private final RoutingGroupKey key;

    @OCIEZapiSource(
        sourceZapi = "net-routing-group-route-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.routing-group-route-info.address-family"
    )
    @OCIEValue(
        ocieColumn = "addressFamily"
    )
    private AddressFamily addressFamily;

    @OCIEZapiSource(
        sourceZapi = "net-routing-group-route-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.routing-group-route-info.address-family"
    )
    @OCIEValue(
        ocieColumn = "addressFamilyRaw"
    )
    private String addressFamilyRaw;

    private final RoutingGroupDestinations routingGroupDestinations;

    RoutingGroup(final VServer parent, final String groupUUID) {
        super(parent, RoutingGroup.class.getSimpleName());
        final String clusterUUID = parent.getKey().getID(VServer.VServerKey.Field.cluster_uuid);
        final String vServerUUID = parent.getUUID();
        this.key = new RoutingGroupKey(clusterUUID, vServerUUID, groupUUID);
        this.routingGroupDestinations = new RoutingGroupDestinations(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public RoutingGroupKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public AddressFamily getAddressFamily() {
        return addressFamily;
    }

    public void setAddressFamily(final AddressFamily addressFamily) {
        this.addressFamily = addressFamily;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getAddressFamilyRaw() {
        return addressFamilyRaw;
    }

    public void setAddressFamilyRaw(final String addressFamilyRaw) {
        this.addressFamilyRaw = addressFamilyRaw;
    }

    public RoutingGroupDestinations getRoutingGroupDestinations() {
        return routingGroupDestinations;
    }

    public RoutingGroupDestination createRoutingGroupDestination(final String destinationAddress) {
        final String clusterUUID = key.getID(RoutingGroup.RoutingGroupKey.Field.cluster_uuid);
        final String vServerUUID = key.getID(RoutingGroup.RoutingGroupKey.Field.vserver_uuid);
        final String routingGroupName = key.getID(RoutingGroup.RoutingGroupKey.Field.routing_group_name);
        return new RoutingGroupDestination(this, clusterUUID, vServerUUID, routingGroupName, destinationAddress);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum AddressFamily {
        NOT_MAPPED,
        IPV4,
        IPV6,
    }



    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "RoutingGroupName", key.getID(RoutingGroupKey.Field.routing_group_name));
        addRequiredAttributeAndCRC(writer, "AddressFamily", addressFamily);
        addRequiredAttributeAndCRC(writer, "AddressFamilyRaw", addressFamilyRaw);
    }

    @Override
    protected boolean hasChildren() {
        return !routingGroupDestinations.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        super.marshallChildrenToFileAndCRC(cache, writer, tabs);
        // children martial themselves
        addAllChildrenUnderNodeAndCRC(cache, writer, "RoutingGroupDestinations", routingGroupDestinations.values(), tabs);
    }

    // Destinations are children
    public static final class RoutingGroupDestinations extends NodeMap<RoutingGroup, RoutingGroupDestination.RoutingGroupDestinationKey, RoutingGroupDestination> {
        RoutingGroupDestinations(final RoutingGroup routingGroup) {
            super(routingGroup);
        }

        public boolean add(final RoutingGroupDestination destination) {
            return (put(destination.getKey(), destination) == null);
        }
    }


    public static final class RoutingGroupDestination extends BaseCRCXMLNode<RoutingGroup> {

        private final RoutingGroupDestinationKey key;
        private String gatewayAddress;

        RoutingGroupDestination(final RoutingGroup parent, final String clusterUUID, final String vServerUUID,
                                final String routingGroupName, final String destinationAddress) {
            super(parent, "RoutingGroupDestination");
            this.key = new RoutingGroupDestinationKey(clusterUUID, vServerUUID, routingGroupName, destinationAddress);
        }



        @Override
        @SuppressWarnings({"unchecked"})
        public RoutingGroupDestinationKey getKey() {
            return key;
        }

        @SuppressWarnings({"UnusedDeclaration"})
        public String getGatewayAddress() {
            return gatewayAddress;
        }

        public void setGatewayAddress(final String gatewayAddress) {
            this.gatewayAddress = gatewayAddress;
        }

        public final static class RoutingGroupDestinationKey extends KeyBase<RoutingGroupDestinationKey.Field> {
            public static enum Field {
                cluster_uuid,
                vserver_uuid,
                routing_group_name,
                destination_address,
            }

            public RoutingGroupDestinationKey(final String clusterUUID, final String vServerUUID, final String groupName, final String destinationAddress) {
                super("routing_group", new EnumMap<Field, String>(Field.class) {{
                    put(Field.cluster_uuid, clusterUUID);
                    put(Field.vserver_uuid, vServerUUID);
                    put(Field.routing_group_name, groupName);
                    put(Field.destination_address, destinationAddress);
                }});
            }

            public String getOcieUUID() {
                return getFQID();       // OCIE UUID already includes cluster UUID
            }
        }

        protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
            super.marshallAttributesToFileAndCRC(marshallingCache, writer);
            addRequiredAttributeAndCRC(writer, "DestinationAddress", key.getID(RoutingGroupDestinationKey.Field.destination_address));
            addRequiredAttributeAndCRC(writer, "GatewayAddress", gatewayAddress);
        }        
    }
}
