package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("service_processor")
@OCIEResourceType("service_processor")
public class ServiceProcessor extends BaseCRCXMLNode<Node> {

    public final static class ServiceProcessorKey extends KeyBase<ServiceProcessorKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,


            @OCIEZapiSource(
                sourceZapi = "service-processor-get",
                zapiArgs = "node-name",
                zapiResponsePath = "netapp.results.attributes.service-processor-info.mac-address"
            )
            uuid                // is the MAC address but resource key wants the property name to be "uuid"
        }

        public ServiceProcessorKey(final String clusterUUID, final String macAddress) {
            super("service_processor", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.uuid, macAddress);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.uuid")
        public String getOcieUUID() {
            return getFQID();       // OCIE UUID already includes cluster
        }
    }

    private final ServiceProcessorKey key;

    @OCIEZapiSource(
        sourceZapi = "service-processor-get",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.attributes.service-processor-info.ip-address.ip-address"
    )
    @OCIEValue(
        ocieColumn = "ipAddress"
    )
    private String ipAddress;

    @OCIEZapiSource(
        sourceZapi = "service-processor-get",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.attributes.service-processor-info.is-ip-configured"
    )
    @OCIEValue(
        ocieColumn = "isIpConfigured"
    )
    private Boolean isIpConfigured;

    @OCIEZapiSource(
        sourceZapi = "service-processor-get",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.attributes.service-processor-info.status"
    )
    @OCIEValue(
        ocieColumn = "status"
    )
    private ServiceProcessorStatus status;

    @OCIEZapiSource(
        sourceZapi = "service-processor-get",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.attributes.service-processor-info.status"
    )
    @OCIEValue(
        ocieColumn = "statusRaw"
    )
    private String statusRaw;

    @OCIEZapiSource(
        sourceZapi = "service-processor-get",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.attributes.service-processor-info.type"
    )
    @OCIEValue(
        ocieColumn = "type"
    )
    private ServiceProcessorType type;

    @OCIEZapiSource(
        sourceZapi = "service-processor-get",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.attributes.service-processor-info.type"
    )
    @OCIEValue(
        ocieColumn = "typeRaw"
    )
    private String typeRaw;


    ServiceProcessor(final Node parent, final String macAddress) {
        super(parent, ServiceProcessor.class.getSimpleName());
        final String clusterUUID = parent.getKey().getID(Node.NodeKey.Field.cluster_uuid);
        this.key = new ServiceProcessorKey(clusterUUID, macAddress);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public ServiceProcessorKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getIpAddress() {
        return ipAddress;
    }

    public void setIpAddress(final String ipAddress) {
        this.ipAddress = ipAddress;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isIpConfigured() {
        return isIpConfigured;
    }

    public void setIpConfigured(final Boolean ipConfigured) {
        isIpConfigured = ipConfigured;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public ServiceProcessorStatus getStatus() {
        return status;
    }

    public void setStatus(final ServiceProcessorStatus status) {
        this.status = status;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getStatusRaw() {
        return statusRaw;
    }

    public void setStatusRaw(final String statusRaw) {
        this.statusRaw = statusRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public ServiceProcessorType getType() {
        return type;
    }

    public void setType(final ServiceProcessorType type) {
        this.type = type;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getTypeRaw() {
        return typeRaw;
    }

    public void setTypeRaw(final String typeRaw) {
        this.typeRaw = typeRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public enum ServiceProcessorType {
        NOT_MAPPED,
        SP,
        RLM,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public enum ServiceProcessorStatus {
        NOT_MAPPED,
        ONLINE,
        OFFLINE,
        SP_DAEMON_OFFLINE,
        NODE_OFFLINE,
        DEGRADED,
        REBOOTING,
        UNKNOWN,
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "MacAddress", key.getID(ServiceProcessorKey.Field.uuid));
        addRequiredAttributeAndCRC(writer, "IpAddress", ipAddress);
        addRequiredAttributeAndCRC(writer, "IsIpConfigured", isIpConfigured);
        addRequiredAttributeAndCRC(writer, "Status", status);
        addRequiredAttributeAndCRC(writer, "StatusRaw", statusRaw);
        addRequiredAttributeAndCRC(writer, "Type", type);
        addRequiredAttributeAndCRC(writer, "TypeRaw", typeRaw);
    }
}
