package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;


@OCIETable("sis_policy")
@OCIEResourceType("sis_policy")
public class SisPolicy extends BaseCRCXMLNode<VServer> {

    public final static class SisPolicyKey extends KeyBase<SisPolicyKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "sis-policy-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.sis-policy-info.policy-name"
            )
            sis_policy_name
        }

        public SisPolicyKey(final String clusterUUID, final String vServerUUID, final String sisPolicyName) {
            super("sis_policy", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.sis_policy_name, sisPolicyName);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "vserver_uuid.sis_policy_name")
        public String getOcieUUID() {
            final String clusterUUID = getID(Field.cluster_uuid);
            final String vServerUUID = getID(Field.vserver_uuid);
            final String fqVServerUUID = VServer.VServerKey.getOCIEVServerUUID(clusterUUID, vServerUUID);
            final String sisPolicyName = getID(Field.sis_policy_name);
            return fqVServerUUID + '.' + sisPolicyName;
        }
    }

    private final SisPolicyKey sisPolicyKey;

    @OCIEZapiSource(
        sourceZapi = "sis-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-policy-info.enabled"
    )
    @OCIEValue(
        ocieColumn = "isEnabled"
    )
    private Boolean enabled;

    @OCIEZapiSource(
        sourceZapi = "sis-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-policy-info.comment"
    )
    @OCIEValue(
        ocieColumn = "comment"
    )
    private String comment;

    @OCIEZapiSource(
        sourceZapi = "sis-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-policy-info.duration"
    )
    @OCIEValue(
        ocieColumn = "duration"
    )
    private String duration;

    @OCIEZapiSource(
        sourceZapi = "sis-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-policy-info.schedule"
    )
    @OCIEValue(
        ocieColumn = "jobScheduleId"
    )
    private JobSchedule.JobScheduleKey jobScheduleKey;

    @OCIEZapiSource(
        sourceZapi = "sis-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-policy-info.policy-name"
    )
    @OCIEValue(
        ocieColumn = "qosPolicy"
    )
    private QosPolicy qosPolicy;

    @OCIEZapiSource(
        sourceZapi = "sis-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-policy-info.policy-name"
    )
    @OCIEValue(
        ocieColumn = "qosPolicyRaw"
    )
    private String qosPolicyRaw;

    SisPolicy(final VServer parent, final String vServerUUID, final String sisPolicyName) {
        super(parent, "SisPolicy");
        this.sisPolicyKey = new SisPolicyKey(parent.getKey().getID(VServer.VServerKey.Field.cluster_uuid), vServerUUID, sisPolicyName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public SisPolicyKey getKey() {
        return sisPolicyKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isEnabled() {
        return enabled;
    }

    public void setEnabled(final Boolean enabled) {
        this.enabled = enabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getComment() {
        return comment;
    }

    public void setComment(final String comment) {
        this.comment = comment;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getDuration() {
        return duration;
    }

    public void setDuration(final String duration) {
        this.duration = duration;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public JobSchedule.JobScheduleKey getJobScheduleKey() {
        return jobScheduleKey;
    }

    public void setJobScheduleKey(final JobSchedule.JobScheduleKey jobScheduleKey) {
        this.jobScheduleKey = jobScheduleKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public QosPolicy getQosPolicy() {
        return qosPolicy;
    }

    public void setQosPolicy(final QosPolicy qosPolicy) {
        this.qosPolicy = qosPolicy;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getQosPolicyRaw() {
        return qosPolicyRaw;
    }

    public void setQosPolicyRaw(final String qosPolicyRaw) {
        this.qosPolicyRaw = qosPolicyRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum QosPolicy {
        NOT_MAPPED,
        BACKGROUND,
        BEST_EFFORT,
    }

    


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", sisPolicyKey);
        addRequiredAttributeAndCRC(writer, "UUID", sisPolicyKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", sisPolicyKey.getID(SisPolicyKey.Field.sis_policy_name));
        addRequiredAttributeAndCRC(writer, "IsEnabled", enabled);
        addRequiredAttributeAndCRC(writer, "Comment", comment);
        addRequiredAttributeAndCRC(writer, "Duration", duration);
        addRequiredAttributeAndCRC(writer, "QosPolicy", qosPolicy);
        addRequiredAttributeAndCRC(writer, "QosPolicyRaw", qosPolicyRaw);
        addRequiredAttributeAndCRC(writer, "JobScheduleUUID", KeyBase.getOcieUUIDOrNull(jobScheduleKey));
    }
}
