package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("snap_mirror_self_relationship")
@OCIEResourceType("snap_mirror_self_relationship")
public class SnapMirrorSelfRelationship extends BaseCRCXMLNode<Cluster> {

    public final static class RelationshipKey extends KeyBase<RelationshipKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "snapmirror-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.destination-volume"
            )
            volume_snap_mirror_uuid,

            @OCIEZapiSource(
                    sourceZapi = "snapmirror-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.destination-vserver"
            )
            vServer_snap_mirror_uuid,
        }

        public RelationshipKey(final String volumeSnapMirrorUUID, final String vServerSnapMirrorUUID) {
            super("volume_self_relationship", new EnumMap<Field, String>(Field.class) {{
                put(Field.volume_snap_mirror_uuid, volumeSnapMirrorUUID);
                put(Field.vServer_snap_mirror_uuid, vServerSnapMirrorUUID);
            }});
        }

        public String getOcieUUID() {
            return getFQID();
        }
    }

    private final RelationshipKey relationshipKey;

    SnapMirrorSelfRelationship(final Cluster parent, final String volumeSnapMirrorUUID, final String vServerSnapMirrorUUID) {
        super(parent, "SnapMirrorSelfRelationship");
        this.relationshipKey = new RelationshipKey(volumeSnapMirrorUUID, vServerSnapMirrorUUID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public RelationshipKey getKey() {
        return this.relationshipKey;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "VolumeSnapMirrorUUID", relationshipKey.getID(RelationshipKey.Field.volume_snap_mirror_uuid));
        addRequiredAttributeAndCRC(writer, "VServerSnapMirrorUUID", relationshipKey.getID(RelationshipKey.Field.vServer_snap_mirror_uuid));
    }
}
