package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@SuppressWarnings({"UnusedDeclaration"})
@OCIETable("storage_adapter")
@OCIEResourceType("storage_adapter")
public class StorageAdapter extends BaseCRCXMLNode<Node> {

    public final static class AdapterKey extends KeyBase<AdapterKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "system-node-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.node-details-info.node-uuid"
            )
            node_uuid,

            @OCIEZapiSource(
                sourceZapi = "storage-adapter-get-adapter-list",
                zapiResponsePath = "netapp.results.adapter-list.adapter-name-elem.adapter-name"
            )
            adapter_name
        }

        private boolean isKnown = true;

        public AdapterKey(final String clusterUUID, final String nodeUUID, final String adapterName) {
            super("storage_adapter", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.node_uuid, nodeUUID);
                put(Field.adapter_name, adapterName);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "node_uuid.adapter_name")
        public String getOcieUUID() {
            return getID(Field.node_uuid) + '.' + getID(Field.adapter_name);
        }

        public boolean isKnown() {
            return isKnown;
        }

        public void setKnown(final boolean known) {
            isKnown = known;
        }
    }


    private final AdapterKey adapterKey;

    @OCIEZapiSource(
        sourceZapi = "storage-adapter-get-adapter-info",
        zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-fc.adapter-fc-info.fc-node-name"
    )
    @OCIEValue(
        ocieColumn = "fcNodeName"
    )
    private String nodeWWN;

    @OCIEZapiSource(
        sourceZapi = "storage-adapter-get-adapter-info",
        zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-fc.adapter-fc-info.is-in-use"
    )
    @OCIEValue(
        ocieColumn = "isInUse"
    )
    private Boolean isInUse;

    @OCIEZapiSource(
        sourceZapi = "storage-adapter-get-adapter-info",
        zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-fc.adapter-fc-info.model"
    )
    @OCIEValue(
        ocieColumn = "model"
    )
    private String model;

    @OCIEZapiSource(
        sourceZapi = "fc-config-list-info",
        zapiResponsePath = "netapp.results.attributes-list.fc-config.adapter-status"
    )
    @OCIEValue(
        ocieColumn = "status"
    )
    private Status status;

    @OCIEZapiSource(
        sourceZapi = "storage-adapter-get-adapter-info",
        zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-type"
    )
    @OCIEValue(
        ocieColumn = "type"
    )
    private Type type;

    @OCIEZapiSource(
        sourceZapi = "storage-adapter-get-adapter-info",
        zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-fc.adapter-fc-info.fc-port-name"
    )
    @OCIEValue(
        ocieColumn = "fcPortName"
    )
    private String portWWN;

    @OCIEZapiSource(
            sourceZapi = "storage-adapter-get-adapter-info",
            zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.adapter-state"
    )
    @OCIEValue(
            ocieColumn = "state"
    )
    private State state;

    @OCIEZapiSource(
            sourceZapi = "storage-adapter-get-adapter-info",
            zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.adapter-state"
    )
    @OCIEValue(
            ocieColumn = "stateRaw"
    )
    private String stateRaw;

    @OCIEZapiSource(
            sourceZapi = "storage-adapter-get-adapter-info",
            zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.cable-end-identifier"
    )
    @OCIEValue(
            ocieColumn = "cableEndIdentifier"
    )
    private String cableEndIdentifier;

    @OCIEZapiSource(
            sourceZapi = "storage-adapter-get-adapter-info",
            zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.cable-length"
    )
    @OCIEValue(
            ocieColumn = "cableLength"
    )
    private String cableLength;

    @OCIEZapiSource(
            sourceZapi = "storage-adapter-get-adapter-info",
            zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.cable-manufacturer"
    )
    @OCIEValue(
            ocieColumn = "cableManufacturer"
    )
    private String cableManufacturer;

    @OCIEZapiSource(
            sourceZapi = "storage-adapter-get-adapter-info",
            zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.cable-part-number"
    )
    @OCIEValue(
            ocieColumn = "cablePartNumber"
    )
    private String cablePartNumber;

    @OCIEZapiSource(
            sourceZapi = "storage-adapter-get-adapter-info",
            zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.cable-serial-number"
    )
    @OCIEValue(
            ocieColumn = "cablePartNumber"
    )
    private String cableSerialNumber;

    @OCIEZapiSource(
            sourceZapi = "storage-adapter-get-adapter-info",
            zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.attached-serial-number"
    )
    @OCIEValue(
            ocieColumn = "attachedSerialNumber"
    )
    private String attachedSerialNumber;

    @OCIEZapiSource(
            sourceZapi = "storage-adapter-get-adapter-info",
            zapiResponsePath = "netapp.results.adapter-details.adapter-detail-info.adapter-sas.adapter-sas-info.sas-qsfp-cable.sas-qsfp-cable-info.cable-technology"
    )
    @OCIEValue(
            ocieColumn = "cableTechnology"
    )
    private String cableTechnology;

    StorageAdapter(final Node parent, final String adapterName) {
        super(parent, "StorageAdapter");
        final Node.NodeKey parentKey = parent.getKey();
        final String clusterUUID = parentKey.getID(Node.NodeKey.Field.cluster_uuid);
        final String nodeUUID = parentKey.getID(Node.NodeKey.Field.uuid);
        this.adapterKey = new AdapterKey(clusterUUID, nodeUUID, adapterName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public AdapterKey getKey() {
        return adapterKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public AdapterKey getAdapterKey() {
        return adapterKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getNodeWWN() {
        return nodeWWN;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getInUse() {
        return isInUse;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getModel() {
        return model;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Status getStatus() {
        return status;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Type getType() {
        return type;
    }

    public void setNodeWWN(final String nodeWWN) {
        this.nodeWWN = nodeWWN;
    }

    public void setInUse(final Boolean inUse) {
        isInUse = inUse;
    }

    public void setModel(final String model) {
        this.model = model;
    }

    public void setStatus(final Status status) {
        this.status = status;
    }

    public void setType(final Type type) {
        this.type = type;
    }

    public String getPortWWN() {
        return portWWN;
    }

    public void setPortWWN(final String portWWN) {
        this.portWWN = portWWN;
    }

    public State getState() {
        return state;
    }

    public void setState(final State state) {
        this.state = state;
    }

    public String getStateRaw() {
        return stateRaw;
    }

    public void setStateRaw(final String stateRaw) {
        this.stateRaw = stateRaw;
    }

    public String getCableEndIdentifier() {
        return cableEndIdentifier;
    }

    public void setCableEndIdentifier(final String cableEndIdentifier) {
        this.cableEndIdentifier = cableEndIdentifier;
    }

    public String getCableLength() {
        return cableLength;
    }

    public void setCableLength(final String cableLength) {
        this.cableLength = cableLength;
    }

    public String getCableManufacturer() {
        return cableManufacturer;
    }

    public void setCableManufacturer(final String cableManufacturer) {
        this.cableManufacturer = cableManufacturer;
    }

    public String getCablePartNumber() {
        return cablePartNumber;
    }

    public void setCablePartNumber(final String cablePartNumber) {
        this.cablePartNumber = cablePartNumber;
    }

    public String getCableSerialNumber() {
        return cableSerialNumber;
    }

    public void setCableSerialNumber(final String cableSerialNumber) {
        this.cableSerialNumber = cableSerialNumber;
    }

    public String getAttachedSerialNumber() {
        return attachedSerialNumber;
    }

    public void setAttachedSerialNumber(final String attachedSerialNumber) {
        this.attachedSerialNumber = attachedSerialNumber;
    }

    public String getCableTechnology() {
        return cableTechnology;
    }

    public void setCableTechnology(final String cableTechnology) {
        this.cableTechnology = cableTechnology;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Status {
        NOT_MAPPED,
        OFFLINE,
        ONLINE,
    }

    public static enum State {
        NOT_MAPPED,
        UP,
        DOWN,
        OFFLINE_SOFT,
        OFFLINE_HARD,
        OFFLINE_LOOPBACK
    }


    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Type {
        NOT_MAPPED,
        ADT_IF_ATA,
        ADT_IF_PARALLEL_SCSI,
        ADT_IF_SAS,
        ADT_IF_FC,
    }

    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", adapterKey);
        addRequiredAttributeAndCRC(writer, "UUID", adapterKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", adapterKey.getID(AdapterKey.Field.adapter_name));
        addRequiredAttributeAndCRC(writer, "FcNodeName", nodeWWN);
        addRequiredAttributeAndCRC(writer, "FcPortName", portWWN);
        addRequiredAttributeAndCRC(writer, "IsInUse", isInUse);
        addRequiredAttributeAndCRC(writer, "Model", model);
        addRequiredAttributeAndCRC(writer, "Status", status);
        addRequiredAttributeAndCRC(writer, "AdapterType", type);
        addAttributeNoCRC(writer, "State", state);
        addAttributeNoCRC(writer, "StateRaw", stateRaw);
        addAttributeNoCRC(writer, "CableEndIdentifier", cableEndIdentifier);
        addAttributeNoCRC(writer, "CableLength", cableLength);
        addAttributeNoCRC(writer, "CableManufacturer", cableManufacturer);
        addAttributeNoCRC(writer, "CablePartNumber", cablePartNumber);
        addAttributeNoCRC(writer, "CableSerialNumber", cableSerialNumber);
        addAttributeNoCRC(writer, "AttachedSerialNumber", attachedSerialNumber);
        addAttributeNoCRC(writer, "CableTechnology", cableTechnology);
    }
}
