package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.*;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("storage_pool")
@OCIEResourceType("storage_pool")
public class StoragePool extends BaseCRCXMLNode<Cluster> {

    public final static class StoragePoolKey extends KeyBase<StoragePoolKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "cluster-identity-get",
                    zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                    sourceZapi = "storage-pool-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.storage-pool-info.shared-storage-pool"
            )
            storage_pool_name
        }

        public StoragePoolKey(final String clusterUUID, final String storagePoolName) {
            super("storage_pool",
                    new EnumMap<Field, String>(Field.class) {{
                        put(Field.cluster_uuid, clusterUUID);
                        put(Field.storage_pool_name, storagePoolName);
                    }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.storage_pool_name")
        public String getOcieUUID() {
            final String clusterUUID = getID(Field.cluster_uuid);
            final String storagePoolName = getID(Field.storage_pool_name);
            return clusterUUID + '.' + storagePoolName;
        }
    }

    @OCIEZapiSource(
            sourceZapi = "storage-pool-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-info.shared-storage-pool"
    )
    @OCIEValue(
            ocieColumn = "name"
    )
    private String name;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-info.allocation-unit-size"
    )
    @OCIEValue(
            ocieColumn = "allocationUnitSize"
    )
    private Long allocationUnitSize;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-info.disk-count"
    )
    @OCIEValue(
            ocieColumn = "diskCount"
    )
    private Long diskCount;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-info.is-healthy"
    )
    @OCIEValue(
            ocieColumn = "isHealthy"
    )
    private Boolean isHealthy;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-info.pool-usable-size"
    )
    @OCIEValue(
            ocieColumn = "poolUsableSize"
    )
    private Long poolUsableSize;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-info.pool-total-size"
    )
    @OCIEValue(
            ocieColumn = "poolTotalSize"
    )
    private Long poolTotalSize;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-info.storage-type"
    )
    @OCIEValue(
            ocieColumn = "storageType",
            notes = "derived enum"
    )
    private StorageType storageType;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-info.storage-type"
    )
    @OCIEValue(
            ocieColumn = "storageTypeRaw"
    )
    private String storageTypeRaw;

    private final StoragePoolKey key;

    private final StoragePoolAvailableSizes storagePoolAvailableSizes;
    private final StoragePoolAggregates storagePoolAggregates;

    protected StoragePool(final Cluster cluster, final String storagePoolName) {
        super(cluster, StoragePool.class.getSimpleName());
        this.key = new StoragePoolKey(cluster.getClusterUUID(), storagePoolName);
        this.name = storagePoolName;
        this.storagePoolAvailableSizes = new StoragePoolAvailableSizes(this);
        this.storagePoolAggregates = new StoragePoolAggregates(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public StoragePoolKey getKey() {
        return this.key;
    }

    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getAllocationUnitSize() {
        return allocationUnitSize;
    }

    public void setAllocationUnitSize(final Long allocationUnitSize) {
        this.allocationUnitSize = allocationUnitSize;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getDiskCount() {
        return diskCount;
    }

    public void setDiskCount(final Long diskCount) {
        this.diskCount = diskCount;
    }

    public Boolean getHealthy() {
        return isHealthy;
    }

    public void setHealthy(Boolean healthy) {
        isHealthy = healthy;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getPoolUsableSize() {
        return poolUsableSize;
    }

    public void setPoolUsableSize(final Long poolUsableSize) {
        this.poolUsableSize = poolUsableSize;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getPoolTotalSize() {
        return poolTotalSize;
    }

    public void setPoolTotalSize(final Long poolTotalSize) {
        this.poolTotalSize = poolTotalSize;
    }

    @SuppressWarnings({"unused declaration"})
    public StorageType getStorageType() {
        return storageType;
    }

    public void setStorageType(final StorageType storageType) {
        this.storageType = storageType;
    }

    @SuppressWarnings({"unused declaration"})
    public String getStorageTypeRaw() {
        return storageTypeRaw;
    }

    public void setStorageTypeRaw(final String storageTypeRaw) {
        this.storageTypeRaw = storageTypeRaw;
    }

    public StoragePoolAvailableSizes getStoragePoolAvailableSizes() {
        return storagePoolAvailableSizes;
    }

    public StoragePoolAvailableSize createStoragePoolAvailableSize(final Node.NodeKey nodeKey) {
        return new StoragePoolAvailableSize(this, nodeKey);
    }

    public StoragePoolAggregates getStoragePoolAggregates() {
        return storagePoolAggregates;
    }

    public StoragePoolAggregate createStoragePoolAggregate(final Aggregate.AggregateKey aggregateKey) {
        return new StoragePoolAggregate(this, aggregateKey);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum StorageType {
        NOT_MAPPED,
        ATA,
        BSAS,
        FCAL,
        FSAS,
        LUN,
        MSATA,
        SAS,
        SATA,
        SSD,
        UNKNOWN,
    }

    protected boolean hasChildren() {
        return !storagePoolAvailableSizes.isEmpty() || !storagePoolAggregates.isEmpty();
    }

    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        if (!storagePoolAggregates.isEmpty()) {
            addAllChildrenUnderNodeAndCRC(cache, writer, StoragePoolAggregates.class.getSimpleName(), storagePoolAggregates.values(), tabs);
        }
        if (!storagePoolAvailableSizes.isEmpty()) {
            addAllChildrenUnderNodeAndCRC(cache, writer, StoragePoolAvailableSizes.class.getSimpleName(), storagePoolAvailableSizes.values(), tabs);
        }
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(cache, writer);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "Name", name);
        addRequiredAttributeAndCRC(writer, "AllocationUnitSize", allocationUnitSize);
        addRequiredAttributeAndCRC(writer, "DiskCount", diskCount);
        addRequiredAttributeAndCRC(writer, "IsHealthy", isHealthy);
        addRequiredAttributeAndCRC(writer, "PoolUsableSize", poolUsableSize);
        addRequiredAttributeAndCRC(writer, "PoolTotalSize", poolTotalSize);
        addRequiredAttributeAndCRC(writer, "StorageType", storageType);
        addRequiredAttributeAndCRC(writer, "StorageTypeRaw", storageTypeRaw);
    }
}
