package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("storage_pool_aggregate_relationship")
@OCIEResourceType("storage_pool_aggregate_relationship")
public class StoragePoolAggregate extends BaseCRCXMLNode<StoragePool> {

    public final static class StoragePoolAggregateKey extends KeyBase<StoragePoolAggregateKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "cluster-identity-get",
                    zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                    sourceZapi = "storage-pool-aggregate-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.storage-pool-aggregate-info.shared-storage-pool"
            )
            storage_pool_name,

            @OCIEZapiSource(
                    sourceZapi = "storage-pool-aggregate-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.storage-pool-aggregate-info.aggregate"
            )
            aggregate_uuid
        }

        public StoragePoolAggregateKey(final String clusterUUID, final String storagePoolName, final String aggregateUUID) {
            super("storage_pool_aggregate_relationship",
                    new EnumMap<Field, String>(Field.class) {{
                        put(Field.cluster_uuid, clusterUUID);
                        put(Field.storage_pool_name, storagePoolName);
                        put(Field.aggregate_uuid, aggregateUUID);
                    }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.storage_pool_name.aggregateUUID")
        public String getOcieUUID() {
            final String clusterUUID = getID(Field.cluster_uuid);
            final String storagePoolName = getID(Field.storage_pool_name);
            final String aggregateUUID = getID(Field.aggregate_uuid);
            return clusterUUID + '.' + storagePoolName + '.' + aggregateUUID;
        }
    }

    private final StoragePoolAggregateKey key;

    @SuppressWarnings({"unchecked"})
    public StoragePoolAggregateKey getKey() {
        return key;
    }

    @OCIEZapiSource(
            sourceZapi = "storage-pool-aggregate-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-aggregate-info.aggregate"
    )
    @OCIEValue(
            ocieColumn = "aggregateId"
    )
    private final Aggregate.AggregateKey aggregateKey;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-aggregate-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-aggregate-info.used-capacity"
    )
    @OCIEValue(
            ocieColumn = "usedCapacity"
    )
    private Long usedCapacity;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-aggregate-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-aggregate-info.allocated-unit-count"
    )
    @OCIEValue(
            ocieColumn = "allocatedUnitCount"
    )
    private Long allocatedUnitCount;

    protected StoragePoolAggregate(final StoragePool storagePool, final Aggregate.AggregateKey aggregateKey) {
        super(storagePool, StoragePoolAggregate.class.getSimpleName());
        this.key = new StoragePoolAggregateKey(storagePool.getParent().getClusterUUID(), storagePool.getName(), aggregateKey.getOcieUUID());
        this.aggregateKey = aggregateKey;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getUsedCapacity() {
        return usedCapacity;
    }

    public void setUsedCapacity(final Long usedCapacity) {
        this.usedCapacity = usedCapacity;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getAllocatedUnitCount() {
        return allocatedUnitCount;
    }

    public void setAllocatedUnitCount(final Long allocatedUnitCount) {
        this.allocatedUnitCount = allocatedUnitCount;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(cache, writer);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "AggregateUUID", aggregateKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "UsedCapacity", usedCapacity);
        addRequiredAttributeAndCRC(writer, "AllocatedUnitCount", allocatedUnitCount);
    }
}