package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("storage_pool_available")
@OCIEResourceType("storage_pool_available")
public class StoragePoolAvailableSize extends BaseCRCXMLNode<StoragePool> {

    public final static class StoragePoolAvailableSizeKey extends KeyBase<StoragePoolAvailableSizeKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "cluster-identity-get",
                    zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                    sourceZapi = "storage-pool-available-capacity-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.storage-pool-available-capacity-info.node"
            )
            node_uuid,

            @OCIEZapiSource(
                    sourceZapi = "storage-pool-available-capacity-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.storage-pool-available-capacity-info.shared-storage-pool"
            )
            storage_pool_name
        }

        public StoragePoolAvailableSizeKey(final String clusterUUID, final String nodeUUID, final String storagePoolName) {
            super("storage_pool_available",
                    new EnumMap<Field, String>(Field.class) {{
                        put(Field.cluster_uuid, clusterUUID);
                        put(Field.node_uuid, nodeUUID);
                        put(Field.storage_pool_name, storagePoolName);
                    }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.node_name.storage_pool_name")
        public String getOcieUUID() {
            final String clusterUUID = getID(Field.cluster_uuid);
            final String nodeUUID = getID(Field.node_uuid);
            final String storagePoolName = getID(Field.storage_pool_name);
            return clusterUUID + '.' + nodeUUID + '.' + storagePoolName;
        }
    }

    private final StoragePoolAvailableSizeKey key;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-available-capacity-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-available-capacity-info.node"
    )
    @OCIEValue(
            ocieColumn = "nodeId"
    )
    private final Node.NodeKey spareNodeKey;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-available-capacity-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-available-capacity-info.allocation-unit-count"
    )
    @OCIEValue(
            ocieColumn = "allocationUnitCount"
    )
    private Long allocationUnitCount;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-available-capacity-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-available-capacity-info.allocation-unit-size"
    )
    @OCIEValue(
            ocieColumn = "allocationUnitSize"
    )
    private Long allocationUnitSize;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-available-capacity-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-available-capacity-info.available-size"
    )
    @OCIEValue(
            ocieColumn = "availableSize"
    )
    private Long availableSize;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-available-capacity-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-available-capacity-info.storage-type"
    )
    @OCIEValue(
            ocieColumn = "storageTypeRaw"
    )
    private String storageTypeRaw;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-available-capacity-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-available-capacity-info.storage-type"
    )
    @OCIEValue(
            ocieColumn = "storageType",
            notes = "derived enum"
    )
    private StoragePool.StorageType storageType;

    @OCIEZapiSource(
            sourceZapi = "storage-pool-available-capacity-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-pool-available-capacity-info.syncmirror-pool"
    )
    @OCIEValue(
            ocieColumn = "syncmirrorPool"
    )
    private String syncMirrorPool;

    protected StoragePoolAvailableSize(final StoragePool storagePool, final Node.NodeKey spareNodeKey) {
        super(storagePool, StoragePoolAvailableSize.class.getSimpleName());
        this.key = new StoragePoolAvailableSizeKey(storagePool.getParent().getClusterUUID(), spareNodeKey.getOcieUUID(), storagePool.getName());
        this.spareNodeKey = spareNodeKey;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public StoragePoolAvailableSizeKey getKey() {
        return this.key;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getAllocationUnitCount() {
        return allocationUnitCount;
    }

    public void setAllocationUnitCount(final Long allocationUnitCount) {
        this.allocationUnitCount = allocationUnitCount;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getAllocationUnitSize() {
        return allocationUnitSize;
    }

    public void setAllocationUnitSize(final Long allocationUnitSize) {
        this.allocationUnitSize = allocationUnitSize;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getAvailableSize() {
        return availableSize;
    }

    public void setAvailableSize(final Long availableSize) {
        this.availableSize = availableSize;
    }

    @SuppressWarnings({"unused declaration"})
    public StoragePool.StorageType getStorageType() {
        return storageType;
    }

    public void setStorageType(final StoragePool.StorageType storageType) {
        this.storageType = storageType;
    }

    @SuppressWarnings({"unused declaration"})
    public Node.NodeKey getSpareNodeKey() {
        return spareNodeKey;
    }

    @SuppressWarnings({"unused declaration"})
    public String getSyncMirrorPool() {
        return syncMirrorPool;
    }

    public void setSyncMirrorPool(final String syncMirrorPool) {
        this.syncMirrorPool = syncMirrorPool;
    }

    @SuppressWarnings({"unused declaration"})
    public String getStorageTypeRaw() {
        return storageTypeRaw;
    }

    public void setStorageTypeRaw(final String storageTypeRaw) {
        this.storageTypeRaw = storageTypeRaw;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(cache, writer);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "NodeUUID", spareNodeKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "AllocationUnitCount", allocationUnitCount);
        addRequiredAttributeAndCRC(writer, "AllocationUnitSize", allocationUnitSize);
        addRequiredAttributeAndCRC(writer, "AvailableSize", availableSize);
        addRequiredAttributeAndCRC(writer, "StorageType", storageType);
        addRequiredAttributeAndCRC(writer, "StorageTypeRaw", storageTypeRaw);
        addRequiredAttributeAndCRC(writer, "SyncmirrorPool", syncMirrorPool);
    }

}
