package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("storage_shelf")
@OCIEResourceType("storage_shelf")
public class StorageShelf extends BaseCRCXMLNode<Cluster> {

    public final static class StorageShelfKey extends KeyBase<StorageShelfKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "storage-shelf-list-info",
                zapiArgs = "node-name",
                zapiResponsePath = "netapp.results.shelf-list.shelf-info.shelf-uid"
            )
            shelf_uuid
        }

        public StorageShelfKey(final String clusterUUID, final String shelfUID) {
            super("storage_shelf", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.shelf_uuid, shelfUID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "shelf_uuid")
        public String getOcieUUID() {
            return getID(Field.shelf_uuid);
        }
    }

    private final StorageShelfKey key;

    private String nodeUUID;

    private String partnerNodeUUID;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-list.shelf-info.channel-name"
    )
    @OCIEValue(
        ocieColumn = "channelName"
    )
    private String channelName;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.results.shelf-list.shelf-info.channel-name"
    )
    @OCIEValue(
            ocieColumn = "channelName"
    )
    private String partnerChannelName;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-list.shelf-info.shelf-id"
    )
    @OCIEValue(
        ocieColumn = "shelfIdentifier"
    )
    private String shelfID;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-list.shelf-info.shelf-type"
    )
    @OCIEValue(
        ocieColumn = "shelfType"
    )
    private String shelfType;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-list.shelf-info.firmware-rev-A"
    )
    @OCIEValue(
        ocieColumn = "firmwareRevisionA"
    )
    private String firmwareRevisionA;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-list.shelf-info.firmware-rev-b"
    )
    @OCIEValue(
        ocieColumn = "firmwareRevisionB"
    )
    private String firmwareRevisionB;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-list.shelf-info.shelf-state"
    )
    @OCIEValue(
        ocieColumn = "state"
    )
    private State shelfState;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-list.shelf-info.shelf-state"
    )
    @OCIEValue(
        ocieColumn = "stateRaw"
    )
    private String shelfStateRaw;
    
    @OCIEZapiSource(
            sourceZapi = "storage-shelf-info-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-shelf-info.stack-id"
        )
        @OCIEValue(
            ocieColumn = "stackId"
        )
    private String stackID;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-info-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-shelf-info.is-local-attach"
    )
    @OCIEValue(
            ocieColumn = "isLocalAttach"
    )
    private Boolean isLocalAttach;
    /*
    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.ses-generic-info.ses-contact-state"
    )
    private String sesContactState;


    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.ses-generic-info.ses-logical-id"
    )
    private String sesLogicalID;
    */

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.ses-generic-info.ses-product-id"
    )
    @OCIEValue(
        ocieColumn = "sesProductId"
    )
    private String sesProductID;

    /*
    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.ses-generic-info.ses-vendor-id"
    )
    private String sesProductVendor;


    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.ses-generic-info.ses-product-revision"
    )
    private String sesProductRevision;
    */

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.shelf-module"
    )
    @OCIEValue(
        ocieColumn = "module"
    )
    private String module;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-info.temp-sensor-list.temp-sensor-info"
    )
    @OCIEValue(
        ocieColumn = "temperatureSensorCount",
        notes = "derived count"
    )
    private Integer temperatureSensorCount;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-info.temp-sensor-list.temp-sensor-info.temp-sensor-is-error"
    )
    @OCIEValue(
        ocieColumn = "temperatureSensorFailedCount",
        notes = "derived count"
    )
    private Integer temperatureSensorFailedCount;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-info.cooling-element-list.cooling-element-info"
    )
    @OCIEValue(
        ocieColumn = "coolingElementCount",
        notes = "derived count"
    )
    private Integer coolingElementCount;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-info.cooling-element-list.cooling-element-info.cooling-element-is-error"
    )
    @OCIEValue(
        ocieColumn = "coolingElementFailedCount",
        notes = "derived count"
    )
    private Integer coolingElementFailedCount;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-info.power-supply-list.power-supply-info"
    )
    @OCIEValue(
        ocieColumn = "powerSupplyCount",
        notes = "derived count"
    )
    private Integer powerSupplyCount;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-info.power-supply-list.power-supply-info.power-supply-is-error"
    )
    @OCIEValue(
        ocieColumn = "powerSupplyFailedCount",
        notes = "derived count"
    )
    private Integer powerSupplyFailedCount;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-info.current-sensor-list.current-sensor-info"
    )
    @OCIEValue(
        ocieColumn = "currentSensorCount",
        notes = "derived count"
    )
    private Integer currentSensorCount;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-info.current-sensor-list.current-sensor-info.is-sensor-error"
    )
    @OCIEValue(
        ocieColumn = "currentSensorFailedCount",
        notes = "derived count"
    )
    private Integer currentSensorFailedCount;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-info.voltage-sensor-list.voltage-sensor-info"
    )
    @OCIEValue(
        ocieColumn = "voltageSensorCount",
        notes = "derived count"
    )
    private Integer voltageSensorCount;

    @OCIEZapiSource(
        sourceZapi = "storage-shelf-environment-list-info",
        zapiArgs = "node-name",
        zapiResponsePath = "netapp.results.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-info.voltage-sensor-list.voltage-sensor-info.is-sensor-error"
    )
    @OCIEValue(
        ocieColumn = "voltageSensorFailedCount",
        notes = "derived count"
    )
    private Integer voltageSensorFailedCount;

    private final StorageShelfCables storageShelfCables;


    StorageShelf(final Cluster parent, final String shelfUID) {
        super(parent, StorageShelf.class.getSimpleName());
        final String clusterUUID = parent.getKey().getOcieUUID();
        this.key = new StorageShelfKey(clusterUUID, shelfUID);
        this.storageShelfCables = new StorageShelfCables(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public StorageShelfKey getKey() {
        return key;
    }

    public String getNodeUUID() {
        return nodeUUID;
    }

    public void setNodeUUID(String nodeUUID) {
        this.nodeUUID = nodeUUID;
    }

    public String getPartnerNodeUUID() {
        return partnerNodeUUID;
    }

    public void setPartnerNodeUUID(String partnerNodeUUID) {
        this.partnerNodeUUID = partnerNodeUUID;
    }

    @SuppressWarnings({"unused declaration"})
    public String getShelfType() {
        return shelfType;
    }

    public void setShelfType(final String shelfType) {
        this.shelfType = shelfType;
    }

    @SuppressWarnings({"unused declaration"})
    public String getChannelName() {
        return channelName;
    }

    public void setChannelName(final String channelName) {
        this.channelName = channelName;
    }

    @SuppressWarnings({"unused declaration"})
    public String getPartnerChannelName() {
        return partnerChannelName;
    }

    public void setPartnerChannelName(String partnerChannelName) {
        this.partnerChannelName = partnerChannelName;
    }

    @SuppressWarnings({"unused declaration"})
    public String getShelfID() {
        return shelfID;
    }

    public void setShelfID(final String shelfID) {
        this.shelfID = shelfID;
    }
    
    @SuppressWarnings({"unused declaration"})
    public String getStackID() {
        return stackID;
    }

    public void setStackID(final String stackID) {
        this.stackID = stackID;
    }

    public Boolean getIsLocalAttach() {
        return isLocalAttach;
    }

    public void setIsLocalAttach(final Boolean isLocalAttach) {
        this.isLocalAttach = isLocalAttach;
    }
    
    @SuppressWarnings({"unused declaration"})
    public String getFirmwareRevisionA() {
        return firmwareRevisionA;
    }

    public void setFirmwareRevisionA(final String firmwareRevisionA) {
        this.firmwareRevisionA = firmwareRevisionA;
    }

    @SuppressWarnings({"unused declaration"})
    public String getFirmwareRevisionB() {
        return firmwareRevisionB;
    }

    public void setFirmwareRevisionB(final String firmwareRevisionB) {
        this.firmwareRevisionB = firmwareRevisionB;
    }

    @SuppressWarnings({"unused declaration"})
    public State getShelfState() {
        return shelfState;
    }

    public void setShelfState(final State shelfState) {
        this.shelfState = shelfState;
    }

    @SuppressWarnings({"unused declaration"})
    public String getShelfStateRaw() {
        return shelfStateRaw;
    }

    public void setShelfStateRaw(final String shelfStateRaw) {
        this.shelfStateRaw = shelfStateRaw;
    }

    @SuppressWarnings({"unused declaration"})
    public String getSesProductID() {
        return sesProductID;
    }

    public void setSesProductID(final String sesProductID) {
        this.sesProductID = sesProductID;
    }

    @SuppressWarnings({"unused declaration"})
    public String getModule() {
        return module;
    }

    public void setModule(final String module) {
        this.module = module;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getTemperatureSensorCount() {
        return temperatureSensorCount;
    }

    public void setTemperatureSensorCount(final Integer temperatureSensorCount) {
        this.temperatureSensorCount = temperatureSensorCount;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getTemperatureSensorFailedCount() {
        return temperatureSensorFailedCount;
    }

    public void setTemperatureSensorFailedCount(final Integer temperatureSensorFailedCount) {
        this.temperatureSensorFailedCount = temperatureSensorFailedCount;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getCoolingElementCount() {
        return coolingElementCount;
    }

    public void setCoolingElementCount(final Integer coolingElementCount) {
        this.coolingElementCount = coolingElementCount;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getCoolingElementFailedCount() {
        return coolingElementFailedCount;
    }

    public void setCoolingElementFailedCount(final Integer coolingElementFailedCount) {
        this.coolingElementFailedCount = coolingElementFailedCount;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getPowerSupplyCount() {
        return powerSupplyCount;
    }

    public void setPowerSupplyCount(final Integer powerSupplyCount) {
        this.powerSupplyCount = powerSupplyCount;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getPowerSupplyFailedCount() {
        return powerSupplyFailedCount;
    }

    public void setPowerSupplyFailedCount(final Integer powerSupplyFailedCount) {
        this.powerSupplyFailedCount = powerSupplyFailedCount;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getCurrentSensorCount() {
        return currentSensorCount;
    }

    public void setCurrentSensorCount(final Integer currentSensorCount) {
        this.currentSensorCount = currentSensorCount;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getCurrentSensorFailedCount() {
        return currentSensorFailedCount;
    }

    public void setCurrentSensorFailedCount(final Integer currentSensorFailedCount) {
        this.currentSensorFailedCount = currentSensorFailedCount;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getVoltageSensorCount() {
        return voltageSensorCount;
    }

    public void setVoltageSensorCount(final Integer voltageSensorCount) {
        this.voltageSensorCount = voltageSensorCount;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getVoltageSensorFailedCount() {
        return voltageSensorFailedCount;
    }

    public void setVoltageSensorFailedCount(final Integer voltageSensorFailedCount) {
        this.voltageSensorFailedCount = voltageSensorFailedCount;
    }

    public StorageShelfCable createAndAddStorageShelfCable(final String nodeUUID, final String shelfId, final String channelName, final String partNumber, final String serialNumber, final Integer connectorNumber) {
        final StorageShelfCable storageShelfCable = new StorageShelfCable(this, nodeUUID, shelfId, channelName, partNumber, serialNumber, connectorNumber);
        storageShelfCables.add(storageShelfCable);
        return storageShelfCable;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public StorageShelfCables getStorageShelfCables() {
        return storageShelfCables;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum State {
        NOT_MAPPED,
        NO_STATUS,
        INIT_REQUIRED,
        ONLINE,
        OFFLINE,
        MISSING,
        UNKNOWN,
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "ShelfUUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "NodeUUID", nodeUUID);
        addRequiredAttributeAndCRC(writer, "PartnerNodeUUID", partnerNodeUUID);
        addRequiredAttributeAndCRC(writer, "ShelfIdentifier", shelfID);
        addRequiredAttributeAndCRC(writer, "SESProductID", sesProductID);
        addRequiredAttributeAndCRC(writer, "FirmwareRevisionA", firmwareRevisionA);
        addRequiredAttributeAndCRC(writer, "FirmwareRevisionB", firmwareRevisionB);
        addRequiredAttributeAndCRC(writer, "Module", module);
        addRequiredAttributeAndCRC(writer, "ChannelName", channelName);
        addRequiredAttributeAndCRC(writer, "PartnerChannelName", partnerChannelName);
        addRequiredAttributeAndCRC(writer, "State", shelfState);
        addRequiredAttributeAndCRC(writer, "StateRaw", shelfStateRaw);
        addRequiredAttributeAndCRC(writer, "StackId" , stackID);
        addRequiredAttributeAndCRC(writer, "ShelfType", shelfType);
        addRequiredAttributeAndCRC(writer, "TemperatureSensorCount", temperatureSensorCount);
        addRequiredAttributeAndCRC(writer, "TemperatureSensorFailedCount", temperatureSensorFailedCount);
        addRequiredAttributeAndCRC(writer, "CoolingElementCount", coolingElementCount);
        addRequiredAttributeAndCRC(writer, "CoolingElementFailedCount", coolingElementFailedCount);
        addRequiredAttributeAndCRC(writer, "PowerSupplyCount", powerSupplyCount);
        addRequiredAttributeAndCRC(writer, "PowerSupplyFailedCount", powerSupplyFailedCount);
        addRequiredAttributeAndCRC(writer, "CurrentSensorCount", currentSensorCount);
        addRequiredAttributeAndCRC(writer, "CurrentSensorFailedCount", currentSensorFailedCount);
        addRequiredAttributeAndCRC(writer, "VoltageSensorCount", voltageSensorCount);
        addRequiredAttributeAndCRC(writer, "VoltageSensorFailedCount", voltageSensorFailedCount);
        addRequiredAttributeAndCRC(writer, "IsLocalAttach" , isLocalAttach);
    }

    @Override
    protected boolean hasChildren(){
        return !storageShelfCables.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        super.marshallChildrenToFileAndCRC(cache, writer, tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "StorageShelfCables", storageShelfCables.values(), tabs);
    }

}
