/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEUUID;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEValue;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

/**
 * @author Al Velella
 *         Date: 10/22/13
 *         Time: 3:44 PM
 */
public class StorageSwitch extends BaseCRCXMLNode<Cluster> {

    public final static class StorageSwitchKey extends KeyBase<StorageSwitchKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "cluster-identity-get",
                    zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                    sourceZapi = "storage-switch-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.storage-switch-info.wwn"
            )
            wwn
        }

        public StorageSwitchKey(final String clusterUUID, final String wwn) {
            super("storage_switch", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.wwn, wwn);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "wwn")
        public String getOcieUUID() {
            return getID(Field.wwn);
        }
    }

    private final StorageSwitchKey storageSwitchKey;

    @OCIEValue(
            ocieColumn = "name"
    )
    @OCIEZapiSource(
            sourceZapi = "storage-switch-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-switch-info.name"
    )
    private String name = null;

    @OCIEValue(
            ocieColumn = "firmwareVersion"
    )
    @OCIEZapiSource(
            sourceZapi = "storage-switch-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-switch-info.firmware-version"
    )
    private String firmwareVersion = null;

    @OCIEValue(
            ocieColumn = "ipAddress"
    )
    @OCIEZapiSource(
            sourceZapi = "storage-switch-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-switch-info.ip-address"
    )
    private String ipAddress = null;

    @OCIEValue(
            ocieColumn = "model"
    )
    @OCIEZapiSource(
            sourceZapi = "storage-switch-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-switch-info.model"
    )
    private String model = null;

    @OCIEValue(
            ocieColumn = "role"
    )
    @OCIEZapiSource(
            sourceZapi = "storage-switch-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-switch-info.role"
    )
    private Role role = null;

    @OCIEValue(
            ocieColumn = "roleRaw"
    )
    @OCIEZapiSource(
            sourceZapi = "storage-switch-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-switch-info.role"
    )
    private String roleRaw = null;

    @OCIEValue(
            ocieColumn = "status"
    )
    @OCIEZapiSource(
            sourceZapi = "storage-switch-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-switch-info.status"
    )
    private Status status = null;

    @OCIEValue(
            ocieColumn = "statusRaw"
    )
    @OCIEZapiSource(
            sourceZapi = "storage-switch-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-switch-info.status"
    )
    private String statusRaw = null;

    @OCIEValue(
            ocieColumn = "vendor"
    )
    @OCIEZapiSource(
            sourceZapi = "storage-switch-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-switch-info.vendor"
    )
    private String vendor = null;

    @OCIEValue(
            ocieColumn = "sensorCount",
            notes = "Count of temperature-sensor-info"
    )
    @OCIEZapiSource(
            sourceZapi = "storage-switch-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-switch-info.storage-switch-temperature-sensor-list.temperature-sensor-info"
    )
    private int sensorCount = 0;

    @OCIEValue(
            ocieColumn = "sensorCount",
            notes = "Count of storage-switch-power-supply-unit-info"
    )
    @OCIEZapiSource(
            sourceZapi = "storage-switch-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-switch-info.storage-switch-power-supply-unit-list.storage-switch-power-supply-unit-info"
    )
    private int powerSupplyCount = 0;

    @OCIEValue(
            ocieColumn = "sensorCount",
            notes = "Count of storage-switch-power-supply-unit-info"
    )
    @OCIEZapiSource(
            sourceZapi = "storage-switch-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-switch-info.storage-switch-power-supply-unit-list.storage-switch-power-supply-unit-info"
    )
    private int fanCount = 0;

    StorageSwitch(final Cluster parent, final String wwn) {
        super(parent, "StorageSwitch");
        storageSwitchKey = new StorageSwitchKey(parent.getClusterUUID(), wwn);
    }

    @SuppressWarnings("unchecked")
    @Override
    public StorageSwitchKey getKey() {
        return storageSwitchKey;
    }

    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    @SuppressWarnings("UnusedDeclaration")
    public String getIpAddress() {
        return ipAddress;
    }

    public void setIpAddress(final String ipAddress) {
        this.ipAddress = ipAddress;
    }

    public String getModel() {
        return model;
    }

    public void setModel(final String model) {
        this.model = model;
    }

    public Role getRole() {
        return role;
    }

    public void setRole(final Role role) {
        this.role = role;
    }

    @SuppressWarnings("UnusedDeclaration")
    public String getRoleRaw() {
        return roleRaw;
    }

    public void setRoleRaw(final String roleRaw) {
        this.roleRaw = roleRaw;
    }

    public Status getStatus() {
        return status;
    }

    public void setStatus(final Status status) {
        this.status = status;
    }

    @SuppressWarnings("UnusedDeclaration")
    public String getStatusRaw() {
        return statusRaw;
    }

    public void setStatusRaw(final String statusRaw) {
        this.statusRaw = statusRaw;
    }

    @SuppressWarnings("UnusedDeclaration")
    public String getVendor() {
        return vendor;
    }

    public void setVendor(final String vendor) {
        this.vendor = vendor;
    }

    @SuppressWarnings("UnusedDeclaration")
    public int getSensorCount() {
        return sensorCount;
    }

    public void setSensorCount(final int sensorCount) {
        this.sensorCount = sensorCount;
    }

    @SuppressWarnings("UnusedDeclaration")
    public int getPowerSupplyCount() {
        return powerSupplyCount;
    }

    public void setPowerSupplyCount(final int powerSupplyCount) {
        this.powerSupplyCount = powerSupplyCount;
    }

    @SuppressWarnings("UnusedDeclaration")
    public int getFanCount() {
        return fanCount;
    }

    public void setFanCount(final int fanCount) {
        this.fanCount = fanCount;
    }

    @SuppressWarnings("UnusedDeclaration")
    public String getFirmwareVersion() {
        return firmwareVersion;
    }

    public void setFirmwareVersion(final String firmwareVersion) {
        this.firmwareVersion = firmwareVersion;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "UUID", storageSwitchKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "ResourceKey", storageSwitchKey);
        addRequiredAttributeAndCRC(writer, "WWN", storageSwitchKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", name);
        addRequiredAttributeAndCRC(writer, "IPAddress", ipAddress);
        addRequiredAttributeAndCRC(writer, "FirmwareVersion", firmwareVersion);
        addRequiredAttributeAndCRC(writer, "Model", model);
        addRequiredAttributeAndCRC(writer, "Role", role);
        addRequiredAttributeAndCRC(writer, "RoleRaw", roleRaw);
        addRequiredAttributeAndCRC(writer, "Status", status);
        addRequiredAttributeAndCRC(writer, "StatusRaw", statusRaw);
        addRequiredAttributeAndCRC(writer, "Vendor", vendor);
        addRequiredAttributeAndCRC(writer, "SensorCount", sensorCount);
        addRequiredAttributeAndCRC(writer, "PowerSupplyCount", powerSupplyCount);
        addRequiredAttributeAndCRC(writer, "FanCount", fanCount);
    }

    /**
     * Storage switch roles
     */
    public enum Role {
        UNKNOWN,
        PRIMARY,
        SUBORDINATE,
        NOT_MAPPED
    }

    /**
     * Storage switch status
     */
    public enum Status {
        UNKNOWN,
        OK,
        ERROR,
        NOT_MAPPED
    }
}
