package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;


@OCIETable("vserver_name_mapping")
@OCIEResourceType("vserver_name_mapping")
public class VServerNameMapping extends BaseCRCXMLNode<VServer> {

    public final static class VServerNameMappingKey extends KeyBase<VServerNameMappingKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "name-mapping-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.name-mapping-info.position"
            )
            position,

            @OCIEZapiSource(
                sourceZapi = "name-mapping-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.name-mapping-info.direction"
            )
            direction
        }

        public VServerNameMappingKey(final String clusterUUID, final String vServerUUID, final String direction, final String position) {
            super("vserver_name_mapping",
                new EnumMap<Field, String>(Field.class) {{
                    put(Field.cluster_uuid, clusterUUID);
                    put(Field.vserver_uuid, vServerUUID);
                    put(Field.direction, direction);
                    put(Field.position, position);
                }});
        }

        @Override
        @OCIEUUID(uuidDescr = "vserver_uuid.direction.position")
        public String getOcieUUID() {
            final String clusterUUID = getID(Field.cluster_uuid);
            final String vServerUUID = getID(Field.vserver_uuid);
            final String direction = getID(Field.direction);
            final String position = getID(Field.position);
            return VServer.VServerKey.getOCIEVServerUUID(clusterUUID, vServerUUID) + '.' + direction + '.' + position;
        }
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Direction {
        NOT_MAPPED,
        KRB_UNIX,
        WIN_UNIX,
        UNIX_WIN,
    }

    private final VServerNameMappingKey key;

    @OCIEZapiSource(
        sourceZapi = "name-mapping-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.name-mapping-info.position"
    )
    @OCIEValue(
        ocieColumn = "position"
    )
    private final Integer position;

    @OCIEZapiSource(
        sourceZapi = "name-mapping-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.name-mapping-info.direction"
    )
    @OCIEValue(
        ocieColumn = "direction"
    )
    private Direction direction;

    @OCIEZapiSource(
        sourceZapi = "name-mapping-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.name-mapping-info.direction"
    )
    @OCIEValue(
        ocieColumn = "directionRaw"
    )
    private String directionRaw;

    @OCIEZapiSource(
        sourceZapi = "name-mapping-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.name-mapping-info.pattern"
    )
    @OCIEValue(
        ocieColumn = "pattern"
    )
    private String pattern;

    @OCIEZapiSource(
        sourceZapi = "name-mapping-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.name-mapping-info.replacement"
    )
    @OCIEValue(
        ocieColumn = "replacement"
    )
    private String replacement;

    VServerNameMapping(final VServer parent, final String direction, final Integer position) {
        super(parent, "NameMapping");
        final VServer.VServerKey vServerKey = parent.getKey();
        final String clusterUUID = vServerKey.getID(VServer.VServerKey.Field.cluster_uuid);
        final String vServerUUID = vServerKey.getID(VServer.VServerKey.Field.uuid);
        final String positionString = String.valueOf(position);
        this.key = new VServerNameMappingKey(clusterUUID, vServerUUID, direction, positionString);
        this.position = position;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public VServerNameMappingKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getPosition() {
        return position;
    }

    public void setDirection(final Direction direction) {
        this.direction = direction;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Direction getDirection() {
        return direction;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getDirectionRaw() {
        return directionRaw;
    }

    public void setDirectionRaw(final String directionRaw) {
        this.directionRaw = directionRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getPattern() {
        return pattern;
    }

    public void setPattern(final String pattern) {
        this.pattern = pattern;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getReplacement() {
        return replacement;
    }

    public void setReplacement(final String replacement) {
        this.replacement = replacement;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Direction", direction);
        addRequiredAttributeAndCRC(writer, "DirectionRaw", directionRaw);
        addRequiredAttributeAndCRC(writer, "Pattern", pattern);
        addRequiredAttributeAndCRC(writer, "Position", position);
        addRequiredAttributeAndCRC(writer, "Replacement", replacement);
    }
}
