package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import java.io.IOException;
import java.util.EnumMap;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEUUID;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEValue;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

@OCIETable("vserver_peer")
@OCIEResourceType("vserver_peer")
public class VServerPeer extends BaseCRCXMLNode<Cluster>  {
    public final static class VServerPeerKey extends KeyBase<VServerPeerKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "vserver-peer-get-iter",
                    zapiResponsePath = "netapp.results.attributes.vserver-peer-info.peer-cluster"
            )
            peer_cluster_name,
            
            @OCIEZapiSource(
                    sourceZapi = "vserver-peer-get-iter",
                    zapiResponsePath = "netapp.results.attributes.vserver-peer-info.peer-vserver"
            )
            peer_vserver_name,
            
            @OCIEZapiSource(
                    sourceZapi = "vserver-peer-get-iter",
                    zapiResponsePath = "netapp.results.attributes.vserver-peer-info.vserver"
            )
            vserver_name,
            
            @OCIEZapiSource(
                    sourceZapi = "vserver-peer-get-iter",
                    zapiResponsePath = "netapp.results.attributes.vserver-peer-info.vserver-uuid"
            )
            vserver_uuid,
            
            @OCIEZapiSource(
                    sourceZapi = "vserver-peer-get-iter",
                    zapiResponsePath = "netapp.results.attributes.vserver-peer-info.peer-vserver-uuid"
            )
            peer_vserver_uuid,
        }

        public VServerPeerKey(final String peerClusterName, final String peerVServerName, final String vServerName) {
            super("vserver_peer", new EnumMap<Field, String>(Field.class) {{
                put(Field.peer_cluster_name, peerClusterName);
                put(Field.peer_vserver_name, peerVServerName);
                put(Field.vserver_name, vServerName);
            }});
        }
        
        public VServerPeerKey(final String peerVServerUUID, final String vServerUUID) {
            super("vserver_peer", new EnumMap<Field, String>(Field.class) {{
                put(Field.peer_vserver_uuid, peerVServerUUID);
                put(Field.vserver_uuid, vServerUUID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "peer_cluster_name.peer_vserver_name.vserver_name or peer_vserver_uuid.vserver_uuid")
        public String getOcieUUID() {
            final String vServerName = getID(Field.vserver_name);
            
            if (vServerName != null) {
                final String peerClusterName = getID(Field.peer_cluster_name);
                final String peerVServerName = getID(Field.peer_vserver_name);
                return peerClusterName + "." + peerVServerName + "." + vServerName;
            }
            else {
                final String peerVServerUUID = getID(Field.peer_vserver_uuid);
                final String vServerUUID = getID(Field.vserver_uuid);
                return peerVServerUUID+ "." + vServerUUID;
            }
        }
    }

    private final VServerPeerKey key;
    
    @OCIEZapiSource(
            sourceZapi = "vserver-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.vserver-peer-info.vserver-uuid"
    )
    @OCIEValue(
            ocieColumn = "vServerUUID"
    )
    private String vServerUUID;
    
    @OCIEZapiSource(
            sourceZapi = "vserver-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.vserver-peer-info.peer-vserver-uuid"
    )
    @OCIEValue(
            ocieColumn = "peerVServerUUID"
    )
    private String peerVServerUUID;
    
    @OCIEZapiSource(
            sourceZapi = "vserver-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.vserver-peer-info.peer-cluster"
    )
    @OCIEValue(
            ocieColumn = "peerClusterName"
    )
    private String peerClusterName;
    
    @OCIEZapiSource(
            sourceZapi = "vserver-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.vserver-peer-info.peer-vserver"
    )
    @OCIEValue(
            ocieColumn = "peerVServerName"
    )
    private String peerVServerName;
    
    @OCIEZapiSource(
            sourceZapi = "vserver-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.vserver-peer-info.vserver"
    )
    @OCIEValue(
            ocieColumn = "vServerName"
    )
    private String vServerName;
    
    @OCIEZapiSource(
            sourceZapi = "vserver-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.vserver-peer-info.peer-state"
    )
    @OCIEValue(
            ocieColumn = "peerState"
    )
    private PeerState peerState;
    
    @OCIEZapiSource(
            sourceZapi = "vserver-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.vserver-peer-info.peer-state"
    )
    @OCIEValue(
            ocieColumn = "peerStateRaw"
    )
    private String peerStateRaw;
    
    @OCIEZapiSource(
            sourceZapi = "vserver-peer-get-iter",
            zapiResponsePath = "netapp.results.attributes.vserver-peer-info.applications"
    )
    @OCIEValue(
            ocieColumn = "applications"
    )
    private String applications;
    
    VServerPeer(final Cluster parent, final String peerClusterName, final String peerVServerName, final String vServerName) {
        super(parent, VServerPeer.class.getSimpleName());
        this.key = new VServerPeerKey(peerClusterName, peerVServerName, vServerName);
        this.peerClusterName = peerClusterName;
        this.peerVServerName = peerVServerName;
        this.vServerName = vServerName;
    }
    
    VServerPeer(final Cluster parent, final String peerVServerUUID, final String vServerUUID) {
        super(parent, VServerPeer.class.getSimpleName());
        this.key = new VServerPeerKey(peerVServerUUID, vServerUUID);
        this.peerVServerUUID = peerVServerUUID;
        this.vServerUUID = vServerUUID;
    }
    
    public static enum PeerState {
        NOT_MAPPED,
        PEERED, 
        PENDING, 
        INITIALIZING, 
        INITIATED, 
        REJECTED, 
        SUSPENDED, 
        DELETED,
    }
    
    public VServerPeerKey getKey() {
        return key;
    }
    
    public String getVServerUUID() {
        return vServerUUID;
    }
    
    public String getPeerVServerUUID() {
        return peerVServerUUID;
    }
    
    public String getPeerClusterName() {
        return peerClusterName;
    }
    
    public String getPeerVServerName() {
        return peerVServerName;
    }
    
    public String getVServerName() {
        return vServerName;
    }
    
    public PeerState getPeerState() {
        return peerState;
    }
    
    public String getPeerStateRaw() {
        return peerStateRaw;
    }
    
    public String getApplications() {
        return applications;
    }
    
    public void setVServerUUID(String vServerUUID) {
        this.vServerUUID = vServerUUID;
    }
    
    public void setPeerVServerUUID(String peerVServerUUID) {
        this.peerVServerUUID = peerVServerUUID;
    }
    
    public void setPeerClusterName(String peerClusterName) {
        this.peerClusterName = peerClusterName;
    }
    
    public void setPeerVServerName(String peerVServerName) {
        this.peerVServerName = peerVServerName;
    }
    
    public void setVServerName(String vServerName) {
        this.vServerName = vServerName;
    }
    
    public void setPeerState(PeerState peerState) {
        this.peerState = peerState;
    }
    
    public void setPeerStateRaw(String peerStateRaw) {
        this.peerStateRaw = peerStateRaw;
    }
    
    public void setApplications(String applications) {
        this.applications = applications;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "VServerUUID", vServerUUID);
        addRequiredAttributeAndCRC(writer, "PeerVServerUUID", peerVServerUUID);
        addRequiredAttributeAndCRC(writer, "PeerClusterName", peerClusterName);
        addRequiredAttributeAndCRC(writer, "PeerVServerName", peerVServerName);
        addRequiredAttributeAndCRC(writer, "PeerState", peerState);
        addRequiredAttributeAndCRC(writer, "PeerStateRaw", peerStateRaw);
        addRequiredAttributeAndCRC(writer, "Applications", applications);
    }
}
