package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;

@OCIETable("volume")
@OCIEResourceType("volume")
public class Volume extends BaseCRCXMLNode<VServer> {

    public final static class VolumeKey extends KeyBase<VolumeKey.Field> {
        @OCIEZapiSource(
            sourceZapi = "volume-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.instance-uuid"
        )
        private final String volumeInstanceUUID;

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "volume-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.uuid"
            )
            uuid            // JSD NOT instance uuid, regular uuid
        }

        public VolumeKey(final String clusterUUID, final String volumeUUID, final String volumeInstUUID) {
            super("volume",
                  new EnumMap<Field, String>(Field.class) {{
                        put(Field.cluster_uuid, clusterUUID);
                        put(Field.uuid, volumeUUID);
                  }});
            if (volumeInstUUID == null) {
                throw new NullPointerException("volumeInstUUID may not be null");
            }
            this.volumeInstanceUUID = volumeInstUUID;
        }

        public String getVolumeInstanceUUID() {
            return volumeInstanceUUID;
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getID(Field.uuid);
        }

        public static String getVolumeInstanceUUIDOrNull(final VolumeKey key) {
            return (key == null) ? null : key.getVolumeInstanceUUID();
        }

        @Override
        public String getResourceKey() {
            final String clusterUUID = getID(Field.cluster_uuid);
            final String instanceUUID = getVolumeInstanceUUID();
            return clusterUUID + ":type=volume,uuid=" + instanceUUID;
        }
    }

    // volume-id-attributes from volume-get-iter, volume-get
    private final VolumeKey volumeKey;                      // uuid

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.instance-uuid"
    )
    @OCIEValue(
        ocieColumn = "instanceUUID"
    )
    private String instanceUUID = null;                     // instance-uuid

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.name"
    )
    @OCIEValue(
        ocieColumn = "name"
    )
    private String volumeName = null;                       // name

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.containing-aggregate-uuid"
    )
    @OCIEValue(
        ocieColumn = "aggregateId",
        notes = "null for infinivols"
    )
    private Aggregate.AggregateKey aggregateKey = null;     // containing-aggregate-uuid

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.junction-path"
    )
    @OCIEValue(
        ocieColumn = "junctionPath"
    )
    private String junctionPath = null;                     // junction-path

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.style"
    )
    @OCIEValue(
        ocieColumn = "volumeStyle",
        notes = "derived enum"
    )
    private final VolumeStyle volumeStyle;                  // style

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.style"
    )
    @OCIEValue(
        ocieColumn = "volumeStyleRaw"
    )
    private String volumeStyleRaw = null;                   // style


    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.type"
    )
    @OCIEValue(
        ocieColumn = "volType",
        notes = "derived enum"
    )
    private VolumeType volumeType = null;                   // type

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.type"
    )
    @OCIEValue(
        ocieColumn = "volTypeRaw"
    )
    private String volumeTypeRaw = null;                    // type

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.qos-policy-group-name",
        alternateResponsePaths = "netapp.results.attributes-list.volume-attributes.volume-qos-attributes.policy-group-name"
    )
    @OCIEValue(
        ocieColumn = "policyGroupId"
    )
    private QosPolicyGroup.QosPolicyGroupKey qosPolicyGroupKey = null;  // qos-policy-group-name

    // volume-infinivol-attributes from volume-get-iter
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-infinitevol-attributes.storage-service"
    )
    @OCIEValue(
        ocieColumn = "storageServiceId"
    )
    private String storageServiceUUID;                      // storage-service

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-infinitevol-attributes.enable-snapdiff"
    )
    @OCIEValue(
        ocieColumn = "enableSnapDiff"
    )
    private Boolean enableSnapDiff;                         // enable-snapdiff

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-infinitevol-attributes.is-managed-by-service"
    )
    @OCIEValue(
        ocieColumn = "isManagedByService"
    )
    private Boolean isManagedByService;                     // is-managed-by-service

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-infinitevol-attributes.max-data-constituent-size"
    )
    @OCIEValue(
        ocieColumn = "maxDataConstituentSize"
    )
    private Long maxDataConstituentSize;                    // max-data-constituent-size

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-infinitevol-attributes.max-namespace-constituent-size"
    )
    @OCIEValue(
        ocieColumn = "maxDataConstituentSize"
    )
    private Long maxNamespaceConstituentSize;               // max-namespace-constituent-size

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-infinitevol-attributes.constituent-role"
    )
    @OCIEValue(
        ocieColumn = "constituentRole",
        notes = "derived enum"
    )
    private ConstituentRole constituentRole;                // constituent-role

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-infinitevol-attributes.constituent-role"
    )
    @OCIEValue(
        ocieColumn = "constituentRoleRaw"
    )
    private String constituentRoleRaw;                      // constituent-role

    // volume-space-attributes from volume-get-iter
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.size"
    )
    @OCIEValue(
        ocieColumn = "size"
    )
    private Long size = null;                               // size

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.size-total"
    )
    @OCIEValue(
        ocieColumn = "sizeTotal"
    )
    private Long sizeTotal = null;                          // size-total

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.size-used"
    )
    @OCIEValue(
        ocieColumn = "sizeUsed"
    )
    private Long sizeUsed = null;                           // size-used

    @OCIEZapiSource(
        sourceZapi = "derived"
    )
    @OCIEValue(
        ocieColumn = "sizeAvailable",
        notes = "sizeTotal - sizeUsed"
    )
    private Long sizeAvailable = null;                      // size-total - size-used

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.percent-used-capacity"
    )
    @OCIEValue(
        ocieColumn = "sizeUsedPercent"
    )
    private Integer sizeUsedPercent = null;                 // percent-used-capacity

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.is-space-guarantee-enabled"
    )
    @OCIEValue(
        ocieColumn = "isSpaceGuarantee"
    )
    private Boolean isSpaceGuarantee = null;                // is-space-guarantee-enabled

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.overwrite-reserve"
    )
    @OCIEValue(
        ocieColumn = "overwriteReserve"
    )
    private Long overwriteReserve = null;                   // overwrite-reserve

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.overwrite-reserve-required"
    )
    @OCIEValue(
        ocieColumn = "overwriteReserveRequired"
    )
    private Long overwriteReserveRequired = null;           // overwrite-reserve-required

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.overwrite-reserve-used"
    )
    @OCIEValue(
        ocieColumn = "overwriteReserveUsed"
    )
    private Long overwriteReserveUsed = null;               // overwrite-reserve-used

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.overwrite-reserve-used-actual"
    )
    @OCIEValue(
        ocieColumn = "overwriteReserveActualUsed"
    )
    private Long overwriteReserveActualUsed = null;         // overwrite-reserve-used-actual

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.snapshot-reserve-size"
    )
    @OCIEValue(
        ocieColumn = "snapshotReserveSize"
    )
    private Long snapshotReserveSize = null;                // snapshot-reserve-size

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.percentage-snapshot-reserve"
    )
    @OCIEValue(
        ocieColumn = "percentSnapshotReserved"
    )
    private Integer percentSnapshotReserved = null;         // percentage-snapshot-reserve

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.percentage-snapshot-reserve-used"
    )
    @OCIEValue(
        ocieColumn = "percentSnapshotReserveUsed"
    )
    private Integer percentSnapshotReserveUsed = null;      // percentage-snapshot-reserve-used

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.size-available-for-snapshots"
    )
    @OCIEValue(
        ocieColumn = "sizeAvailableForSnapshots"
    )
    private Long sizeAvailableForSnapshots = null;          // size-available-for-snapshots

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.percentage-fractional-reserve"
    )
    @OCIEValue(
        ocieColumn = "percentFractionalReserve"
    )
    private Integer percentFractionalReserve = null;        // percentage-fractional-reserve

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.space-mgmt-option-try-first"
    )
    @OCIEValue(
        ocieColumn = "spaceMgmtTryFirst",
        notes = "derived enum"
    )
    private SpaceMgmtOptionTryFirst spaceMgmtTryFirst = null;// space-mgmt-option-try-first

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.space-mgmt-option-try-first"
    )
    @OCIEValue(
        ocieColumn = "spaceMgmtTryFirstRaw"
    )
    private String spaceMgmtTryFirstRaw;                    // space-mgmt-option-try-first

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.size-used-by-snapshots"
    )
    @OCIEValue(
        ocieColumn = "sizeUsedBySnapshots"
    )
    private Long sizeUsedBySnapshots = null;                // size-used-by-snapshots

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.space-guarantee"
    )
    @OCIEValue(
        ocieColumn = "spaceGuarantee",
        notes = "derived enum"
    )
    private SpaceGuarantee spaceGuarantee = null;           // space-guarantee

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-space-attributes.space-guarantee"
    )
    @OCIEValue(
        ocieColumn = "spaceGuaranteeRaw"
    )
    private String spaceGuaranteeRaw = null;                // space-guarantee

    // volume-state-attributes from volume-get-iter
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-state-attributes.is-vserver-root"
    )
    @OCIEValue(
        ocieColumn = "isVserverRoot"
    )
    private Boolean isVServerRoot = null;                   // is-node-root

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-state-attributes.state"
    )
    @OCIEValue(
        ocieColumn = "state",
        notes = "derived enum"
    )
    private State state = null;                             // state

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-state-attributes.state"
    )
    @OCIEValue(
        ocieColumn = "stateRaw"
    )
    private String stateRaw = null;                         // state

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-state-attributes.is-junction-active"
    )
    @OCIEValue(
        ocieColumn = "isJunctionActive"
    )
    private Boolean isJunctionActive = null;                // is-junction-active

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-state-attributes.is-constituent"
    )
    @OCIEValue(
        ocieColumn = "isConstituent"
    )
    private final Boolean isConstituent;                    // is-constituent

    // volume-sis-attributes from volume-get-iter
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-sis-attributes.is-sis-volume"
    )
    @OCIEValue(
        ocieColumn = "isSisVolume"
    )
    private Boolean isSISVolume = null;                     // is-sis-volume

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-sis-attributes.deduplication-space-saved"
    )
    @OCIEValue(
        ocieColumn = "deduplicationSpaceSaved"
    )
    private Long deduplicationSpaceSaved = null;            // deduplication-space-saved

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-sis-attributes.compression-space-saved"
    )
    @OCIEValue(
        ocieColumn = "compressionSpaceSaved"
    )
    private Long compressionSpaceSaved = null;              // compression-space-saved

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-sis-attributes.percentage-deduplication-space-saved"
    )
    @OCIEValue(
        ocieColumn = "percentageDeduplicationSpaceSaved"
    )
    private Integer percentageDeduplicationSpaceSaved = null; // percentage-deduplication-space-saved

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-sis-attributes.percentage-compression-space-saved"
    )
    @OCIEValue(
        ocieColumn = "percentageCompressionSpaceSaved"
    )
    private Integer percentageCompressionSpaceSaved = null; // percentage-compression-space-saved

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-sis-attributes.policy"
    )
    @OCIEValue(
        ocieColumn = "sisPolicyId"
    )
    private SisPolicy.SisPolicyKey sisPolicyKey = null;     // policy

    // volume-mirror-attributes from volume-get-iter
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-mirror-attributes.is-data-protection-mirror"
    )
    @OCIEValue(
        ocieColumn = "isDataProtectionMirror"
    )
    private Boolean isDataProtectionMirror = null;          // is-data-protection-mirror

    // volume-mirror-attributes from volume-get-iter
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-mirror-attributes.is-load-sharing-mirror"
    )
    @OCIEValue(
        ocieColumn = "isLoadSharingMirror"
    )
    private Boolean isLoadSharingMirror = null;             // is-load-sharing-mirror

    // volume-mirror-attributes from volume-get-iter
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-mirror-attributes.is-move-mirror"
    )
    @OCIEValue(
        ocieColumn = "isMoveMirror"
    )
    private Boolean isMoveMirror = null;                    // is-move-mirror

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-mirror-attributes.is-replica-volume"
    )
    @OCIEValue(
        ocieColumn = "isReplicaVolume"
    )
    private Boolean isReplicaVolume = null;                 // is-replica-volume

    // volume-security-attributes from volume-get-iter
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-security-attributes.style"
    )
    @OCIEValue(
        ocieColumn = "securityStyle",
        notes = "derived enum"
    )
    private SecurityStyle securityStyle = null;             // volume-security-attributes.style

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-security-attributes.style"
    )
    @OCIEValue(
        ocieColumn = "securityStyle"
    )
    private String securityStyleRaw = null;                 // volume-security-attributes.style

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-security-attributes.volume-security-unix-attributes.group-id"
    )
    @OCIEValue(
        ocieColumn = "groupId"
    )
    private String groupID = null;                          // volume-security-attributes.volume-security-unix-attributes.group-id

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-security-attributes.volume-security-unix-attributes.permissions"
    )
    @OCIEValue(
        ocieColumn = "permissions"
    )
    private String permissions = null;                      // volume-security-attributes.volume-security-unix-attributes.permissions

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-security-attributes.volume-security-unix-attributes.user-id"
    )
    @OCIEValue(
        ocieColumn = "userId"
    )
    private String userID = null;                           // volume-security-attributes.volume-security-unix-attributes.user-id

    // volume-hybrid-cache-attributes from volume-get-iter
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-hybrid-cache-attributes.eligibility"
    )
    @OCIEValue(
        ocieColumn = "hybridCacheEligibility",
        notes = "derived enum"
    )
    private HybridCacheEligibility hybridCacheEligibility = null;// eligibility

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-hybrid-cache-attributes.eligibility"
    )
    @OCIEValue(
        ocieColumn = "hybridCacheEligibilityRaw"
    )
    private String hybridCacheEligibilityRaw = null;        // eligibility

    // volume-vm-align-attributes
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-vm-align-attributes.vm-align-sector"
    )
    @OCIEValue(
        ocieColumn = "vmAlignSector"
    )
    private Integer vmAlignSector = null;                   // vm-align-sector

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-vm-align-attributes.vm-align-suffix"
    )
    @OCIEValue(
        ocieColumn = "vmAlignSuffix"
    )
    private String vmAlignSuffix = null;                    // vm-align-suffix

    // volume-export-attributes
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-export-attributes.policy"
    )
    @OCIEValue(
        ocieColumn = "exportPolicyId"
    )
    private ExportPolicy.ExportPolicyKey exportPolicyKey = null;                 // policy

    // volume-directory-attributes
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-directory-attributes.i2p-enabled"
    )
    @OCIEValue(
        ocieColumn = "i2pEnabled"
    )
    private Boolean i2pEnabled = null;                      // i2p-enabled

    // volume-snapshot-attributes
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-attributes.auto-snapshots-enabled"
    )
    @OCIEValue(
        ocieColumn = "autoSnapshotsEnabled"
    )
    private Boolean autoSnapshotsEnabled = null;            // auto-snapshots-enabled

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-attributes.snapdir-access-enabled"
    )
    @OCIEValue(
        ocieColumn = "snapdirAccessEnabled"
    )
    private Boolean snapdirAccessEnabled = null;            // snapdir-access-enabled

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-attributes.snapshot-clone-dependency-enabled"
    )
    @OCIEValue(
        ocieColumn = "snapshotCloneDependencyEnabled"
    )
    private Boolean snapshotCloneDependencyEnabled = null;  // snapshot-clone-dependency-enabled

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-attributes.snapshot-count"
    )
    @OCIEValue(
        ocieColumn = "snapshotCloneDependencyEnabled"
    )
    private Long snapshotCount = null;                      // snapshot-count

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-attributes.snapshot-policy"
    )
    @OCIEValue(
        ocieColumn = "snapshotPolicyId"
    )
    private SnapshotPolicy.SnapshotPolicyKey snapshotPolicyKey;     // snapshot-policy

    // volume-autosize-attributes
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-autosize-attributes.mode"
    )
    @OCIEValue(
        ocieColumn = "autoSizeMode",
        notes = "derived enum"
    )
    private VolumeAutoSizeMode autoSizeMode = null;         // mode

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-autosize-attributes.mode"
    )
    @OCIEValue(
        ocieColumn = "autoSizeMode"
    )
    private String autoSizeModeRaw = null;                  // mode

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-autosize-attributes.maximum-size"
    )
    @OCIEValue(
        ocieColumn = "maxAutosizeSize"
    )
    private Long maxAutosizeSize = null;                    // maximum-size

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-autosize-attributes.increment-size"
    )
    @OCIEValue(
        ocieColumn = "autosizeIncrementSize"
    )
    private Long autosizeIncrementSize = null;              // increment-size

    // volume-performance-attributes
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-performance-attributes.is-atime-update-enabled"
    )
    @OCIEValue(
        ocieColumn = "isAtimeUpdateEnabled"
    )
    private Boolean isAtimeUpdateEnabled = null;            // is-atime-update-enabled

    // volume-inode-attributes
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-inode-attributes.block-type"
    )
    @OCIEValue(
        ocieColumn = "blockType",
        notes = "derived enum"
    )
    private Volume.BlockType blockType = null;              // block-type

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-inode-attributes.block-type"
    )
    @OCIEValue(
        ocieColumn = "blockTypeRaw"
    )
    private String blockTypeRaw = null;                     // block-type

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-inode-attributes.files-total"
    )
    @OCIEValue(
        ocieColumn = "inodeFilesTotal"
    )
    private Long inodeFilesTotal;                           // files-total

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-inode-attributes.files-used"
    )
    @OCIEValue(
        ocieColumn = "inodeFilesUsed"
    )
    private Long inodeFilesUsed;                            // files-used

    // volume-language-attributes
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-language-attributes.language-code"
    )
    @OCIEValue(
        ocieColumn = "languageCode"
    )
    private String languageCode = null;                     // language-code

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-language-attributes.is-create-ucode-enabled"
    )
    @OCIEValue(
        ocieColumn = "isCreateUcodeEnabled"
    )
    private Boolean isCreateUcodeEnabled = null;            // is-create-ucode-enabled

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-language-attributes.is-convert-ucode-enabled"
    )
    @OCIEValue(
        ocieColumn = "isConvertUcodeEnabled"
    )
    private Boolean isConvertUcodeEnabled = null;           // is-convert-ucode-enabled

    // volume-clone-attributes
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-clone-attributes.clone-child-count"
    )
    @OCIEValue(
        ocieColumn = "cloneChildCount"
    )
    private Integer cloneChildCount = null;                 // clone-child-count

    // volume-snapshot-autodelete-attributes
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-autodelete-attributes.is-autodelete-enabled"
    )
    @OCIEValue(
        ocieColumn = "isAutoDeleteEnabled"
    )
    private Boolean isAutoDeleteEnabled = null;             // is-autodelete-enabled

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-autodelete-attributes.commitment"
    )
    @OCIEValue(
        ocieColumn = "commitment",
        notes = "derived enums"
    )
    private Commitment commitment = null;                   // commitment

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-autodelete-attributes.commitment"
    )
    @OCIEValue(
        ocieColumn = "commitmentRaw"
    )
    private String commitmentRaw = null;                    // commitment

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-autodelete-attributes.delete-order"
    )
    @OCIEValue(
        ocieColumn = "deleteOrder",
        notes = "derived enum"
    )
    private DeleteOrder deleteOrder = null;                 // delete-order

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-autodelete-attributes.delete-order"
    )
    @OCIEValue(
        ocieColumn = "deleteOrderRaw"
    )
    private String deleteOrderRaw = null;                   // delete-order

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-autodelete-attributes.defer-delete"
    )
    @OCIEValue(
        ocieColumn = "deferDelete",
        notes = "derived enum"
    )
    private DeferDelete deferDelete = null;                 // defer-delete

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-autodelete-attributes.defer-delete"
    )
    @OCIEValue(
        ocieColumn = "deferDeleteRaw"
    )
    private String deferDeleteRaw = null;                   // defer-delete

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-autodelete-attributes.target-free-space"
    )
    @OCIEValue(
        ocieColumn = "targetFreeSpace"
    )
    private Integer targetFreeSpace = null;                 // target-free-space

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-autodelete-attributes.trigger"
    )
    @OCIEValue(
        ocieColumn = "trigger",
        notes = "derived enum"
    )
    private Trigger trigger = null;                         // trigger

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-autodelete-attributes.trigger"
    )
    @OCIEValue(
        ocieColumn = "triggerRaw"
    )
    private String triggerRaw = null;                       // trigger

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-autodelete-attributes.prefix"
    )
    @OCIEValue(
        ocieColumn = "triggerRaw"
    )
    private String prefix = null;                           // prefix

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-autodelete-attributes.destroy-list"
    )
    @OCIEValue(
        ocieColumn = "destroyList"
    )
    private EnumSet<DestroyList> destroyList = EnumSet.noneOf(Volume.DestroyList.class); // destroy-list

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-snapshot-autodelete-attributes.destroy-list"
    )
    @OCIEValue(
        ocieColumn = "destroyListRaw"
    )
    private String destroyListRaw = null;                   // destroy-list

    // volume-flexcache-attributes
    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-flexcache-attributes.min-reserve"
    )
    @OCIEValue(
        ocieColumn = "flexCacheMinReserve"
    )
    private Long flexCacheMinReserve = null;                // min-reserve

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-flexcache-attributes.origin"
    )
    @OCIEValue(
        ocieColumn = "originVolumeName"
    )
    private String originVolumeName = null;                 // origin

    // sis-get-iter sis-status-info
    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.status"
    )
    @OCIEValue(
        ocieColumn = "sisStatus",
        notes = "derived enum"
    )
    private VolumeSisStatus sisStatus = null;               // status

    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.status"
    )
    @OCIEValue(
        ocieColumn = "sisStatusRaw"
    )
    private String sisStatusRaw = null;                     // status

    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.state"
    )
    @OCIEValue(
        ocieColumn = "sisState",
        notes = "derived enum"
    )
    private VolumeSisState sisState = null;                 // state

    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.state"
    )
    @OCIEValue(
        ocieColumn = "sisStateRaw"
    )
    private String sisStateRaw = null;                      // state

    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.type"
    )
    @OCIEValue(
        ocieColumn = "sisType",
        notes = "derived enum"
    )
    private VolumeSisType sisType = null;                   // type

    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.type"
    )
    @OCIEValue(
        ocieColumn = "sisTypeRaw"
    )
    private String sisTypeRaw = null;                       // type

    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.last-op-state"
    )
    @OCIEValue(
        ocieColumn = "lastSisOpState",
        notes = "derived enum"
    )
    private VolumeSisOpState lastSisOpState = null;         // last-op-state

    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.last-op-state"
    )
    @OCIEValue(
        ocieColumn = "lastSisOpStateRaw"
    )
    private String lastSisOpStateRaw = null;                // last-op-state

    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.schedule"
    )
    @OCIEValue(
        ocieColumn = "schedule"
    )
    private String schedule = null;                         // schedule

    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.progress"
    )
    @OCIEValue(
        ocieColumn = "progress"
    )
    private String progress = null;                         // progress

    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.last-op-begin-timestamp"
    )
    @OCIEValue(
        ocieColumn = "lastOpBeginTS"
    )
    private Long lastOpBeginTS = null;                      // last-op-begin-timestamp


    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.last-end-begin-timestamp"
    )
    @OCIEValue(
        ocieColumn = "lastOpEndTS"
    )
    private Long lastOpEndTS = null;                        // last-op-end-timestamp


    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.last-op-error"
    )
    @OCIEValue(
        ocieColumn = "lastOpError"
    )
    private String lastOpError = null;                      // last-op-error

    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.last-op-size"
    )
    @OCIEValue(
        ocieColumn = "lastSisOpSize"
    )
    private Long lastSisOpSize = null;                      // last-op-size

    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.is-compression-enabled"
    )
    @OCIEValue(
        ocieColumn = "isCompressionEnabled"
    )
    private Boolean isCompressionEnabled = null;            // is-compression-enabled

    @OCIEZapiSource(
        sourceZapi = "sis-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.sis-status-info.is-inline-compression-enabled"
    )
    @OCIEValue(
        ocieColumn = "isInlineCompressionEnabled"
    )
    private Boolean isInlineCompressionEnabled = null;      // is-inline-compression-enabled

    @OCIEZapiSource(
        sourceZapi = "snapshot-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapshot-info.access-time"
    )
    @OCIEValue(
        ocieColumn = "oldestSnapshotTimestamp",
        notes = "oldest access-time value of all snapshots, only when snapshot discovery is enabled"
    )
    private Long oldestSnapshotTimestamp = null;            // lowest value of all snapshots' creation time

    @OCIEZapiSource(
        sourceZapi = "volume-clone-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-clone-info.split-estimate"
    )
    @OCIEValue(
        ocieColumn = "cloneSplitEstimate",
        notes = "in bytes, split-estimate *4096"
    )
    private Long cloneSplitEstimate = null;                 // split-estimate

    // flexcache-get-iter flexcache-info

    // derived
    @OCIEZapiSource(
        sourceZapi = "derived"
    )
    @OCIEValue(
        ocieColumn = "quotaCommit",
        notes = "Sum of Qtree quota bytes limit"
    )
    private Long quotaCommit = 0L;                          // Sum of Qtree quota bytes limit


    @OCIEZapiSource(
            sourceZapi = "quota-status-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota-status-attributes.status"
    )
    @OCIEValue(
            ocieColumn = "quotaStatus"
    )
    private QuotaStatus quotaStatus;  // Primary status of quota

    @OCIEZapiSource(
            sourceZapi = "quota-status-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota-status-attributes.status"
    )
    @OCIEValue(
            ocieColumn = "quotaStatusRaw"
    )
    private String quotaStatusRaw;    // Raw value of primary status of quota

//    @OCIEZapiSource(
//            sourceZapi = "volume-footprint-get-iter",
//            zapiResponsePath = "netapp.results.attributes-list.footprint-info.volume-blocks-footprint"
//    )
//    @OCIEValue(
//            ocieColumn = "volumeBlocksFootPrint"
//    )
    private Long volumeBlocksFootPrint;

    @OCIEZapiSource(
            sourceZapi = "volume-get-iter",
            zapiResponsePath = "volume-attributes.volume-hybrid-cache-attributes.caching-policy"
    )
    @OCIEValue(
            ocieColumn = "cachingPolicyRaw"
    )
    private String cachingPolicyRaw;    // Raw value of caching-policy

    @OCIEZapiSource(
            sourceZapi = "volume-get-iter",
            zapiResponsePath = "volume-attributes.volume-hybrid-cache-attributes.caching-policy"
    )
    @OCIEValue(
            ocieColumn = "cachingPolicy"
    )
    private CachingPolicy cachingPolicy;


    // Children
    private final QTrees qtrees;
    private final LUNs luns;
    private final TreeQuotas treeQuotas;
    private final UserQuotas userQuotas;

    Volume(final VServer parent, final String volumeUUID, final String volumeInstUUID, final VolumeStyle volumeStyle, final Boolean isConstituent) {
        super(parent, "Volume");
        this.isConstituent = isConstituent;
        this.volumeStyle = volumeStyle;
        final String clusterUUID = parent.getParent().getKey().getID(Cluster.ClusterKey.Field.uuid);
        this.volumeKey = new VolumeKey(clusterUUID, volumeUUID, volumeInstUUID);
        this.qtrees = new QTrees(this);
        this.luns = new LUNs(this);
        this.treeQuotas = new TreeQuotas(this);
        this.userQuotas = new UserQuotas(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public VolumeKey getKey() {
        return this.volumeKey;
    }

    public String getInstanceUUID() {
        return instanceUUID;
    }

    public void setInstanceUUID(final String instanceUUID) {
        this.instanceUUID = instanceUUID;
    }

    public String getVolumeName() {
        return volumeName;
    }

    public void setVolumeName(final String volumeName) {
        this.volumeName = volumeName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Aggregate.AggregateKey getAggregateKey() {
        return aggregateKey;
    }

    public void setAggregateKey(final Aggregate.AggregateKey aggregateKey) {
        this.aggregateKey = aggregateKey;
    }

    public String getJunctionPath() {
        return junctionPath;
    }

    public void setJunctionPath(final String junctionPath) {
        this.junctionPath = junctionPath;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public VolumeStyle getVolumeStyle() {
        return volumeStyle;
    }



    public DerivedVolumeStyle getDerivedVolumeStyle() {
        if (isConstituent != null && isConstituent) {
            return DerivedVolumeStyle.CONSTITUENT;
        }

        DerivedVolumeStyle style = DerivedVolumeStyle.UNKNOWN;
        if (volumeStyle != null) {
            switch (volumeStyle) {
                case FLEX:
                    style = DerivedVolumeStyle.FLEXVOL;
                    break;
                case INFINITEVOL:
                    style = DerivedVolumeStyle.INFINIVOL;
                    break;
                case STRIPED:
                    style = DerivedVolumeStyle.STRIPED;
                    break;
                default:
                    style= DerivedVolumeStyle.UNKNOWN;

            }
        }
        return style;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public VolumeType getVolumeType() {
        return volumeType;
    }

    public void setVolumeType(final VolumeType volumeType) {
        this.volumeType = volumeType;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSize() {
        return size;
    }

    public void setSize(final Long size) {
        this.size = size;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSizeTotal() {
        return sizeTotal;
    }

    public void setSizeTotal(final Long sizeTotal) {
        this.sizeTotal = sizeTotal;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSizeUsed() {
        return sizeUsed;
    }

    public void setSizeUsed(final Long sizeUsed) {
        this.sizeUsed = sizeUsed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSizeAvailable() {
        return sizeAvailable;
    }

    public void setSizeAvailable(final Long sizeAvailable) {
        this.sizeAvailable = sizeAvailable;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getSizeUsedPercent() {
        return sizeUsedPercent;
    }

    public void setSizeUsedPercent(final Integer sizeUsedPercent) {
        this.sizeUsedPercent = sizeUsedPercent;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getSizeAvailPercent() {
        return (sizeUsedPercent == null) ? null : 100 - sizeUsedPercent;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isSpaceGuarantee() {
        return isSpaceGuarantee;
    }

    public void setSpaceGuarantee(final Boolean spaceGuarantee) {
        isSpaceGuarantee = spaceGuarantee;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getOverwriteReserve() {
        return overwriteReserve;
    }

    public void setOverwriteReserve(final Long overwriteReserve) {
        this.overwriteReserve = overwriteReserve;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getOverwriteReserveRequired() {
        return overwriteReserveRequired;
    }

    public void setOverwriteReserveRequired(final Long overwriteReserveRequired) {
        this.overwriteReserveRequired = overwriteReserveRequired;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getOverwriteReserveUsed() {
        return overwriteReserveUsed;
    }

    public void setOverwriteReserveUsed(final Long overwriteReserveUsed) {
        this.overwriteReserveUsed = overwriteReserveUsed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getOverwriteReserveActualUsed() {
        return overwriteReserveActualUsed;
    }

    public void setOverwriteReserveActualUsed(final Long overwriteReserveActualUsed) {
        this.overwriteReserveActualUsed = overwriteReserveActualUsed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSnapshotReserveSize() {
        return snapshotReserveSize;
    }

    public void setSnapshotReserveSize(final Long snapshotReserveSize) {
        this.snapshotReserveSize = snapshotReserveSize;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getPercentSnapshotReserved() {
        return percentSnapshotReserved;
    }

    public void setPercentSnapshotReserved(final Integer percentSnapshotReserved) {
        this.percentSnapshotReserved = percentSnapshotReserved;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getPercentSnapshotReserveUsed() {
        return percentSnapshotReserveUsed;
    }

    public void setPercentSnapshotReserveUsed(final Integer percentSnapshotReserveUsed) {
        this.percentSnapshotReserveUsed = percentSnapshotReserveUsed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSizeAvailableForSnapshots() {
        return sizeAvailableForSnapshots;
    }

    public void setSizeAvailableForSnapshots(final Long sizeAvailableForSnapshots) {
        this.sizeAvailableForSnapshots = sizeAvailableForSnapshots;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getPercentFractionalReserve() {
        return percentFractionalReserve;
    }

    public void setPercentFractionalReserve(final Integer percentFractionalReserve) {
        this.percentFractionalReserve = percentFractionalReserve;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public SpaceMgmtOptionTryFirst getSpaceMgmtTryFirst() {
        return spaceMgmtTryFirst;
    }

    public void setSpaceMgmtTryFirst(final SpaceMgmtOptionTryFirst spaceMgmtTryFirst) {
        this.spaceMgmtTryFirst = spaceMgmtTryFirst;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSizeUsedBySnapshots() {
        return sizeUsedBySnapshots;
    }

    public void setSizeUsedBySnapshots(final Long sizeUsedBySnapshots) {
        this.sizeUsedBySnapshots = sizeUsedBySnapshots;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public SpaceGuarantee getSpaceGuarantee() {
        return spaceGuarantee;
    }

    public void setSpaceGuarantee(final SpaceGuarantee spaceGuarantee) {
        this.spaceGuarantee = spaceGuarantee;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getSpaceGuaranteeRaw() {
        return spaceGuaranteeRaw;
    }

    public void setSpaceGuaranteeRaw(final String spaceGuaranteeRaw) {
        this.spaceGuaranteeRaw = spaceGuaranteeRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isVServerRoot() {
        return isVServerRoot;
    }

    public void setVServerRoot(final Boolean VServerRoot) {
        isVServerRoot = VServerRoot;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public State getState() {
        return state;
    }

    public void setState(final State state) {
        this.state = state;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isJunctionActive() {
        return isJunctionActive;
    }

    public void setJunctionActive(final Boolean junctionActive) {
        isJunctionActive = junctionActive;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isConstituent() {
        return isConstituent;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isSISVolume() {
        return isSISVolume;
    }

    public void setSISVolume(final Boolean SISVolume) {
        isSISVolume = SISVolume;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getDeduplicationSpaceSaved() {
        return deduplicationSpaceSaved;
    }

    public void setDeduplicationSpaceSaved(final Long deduplicationSpaceSaved) {
        this.deduplicationSpaceSaved = deduplicationSpaceSaved;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getCompressionSpaceSaved() {
        return compressionSpaceSaved;
    }

    public void setCompressionSpaceSaved(final Long compressionSpaceSaved) {
        this.compressionSpaceSaved = compressionSpaceSaved;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getPercentageDeduplicationSpaceSaved() {
        return percentageDeduplicationSpaceSaved;
    }

    public void setPercentageDeduplicationSpaceSaved(final Integer percentageDeduplicationSpaceSaved) {
        this.percentageDeduplicationSpaceSaved = percentageDeduplicationSpaceSaved;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getPercentageCompressionSpaceSaved() {
        return percentageCompressionSpaceSaved;
    }

    public void setPercentageCompressionSpaceSaved(final Integer percentageCompressionSpaceSaved) {
        this.percentageCompressionSpaceSaved = percentageCompressionSpaceSaved;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public SisPolicy.SisPolicyKey getSisPolicyKey() {
        return sisPolicyKey;
    }

    public void setSisPolicyKey(final SisPolicy.SisPolicyKey sisPolicyKey) {
        this.sisPolicyKey = sisPolicyKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isDataProtectionMirror() {
        return isDataProtectionMirror;
    }

    public void setDataProtectionMirror(final Boolean dataProtectionMirror) {
        isDataProtectionMirror = dataProtectionMirror;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isLoadSharingMirror() {
        return isLoadSharingMirror;
    }

    public void setLoadSharingMirror(final Boolean loadSharingMirror) {
        isLoadSharingMirror = loadSharingMirror;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isMoveMirror() {
        return isMoveMirror;
    }

    public void setMoveMirror(final Boolean moveMirror) {
        isMoveMirror = moveMirror;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isReplicaVolume() {
        return isReplicaVolume;
    }

    public void setReplicaVolume(final Boolean replicaVolume) {
        isReplicaVolume = replicaVolume;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public SecurityStyle getSecurityStyle() {
        return securityStyle;
    }

    public void setSecurityStyle(final SecurityStyle securityStyle) {
        this.securityStyle = securityStyle;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public HybridCacheEligibility getHybridCacheEligibility() {
        return hybridCacheEligibility;
    }

    public void setHybridCacheEligibility(final HybridCacheEligibility hybridCacheEligibility) {
        this.hybridCacheEligibility = hybridCacheEligibility;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getVmAlignSector() {
        return vmAlignSector;
    }

    public void setVmAlignSector(final Integer vmAlignSector) {
        this.vmAlignSector = vmAlignSector;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getVmAlignSuffix() {
        return vmAlignSuffix;
    }

    public void setVmAlignSuffix(final String vmAlignSuffix) {
        this.vmAlignSuffix = vmAlignSuffix;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public ExportPolicy.ExportPolicyKey getExportPolicyKey() {
        return exportPolicyKey;
    }

    public void setExportPolicyKey(final ExportPolicy.ExportPolicyKey exportPolicyKey) {
        this.exportPolicyKey = exportPolicyKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isI2pEnabled() {
        return i2pEnabled;
    }

    public void setI2pEnabled(final Boolean i2pEnabled) {
        this.i2pEnabled = i2pEnabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isAutoSnapshotsEnabled() {
        return autoSnapshotsEnabled;
    }

    public void setAutoSnapshotsEnabled(final Boolean autoSnapshotsEnabled) {
        this.autoSnapshotsEnabled = autoSnapshotsEnabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isSnapdirAccessEnabled() {
        return snapdirAccessEnabled;
    }

    public void setSnapdirAccessEnabled(final Boolean snapdirAccessEnabled) {
        this.snapdirAccessEnabled = snapdirAccessEnabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isSnapshotCloneDependencyEnabled() {
        return snapshotCloneDependencyEnabled;
    }

    public void setSnapshotCloneDependencyEnabled(final Boolean snapshotCloneDependencyEnabled) {
        this.snapshotCloneDependencyEnabled = snapshotCloneDependencyEnabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSnapshotCount() {
        return snapshotCount;
    }

    public void setSnapshotCount(final Long snapshotCount) {
        this.snapshotCount = snapshotCount;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public SnapshotPolicy.SnapshotPolicyKey getSnapshotPolicyKey() {
        return snapshotPolicyKey;
    }

    public void setSnapshotPolicyKey(final SnapshotPolicy.SnapshotPolicyKey snapshotPolicyKey) {
        this.snapshotPolicyKey = snapshotPolicyKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getMaxAutosizeSize() {
        return maxAutosizeSize;
    }

    public void setMaxAutosizeSize(final Long maxAutosizeSize) {
        this.maxAutosizeSize = maxAutosizeSize;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getAutosizeIncrementSize() {
        return autosizeIncrementSize;
    }

    public void setAutosizeIncrementSize(final Long autosizeIncrementSize) {
        this.autosizeIncrementSize = autosizeIncrementSize;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isAtimeUpdateEnabled() {
        return isAtimeUpdateEnabled;
    }

    public void setAtimeUpdateEnabled(final Boolean atimeUpdateEnabled) {
        isAtimeUpdateEnabled = atimeUpdateEnabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public BlockType getBlockType() {
        return blockType;
    }

    public void setBlockType(final BlockType blockType) {
        this.blockType = blockType;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getInodeFilesTotal() {
        return inodeFilesTotal;
    }

    public void setInodeFilesTotal(final Long filesTotal) {
        this.inodeFilesTotal = filesTotal;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getInodeFilesUsed() {
        return inodeFilesUsed;
    }

    public void setInodeFilesUsed(final Long filesUsed) {
        this.inodeFilesUsed = filesUsed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getQuotaCommit() {
        return quotaCommit;
    }

    public void addToQuotaCommit(final Long qtreeDiskLimit) {
        if (qtreeDiskLimit != null) {
            this.quotaCommit += qtreeDiskLimit;
        }
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getQuotaOvercommit() {
        final long sizeTotalForCalc = (sizeTotal == null) ? 0L : sizeTotal;
        final long snapshotTotalForCalc = (sizeUsedBySnapshots == null) ? 0L : sizeUsedBySnapshots;
        if (quotaCommit == 0) {
            return 0L;
        }
        final Long quotaOverCommit = quotaCommit - (sizeTotalForCalc + snapshotTotalForCalc);
        return (quotaOverCommit < 0) ? 0L : quotaOverCommit;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public QuotaStatus getQuotaStatus() {
        return quotaStatus;
    }

    public void setQuotaStatus(final QuotaStatus quotaStatus) {
        this.quotaStatus = quotaStatus;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getQuotaStatusRaw() {
        return quotaStatusRaw;
    }

    public void setQuotaStatusRaw(final String quotaStatusRaw) {
        this.quotaStatusRaw = quotaStatusRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLanguageCode() {
        return languageCode;
    }

    public void setLanguageCode(final String languageCode) {
        this.languageCode = languageCode;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isCreateUcodeEnabled() {
        return isCreateUcodeEnabled;
    }

    public void setCreateUcodeEnabled(final Boolean createUcodeEnabled) {
        isCreateUcodeEnabled = createUcodeEnabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isConvertUcodeEnabled() {
        return isConvertUcodeEnabled;
    }

    public void setConvertUcodeEnabled(final Boolean convertUcodeEnabled) {
        isConvertUcodeEnabled = convertUcodeEnabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getCloneChildCount() {
        return cloneChildCount;
    }

    public void setCloneChildCount(final Integer cloneChildCount) {
        this.cloneChildCount = cloneChildCount;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getCloneSplitEstimate() {
        return cloneSplitEstimate;
    }

    public void setCloneSplitEstimate(final Long cloneSplitEstimate) {
        this.cloneSplitEstimate = cloneSplitEstimate;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isAutoDeleteEnabled() {
        return isAutoDeleteEnabled;
    }

    public void setAutoDeleteEnabled(final Boolean autoDeleteEnabled) {
        isAutoDeleteEnabled = autoDeleteEnabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Commitment getCommitment() {
        return commitment;
    }

    public void setCommitment(final Commitment commitment) {
        this.commitment = commitment;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public DeleteOrder getDeleteOrder() {
        return deleteOrder;
    }

    public void setDeleteOrder(final DeleteOrder deleteOrder) {
        this.deleteOrder = deleteOrder;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public DeferDelete getDeferDelete() {
        return deferDelete;
    }

    public void setDeferDelete(final DeferDelete deferDelete) {
        this.deferDelete = deferDelete;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getTargetFreeSpace() {
        return targetFreeSpace;
    }

    public void setTargetFreeSpace(final Integer targetFreeSpace) {
        this.targetFreeSpace = targetFreeSpace;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Trigger getTrigger() {
        return trigger;
    }

    public void setTrigger(final Trigger trigger) {
        this.trigger = trigger;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getPrefix() {
        return prefix;
    }

    public void setPrefix(final String prefix) {
        this.prefix = prefix;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public EnumSet<DestroyList> getDestroyList() {
        return destroyList;
    }

    public void setDestroyList(final EnumSet<DestroyList> destroyList) {
        this.destroyList = destroyList;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public VolumeSisStatus getSisStatus() {
        return sisStatus;
    }

    public void setSisStatus(final VolumeSisStatus sisStatus) {
        this.sisStatus = sisStatus;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public VolumeSisState getSisState() {
        return sisState;
    }

    public void setSisState(final VolumeSisState sisState) {
        this.sisState = sisState;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public VolumeSisType getSisType() {
        return sisType;
    }

    public void setSisType(final VolumeSisType sisType) {
        this.sisType = sisType;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public VolumeSisOpState getLastSisOpState() {
        return lastSisOpState;
    }

    public void setLastSisOpState(final VolumeSisOpState lastSisOpState) {
        this.lastSisOpState = lastSisOpState;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getSchedule() {
        return schedule;
    }

    public void setSchedule(final String schedule) {
        this.schedule = schedule;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getProgress() {
        return progress;
    }

    public void setProgress(final String progress) {
        this.progress = progress;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public VolumeAutoSizeMode getAutoSizeMode() {
        return autoSizeMode;
    }

    public void setAutoSizeMode(final VolumeAutoSizeMode autoSizeMode) {
        this.autoSizeMode = autoSizeMode;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getLastOpBeginTS() {
        return lastOpBeginTS;
    }

    public void setLastOpBeginTS(final Long lastOpBeginTS) {
        this.lastOpBeginTS = lastOpBeginTS;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getLastOpEndTS() {
        return lastOpEndTS;
    }

    public void setLastOpEndTS(final Long lastOpEndTS) {
        this.lastOpEndTS = lastOpEndTS;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLastOpError() {
        return lastOpError;
    }

    public void setLastOpError(final String lastOpError) {
        this.lastOpError = lastOpError;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getLastSisOpSize() {
        return lastSisOpSize;
    }

    public void setLastSisOpSize(final Long lastSisOpSize) {
        this.lastSisOpSize = lastSisOpSize;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isCompressionEnabled() {
        return isCompressionEnabled;
    }

    public void setCompressionEnabled(final Boolean compressionEnabled) {
        isCompressionEnabled = compressionEnabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isInlineCompressionEnabled() {
        return isInlineCompressionEnabled;
    }

    public void setInlineCompressionEnabled(final Boolean inlineCompressionEnabled) {
        isInlineCompressionEnabled = inlineCompressionEnabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getFlexCacheMinReserve() {
        return flexCacheMinReserve;
    }

    public void setFlexCacheMinReserve(final Long flexCacheMinReserve) {
        this.flexCacheMinReserve = flexCacheMinReserve;
    }

    public boolean isFlexCache() {
        return getFlexCacheMinReserve() != null && getFlexCacheMinReserve() != 0;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getOriginVolumeName() {
        return originVolumeName;
    }

    public void setOriginVolumeName(final String originVolumeName) {
        this.originVolumeName = originVolumeName;
    }

    public void setVolumeStyleRaw(final String volumeStyleRaw) {
        this.volumeStyleRaw = volumeStyleRaw;
    }

    public void setVolumeTypeRaw(final String volumeTypeRaw) {
        this.volumeTypeRaw = volumeTypeRaw;
    }

    public void setSpaceMgmtTryFirstRaw(final String spaceMgmtTryFirstRaw) {
        this.spaceMgmtTryFirstRaw = spaceMgmtTryFirstRaw;
    }

    public void setSecurityStyleRaw(final String securityStyleRaw) {
        this.securityStyleRaw = securityStyleRaw;
    }

    public void setHybridCacheEligibilityRaw(final String hybridCacheEligibilityRaw) {
        this.hybridCacheEligibilityRaw = hybridCacheEligibilityRaw;
    }

    public void setAutoSizeModeRaw(final String autoSizeModeRaw) {
        this.autoSizeModeRaw = autoSizeModeRaw;
    }

    public void setBlockTypeRaw(final String blockTypeRaw) {
        this.blockTypeRaw = blockTypeRaw;
    }

    public void setCommitmentRaw(final String commitmentRaw) {
        this.commitmentRaw = commitmentRaw;
    }

    public void setDeleteOrderRaw(final String deleteOrderRaw) {
        this.deleteOrderRaw = deleteOrderRaw;
    }

    public void setDeferDeleteRaw(final String deferDeleteRaw) {
        this.deferDeleteRaw = deferDeleteRaw;
    }

    public void setDestroyListRaw(final String destroyListRaw) {
        this.destroyListRaw = destroyListRaw;
    }

    public void setSisStatusRaw(final String sisStatusRaw) {
        this.sisStatusRaw = sisStatusRaw;
    }

    public void setSisStateRaw(final String sisStateRaw) {
        this.sisStateRaw = sisStateRaw;
    }

    public void setSisTypeRaw(final String sisTypeRaw) {
        this.sisTypeRaw = sisTypeRaw;
    }

    public void setLastSisOpStateRaw(final String lastSisOpStateRaw) {
        this.lastSisOpStateRaw = lastSisOpStateRaw;
    }

    public void setStateRaw(final String stateRaw) {
        this.stateRaw = stateRaw;
    }

    public void setTriggerRaw(final String triggerRaw) {
        this.triggerRaw = triggerRaw;
    }

    public Long getOldestSnapshotTimestamp() {
        return oldestSnapshotTimestamp;
    }

    public void setOldestSnapshotTimestamp(final Long oldestSnapshotTimestamp) {
        this.oldestSnapshotTimestamp = oldestSnapshotTimestamp;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isEnableSnapDiff() {
        return enableSnapDiff;
    }

    public void setEnableSnapDiff(final Boolean enableSnapDiff) {
        this.enableSnapDiff = enableSnapDiff;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getStorageServiceUUID() {
        return storageServiceUUID;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public void setStorageServiceUUID(final String storageServiceUUID) {
        this.storageServiceUUID = storageServiceUUID;        // JSD not here, later in relation table
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isManagedByService() {
        return isManagedByService;
    }

    public void setManagedByService(final Boolean managedByService) {
        isManagedByService = managedByService;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getMaxDataConstituentSize() {
        return maxDataConstituentSize;
    }

    public void setMaxDataConstituentSize(final Long maxDataConstituentSize) {
        this.maxDataConstituentSize = maxDataConstituentSize;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getMaxNamespaceConstituentSize() {
        return maxNamespaceConstituentSize;
    }

    public void setMaxNamespaceConstituentSize(final Long maxNamespaceConstituentSize) {
        this.maxNamespaceConstituentSize = maxNamespaceConstituentSize;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public ConstituentRole getConstituentRole() {
        return constituentRole;
    }

    public void setConstituentRole(final ConstituentRole constituentRole) {
        this.constituentRole = constituentRole;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getConstituentRoleRaw() {
        return constituentRoleRaw;
    }

    public void setConstituentRoleRaw(final String constituentRoleRaw) {
        this.constituentRoleRaw = constituentRoleRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public QosPolicyGroup.QosPolicyGroupKey getQosPolicyGroupKey() {
        return qosPolicyGroupKey;
    }

    public void setQosPolicyGroupKey(final QosPolicyGroup.QosPolicyGroupKey qosPolicyGroupKey) {
        this.qosPolicyGroupKey = qosPolicyGroupKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getUnixSecurityGroupID() {
        return groupID;
    }

    public void setUnixSecurityGroupID(final String groupID) {
        this.groupID = groupID;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getUnixSecurityPermissions() {
        return permissions;
    }

    public void setUnixSecurityPermissions(final String permissions) {
        this.permissions = permissions;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getUnixSecurityUserID() {
        return userID;
    }

    public void setUnixSecurityUserID(final String userID) {
        this.userID = userID;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getVolumeBlocksFootPrint() {
        return volumeBlocksFootPrint;
    }

    public void setVolumeBlocksFootPrint(final Long volumeBlocksFootPrint) {
        this.volumeBlocksFootPrint = volumeBlocksFootPrint;
    }

    public String getCachingPolicyRaw() { return cachingPolicyRaw; }

    public void setCachingPolicyRaw(String cachingPolicyRaw) { this.cachingPolicyRaw = cachingPolicyRaw; }

    public void setCachingPolicy(CachingPolicy cachingPolicy) { this.cachingPolicy = cachingPolicy; }

    public CachingPolicy getCachingPolicy() { return cachingPolicy; }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum State {
        NOT_MAPPED,
        ONLINE,
        RESTRICTED,
        OFFLINE,
        MIXED,
    }

    /**
     * Volume style derived by OCIE.  Combination of volume style and constituent status.
     */
    public static enum DerivedVolumeStyle {
        FLEXVOL,
        INFINIVOL,
        STRIPED,
        CONSTITUENT,
        UNKNOWN
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum VolumeStyle {
        NOT_MAPPED,
        FLEX("FLEXVOL"),
        STRIPED,
        INFINITEVOL("INFINIVOL");

        private final String xmlValue;

        private VolumeStyle() {
            this(null);
        }

        private VolumeStyle(final String xmlValue) {
            this.xmlValue = xmlValue;
        }

        public String toXML() {
            return (xmlValue == null ? name() : xmlValue);
        }

        public static VolumeStyle fromString(final String stringVal) {
            if (stringVal == null) {
                return NOT_MAPPED;
            }
            for (final VolumeStyle candidateStyle : VolumeStyle.values()) {
                if (candidateStyle.toXML().equalsIgnoreCase(stringVal)) {
                    return candidateStyle;
                }
            }
            return NOT_MAPPED;
        }


    }    // infinitevol

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum SecurityStyle {
        NOT_MAPPED,
        MIXED,
        NTFS,
        UNIX,
        UNIFIED, }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum HybridCacheEligibility {
        NOT_MAPPED,
        READ,
        READ_WRITE, }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum SpaceMgmtOptionTryFirst {
        NOT_MAPPED,
        VOLUME_GROW,
        SNAP_DELETE, }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum SpaceGuarantee {
        NOT_MAPPED,
        NONE,
        VOLUME,
        FILE, }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum BlockType {
        NOT_MAPPED,
        BIT_32("32_BIT"),
        BIT_64("64_BIT");

        private final String xmlValue;

        private BlockType() {
            this(null);
        }

        private BlockType(final String xmlValue) {
            this.xmlValue = xmlValue;
        }

        public String toXML() {
            return (xmlValue == null ? name() : xmlValue);
        }

        public static BlockType fromString(final String stringVal) {
            if (stringVal == null) {
                return NOT_MAPPED;
            }
            for (final BlockType candidateBlockType : BlockType.values()) {
                if (candidateBlockType.toXML().equalsIgnoreCase(stringVal)) {
                    return candidateBlockType;
                }
            }
            return NOT_MAPPED;
        }

        public static String toXML(final BlockType blockType) {
            return (blockType == null) ? null : blockType.toXML();
        }
    }

    private Long getOverWriteReserveAvail() {
        if (overwriteReserve == null || overwriteReserveUsed == null) {
            return null;
        }
        return overwriteReserve - overwriteReserveUsed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Commitment {
        NOT_MAPPED,
        TRY,
        DISRUPT,
        DESTROY,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum DeleteOrder {
        NOT_MAPPED,
        OLDEST_FIRST,
        NEWEST_FIRST,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum DeferDelete {
        NOT_MAPPED,
        SCHEDULED,
        USER_CREATED,
        PREFIX,
        NONE,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Trigger {
        NOT_MAPPED,
        VOLUME,
        SNAP_RESERVE,
        SPACE_RESERVE,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum DestroyList {
        NOT_MAPPED,
        LUN_CLONE,
        VOL_CLONE,
        CIFS_SHARE,
        FILE_CLONE,
        SFSR,
        NONE, }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum VolumeType {
        NOT_MAPPED,
        RW,
        LS,
        DP,
        DC,
        TMP,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum VolumeAutoSizeMode {
        NOT_MAPPED,
        OFF,
        GROW,
        GROW_SHRINK,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum VolumeSisStatus {
        NOT_MAPPED,
        IDLE,
        INITIALIZING,
        ACTIVE,
        UNDOING,
        PENDING,
        DOWNGRADING,
        DISABLED,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum VolumeSisState {
        NOT_MAPPED,
        DISABLED,
        ENABLED,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum VolumeSisType {
        NOT_MAPPED,
        REGULAR,
        SNAPVAULT,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum VolumeSisOpState {
        NOT_MAPPED,
        SUCCESS,
        FAILURE,
    }


    @SuppressWarnings({"UnusedDeclaration"})
    public static enum ConstituentRole {
        NOT_MAPPED,
        NAMESPACE,
        DATA,
        NS_MIRROR,
        OLS,
        UNKNOWN,
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum QuotaStatus {
        NOT_MAPPED,
        CORRUPT,
        INITIALIZING,
        OFF,
        ON,
        RESIZING,
        REVERTING,
        UPGRADING
    }

    public static enum CachingPolicy {
        NOT_MAPPED,
        NONE,
        AUTO,
        ALL
    }



    public QTree createQTree(final String uuID, final String volumeUUID) {
        return new QTree(this, uuID, volumeUUID);
    }

    public QTrees getQTrees() {
        return qtrees;
    }

    public LUN createLUN(final String uuID) {
        return new LUN(this, uuID);
    }

    public final LUNs getLUNs() {
        return luns;
    }

    public TreeQuota createTreeQuota(final QTree.QTreeKey qTreeKey, final TreeQuota.QuotaType quotaType, final String quotaTarget) {
        return new TreeQuota(this, qTreeKey, quotaType, quotaTarget);
    }

    public UserQuota createUserQuota(final QTree.QTreeKey qTreeKey, final TreeQuota.QuotaType quotaType, final String quotaTarget,
                                     final String userHash) {
        return new UserQuota(this, qTreeKey, quotaType, quotaTarget, userHash);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public final TreeQuotas getTreeQuotas() {
        return treeQuotas;
    }

    public UserQuotas getUserQuotas() {
        return userQuotas;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public QTrees getQtrees() {
        return qtrees;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public LUNs getLuns() {
        return luns;
    }


    protected boolean hasChildren() {
        return !qtrees.isEmpty() || !luns.isEmpty() || !treeQuotas.isEmpty() || !userQuotas.isEmpty();
    }

    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, "Qtrees", qtrees.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "LUNs", luns.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "TreeQuotas", treeQuotas.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "UserQuotas", userQuotas.values(), tabs);
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", volumeKey);
        // Server uses Instance UUID as the Key.
        // Performance and everything else refers to Volumes by instance UUID.
        // However, we still martial UUID from the "global" UUID.
        // This is intentional.
        addRequiredAttributeAndCRC(writer, "UUID", volumeKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", volumeName);
        addRequiredAttributeAndCRC(writer, "InstanceUUID", instanceUUID);
        addRequiredAttributeAndCRC(writer, "AggregateUUID", KeyBase.getOcieUUIDOrNull(aggregateKey));   // infinivols have no aggr
        addRequiredAttributeAndCRC(writer, "VolType", volumeType);
        addRequiredAttributeAndCRC(writer, "VolTypeRaw", volumeTypeRaw);
        addRequiredAttributeAndCRC(writer, "StorageServiceUUID", storageServiceUUID);
        addRequiredAttributeAndCRC(writer, "IsSnapDiffEnabled", enableSnapDiff);
        addRequiredAttributeAndCRC(writer, "IsManagedByService", isManagedByService);
        addRequiredAttributeAndCRC(writer, "MaxDataConstituentSize", maxDataConstituentSize);
        addRequiredAttributeAndCRC(writer, "MaxNamespaceConstituentSize", maxNamespaceConstituentSize);
        addRequiredAttributeAndCRC(writer, "ConstituentRole", constituentRole);
        addRequiredAttributeAndCRC(writer, "ConstituentRoleRaw", constituentRoleRaw);
        addRequiredAttributeAndCRC(writer, "Size", size);
        addRequiredAttributeAndCRC(writer, "SizeTotal", sizeTotal);
        addRequiredAttributeAndCRC(writer, "SizeAvail", sizeAvailable);
        addRequiredAttributeAndCRC(writer, "SizeUsed", sizeUsed);
        addRequiredAttributeAndCRC(writer, "IsVserverRoot", isVServerRoot);
        addRequiredAttributeAndCRC(writer, "State", state);
        addRequiredAttributeAndCRC(writer, "StateRaw", stateRaw);
        addRequiredAttributeAndCRC(writer, "JunctionPath", junctionPath);
        addRequiredAttributeAndCRC(writer, "IsJunctionActive", isJunctionActive);
        addRequiredAttributeAndCRC(writer, "QosPolicyGroupUUID", KeyBase.getOcieUUIDOrNull(qosPolicyGroupKey));
        addRequiredAttributeAndCRC(writer, "IsConstituent", isConstituent);
        addRequiredAttributeAndCRC(writer, "Style", volumeStyle.toXML());
        addRequiredAttributeAndCRC(writer, "StyleRaw", volumeStyleRaw);
        addRequiredAttributeAndCRC(writer, "DerivedStyle", getDerivedVolumeStyle());
        addRequiredAttributeAndCRC(writer, "IsSisVolume", isSISVolume);
        addRequiredAttributeAndCRC(writer, "IsDataProtectionMirror", isDataProtectionMirror);
        addRequiredAttributeAndCRC(writer, "IsLoadSharingMirror", isLoadSharingMirror);
        addRequiredAttributeAndCRC(writer, "IsMoveMirror", isMoveMirror);
        addRequiredAttributeAndCRC(writer, "IsReplicaVolume", isReplicaVolume);
        addRequiredAttributeAndCRC(writer, "IsSpaceGuaranteeEnabled", isSpaceGuarantee);
        addRequiredAttributeAndCRC(writer, "VMAlignSector", vmAlignSector);
        addRequiredAttributeAndCRC(writer, "VMAlignSuffix", vmAlignSuffix);
        addRequiredAttributeAndCRC(writer, "OverwriteReserve", overwriteReserve);
        addRequiredAttributeAndCRC(writer, "OverwriteReserveRequired", overwriteReserveRequired);
        addRequiredAttributeAndCRC(writer, "OverwriteReserveUsed", overwriteReserveUsed);
        addRequiredAttributeAndCRC(writer, "OverwriteReserveActualUsed", overwriteReserveActualUsed);
        addRequiredAttributeAndCRC(writer, "OverwriteReserveAvail", getOverWriteReserveAvail());
        addRequiredAttributeAndCRC(writer, "SnapshotReserveSize", snapshotReserveSize);
        addRequiredAttributeAndCRC(writer, "PercentageSnapshotReserve", percentSnapshotReserved);
        addRequiredAttributeAndCRC(writer, "PercentageSnapshotReserveUsed", percentSnapshotReserveUsed);
        addRequiredAttributeAndCRC(writer, "SizeAvailableForSnapshot", sizeAvailableForSnapshots);
        addRequiredAttributeAndCRC(writer, "PercentageFractionalReserve", percentFractionalReserve);
        addRequiredAttributeAndCRC(writer, "ExportPolicyUUID", KeyBase.getOcieUUIDOrNull(exportPolicyKey));
        addRequiredAttributeAndCRC(writer, "DeduplicationSpaceSaved", deduplicationSpaceSaved);
        addRequiredAttributeAndCRC(writer, "CompressionSpaceSaved", compressionSpaceSaved);
        addRequiredAttributeAndCRC(writer, "PercentageDeduplicationSpaceSaved", percentageDeduplicationSpaceSaved);
        addRequiredAttributeAndCRC(writer, "PercentageCompressionSpaceSaved", percentageCompressionSpaceSaved);
        addRequiredAttributeAndCRC(writer, "SecurityStyle", securityStyle);
        addRequiredAttributeAndCRC(writer, "SecurityStyleRaw", securityStyleRaw);
        addRequiredAttributeAndCRC(writer, "SecurityGroupID", groupID);
        addRequiredAttributeAndCRC(writer, "SecurityUserID", userID);
        addRequiredAttributeAndCRC(writer, "SecurityPermissions", permissions);
        addRequiredAttributeAndCRC(writer, "HybridCacheEligibility", hybridCacheEligibility);
        addRequiredAttributeAndCRC(writer, "HybridCacheEligibilityRaw", hybridCacheEligibilityRaw);
        addRequiredAttributeAndCRC(writer, "SpaceMgmtOptionTryFirst", spaceMgmtTryFirst);
        addRequiredAttributeAndCRC(writer, "SpaceMgmtOptionTryFirstRaw", spaceMgmtTryFirstRaw);
        addRequiredAttributeAndCRC(writer, "SizeUsedBySnapshots", sizeUsedBySnapshots);
        addRequiredAttributeAndCRC(writer, "SpaceGuarantee", spaceGuarantee);
        addRequiredAttributeAndCRC(writer, "SpaceGuaranteeRaw", spaceGuaranteeRaw);
        addRequiredAttributeAndCRC(writer, "IsI2pEnabled", i2pEnabled);
        addRequiredAttributeAndCRC(writer, "IsAutoSnapshotsEnabled", autoSnapshotsEnabled);
        addRequiredAttributeAndCRC(writer, "IsSnapDirAccessEnabled", snapdirAccessEnabled);
        addRequiredAttributeAndCRC(writer, "IsSnapshotCloneDependencyEnabled", snapshotCloneDependencyEnabled);
        addRequiredAttributeAndCRC(writer, "SnapshotCount", snapshotCount);
        addRequiredAttributeAndCRC(writer, "SnapshotPolicyUUID", KeyBase.getOcieUUIDOrNull(snapshotPolicyKey));
        addRequiredAttributeAndCRC(writer, "AutoSizeMode", autoSizeMode);
        addRequiredAttributeAndCRC(writer, "AutoSizeModeRaw", autoSizeModeRaw);
        addRequiredAttributeAndCRC(writer, "AutoSizeMaximumSize", maxAutosizeSize);
        addRequiredAttributeAndCRC(writer, "AutoSizeIncrementSize", autosizeIncrementSize);
        addRequiredAttributeAndCRC(writer, "IsAtimeUpdateEnabled", isAtimeUpdateEnabled);
        addRequiredAttributeAndCRC(writer, "InodeBlockType", BlockType.toXML(blockType));
        addRequiredAttributeAndCRC(writer, "InodeBlockTypeRaw", blockTypeRaw);
        addRequiredAttributeAndCRC(writer, "InodeFilesTotal", inodeFilesTotal);
        addRequiredAttributeAndCRC(writer, "InodeFilesUsed", inodeFilesUsed);
        addRequiredAttributeAndCRC(writer, "LanguageCode", languageCode);
        addRequiredAttributeAndCRC(writer, "IsCreateUcodeEnabled", isCreateUcodeEnabled);
        addRequiredAttributeAndCRC(writer, "IsConvertUcodeEnabled", isConvertUcodeEnabled);
        addRequiredAttributeAndCRC(writer, "CloneChildCount", cloneChildCount);
        addRequiredAttributeAndCRC(writer, "CloneSplitEstimate", cloneSplitEstimate);
        addRequiredAttributeAndCRC(writer, "IsSnapshotAutoDeleteEnabled", isAutoDeleteEnabled);
        addRequiredAttributeAndCRC(writer, "SnapshotAutoDeleteCommitment", commitment);
        addRequiredAttributeAndCRC(writer, "SnapshotAutoDeleteCommitmentRaw", commitmentRaw);
        addRequiredAttributeAndCRC(writer, "SnapshotAutoDeleteDeleteOrder", deleteOrder);
        addRequiredAttributeAndCRC(writer, "SnapshotAutoDeleteDeleteOrderRaw", deleteOrderRaw);
        addRequiredAttributeAndCRC(writer, "SnapshotAutoDeleteDeferDelete", deferDelete);
        addRequiredAttributeAndCRC(writer, "SnapshotAutoDeleteDeferDeleteRaw", deferDeleteRaw);
        addRequiredAttributeAndCRC(writer, "SnapshotAutoDeleteTargetFreeSpace", targetFreeSpace);
        addRequiredAttributeAndCRC(writer, "SnapshotAutoDeleteTrigger", trigger);
        addRequiredAttributeAndCRC(writer, "SnapshotAutoDeleteTriggerRaw", triggerRaw);
        addRequiredAttributeAndCRC(writer, "SnapshotAutoDeletePrefix", prefix);
        addRequiredAttributeAndCRC(writer, "SnapshotAutoDeleteDestroyList", destroyListRaw);
        addRequiredAttributeAndCRC(writer, "OldestSnapshotTimestamp", oldestSnapshotTimestamp);
        addRequiredAttributeAndCRC(writer, "SisStatus", sisStatus);
        addRequiredAttributeAndCRC(writer, "SisStatusRaw", sisStatusRaw);
        addRequiredAttributeAndCRC(writer, "SisState", sisState);
        addRequiredAttributeAndCRC(writer, "SisStateRaw", sisStateRaw);
        addRequiredAttributeAndCRC(writer, "SisType", sisType);
        addRequiredAttributeAndCRC(writer, "SisTypeRaw", sisTypeRaw);
        addRequiredAttributeAndCRC(writer, "SisProgress", progress);
        addRequiredAttributeAndCRC(writer, "SisSchedule", schedule);
        addRequiredAttributeAndCRC(writer, "SisLastOpBeginTimestamp", lastOpBeginTS);
        addRequiredAttributeAndCRC(writer, "SisLastOpEndTimestamp", lastOpEndTS);
        addRequiredAttributeAndCRC(writer, "SisLastOpError", lastOpError);
        addRequiredAttributeAndCRC(writer, "SisLastOpState", lastSisOpState);
        addRequiredAttributeAndCRC(writer, "SisLastOpStateRaw", lastSisOpStateRaw);
        addRequiredAttributeAndCRC(writer, "SisLastOpSize", lastSisOpSize);
        addRequiredAttributeAndCRC(writer, "SisPolicyUUID", KeyBase.getOcieUUIDOrNull(sisPolicyKey));
        addRequiredAttributeAndCRC(writer, "IsSisCompressionEnabled", isCompressionEnabled);
        addRequiredAttributeAndCRC(writer, "IsSisInlineCompressionEnabled", isInlineCompressionEnabled);
        addRequiredAttributeAndCRC(writer, "SizeUsedPercent", sizeUsedPercent);
        addRequiredAttributeAndCRC(writer, "SizeAvailPercent", getSizeAvailPercent());
        addRequiredAttributeAndCRC(writer, "QuotaCommitted", quotaCommit);
        addRequiredAttributeAndCRC(writer, "QuotaOverCommitted", getQuotaOvercommit());
        addRequiredAttributeAndCRC(writer, "QuotaStatus", quotaStatus);
        addRequiredAttributeAndCRC(writer, "QuotaStatusRaw", quotaStatusRaw);
        addRequiredAttributeAndCRC(writer, "FlexCacheMinReserve", flexCacheMinReserve);
        addRequiredAttributeAndCRC(writer, "VolumeBlocksFootPrint", volumeBlocksFootPrint);
        addRequiredAttributeAndCRC(writer, "CachingPolicy", cachingPolicy);
        addRequiredAttributeAndCRC(writer, "CachingPolicyRaw", cachingPolicyRaw);
    }
}
