package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("volume")
@OCIEResourceType("volume")
public class VolumeFlexCache extends BaseCRCXMLNode<VServer> {

    public final static class FlexCacheKey extends KeyBase<FlexCacheKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "flexcache-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.flexcache-info.cache-volume"
            )
            volume_uuid
        }

        public FlexCacheKey(final String clusterUUID, final String volumeInstanceUUID) {
            super("flex_cache", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.volume_uuid, volumeInstanceUUID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "volume_uuid")
        public String getOcieUUID() {
            return getID(Field.volume_uuid);
        }
    }

    // flexcache-get-iter
    private final FlexCacheKey flexCacheKey;

    @OCIEZapiSource(
        sourceZapi = "flexcache-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.flexcache-info.origin-volume"
    )
    @OCIEValue(
        ocieColumn = "flexCacheOriginId"
    )
    private Volume.VolumeKey flexCacheOriginKey;

    VolumeFlexCache(final VServer parent, final String volumeInstanceUUID) {
        super(parent, "VolumeFlexCache");
        this.flexCacheKey = new FlexCacheKey(parent.getKey().getID(VServer.VServerKey.Field.cluster_uuid), volumeInstanceUUID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public FlexCacheKey getKey() {
        return flexCacheKey;
    }

    public void setFlexCacheOriginKey(final Volume.VolumeKey flexCacheOriginKey) {
        this.flexCacheOriginKey = flexCacheOriginKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Volume.VolumeKey getFlexCacheOriginKey() {
        return flexCacheOriginKey;
    }


    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        //addRequiredAttributeAndCRC(writer, "ResourceKey", flexCacheKey);
        addRequiredAttributeAndCRC(writer, "VolumeFlexCacheUUID", flexCacheKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "VolumeOriginUUID", flexCacheOriginKey.getVolumeInstanceUUID());  // JSD Or other volume uuid?
    }
}
