package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("volume_move_job")
@OCIEResourceType("volume_move")
public class VolumeMove extends BaseCRCXMLNode<VServer> {

    public final static class VolumeMoveKey extends KeyBase<VolumeMoveKey.Field> {

        public static enum Field {   // TODO: volume move has no vserver but volume does...
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "volume-move-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.volume-move-info.job-uuid"
            )
            uuid
        }

        public VolumeMoveKey(final String clusterUUID, final String jobUUID) {
            super("volume_move", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.uuid, jobUUID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getID(Field.uuid);
        }
    }


    private final VolumeMoveKey volumeMoveKey;

    @OCIEZapiSource(
        sourceZapi = "volume-move-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-move-info.volume"
    )
    @OCIEValue(
        ocieColumn = "volumeId"
    )
    private Volume.VolumeKey volumeKey;

    @OCIEZapiSource(
        sourceZapi = "volume-move-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-move-info.source-aggregate"
    )
    @OCIEValue(
        ocieColumn = "sourceAggregateId"
    )
    private Aggregate.AggregateKey sourceAggregateKey;

    @OCIEZapiSource(
            sourceZapi = "volume-move-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.volume-move-info.destination-aggregate"
    )
    @OCIEValue(
            ocieColumn = "destinationAggregateId"
    )
    private Aggregate.AggregateKey destinationAggregateKey;

    @OCIEZapiSource(
            sourceZapi = "volume-move-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.volume-move-info.job-id"
    )
    @OCIEValue(
            ocieColumn = "jobIdentifier"
    )
    private Long jobIdentifier;

    @OCIEZapiSource(
            sourceZapi = "volume-move-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.volume-move-info.start-timestamp"
    )
    @OCIEValue(
            ocieColumn = "startTime"
    )
    private Long startTime;

    @OCIEZapiSource(
            sourceZapi = "volume-move-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.volume-move-info.actual-completion-timestamp"
    )
    @OCIEValue(
            ocieColumn = "completionTime"
    )
    private Long completionTime;

    @OCIEZapiSource(
            sourceZapi = "volume-move-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.volume-move-info.phase"
    )
    @OCIEValue(
            ocieColumn = "phase"
    )
    private Phase phase;

    @OCIEZapiSource(
            sourceZapi = "volume-move-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.volume-move-info.phase"
    )
    @OCIEValue(
            ocieColumn = "phaseRaw"
    )
    private String phaseRaw;

    @OCIEZapiSource(
            sourceZapi = "volume-move-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.volume-move-info.state"
    )
    @OCIEValue(
            ocieColumn = "state"
    )
    private State state;

    @OCIEZapiSource(
            sourceZapi = "volume-move-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.volume-move-info.state"
    )
    @OCIEValue(
            ocieColumn = "stateRaw"
    )
    private String stateRaw;

    @OCIEZapiSource(
            sourceZapi = "volume-move-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.volume-move-info.details"
    )
    @OCIEValue(
            ocieColumn = "stateRaw"
    )
    private String details;

    @OCIEZapiSource(
            sourceZapi = "volume-move-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.volume-move-info.percent-complete"
    )
    @OCIEValue(
            ocieColumn = "percentComplete"
    )
    private Integer percentComplete;

    @OCIEZapiSource(
            sourceZapi = "volume-move-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.volume-move-info.estimated-completion-time"
    )
    @OCIEValue(
            ocieColumn = "estimatedCompletionTime"
    )
    private Long estimatedCompletionTime;

    VolumeMove(final VServer parent, final String jobUUID) {
        super(parent, "VolumeMoveJob");
        this.volumeMoveKey = new VolumeMoveKey(parent.getKey().getID(VServer.VServerKey.Field.cluster_uuid), jobUUID);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Phase {
        NOT_MAPPED,
        QUEUED,
        INITIALIZING,
        REPLICATING,
        CUTOVER,
        CUTOVER_HARD_DEFERRED,
        CUTOVER_SOFT_DEFERRED,
        COMPLETED,
        CLEANING_UP,
        FAILED,
        RESTARTING, }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum State {
        NOT_MAPPED,
        HEALTHY,
        DONE,
        FAILED,
        WARNING,
        ALERT, }

    @Override
    @SuppressWarnings({"unchecked"})
    public VolumeMoveKey getKey() {
        return this.volumeMoveKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getCompletionTime() {
        return completionTime;
    }

    public void setCompletionTime(final Long completionTime) {
        this.completionTime = completionTime;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Aggregate.AggregateKey getDestinationAggregateKey() {
        return destinationAggregateKey;
    }

    public void setDestinationAggregateKey(final Aggregate.AggregateKey destinationAggregateKey) {
        this.destinationAggregateKey = destinationAggregateKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getDetails() {
        return details;
    }

    public void setDetails(final String details) {
        this.details = details;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getJobIdentifier() {
        return jobIdentifier;
    }

    public void setJobIdentifier(final Long jobIdentifier) {
        this.jobIdentifier = jobIdentifier;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getPercentComplete() {
        return percentComplete;
    }

    public void setPercentComplete(final Integer percentComplete) {
        this.percentComplete = percentComplete;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getPhaseRaw() {
        return phaseRaw;
    }

    public void setPhaseRaw(final String phaseRaw) {
        this.phaseRaw = phaseRaw;
    }

    public void setPhaseEnum(final Phase phase) {
        this.phase = phase;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Aggregate.AggregateKey getSourceAggregateKey() {
        return sourceAggregateKey;
    }

    public void setSourceAggregateKey(final Aggregate.AggregateKey sourceAggregateKey) {
        this.sourceAggregateKey = sourceAggregateKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getStartTime() {
        return startTime;
    }

    public void setStartTime(final Long startTime) {
        this.startTime = startTime;
    }

    @SuppressWarnings("UnusedDeclaration")
    public State getState() {
        return state;
    }

    public void setState(final State state) {
        this.state = state;
    }

    @SuppressWarnings("UnusedDeclaration")
    public String getStateRaw() {
        return stateRaw;
    }

    public void setEstimatedCompletionTime(final Long estimatedCompletionTime) {
        this.estimatedCompletionTime = estimatedCompletionTime;
    }

    public void setStateRaw(final String stateRaw) {
        this.stateRaw = stateRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Volume.VolumeKey getVolumeKey() {
        return volumeKey;
    }

    public void setVolumeKey(final Volume.VolumeKey volumeKey) {
        this.volumeKey = volumeKey;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", volumeMoveKey);
        addRequiredAttributeAndCRC(writer, "UUID", volumeMoveKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "VolumeUUID", volumeKey.getVolumeInstanceUUID());      // JSD should ref inst UUID?
        addRequiredAttributeAndCRC(writer, "SourceAggregateUUID", KeyBase.getOcieUUIDOrNull(sourceAggregateKey));
        addRequiredAttributeAndCRC(writer, "DestinationAggregateUUID", KeyBase.getOcieUUIDOrNull(destinationAggregateKey));
        addRequiredAttributeAndCRC(writer, "JobIdentifier", jobIdentifier);
        addRequiredAttributeAndCRC(writer, "StartTime", startTime);
        addRequiredAttributeAndCRC(writer, "CompletionTime", completionTime);
        addRequiredAttributeAndCRC(writer, "Phase", phase);
        addRequiredAttributeAndCRC(writer, "PhaseRaw", phaseRaw);
        addRequiredAttributeAndCRC(writer, "State", state);
        addRequiredAttributeAndCRC(writer, "StateRaw", stateRaw);
        addRequiredAttributeAndCRC(writer, "Details", details);
        addRequiredAttributeAndCRC(writer, "PercentComplete", percentComplete);
        addRequiredAttributeAndCRC(writer, "EstimatedCompletionTime", estimatedCompletionTime);
    }
}
