package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEValue;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("volume")
@OCIEResourceType("volume")
public class VolumeSelfRelationship extends BaseCRCXMLNode<VServer> {

    public final static class RelationshipKey extends KeyBase<RelationshipKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            aggregate_uuid,

            @OCIEZapiSource(
                sourceZapi = "volume-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.uuid"
            )
            volume_uuid,
        }

        public RelationshipKey(final String clusterUUID, final String volumeUUID) {
            super("volume_self_relationship", new EnumMap<Field, String>(Field.class) {{
                put(Field.aggregate_uuid, clusterUUID);
                put(Field.volume_uuid, volumeUUID);
            }});
        }

        public String getOcieUUID() {
            return getFQID();
        }
    }

    private final RelationshipKey relationshipKey;

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.junction-parent-name"
    )
    @OCIEValue(
        ocieColumn = "junctionParentId"
    )
    private String volumeJunctionParentUUID;

    @OCIEZapiSource(
        sourceZapi = "volume-clone-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-clone-info.parent-volume"
    )
    @OCIEValue(
        ocieColumn = "cloneParentId"
    )
    private String volumeCloneParentUUID;

    VolumeSelfRelationship(final VServer parent, final String volumeUUID) {
        super(parent, "VolumeSelfRelationship");
        final String clusterUUID = parent.getParent().getClusterUUID();
        this.relationshipKey = new RelationshipKey(clusterUUID, volumeUUID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public RelationshipKey getKey() {
        return this.relationshipKey;
    }

    public String getVolumeJunctionParentUUID() {
        return volumeJunctionParentUUID;
    }

    public void setVolumeJunctionParentUUID(final String volumeJunctionParentUUID) {
        this.volumeJunctionParentUUID = volumeJunctionParentUUID;
    }

    public String getVolumeCloneParentUUID() {
        return volumeCloneParentUUID;
    }

    public void setVolumeCloneParentUUID(final String volumeCloneParentUUID) {
        this.volumeCloneParentUUID = volumeCloneParentUUID;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "VolumeUUID", relationshipKey.getID(RelationshipKey.Field.volume_uuid));
        addRequiredAttributeAndCRC(writer, "VolumeJunctionParentUUID", getVolumeJunctionParentUUID());
        addRequiredAttributeAndCRC(writer, "VolumeCloneParentUUID", getVolumeCloneParentUUID());
    }
}
