package com.onaro.sanscreen.acquisition.netappstorageperformance.datasource;

import org.apache.log4j.Layout;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.RollingFileAppender;

import java.io.IOException;

public class NetAppStoragePerformanceLogger {
    private static final String performanceLogFile = "perf.log";
    private final static Logger perfLogger;

    static {
        try {
            perfLogger = Logger.getLogger(NetAppStoragePerformanceLogger.class);

            // Set the appenders for this logger. This would be a special file dedicated only for performance entries
            perfLogger.removeAllAppenders();

            final Layout layout = new PatternLayout(PatternLayout.DEFAULT_CONVERSION_PATTERN);
            final RollingFileAppender appender = new RollingFileAppender(layout, performanceLogFile, true);
            appender.setMaxFileSize("2000KB");
            appender.setMaxBackupIndex(1);

            perfLogger.addAppender(appender);
        }
        catch (IOException e) {
            throw new RuntimeException("Failed to create performance log file", e);
        }
    }

    private final String logPrefix;

    public NetAppStoragePerformanceLogger(final String logPrefix) {
        this.logPrefix = logPrefix;
    }

    /**
     * This logs an entry for the specified data source in the special performance log file
     *
     * @param portWWN
     * @param values A list of key/value pairs for this performance run. The keys will be vendor specific to allow the
     *               user maximum details regarding the entry.
     */
    public void logPerformance(final String portWWN, final String... values) {
        final StringBuilder sb = new StringBuilder(1000);

        if (values.length % 2 == 0)
            throw new IllegalArgumentException("Values must come in pairs!");

        sb.append("[").append(logPrefix).append(":").append(portWWN).append("] ");

        for (int i = 0; i < values.length; i += 2)
            sb.append("{").append(values[i]).append(",").append(values[i + 1]).append("}");

        perfLogger.info(sb.toString());
    }
}
