package com.onaro.sanscreen.acquisition.netappstorageperformance.datasource;

import com.netapp.oci.netapp.performance.aggregate.interfaces.data.AggregateMeasurements;
import com.netapp.oci.netapp.performance.aggregate.interfaces.data.AggregateRawMeasurements;
import com.netapp.oci.netapp.performance.aggregate.interfaces.data.WaflHyaAggrMeasurements;
import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverMeasurements;
import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverRawMeasurements;
import com.netapp.oci.netapp.performance.cluster.interfaces.data.ClusterMeasurements;
import com.netapp.oci.netapp.performance.disk.interfaces.data.DiskMeasurements;
import com.netapp.oci.netapp.performance.disk.interfaces.data.DiskRawMeasurements;
import com.netapp.oci.netapp.performance.ext_cache_obj.interfaces.data.ExtCacheObjMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifRawMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverRawMeasurements;
import com.netapp.oci.netapp.performance.fcpport.interfaces.data.FcpPortMeasurements;
import com.netapp.oci.netapp.performance.fcpport.interfaces.data.FcpPortRawMeasurements;
import com.netapp.oci.netapp.performance.hostadapter.interfaces.data.HostadapterMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifRawMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifVserverMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifVserverRawMeasurements;
import com.netapp.oci.netapp.performance.lun.interfaces.data.LunMeasurements;
import com.netapp.oci.netapp.performance.lun.interfaces.data.LunRawMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifRawMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverRawMeasurements;
import com.netapp.oci.netapp.performance.networkport.interfaces.data.NetworkPortMeasurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.*;
import com.netapp.oci.netapp.performance.nic.interfaces.data.NicMeasurements;
import com.netapp.oci.netapp.performance.nic.interfaces.data.NicRawMeasurements;
import com.netapp.oci.netapp.performance.node.interfaces.data.*;
import com.netapp.oci.netapp.performance.processor.interfaces.data.ProcessorMeasurements;
import com.netapp.oci.netapp.performance.qosservicecenter.interfaces.data.QosServiceCenterMeasurements;
import com.netapp.oci.netapp.performance.qosworkload.interfaces.data.QosVolumeWorkloadMeasurements;
import com.netapp.oci.netapp.performance.qosworkloaddblade.interfaces.data.QosWorkloadQueueDBladeMeasurements;
import com.netapp.oci.netapp.performance.qosworkloaddetail.interfaces.data.QosWorkloadDetailMeasurements;
import com.netapp.oci.netapp.performance.qosworkloadnblade.interfaces.data.QosWorkloadQueueNBladeMeasurements;
import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeMeasurements;
import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeRawMeasurements;
import com.netapp.oci.netapp.performance.volumenode.interfaces.data.VolumeNodeMeasurements;
import com.netapp.oci.netapp.performance.vserver.interfaces.data.VolumeVserverMeasurements;
import com.netapp.oci.netapp.performance.vserver.interfaces.data.VserverMeasurements;
import com.netapp.oci.netapp.performance.wafl.interfaces.data.WaflMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer;

import java.util.Collection;

/**
 * Interface defining what a performance data source can report on.
 */
public interface NetAppStoragePerformanceUpdatesListener {

    /**
     * Called when updating aggregate data.
     *
     * @param aPackage
     * @param clusterKey
     * @param pollStartTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */ 	
	public boolean reportAggregatePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<AggregateMeasurements>> samples, final long leaseTime);
	
    /**
     * Called when updating aggregate data.
     *
     * @param aPackage
     * @param clusterKey
     * @param leaseTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */ 	
	public boolean reportAggregateRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<AggregateRawMeasurements>> samples, final long leaseTime);

    /**
     * Called when updating raw cifs:vserver data.
     *
     * @param aPackage
     * @param clusterKey
     * @param leaseTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */ 	
	public boolean reportCifsVserverRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<CifsVserverRawMeasurements>> samples, final long leaseTime);

    /**
     * Called when updating cifs:vserver data.
     *
     * @param aPackage
     * @param clusterKey
     * @param leaseTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */     
    public boolean reportCifsVserverPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<CifsVserverMeasurements>> samples, final long leaseTime);

    /**
     * Called when updating raw lif:vserver data.
     *
     * @param aPackage
     * @param clusterKey
     * @param leaseTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */ 	
	public boolean reportLifVserverRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<NetworkLifVserverRawMeasurements>> samples, final long leaseTime);
	
	/**
     * Called when updating lif:vserver data.
     *
     * @param aPackage
     * @param clusterKey
     * @param leaseTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */     
    public boolean reportLifVserverPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<NetworkLifVserverMeasurements>> samples, final long leaseTime);
    
    /**
     * Called when updating nfsv3 data.
     *
     * @param aPackage
     * @param clusterKey
     * @param leaseTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */ 	
	public boolean reportNfsv3Performance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<Nfsv3Measurements>> samples, final long leaseTime);
	
	/**
     * Called when updating nfsv3 data.
     *
     * @param aPackage
     * @param clusterKey
     * @param leaseTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */ 	
	public boolean reportNfsv3RawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<Nfsv3RawMeasurements>> samples, final long leaseTime);
	
	/**
     * Called when updating nfsv4 data.
     *
     * @param aPackage
     * @param clusterKey
     * @param leaseTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */ 	
	public boolean reportNfsv4Performance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<Nfsv4Measurements>> samples, final long leaseTime);
	
    /**
     * Called when updating nfsv4 data.
     *
     * @param aPackage
     * @param clusterKey
     * @param leaseTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */ 	
	public boolean reportNfsv4RawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<Nfsv4RawMeasurements>> samples, final long leaseTime);

	 /**
     * Called when updating nfsv41 data.
     *
     * @param aPackage
     * @param clusterKey
     * @param leaseTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */ 	
	public boolean reportNfsv41Performance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<Nfsv41Measurements>> samples, final long leaseTime);
	
    /**
     * Called when updating nfsv41 data.
     *
     * @param aPackage
     * @param clusterKey
     * @param leaseTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */ 	
	public boolean reportNfsv41RawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<Nfsv41RawMeasurements>> samples, final long leaseTime);

    /**
     * Called when updating nfs data.
     *
     * @param aPackage
     * @param clusterKey
     * @param pollStartTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */

    public boolean reportNfsPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<NfsMeasurements>> samples, final long leaseTime);

    /**
     * Called when updating vserver data.
     *
     * @param aPackage
     * @param clusterKey
     * @param pollStartTime
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportVserverPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<VserverMeasurements>> samples, final long leaseTime);

    /**
     * Called when updating disk data.
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */ 
    public boolean reportDiskPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<DiskMeasurements>> samples, long leaseTime);
    
    /**
     * Called when updating disk raw data
     * 
     * @param aPackage
     * @param clusterKey
     * @param pollStartTime
     * @param samples
     * @param leaseTime
     * @return
     */
    public boolean reportDiskRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long pollStartTime, final Collection<RawSample<DiskRawMeasurements>> samples,long leaseTime);

    /**
     * Called when updating volume data.
     *
     * @param aPackage
     * @param clusterKey
     * @param vServerKey
     * @param time
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportVolumePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final VServer.VServerKey vServerKey, final long time, final Collection<RawSample<VolumeMeasurements>> samples);

    /**
     * Called when updating volume data.
     *
     * @param aPackage
     * @param clusterKey
     * @param vServerKey
     * @param time
     * @param samples The performance data to report
     * @param leaseTime the time for datasource to be resgister in the in the registry (in milliseconds)
     *
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportVolumePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final VServer.VServerKey vServerKey, final long time, final Collection<RawSample<VolumeMeasurements>> samples, long leaseTime);

    /**
     * Called when updating volume raw data.
     *
     * @param aPackage
     * @param clusterKey
     * @param vServerKey
     * @param time
     * @param samples The performance data to report
     * @param leaseTime the time for datasource to be resgister in the in the registry (in milliseconds)
     *
     * @return true if report went all the way to the server, false otherwise
     */    
    public boolean reportVolumeRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final VServer.VServerKey vServerKey, final long time, final Collection<RawSample<VolumeRawMeasurements>> samples, long leaseTime);
    
    /**
     * Called when updating volume:node data.
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportVolumeNodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<VolumeNodeMeasurements>> samples);

    /**
     * Called when updating volume:node data.
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples The performance data to report
     * @param leaseTime the time for datasource to be resgister in the in the registry (in milliseconds)
     *
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportVolumeNodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<VolumeNodeMeasurements>> samples, long leaseTime);


    /**
     *
     * @param aPackage
     * @param clusterKey
     * @param vServerKey
     * @param time
     * @param samples
     * @param leaseTime
     * @return
     */
    public boolean reportLunPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final VServer.VServerKey vServerKey, final long time, final Collection<RawSample<LunMeasurements>> samples, final long leaseTime);

    /**
    *
    * @param aPackage
    * @param clusterKey
    * @param vServerKey
    * @param time
    * @param samples
    * @param leaseTime
    * @return
    */
   public boolean reportLunRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final VServer.VServerKey vServerKey, final long time, final Collection<RawSample<LunRawMeasurements>> samples, final long leaseTime);

    /**
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples
     * @return
     */
    public boolean reportNodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<NodeMeasurements>> samples);

    /**
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples
     * @param leaseTime
     * @return
     */
    public boolean reportNodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<NodeMeasurements>> samples, final long leaseTime);
    
    public boolean reportNodeRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<NodeRawMeasurements>> samples, final long leaseTime);

    /**
     * FcpLif Performance Statistics
     */
    public boolean reportFcpLifPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final long time, final Collection<RawSample<FcpLifMeasurements>> samples, final long leaseTime);
    public boolean reportFcpLifRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final long time, final Collection<RawSample<FcpLifRawMeasurements>> samples, final long leaseTime);

    public boolean reportFcpLifVserverPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<FcpLifVserverMeasurements>> samples, final long leaseTime);
    public boolean reportFcpLifVserverRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<FcpLifVserverRawMeasurements>> samples, final long leaseTime);

    public boolean reportFcpPortPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<FcpPortMeasurements>> samples, final long leaseTime);

    public boolean reportFcpPortRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<FcpPortRawMeasurements>> samples, final long leaseTime);
    
    public boolean reportIpPortPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<NetworkPortMeasurements>> samples, final long leaseTime);

    /**
     * Called when updating nic raw data.
     * 
     * @param aPackage
     * @param key
     * @param time
     * @param samples
     * @param leaseTime
     * 
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportNicRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<NicRawMeasurements>> samples, final long leaseTime);
    
    /**
     * Called when updating nic common data.
     * 
     * @param aPackage
     * @param key
     * @param time
     * @param samples
     * @param leaseTime
     * 
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportNicPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<NicMeasurements>> samples, final long leaseTime);
    
    public boolean reportVolumeVServerPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<VolumeVserverMeasurements>> samples, final long leaseTime);

    public boolean reportLifPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final long time, final Collection<RawSample<NetworkLifMeasurements>> samples, final long leaseTime);

    public boolean reportLifRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final long time, final Collection<RawSample<NetworkLifRawMeasurements>> samples, final long leaseTime);
    
    public boolean reportIscsiLifPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final long time, final Collection<RawSample<IscsiLifMeasurements>> samples, final long leaseTime);
    
    public boolean reportIscsiLifRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final long time, final Collection<RawSample<IscsiLifRawMeasurements>> samples, final long leaseTime);

    public boolean reportIscsiLifVserverPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<IscsiLifVserverMeasurements>> samples, final long leaseTime);
    
    public boolean reportIscsiLifVserverRawPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<IscsiLifVserverRawMeasurements>> samples, final long leaseTime);
    
    public boolean reportProcessorPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<ProcessorMeasurements>> sample, final long leaseTime);
    
    /**
     * Called when updating workload data.
     *
     * @param aPackage
     * @param vServerKey
     * @param time
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportQosVolumeWorkloadPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey vServerKey, final long time, final Collection<RawSample<QosVolumeWorkloadMeasurements>> samples);

    /**
     * Called when updating workload data.
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples The performance data to report
     * @param leaseTime the time for datasource to be resgister in the in the registry (in milliseconds)
     *
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportQosVolumeWorkloadPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<QosVolumeWorkloadMeasurements>> samples, long leaseTime);

    /**
     * Called when updating workload detail data.
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportQosWorkloadDetailPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<QosWorkloadDetailMeasurements>> samples);

    /**
     * Called when updating workload detail data.
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples The performance data to report
     * @param leaseTime the time for datasource to be resgister in the in the registry (in milliseconds)
     *
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportQosWorkloadDetailPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<QosWorkloadDetailMeasurements>> samples, long leaseTime);

    
    
    /**
     * Called when updating workload data.
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples    The performance data to report
     * @param leaseTime  the time for datasource to be resgister in the in the registry (in milliseconds)
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportQosWorkloadQueueDBladePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<QosWorkloadQueueDBladeMeasurements>> samples, long leaseTime);

   /**
     * Called when updating workload data.
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples    The performance data to report
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportQosWorkloadQueueDBladePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<QosWorkloadQueueDBladeMeasurements>> samples);

    /**
     * Called when updating workload data.
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples    The performance data to report
     * @param leaseTime  the time for datasource to be resgister in the in the registry (in milliseconds)
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportQosWorkloadQueueNBladePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<QosWorkloadQueueNBladeMeasurements>> samples, long leaseTime);

   /**
     * Called when updating workload data.
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples    The performance data to report
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportQosWorkloadQueueNBladePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<QosWorkloadQueueNBladeMeasurements>> samples);

    
    /**
     * Called when updating service center detail data.
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples The performance data to report
     *
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportQosServiceCenterPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<QosServiceCenterMeasurements>> samples);

    /**
     * Called when updating service center data.
     *
     * @param aPackage
     * @param clusterKey
     * @param time
     * @param samples The performance data to report
     * @param leaseTime the time for datasource to be resgister in the in the registry (in milliseconds)
     *
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportQosServiceCenterPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<QosServiceCenterMeasurements>> samples, final long leaseTime);

    public boolean  reportWaflPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<WaflMeasurements>> samples);
    public boolean  reportWaflPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<WaflMeasurements>> samples, final long leaseTime);

    public boolean  reportWaflHyaAggrPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<WaflHyaAggrMeasurements>> samples);
    public boolean  reportWaflHyaAggrPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<WaflHyaAggrMeasurements>> samples, long leaseTime);

    public boolean  reportExtCachePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<ExtCacheObjMeasurements>> samples);
    public boolean  reportExtCachePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey clusterKey, final long time, final Collection<RawSample<ExtCacheObjMeasurements>> samples, final long leaseTime);


    public boolean reportCIFSNodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<CIFSNodeMeasurements>> samples) throws DataSourceErrorException;
    public boolean reportCIFSNodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<CIFSNodeMeasurements>> samples, final long leaseTime) throws DataSourceErrorException;

    public boolean reportSMB1NodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<SMB1NodeMeasurements>> samples) throws DataSourceErrorException;
    public boolean reportSMB1NodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<SMB1NodeMeasurements>> samples, final long leaseTime) throws DataSourceErrorException;

    public boolean reportSMB2NodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<SMB2NodeMeasurements>> samples) throws DataSourceErrorException;
    public boolean reportSMB2NodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<SMB2NodeMeasurements>> samples, final long leaseTime) throws DataSourceErrorException;

    public boolean reportNFSv3NodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<NFSv3NodeMeasurements>> samples) throws DataSourceErrorException;
    public boolean reportNFSv3NodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<NFSv3NodeMeasurements>> samples, final long leaseTime) throws DataSourceErrorException;

    public boolean reportNFSv4NodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<NFSv4NodeMeasurements>> samples) throws DataSourceErrorException;
    public boolean reportNFSv4NodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<NFSv4NodeMeasurements>> samples, final long leaseTime) throws DataSourceErrorException;

    public boolean reportNFSv41NodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<NFSv41NodeMeasurements>> samples) throws DataSourceErrorException;
    public boolean reportNFSv41NodePerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<NFSv41NodeMeasurements>> samples, final long leaseTime) throws DataSourceErrorException;





   
    /**
     * Called when updating Hostadapter data.
     * 
     * @param aPackage
     * @param key
     * @param time
     * @param samples
     * @param leaseTime
     * 
     * @return true if report went all the way to the server, false otherwise
     */
    public boolean reportHostadapterPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<HostadapterMeasurements>> samples, final long leaseTime);
    

    public boolean reportClusterPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<ClusterMeasurements>> samples) throws DataSourceErrorException;
    public boolean reportClusterPerformance(final NetAppStoragePerformancePackage aPackage, final Cluster.ClusterKey key, final long time, final Collection<RawSample<ClusterMeasurements>> samples, final long leaseTime) throws DataSourceErrorException;


}
