package com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.integrity;

import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.netappfoundation.datasource.dependant.integrity.NetAppFoundationDependentIntegrityChecker;
import com.onaro.sanscreen.acquisition.netappfoundation.datasource.dependant.integrity.NetAppIntegrityCheckType;
import com.onaro.sanscreen.acquisition.framework.datasource.IntegrityException;

import java.util.Collection;

public class NetAppVolumePerformanceIntegrityChecker extends NetAppFoundationDependentIntegrityChecker<VolumeMeasurements> {

    public NetAppVolumePerformanceIntegrityChecker(final Collection<RawSample<VolumeMeasurements>> samples,
                                             final String deviceId,
                                             final String logPrefix) {
        super(samples, deviceId, logPrefix, NetAppIntegrityCheckType.VOLUME);
    }

    @Override
    public void checkIntegrity() throws IntegrityException {

        int negativeCount = 0;
        int accessError = 0;
        int badTotalCacheHitRatio = 0;
        int badPartialBlockRatio = 0;

        int maxCacheHitViolations = 0;


        for (final RawSample<VolumeMeasurements> sample : getSamples()) {

            final VolumeMeasurements counters = sample.getCounters();

            //check negatives. Note -1 is a flag to the server as an unknown value.
//            if (counters.getReadIops() < -1 || counters.getReadThroughput() < -1
//                || counters.getReadCacheHitRatio() < -1
//                || counters.getReadResponseTime() < -1 || counters.getWriteIops() < -1
//                || counters.getWriteThroughput() < -1
//                || counters.getWriteCacheHitRatio() < -1 || counters.getWritePending() < -1
//                || counters.getWriteResponseTime() < -1
//                || counters.getTotalCacheHitRatio() < -1 || counters.getTotalResponseTime() < -1
//                || counters.getPartialBlocksRatio() < -1) {
//
//                negativeCount++;
//            }
//
//            if (!counters.isAccessed() &&
//                (counters.getReadIops() > 0 && counters.getWriteIops() > 0)) {
//                accessError++;
//            }
//
//            if (counters.getReadCacheHitRatio() > 100.0 || counters.getWriteCacheHitRatio() > 100.0
//                || counters.getTotalCacheHitRatio() > 100.0) {
//                maxCacheHitViolations++;
//            }
//            else {
//                //only further check cache hit ratio if they are sane.
//                if (counters.getTotalCacheHitRatio() < counters.getReadCacheHitRatio()
//                    && counters.getTotalCacheHitRatio() < counters.getWriteCacheHitRatio()) {
//                    badTotalCacheHitRatio++;
//                }
//
//            }
//
//            if (counters.getPartialBlocksRatio() > 100.0) {
//                badPartialBlockRatio++;
//            }

        }

        //process error messaging.

        if (negativeCount > 0) {
            addError(negativeCount + " volumes had illegal negative counter values.");
        }
        if (accessError > 0) {
            addError(accessError + " volumes registered as not being accessed but had finite IOPS.");
        }
        if (badTotalCacheHitRatio > 0) {
            addError(badTotalCacheHitRatio + " volumes had an illegal total cache hit ratio less than both the read and write cache hit ratios.");
        }
        if (maxCacheHitViolations > 0) {
            addError(maxCacheHitViolations + " volumes had read/write/total cache hit ratios > 100%.");
        }
        if (badPartialBlockRatio > 0) {
            addError(badPartialBlockRatio + " volumes had partial block hit ratios > 100%.");
        }

        if (hasErrors()) {
            dumpErrors();
            throw generateException();
        }

    }

}
