package com.onaro.sanscreen.acquisition.schema;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.util.*;

public abstract class BaseCRCModelRoot extends BaseCRCXMLNode<BaseRootFactory> implements StreamingCRCRoot, MarshallingCache {
    private static final Logger logger = Logger.getLogger(BaseCRCModelRoot.class);
    protected final Map<Comparable, Object> cache = new TreeMap<Comparable, Object>();

    // keep ordered in a tree set to keep diffing xml easier
    private final Map<String, String> stringAttributes = new TreeMap<String, String>();
    private final List<BaseCRCXMLNode<MarshallingCache>> children = new ArrayList<BaseCRCXMLNode<MarshallingCache>>();
    private final Map<String, List<BaseCRCXMLNode<MarshallingCache>>> childrenUnderNode = new TreeMap<String, List<BaseCRCXMLNode<MarshallingCache>>>();

    protected BaseCRCModelRoot() throws SchemaException  {
        super(null, "root");
    }

    /**
     * Set up root attributes and children to be marshaled
     *  use methods:
     *
     * @throws SchemaException If an XML integrity error occurs
     */
    abstract public void addAttributesAndChildrenCRC() throws SchemaException;

    @Override
    protected final boolean hasChildren() {
        return !children.isEmpty() || !childrenUnderNode.isEmpty();
    }

    /**
     *
     * @param writer Writer to dump XML to
     *
     * @throws SchemaException If an XML integrity error occurs
     * @throws java.io.IOException If an IO error occurs
     */
    public void marshallAndCRC(final DSFileCRCWriter writer) throws IOException, SchemaException{
        addAttributesAndChildrenCRC();
        marshallAndCRC(this, writer);
    }

    @Override
    protected final void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        super.marshallChildren(cache, writer, tabs);

        for (final Map.Entry<String, String> entry: stringAttributes.entrySet()){
            addAttribute(writer, entry.getKey(), entry.getValue());
        }

        for (final BaseCRCXMLNode child: children){
            addChildAndCRC(cache, writer, child, tabs);
        }

        for (final Map.Entry<String, List<BaseCRCXMLNode<MarshallingCache>>> entry: childrenUnderNode.entrySet()){
            final String nodeName = entry.getKey();
            final Collection<BaseCRCXMLNode<MarshallingCache>> childList =  entry.getValue();
            this.addAllChildrenUnderNode(cache, writer, nodeName, childList, tabs);
        }
    }


    /**
     * Adds an XMLNode to be marshaled as a child of the root.
     *
     * @param childNode Child object to marshal as the object's XML root children elements
     *
     * @throws SchemaException If <code>childList</code> is null or an element in <code>childList</code> is null
     */
    protected final void addMarshalingChildCRC(final BaseCRCXMLNode<MarshallingCache> childNode) throws SchemaException{
        if (childNode == null)
                throw new SchemaException("model root child can not be null");

        children.add(childNode);
    }


    public final Object put(final Comparable key, final Object value) {
        return cache.put(key, value);
    }

    public final Object get(final Comparable key) {
        return cache.get(key);
    }

    public <K extends Key> K getKey() {
        return null;
    }

}
