package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

/**
 * EMC Symmetrix only.
 *
 * The configuration representing the amount of space a volume can be allocated to a specific tier (a group of storage pools).
 * Tiers are not yet modeled, but the auto tier policy constraint and storage pools have references to their names.
 *
 * The Auto Tier Policy is usually configured with multiple constraints. 
 *
 */

public class AutoTierPolicyConstraint extends XMLNode<AutoTierPolicy> {
    private Integer limitPercent;

    private final AutoTierPolicyConstraintKey key;

    protected AutoTierPolicyConstraint(final AutoTierPolicy parent, final String tier) throws SchemaException {
        super(parent, "AutoTierPolicyConstraint");

        if (tier == null)
            throw new SchemaException(getNodeName() + " Type requires non-null tier");

        this.key = new AutoTierPolicyConstraintKey(tier, parent.getKey());
    }

    @SuppressWarnings({"unchecked"})
    public AutoTierPolicyConstraintKey getKey() {
        return key;
    }

    public String getVendorTier() {
        return key.getVendorTier();
    }

    public Integer getLimitPercent() {
        return limitPercent;
    }

    public void setLimitPercent(final Integer limitPercent) {
        this.limitPercent = limitPercent;
    }

    protected void marshallAttributes(final MarshallingCache marshallingCache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttribute(writer, "VendorTier", getVendorTier());
        addAttribute(writer, "LimitPercent", getLimitPercent());
    }

    public final static class AutoTierPolicyConstraintKey implements Key<AutoTierPolicyConstraintKey> {
        private final String vendorTier;
        private final AutoTierPolicy.AutoTierPolicyKey policyKey;

        public AutoTierPolicyConstraintKey(final String vendorTier, final AutoTierPolicy.AutoTierPolicyKey policyKey) {
            this.vendorTier = vendorTier;
            this.policyKey = policyKey;
        }

        public int compareTo(final AutoTierPolicyConstraintKey o) {
            int i = policyKey.compareTo(o.policyKey);
            if (i != 0)
                return i;

            i = vendorTier.compareTo(o.vendorTier);
            if (i != 0)
                return i;

            return 0;
        }

        public String getVendorTier() {
            return vendorTier;
        }

        public String toString() {
            return policyKey + ":" + vendorTier;
        }

        public AutoTierPolicy.AutoTierPolicyKey getPolicyKey() {
            return policyKey;
        }

        @SuppressWarnings({"RedundantIfStatement"})
        public boolean equals(final Object o) {
            if (this == o) return true;
            if (!(o instanceof AutoTierPolicyConstraintKey)) return false;

            final AutoTierPolicyConstraintKey that = (AutoTierPolicyConstraintKey) o;

            if (vendorTier != null ? !vendorTier.equals(that.vendorTier) : that.vendorTier != null) return false;
            if (policyKey != null ? !policyKey.equals(that.policyKey) : that.policyKey != null) return false;
            return true;
        }

        public int hashCode() {
            int result;
            result = (vendorTier != null ? vendorTier.hashCode() : 0);
            result = 31 * result + (policyKey != null ? policyKey.hashCode() : 0);
            return result;
        }
    }
}
