package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

/**
 * BackEndVolumeConnection class represents connection between back end volume of virtualizer and
 * actual volume on the back end storage system
 */
public class BackEndPath extends XMLNode<BackEndLun> {
    private final String localPortWWN;
    private final String remotePortWWN;
    private final long lun;
    //todo change the name
    private final BackEndPathKey key;

    BackEndPath(final BackEndLun parent, final String localPortWWN, final String remotePortWWN, final long lun) throws SchemaException {
        super(parent, "BackendPath");

        if (remotePortWWN == null || remotePortWWN.length() == 0)
            throw new SchemaException("Remote port WWN should not be null or empty string");

        this.remotePortWWN = remotePortWWN.toUpperCase();

        if (localPortWWN != null)
            this.localPortWWN = localPortWWN.toUpperCase();
        else
            this.localPortWWN = null;

        this.lun = lun;

        key = new BackEndPathKey(parent.getKey(), localPortWWN, remotePortWWN, lun);
    }

    public final Key getKey() {
        return key;
    }

    public final String getLocalPortWWN() {
        return localPortWWN;
    }

    public final String getRemotePortWWN() {
        return remotePortWWN;
    }

    public final long getLun() {
        return lun;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        addAttribute(writer, "LocalPortWWN", localPortWWN);
        addAttribute(writer, "RemotePortWWN", remotePortWWN);
        addAttribute(writer, "LUN", lun);
    }

    public final static class BackEndPathKey implements Key<BackEndPathKey> {
        private final BackEndLun.BackEndLunKey backEndVolumeKey;
        private final String localPortWWN;
        private final String remotePortWWN;
        private final long lun;

        public BackEndPathKey(final BackEndLun.BackEndLunKey backEndVolumeKey, final String localPortWWN, final String remotePortWWN, final long lun) {
            this.backEndVolumeKey = backEndVolumeKey;
            this.localPortWWN = localPortWWN;
            this.remotePortWWN = remotePortWWN;
            this.lun = lun;
        }

        public final int compareTo(final BackEndPathKey o) {
            int r = getBackEndVolumeKey().compareTo(o.getBackEndVolumeKey());
            if (r != 0)
                return r;

            r = compareObjects(getLocalPortWWN(), o.getLocalPortWWN());
            if (r != 0)
                return r;

            r = getRemotePortWWN().compareTo(o.getRemotePortWWN());
            if (r != 0)
                return r;

            if (getLUN() == o.getLUN())
                return 0;

            return getLUN() < o.getLUN() ? -1 : 1;
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final BackEndPathKey that = (BackEndPathKey) o;

            if (lun != that.lun) return false;
            if (backEndVolumeKey != null ? !backEndVolumeKey.equals(that.backEndVolumeKey) : that.backEndVolumeKey != null)
                return false;
            if (localPortWWN != null ? !localPortWWN.equals(that.localPortWWN) : that.localPortWWN != null)
                return false;
            if (remotePortWWN != null ? !remotePortWWN.equals(that.remotePortWWN) : that.remotePortWWN != null)
                return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (backEndVolumeKey != null ? backEndVolumeKey.hashCode() : 0);
            result = 29 * result + (localPortWWN != null ? localPortWWN.hashCode() : 0);
            result = 29 * result + (remotePortWWN != null ? remotePortWWN.hashCode() : 0);
            result = 29 * result + (int) (lun ^ (lun >>> 32));
            return result;
        }

        public final String toString() {
            return backEndVolumeKey.toString() + ":" + remotePortWWN + ":" + localPortWWN + ":" + lun;
        }

        public final BackEndLun.BackEndLunKey getBackEndVolumeKey() {
            return backEndVolumeKey;
        }

        public final String getLocalPortWWN() {
            return localPortWWN;
        }

        public final String getRemotePortWWN() {
            return remotePortWWN;
        }

        public final long getLUN() {
            return lun;
        }
    }
}
