package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

public class Controller extends XMLNode<Storage> {
    private final String wwn;
    private String name = null;
    private String ip = null;
    private String manufacturer = null;
    private String vendor = null;
    private String firmware = null;
    private String driver = null;
    private String model = null;
    private StoragePorts ports;

    private final ControllerKey key;

    /**
     * Use {@link Storage#createController} ChassisSwitch} to create this object
     */
    Controller(final Storage parent, final String wwn) throws SchemaException {
        super(parent, "Controller");

        if (wwn == null || wwn.length() == 0)
            throw new SchemaException(getNodeName() + " Type requires non-null oid and wwn");

        this.wwn = wwn.toUpperCase();

        ports = new StoragePorts(this);

        key = new ControllerKey(parent.getKey(), wwn);
    }

    public final ControllerKey getKey() {
        return key;
    }

    public final Storage getStorage() {
        return getParent();
    }

    public final StoragePorts getPorts() {
        return ports;
    }

    public final StoragePort createPort(final String wwn) throws SchemaException {
        return new StoragePort(this, wwn);
    }

    public final String getDriver() {
        return driver;
    }

    public final void setDriver(String driver) {
        this.driver = driver;
    }

    public final String getFirmware() {
        return firmware;
    }

    public final void setFirmware(String firmware) {
        this.firmware = firmware;
    }

    public final String getIP() {
        return ip;
    }

    public final void setIP(String ip) {
        this.ip = ip;
    }

    public final String getManufacturer() {
        return manufacturer;
    }

    public final void setManufacturer(String manufacturer) {
        this.manufacturer = manufacturer;
    }

    public final String getModel() {
        return model;
    }

    public final void setModel(String model) {
        this.model = model;
    }

    public final String getName() {
        return name;
    }

    public final void setName(String name) {
        this.name = name;
    }

    public final String getVendor() {
        return vendor;
    }

    public final void setVendor(String vendor) {
        this.vendor = vendor;
    }

    public final String getWWN() {
        return wwn;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        addAttribute(writer, "WWN", wwn);
        addAttribute(writer, "Name", name);
        addAttribute(writer, "IP", ip);
        addAttribute(writer, "Manufacturer", manufacturer);
        addAttribute(writer, "Vendor", vendor);
        addAttribute(writer, "Firmware", firmware);
        addAttribute(writer, "Driver", driver);
        addAttribute(writer, "Model", model);
    }

    protected void marshallChildren(final MarshallingCache cache, final Writer writer, final int tabs) throws IOException, SchemaException {
        addAllChildren(cache, writer, ports, tabs);
    }

    protected boolean hasChildren() {
        return (!ports.isEmpty());
    }

    public final static class ControllerKey implements Key<ControllerKey> {
        private final Storage.StorageKey storageKey;
        private final String wwn;

        public ControllerKey(final Storage.StorageKey storageKey, final String wwn) {
            this.storageKey = storageKey;
            this.wwn = wwn;
        }

        public final int compareTo(final ControllerKey o) {
            int i;

            i = getWWN().compareTo(o.getWWN());
            if (i != 0)
                return i;

            return storageKey.compareTo(o.getStorageKey());
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final ControllerKey that = (ControllerKey) o;

            if (storageKey != null ? !storageKey.equals(that.storageKey) : that.storageKey != null) return false;
            if (wwn != null ? !wwn.equals(that.wwn) : that.wwn != null) return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (storageKey != null ? storageKey.hashCode() : 0);
            result = 29 * result + (wwn != null ? wwn.hashCode() : 0);
            return result;
        }

        public final String toString() {
            return storageKey.toString() + ":" + wwn;
        }

        public final Storage.StorageKey getStorageKey() {
            return storageKey;
        }

        public final String getWWN() {
            return wwn;
        }
    }
}
