package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.XMLNode;
import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.util.Capacity;

import java.io.Writer;
import java.io.IOException;


public class FsHost extends XMLNode<HostDataRoot> {

    public static final String ELEMENT_NAME = "FsHost" ;
    /**
     * IP, used as identifier by SANscreen server. Needs to be exactly same as existing host IP to be processed by
     * server
     */
    private final String ip ;
    /**
     * Host name (just for reference)
     */
    private String hostName ;

    /**
     * Time when information about this host was updataed in the system used as source of this information (last update from agent)
     * Use 0 if information is unavailable
     * Use current time if communicating to host directly
     * Time - unix timestamp in seconds
     */
    private long lastAgentReportTime;

    private FileSystemHostKey key ;

    private final FileSystems hostFileSystems;

    FsHost(final String ip, final String hostName, final long lastAgentReportTime, final HostDataRoot parent) throws SchemaException {
        super(parent, ELEMENT_NAME);
        hostFileSystems = new FileSystems(this) ;
        this.ip = ip ;
        this.hostName = hostName ;
        this.lastAgentReportTime = lastAgentReportTime;
        this.key = new FileSystemHostKey(ip) ;
    }

    public FileSystem createFileSystem(final String name, final Capacity allocatedCapacity, final Capacity usedCapacity){
        return new FileSystem(name, allocatedCapacity, usedCapacity, this) ;
    }

    public FileSystems getFileSystems() {
        return hostFileSystems;
    }

    public String getIp() {
        return ip;
    }

    public String getHostName() {
        return hostName;
    }

    public long getLastAgentReportTime() {
        return lastAgentReportTime;
    }

    @SuppressWarnings("unchecked")
    public FileSystemHostKey getKey() {
        return key;
    }

    protected void marshallAttributes(final MarshallingCache marshallingCache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);

        addAttribute(writer, "IP", ip);
        addAttribute(writer, "HostName", hostName);
        addAttribute(writer, "LastAgentReportTime", lastAgentReportTime);

    }

    protected void marshallChildren(final MarshallingCache marshallingCache, final Writer writer, final int tabs) throws IOException, SchemaException {
        super.marshallChildren(marshallingCache, writer, tabs);
        addAllChildren(marshallingCache, writer, hostFileSystems, tabs);
    }

    protected boolean hasChildren() {
        return hostFileSystems.size() > 0 ;
    }

    public static class FileSystemHostKey implements Key<FileSystemHostKey> {

        private String ip ;

        public FileSystemHostKey(final String ip) {
            this.ip = ip;
        }

        public String getIp() {
            return ip;
        }

        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final FileSystemHostKey that = (FileSystemHostKey) o;

            if (ip != null ? !ip.equals(that.ip) : that.ip != null) return false;

            return true;
        }

        public int hashCode() {
            return (ip != null ? ip.hashCode() : 0);
        }

        public int compareTo(final FileSystemHostKey fileSystemHostKey) {

            return ip.compareTo(fileSystemHostKey.ip) ;

        }


    }


}
