package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.SchemaException;

/**
 * Represents a hub port - this is part of an arbitrated loop connected to a switch
 */
public class HubPort extends PortType<Hub> implements Comparable {
    private Boolean parentKnown = null;

    /**
     * The ID of the VSAN this port is logged into.
     */
    private Integer vsanID = null;

    /**
     * Use {@link Hub#createPort} to create this object
     * @param parent the parent this HubPort object belongs to
     * @param wwn the hub port wwn
     * @throws com.onaro.sanscreen.acquisition.schema.SchemaException throws when the wwn is invalid
     */
    HubPort(final Hub parent, final String wwn) throws SchemaException {
        super(parent, new PortTypeKey(wwn));
    }

    public Hub getHub() {
        return getParent();
    }

    public boolean isPortParentKnown() {
        // If not set, return default value
        if (parentKnown == null)
            return true;

        return parentKnown;
    }

    public void setPortParentKnown(final boolean parentKnown) {
        this.parentKnown = parentKnown;
    }

    /**
     * Used to sort hub ports by their WWNs
     */
    @Override
    public int compareTo(final Object o) {
        //noinspection unchecked
        return getKey().compareTo(((HubPort) o).getKey());
    }

    public String getWWN() {
        return getKey().toString();
    }

    public Integer getVsanID() {
        return vsanID;
    }

    public void setVsanID(final int vsanID) {
        this.vsanID = vsanID;
    }

}
