package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;


public class ISCSIAuthentication extends XMLNode<Storage> {
    private final ISCSISession.ISCSISecurity authType = ISCSISession.ISCSISecurity.CHAP; //
    private final String initiatorNodeName;
    private final String targetNodeName;
    private String inboundKeyword;
    private String outboundKeyword;

    private final ISCSIAuthenticationKey key;

    protected ISCSIAuthentication(final Storage parent, final String initiatorNodeName, final String targetNodeName) throws SchemaException {
        super(parent, "ISCSISecurityInfo");
        if (initiatorNodeName == null || targetNodeName == null)
            throw new SchemaException("initiator and target must not be null");

        this.initiatorNodeName = initiatorNodeName;
        this.targetNodeName = targetNodeName;


        key = new ISCSIAuthenticationKey(initiatorNodeName, targetNodeName);
    }

    public ISCSIAuthenticationKey getKey() {
        return key;
    }

    public void setInboundKeyword(final String inboundKeyword) {
        this.inboundKeyword = inboundKeyword;
    }

    public void setOutboundKeyword(final String outboundKeyword) {
        this.outboundKeyword = outboundKeyword;
    }

    public ISCSISession.ISCSISecurity getAuthType() {
        return authType;
    }

    public String getInitiatorNodeName() {
        return initiatorNodeName;
    }

    public String getTargetNodeName() {
        return targetNodeName;
    }

    public String getInboundKeyword() {
        return inboundKeyword;
    }

    public String getOutboundKeyword() {
        return outboundKeyword;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {

        addAttribute(writer, "OID", getKey().toString());
        addAttribute(writer, "InitiatorNodeName", initiatorNodeName);
        addAttribute(writer, "TargetNodeName", targetNodeName);
        addAttribute(writer, "AuthType", authType);
        addAttribute(writer, "InboundKeyword", inboundKeyword);
        addAttribute(writer, "OutboundKeyword", outboundKeyword);
    }


    public final static class ISCSIAuthenticationKey implements Key<ISCSIAuthenticationKey> {
        private final String initiatorNodeName;
        private final String targetNodeName;

        public ISCSIAuthenticationKey(final String initiatorNodeName, final String targetNodeName) throws SchemaException {
            if (initiatorNodeName == null || targetNodeName == null)
                throw new SchemaException("initiator and target must not be null");
            this.initiatorNodeName = initiatorNodeName;
            this.targetNodeName = targetNodeName;
        }

        public int compareTo(final ISCSIAuthenticationKey o) {
            int i = o.getInitiatorNodeName().compareTo(getInitiatorNodeName());
            if (i != 0)
                return i;
            return o.getTargetNodeName().compareTo(getTargetNodeName());
        }

        public String getInitiatorNodeName() {
            return initiatorNodeName;
        }

        public String getTargetNodeName() {
            return targetNodeName;
        }

        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final ISCSIAuthenticationKey that = (ISCSIAuthenticationKey) o;

            if (initiatorNodeName != null ? !initiatorNodeName.equals(that.initiatorNodeName) : that.initiatorNodeName != null)
                return false;
            if (targetNodeName != null ? !targetNodeName.equals(that.targetNodeName) : that.targetNodeName != null)
                return false;

            return true;
        }

        public int hashCode() {
            int result;
            result = (initiatorNodeName != null ? initiatorNodeName.hashCode() : 0);
            result = 31 * result + (targetNodeName != null ? targetNodeName.hashCode() : 0);
            return result;
        }

        public String toString() {
            return getInitiatorNodeName() + ":" + getTargetNodeName();
        }
    }

}

