package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Writer;

public class ISCSITargetNetworkPortal extends ISCSINetworkPortal {
    public final static int defaultPort = 2360;

    private final int listeningPort;

    private final ISCSITargetNetworkPortalKey key;

    ISCSITargetNetworkPortal(final ISCSINetworkPortalGroup groupNetwork, final String ip, final int port) throws SchemaException {
        super(groupNetwork, ip);
        this.listeningPort = port;
        this.key = new ISCSITargetNetworkPortalKey(groupNetwork.getKey(), ip, port);
    }

    public int getListeningPort() {
        return listeningPort;
    }


    public ISCSITargetNetworkPortalKey getKey() {
        return key;
    }

    protected void marshallAttributes(final MarshallingCache marshallingCache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttribute(writer, "ListeningPort", listeningPort);
    }

    public static class ISCSITargetNetworkPortalKey extends ISCSINetworkPortalKey {
        private int port = 0;

        public ISCSITargetNetworkPortalKey(final Key key, final String ip, final int port) throws SchemaException {
            super(key, ip);
            if (ip == null || port <= 0)
                throw new SchemaException("ip  must not be null");
            this.port = port;
        }

        public ISCSITargetNetworkPortalKey(final Key key, final String ip) throws SchemaException {
            super(key, ip);
        }

        public int compareTo(final ISCSITargetNetworkPortalKey o) {
            int i = o.getIp().compareTo(getIp());
            if (i != 0)
                return i;

            i = o.getKey().toString().compareTo(getKey().toString());
            if (i != 0)
                return i;

            return o.getPort() - getPort();
        }


        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;

            final ISCSITargetNetworkPortalKey that = (ISCSITargetNetworkPortalKey) o;

            if (port != that.port) return false;

            return true;
        }

        public int hashCode() {
            int result = super.hashCode();
            result = 31 * result + port;
            return result;
        }

        public int getPort() {
            return port;
        }

        public String toString() {
            return super.toString() + ":" + port;
        }
    }

}
