package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

/**
 * Represents an IVR zone member
 */
public class IVRZoneMember extends XMLNode<IVRZone> {
    private final String wwn;
    private final int vsan;
    private final int afid;

    private final IVRZoneMemberKey key;

    IVRZoneMember(final IVRZone parent, final String wwn, final int vsan, final int afid) throws SchemaException {
        super(parent, "IVRZoneMember");

        if (wwn == null || wwn.length() == 0)
            throw new SchemaException(getNodeName() + " Type requires non-null wwn");

        this.wwn = wwn.toUpperCase();
        this.vsan = vsan;
        this.afid = afid;

        key = new IVRZoneMemberKey(wwn, vsan, afid);
    }

    public final IVRZoneMemberKey getKey() {
        return key;
    }

    public final IVRZone getIVRZone() {
        return getParent();
    }

    public final String getWWN() {
        return wwn;
    }

    public final int getVSAN() {
        return vsan;
    }

    public final int getAFID() {
        return afid;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        addAttribute(writer, "WWN", wwn);
        addAttribute(writer, "VsanId", vsan);
        addAttribute(writer, "AFId", afid);
    }

    public static final class IVRZoneMemberKey implements Key<IVRZoneMemberKey> {
        private final String wwn;
        private final int vsan;
        private final int afid;

        public IVRZoneMemberKey(final String wwn, final int vsan, final int afid) {
            this.wwn = wwn;
            this.vsan = vsan;
            this.afid = afid;
        }

        public final int compareTo(final IVRZoneMemberKey o) {
            int i;

            i = getWWN().compareTo(o.getWWN());
            if (i != 0)
                return i;

            i = getVSAN() - o.getVSAN();
            if (i != 0)
                return i;

            return getAFID() - o.getAFID();
        }

        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final IVRZoneMemberKey that = (IVRZoneMemberKey) o;

            if (afid != that.afid) return false;
            if (vsan != that.vsan) return false;
            if (wwn != null ? !wwn.equals(that.wwn) : that.wwn != null) return false;

            return true;
        }

        public int hashCode() {
            int result;
            result = (wwn != null ? wwn.hashCode() : 0);
            result = 29 * result + vsan;
            result = 29 * result + afid;
            return result;
        }

        public final String toString() {
            return wwn + ":" + vsan + ":" + afid;
        }

        public final String getWWN() {
            return wwn;
        }

        public final int getVSAN() {
            return vsan;
        }

        public final int getAFID() {
            return afid;
        }
    }
}
