package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Writer;

public class LocalStorageSynchronization extends StorageSynchronization {
    private final LocalStorageSynchronizationKey key;

    /**
     * Use {@link com.onaro.sanscreen.acquisition.schema.odata2.Storage#createLocalStorageSynchronization} to create this object
     */
    LocalStorageSynchronization(final Storage parent, final String sourceVolume, final String targetVolume) throws SchemaException {
        super(parent, "StorageSynchronization", sourceVolume, targetVolume);

        key = new LocalStorageSynchronizationKey(parent.getKey(), sourceVolume, targetVolume);
    }

    public final LocalStorageSynchronizationKey getKey() {
        return key;
    }

    protected boolean hasChildren() {
        return false;
    }

    @Override
    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        super.marshallAttributes(cache, writer);
        addAttribute(writer, "SourceVolume", getSource());
        addAttribute(writer, "TargetVolume", getTarget());
        addAttribute(writer, "Mode", getMode()!= null ? getMode() : getModeEnum().name() );    // if mode is not explicitly set, take the enum
        addAttribute(writer, "ModeEnum", getModeEnum().name());
        addAttribute(writer, "State", getState()!= null ? getState() : getStateEnum().name());   // if state is not explicitly set, take the enum
        addAttribute(writer, "StateEnum", getStateEnum().name());
        addAttribute(writer, "Technology", getTechnology());

    }

    public static final class LocalStorageSynchronizationKey implements Key<LocalStorageSynchronizationKey> {
        private final Storage.StorageKey storageKey;
        private final String sourceVolume;
        private final String targetVolume;

        public LocalStorageSynchronizationKey(final Storage.StorageKey storageKey, final String sourceVolume, final String targetVolume) {
            this.storageKey = storageKey;
            this.sourceVolume = sourceVolume;
            this.targetVolume = targetVolume;
        }

        public final int compareTo(final LocalStorageSynchronizationKey o) {
            int i;

            // Must be the same volume
            i = getStorageKey().compareTo(o.getStorageKey());
            if (i != 0)
                return i;

            i = getSourceVolume().compareTo(o.getSourceVolume());
            if (i != 0)
                return i;

            return getTargetVolume().compareTo(o.getTargetVolume());
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final LocalStorageSynchronizationKey that = (LocalStorageSynchronizationKey) o;

            if (storageKey != null ? !storageKey.equals(that.storageKey) : that.storageKey != null) return false;
            if (sourceVolume != null ? !sourceVolume.equals(that.sourceVolume) : that.sourceVolume != null)
                return false;
            if (targetVolume != null ? !targetVolume.equals(that.targetVolume) : that.targetVolume != null)
                return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (storageKey != null ? storageKey.hashCode() : 0);
            result = 29 * result + (sourceVolume != null ? sourceVolume.hashCode() : 0);
            result = 29 * result + (targetVolume != null ? targetVolume.hashCode() : 0);
            return result;
        }

        public final String toString() {
            return storageKey.toString() + ":" + getSourceVolume() + ":" + getTargetVolume();
        }

        public final Storage.StorageKey getStorageKey() {
            return storageKey;
        }

        public final String getSourceVolume() {
            return sourceVolume;
        }

        public final String getTargetVolume() {
            return targetVolume;
        }
    }
}
