package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Writer;

public class MetaVolume extends Volume {
    protected final Members members;

    /**
     * Use {@link Storage#createMetaVolume} to create this object
     */
    MetaVolume(final Storage parent,
               final String name,
               final StoragePool storagePool,
               final Qtree qtree,
               final boolean isDrPackageInstalled) throws SchemaException {
        super(parent, name, storagePool, qtree, isDrPackageInstalled);
        members = new Members(this);

    }

    /**
     * MetaVolume can not have a storage pool reference. Calling this method will generate a runtime exception.
     *
     * @return The storage pool
     */
    @Deprecated
    public final StoragePool getStoragePool() {
        throw new RuntimeException("Cannot access a storage pool from a metavolume");
    }
    
    public Members getMembers() {
        return members;
    }

    public VolumeMember createVolumeMember(final String name, final int rank, final StoragePool pool) throws SchemaException {
        return new VolumeMember(this, name, rank, pool, null, isDrPackageInstalled());
    }

    public VolumeMember createVolumeMember(final String name, final int rank, final Qtree qtree) throws SchemaException {
        return new VolumeMember(this, name, rank, null, qtree, isDrPackageInstalled());
    }

    /**
     * Only used by Symmetrix to allow it to create a volume
     * without a storage pool, setting it later.
     * @param name
     * @param rank
     * @return
     * @throws SchemaException
     */
    @Deprecated
    public VolumeMember createVolumeMember(final String name, final int rank) throws SchemaException {
        return new VolumeMember(this, name, rank, null, null, isDrPackageInstalled());
    }


    protected void marshallAttributes(final MarshallingCache marshallingCache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttribute(writer, "Meta", Boolean.toString(members.size() > 0));
    }

    protected boolean hasChildren() {
        return (members.size() > 0) || super.hasChildren();
    }

    protected void marshallChildren(final MarshallingCache marshallingCache, final Writer writer, final int tabs) throws IOException, SchemaException {
        super.marshallChildren(marshallingCache, writer, tabs);
        addAllChildrenUnderNode(marshallingCache, writer, "Members", members, tabs);
    }
}
