package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;
import com.onaro.sanscreen.acquisition.schema.utils.NasShareUtils;

import java.io.IOException;
import java.io.Writer;

public class NasShare extends XMLNode<VirtualizationRoot> {
    private final String virtualCenterIp;
    private final String sharePath;
    private final String filerIp;

    private String filerName;
    private int capacity;
    private final Type type;

    private final NasShareKey key;

    public NasShare(final VirtualizationRoot parent, final String virtualCenterIp, final String filerIp,
                    final String sharePath, final Type type) throws SchemaException {
        super(parent, "HV_NasShare");

        if (virtualCenterIp == null || filerIp == null || sharePath == null) {
            throw new SchemaException("virtualCenterIp and filerIp and sharePath must not be null");
        }

        this.virtualCenterIp = virtualCenterIp;
        this.sharePath = sharePath;
        this.filerIp = filerIp;
        this.type = type;

        key = new NasShareKey(virtualCenterIp, filerIp, sharePath);

    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getVirtualCenterIp() {
        return virtualCenterIp;
    }

    @SuppressWarnings({"unchecked"})
    public NasShareKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getFilerName() {
        return filerName;
    }

    public void setFilerName(final String filerName) {
        this.filerName = filerName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getFilerIp() {
        return filerIp;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getSharePath() {
        return sharePath;
    }

    public int getCapacity() {
        return capacity;
    }

    public void setCapacity(final int capacity) {
        this.capacity = capacity;
    }

    public Type getType() {
        return type;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(cache, writer);
        addAttribute(writer, "FilerName", filerName);
        addAttribute(writer, "FilerIp", filerIp);
        addAttribute(writer, "SharePath", NasShareUtils.normalizeNasShareName(sharePath, type == Type.CIFS));
        addAttribute(writer, "Capacity", capacity);
        addAttribute(writer, "Type", type.toString());
        addAttribute(writer, "OID", getKey());

    }

    public static class NasShareKey implements Key<NasShareKey> {
        private final String virtualCenterIp;
        private final String sharePath;
        private final String filerIp;

        public String getVirtualCenterIp() {
            return virtualCenterIp;
        }

        public String getSharePath() {
            return sharePath;
        }

        public String getFilerIp() {
            return filerIp;
        }

        public NasShareKey(final String virtualCenterIp, final String filerIp, final String sharePath) throws SchemaException {
            if (virtualCenterIp == null || filerIp == null || sharePath == null) {
                throw new SchemaException("virtualCenterIp and filerIp and sharePath must not be null");
            }

            this.virtualCenterIp = virtualCenterIp;
            this.sharePath = sharePath;
            this.filerIp = filerIp;
        }

        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final NasShareKey that = (NasShareKey) o;

            return !(filerIp != null ? !filerIp.equals(that.filerIp) : that.filerIp != null) && !(sharePath != null ? !sharePath.equals(that.sharePath) : that.sharePath != null) && !(virtualCenterIp != null ? !virtualCenterIp.equals(that.virtualCenterIp) : that.virtualCenterIp != null);

        }

        public int hashCode() {
            int result;
            result = (virtualCenterIp != null ? virtualCenterIp.hashCode() : 0);
            result = 31 * result + (sharePath != null ? sharePath.hashCode() : 0);
            result = 31 * result + (filerIp != null ? filerIp.hashCode() : 0);
            return result;
        }

        public int compareTo(final NasShareKey o) {
            int i = virtualCenterIp.compareTo(o.getVirtualCenterIp());
            if (i != 0) return i;
            i = filerIp.compareTo(o.getFilerIp());
            if (i != 0) return i;
            return sharePath.compareTo(o.getSharePath());

        }

        public String toString() {
            return filerIp + ":" + sharePath + ":" + virtualCenterIp;
        }
    }

    public enum Type {
        NFS, CIFS
    }
}
