package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

public class NetworkPipe extends XMLNode<Storage> {
    private final String sourceID;
    private final String targetID;
    private Boolean biDirectional = Boolean.TRUE;
    private Boolean autoRecover = null;
    private OperationalStatus operationalStatus = null;

    private final NetworkPipeKey key;

    /**
     * Use {@link Storage#createNetworkPipe} to create this object
     */
    NetworkPipe(final Storage parent, final String sourceID, final String targetID) throws SchemaException {
        super(parent, "NetworkPipe");

        if (sourceID == null || sourceID.length() == 0 || targetID == null || targetID.length() == 0)
            throw new SchemaException(getNodeName() + " Type requires non-null SourceID and TargetID");

        this.sourceID = sourceID;
        this.targetID = targetID;

        key = new NetworkPipeKey(parent.getKey(), sourceID, targetID);
    }

    public final NetworkPipeKey getKey() {
        return key;
    }

    public final Storage getStorage() {
        return getParent();
    }

    public final Boolean isAutoRecover() {
        return autoRecover;
    }

    public final void setAutoRecover(final Boolean autoRecover) {
        this.autoRecover = autoRecover;
    }

    public final boolean isBiDirectional() {
        return biDirectional;
    }

    public final void setBiDirectional(boolean biDirectional) {
        this.biDirectional = biDirectional;
    }

    public final OperationalStatus getOperationalStatus() {
        return operationalStatus;
    }

    public final void setOperationalStatus(OperationalStatus operationalStatus) {
        this.operationalStatus = operationalStatus;
    }

    public final String getSourceID() {
        return sourceID;
    }

    public final String getTargetID() {
        return targetID;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        addAttribute(writer, "SourceId", sourceID);
        addAttribute(writer, "TargetId", targetID);
        addAttribute(writer, "Bidirectional", biDirectional);
        addAttribute(writer, "AutoRecover", autoRecover);
        addAttribute(writer, "OperationalStatus", operationalStatus);
    }

    public static enum OperationalStatus {
        OK("OK"),
        STOPPED("STOPPED");

        private String e;

        OperationalStatus(String e) {
            this.e = e;
        }

        public String toString() {
            return e;
        }
    }

    public static final class NetworkPipeKey implements Key<NetworkPipeKey> {
        private final Storage.StorageKey storageKey;
        private final String sourceID;
        private final String targetID;

        public NetworkPipeKey(final Storage.StorageKey storageKey, final String sourceID, final String targetID) {
            this.storageKey = storageKey;
            this.sourceID = sourceID;
            this.targetID = targetID;
        }

        public final int compareTo(final NetworkPipeKey o) {
            int i;

            i = getSourceID().compareTo(o.getSourceID());
            if (i != 0)
                return i;

            return getTargetID().compareTo(o.getTargetID());
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final NetworkPipeKey that = (NetworkPipeKey) o;

            if (storageKey != null ? !storageKey.equals(that.storageKey) : that.storageKey != null) return false;
            if (sourceID != null ? !sourceID.equals(that.sourceID) : that.sourceID != null) return false;
            if (targetID != null ? !targetID.equals(that.targetID) : that.targetID != null) return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (storageKey != null ? storageKey.hashCode() : 0);
            result = 29 * result + (sourceID != null ? sourceID.hashCode() : 0);
            result = 29 * result + (targetID != null ? targetID.hashCode() : 0);
            return result;
        }

        public final String toString() {
            return storageKey.toString() + ":" + getSourceID() + ":" + getTargetID();
        }

        public final String getSourceID() {
            return sourceID;
        }

        public final String getTargetID() {
            return targetID;
        }

        public final Storage.StorageKey getStorageKey() {
            return storageKey;
        }
    }
}
