package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

public class NetworkPipePortWwn extends XMLNode<ProtocolEndPoint> {
    private final String wwn;

    private final NetworkPipePortWwnKey key;

    /**
     * Use {@link ProtocolEndPoint#createNetworkPipePortWwn} to create this object
     */
    NetworkPipePortWwn(final ProtocolEndPoint parent, final String wwn) throws SchemaException {
        super(parent, "NetworkPipePortWwn");

        if (wwn == null)
            throw new SchemaException(getNodeName() + " Type requires non-null wwn");

        this.wwn = wwn.toUpperCase();

        key = new NetworkPipePortWwnKey(parent.getKey(), wwn);
    }

    public final NetworkPipePortWwnKey getKey() {
        return key;
    }

    public final ProtocolEndPoint getProtocolEndPoint() {
        return getParent();
    }

    public final String getWWN() {
        return wwn;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        addAttribute(writer, "WWN", wwn);
    }

    public static final class NetworkPipePortWwnKey implements Key<NetworkPipePortWwnKey> {
        private final ProtocolEndPoint.ProtocolEndPointKey pepKey;
        private final String wwn;

        public NetworkPipePortWwnKey(final ProtocolEndPoint.ProtocolEndPointKey pepKey, final String wwn) {
            this.pepKey = pepKey;
            this.wwn = wwn;
        }

        public final int compareTo(final NetworkPipePortWwnKey o) {
            int i = getWWN().compareTo(o.getWWN());
            if (i != 0)
                return i;

            return pepKey.compareTo(o.getProtocolEndPointKey());
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final NetworkPipePortWwnKey that = (NetworkPipePortWwnKey) o;

            if (pepKey != null ? !pepKey.equals(that.pepKey) : that.pepKey != null) return false;
            if (wwn != null ? !wwn.equals(that.wwn) : that.wwn != null) return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (pepKey != null ? pepKey.hashCode() : 0);
            result = 29 * result + (wwn != null ? wwn.hashCode() : 0);
            return result;
        }

        public final String toString() {
            return pepKey.toString() + ":" + wwn;
        }

        public final String getWWN() {
            return wwn;
        }

        public final ProtocolEndPoint.ProtocolEndPointKey getProtocolEndPointKey() {
            return pepKey;
        }
    }
}
