package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.IntegrityException;
import com.onaro.sanscreen.acquisition.schema.Root;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Writer;

/**
 * Defines the basic functionality of a odata root object.
 *
 * <p/>Each acquisition report is sent to the SANscreen server in XML format. On the data source package implementation
 * side, building this report is done by utilizing the odata schema library (<code>com.onaro.sanscreen.acquisition.schema.odata2.*</code>).
 *
 * <p/>To start building an <code>odata</code> object tree, a <code>Root</code> object has to be created. Using the {@link RootFactory}
 * factory class, root of different kinds can be created, depending on the type of report the data source will generate.
 * There is a specific root object for every type of report that can be generated by the package ({@link StorageRoot} for
 * example is used to report about a storage array to the server).
 *
 * <p/>A package implementation can choose to report more than one roots in a single sample (this is achieved by construction
 * a "Multiple Originator ID (MOID)" package).
 */
public interface OData2Root extends Root {
    public WWNAliases getWWNAliases();

    public Unknowns getUnknowns();

    public Unknown createUnknown(final String wwn) throws SchemaException;

    /**
     * Used by the framework to convert the schema object hierarchy into XML. This is done when sending the report to
     * the server.
     *
     * @param writer Writer to dump XML to
     *
     * @throws IOException
     * @throws SchemaException
     */
    public void marshall(final Writer writer) throws IOException, SchemaException;

    /**
     * Perform integrity checks on device data
     *
     * @throws com.onaro.sanscreen.acquisition.framework.datasource.IntegrityException
     *
     */
    public void checkIntegrity() throws IntegrityException;
}
