package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

public class ProtocolEndPoint extends XMLNode<Storage> {
    private final String id;
    private final String name;
    private final String storageIP;

    private final NetworkPipePortWWNs networkPipePortWWNs;

    private final ProtocolEndPointKey key;

    /**
     * Use {@link Storage#createProtocolEndPoint} to create this object
     */
    ProtocolEndPoint(final Storage parent, final String id, final String name, final String storageIP) throws SchemaException {
        super(parent, "ProtocolEndPoint");

        if (id == null || id.length() == 0 || name == null || name.length() == 0 || storageIP == null || storageIP.length() == 0)
            throw new SchemaException(getNodeName() + " Type requires non-null ID, Name and StorageIP");

        this.id = id;
        this.name = name;
        this.storageIP = storageIP;

        networkPipePortWWNs = new NetworkPipePortWWNs(this);

        key = new ProtocolEndPointKey(id, name, storageIP);
    }

    public ProtocolEndPointKey getKey() {
        return key;
    }

    public NetworkPipePortWwn createNetworkPipePortWwn(final String wwn) throws SchemaException {
        if (wwn == null || wwn.length() == 0)
            throw new SchemaException(getParent().getNodeName() + " Type requires non-null wwn to create NetworkPipePortWWN");

        return new NetworkPipePortWwn(this, wwn);
    }

    public NetworkPipePortWwn createEmptyNetworkPipePortWwn() throws SchemaException {
        return new NetworkPipePortWwn(this, "");
    }

    public Storage getStorage() {
        return getParent();
    }

    public String getID() {
        return id;
    }

    public String getName() {
        return name;
    }

    public String getStorageIP() {
        return storageIP;
    }

    public NetworkPipePortWWNs getNetworkPipePortWWNs() {
        return networkPipePortWWNs;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        addAttribute(writer, "ID", id);
        addAttribute(writer, "Name", name);
        addAttribute(writer, "StorageIp", storageIP);
    }

    protected void marshallChildren(final MarshallingCache cache, final Writer writer, final int tabs) throws IOException, SchemaException {
        addAllChildren(cache, writer, networkPipePortWWNs, tabs);
    }

    protected boolean hasChildren() {
        return (!networkPipePortWWNs.isEmpty());
    }

    public static final class ProtocolEndPointKey implements Key<ProtocolEndPointKey> {
        private final String id;
        private final String name;
        private final String storageIP;

        public ProtocolEndPointKey(final String id, final String name, final String storageIP) {
            this.id = id;
            this.name = name;
            this.storageIP = storageIP;
        }

        public final int compareTo(final ProtocolEndPointKey o) {
            int i;

            i = id.compareTo(o.getID());
            if (i != 0)
                return i;

            i = name.compareTo(o.getName());
            if (i != 0)
                return i;

            return storageIP.compareTo(o.getStorageIP());
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final ProtocolEndPointKey that = (ProtocolEndPointKey) o;

            if (id != null ? !id.equals(that.id) : that.id != null) return false;
            if (name != null ? !name.equals(that.name) : that.name != null) return false;
            if (storageIP != null ? !storageIP.equals(that.storageIP) : that.storageIP != null) return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (id != null ? id.hashCode() : 0);
            result = 29 * result + (name != null ? name.hashCode() : 0);
            result = 29 * result + (storageIP != null ? storageIP.hashCode() : 0);
            return result;
        }

        public final String toString() {
            return id + ":" + name + ":" + storageIP;
        }

        public final String getID() {
            return id;
        }

        public final String getName() {
            return name;
        }

        public final String getStorageIP() {
            return storageIP;
        }
    }
}
