package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.util.Capacity;

import java.io.Serializable;

@SuppressWarnings({"UnusedDeclaration"})
public class StoragePoolCapacity implements Serializable {
    private Capacity snapshotUsed;
    private Capacity snapshotAllocated;
    private Capacity dataUsed;
    private Capacity dataAllocated;
    private Capacity reservedCapacity;
    private Capacity totalUsed;
    private Capacity totalAllocated;
    private Capacity otherUsedCapacity;
    private Capacity otherAllocatedCapacity;
    private Capacity physicalDiskCapacity;

    private float rawToUsableRatio = 1;

    private String redundancy;

    private Boolean includeInDwhCapacity;

    protected StoragePoolCapacity() {
    }

    public Capacity getSnapshotUsed() {
        return snapshotUsed;
    }

    public void setSnapshotUsed(final Capacity snapshotUsed) {
        this.snapshotUsed = Capacity.create(snapshotUsed);
    }

    public Capacity getSnapshotAllocated() {
        return snapshotAllocated;
    }

    public void setSnapshotAllocated(final Capacity snapshotAllocated) {
        this.snapshotAllocated = Capacity.create(snapshotAllocated);
    }

    public Capacity getDataUsed() {
        return dataUsed;
    }

    public void setDataUsed(final Capacity dataUsed) {
        this.dataUsed = Capacity.create(dataUsed);
    }

    public Capacity getDataAllocated() {
        return dataAllocated;
    }

    public void setDataAllocated(final Capacity dataAllocated) {
        this.dataAllocated = Capacity.create(dataAllocated);
    }

    public Capacity getTotalUsed() {
        return totalUsed;
    }

    public void setTotalUsed(final Capacity totalUsed) {
        this.totalUsed = Capacity.create(totalUsed);
    }

    public Capacity getTotalAllocated() {
        return totalAllocated;
    }

    public void setTotalAllocated(final Capacity totalAllocated) {
        this.totalAllocated = Capacity.create(totalAllocated);
    }

    /**
     * Ratio to convert logical capacity to raw.
     *
     * <p/>With RAID1 the calculation will be used as such:
     * <code>10GB (logical capacity) * 2 (ratio) = 20GB (raw capacity)</code>
     *
     * @return Ratio to convert logical capacity to raw.
     */
    public float getRawToUsableRatio() {
        return rawToUsableRatio;
    }

    /**
     * Ratio to convert logical capacity to raw.
     *
     * <p/>With RAID1 the calculation will be used as such:
     * <code>10GB (logical capacity) * 2 (ratio) = 20GB (raw capacity)</code>
     *
     * @param rawToUsableRatio ratio to set
     */
    public void setRawToUsableRatio(final float rawToUsableRatio) {
        this.rawToUsableRatio = rawToUsableRatio;
    }

    public String getRedundancy() {
        return redundancy;
    }

    public void setRedundancy(final String redundancy) {
        this.redundancy = redundancy;
    }

    public Capacity getReservedCapacity() {
        return reservedCapacity;
    }

    public void setReservedCapacity(final Capacity reservedCapacity) {
        this.reservedCapacity = Capacity.create(reservedCapacity);
    }

    public Capacity getOtherUsedCapacity() {
        return otherUsedCapacity;
    }

    public void setOtherUsedCapacity(final Capacity otherUsedCapacity) {
        this.otherUsedCapacity = Capacity.create(otherUsedCapacity);
    }

    public Capacity getOtherAllocatedCapacity() {
        return otherAllocatedCapacity;
    }

    public void setOtherAllocatedCapacity(final Capacity otherAllocatedCapacity) {
        this.otherAllocatedCapacity = Capacity.create(otherAllocatedCapacity);
    }

    public Boolean getIncludeInDwhCapacity() {
        return includeInDwhCapacity;
    }

    public void setIncludeInDwhCapacity(final Boolean includeInDwhCapacity) {
        this.includeInDwhCapacity = includeInDwhCapacity;
    }

    /**
     * @return Amount of storage allocated by physical disks for this storage pool. This is relevant only for mixed storage pools
     *         (based on both LUNs and disks) where the server and DWH need to understand how capacity is distributed for chargeback.
     */
    public Capacity getPhysicalDiskCapacity() {
        return physicalDiskCapacity;
    }

    /**
     * @param physicalDiskCapacity Amount of storage allocated by physical disks for this storage pool. This is relevant only for mixed storage pools
     * (based on both LUNs and disks) where the server and DWH need to understand how capacity is distributed for chargeback.
     */
    public void setPhysicalDiskCapacity(final Capacity physicalDiskCapacity) {
        this.physicalDiskCapacity = Capacity.create(physicalDiskCapacity);
    }

}
