package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Serializable;
import java.io.Writer;

/**
 * Reperesents a switch port
 */
public class SwitchPort extends Port<SwitchType> implements Serializable {
    private final static long serialVersionUID = 1;

    /**
     * Use {@link Switch#createPort}, {@link LogicalSwitch#createPort} or {@link ChassisSwitch#createPort} to create this object
     */
    SwitchPort(final SwitchType parent, final String wwn) throws SchemaException {
        super(parent, wwn);
    }

    public SwitchType getSwitch() {
        return getParent();
    }

    /**
     * Creates an arbitrated loop object (represented as a hub) connected to this port
     *
     * @return A new Hub object associated with this switch port.
     *
     * @throws SchemaException
     */
    public final Hub createHub() throws SchemaException {
        return new Hub(this);
    }

    /**
     * HEADSUP: Promote access priviledges for this method for the use of {@link Hub#marshall}
     */
    public void marshall(final MarshallingCache cache, final Writer writer, int tabs) throws IOException, SchemaException {
        super.marshall(cache, writer, tabs);
    }
}
