package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.*;

import java.io.IOException;
import java.io.Writer;

public class Unknown extends XMLNode<OData2Root> {
    private final String wwn;

    private String manufacturer = null;
    private String vendor = null;
    private String firmware = null;
    private String driver = null;
    private String name = null;
    private String model = null;
    private String host = null;

    private final UnknownPorts ports;

    private final UnknownKey key;

    /**
     * Use {@link OData2Root#createUnknown} to create this object
     * @param parent the odata parent
     * @param wwn the wwn as key for the unknown port
     * @throws com.onaro.sanscreen.acquisition.schema.SchemaException throws when there is a schema error
     */
    Unknown(final OData2Root parent, final String wwn) throws SchemaException {
        super(parent, "Unknown");

        if (wwn == null || wwn.length() == 0)
            throw new SchemaException(getNodeName() + " Type requires non-null OID and WWN");

        this.wwn = wwn.toUpperCase();

        ports = new UnknownPorts(this);

        key = new UnknownKey(wwn);
    }

    @SuppressWarnings("unchecked")
    public final UnknownKey getKey() {
        return key;
    }

    public final Root getRoot() {
        return getParent();
    }

    public final UnknownPort createPort(final String wwn) throws SchemaException {
        return new UnknownPort(this, wwn);
    }

    public final String getDriver() {
        return driver;
    }

    public final void setDriver(final String driver) {
        this.driver = driver;
    }

    public final String getFirmware() {
        return firmware;
    }

    public final void setFirmware(final String firmware) {
        this.firmware = firmware;
    }

    public final String getManufacturer() {
        return manufacturer;
    }

    public final void setManufacturer(final String manufacturer) {
        this.manufacturer = manufacturer;
    }

    public final String getModel() {
        return model;
    }

    public final void setModel(final String model) {
        this.model = model;
    }

    public final String getName() {
        return name;
    }

    public final void setName(final String name) {
        this.name = name;
    }

    public final String getVendor() {
        return vendor;
    }

    public final void setVendor(final String vendor) {
        this.vendor = vendor;
    }

    public String getHost() {
        return host;
    }

    public final void setHost(final String host) {
        this.host = host;
    }

    public final String getWWN() {
        return wwn;
    }

    public final UnknownPorts getPorts() {
        return ports;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        addAttribute(writer, "WWN", wwn);
        addAttribute(writer, "Manufacturer", manufacturer);
        addAttribute(writer, "Vendor", vendor);
        addAttribute(writer, "Firmware", firmware);
        addAttribute(writer, "Driver", driver);
        addAttribute(writer, "Model", model);
        addAttribute(writer, "Name", name);
    }

    protected void marshallChildren(final MarshallingCache cache, final Writer writer, final int tabs) throws IOException, SchemaException {
        addAllChildren(cache, writer, ports, tabs);
    }

    protected boolean hasChildren() {
        return (!ports.isEmpty());
    }

    public static final class UnknownKey implements Key<UnknownKey> {
        private final String wwn;

        public UnknownKey(final String wwn) {
            this.wwn = wwn;
        }

        public final int compareTo(final UnknownKey o) {
            return getWWN().compareTo(o.getWWN());
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final UnknownKey that = (UnknownKey) o;

            if (wwn != null ? !wwn.equals(that.wwn) : that.wwn != null) return false;

            return true;
        }

        public final int hashCode() {
            return (wwn != null ? wwn.hashCode() : 0);
        }

        public final String toString() {
            return wwn;
        }

        public final String getWWN() {
            return wwn;
        }
    }
}
