package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.util.Capacity;

import java.io.IOException;
import java.io.Writer;

public class VolumeMember extends BaseVolume<MetaVolume> {
    private final int rank;
    private Integer cylinders = null;

    private final VolumeCapacity capacity;

    /**
     * Use {@link MetaVolume#createVolumeMember} to create this object
     */
    VolumeMember(final MetaVolume parent,
                 final String name,
                 final int rank,
                 final StoragePool storagePool,
                 final Qtree qtree,
                 final boolean isDrPackageInstalled) throws SchemaException {
        super(parent, parent.getParent(), "VolumeMember", name, storagePool, qtree, isDrPackageInstalled);

        if (rank <= 0)
            throw new SchemaException(getNodeName() + " Type require rank which is one based");

        this.rank = rank;
        this.capacity = new VolumeCapacity(this);
    }

    public Storage getStorage() {
        return getVolume().getStorage();
    }

    public Volume getVolume() {
        return getParent();
    }

    public Integer getRank() {
        return rank;
    }

    public Integer getCylinders() {
        return cylinders;
    }

    public void setCylinders(final Integer cylinders) {
        this.cylinders = cylinders;
    }

    public VolumeCapacity getCapacity() {
        return capacity;
    }

    protected boolean hasChildren() {
        return false;
    }

    public void setRedundancy(final String redundancy) {
        capacity.setRedundancy(redundancy);
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(cache, writer);

        addAttribute(writer, "Rank", getRank());
        addAttribute(writer, "Cylinders", getCylinders());
        addAttribute(writer, "DiskGroup", diskGroup);

        addAttribute(writer, "CapacityMB", getCapacity().getCapacity(), Capacity.Unit.MB_1024);
        addAttribute(writer, "RawCapacityMB", getCapacity().getRawCapacity(), Capacity.Unit.MB_1024);
        addAttribute(writer, "UsedCapacityMB", getCapacity().getUsedCapacity(), Capacity.Unit.MB_1024);
        addAttribute(writer, "Redundancy", getCapacity().getRedundancy());

    }
}
