package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

public class Zone extends XMLNode<ZoningCapabilities> {
    private final String zoneName;
    private String zoneType = null;
    private ZoneMembers zoneMembers;

    private final ZoneKey key;

    /**
     * Use {@link ActiveZoningCapabilities#createZone} to create this object
     */
    Zone(final ZoningCapabilities parent, final String zoneName) throws SchemaException {
        super(parent, "Zone");

        if (zoneName == null || zoneName.length() == 0)
            throw new SchemaException(getNodeName() + " Type required non-null oid and zoneName");

        this.zoneName = zoneName;
        zoneMembers = new ZoneMembers(this);

        key = new ZoneKey(parent.getKey(), zoneName);
    }

    public final ZoneKey getKey() {
        return key;
    }

    public final ZoningCapabilities getZoningCapabilities() {
        return getParent();
    }

    public final String getZoneName() {
        return zoneName;
    }

    public final String getZoneType() {
        return zoneType;
    }

    public final void setZoneType(String zonetype) {
        this.zoneType = zonetype;
    }

    public final ZoneMembers getZoneMembers() {
        return zoneMembers;
    }

    public final SwitchPortZoneMember createPortZoneMember(final String wwn) throws SchemaException {
        return new SwitchPortZoneMember(this, wwn);
    }

    public final WWNZoneMember createWWNZoneMember(final String wwn) throws SchemaException {
        return new WWNZoneMember(this, wwn);
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        addAttribute(writer, "ZoneName", zoneName);
        addAttribute(writer, "ZoneType", zoneType);
    }

    protected void marshallChildren(final MarshallingCache cache, final Writer writer, final int tabs) throws IOException, SchemaException {
        addAllChildren(cache, writer, zoneMembers, tabs);
    }

    protected boolean hasChildren() {
        return (!zoneMembers.isEmpty());
    }

    public static final class ZoneKey implements Key<ZoneKey> {
        private final ZoningCapabilities.ZoningCapabilitiesKey zoneSetKey;
        private final String zoneName;

        public ZoneKey(final ZoningCapabilities.ZoningCapabilitiesKey zoneSetKey, final String zoneName) {
            this.zoneSetKey = zoneSetKey;
            this.zoneName = zoneName;
        }

        public final int compareTo(final ZoneKey o) {
            // To be unique within the SAN, we need to include the parent key as well!
            int i = zoneSetKey.compareTo(o.getZoningCapabilitiesKey());
            if (i != 0)
                return i;

            return getZoneName().compareTo(o.getZoneName());
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final ZoneKey zoneKey = (ZoneKey) o;

            if (zoneName != null ? !zoneName.equals(zoneKey.zoneName) : zoneKey.zoneName != null) return false;
            if (zoneSetKey != null ? !zoneSetKey.equals(zoneKey.zoneSetKey) : zoneKey.zoneSetKey != null) return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (zoneSetKey != null ? zoneSetKey.hashCode() : 0);
            result = 29 * result + (zoneName != null ? zoneName.hashCode() : 0);
            return result;
        }

        public final String toString() {
            return zoneSetKey.toString() + ":" + zoneName;
        }

        public final String getZoneName() {
            return zoneName;
        }

        public final ZoningCapabilities.ZoningCapabilitiesKey getZoningCapabilitiesKey() {
            return zoneSetKey;
        }
    }
}
