package com.onaro.sanscreen.acquisition.schema.utils;

public final class NasShareUtils {
    private static final String SLASH = "/";

    /**
     * Normalize the NAS share name or path. If it is NFS, append the trailing slash.
     * If it is CIFS, besides the trailing slash, normalize the share name or path to lower case.
     * See IBG-1575
     *
     * @param nasShareName the NAS share name or path
     * @param isCifsShare a flag to indicate whether it is CIFS share or NFS share
     * @return the normalized NAS share name or path
     */
    public static String normalizeNasShareName(final String nasShareName, final boolean isCifsShare) {
        if (nasShareName == null) {
            return null;
        }

        if (isCifsShare) {
            return nasShareName.endsWith(SLASH) ? nasShareName.toLowerCase() : nasShareName.toLowerCase() + SLASH;
        } else {
            return nasShareName.endsWith(SLASH) ? nasShareName : nasShareName + SLASH;
        }
    }
}
