package com.onaro.sanscreen.acquisition.util.servercache;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.adapters.XmlAdapter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

class MapXmlAdapter<T> extends XmlAdapter<MapXmlAdapter.MapType<T>, Map<Long, Set<T>>> {
    @Override
    public Map<Long, Set<T>> unmarshal(final MapType<T> mapType) throws Exception {
        final Map<Long, Set<T>> map = new HashMap<Long, Set<T>>();
        for (final MapElement<T> mapElement : mapType.mapElements) {
            map.put(mapElement.storageId, mapElement.set);
        }
        return map;
    }

    @Override
    public MapType<T> marshal(final Map<Long, Set<T>> map) throws Exception {
        final MapType<T> mapType = new MapType<T>();
        for (final Map.Entry<Long, Set<T>> entry : map.entrySet()) {
            mapType.addMapElement(entry);
        }
        return mapType;
    }

    static class MapType<T> {
        @XmlElement(name = "key")
        private final List<MapElement<T>> mapElements = new ArrayList<MapElement<T>>();

        private MapType() {
        }

        private void addMapElement(final Map.Entry<Long, Set<T>> entry) {
            mapElements.add(new MapElement<T>(entry.getKey(), entry.getValue()));
        }
    }

    private static class MapElement<T> {
        @XmlAttribute
        private final Long storageId;
        @XmlElement(name = "value")
        private final Set<T> set;

        @SuppressWarnings("UnusedDeclaration")
        private MapElement() {
            storageId = 0L;
            set = null;
        }

        private MapElement(final Long storageId, final Set<T> set) {
            this.storageId = storageId;
            this.set = set;
        }
    }
}
