package com.onaro.util.collect;

import java.io.Serializable;
import java.util.AbstractSet;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

/**
 * Set of elements, indexed by one or more element keys.
 */
public abstract class IndexedSet<E> extends AbstractSet<E>
        implements Set<E>, Serializable {

    private final static long serialVersionUID = 1L;

    protected final Map<Tuple, E> map = new TreeMap<Tuple, E>();

    public IndexedSet() {
    }

    abstract protected Tuple createKey(E e);

    @Override
    public int size() {
        return map.size();
    }

    /**
     * Return <tt>true</tt> if this set contains the specified element.
     */
    @SuppressWarnings("unchecked")
    @Override
    public boolean contains(final Object node) {
        return map.containsKey(createKey((E)node));
    }

    @Override
    public boolean add(final E e) {
        final E oldNode = map.put(createKey(e), e);
        return (oldNode != e);
    }

    @SuppressWarnings("unchecked")
    @Override
    public boolean remove(final Object e) {
        return (map.remove(createKey((E)e)) != null);
    }

    @Override
    public void clear() {
        map.clear();
    }

    @Override
    public Iterator<E> iterator() {
        return map.values().iterator();
    }

    @SuppressWarnings("unchecked")
    @Override
    public boolean retainAll(final Collection<?> c) {
        if (c.size() == 0) {
            return false;
        }
        final Collection<Tuple> retainedKeys = new ArrayList<Tuple>(c.size());
        for (final Object obj : c) {
            retainedKeys.add(createKey((E)obj));
        }
        return map.keySet().retainAll(retainedKeys);
    }

    @Override
    public boolean equals(final Object obj) {
        if (obj == this) {
            return true;
        }
        if (obj instanceof Set) {
            // The collection returned by map.values() doesn't implement
            // equals() so we have to write our own. For an explanation, see:
            // http://stackoverflow.com/questions/341377/why-abstractcollection-does-not-implement-equals
            final Collection<E> values = map.values();
            final Set<?> other = (Set)obj;
            if (other.size() == values.size()) {
                for (final E e : values) {
                    if (!other.contains(e)) {
                        return false;
                    }
                }
                return true;
            }
        }
        return false;
    }

    @Override
    public int hashCode() {
        // The collection returned by map.values() doesn't implement hashCode()
        // so we have to write our own. For an explanation, see:
        // http://stackoverflow.com/questions/12094202/why-does-transformedcollection-not-implement-hashcode
        int h = 0;
        for (final E e : map.values()) {
            if (e != null) {
                h += e.hashCode();
            }
        }
        return h;
    }

}
