package com.onaro.util.collect;

import org.apache.commons.lang3.ObjectUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Used to create multi-values keys suitable for hash maps and tree maps.
 */
public class Tuple implements Comparable<Tuple> {

    /** Ordered list of values. */
    protected final List<Comparable> values;

    public Tuple(final Comparable<?>... values) {
        this.values = new ArrayList<Comparable>(values.length);
        Collections.addAll(this.values, values);
    }

    @Override
    public int hashCode() {
        return this.values.hashCode();
    }

    @Override
    public boolean equals(final Object obj) {
        return (obj instanceof Tuple) && values.equals(((Tuple)obj).values);
    }

    /**
     * Tuples are ordered by their first element, then their second, etc.
     * For example: (a) &lt; (a, b) &lt; (a, c) &lt; (a, c, a) &lt; (b) &lt; (b, a)
     */
    @Override
    public int compareTo(final Tuple that) {
        final List<Comparable> v1 = this.values;
        final List<Comparable> v2 = that.values;
        final int len1 = v1.size();
        final int len2 = v2.size();
        for (int i = 0, len = Math.min(len1, len2); i < len; i++) {
            final int cmp = ObjectUtils.compare(v1.get(i), v2.get(i));
            if (cmp != 0) {
                return cmp;
            }
        }
        // One tuple is the prefix of the other. Now sort on length.
        if (len1 < len2) {
            return -1;
        }
        else if (len1 > len2) {
            return 1;
        }
        else {
            return 0;
        }
    }

}
