#!/bin/sh

#set default lib directory(Ubuntu has /usr/local/lib as defaul, not RHEL)
libdir="/usr/local/lib"
ldconfigdir="/etc/ld.so.conf"
if [ $(grep $libdir $ldconfigdir | wc -l) -eq 0 ];
then
    #append default lib location /usr/local/lib to ldconfigdir
    echo $libdir >> $ldconfigdir
fi

# Cache the new libraries(cmapl and cmapl wrapper) added to /usr/local/lib
ldconfig

# this section creates a state file needed by logrotate to cmaplwrapper.log
touch /var/log/cmapl_log_rot_state

# Start rsyslog/syslog - an operating system's logging facility used by cmapl wrapper library.
logsysfile=""
cmaplwrapperlogname="cmaplwrapper.log"
wrapperlogfile="/var/log/ocie/$cmaplwrapperlogname"
cmaplscripts="/opt/netapp/essentials/au/cmaplscripts"
filecreatecopymode="\$FileCreateMode 0644"
filecreatedefaultmode="\$FileCreateMode 0640"
fileowner="\$FileOwner jboss"
filegroup="\$FileGroup jboss"
rotataionsize="5242880"
rotataionscript="$cmaplscripts/./cmapl_log_rotate"
channelname="cmaplwrapperchannel"
logcreatechannel="\$outchannel $channelname,$wrapperlogfile,$rotataionsize,$rotataionscript"

# Below line is added in the syslog/rsyslog.conf to prepare for logging.
wrapperlogchannel="local1.* \$$channelname"

if [ -s /etc/syslog.conf ]
then
    logsysfile="/etc/syslog.conf"
elif [ -s /etc/rsyslog.conf ]
then
    logsysfile="/etc/rsyslog.conf"
else
    echo "Could not find syslog or rsyslog configuration file, cmapl log info will be printed to stdout."
fi

# Run log file as user group jboss jboss
if [ `cat $logsysfile | grep -i "$fileowner" | wc -l` -eq 0 ];
then
    echo $fileowner >> $logsysfile
    echo $filegroup >> $logsysfile
fi

# Add Umask to config file if not alredy there. In RHEL this is not there by default unlike Ubuntu.
umaskString="\$umask 0000"
if [ `cat $logsysfile | grep -i "Umask" | wc -l` -eq 0 ];
then
    echo $umaskString >> $logsysfile
fi

# Add FileCreateMode 644, if its not already there. RHEL doesnt have it by default
if [ `cat $logsysfile | grep -i "$filecreatecopymode" | wc -l` -eq 0 ];
then
    echo $filecreatecopymode >> $logsysfile
else
    # FileCreatemode exists, check the creation mode. e.g. make sure $FileCreateMode 0644 should be before the logchannel.
    filecreatedefaultline=`grep -n "$filecreatedefaultmode" $logsysfile | awk -F: '{print $1}' | tail -1`
    filecreatereadmodeline=`grep -n "$filecreatecopymode" $logsysfile | awk -F: '{print $1}' | tail -1`

    if [ "x$filecreatedefaultline" != "x" ];
    then
        if [ $filecreatedefaultline -gt $filecreatereadmodeline ];
        then
            echo $filecreatecopymode >> $logsysfile
        fi
    fi
fi

# Optionally  make sure the logging directives are deleted from rsyslog.conf prior to adding new entries.
sed -i /$cmaplwrapperlogname/d $logsysfile
sed -i /$channelname/d $logsysfile
# we add read permissions to other users for this log file to give access to copy or read.
echo $logcreatechannel >> $logsysfile
echo $wrapperlogchannel >> $logsysfile

# Execute permissions for the log rotation script.
chmod 500 $rotataionscript
